/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.util.rvexception;

import java.io.PrintStream;
import java.io.ByteArrayOutputStream;

/**
 * Base class for all throwables in the JRP project
 * This class can wrap other thrown throwables.
 **/
public class RVThrowable extends Throwable
						             implements ErrorSeverity
{
	protected int       _severity = ERROR;
	protected Throwable _baseThrowable = null;
	protected RVThrowable _baseRVThrowable = null;

  public RVThrowable()
	{
		super();
	}

  public RVThrowable(String message)
	{
		super(message);
	}

  public RVThrowable(int severity)
	{
		_severity = severity;
	}

  public RVThrowable(String message, int severity)
	{
		super(message);
		_severity = severity;
	}

  public RVThrowable(Throwable e)
	{
		super();
		_baseThrowable = e;
	}

  public RVThrowable(String message, Throwable e)
	{
		super(message);
		_baseThrowable = e;
	}

  public RVThrowable(int severity, Throwable e)
	{
		_severity = severity;
		_baseThrowable = e;
	}

  public RVThrowable(String message, int severity, Throwable e)
	{
		super(message);
		_severity = severity;
		_baseThrowable = e;
	}

  public RVThrowable(String message, int severity, RVThrowable e)
	{
		super(message);
		_severity = severity;
		_baseRVThrowable = e;
	}

	/**
	 * Converts Throwable's stack trace to String
	 */
	public static String stackTraceToString(Throwable e)
	{
		ByteArrayOutputStream o = new ByteArrayOutputStream();
		PrintStream p = new PrintStream(o, true);
		e.printStackTrace(p);
		return o.toString();
	}

	public void setSeverity(int severity)
	{
		_severity = severity;
	}

	public int getSeverity()
	{
		return _severity;
	}

	public void setBaseThrowable(Throwable e)
	{
		_baseThrowable = e;
	}

	public Throwable getBaseThrowable()
	{
		return _baseThrowable;
	}

	/**
	 * Converts the stack trace recursivly into a String
	 */
	public String stackTraceToString()
	{

		// recursion's stop condition: this wraps Throwable, not RVThrowable
		if (_baseThrowable != null)
			return stackTraceToString(_baseThrowable);

		// recursion's continue condition: this wraps RVThrowable
		String s = "";
		if (_baseRVThrowable != null) {
			  s += _baseRVThrowable.stackTraceToString(); // get _baseRVException stack trace recursivly
				s += stackTraceToString(this); //get rest of stack trace
		}

		return s;
	}

	public String ToString()
	{
		String s = "\n" + super.toString();
		s += "\n\tSeverity: ";
		switch(_severity) {
			case ErrorSeverity.INFO:
				s += "INFO";
				break;
			case ErrorSeverity.WARNING:
				s += "WARNING";
				break;
			case ErrorSeverity.ERROR:
				s += "ERROR";
				break;
			case ErrorSeverity.FATAL:
				s += "FATAL";
				break;
			default:
				s += "UNKNOWN";
				break;
		}

		s += "\n\t" + stackTraceToString(); // is Exception.toString() also retireves the stackTrace?

		return s;
	}

}
