/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.util.testtalk;


import com.radview.util.datastructures.*;


/**
 * class NetKernelMessageHandler
 **/
public abstract class NetKernelMessageHandler
{

	protected 	 String          clientName     = null;
	protected 	 TestTalkClientC testtalkClient = null;
	protected 	 ReaderThread    readerThread   = null;
	protected 	TestTalkC	 testTalk       = null;
	protected boolean privateTestTalk;
	protected        boolean         isActive       = false;


	public NetKernelMessageHandler(String baseName)
	{
		clientName = baseName;
		privateTestTalk = true;
	}

	public NetKernelMessageHandler(String baseName, TestTalkC testtalk)
	{
		clientName = baseName;
		testTalk = testtalk;
		privateTestTalk = false;
	}


	public void init(int port) throws ErrorLocatingLocalHostAddressException,
                           			  CouldNotListenOnPortException
	{
		init(port, false);
	}

	public void init() throws ErrorLocatingLocalHostAddressException,
                           	  CouldNotListenOnPortException
	{
		init(0, false);
	}

	protected void init(int port, boolean syncWorkMode) throws ErrorLocatingLocalHostAddressException,
                           	  				   CouldNotListenOnPortException
	{
		   if (privateTestTalk) {
			  testTalk = new TestTalkC(port,   //port 0 means: catch any port
						   true,
						   true,  //debug mode
						   20000); //sending timeout
			}

		testtalkClient = new TestTalkClientC(testTalk);
		readerThread   = new ReaderThread();

		testtalkClient.NetInit(clientName);

		if(syncWorkMode == false) {
			readerThread.start();
		}
		isActive = true;
	}

	public void terminate()
	{
	  isActive = false;
	  testtalkClient.NetTerminate();
//	  if (privateTestTalk)
//	  	testTalk.terminate();
	}

	public boolean isActive()
	{
	  return isActive;
	}

	public String whoAmI()
	{
	  return testtalkClient.NetWhoAmI();
	}

	public void sendMessage(AssociativeArrayC message, String sendTo, int priority)
	throws ErrorSendingMessageException, WrongAddressException
	{
		message.AddCouple("FROM", testtalkClient.NetWhoAmI());

		testtalkClient.NetSendMessage(message.ToString(), sendTo, priority);
	}

	//
	// Callback method that should be implemented by the sub class
	// of this class. The method will get a reference to an AssociativeArrayC
	// object
	public abstract void messageHandler(AssociativeArrayC message);


	/**
	 * private class NetKernelMessageHandler.ReaderThread
	 **/
	private class ReaderThread extends Thread
	{
		public void run()
		{
			String buffer;
			AssociativeArrayC message = new AssociativeArrayC();

			try {
				while( (buffer = testtalkClient.NetWaitMessage()) != null ) {
					message.FromString(buffer);
					messageHandler(message);
					message.Reset();
				}
			}
			catch(IllegalAssoSyntaxException iae) {
				System.out.println(iae.toString());
			}
		}
	}
}
