/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.util.testtalk;

import java.io.DataInputStream;
import java.io.BufferedOutputStream;

// radview classes
import com.radview.util.conversions.ByteArrayConversion;
import com.radview.util.general.DebugMessages;

// the low level class which conains the data to be sent/read. The
// data is passed in the following order :
//      * length  - int   (4 bytes - the length of data)
//      * type    - short (2 bytes - used by TestTalk)
//      * subtype - short (2 bytes - used by TestTalk)
//      * data    - byte array
class PackedBufC {
    protected short  Type;
    protected short  SubType;
    protected String Data;

    // read the transaction from the socket at the documented order
    protected void ReadFromInputStream(DataInputStream IS)
                            throws ErrorReadingFromInputStreamException {
        try {
            int length = IS.readInt();
            Type = IS.readShort();
            SubType = IS.readShort();
            byte[] bytearray = new byte[length];
            int numread = IS.read(bytearray);
            Data = new String(bytearray,0); // this constructor is deprecated

            if (TestTalkConstants.DEBUG_MODE)
                DebugMessages.WriteMessage(
                        "TestTalk Read From Stream : " +
                        length + " " + Type + " " + SubType + " " + Data);

        }
        catch (Exception e) {  //Throwable
            throw new ErrorReadingFromInputStreamException("" + e);
        }
    }

    // The method which converts PackedBuf to byte array and takes
    // care of the write procedure to the socket.
    protected void WriteToOutputStream(BufferedOutputStream OS)
                            throws ErrorWritingToOutputStreamException {
        try {

            // write length : convert to byte array and write bytes
            OS.write(ByteArrayConversion.IntToByteArray(Data.length()));

            // write type : convert to byte array and write bytes
            OS.write(ByteArrayConversion.ShortToByteArray(Type));

            // write subtype : convert to byte array and write bytes
            OS.write(ByteArrayConversion.ShortToByteArray(SubType));

            // write data : convert to byte array and write bytes
            byte[] ByteArray = new byte[Data.length()];
            Data.getBytes(0,Data.length(),ByteArray,0); // deprecated
            OS.write(ByteArray);

            // end transaction
            OS.flush();

            if (TestTalkConstants.DEBUG_MODE)
                DebugMessages.WriteMessage(
                    "TestTalk Write To Stream : " +
                    Data.length() + " " + Type + " " + SubType + " " + Data);

        }
        catch (Exception e) {  //Throwable
            throw new ErrorWritingToOutputStreamException("" + e);
        }

    }
}
