/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.util.testtalk;

// This class is used by each client. Any Client MUST create object
// of this class and MUST call the NetInit method to register itself
// as a client.
public class TestTalkClientC extends Thread{

    private TestTalkC TestTalk;     // the TestTalk engine which the
                                    // the client is connected to.
    private String MyClientName;    // The name of the current client

    static public byte ACKQ = TestTalkConstants.ACKQ;
    static public byte DATAC = TestTalkConstants.DATAQ;

    // Ctor
    public TestTalkClientC(TestTalkC testtalk) {
        TestTalk = testtalk;
    }

    // Init Method for registering and identify the current client
    public void NetInit(String ClientName) {
        TestTalk.NetInit(ClientName);
        MyClientName = ClientName;
    }

    // Terminate method which erase the current client from TestTalk
    public void NetTerminate() {
        TestTalk.NetTerminate(MyClientName);
    }

    // return the full address of the current client
    public String NetWhoAmI() {
        return MyClientName + "@" +
               NetLocalHost() + ":" +
               NetLocalPort();
    }

    // Local Host
    public String NetLocalHost() {
        return TestTalk.NetLocalHost();
    }

    // Local Port
    public int NetLocalPort() {
        return TestTalk.NetLocalPort();
    }

    // My Client Name
    public String NetClientName() {
        return MyClientName;
    }

    // clear messages queue
    public void NetClearMessagesQueue() {
        ClientC Client = TestTalk.ClientObjectOf(MyClientName);
        try {
            Client.ClearQueue(TestTalkConstants.DATAQ);
        }
        catch (WrongQueueTypeException e) {}
    }

    // clear ack messages queue
    public void NetClearAckMessagesQueue() {
        ClientC Client = TestTalk.ClientObjectOf(MyClientName);
        try {
            Client.ClearQueue(TestTalkConstants.ACKQ);
        }
        catch (WrongQueueTypeException e) {}
    }

    // get the size of the messages queue
    public int NetMessagesQueueSize() {
        ClientC Client = TestTalk.ClientObjectOf(MyClientName);
        try {
            return Client.QueueSize(TestTalkConstants.DATAQ);
        }
        catch (WrongQueueTypeException e) {}
        return -1;
    }

    // get the size of the ack messages queue
    public int NetAckMessagesQueueSize() {
        ClientC Client = TestTalk.ClientObjectOf(MyClientName);
        try {
            return Client.QueueSize(TestTalkConstants.ACKQ);
        }
        catch (WrongQueueTypeException e) {}
        return -1;
    }

    // Waiting for Message
    public String NetWaitMessage() {
        try {
            return NetWaitMessage(-1); // -1 for waiting forever
        }
        catch (TimeoutExpiredException e) {}
        return null;
    }

    // Waiting for Message until timeout expired
    public String NetWaitMessage(int TimeoutInMiliSeconds)
                                    throws TimeoutExpiredException {
        return TestTalk.NetWaitMessage(MyClientName,
                                       TimeoutInMiliSeconds);
    }

    // Fetch Message from Client Messages Queue
    public String NetFetchMessage()
                        throws EmptyQueueException {
        return TestTalk.NetFetchMessage(MyClientName);
    }

    // Waiting for Ack Message
    public String NetWaitAckMessage() {
        try {
            return NetWaitAckMessage(-1); // -1 for waiting forever
        }
        catch (TimeoutExpiredException e) {}
        return null;
    }

    // Waiting for Ack Message until timeout expired
    public String NetWaitAckMessage(int TimeoutInMiliSeconds)
                                    throws TimeoutExpiredException {
        return TestTalk.NetWaitAckMessage(MyClientName,
                                          TimeoutInMiliSeconds);
    }

    // Fetch Ack Message from Client Ack Messages Queue
    public String NetFetchAckMessage()
                        throws EmptyQueueException {
        return TestTalk.NetFetchAckMessage(MyClientName);
    }

    // Sending Message to client. There are 3 legal addresses :
    //   1) <name>  - TestTalk assume that client is in the local JVM
    //   2) <name>@<host> - TestTalk assume that the client is located
    //                  in different JVM and try to write Message to
    //                  that host in DEFAULT_PORT no.
    //   3) <name>@<host>:<port> - TestTalk assume that the client
    //                  is located in different JVM.
    public void NetSendMessage(String Data,
                               String Address,
                               int    RetryTimes)
                                throws ErrorSendingMessageException,
                                       WrongAddressException {
        ErrorSendingMessageException er =
                            new ErrorSendingMessageException();
        for (int i=0; i < RetryTimes + 1; i++) {
            try {
                TestTalk.NetSendMessage(MyClientName,Data,Address);
                return;
            }
            catch (ErrorSendingMessageException e) {
                er = e;
            }
        }
        throw er;
    }

    // Sending Ack Message to client. There are 3 legal addresses :
    //   1) <name>  - TestTalk assume that client is in the local JVM
    //   2) <name>@<host> - TestTalk assume that the client is located
    //                  in different JVM and try to write Message to
    //                  that host in DEFAULT_PORT no.
    //   3) <name>@<host>:<port> - TestTalk assume that the client
    //                  is located in different JVM.
    public void NetSendAckMessage(String Data,
                                  String Address,
                                  int    RetryTimes)
                                throws ErrorSendingMessageException,
                                       WrongAddressException {
        ErrorSendingMessageException er =
                            new ErrorSendingMessageException();
        for (int i=0; i < RetryTimes + 1; i++) {
            try {
                TestTalk.NetSendAckMessage(MyClientName,Data,Address);
                return;
            }
            catch (ErrorSendingMessageException e) {
                er = e;
            }
        }
        throw er;
    }

    public void NetNotifyOnConnectionFail(String  CheckAddress,
                                          boolean CheckAbnormalExit,
                                          String  DataToBeSent,
                                          String  TargetAddress,
                                          byte    QueueType) {

        TestTalk.NetNotifyOnConnectionFail(CheckAddress,
                                           CheckAbnormalExit,
                                           DataToBeSent,
                                           TargetAddress,
                                           QueueType);

    }

    public void NetNormalExitTestTalk() {
        TestTalk.NetNormalExitTestTalk();
    }

    public void run() {
        String message;
        while (true) {
            message = NetWaitAckMessage();
            if (message.startsWith("Normal Exit : ")) {
                TestTalk.NetNormalExitTestTalk(message.substring(14));
            }
        }
    }

}
