/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_ELEMENT__
#define __WL_ELEMENT__
#include <RVCString.h>
#include <ordered.h>
#include "platform.h"
#include "wlarray.h"

class RVOrdered;

enum Set_Result{
	Element_Set,
	Element_No_Value_Set,
	Element_No_Checked_Set,
	Element_No_Selected_Index_Set
};

class CWLDocument;

//===============================CWLELEMENT==========================
class DllExport CWLElement : public CKVP {
public:
	CWLDocument *m_DocPtr;
private:
	RVOrdered*		m_OptionsPtr;		 //! SELECT
	RVCString			m_Type;
	RVCString			m_Id;
	RVCString			m_DefaultValue;	 //! TEXTAREA, TEXT
	int						m_SelectedIndex; //! SELECT
	bool					m_DefaultChecked;//! CHECKBOX, RADIO
	bool					m_Checked;			 //! CHECKBOX, RADIO

public:
	CWLElement(){}
	CWLElement(const char* key_, const char* value_, const char* type_, const char* id_, bool checked_, unsigned int selectedIndex, 	RVOrdered* OptionsPtr):CKVP(key_, value_)
	{
		setType(type_);
		m_SelectedIndex			= selectedIndex;
		if(m_Type.compareTo("select", RVCString::ignoreCase) == 0)
			m_OptionsPtr				= OptionsPtr;
		else {
			if (OptionsPtr) {
				OptionsPtr->clearAndDestroy();
				delete OptionsPtr;
			}
			m_OptionsPtr				= NULL;
		}
		m_DefaultChecked		= m_Checked = checked_;
		if(value_)
			m_DefaultValue		= value_;
		if(id_)
			m_Id = id_;
		m_StartInnerHTML = -1;
	    m_LenInnerHTML = -1;
	}
	virtual ~CWLElement();

	CWLElement&		operator=(CWLElement&);
	bool					operator==(CWLElement&);
	void					Reset();
	int	Length();//! only for SELECT
	const char*		DefaultValue();
	int						SelectedIndex(){return m_SelectedIndex;}
	bool					DefaultChecked(){ return m_DefaultChecked; }
	bool					Checked(){ return m_Checked; }
	const char*		Name() { return Key(); } //! just a change of name from value
	const char*		Type() { return m_Type.data(); }
	const char*		Id()			{ return m_Id.data();}
	RVOrdered*		Options();
	Set_Result		setType(const char* type);
	Set_Result		setName(const char* name);
	Set_Result		setChecked(bool checked);
	Set_Result		setSelectedIndex(unsigned int index);
	Set_Result		setValue(const char* newValue)	;

private:
	int             m_StartInnerHTML;
	int             m_LenInnerHTML;

public:
	void SetInnerHTMLOffset(int start, int len) { m_StartInnerHTML = start; m_LenInnerHTML = len; }
	void SetInnerHTMLStart(int start) { m_StartInnerHTML = start; }
	void SetInnerHTMLLen(int len)     { m_LenInnerHTML = len; }
	int GetStartInnerHTML() {return m_StartInnerHTML; }
	int GetLenInnerHTML() {return m_LenInnerHTML; }
	RVCSubString GetInnerHTML();
};
#endif

