/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_FORM__
#define __WL_FORM__

#include <RVCString.h>
#include <hashdict.h>
#include <collectable.h>
#include "platform.h"
#include "wlhtmlfuncs.h"
#include "wlurlbase.h"
#include "wlarray.h"
#include "wllocation.h"
#include "wltable.h"

class urlBase;
class CWLOption;
class CWLLinkArray;
class parseSrc;
class CWLDocument;
class CWLHtmlVirtualQueue;

// ============================CWLFORM===============================
class DllExport CWLForm:public RVCollectable
{
private:
	formElementState	optionsState;
	formFlag					formState;
	enum{
			FIND_ATT_VALUE,
			FIND_NAME_VALUE,
			FIND_OPTIONS
			}elementState, inOptionState;
	RVCString					value, name, type, id, selectId;
	bool							checked, actionFound;
	unsigned int			selectedIndex;
	RVOrdered*				optionsPtr;
	bool currentOptionSelected , previousOptionSelected;
	size_t currentOptionIndex;
	long m_Index;

private:
	CWLDocument*				m_DocPtr;

	CWLArray     m_Elements;
	CWLLocation*  m_pAction;
	RVCString    m_Method;
	RVCString    m_Name;
	RVCString    m_Target;
	RVCString    m_Encoding;
	RVCString    m_Id;
private:
  elementReadState extractProperties(RVCString& buff, urlBase* urlBasePtr);    //! extract the url to add to the appropriate list

  int					getValue(RVCString& buff, RVCString& str, size_t buffPtr,  size_t delimiterIndex);
  int					getType(RVCString& buff, RVCString& str, size_t buffPtr,  size_t delimiterIndex);
  int					getName(RVCString& buff, RVCString& str, size_t buffPtr, size_t delimiterIndex);
  RVCString&  getString(RVCString& str, RVCString& buff, size_t buffPtr);


	formElementState								findInstance(RVCString& buff);    //! find tokens
  elementReadState								parseSelect(RVCString& buff);      //! <SELECT
  elementReadState								parseInput(RVCString& buff, urlBase *urlBasePtr);    //! <INPUT
  elementReadState								parseTextArea(RVCString& buff) ;//! <TEXTAREA

  inline int      findSpace(char& c);
  inline size_t   findSpaceIndex(RVCString& buff, size_t startUrlIndex);

public:
	CWLForm(CWLDocument* docPtr);

	virtual						~CWLForm();
	CWLForm&					operator=(CWLForm&);
	bool							operator==(const CWLForm&);
	elementReadState	parse(RVCString& buff, urlBase* urlBasePtr);



	CWLElement*		Element(const char* elementName);
	CWLArray&     Elements(){return m_Elements;}
	CWLLocation&  Action() {return *m_pAction;}
	CWLLocation*  Action_(){return m_pAction;}
	RVCString&    Method(){return m_Method;}
	RVCString&    Name(){return m_Name;}
	RVCString&    Encoding(){return m_Encoding;}
	RVCString&    Target(){return m_Target;}
	RVCString&		Id(){return m_Id;}
	size_t				Length(){return m_Elements.entries() != RV_NPOS ? m_Elements.entries() : 0;}
	int						findEntryByName(const char*, const char*&);//changed - have to change above
	 void					printAll();
	long          GetIndex() { return m_Index; }

private:
	int             m_StartInnerHTML;
	int             m_LenInnerHTML;

public:
	void SetInnerHTMLOffset(int start, int len) { m_StartInnerHTML = start; m_LenInnerHTML = len; }
	void SetInnerHTMLStart(int start) { m_StartInnerHTML = start; }
	void SetInnerHTMLLen(int len)     { m_LenInnerHTML = len; }
	int GetStartInnerHTML() {return m_StartInnerHTML; }
	int GetLenInnerHTML() {return m_LenInnerHTML; }
	RVCSubString GetInnerHTML();
};

#endif


