/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_META__
#define __WL_META__

#include <RVCString.h>
#include <ordered.h>
#include <collectable.h>
#include <platform.h>
#include "wlurlbase.h"
#include "wlurl.h"
#include "wlhtmlfuncs.h"

class CWLDocument;

class DllExport CWLMeta : public RVCollectable
{
private:
	CWLDocument *m_DocPtr;
	RVCString   m_Content;
	RVCString   m_HttpEquiv;
	RVCString   m_Name;

	CWLUrl      *m_Url;

private:

	void        parseUrl(urlBase* urlBasePtr);

public:

	CWLMeta(CWLDocument *docPtr) : m_DocPtr(docPtr), m_Url(NULL)
	{}

	virtual ~CWLMeta()
	{
		if( m_Url )
			delete m_Url;
	}

	const char *getContent()   { return (const char *)m_Content; }
	const char *getHttpEquiv() { return (const char *)m_HttpEquiv; }
	const char *getName()      { return (const char *)m_Name; }

	CWLUrl     *getUrl()       { return m_Url; }

	void  parse(RVCString &buff , urlBase* urlBasePtr);
	void  printAll();
};


class DllExport CWLMetaCollection : public RVOrdered
{
private:
	CWLDocument *m_DocPtr;
public:
	CWLMetaCollection(CWLDocument *docPtr) : m_DocPtr(docPtr) {}
	virtual ~CWLMetaCollection()
	{
		clearAndDestroy();
	}

    elementReadState parse(RVCString& buff, urlBase* urlBasePtr);
	void printAll();
};

#endif

