/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef _WLTRANSACTIONDATA_H_
#define _WLTRANSACTIONDATA_H_

#include "wlerrorlib.h"
#include <RVCString.h>
#include <collectable.h>
#include <slistcol.h>
#include "wlhtmlvirtualqueue.h"

class DllExport CWLEvent;
class DllExport CWLTransactionData ;
class DllExport CWLTransactionsObject ;

#define CHECKSUM_VAR unsigned char

class CWLEvent : public RVCollectable
{
public:
  CWLEvent(
           double dTime,
           const char* reason, 
           const char* description, 
           WLLogMessage::Severity severity = WLLogMessage::Success, 
           const char* extraDataType = NULL, 
           void* extraData = NULL, 
           unsigned long len = 0,
           bool bLog = true);
  CWLEvent(const CWLEvent& cpyEvent);
  CWLEvent& operator=(const CWLEvent& cpyEvent);
  ~CWLEvent();
  
public:
  double    m_dTime   ;
  RVCString m_sReason ;
  RVCString m_sDescription;
  WLLogMessage::Severity m_Severity ;
  RVCString m_sExtraDataType ;
  void* m_ExtraData ;
  unsigned long m_ulLength ;
  bool m_bLog;

};



class CWLTransactionData : public RVCollectable
{
public:
#ifndef __GNUC__
  // ATTENTION GNU compiler does not knwo how to handle static enum definitions
  // so we omit the static ONLT from GNU 
  static 
#endif //__GNUC__
  enum TransactionType{ 
    TRANS_GENERAL,
    TRANS_OPEN_CONNECTION, 
    TRANS_SEND, 
    TRANS_WAIT, 
    TRANS_RECEIVE };

  CWLTransactionData(const char* pName=NULL, double dTime=0, TransactionType eType=TRANS_GENERAL);
  CWLTransactionData(const CWLTransactionData& cpyTrans);
  ~CWLTransactionData();

  void SetName(const char*pName);
  void SetSeverity(WLLogMessage::Severity severity);
	void ResetSeverity();
  void SetBeginTransaction(double dTime);
  void SetEndTransaction(double dTime, WLLogMessage::Severity severity, bool save=false);
  void SetEndTransactionFailure(double dTime, WLLogMessage::Severity severity, const char* reason);
  void SetFailureReason(double dTime, const char* reason, const char* description);
  void SetFailureReason(const char* reason, const char* description=NULL);
	void SetBeginSubTransaction(double dTime);
	double GetSubTransBegin(){return m_dBeginSubTrans;}

  const char* GetName(){return m_sName.data();}
  const char* GetFailureReason(){return m_sFailureReason.data();}
  const char* GetDescription(){return m_sDescription.data();}
  bool   IsFailureReason(){return (!m_sFailureReason.isNull() || !m_sDescription.isNull());}
  double GetTransBegin(){return m_dBeginTrans;}
  double GetTransEnd(){return m_dEndTrans;}
  WLLogMessage::Severity GetSeverity(){return m_Severity;}
  bool GetSave(){return m_bSave;}
  bool IsFirstSubTrans(){ return m_bFirstSubTrans; }
  void SetFirstSubTrans(bool bSet=true){m_bFirstSubTrans = bSet; }
  bool IsErrorUnder(){return m_bErrorUnder;}
  void SetErrorUnder(bool bSet=true){m_bErrorUnder = bSet; }

  void RecordEvent(
                    double dTime,
                    const char* reason, 
                    const char* description, 
                    WLLogMessage::Severity severity = WLLogMessage::Success, 
                    const char* extraDataType = NULL, 
                    void* extraData = NULL, 
                    unsigned long len = 0,
                    bool bLog = true);

  void AddEvent(
                    const char* reason, 
                    const char* description, 
                    WLLogMessage::Severity severity = WLLogMessage::Success, 
                    const char* extraDataType = NULL, 
                    void* extraData = NULL, 
                    unsigned long len = 0,
                    bool bLog = true);

  bool IsEventEmpty() {return m_EventList.isEmpty() ? true : false ;}
  int  NumOfEvents()  {return m_EventList.entries();}
  CWLEvent* GetNextEvent(int &pos);
  RVSlistCollectables* GetEventList(){return &m_EventList;}  // Chaim - changed to return pointer
  void SetOrigName(const char* cOrigName){m_sOrigName  = cOrigName ? cOrigName : "" ; }
  const char *GetOrigName(){return m_sOrigName.data();}
  TransactionType GetTransType(){return m_eTransType;}

protected:
  double m_dBeginTrans ;
  double m_dEndTrans   ;
	double m_dBeginSubTrans;
  RVCString m_sDescription ;
  RVCString m_sName    ;
  RVCString m_sOrigName    ; //patch !
  RVCString m_sFailureReason ;
  WLLogMessage::Severity m_Severity ;
  bool m_bSave ;
  bool m_bFirstSubTrans;
  bool m_bErrorUnder;
  TransactionType m_eTransType ;

  RVSlistCollectables m_EventList;

};

// Chaim - For ParseOnce
class CWLGeneralObject : public RVCollectable
{
public:
  CWLGeneralObject() {};
  CWLGeneralObject(const CWLGeneralObject& cpyGeneralObj) { m_OriginalUrl = cpyGeneralObj.m_OriginalUrl; m_OriginalUrl = cpyGeneralObj.m_OriginalUrl; }
  CWLGeneralObject(RVCString& OriginalUrl, RVCString& Referrer);
  CWLGeneralObject(const char* OriginalUrl, const char* Referrer);
  ~CWLGeneralObject() {};
  RVCString& GetReferrer()    { return m_Referrer; }
  RVCString& GetOriginalUrl() { return m_OriginalUrl; }

private:
  RVCString		  m_OriginalUrl;
  RVCString		  m_Referrer;
};

class CWLTransactionsObject : public RVCollectable
{

public:
  CWLTransactionsObject(RVCollectable* pObject, HtmlQueueItemType objType = NONE);
  CWLTransactionsObject(const CWLTransactionsObject& cpyObj);

  // Chaim - new copy constructor for Parse Once.
  CWLTransactionsObject(CWLTransactionsObject& cpyObj, bool CopySubTransactionList);

  ~CWLTransactionsObject();

  CWLTransactionData* GetTransaction(){return &m_Trans;}
  CWLTransactionData* GetLastSubTransation(){return (CWLTransactionData*)m_SubTransList.at(m_SubTransList.entries()-1);}
  CWLTransactionData* AddSubTransaction(const char* pName, double dTime=0, CWLTransactionData::TransactionType eType=CWLTransactionData::TRANS_GENERAL);
  RVSlistCollectables* GetSubTransList(){return &m_SubTransList;}  // Chaim - return pointer
  bool IsListEmpty() {return m_SubTransList.isEmpty() ? true : false ;}
  int  NumOfTransactions()  {return m_SubTransList.entries()+1;}
  int  NumOfSubTransactions(){return m_SubTransList.entries();}
  int  GetFirstSubTransPos() {return 1;}
  CWLTransactionData* GetNextTrans(int &pos);
  CWLTransactionData* FindTrans(const char* pName);
  void AddEvent(  const char* reason, 
                  const char* description, 
                  WLLogMessage::Severity severity = WLLogMessage::Success, 
                  const char* extraDataType = NULL, 
                  void* extraData = NULL, 
                  unsigned long len = 0);


  const char* GetObjUrl(){return m_Trans.GetName();}
  HtmlQueueItemType GetObjType(){return m_ObjType;}
  void SetObjType(HtmlQueueItemType objType){m_ObjType = objType;}
  void SetObj(RVCollectable* pObject){m_pObject = pObject;}
  RVCollectable* GetObject(){return m_pObject;}
  const char* GetOriginalUrl();
  const char* GetReferer();
  bool        IsCached(){return m_bCached;}
  void        SetCached(bool bCached){m_bCached = bCached;}

  void SetFailureReason(const char* reason, const char* description);

  // Chaim - for Parse Once.
  void Reset();

  void SetUrlOffset(int offset) { m_nUrlOffset = offset; }
  int  GetUrlOffset() { return m_nUrlOffset; }

  void ResetFileSize(int p_nFileSize = 0);
  void ResetChecksum(CHECKSUM_VAR p_CheckSum = 0);
  int  AddFileSize(int p_nFileSize);
  static CHECKSUM_VAR CalcCheckSum(const void* p_pvBuff, int p_nLength);
  CHECKSUM_VAR AddChecksum(const void* p_pvBuff, int p_nLength);
  int GetFileSize();
  CHECKSUM_VAR GetCheckSum();
  double GetSubTransTime(CWLTransactionData::TransactionType eType);

protected:
  CWLTransactionData  m_Trans ;
  RVSlistCollectables m_SubTransList ;  
  
  HtmlQueueItemType   m_ObjType ;
  RVCollectable*      m_pObject ;
  bool                m_bCached ;
  int	                m_nUrlOffset ;
  int                 m_nFileSize ; 
  CHECKSUM_VAR        m_Checksum ;

protected:
  CWLTransactionsObject *m_NextTransObj;
public:
  CWLTransactionsObject* next() { return m_NextTransObj; }
  void setNext(CWLTransactionsObject *tObj) { m_NextTransObj = tObj; }

};


#endif  //_WLTRANSACTIONDATA_H_
