/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include <iostream.h>
#include <regexp.h>
#include "wlhtmlfuncs.h"
#include "wlform.h"
#include "wlelement.h"
#include "wloption.h"
#include "platform.h"
#include "wlurlarray.h"
#include "wlsrc.h"
#include "wldocument.h"
#include "wlhtmlvirtualqueue.h" 


CWLForm::CWLForm(CWLDocument* docPtr):
								 m_DocPtr(docPtr),
								 m_Method("POST"),
								 formState(GET_ACTION_PROPERTIES),
								 elementState(FIND_ATT_VALUE),
								 optionsState(FORM_READ_MORE),
								 inOptionState(FIND_ATT_VALUE),
								 currentOptionSelected(false), 
								 previousOptionSelected(false),
								 currentOptionIndex(0),
								 m_pAction(NULL)
{
	checked = actionFound = false;
	optionsPtr = NULL;
	m_Index = m_DocPtr->m_Forms->entries();
    m_StartInnerHTML = -1;
    m_LenInnerHTML = -1;
	selectedIndex = 0;
}

CWLForm:: ~CWLForm()
{
	m_Elements.clearAndDestroy();
	if(m_pAction)
		delete m_pAction;
}


CWLForm& CWLForm::operator=(CWLForm& aCWLForm)
{
  m_Elements			= aCWLForm.Elements();
	m_pAction				= new CWLLocation;
	*m_pAction			= aCWLForm.Action();
	m_Method				=	aCWLForm.Method();
	m_Name					=	aCWLForm.Name();
	return (*this);
}



inline size_t CWLForm::findSpaceIndex(RVCString& buff, size_t startUrlIndex)
{
	RVCRegexp sp("[\\t\\r\\n ]");
  return buff.index(sp, startUrlIndex);
}

inline int		CWLForm::findSpace(char& c)
{
  if(c == ' ' || c == '\t' || c == '\n' || c == '\r' )
    return 1;

  return 0;
}

formElementState  CWLForm::findInstance(RVCString& buff)
{
	RVCRegexp sp("[\\t\\r\\n ]");
	size_t lt = 0;
  while((lt = buff.index("<")) != RV_NPOS)
  {
		if(buff.length() - lt > 9)
		{
			m_DocPtr->m_iBuffPos += lt + 1;
			buff.remove(0, lt + 1);
			if(isElementOfType("input", 5, buff, ' ', m_DocPtr))		
				return FORM_INPUT;
			else if(isElementOfType("select", 6, buff, '>', m_DocPtr))		
				return FORM_SELECT;
			else if(isElementOfType("/select", 7, buff, '>', m_DocPtr))
				return FORM_SELECT_END;
			else if(isElementOfType("textarea", 8, buff, '>', m_DocPtr))		
				return FORM_TEXTAREA;
			else if(isElementOfType("/form", 5, buff, '>', m_DocPtr)) {
				m_LenInnerHTML = m_DocPtr->m_iBuffPos - 6 - m_StartInnerHTML;
				return FORM_END;
			} else if(isElementOfType("a", 1, buff, ' ', m_DocPtr)) {
				m_DocPtr->m_CurLink = NULL;
				return FORM_A_ANCHOR;
			} else if(isElementOfType("/a", 2, buff, '>', m_DocPtr)) {
				m_DocPtr->m_CurLink = NULL;
				continue;
			} else if(isElementOfType("area", 4, buff, ' ', m_DocPtr))
				return FORM_AREA_ANCHOR;
			else if(isElementOfType("img", 3, buff, ' ', m_DocPtr))
				return FORM_IMG;
			else if(isElementOfType("bgsound", 7, buff, ' ', m_DocPtr))
				return FORM_BGSOUND;
			else if(isElementOfType("embed", 5, buff, ' ', m_DocPtr))
				return FORM_EMBED;
			else if(isElementOfType("object", 6, buff, ' ', m_DocPtr))
				return FORM_OBJECT;
			else if(isElementOfType("applet", 6, buff, ' ', m_DocPtr))
				return FORM_APPLET;
			else if(isElementOfType("script", 6, buff, '>', m_DocPtr))
				return FORM_SCRIPT;
			else if(isElementOfType("xml", 3, buff, '>', m_DocPtr))
				return FORM_XML;
			else if(isElementOfType("option", 6, buff, '>', m_DocPtr))
				return FORM_OPTION;
			else if(isElementOfType("/option", 7, buff, '>', m_DocPtr))
				return FORM_END_OPTION;
			else if(isElementOfType("!--", 3, buff, '~', m_DocPtr))
				return FORM_COMMENT;
			else if(isElementOfType("table", 5, buff, '>', m_DocPtr))
				return FORM_TABLE;
			else if(isElementOfType("iframe", 6, buff, ' ', m_DocPtr))
				return FORM_IFRAME_TAG;
			else//!-- different tag  						
				continue;//!while , look for next tag						
		}
		else//!if -- buffLength - lt < 10
			return FORM_READ_MORE;
	}//!while
	return FORM_READ_MORE;
}

int CWLForm::getName(RVCString& buff, RVCString& str, size_t buffPtr, size_t delimiterIndex)    //!<INPUT
{
	RVCRegexp REname("[Nn][Aa][Mm][Ee][\\r\\n\\t ]*=");
  size_t nameIndex = buff.index(REname, buffPtr);    
  if(nameIndex != RV_NPOS && nameIndex < delimiterIndex)
  {
    buffPtr = nameIndex + 4;      
    str = getString(str, buff, buffPtr); //!find the name- - - - 
  }
  
    else//! no "NAME ="
	{
      str.remove(0);
      return -1;
	}
  
  return 0;//
}


int  CWLForm::getValue(RVCString& buff, RVCString& str, size_t buffPtr, size_t delimiterIndex)    //!<INPUT
{
	RVCRegexp REvalue("[Vv][Aa][Ll][Uu][Ee][\\r\\n\\t ]*=");
  size_t valueIndex = buff.index(REvalue, buffPtr);    
  if(valueIndex != RV_NPOS && valueIndex < delimiterIndex)
  {      
    buffPtr = valueIndex + 5;
    str = getString(str, buff, buffPtr); //!find the value- - - - 
  }
  
  else
	{
    str.remove(0);
    return -1;
	}
  
  return 0;
}
  


int CWLForm::getType(RVCString& buff, RVCString& str, size_t buffPtr, size_t delimiterIndex)    //!<INPUT
{ 
	RVCRegexp REtype("[Tt][Yy][Pp][Ee][\\r\\n\\t ]*=");
  size_t typeIndex = buff.index(REtype, buffPtr);    
  if(typeIndex!=RV_NPOS && typeIndex < delimiterIndex)
  {
    buffPtr = typeIndex + 4;      
    str = getString(str, buff, buffPtr); //!find the name- - - - 
  }
  else
	{
    str.remove(0);
    return -1;//!NO TYPE
	}
  
  return 0;
}
                  


RVCString& CWLForm::getString(RVCString& str, RVCString& buff, size_t buffPtr)  //!get a desired string value(URL, NAME= , VALUE=, ....)
{
  int beginIndex = buff.index("=", buffPtr);
  int endIndex; 
  while(findSpace(buff(++beginIndex))){}
  if(buff(beginIndex)=='"')  
    endIndex = buff.index('"', ++beginIndex );
  else  if(buff(beginIndex)=='\'' )
  {  
    ++beginIndex;
    endIndex = buff.index('\'', beginIndex + 1);
  }  
  else
  {
    size_t spaceIndex = findSpaceIndex(buff, beginIndex);
		size_t delimiterIndex = buff.index(">",beginIndex);           
    if (spaceIndex < delimiterIndex && spaceIndex != RV_NPOS) 
      endIndex = spaceIndex;
    else 
      endIndex = delimiterIndex;
  }

  str = buff(beginIndex, endIndex - beginIndex );//! in  the book the type is RWCSubstring
  return str;
}
/******************* parse text area*************************/
elementReadState CWLForm::parseTextArea(RVCString& buff)
{
	size_t buffPtr = 0, attValueStart = RV_NPOS;
	type = "textarea";
  RVCString attName, attValue;

  while(1)
  {
    switch(elementState) 
    {  
			case(FIND_ATT_VALUE):  
				while(1)
				{
					attState textState = htmlFindNextAttribute(buff, attName, attValue, buffPtr, attValueStart);
					if(textState == HTML_CURRENT_ATT_COMPLETE)
					{
						switch(kindOfAttribute(attName)) {
						case ID:
							id = attValue.data();
							break;
						case NAME:
							name = attValue.data();
							break;
						}
						continue;
					}										
					else if (textState == HTML_FOUND_DELIMITER)
					{
						elementState = FIND_NAME_VALUE;
						size_t _lt = buff.index(">");
						m_DocPtr->m_iBuffPos += _lt + 1;
						buff.remove(0, _lt + 1);//!get ready to extract name
						break;//!while
					}
					else//!HTML_READ_MORE_DATA
						return READ_MORE;
				}//!while						
			case (FIND_NAME_VALUE):  
				size_t endIndex = buff.index("</textarea>",0 , RVCString::ignoreCase);
				if(endIndex != RV_NPOS)
				{
					value = buff(0, endIndex);
					m_DocPtr->m_iBuffPos += endIndex + 11;
					buff.remove(0, endIndex + 11);
					elementState = FIND_ATT_VALUE;//!reset for next element
					return END_OF_ELEMENT;				
				}
				else//!couldnt find </textarea>
					return READ_MORE;				
		}//!switch
	}//!while
	return READ_MORE;
}

elementReadState CWLForm::parseSelect(RVCString& buff)
{
	size_t buffPtr = 0, attValueStart = RV_NPOS;
	type = "select";
	RVCString attName, attValue;
	RVCString optionText;
	while(1)
	{
		switch(elementState)
		{
			case(FIND_ATT_VALUE)://!first we have to find the atts in <select...>
				while(1)
				{
					attState selectAttState = htmlFindNextAttribute(buff, attName, attValue, buffPtr, attValueStart);
					if(selectAttState == HTML_CURRENT_ATT_COMPLETE)
					{
						switch(kindOfAttribute(attName)){
						case ID:   selectId = attValue.data(); break;
						case NAME: name = attValue.data(); break; // oooo ????
						case TYPE: type = attValue.data(); break;
						}
					}										
					else if (selectAttState == HTML_FOUND_DELIMITER)
					{
						elementState = FIND_OPTIONS;
						selectedIndex = 0;
						optionsPtr = new RVOrdered;//!done once
						size_t _lt = buff.index(">");
						m_DocPtr->m_iBuffPos += _lt + 1;
						buff.remove(0, _lt + 1);//!get ready to extract name
						buffPtr = 0;
						break;//!while
					}
					else//!HTML_READ_MORE_DATA
						return READ_MORE;
				}//!while
			case(FIND_OPTIONS)://!more like FIND_OPTIONS_VALUE, dont want to add another enum
				bool foundValueArgument = false;
				while(1)
				{
					if(optionsState != FORM_OPTION)			//!if we are not in this state - look for next option,
						optionsState = findInstance(buff);	//!initially its FORM_READ_MORE
						
					switch(optionsState)
					{
						case FORM_COMMENT:
							switch(skipHtmlBlock(buff,"-->", 3, m_DocPtr))
							{
								case END_OF_ELEMENT:
									//!formState = GET_INSTANCE;
									optionsState = FORM_READ_MORE;
									break;
								case(READ_MORE):
									return READ_MORE;
							}
							break;

						case FORM_OPTION:
							switch(inOptionState)
							{
								case(FIND_ATT_VALUE):
									while(1)
									{
										attState optionParseState = htmlFindNextAttribute(buff, attName, attValue, buffPtr, attValueStart);

										if(optionParseState == HTML_CURRENT_ATT_COMPLETE)
										{
											switch(kindOfAttribute(attName)){
											case ID:    id = attValue.data(); break;
											case VALUE:
												value = attValue;
												foundValueArgument = true;
												break;
											}
											continue;
										}										
										else if (optionParseState == HTML_FOUND_DELIMITER)
										{
											RVCRegexp REselected("[\\t\\r\\n ][sS][eE][lL][eE][cC][tT][eE][dD]");
											size_t delimiterIndex = buff.index(">");
											RVCString tempBuff = buff(0, delimiterIndex);
											++currentOptionIndex;
											if(tempBuff.index(REselected) != RV_NPOS)										
											{
												currentOptionSelected  = true;
												if(!previousOptionSelected)
												{
													selectedIndex	= currentOptionIndex - 1;
													previousOptionSelected	= true;
												}
											}

											inOptionState = FIND_NAME_VALUE;							
											size_t _lt = buff.index(">");
						                    m_DocPtr->m_iBuffPos += _lt + 1;
											buff.remove(0, _lt + 1);//!get ready to extract text
											break;//!while
										}
										else//!HTML_READ_MORE_DATA
											return READ_MORE;
									}//!while
								
									case(FIND_NAME_VALUE):
										buffPtr = buff.index("<");
										if(buffPtr != RV_NPOS)
										{
											optionText = buff(0, buffPtr);
											while(optionText.length() > 0 && findSpace(optionText(0)))
												optionText.remove(0, 1);
											while(optionText.length() > 0 && findSpace(optionText(optionText.length() - 1)))
												optionText.remove(optionText.length() - 1);

											if (!foundValueArgument)
											{
												value = optionText;
											}

											CWLOption* currentOptionPtr = new CWLOption(m_DocPtr, value.data(), optionText.data(),  id.data(), currentOptionSelected);
											currentOptionPtr->SetInnerHTMLOffset(m_DocPtr->m_iBuffPos, buffPtr);
											optionsPtr->insert(currentOptionPtr);
						                    m_DocPtr->m_iBuffPos += buffPtr;
											buff.remove(0, buffPtr);//!leave the '<' in buff
											buffPtr = 0;											
											inOptionState = FIND_ATT_VALUE;//!reset
			 								optionsState = FORM_READ_MORE;//!reset
											currentOptionSelected = false;//!reset
											value = ""; //!reset
											optionText = "";//!reset
											id = ""; //!reset
											foundValueArgument = false;
											break;//!switch -- inOptionState
										}
										else
											return READ_MORE;											
							}//!switch -- inOptionState
							break;//!switch - optionsState						
						case(FORM_SELECT_END):
							elementState = FIND_ATT_VALUE;//!reset - done with select
							currentOptionSelected = false;
							return END_OF_ELEMENT;

						case(FORM_READ_MORE):
							return READ_MORE; 
						default:
							break;
					}//!switch -- optionsState
				}//!while
		}//!switch -- elementFlag
	}//!while
}
				


/************************parse input****************************/
elementReadState CWLForm::parseInput(RVCString& buff, urlBase *urlBasePtr)    //!<INPUT
{
	RVCString attributeName, attributeValue;
	RVCString url, id, origUrl;
	size_t buffPtr = 0, attValueStart = RV_NPOS, attValueStartF = RV_NPOS;

	while(1)
	{
		attState parseState = htmlFindNextAttribute(buff, attributeName, attributeValue, buffPtr, attValueStart);
		if(parseState == HTML_READ_MORE_DATA)
			return READ_MORE;
		else if(parseState == HTML_CURRENT_ATT_COMPLETE) {
			switch(kindOfAttribute(attributeName)) {
			case NAME: name = attributeValue.data(); 
				break;
			case SRC:  
				attValueStartF = attValueStart;
				origUrl = attributeValue;
				url = urlBasePtr->calcUrl(attributeValue);
				break;
			case VALUE: value = attributeValue.data(); 
				break;
			case TYPE: 
				type = attributeValue.data();
                if(!type.compareTo("checkbox", RVCString::ignoreCase) ||
                   !type.compareTo("radio",    RVCString::ignoreCase)  ) {
					if(!value.data()[0] || value.isNull())
						value = "on";
                } 
				break;
			case ID: id = attributeValue.data();
				break;
			case CHECKED:   checked = true;
				break;
			case UNCHECKED: checked = false;
				break;
			}
		} else { //!parseState == HTML_FOUND_DELIMITER
			//! parseSrc* doc_Srcs = m_DocPtr->m_SrcList;
			RVOrdered *doc_Srcs = m_DocPtr->m_wlAll;
			if (doc_Srcs && !type.compareTo("image", RVCString::ignoreCase)) {
				CWLEmbed* srcEntryPtr = new CWLEmbed("input", "image", url, origUrl, m_DocPtr, name.data(), id.data(), "");
				if(!doc_Srcs->insert(srcEntryPtr))
					delete srcEntryPtr;
				else if (!origUrl.isNull()) {
					CWLTransactionsObject* TOptr = srcEntryPtr->GetTransactionObj();
					if (attValueStartF != RV_NPOS) {
						size_t uo = m_DocPtr->m_iBuffPos + attValueStartF;
						TOptr->SetUrlOffset(uo);
						srcEntryPtr->setUrlOffset(uo);
					}
					m_DocPtr->insertTransObjToQueue(TOptr);
				}
			}
			break;
		} 
	}
	return END_OF_ELEMENT;;
}
      
elementReadState CWLForm::parse(RVCString& buff, urlBase* urlBasePtr)
{
	RVCString actionStr_;
  while(1)
  {
    switch(formState)  
    {
    
    case GET_ACTION_PROPERTIES:	
		{
			switch(extractProperties( buff, urlBasePtr))
			{
				case READ_MORE:		
					return READ_MORE;
				
				case END_OF_ELEMENT:
					formState = GET_INSTANCE;
			}
		}
		case GET_INSTANCE:
			switch(findInstance(buff))
			{
				case FORM_INPUT:
					formState = GET_INPUT;
					break;
				case FORM_TEXTAREA:
					formState = GET_TEXTAREA;
					break;
				case FORM_SELECT:
					formState = GET_SELECT;
					break;
				case FORM_A_ANCHOR:
					if(m_DocPtr->m_Links)
						formState = GET_A_ANCHOR;
					break;
				case FORM_AREA_ANCHOR:
					if(m_DocPtr->m_Links)
						formState = GET_AREA_ANCHOR;
					break;
				case FORM_IMG:
					if(m_DocPtr->m_SrcList)
						formState = GET_IMG;
					break;
				case FORM_BGSOUND:
					if(m_DocPtr->m_EmbedList)
						formState = GET_BGSOUND;
					break;
				case FORM_EMBED:
					if(m_DocPtr->m_EmbedList)
						formState = GET_EMBED;
					break;
				case FORM_OBJECT:
					if(m_DocPtr->m_AppletList)
						formState = GET_OBJECT;
					break;
				case FORM_APPLET:
					if(m_DocPtr->m_AppletList)
						formState = GET_APPLET;
					break;
				case FORM_SCRIPT:
// oooo					if(m_DocPtr->m_Scripts)
						formState = GET_SCRIPT;
					break;
				case FORM_XML:
					formState = GET_XML;
					break;
				case FORM_TABLE:
					if(m_DocPtr->m_Tables)
						formState = GET_TABLE;
					break;
				case FORM_IFRAME_TAG:
					formState = GET_IFRAME_TAG;
					break;
				case FORM_READ_MORE:
					return  READ_MORE;
				case FORM_END:
					return END_OF_ELEMENT;
				case FORM_COMMENT:
					formState = GET_COMMENT;
				default:
					break;
			}
			continue;//
			
		case GET_INPUT:
			switch(parseInput(buff, urlBasePtr))
			{
				case END_OF_ELEMENT:
					formState = COMPLETE_ELEMENT;
					break;
				case READ_MORE:
					return READ_MORE;
			}
			break;
			
		case GET_TEXTAREA:
			switch(parseTextArea(buff))
			{
				case END_OF_ELEMENT:
					formState = COMPLETE_ELEMENT;
					break;
				case READ_MORE:
					return READ_MORE;
			}
			break;
		case GET_SELECT:
			switch(parseSelect(buff))
			{
				case END_OF_ELEMENT:
					formState = COMPLETE_ELEMENT;
					break;
				case READ_MORE:
					return READ_MORE;
			}
			break;
		case GET_A_ANCHOR:
			switch(m_DocPtr->m_Links->parse(buff, A_ANCHOR, urlBasePtr))
			{
				case END_OF_ELEMENT:
					formState = GET_INSTANCE;
					break;
				case READ_MORE:
					return READ_MORE;
			}
			break; 

		case GET_AREA_ANCHOR:
			switch(m_DocPtr->m_Links->parse(buff, AREA_ANCHOR, urlBasePtr))
			{
				case END_OF_ELEMENT:
					formState = GET_INSTANCE;
					break;
				case READ_MORE:
					return READ_MORE;
			}
			break;

		case GET_IMG:
			switch(m_DocPtr->m_SrcList->parse(buff, urlBasePtr, "img"))
			{
				case END_OF_ELEMENT:
					formState = GET_INSTANCE;
					break;
				case READ_MORE:
					return READ_MORE;
			}
			break;
		case GET_BGSOUND:
			switch(m_DocPtr->m_EmbedList->parse(buff, urlBasePtr, "bgsound"))
			{
				case END_OF_ELEMENT:
					formState = GET_INSTANCE;
					break;
				case READ_MORE:
					return READ_MORE;
			}
			break;
		case GET_EMBED:
			switch(m_DocPtr->m_EmbedList->parse(buff, urlBasePtr, "embed"))
			{
				case END_OF_ELEMENT:
					formState = GET_INSTANCE;
					break;
				case READ_MORE:
					return READ_MORE;				}
			break;
		case GET_OBJECT:
			switch(m_DocPtr->addToApplets(buff, "object")) {
				case END_OF_ELEMENT:
					formState = GET_INSTANCE;
					break;
				case READ_MORE:
					return READ_MORE;
			}
			break;
		case GET_APPLET:
			switch(m_DocPtr->addToApplets(buff, "applet")) {
				case END_OF_ELEMENT:
					formState = GET_INSTANCE;
					break;
				case READ_MORE:
					return READ_MORE;
			}
			break;		
		case GET_SCRIPT:
			if (m_DocPtr->m_Scripts) {
				switch(m_DocPtr->addToScripts(buff))
				{
					case END_OF_ELEMENT:
						formState = GET_INSTANCE;
						break;
					case READ_MORE:
						return READ_MORE;
				}
				break;		
			} else {
				switch(skipHtmlBlock(buff,"</script>", 9, m_DocPtr))
				{
					case END_OF_ELEMENT:
						formState = GET_INSTANCE;
						break;
					case(READ_MORE):
						return READ_MORE;
				}
				break;
			}

		case GET_XML:
			switch(m_DocPtr->addToXmls(buff))
			{
				case END_OF_ELEMENT:
					formState = GET_INSTANCE;
					break;
				case READ_MORE:
					return READ_MORE;
			}
			break;			
//
		case(GET_COMMENT):
			switch(skipHtmlBlock(buff,"-->", 3, m_DocPtr))
			{
				case END_OF_ELEMENT:
					formState = GET_INSTANCE;
					break;
				case(READ_MORE):
					return READ_MORE;
			}
			break;
//
		case GET_TABLE:
			switch(m_DocPtr->m_Tables->parse(buff, urlBasePtr))
				{
					case END_OF_ELEMENT:
						formState = GET_INSTANCE;
						break;
					case READ_MORE:
						return READ_MORE;
				}
			break;

		case GET_IFRAME_TAG:
			switch(m_DocPtr->m_Frames->parse(buff, urlBasePtr))	{
				case(END_OF_ELEMENT):
					formState = GET_IFRAME_BODY;
					break;
				case(READ_MORE):
					return READ_MORE;
			}
			break;

		case(GET_IFRAME_BODY):
			switch(skipHtmlBlock(buff,"</iframe>", 9, m_DocPtr))
			{
				case(END_OF_ELEMENT):
					formState = GET_INSTANCE;
					break;
				case(READ_MORE):
					return READ_MORE;
			}
			break;

//
		case COMPLETE_ELEMENT: 
			{
				if (type != "select") selectedIndex = 0;
				CWLElement* elementPtr = new CWLElement(name.data(), value.data(),type.data(), type == "select" ? selectId.data() : id.data(), checked, selectedIndex, optionsPtr);
				optionsPtr = NULL;
			  m_Elements.insert(elementPtr);
				name.remove(0);value.remove(0);type.remove(0); id.remove(0);selectId.remove(0);			
				checked = false;selectedIndex = 0;
				formState = GET_INSTANCE;
				break;;
			}
  
    }//!switch
	
  }//!while

  return END_OF_ELEMENT;
}

CWLElement* CWLForm::Element(const char* elementName)
{
	size_t entries_ = m_Elements.entries();
	if(entries_ < 1)
		return NULL;
	for(size_t elementIndex = 0; elementIndex < entries_; ++elementIndex)
	{
		if( ( *(CWLElement*)(m_Elements[elementIndex]) ).Name() == RVCString(elementName))
			return (CWLElement*)(m_Elements[elementIndex]);
	}
	return NULL;
}

elementReadState CWLForm::extractProperties(RVCString& buff, urlBase* urlBasePtr)
{//!we get ....action=value name=value...>
	RVCString attName, attValue;
	RVCString actionStr , actionOrigStr; 
	size_t buffPtr = 0, attValueStart = RV_NPOS;
	while(1)
	{
		attState formState = htmlFindNextAttribute(buff, attName, attValue, buffPtr, attValueStart);
		if(formState == HTML_CURRENT_ATT_COMPLETE)
		{
			switch(kindOfAttribute(attName)) {
			case ACTION:
				actionOrigStr = attValue;
				actionStr = urlBasePtr->calcUrl(attValue);
				break;
			case METHOD:
				if(attValue.compareTo("get", RVCString::ignoreCase) == 0)
					m_Method = "GET";
				break;
			case NAME:
				m_Name = attValue.data();
				break;
			case ID:
				m_Id = attValue.data();
				break;
			case ENCODING:
			case ENCTYPE:
				m_Encoding = attValue.data();
				break;
			case TARGET:
				m_Target = attValue.data();
				break;
			}
		}																		
		else if (formState == HTML_FOUND_DELIMITER)
		{
				m_pAction = new  CWLLocation(actionStr, actionOrigStr, m_DocPtr->GetDocUrl(), m_Name.data(), m_Id.data(), m_Target.data());
				size_t _lt = buff.index(">");
				m_DocPtr->m_iBuffPos += _lt + 1;
				buff.remove(0, _lt + 1);//!get ready to extract name
				m_StartInnerHTML = m_DocPtr->m_iBuffPos;
				break;//!while
		}
			else//!HTML_READ_MORE_DATA
				return READ_MORE;
	}	
	return END_OF_ELEMENT;	
}
  
int CWLForm::findEntryByName(const char* name_ ,const char*& valuePtr )
{
  size_t _entries = m_Elements.entries();
  RVCString name(name_);   
  if(m_Elements.isEmpty()) //!no elements in form 
    return -3;
  for(size_t i = 0; i < _entries; ++i)
  {
	  RVCString elementName(m_Elements.Key(i));
	  if(!name.compareTo(elementName, RVCString::ignoreCase))//!return the first one you find
	  {
		  valuePtr = m_Elements.Value(i);
		  return 0;
	  }
	  	
  }
  return -2;      
}

RVCSubString CWLForm::GetInnerHTML()
{ 
	if (m_DocPtr && m_DocPtr->m_HtmlPagePtr) {
		RVCString &temp = *(m_DocPtr->m_HtmlPagePtr);
		return temp(GetStartInnerHTML(), GetLenInnerHTML());
	} else {
		RVCString temp;
		return temp(0,0);
	}
}

void CWLForm::printAll()
{
//	cout<<endl<<"FORM"<<endl;
//  cout<<"===="<<endl;
	cout<<"Form Name    :"<<Name().data()<<endl;
	cout<<"Form Id			:"<<Id().data()<<endl;
	cout<<"Form Method  :"<<Method().data()<<endl;
	cout<<"Form Action  :"<<Action().Href()<<endl;
	cout<<"Form Encoding: "<<Encoding()<<endl;
	cout<<"Form Target: "<<Target()<<endl;
	cout<<"Form Elements:"<<endl;
	for(size_t index = 0;index < Length(); index++)
	{
		CWLElement* elementPtr = (CWLElement*)(Elements()[index]);
		RVOrdered* optionsPtr = elementPtr->Options();
		cout<<(index + 1)<<")"<<" Name:    "<<elementPtr->Name()<<endl
													<<" Id:			 "<<elementPtr->Id()<<endl
													<<" Type:    "<<elementPtr->Type()<<endl
													<<" Value:   "<<elementPtr->Value()<<endl
													<<" Checked: "<<elementPtr->Checked()<<endl;
						
		size_t len;
		if(optionsPtr)
		{
			cout<<"selected index: "<<elementPtr->SelectedIndex();
			cout<<endl;
			len = (*optionsPtr).entries(); 
			cout<<"Num Options: "<<len<<endl;		
			for(size_t j = 0; j < len ; j++)
			{
	
				CWLOption* optionPtr = (CWLOption*)((*optionsPtr)[j]);
				cout<<"Option "<<j<<": value: "<<optionPtr->Value()
				<<"  text: "<<optionPtr->Text()
				<<"  Id:	 "<<optionPtr->Id();
				if(optionPtr->Selected()) 
					cout<<"[This option selected]";
				cout<<endl;
				cout << "StartInnerHTML: " << optionPtr->GetStartInnerHTML() << endl;
				cout << "LenInnerHTML:   " << optionPtr->GetLenInnerHTML()   << endl;
				RVCString iH = optionPtr->GetInnerHTML();
				cout << "InnerHTML:      " << iH.data()      << endl;
			}
		}													
	}
	cout << "StartInnerHTML: " << GetStartInnerHTML() << endl;
	cout << "LenInnerHTML:   " << GetLenInnerHTML()   << endl;
	RVCString iH = GetInnerHTML();
	cout << "InnerHTML:      " << iH.data()      << endl;
}
