/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include "wlframelist.h"
#include "wlhtmlvirtualqueue.h"
#include "wlhtmlfuncs.h"
#include "tracedll.h"


#ifdef __TV_UNIX__
extern const char* _itoa(int i, char* buf, int size);
#endif



parseFrame::parseFrame(	RVCString& pageName,
					   CWLUserParseFlags& parseFlags,
					   frameContainer* container,
					   CWLDocument* docPtr):												
    m_DocPtr(docPtr) 
{
	m_ParseFlags	= parseFlags;
	frameCont			= container;
	stateFlag			= 0;
	entityCount		= 0;
	buffPtr				= 0;
	name					= pageName;
}

parseFrame& parseFrame::operator=(parseFrame& aparseFrame)
{
	size_t _entries = aparseFrame.entries();
	if(!aparseFrame.isEmpty())
		for(size_t i = 0; i < _entries; ++i)
		{
			CWLDocument* docPtr = new CWLDocument();
			(*docPtr) = *(CWLDocument*)aparseFrame[i];
			insert(docPtr);
		}
		return (*this);
}

parseFrame::~parseFrame() {
	//! Here we go on all over the frames and tell each one of them that
	//! decrease our refernce to them. If the reference to them after that
	//! is Zero then we destroy that frame else we keep it alive
	//! and it will be alive until the reference from the Java Script world
	//! Will terminate it
	RVOrderedIterator iter(*this);
	while (iter())
	{
		CWLDocument* Frame = (CWLDocument*)iter.key();
		WlTraceMsg (">>DECREASING reference count for frame : %d",(int)Frame);
		if (Frame->decRefCount() == 0) {
			WlTraceMsg ("Deleting frame : %d",(int)Frame);
			delete Frame;
		}
	}
	clear();
}

RVCollectable *parseFrame::comparedObject(int index)
{
    return (RVCollectable *)&(((CWLDocument *)at(index))->GetLocation());
}

/* new 7.3.99************************************************/
elementReadState parseFrame::parse(RVCString& buff, urlBase* urlBasePtr)
{
	size_t buffPtr = 0, attValueStart = RV_NPOS, attValueStartF = RV_NPOS;
	RVCString attName, attValue, origUrl;
	RVCString url, frameUrl, frameName, id, target;

	while(1)
	{
		attState frameState = htmlFindNextAttribute(buff, attName, attValue, buffPtr, attValueStart);
		if(frameState == HTML_CURRENT_ATT_COMPLETE)
		{
			switch(kindOfAttribute(attName)) {
			case ID:
				id = attValue.data();
				break;
			case NAME:
				frameName = attValue.data();
				break;
			case SRC:
				attValueStartF = attValueStart;
				origUrl = attValue;
				url = urlBasePtr->calcUrl(attValue);
				break;
			case TARGET:
				target = attValue.data();
				break;
			}
		}
		else if (frameState == HTML_FOUND_DELIMITER)
		{
			char    buf[3];
			RVCString name_;		
			name_ = name + "." + _itoa(++entityCount, buf, 10);
			CWLDocument* frameEntryPtr = new CWLDocument(name_, frameName, url, 
				origUrl, id, target, m_ParseFlags, frameCont, m_DocPtr);

			//! If we create a new frame document ,
			//! it mean that it is referenced by it's parent
			WlTraceMsg ("Parsing a new frame, setting reference count to 1 (reference from parent frame) for the new frame: %d",(int)frameEntryPtr);
			frameEntryPtr ->incRefCount ();

			//! copy father's frameTarget and append the level
			CWLIdentLocation frameTarget;
			if(m_DocPtr) {
				frameTarget = m_DocPtr->GetFrameTarget();
				frameTarget.appendLevel( entries(), frameName); 
        frameEntryPtr->SetFrameTarget(frameTarget);
			}

			if(!insert(frameEntryPtr))
				delete frameEntryPtr;
			else if (!origUrl.isNull() && origUrl.compareTo("about:blank", RVCString::ignoreCase)) { 
				CWLTransactionsObject* TOptr = frameEntryPtr->GetTransactionObj();
				if (attValueStartF != RV_NPOS) {
					size_t uo = m_DocPtr->m_iBuffPos + attValueStartF;
					TOptr->SetUrlOffset(uo);
					frameEntryPtr->setUrlOffset(uo);
				}
				m_DocPtr->insertTransObjToQueue(TOptr);
			}
			
			size_t _lt = buff.index(">");
			m_DocPtr->m_iBuffPos += _lt + 1;
			buff.remove(0, _lt + 1);//!ready to continue parsing
			frameName.remove(0);
			id.remove(0);
			target.remove(0);
			return END_OF_ELEMENT;
		}
		else//!HTML_READ_MORE_DATA

			return READ_MORE;
	}

	return READ_MORE;
}

int parseFrame::findEntryByUrlMatch(const char* regExp, CWLDocument**  docPtrAdr)
{
	RVCRegexp regExpUrl(regExp);
	RVOrderedIterator iter(*this);
	if(!iter())
		return -3;
	do
	{
		CWLDocument* docPtr = (CWLDocument*)iter.key();
		*docPtrAdr = docPtr;
		RVCString curUrl((docPtr->GetLocation()).Href());
		size_t ext;
		size_t regIndex = curUrl.index(regExpUrl, &ext);
		if(regIndex == 0  &&  ext == curUrl.length())
			return 0;
	}
	while (iter());
	return -2; //!No entry with same name
}

findFrameStatus	parseFrame::findEntryByLocationMatch(CWLIdentLocation& identLocation, CWLDocument*& retFrame)
{

	RVCString             Name;
	RVCString             FrameName;
	int                   Number;
	int                   iterCount = 0;
	findFrameStatus       retStatus = findFrameStatus_ERROR;

	//! Initialize the return frame pointer to NULL
	retFrame = NULL;

	identLocation.getLevel (0,Number,Name);

	RVOrderedIterator iter(*this);

	//! Have we found the frame we are looking for ?
	bool Found = false;

	//! Have we finished searching - this variable will end the search after we
	//! do the recursive search inside the frames tree
	bool searchCompleted = false;

	while ((iter()) && (!retFrame) && (!searchCompleted))
	{
		CWLDocument* iterFrame = (CWLDocument*)iter.key();
		//! Here we check if the current frame we are on is the frame we are
		//! looking for delete. if we have more on the path it mean that we must
		//! iterate also on the frames hanged on this frame in order to find the
		//! frame we need to delete. If we dont have it mean that we have reaced the desired
		//! frame for deletion
		FrameName = iterFrame -> GetFrameName ();

		if (((!Name.isNull()) && (FrameName == Name)) || ((Number != NULL_IDENT_LOCATION_NUMBER) && (Number == iterCount))) {
			identLocation.removeLevel (0);
			if (identLocation.isEmpty()) {
				retFrame = iterFrame;
				retStatus = findFrameStatus_OK;
				//! Here we return the father and the name and index for
				//! the frame that need to be deleted
				if ((!Name.isNull()) &&
					(FrameName == Name) &&
					(Number != NULL_IDENT_LOCATION_NUMBER) &&
					(Number != iterCount)) {
					retStatus = findFrameStatus_NAME_MATCH_NUMBER_MISMATCH_WARNING;
				}
				else if ((Number != NULL_IDENT_LOCATION_NUMBER) &&
					(Number == iterCount) &&
					(!Name.isNull()) &&
					(FrameName != Name)) {
					retStatus = findFrameStatus_NUMBER_MATCH_NAME_MISMATCH_WARNING;
				}
			}
			else {
				if (iterFrame -> m_Frames) {
					retStatus = iterFrame -> m_Frames -> findEntryByLocationMatch (identLocation,retFrame);
				}
				searchCompleted = true;
			}
		}
		++iterCount;
	}
	return (retStatus);
}

int parseFrame::findEntryByFrameNameMatch(const char* regExp, CWLDocument**  docPtrAdr)
{
	RVCRegexp regExpName(regExp);
	RVOrderedIterator iter(*this);
	if(!iter())
		return -3;
	do
	{
		CWLDocument* docPtr = (CWLDocument*)iter.key();
		*docPtrAdr = docPtr;
		RVCString curFrameName(docPtr->GetFrameName());
		size_t ext;
		size_t regIndex = curFrameName.index(regExpName, &ext);
		if(regIndex == 0  &&  ext == curFrameName.length())
			return 0;
	}
	while (iter());
	return -2; //!No entry with same name
}


int parseFrame::getNameByIndex(RVCString& pn, int index, CWLDocument** docPtrAdr)
{
	char      temp[3];
	RVCString name_;
	name_ = pn + "." + _itoa(index, temp, 10);
	RVCollectableString fn(name_);
	size_t entries_ = entries();
	CWLDocument* docPtr;

	if (entries_ <= 0)
		return -3;//!No frames in this page
	if (index > (int)entries_)
		return -2;//!number of frames smaller than index
	if((docPtr = ((CWLDocument*)((*this)[index - 1])))->GetName() == name_)
	{
		*docPtrAdr = docPtr;
		return 0;
	}
	return -1;//!page not found
}

CWLDocument* parseFrame::frame(const char* frameName)
{
	size_t index, count = entries();
	if(count != RV_NPOS)
		for(index = 0;index < count; ++index)
		{
			CWLDocument* doc  = (CWLDocument*)at(index);
			RVCString& currentFrameName = doc->GetFrameName();
			if(!currentFrameName.compareTo(frameName, RVCString::exact))
				return doc;
		}
		return NULL;

}

void parseFrame::printAll()
{
	CWLDocument* CWLDocumentEntryPtr;
	RVOrderedIterator iter(*this);
	cout << "Frame Collection" << endl;
	cout << "================" << endl;
	while (iter()){
		CWLDocumentEntryPtr =(CWLDocument*)iter.key();
		RVCString docHref((CWLDocumentEntryPtr->GetLocation()).Href());
		RVCString docProtocol((CWLDocumentEntryPtr->GetLocation()).Protocol());
		CWLDocument* doc;
		findEntryByUrlMatch(docHref.data(), &doc);
		cout<<"URL:  "<<docHref<<endl
			<<"NAME: "<<(doc->GetLocation()).Name()<<endl
			<<"FRAME NAME:	"<<doc->GetFrameName()<<endl
			<<"FRAME ID:		"<<endl
			<<"PROTOCOL:  "<<((CWLDocumentEntryPtr->GetLocation()).Protocol())<<endl
			<<"SEARCH:   "<<endl;
		(CWLDocumentEntryPtr->GetLocation()).Array().PrintAllPairs();
		cout<<endl<<endl;
	}
}

