/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include  "wlurlbase.h"
#include "wlhtmlfuncs.h"

static void stripUrl(RVCString& url)
{
  //! exactly the same as parseUrl in httpload
  size_t i;

  // @@Migration
  if (url.length() == 0) return;

  while (url.length() > 0) {
    if (url(0) == '\'' || url(0) == ' ' || url(0) == '\t' || url(0) == '\n' || url(0) == '\r')
	  url.remove(0, 1);
	else
	  break;
  }

  while ((i = url.length() - 1) >= 0) {
    if (url(i) == '\'' || url(i) == ' ' || url(i) == '\t' || url(i) == '\n' || url(i) == '\r')
	  url.remove(i, 1);
	else
	  break;
  }

  for (i = 0; i < url.length(); ) {
    if (url(i) == '\n' || url(i) == '\r' || url(i) == '\t') {
//		if (atoi (url(i)) <= 31) {
	  url.remove(i, 1);
	  continue;
	} else if (url(i) == ' ')
	  url.replace(i, 1, "%20");
	++i;	
  }

}



urlBase::urlBase() :
  m_Parsed(false),
  m_subDirCount (0)
{
}

int urlBase::setBase(const RVCString& newBase_)
{
	size_t firstQMark = newBase_.first('?');
	if(firstQMark != RV_NPOS)
    m_base = newBase_(0,firstQMark);
  else
    m_base = newBase_;

  //! Take out all cr,lf and blanks from both sides (inside blanks are url-encoded)
  stripUrl (m_base);

  m_Parsed = false;

  return 1;
}


int urlBase::parse()
{
  if (m_Parsed)
    return 1;

  m_Parsed = true;

  RVCString newBase(m_base.data());
	
  if(newBase.index("http",0,RVCString::ignoreCase) == RV_NPOS) //!see that it is not <base target =....> 
	  return -1 ;
  //base = newBase_.data();
  m_subDirCount = 0;
  m_host.remove(0);
  m_path.remove(0);
  size_t index = newBase.index("http://",0,RVCString::ignoreCase);
  size_t indexs = newBase.index("https://",0,RVCString::ignoreCase);
  if(index == RV_NPOS && indexs == RV_NPOS) // buggy ??? != 0 check better ??? oooo
	return -1;
  m_protocol= (indexs == RV_NPOS ? "http://":"https://");
  newBase.remove(0, newBase.index("//") + 2);
  index = newBase.index("/");
  if(index == RV_NPOS)
  {
  m_host.append(newBase,newBase.length());
  m_path = "/";
  return 1;
  }
  m_host = newBase(0,index);
  newBase.remove(0,index);
  while((index = newBase.index("/")) != RV_NPOS)
  {
    ++m_subDirCount;
    m_path.append(newBase, index + 1);
    newBase.remove(0,index + 1);
  }
  if(m_path == "") 
    m_path.append("/");
  m_file = newBase(0,newBase.length());
  newBase.remove(0);
  return 0;
}

RVCString urlBase::calcUrl(RVCString& relUrl)
{   
	parse();

  // @@Migration
	  
  //! In any case of the url we remove
  //! the blanks from both sides and also inside cr and lf
  stripUrl (relUrl);
  if (relUrl.length() <= 1 ){
		  RVCString temp(m_protocol);
		  temp.append(m_host);
		  temp.append(m_path);
      return temp;
  }
	
  else {
	  int  qMark = relUrl.index("?");
	  int protoclDelimiter = relUrl.index("://");
	  
    if (protoclDelimiter != RV_NPOS && relUrl(0) != '/' &&
		  (protoclDelimiter < qMark || qMark == RV_NPOS))
		  return relUrl;  

	  if (relUrl(0) == '/' && relUrl(1) == '/') {
      relUrl.remove(0,2); // Chaim - remove // at start. they are redundant
		  relUrl.prepend(m_protocol);
		  return relUrl;
	  }
	  
	  if(!isHttpProtocol(relUrl)) //! check for http/s at the start of relUrl
		  return relUrl;
	  
	  if(relUrl(0) == '/')      //!start from host
	  {
		  RVCString temp(m_protocol);
		  temp.append(m_host);
		  temp.append(relUrl);
		  return temp;
	  }

    // @@Chaim - bug from AJAX demo site 
    else if(relUrl == "./"){
		  RVCString tempPath(m_path);
		  tempPath.prepend(m_host);
		  tempPath.prepend(m_protocol);
		  return tempPath;
    }
	  else if(relUrl(0,3) =="../")
	  {
		  RVCString tempPath(m_path);
		  int subDirPtr=0;
		  while(relUrl(0,3) == "../") //!how far should we go back in the sub dir hierarchy
		  {
			  ++subDirPtr;
			  relUrl.remove(0,3);
		  }
		  
		  if (subDirPtr+1>=m_subDirCount)
			  tempPath="/";
		  else
		  {
			  int counter = 0;  //! counts "../"
			  int index = 0 ;
			  while(m_subDirCount-subDirPtr-1>counter)
			  {
				  index = tempPath.index("/",index +1);
				  ++counter;
			  }
			  tempPath.remove(index+1);
		  }
		  tempPath.prepend(m_host);
		  tempPath.prepend(m_protocol);
		  tempPath.append(relUrl);
		  return tempPath;
	  }
	  else
	  {  
		  RVCString temp(m_protocol);
		  temp.append(m_host);
		  temp.append(m_path);
		  if(relUrl(0,2)=="./")
			  relUrl.remove(0,2);
		  else if(relUrl(0) == '#')
			  temp.append(m_file); 
		  temp.append(relUrl);
		  return temp;
	  }
  }
}

