/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include <RVCString.h>
#include <regexp.h>
#include <collstr.h>
#include <slistcol.h>
#include "wlsrc.h"
#include "tracedll.h"
#include "wlhtmlvirtualqueue.h"
#include "wllocation.h"
#include "wldocument.h"

parseSrc& parseSrc::operator=(const parseSrc& aparseSrc)
{
	size_t _entries = aparseSrc.entries();

	if(!aparseSrc.isEmpty())
		for(size_t i = 0; i < _entries; ++i)
		{
			RVCollectableString* strPtr = new RVCollectableString((*(RVCollectableString*)aparseSrc.at(i)).data());
			insert(strPtr);
		}
	return (*this);
}


//new
elementReadState parseSrc::parse(RVCString& buff, urlBase* urlBasePtr, const char* tagName)
{
	size_t buffPtr = 0, attValueStart = RV_NPOS;
	size_t attValueStartF = RV_NPOS, attValueStartL = RV_NPOS, attValueStartD = RV_NPOS;
  RVCString attName, attValue, origUrl, origLowsrcUrl, origDynsrcUrl;
  RVCString lowsrcUrl, dynsrcUrl;
  RVCString url, name, id, target;
  RVCString width, height, alt, title, useMap, origUseMap;
  RVCString innerHTML;

	while(1)
	{
		attState srcState = htmlFindNextAttribute(buff, attName, attValue, buffPtr, attValueStart);
		if(srcState == HTML_CURRENT_ATT_COMPLETE)
		{
			switch(kindOfAttribute(attName)) {
			case ALT:
				alt = attValue.data();
				break;
			case DYNSRC:
				attValueStartD = attValueStart;
				origDynsrcUrl = attValue;
				dynsrcUrl = urlBasePtr->calcUrl(attValue);
				break;
			case HEIGHT:
				height = attValue.data();
				break;
			case ID:
				id = attValue.data();
				break;
			case LOWSRC:
				attValueStartL = attValueStart;
				origLowsrcUrl = attValue;
				lowsrcUrl = urlBasePtr->calcUrl(attValue);
				break;
			case NAME:
				name = attValue.data();
				break;
			case SRC:
				attValueStartF = attValueStart;
				origUrl = attValue;
				url = urlBasePtr->calcUrl(attValue);
				break;
			case TARGET:
				target = attValue.data();
				break;
			case TITLE:
				title = attValue.data();
				break;
			case USEMAP:
				origUseMap = attValue;
				useMap = urlBasePtr->calcUrl(attValue);
				break;
			case WIDTH:
				width = attValue.data();
				break;
			}
		}
		else if (srcState == HTML_FOUND_DELIMITER)
		{//! tagName may be img|bgsound|embed
			//CWLEmbed* srcEntryPtr = new CWLEmbed(tagName, "src", url, origUrl, m_DocPtr, name.data(), id.data(), target.data());
			CWLEmbed* srcEntryPtr = new CWLEmbed(tagName, "src", url, origUrl, m_DocPtr, name.data(), id.data(), target.data());
			srcEntryPtr->SetProperties(width, height, alt, title, useMap, lowsrcUrl, origUseMap, origLowsrcUrl);

			if (m_DocPtr->m_CurLink)
				m_DocPtr->m_CurLink->SetInnerImg(srcEntryPtr);

			if(!stricmp(tagName, "bgsound")) {
				if(!m_DocPtr->m_wlAll || !m_DocPtr->m_wlAll->insert(srcEntryPtr))
					delete srcEntryPtr;
				else if (!origUrl.isNull()) { 
					CWLTransactionsObject* TOptr = srcEntryPtr->GetTransactionObj();
					if (attValueStartF != RV_NPOS) {
						size_t uo = m_DocPtr->m_iBuffPos + attValueStartF;
						TOptr->SetUrlOffset(uo);
						srcEntryPtr->setUrlOffset(uo);
					}
					m_DocPtr->insertTransObjToQueue(TOptr);
				}
			} else {
				if(!insert(srcEntryPtr))
					delete srcEntryPtr;
				else  if (!origUrl.isNull()) {
					CWLTransactionsObject* TOptr = srcEntryPtr->GetTransactionObj();
					if (attValueStartF != RV_NPOS) {
						size_t uo = m_DocPtr->m_iBuffPos + attValueStartF;
						TOptr->SetUrlOffset(uo);
						srcEntryPtr->setUrlOffset(uo);
					}
					m_DocPtr->insertTransObjToQueue(TOptr);
				}
			}

			if (!lowsrcUrl.isNull() && m_DocPtr->m_wlAll) {//! tagName supposed to be img
				srcEntryPtr = new CWLEmbed(tagName, "lowsrc", lowsrcUrl, origLowsrcUrl, m_DocPtr,
											name.data(), id.data(), target.data());
				if(!m_DocPtr->m_wlAll->insert(srcEntryPtr))
					delete srcEntryPtr;
				else if (!origLowsrcUrl.isNull()) { 
					CWLTransactionsObject* TOptr = srcEntryPtr->GetTransactionObj();
					if (attValueStartL != RV_NPOS) {
						size_t uo = m_DocPtr->m_iBuffPos + attValueStartL;
						TOptr->SetUrlOffset(uo);
						srcEntryPtr->setUrlOffset(uo);
					}
					m_DocPtr->insertTransObjToQueue(TOptr);
				}
			}

			if (!dynsrcUrl.isNull() && m_DocPtr->m_wlAll) { //! tagName supposed to be img
				srcEntryPtr = new CWLEmbed(tagName, "dynsrc", dynsrcUrl, origDynsrcUrl, m_DocPtr,
											name.data(), id.data(), target.data());
				if(!m_DocPtr->m_wlAll->insert(srcEntryPtr))
					delete srcEntryPtr;
				else if (!origDynsrcUrl.isNull()) {
					CWLTransactionsObject* TOptr = srcEntryPtr->GetTransactionObj();
					if (attValueStartD != RV_NPOS) {
						size_t uo = m_DocPtr->m_iBuffPos + attValueStartD;
						TOptr->SetUrlOffset(uo);
						srcEntryPtr->setUrlOffset(uo);
					}
					m_DocPtr->insertTransObjToQueue(TOptr);
				}
			}

			size_t _lt = buff.index(">");
			m_DocPtr->m_iBuffPos += _lt + 1;
			buff.remove(0, _lt + 1);//!ready to continue parsing

			return END_OF_ELEMENT;
		}
		else//!HTML_READ_MORE_DATA

			return READ_MORE;
	}

	return READ_MORE;
}

CWLEmbed* parseSrc::Src(const char* srcName)
{
	size_t entries_ = entries();
	if(entries_  < 1)
		return NULL;
	for(size_t srcIndex = 0; srcIndex < entries_; ++srcIndex)
	{
		if((RVCString) ( ((CWLEmbed*)at(srcIndex))->Name() ) == srcName )
			return (CWLEmbed*)at(srcIndex);
	}
	return NULL;
}

 void parseSrc::printAll()
{
  cout << "src list" << endl;
  cout << "----------------" << endl;
  for ( unsigned int i = 0; i < entries(); ++i)
  {
    CWLEmbed value(*((CWLEmbed* )at(i)));
    value.printAll();
  }
}
