/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//#include "wlregularexpressions.h"
#include  "wlarray.h"


//===========================CKVP=======================


CKVP::CKVP(const char* aKey, const char* aValue) :
  CWLCollectableString(aKey)
{ 
	m_Value.setValue(aValue);
}


CKVP& CKVP::operator=(CKVP& aCKVP) 
{	
	setKey(aCKVP.Key());
	setValue(aCKVP.Value());
	return *this;	
}

const char* CKVP::Key() 
{	
	return data(); 
}
  

const char* CKVP::Value()  
{
	return m_Value.data(); 
}


void CKVP::setKey(const char* key_)
{	
  setValue(key_);
}

void CKVP::setValue(const char* value_)
{
	m_Value.setValue(value_);
}
  

CWLSearchArray::CWLSearchArray() : m_URLEncoded ("") 
{
	if(m_ParseCharInitialize == false)
		ParseCharForHttp();
}

CWLSearchArray& CWLSearchArray::operator=(const CWLSearchArray& aCWLSearchArray)
{
	m_URLEncoded = aCWLSearchArray.retURLEncodedString();
	return (*this);	
}


static void uudecode_string(RVCString& UrlString)
{
  size_t strPtr = 0;
  while ((strPtr = UrlString.index("+", strPtr)) != RV_NPOS)
    UrlString[strPtr] = ' ';
  while ((strPtr = UrlString.index("%", strPtr)) != RV_NPOS)
  {
    int tmpInt = 0;
    RVCString chars(UrlString(strPtr + 1,2)); //the 2 chars that make the ascii code
    sscanf(chars.data(), "%x", &tmpInt);
    char symbol[2] = {(char)tmpInt, '\0'};
    UrlString.replace(strPtr++,3, (const char*)symbol);
  }
}

bool CWLSearchArray::LoadURLEncodedString(const char* urlString)
{
  RVCString UrlString = urlString;
  /*
//==============uudecode===============================
  size_t strPtr = 0;
  while ((strPtr = UrlString.index("+", strPtr)) != RV_NPOS)
    UrlString[strPtr] = ' ';
  while ((strPtr = UrlString.index("%", strPtr)) != RV_NPOS)
  {
    int tmpInt = 0;
    RVCString chars(UrlString(strPtr + 1,2)); //the 2 chars that make the ascii code
    sscanf(chars.data(), "%x", &tmpInt);
    char symbol[2] = {(char)tmpInt, '\0'};
    UrlString.replace(strPtr++,3, (const char*)symbol);
  }
  //=====================================================       
  */
  size_t prevAptr = 0;
  size_t nextAptr = 0;
  if((nextAptr = UrlString.first('&')) == RV_NPOS)
    nextAptr = UrlString.length();

  while(prevAptr < UrlString.length())
  {
    RVCString key ;
    RVCString value;
    size_t eqlPtr = UrlString.index("=", prevAptr);
    if( eqlPtr != RV_NPOS && eqlPtr < nextAptr)//"...&key=[value,""]&...."
    {
      key   = UrlString(prevAptr, eqlPtr - prevAptr );

      value = UrlString(eqlPtr + 1, nextAptr - eqlPtr - 1);            
      if(!value.data()[0] || value.isNull())
        value = "$WL$EMPTY$STRING$";             
    }
    else // "....&key&
    {
      key   = UrlString(prevAptr, nextAptr - prevAptr);
      value = "$WL$VOID$STRING$";             
    }
	uudecode_string(key);
	uudecode_string(value);
    insertKeyAndValue(key.data(), value.data());
    prevAptr = nextAptr + 1;
    if ((nextAptr = UrlString.index("&", prevAptr )) == RV_NPOS)
      nextAptr = UrlString.length();
  }
  return true;
}

const char* CWLSearchArray::GetURLEncodedString()
{
  m_URLEncoded = "";
  size_t _entries = entries();

  if(_entries != RV_NPOS)
    for(size_t i = 0 ; i < _entries; ++i)
    {
			RVCString retVal;
      RVCString kvp = ParseCharFunc(Key(i), retVal);        
      RVCString value = Value(i);
      if (value == "$WL$EMPTY$STRING$")
        kvp += "=";
      else if (value == "$WL$VOID$STRING$")
        NULL;
      else
        kvp += "=" + ParseCharFunc(value, retVal);

      //size_t strPtr=0;
      //while ((strPtr = kvp.index(" ", strPtr)) != RV_NPOS)
       // kvp[strPtr] = '+';
      m_URLEncoded += kvp;

      if (i+1 < _entries)
        m_URLEncoded += "&";
    }
  return m_URLEncoded.data();
}

void CWLSearchArray::ParseCharForHttp()
{		
		m_ParseCharInitialize = true;
		for (int i = 0; i < 256; ++i) {
			char c = (char)i;
			if (c == '*' || c == '-' ||
        c == '.' || (c >= '0' && c <= '9') ||
        (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z')) {
				m_ParseChar[i][0] = c;
				m_ParseChar[i][1] = '\0';
				continue;
    }
    
    if (c == ' ') {
      m_ParseChar[i][0] = '+';
      m_ParseChar[i][1] = '\0';
      continue;
    }
    
    m_ParseChar[i][0] = '%';
    int a = i / 16;
    if (a < 10)
      m_ParseChar[i][1] = '0' + (char)a;
    else 
      m_ParseChar[i][1] = 'A' + (char)(a - 10);
    int b = i - a*16;
    if (b < 10)
      m_ParseChar[i][2] = '0' + (char)b;
    else 
      m_ParseChar[i][2] = 'A' + (char)(b - 10);
      m_ParseChar[i][3] = '\0';
		}
	
}

RVCString& CWLSearchArray::ParseCharFunc(const char* val, RVCString& result)
{
	result = "";
  if (val == NULL)
    return result;
  int len = strlen(val);
  for (int i = 0; i < len; ++i) {
    int a = (int)val[i];
    result += m_ParseChar[a];
  }
  return result;
}

void CWLSearchArray::PrintAllPairs()
{

	for (size_t index = 0;index < entries(); ++index)		
		cout<<index<<")   KEY:				"<<Key(index)<<"     VALUE:				"<<Value(index)<<endl;
}

bool CWLSearchArray::m_ParseCharInitialize = false;
char CWLSearchArray::m_ParseChar[256][4];
/*
RVCString& CWLSearchArray::UrlEncode(const char* inVal , RVCString& retVal)
{
	RVCString tmp(inVal);
	switch()
	{		
		case('<'):          %3C
		case('>'):          %3E
    case('#'):          %23
    case('%'):          %25
    case('{'):          %7B
    case('}'):          %7D
    case('|'):          %7C
    case('\'):          %5C
    case('^'):          %5E
    case('~'):          %7E
    case('['):          %5B
    case(']'):          %5D
    case('`'):          %60
		case(';'):          %3B          
    case('/'):          %2F
    case('?'):          %3F
    case(':'):          %3A
    case('@'):          %40
    case('='):          %3D
    case('&'):          %26
*/


CWLArray::~CWLArray() 
{
	clearAndDestroy();
}

CWLArray&	CWLArray::operator=(const CWLArray& aCWLArray)
{
	size_t _entries = aCWLArray.entries();

	if(!aCWLArray.isEmpty())
		for(size_t i=0; i < _entries; ++i)
		{
			CKVP* arrPtr = new CKVP(*(CKVP*)aCWLArray[i]);
			insert(arrPtr);
		}
	return (*this);
}

void CWLArray::removeAllKeysAndValues(const char* aKey)
{
	size_t _entries = entries();
  size_t _index = 0;

  while (_index < _entries) {
    if (RVCString(Key(_index)) == RVCString(aKey)) {
      removeAt(_index);
      _entries--;
    }
    else {
      _index++;
    }
  }
}

const char* CWLArray::getAllValues(const char* aKey)
{
  RVCString values;
  size_t _entries = entries();

  for (size_t i = 0; i < _entries; ++i)
    if (RVCString(Key(i)) == RVCString(aKey)) 
    {
      if (!values.isNull())
        values += ",";
      values += Value(i);
    }

  return values.data();
}

void CWLArray::print()
{
	cout<<"NO. OF ENTRIES:  "<<entries()<<endl;
  for (size_t i = 0; i < entries(); i++)
    cout << Key(i) << ", " << Value(i) << endl;
}

const char* CWLArray::Value(const char* aKey)
{
	size_t entries_;
	entries_ = entries();
  for(size_t i=0; i< entries_; ++i)
    if(RVCString(Key(i)) == RVCString(aKey))
      return Value(i);
  
  return NULL; //no aKey found

}


RVCollectable* CWLArray::insertKeyAndValue(const char* aKey, const char* aValue)
{ 
	return insert(new CKVP(aKey, aValue)) ; 
}

RVCollectable* CWLArray::addKVP(CKVP* aKVP)
{
	return append(aKVP); 
}

const char* CWLArray::Key(int i)
{
	return ((CKVP*)(*this)[i])->Key(); 
}

const char* CWLArray::Value(int i)
{
	return ((CKVP*)(*this)[i])->Value(); 
}

//=====================================================================

