/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include <ordered.h>
#include "wloption.h"
#include "wlelement.h"

CWLElement& CWLElement::operator=(CWLElement& anElement)
{
//	*this = CWLElement(anElement.Name(), anElement.Value(), anElement.Type());
	setType(anElement.Type());
	setName(anElement.Name());
	setValue(anElement.Value());

	if (m_OptionsPtr && !m_OptionsPtr->isEmpty())
		m_OptionsPtr->clearAndDestroy();
	
	if(m_Type.compareTo("select", RVCString::ignoreCase) == 0) {
		if(!m_OptionsPtr)
			m_OptionsPtr = new RVOrdered;

		size_t entries_ = anElement.Options()->entries();
		for(size_t entry = 0; entry < entries_; ++entry)
		{			
			CWLOption* optionPtr	= new CWLOption();
			CWLOption* tempOptionPtr = (CWLOption*)((*anElement.Options())[entry]);
			(*optionPtr)					= *tempOptionPtr;
			m_OptionsPtr->insert(optionPtr);
		}
	} else {
		if(m_OptionsPtr) {
			delete m_OptionsPtr;
			m_OptionsPtr = NULL;
		}
	}
	return *this;
}

CWLElement::~CWLElement()
{
	
	if(m_OptionsPtr) {	
		m_OptionsPtr->clearAndDestroy();
		delete m_OptionsPtr;
	}
}

bool CWLElement::operator==( CWLElement& anElement)
{
	if ((*m_OptionsPtr).entries() != anElement.Options()->entries())
		return false;

	size_t _entries = (*m_OptionsPtr).entries();
	for (size_t i=0; i < _entries; ++i)
		if ((*(RVCollectableString*)((*m_OptionsPtr)[i])) == (*(RVCollectableString*)( (*(anElement.Options())) [i])))
			return false;

	if (RVCString(Value())  == RVCString(anElement.Value()) &&
		RVCString(Name()) == RVCString(anElement.Name())   &&	
		RVCString(Type()) == RVCString(anElement.Type())
		)
		return true;

	return true;
}

const char* CWLElement::DefaultValue()
{
	if(m_Type.compareTo("text", RVCString::ignoreCase) == 0 || 
		 m_Type.compareTo("textarea", RVCString::ignoreCase) == 0 ||
		 m_Type.compareTo("password", RVCString::ignoreCase) == 0)
		return m_DefaultValue.data();

	return  NULL;
}

Set_Result CWLElement::setChecked(bool checked)
{
	if(m_Type.compareTo("radio"		, RVCString::ignoreCase) == 0 ||
		 m_Type.compareTo("checkbox", RVCString::ignoreCase) == 0		)
	{
		 m_Checked = m_DefaultChecked;
		 return Element_Set;
	}

	return Element_No_Checked_Set;
}

Set_Result	CWLElement::setSelectedIndex(unsigned int newSelectedIndex)
{
	if(m_Type.compareTo("select", RVCString::ignoreCase) == 0 )
	{
		m_SelectedIndex = newSelectedIndex;
		return Element_Set;
	}

	return Element_No_Selected_Index_Set;
}

Set_Result CWLElement::setType(const char* type)
{
	m_Type.remove(0);
	if(type)
		m_Type = type;
	return Element_Set;
}

Set_Result CWLElement::setName(const char* name)
{
	setKey(name);
	return Element_Set;
}

Set_Result CWLElement::setValue(const char* newValue)	
{
	if(m_Type.compareTo("select", RVCString::ignoreCase) == 0 )
		return Element_No_Value_Set;

	CKVP::setValue(newValue);
	return Element_Set;
}

RVOrdered* CWLElement::Options()
{
	if(m_Type.compareTo("select", RVCString::ignoreCase) == 0 )
		return m_OptionsPtr; 
	else
		return NULL;
}

void CWLElement::Reset()
{
	if(m_Type.compareTo("text"		, RVCString::ignoreCase) == 0 ||
		 m_Type.compareTo("textarea", RVCString::ignoreCase) == 0	||	
		 m_Type.compareTo("password", RVCString::ignoreCase) == 0		)

		 setValue(m_DefaultValue.data());
	
	if(m_Type.compareTo("radio"		, RVCString::ignoreCase) == 0 ||
		 m_Type.compareTo("checkbox", RVCString::ignoreCase) == 0		)

		 m_Checked = m_DefaultChecked;
}

int CWLElement::Length()
{
	if(m_OptionsPtr != NULL)
	{
		size_t  tmp;
		if((tmp = m_OptionsPtr->entries()) != RV_NPOS) 
			return (int)tmp;
	}
		return 0;
}

RVCSubString CWLElement::GetInnerHTML()
{ 
	if (m_DocPtr && m_DocPtr->m_HtmlPagePtr) {
		RVCString &temp = *(m_DocPtr->m_HtmlPagePtr);
		return temp(GetStartInnerHTML(), GetLenInnerHTML());
	} else {
		RVCString temp;
		return temp(0,0);
	}
}