/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include "wlframecontainer.h"
#include "wlhtmlvirtualqueue.h"
#include "wlparseflags.h"
#include "wlidentlocation.h"
#include "tracedll.h"
#include "wldocument.h"
#include "wlform.h"
#include "wltransactiondata.h"

#define RET_ERROR -1
#define RET_OK 1

/************************************************************************
************************************************************************/

CWLDomWrapper::CWLDomWrapper(RVCString& sDomName): m_pDom(NULL),m_pPageTransaction(NULL),m_sDomName(sDomName),m_pErrorFrame(NULL)
{
  m_FrameMode = JS_Document;
}

CWLDomWrapper::~CWLDomWrapper()
{
  cleanUp();

  if (m_pErrorFrame)
    delete m_pErrorFrame;
}

void CWLDomWrapper::cleanUp()
{
	m_AllFrames.clear();
	m_AllForms.clear();
	m_AllLinks.clear();
  m_CloneContainer.clear();

}


/************************************************************************
************************************************************************/

frameContainer::frameContainer(CWLHtmlVirtualQueue* queuePtr) : m_pActiveDom(NULL),m_DeletePageLock(NULL),m_pPreviousDom(NULL)
{
	m_pQueuePtr  = queuePtr ;
	m_DeletePageLock = new RVCriticalSection;
}

frameContainer::~frameContainer()
{
	if (m_DomHash.entries())
	{
		deleteAllPages();
    m_DomHash.clearAndDestroy();
	}

	if (m_DeletePageLock)
		delete m_DeletePageLock;
}

void frameContainer::AddFrameToArray(CWLDocument* frame)
{
  if (m_pActiveDom)
    m_pActiveDom->m_AllFrames.insert(frame); 
}
void frameContainer::AddLinkToArray(CWLLink* link) 
{ 
  if (m_pActiveDom)
    m_pActiveDom->m_AllLinks.insert(link); 
}

void frameContainer::AddFormToArray(CWLForm* form)
{
  if (m_pActiveDom)
	  m_pActiveDom->m_AllForms.insert(form);
}

void frameContainer::deleteAllPages()
{
	RVHashDictionaryIterator iter(m_DomHash);
	while (iter()) {
		CWLDocument* docPtr = (CWLDocument*)((CWLDomWrapper*)iter.value())->m_pDom;
		WlTraceMsg ("Decreasing the reference count to the frame: %d",(int)docPtr);
		if (docPtr->decRefCount() <= 0) {
			WlTraceMsg ("Remove and destroy the frame: %d because there is no more reference to it",(int)docPtr);
			delete docPtr;
      ((CWLDomWrapper*)iter.value())->m_pDom = NULL;
		}
		
    WlTraceMsg ("Remove the domWrapper: %s from the frame container", ((CWLDomWrapper*)iter.value())->m_sDomName);    
	}
}


//yuval MT
//here we need to:  create new document
//								  insert the document into our hash dictionary
//								  insert the document into the frameQueue
int frameContainer::insertNewPage(RVCString& pageName ,RVCString& pageUrl,
                                  CWLUserParseFlags& parseFlags)
{
	CWLDomWrapper* pDomWrapper;
	RVCollectableString pn(pageName);

	if ((pDomWrapper = (CWLDomWrapper*)m_DomHash.findValue(&pn)) != NULL)
	{
		if (parseFlags.m_Parse == CWLUserParseFlags::ParseOnce && pDomWrapper->m_pDom->IsValid())
		{
			// insert all frames into Q.
			SetActiveDom(pDomWrapper);
			WlTraceMsg ("Inserting dom fetched from the frame container into queue : %d",(int)m_pActiveDom->m_pDom);
			if (!m_pQueuePtr->insert(m_pActiveDom->m_pDom->GetTransactionObj()))
				return -1;

	    SetPageTransaction(m_pActiveDom->m_pDom);

			WlTraceMsg ("Inserting all the dom frames from the frame container into queue");
			insertSubTreeFrames();
			return (0);
		}
		else
		{
			RVCollectableString* key = (RVCollectableString*)m_DomHash.remove(&pn);
			delete key;
      WlTraceMsg ("Decreasing reference count to dom fetched from frame container: %d",(int)pDomWrapper->m_pDom);
      if (pDomWrapper && pDomWrapper->m_pDom->decRefCount()==0) {
				WlTraceMsg ("Deleting dom fetched from frame container: %d",(int)pDomWrapper->m_pDom);
        delete pDomWrapper->m_pDom ;
        pDomWrapper->m_pDom = NULL ;
			}

      if (m_pPreviousDom == pDomWrapper)
        m_pPreviousDom = NULL ;
      else if (m_pActiveDom == pDomWrapper)
        m_pActiveDom = NULL ;

			delete pDomWrapper;
      SetActiveDom((CWLDomWrapper*)NULL) ;
			//parseFlags.m_Parse = CWLUserParseFlags::ParseOnce;
		}
	}

	RVCString frameName("NO_FRAME_NAME");

  CWLDomWrapper* pDomWrap = new CWLDomWrapper(pageName) ;

	 pDomWrap->m_pDom = new CWLDocument(pageName, frameName, pageUrl, RVCString(), NULL, NULL, parseFlags,
		this, NULL);
	bool ok = true;
//	if (parseFlags.m_Parse == CWLUserParseFlags::ParseOnce)
	{
		ok = m_DomHash.insertKeyAndValue(new RVCollectableString(pn), pDomWrap) ? true : false;
		if (ok) {
			WlTraceMsg ("Creating new Dom, setting reference count to 1 for : %d",(int)pDomWrap->m_pDom);
      SetActiveDom(pDomWrap);
			pDomWrap->m_pDom->incRefCount();
		}
	}
  if (ok && m_pQueuePtr->insert(pDomWrap->m_pDom->GetTransactionObj())) 
  {
	  SetPageTransaction(pDomWrap->m_pDom);
    WlTraceMsg ("Inserting new dom into queue : %d",(int)pDomWrap->m_pDom);
    return (1);
  }
  else {
    WlTraceMsg ("Failed to insert a new dom into queue : %d",(int)pDomWrap->m_pDom);
    return (-1);
  }
}

int frameContainer::insertNewFrame(RVCString&           pageUrl,
                                   CWLUserParseFlags&   parseFlags,
                                   const RVCString&     Target,
                                   replaceFrameStatus&  retStatus)
{

  if (!m_pActiveDom || !m_pActiveDom->m_pDom)
    return (RET_ERROR);

	RVCString         pageName  (m_pActiveDom->m_pDom->GetName());
	RVCString         frameName ;
	int               frameNumber;
	CWLIdentLocation  TargetLocation (Target);
	CWLDocument*      parentFrame;

	retStatus = replaceFrame_REPLACE_OK;

	if (!(parentFrame=deleteOldFrame (m_pActiveDom->m_pDom,TargetLocation,frameNumber,frameName,retStatus))) 
  {
		// Here we need to add warning about frame not found ... TBD
		RVCString frameName("NO_FRAME_NAME");
		m_pActiveDom->m_pErrorFrame = new CWLDocument(pageName, frameName, pageUrl, RVCString(), NULL, NULL, parseFlags,
				  this, NULL);
		SetPageTransaction(m_pActiveDom->m_pErrorFrame);

		CWLTransactionData*  pTrans = m_pActiveDom->m_pErrorFrame->GetTransactionObj()->GetTransaction();
		pTrans->SetBeginTransaction(TVPSysTime());
		pTrans->SetEndTransaction(TVPSysTime(),WLLogMessage::Success);
		return (RET_ERROR);
	}
	else {
		CWLDocument* pFrame = new CWLDocument(
			pageName, frameName, pageUrl, RVCString(), NULL, NULL, parseFlags,
			this, parentFrame);
    pFrame->SetIndex(frameNumber);

			//! copy father's frameTarget and append the level
			CWLIdentLocation frameTarget;
			if(parentFrame) {
				frameTarget = parentFrame->GetFrameTarget();
				frameTarget.appendLevel( frameNumber, frameName); 
        pFrame->SetFrameTarget(frameTarget);
			}

		if (m_pQueuePtr->insert(pFrame->GetTransactionObj())) {
			SetPageTransaction(pFrame);
			if (parentFrame->m_Frames->insertAt(frameNumber,pFrame)) {
				// After we insert the new frame we also increase the reference
				// to it because now it is treated as one that will be referenced
				// by it's father
				WlTraceMsg ("Inserting a new frame, setting reference count to 1 (reference from parent frame) for the new frame: %d",(int)pFrame);
				pFrame->incRefCount();
				return (RET_OK);
			}
			else {
				WlTraceMsg ("Failed to insert a new frame: %d",(int)pFrame);
				retStatus = replaceFrame_INSERT_FRAME_FAILURE;
				return (RET_ERROR);
			}
		}
		else {
			WlTraceMsg ("Failed to insert a new frame to queue: %d",(int)pFrame);
			retStatus = replaceFrame_INSERT_QUEUE_FAILURE;
			return (RET_ERROR);
		}
	}
}

// The method goes recursively on a document frames to find the
// frame that should be replaced.
// It's start by checking what is the frame num or name that
// should we work this is taken from the target value. i.e
// target will e in the structure "1.main.3" we will take the
// value till the first period and go on all the frames of the document
// checking if that is the frame. if it is the frame then if we
// dont have more target hierarchy then we know that this is the
// target that should be deleted. we decrimante the reference count to it
// and if it is now equel zero then we actually delete it (else we just
// leave it to be deleted later) afterwords we delete this ptr from the
// frames list.
// if we do have more values in the target hierarchy then we call again
// the DeleteOldFrame givinig it the current frame as the root documnet
// and the Target truncated from the starting value
CWLDocument* frameContainer::deleteOldFrame(const CWLDocument*  startFrame,
                                            CWLIdentLocation&   TargetLocation,
                                            int&                retFrameNumber,
                                            RVCString&          retFrameName,
                                            replaceFrameStatus& retStatus)
{

	RVCString     Name;
	RVCString     FrameName;
	int           Number;
	int           iterCount = 0;
	CWLDocument*  returnParent = NULL;

	TargetLocation.getLevel (0,Number,Name);

	RVOrderedIterator iter(*startFrame->m_Frames);

	// Have we found the frame we are looking for ?
	bool Found = false;

	// Have we finished searching - this variable will end the search after we
	// do the recursive search inside the frames tree
	bool searchCompleted = false;
	CWLDocument* FoundFrame = NULL;

	while ((iter()) && (!FoundFrame) && (!searchCompleted))
	{
		CWLDocument* Frame = (CWLDocument*)iter.key();
		// Here we check if the current frame we are on is the frame we are
		// looking for delete. if we have more on the path it mean that we must
		// iterate also on the frames hanged on this frame in order to find the
		// frame we need to delete. If we dont have it mean that we have reaced the desired
		// frame for deletion
		FrameName = Frame->GetFrameName ();

		if (((!Name.isNull()) && (FrameName == Name)) || ((Number != NULL_IDENT_LOCATION_NUMBER) && (Number == iterCount))) {
			TargetLocation.removeLevel (0);
			if (TargetLocation.isEmpty()) {
				FoundFrame = Frame;
				// Here we return the father and the name and index for
				// the frame that need to be deleted
				returnParent = (CWLDocument*)startFrame;
				retFrameNumber = iterCount;
				retFrameName   = Name;
				if ((!Name.isNull()) &&
					(FrameName == Name) &&
					(Number != NULL_IDENT_LOCATION_NUMBER) &&
					(Number != iterCount)) {
					retStatus = replaceFrame_DELETE_NAME_MATCH_NUMBER_MISMATCH_WARNING;
				}
				else if ((Number != NULL_IDENT_LOCATION_NUMBER) &&
					(Number == iterCount) &&
					(!Name.isNull()) &&
					(FrameName != Name)) {
					retStatus = replaceFrame_DELETE_NUMBER_MATCH_NAME_MISMATCH_WARNING;
				}
			}
			else {
				returnParent = deleteOldFrame (Frame,TargetLocation,retFrameNumber,retFrameName,retStatus);
				searchCompleted = true;
			}
		}
		++iterCount;
	}
	if (FoundFrame) {
		WlTraceMsg ("Found a frame for delete: %d",(int)FoundFrame);
		if (FoundFrame->decRefCount() == 0) {
			WlTraceMsg ("Decreased the reference count for frame for delete: %d - remove and destroy this frame from frame list (no reference any more)",(int)FoundFrame);
			startFrame->m_Frames->removeAndDestroy (FoundFrame);
		}
		else {
			WlTraceMsg ("Decreased the reference count for frame for delete: %d - remove and destroy this frame from frame list",(int)FoundFrame);
			FoundFrame = (CWLDocument*) startFrame->m_Frames->remove (FoundFrame);
		}
	}
	else {
		if (!returnParent) {
			retStatus = replaceFrame_DELETE_FAILURE;
		}
	}
	return (returnParent);
}

void frameContainer::deletePage(RVCString& pageName)
{
	RVCriticalSection::LockGuard Lock(*m_DeletePageLock);

	CWLDomWrapper* pDomWrapper;
	RVCollectableString pn(pageName);

	if ((pDomWrapper = (CWLDomWrapper*)m_DomHash.findValue(&pn)) != NULL)
  {
    pDomWrapper->m_pDom->SetInvalidFrame();
    if (pDomWrapper == m_pActiveDom)
      SetActiveDom((CWLDomWrapper*)NULL);
    m_DomHash.removeAndDestroy(pDomWrapper);
  }
}


void frameContainer::SetInvalidPage(RVCString& pageName)
{
  //same as old deletePage
	RVCriticalSection::LockGuard Lock(*m_DeletePageLock);

	CWLDomWrapper* pDomWrapper;
	RVCollectableString pn(pageName);

	if ((pDomWrapper = (CWLDomWrapper*)m_DomHash.findValue(&pn)) != NULL)
  {
    pDomWrapper->m_pDom->SetInvalidFrame();
  }
}


void frameContainer::cleanUp()
{
	if (m_DomHash.entries())
		m_DomHash.clearAndDestroy();
}


void frameContainer::insertSubTreeFrames()
{
	RVSlistCollectablesQueue framePtrQ;
	CWLDocument* pDoc = m_pActiveDom->m_pDom;
	do
	{
		parseFrame*  flist;
		if (pDoc->m_Frames)
			flist = pDoc->m_Frames;
		else
			continue;
		RVOrderedIterator iter(*flist);
		do {
			if (iter())
			{
				pDoc = (CWLDocument*)iter.key();
				framePtrQ.insert(pDoc);
				m_pQueuePtr->insert(pDoc->GetTransactionObj());
			}
			else
				pDoc = NULL;
		} while(pDoc);
	} while (pDoc = (CWLDocument*)framePtrQ.get());
}

CWLDocument* frameContainer::findPageByName(RVCString& pageName)
{
	int findPatMatch = 0;
	RVHashDictionaryIterator iter(m_DomHash);
	parseFrame* framePtr;
	while (iter())
	{
		RVCString temp = *(RVCollectableString *)iter.key();
		size_t indx = pageName.index(temp, 0, pageName.ignoreCase);
		if (indx == RV_NPOS  ||  indx != 0)
			continue;
		if (temp.length() == pageName.length())
			return (CWLDocument*)((CWLDomWrapper*)iter.value())->m_pDom;
		if (pageName[temp.length()] != '.')
			continue;
		framePtr = ((CWLDocument*)((CWLDomWrapper*)iter.value())->m_pDom)->m_Frames;
		findPatMatch = 1;
		break;
	}
	if (!findPatMatch)
		return NULL;
	findPatMatch = 0;
	do
	{

		RVOrderedIterator iter(*framePtr);
		while (iter())
		{
			CWLDocument* docPtr = (CWLDocument*)iter.key();
			RVCString temp(docPtr->GetName());
			size_t indx = pageName.index(temp, 0, pageName.ignoreCase);
			if (indx == RV_NPOS  ||  indx != 0)
				continue;
			if (temp.length() == pageName.length())
				return docPtr;
			if (pageName[temp.length()] != '.')
				continue;
			findPatMatch = 1;
			framePtr = (docPtr->m_Frames);
			break;
		}
		if (!findPatMatch)
			break;
		findPatMatch = 0;
	} while (framePtr);
	return NULL;
}

void frameContainer::referenceObject(CWLDocument* docPtr)
{
	docPtr->incRefCount();
}

void frameContainer::deReferenceObject(CWLDocument* docPtr)
{
	if(docPtr->decRefCount())
		return;
	WlTraceMsg ("Removing the frame : %d - from CloneContainer",(int)docPtr);
	m_pActiveDom->m_CloneContainer.removeAndDestroy((RVCollectable*)docPtr);
}

void frameContainer::CleanCloneContainer()
{
  if (!m_pActiveDom)
    return;
	size_t cloneEntries = m_pActiveDom->m_CloneContainer.entries();
	for(size_t index = 0; index < cloneEntries; ++index)
	{
		CWLDocument* docPtr = (CWLDocument*)m_pActiveDom->m_CloneContainer[index];
		if(!(docPtr->getRefCount()))
			m_pActiveDom->m_CloneContainer.removeAndDestroy((RVCollectable*)docPtr);
	}
}

void frameContainer::insertStatus(unsigned value, int n, RVCString& line)
{
	if (m_pActiveDom && m_pActiveDom->m_pDom)
		m_pActiveDom->m_pDom->insertStatus(value, n, line);
}

void frameContainer::insertHeader(const char* name, const char* value)
{
	if (m_pActiveDom && m_pActiveDom->m_pDom)
		m_pActiveDom->m_pDom->insertHeader(name, value);
}

void frameContainer::insertUri(RVCString& uri)
{
	if (m_pActiveDom && m_pActiveDom->m_pDom)
		m_pActiveDom->m_pDom->insertUri(uri);
}

void frameContainer::insertHost(RVCString& name, unsigned port)
{
	if (m_pActiveDom && m_pActiveDom->m_pDom)
		m_pActiveDom->m_pDom->insertHost(name, port);
}

int frameContainer::GetFrameByIndex(CWLDocument* frm, int in, CWLDocument** frameVal)
{
	//  CWLDocument* frm = findPageByName(pn);
	if(frm)
	{
		if(frm->m_Frames)
		{
			int entries_ = int(frm->m_Frames->entries());
			if(entries_ <= 0)
				return -3; // no frames in this page
			else if (in > entries_)
				return -2;//number of frames smaller than index
			else
			{
				*frameVal = (CWLDocument*)((*frm->m_Frames)[in - 1]);
				return 0;
			}
		}
		else
			return -5; //no parsing of frames
	}
	else
		return (-1);//no page by this name
}

int frameContainer::GetFrameByFrameName(CWLDocument* frm, const char* pat, CWLDocument** frameVal)
{
	//  CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->pageTreeSearch(0, 0, pat, NULL, (void**)frameVal));
	else
		return (-1);//no page by this name
}

int frameContainer::GetFrameByUrl(CWLDocument* frm, const char* pat, CWLDocument** frameVal)
{
	//  CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->pageTreeSearch(1, 0, pat, NULL, (void**)frameVal));
	else
		return (-1);//no page by this name
}

int frameContainer::GetFrameUrl(CWLDocument* frm, CWLUrl** urlVal)
{
	//  CWLDocument* frm = findPageByName(pn);
	if (frm) {
		*urlVal = &(frm->GetLocation());
		return 0;
	}
	else
		return (-1);//no page by this name
}

int frameContainer::GetHrefByIndex(CWLDocument* frm, int in, CWLLink** hrefVal)
{
	//  CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->GetHrefByIndex(in, hrefVal));
	else
		return (-1);//no page by this name
}


int frameContainer::GetHrefByName(CWLDocument* frm, const char* pat, CWLLink** hrefVal)
{
	//  CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->pageTreeSearch(2, 0, pat, NULL, (void**)hrefVal));
	else
		return (-1);//no page by this name
}


int frameContainer::GetHrefByUrl(CWLDocument* frm, const char* pat, CWLLink** hrefVal)
{
	//  CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->pageTreeSearch(3, 0, pat, NULL, (void**)hrefVal));
	else
		return (-1);//no page by this name
}

int frameContainer::GetFormAction(CWLDocument* frm, int formIndex, CWLLocation** action)
{
	//  CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->GetFormAction(formIndex, action));
	else
		return (-1);//no page by this name
}

int frameContainer::GetFieldValueByPage(CWLDocument* frm, const char* fn, const char** formVal)
{
	//  CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->pageTreeSearch(4, 0, NULL, fn, (void**)formVal));
	else
		return (-1);//no page by this name
}

int frameContainer::GetFieldValueByForm(CWLDocument* frm, int in,const char* fn, const char** fv)
{
	//  CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->pageTreeSearch(5, in, NULL, fn, (void**)fv));
	else
		return (-1);//no page by this name
}

int frameContainer::GetHeaderValue(CWLDocument* frm, const char* name, const char*& value)
{
	//	CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->GetHeaderValue(name, value));
	else
		return (-1);//no page by this name
}

unsigned frameContainer::GetStatusNumber(CWLDocument* frm)
{
	//	CWLDocument* frm = findPageByName(pn);
	return (frm->GetStatusNumber());

}

int frameContainer::GetStatusLine(CWLDocument* frm, const char*& line)
{
	//	CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->GetStatusLine(line));
	else
		return (-1);//no page by this name
}

int frameContainer::GetHost(CWLDocument* frm, const char*& name)
{
	//	CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->GetHost(name));
	else
		return (-1);//no page by this name
}

int frameContainer::GetHostName(CWLDocument* frm, const char*& name)
{
	//	CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->GetHostName(name));
	else
		return (-1);//no page by this name
}

unsigned frameContainer::GetPortNum(CWLDocument* frm)
{
	//	CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->GetPortNum());
	else
        return ((unsigned)-1);//no page by this name
}

int frameContainer::GetUri(CWLDocument* frm, const char*& uri)
{
	//	CWLDocument* frm = findPageByName(pn);
	if (frm)
		return (frm->GetUri(uri));
	else
		return (-1);//no page by this name
}

CWLDocument* frameContainer::GetGlobalDoc()
{
  return m_pActiveDom ? m_pActiveDom->m_pDom : NULL ;
}

void	frameContainer::SetFrameMode(JSFrameMode frameMode)
{
	if (m_pActiveDom)
    m_pActiveDom->m_FrameMode = frameMode;
}

CWLDocument *frameContainer::GetPageTransaction()
{
  return (m_pActiveDom ? m_pActiveDom->m_pPageTransaction : NULL) ;
}

void	frameContainer::SetPageTransaction(CWLDocument *pFrame)
{
  if (m_pActiveDom)
    m_pActiveDom->m_pPageTransaction = pFrame ;
}

bool frameContainer::SetActiveDom(const char* pcDomName)
{
  bool bRet = false ;
  if (pcDomName)
  {
    if ( m_pActiveDom && (m_pActiveDom->m_sDomName==pcDomName) )
    {
      bRet = true ;
    }
    else
    {
	    CWLDomWrapper* pDomWrapper;
	    RVCollectableString pn(pcDomName);

	    if ((pDomWrapper = (CWLDomWrapper*)m_DomHash.findValue(&pn)) != NULL)
      {
        if (m_pActiveDom)
          m_pPreviousDom = m_pActiveDom ;
        m_pActiveDom = pDomWrapper ;
        bRet = true ;
      }
    }
  }

  return bRet;
}

bool frameContainer::SetActiveDom(CWLDomWrapper* pDom)
{
  if (m_pActiveDom)
    m_pPreviousDom = m_pActiveDom ;

  m_pActiveDom = pDom ;

  return pDom ? true : false ;
}

CWLDomWrapper* frameContainer::GetActiveDom()
{
  return m_pActiveDom;
}

RVOrdered* frameContainer::GetCloneContainer()
{
  return m_pActiveDom ? &m_pActiveDom->m_CloneContainer : NULL ;
}

RVOrdered* frameContainer::GetAllFrames()
{
  return m_pActiveDom ? &m_pActiveDom->m_AllFrames : NULL ;
}

RVOrdered* frameContainer::GetAllForms()
{
  return m_pActiveDom ? &m_pActiveDom->m_AllForms : NULL ;
}

RVOrdered* frameContainer::GetAllLinks()
{
  return m_pActiveDom ? &m_pActiveDom->m_AllLinks : NULL ;
}

void frameContainer::SetPreviousDomAsActive(CWLDomWrapper* pDom /*NULL*/)
{
  if (m_pPreviousDom)
  {
    if (pDom)
    {
      m_pActiveDom = m_pPreviousDom ; 
      m_pPreviousDom = pDom ;
    }
    else
    {
      CWLDomWrapper* pActiveDom = m_pActiveDom ;
      m_pActiveDom = m_pPreviousDom ; 
      m_pPreviousDom = pActiveDom ;
    }
  }
}

