/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include <RVCString.h>	
#include "wlscript.h"
#include "wldocument.h"

CWLScript::CWLScript(CWLDocument* docPtr)
	: m_DocPtr(docPtr), m_TransObj(NULL)
{
	m_TransObj.SetObj(this); 
	m_TransObj.SetObjType(SCRIPT);
	m_Index = m_DocPtr->m_Scripts->entries();
	m_Referrer = m_DocPtr->GetDocUrl();
	m_iUrlOffset = -1;
}

void CWLScript::SetText(const char* text)
{
	m_Text = text;
}

void CWLScript::SetId(const char* id)
{
	m_Id = id;
}

void CWLScript::SetSrc(const char* src)
{
	m_TransObj.GetTransaction()->SetName(src);
	m_Src = src;
}

void CWLScript::SetEvent(const char* event)
{
	m_Event = event;
}

void CWLScript::SetLanguage(const char* language)
{
	m_Language = language;
}

const char* CWLScript::GetText()
{
	return m_Text.isNull() ? NULL : m_Text.data();
}

const char* CWLScript::GetInnerHTML()
{
	return m_Text.isNull() ? NULL : m_Text.data();
}

size_t CWLScript::GetTextLength()
{
	return m_Text.length();
}

const char* CWLScript::GetId()
{
	return m_Id.isNull() ? NULL : m_Id.data();
}

const char* CWLScript::GetSrc()
{
	return m_Src.isNull() ? NULL : m_Src.data();
}

const char* CWLScript::GetEvent()
{
	return m_Event.isNull() ? NULL : m_Event.data();
}

const char* CWLScript::GetLanguage()
{
	return m_Language.isNull() ? NULL : m_Language.data();
}

void CWLScript::printAll()
{
	cout<<"ID:       "<<m_Id.data()<<'\n';
	cout<<"SRC:      "<<m_Src.data()<<'\n';
	cout<<"LANGUAGE: "<<m_Language.data()<<'\n';
	cout<<"EVENT:    "<<m_Event.data()<<'\n'<<'\n';
	cout<<"TEXT:     "<<'\n'<<m_Text.data()<<'\n'<<'\n';
}
