/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wlregularexpressions.h"
#include "wlxml.h"
#include "wlxmlentityresolver.h"
#include "wlxmlhelperfuncs.h"
#include "wlurlbase.h"
#include <RVCString.h>



CWLXmlDocument::CWLXmlDocument(CWLDocument *docPtr): CWLDocument()
{
	m_DocPtr                        = docPtr;
	m_Index                         = docPtr->m_Xmls->entries();
	m_TransObj.SetObj(this);
	m_TransObj.SetObjType(XML);
	m_pInterfacePtr					= NULL;
	m_QueuePtr							= docPtr->m_QueuePtr;
	m_DocTypeState					= ENCODING_REMOVE;
	strPtr									= 0;
	m_pDtdExternal					= m_pDtdInternal = NULL;
	m_pDom									= NULL;
	m_bSearchForCommentEnd	= false;
  	m_Referrer              = docPtr ? docPtr->GetDocUrl() : RVCString("")  ;
}

CWLXmlDocument::~CWLXmlDocument()
{
//	if(m_pPrivate)
//		delete m_pPrivate;
//	delete m_XmlPrivatesCountMutex;
	if(m_pDtdExternal)
		delete m_pDtdExternal;
	if(m_pDtdInternal)
		delete m_pDtdInternal;
	if(m_pInterfacePtr)
		delete m_pInterfacePtr;
}

void	CWLXmlDocument::parse(const char* newData, size_t size, bool* isDone)
{

	size_t newSize = size;
	appendNoNull(m_InnerHTML, newData, newSize);
	//m_InnerHTML.append(newData, size);
 	if(m_DocTypeState != DOCTYPE_PARSE_ERROR && m_DocTypeState != DOCTYPE_PARSE_DONE)
		switch(parseXML(newData, size)){
		case(PARSE_ERROR):
			m_DocTypeState = DOCTYPE_PARSE_ERROR;
			break;
		case(END_OF_ELEMENT):
			m_DocTypeState = DOCTYPE_PARSE_DONE;
		case(READ_MORE):
			break;
	}
	return;


}

elementReadState CWLXmlDocument::parseXML(const char* newData, size_t size)
{
	RVCRegexp REName("[\"\'][^\"\']*[\"\']");
	char firstNoneWhite;
	size_t nameLen							= 0;
	size_t nameIndex						= 0;
	size_t length								= m_InnerHTML.length();

	while(1){
		switch(m_DocTypeState){
			case(ENCODING_REMOVE):
			{
					size_t xmlDec = m_InnerHTML.index('<', strPtr);
					if(xmlDec != RV_NPOS)
						if(m_InnerHTML.length() > xmlDec + 4)
							if(m_InnerHTML.index("<?xml", xmlDec) == xmlDec){
								size_t endXmlDec;
								if((endXmlDec = m_InnerHTML.index(">", xmlDec)) != RV_NPOS){
									size_t encodingLength = 0;
									RVCRegexp REencoding("encoding[\\n\\r\\t ]*=[\\n\\r\\t ]*[\"\'][^\'\"]*[\'\"]");
									size_t encoding;
									if((encoding = m_InnerHTML.index(REencoding, &encodingLength, xmlDec)) != RV_NPOS)
										if(encoding < endXmlDec){
											m_InnerHTML.remove(encoding, encodingLength);
											length -= encodingLength;
										}
									m_DocTypeState = DOCTYPE_FIND;
									break;
								}
							}
					return READ_MORE;
			}
			case(DOCTYPE_FIND):
				while(1){
					if(strPtr >= length - 10 || length < 10)
						return READ_MORE;
					else if(!m_bSearchForCommentEnd){
						if(m_InnerHTML(strPtr) == '<'){
							++strPtr;
							char firstCharInDeclaration = m_InnerHTML(strPtr);
							//!check that there is anything to do at all
							if(firstCharInDeclaration != '!' && firstCharInDeclaration != '?')
								return END_OF_ELEMENT;
							else if(m_InnerHTML(strPtr, 8) == "!DOCTYPE"){
								strPtr += 8;
								switch(xmlSkipWhiteSpaces(m_InnerHTML, length, strPtr, firstNoneWhite)){
								case(WHITE_NONE_TO_SKIP):{//!error
										appendToErrorString("White space expected: ", strPtr - 9, 9, "^");
										//!m_DocTypeErrorString +=
										return PARSE_ERROR;
																				 }
									case(WHITE_SKIPPED_ALL):
										break;
									case(WHITE_READ_MORE):
										break;
									default:
										break;
								}
								m_DocTypeState = DOCTYPE_ROOT_NAME;
								break;//!while
							}
							//!comment - have to skip to end -->
							else if(m_InnerHTML(strPtr, 3) == "!--"){
								strPtr += 4;
								m_bSearchForCommentEnd = true;
								continue;
							}
						}
						else{
							++strPtr;
							continue;
						}
					}
					else{
						size_t endComment;
						if((endComment = m_InnerHTML.index("-->", strPtr)) != RV_NPOS){
							strPtr									= endComment + 4;
							m_bSearchForCommentEnd	= false;
							continue;
						}
						else
							return READ_MORE;
					}
				}//!while
				break;//!switch(m_DocTypeState)

			//!<!DOCTYPE {rootname PUBLIC | rootname SYSTEM | rootname[<!  <--white spaces optional here
			case(DOCTYPE_ROOT_NAME):
				//!skip white spaces first
				switch(xmlSkipWhiteSpaces(m_InnerHTML, length, strPtr, firstNoneWhite)){
					case(WHITE_NONE_TO_SKIP):
					case(WHITE_SKIPPED_ALL):
						break;
					case(WHITE_READ_MORE):
						return READ_MORE;
					default:
						break;
				}
				//!skip the name
				while(!htmlIsWhiteSpace(m_InnerHTML(strPtr))  && m_InnerHTML(strPtr) != '[')
					if( strPtr < length - 1)
						++strPtr;
					else
						return READ_MORE;
				//!act according to reason for exisiting the while above
				if(htmlIsWhiteSpace(m_InnerHTML[strPtr]))
					m_DocTypeState = DOCTYPE_SEARCH_EXTERNAL;
				else{//!m_InnerHTML[strPtr] == '['
					m_pDtdInternal	= new CWLXmlEntityResolver(DOCTYPE_INTERNAL, m_QueuePtr, strPtr + 1,  1, &m_DocUrl, NULL,m_DocPtr, true);//!dont have the end ptr for now.
					newData					= newData + size + strPtr - length + 1;
					size						=  length - strPtr - 1;
					m_DocTypeState	= DOCTYPE_PARSE_INTERNAL;//!doctype is retrieved by this object and fed into
				}	  																	 //!the above object for parsing
				startDocTypePtr = strPtr;	//!mark this position in order to insert retrieved DTD later - note that the '[' is necessary
				break;
			case(DOCTYPE_SEARCH_EXTERNAL):
				switch(xmlSkipWhiteSpaces(m_InnerHTML, length, strPtr, firstNoneWhite)){
					case(WHITE_READ_MORE):
						return READ_MORE;
					case(WHITE_NONE_TO_SKIP):
					case(WHITE_SKIPPED_ALL):
						if(firstNoneWhite == '[')
							m_DocTypeState = DOCTYPE_PARSE_INTERNAL;
					default:
						break;
				}
				if(m_DocTypeState == DOCTYPE_PARSE_INTERNAL){
					m_pDtdInternal = new CWLXmlEntityResolver(DOCTYPE_INTERNAL, m_QueuePtr, strPtr + 1,  1, &m_DocUrl, NULL, m_DocPtr, true);//!dont have the end ptr for now.
					if(newData)//!from http client queue...
						newData = newData + size + strPtr - length + 1;
					else//!from html parser - newData is NULL, all of the string is available
						newData = m_InnerHTML.data() + strPtr + 1;
					size = length - strPtr - 1;
					/*
					newData = newData + size + strPtr - length + 1;
					size    =  length - strPtr - 1;
					*/
					break;
				}
					if(strPtr + 6 < length){
						if(m_InnerHTML(strPtr, 6) == "PUBLIC" && htmlIsWhiteSpace(m_InnerHTML[strPtr + 6])){
							strPtr += 6;
							m_DocTypeState = DOCTYPE_PUBLIC;
						}
						else if(m_InnerHTML(strPtr, 6) == "SYSTEM" && htmlIsWhiteSpace(m_InnerHTML[strPtr + 6])){
							strPtr += 6;
							m_DocTypeState = DOCTYPE_SYSTEM;
						}
						else {
								appendToErrorString("External ID is invalid, expecting SYSTEM/PUBLIC: ", strPtr -6, 6);
								return PARSE_ERROR;
						}
						break;
				}
				else return READ_MORE;
			//!skip the public name
			case(DOCTYPE_PUBLIC):
				switch(xmlSkipWhiteSpaces(m_InnerHTML, length, strPtr, firstNoneWhite)){
					case(WHITE_SKIPPED_ALL):
					case(WHITE_NONE_TO_SKIP)://!
						break;
					case(WHITE_READ_MORE):
						return READ_MORE;
					default:break;
				}
				nameIndex = m_InnerHTML.index(REName, &nameLen, strPtr);
				if(nameIndex == strPtr){
					strPtr = nameIndex + nameLen;
					m_DocTypeState = DOCTYPE_SYSTEM;
				}
				else{//!error expected: PUBLIC "name"
					appendToErrorString("Unexpected character string in DTD: ", nameIndex, nameLen);
					return PARSE_ERROR;
				}
			//!can find the url now
			case(DOCTYPE_SYSTEM):
				switch(xmlSkipWhiteSpaces(m_InnerHTML, length, strPtr, firstNoneWhite)){
					case(WHITE_SKIPPED_ALL):
					case(WHITE_NONE_TO_SKIP)://!
						break;
					case(WHITE_READ_MORE):
						return READ_MORE;
					default:break;
				}
				nameIndex = m_InnerHTML.index(REName, &nameLen, strPtr);
				if(nameIndex != RV_NPOS){
					if(nameIndex == strPtr){
						strPtr						= nameIndex + nameLen;
						RVCString relUrl	= m_InnerHTML(nameIndex + 1, nameLen - 2);
						RVCString url			= m_UrlBase.calcUrl(relUrl);
						m_pDtdExternal		= new CWLXmlEntityResolver(DOCTYPE_EXTERNAL, m_QueuePtr, startDocTypePtr, strPtr, &url, NULL, m_DocPtr, true);
						m_DocTypeState		= DOCTYPE_PARSE_EXTERNAL;//!doctype is retrieved & parsed by above object
						break;
					}
					else{//!error expected: PUBLIC "name" "url"
						appendToErrorString("Unexpected character in DTD: ", strPtr - 20, 20, "^");
						return PARSE_ERROR;
					}
				}
				else return READ_MORE;
			case(DOCTYPE_PARSE_EXTERNAL):
				switch(xmlSkipWhiteSpaces(m_InnerHTML, length, strPtr, firstNoneWhite)){
					case(WHITE_SKIPPED_ALL):
					case(WHITE_NONE_TO_SKIP)://!
						if(firstNoneWhite == '['){//!have to parse the internal part of the dtd
							m_pDtdInternal = new CWLXmlEntityResolver(DOCTYPE_INTERNAL, m_QueuePtr, strPtr, 2, &m_DocUrl, NULL, m_DocPtr, true);//!dont have the end ptr for now.
							//!from here we skipp to DOCTYPE_PARSE_INTERNAL, have to adjust newData and size
							//!because parsing should start from after the '['
							if(newData)
								newData = newData + size + strPtr - length + 1;
							else
								newData = m_InnerHTML.data() + strPtr + 1;
							size = length - strPtr - 1;
							m_DocTypeState = DOCTYPE_PARSE_INTERNAL;
							break;
						}
						else if(firstNoneWhite == '>')//!end of DOCTYPE element
							return END_OF_ELEMENT;
						else{ //!error
							appendToErrorString("Unexpected character in DTD: ", strPtr - 30, 30, "^");
							return PARSE_ERROR;
						}
					case(WHITE_READ_MORE):
						return READ_MORE;
					default:break;
				}

			case(DOCTYPE_PARSE_INTERNAL):{
				bool isDoneParsing = false;
				m_pDtdInternal->parse(newData, size, &isDoneParsing);
				if(isDoneParsing){
					m_DocTypeState = DOCTYPE_PARSE_DONE;
					return END_OF_ELEMENT;
				}
				else if(m_pDtdInternal->GetError())
					return PARSE_ERROR;
				else
					return READ_MORE;
			}
			default:
				break;
		}
	}
}


void CWLXmlDocument::appendToErrorString(char* description, size_t start, size_t length, char* ptr)
{
	m_DocTypeErrorString.append(description);
	if(start < 0)
		start = 0;
	if(start + length >  m_InnerHTML.length())
		length = m_InnerHTML.length();
	RVCString tmp = m_InnerHTML(start, length);
	m_DocTypeErrorString.append(tmp.data());
	if(ptr)
		m_DocTypeErrorString.append(ptr);
}

RVCString& CWLXmlDocument::GetDocUrl()
{
	return m_DocUrl;
}

void CWLXmlDocument::SetInnerHTML(const char* text, size_t length)
{
	appendNoNull(m_InnerHTML, text, length);
}

void CWLXmlDocument::SetId(const char* id)
{
	m_Id = id;
}

void CWLXmlDocument::SetSrc(const char* src)
{
	m_Src = src;
	m_UrlBase.setBase(m_Src);
	m_TransObj.GetTransaction()->SetName(src);
}

const char* CWLXmlDocument::GetInnerHTML()
{
	return m_InnerHTML.data();
}

const char* CWLXmlDocument::GetId(){
	return m_Id.data();
}

const char* CWLXmlDocument::GetSrc()
{//!IE5 returns relative address as is
	return (m_Src.isNull() ? NULL : m_Src.data());
}

/*
int CWLXmlDocument::IncPrivatesCount()
{
	int i;
	{
		RVCriticalSection::LockGuard lock(*m_XmlPrivatesCountMutex);
		i = m_PrivatesCount++;
	}
	return i;

}

int CWLXmlDocument::DecPrivatesCount()
{
	int i;
	{
		RVCriticalSection::LockGuard lock(*m_XmlPrivatesCountMutex);
		i = m_PrivatesCount--;
	}
	return i;
}

int CWLXmlDocument::GetPrivatesCount()
{
	int i;
	{
		RVCriticalSection::LockGuard lock(*m_XmlPrivatesCountMutex);
		i =  m_PrivatesCount;
	}
	return i;
}


IXMLDOMDocumentPtr* CWLXmlDocument::GetPrivateInterface()
{
	if(!m_pPrivate)
	return m_pPrivate;
}
*/

CWLXmlParseInterfacePointer* CWLXmlDocument::GetPrivateInterface()
{
	return m_pInterfacePtr;
}


void CWLXmlDocument::SetDom(CWLDocument* doc)
{
	m_pDom = doc;
}

void CWLXmlDocument::SetPrivateInterface(CWLXmlParseInterfacePointer* prv)
{
	m_pInterfacePtr = prv;
}

CWLDocument* CWLXmlDocument::GetDom()
{
	return m_pDom;
}

const char* CWLXmlDocument::GetXmlStandAloneStr(RVCString& value, RVCString& error)
{
	RVCString entityStr;
	const char* entityChar;
	if(!m_DocTypeErrorString.isNull()){
		error = m_DocTypeErrorString;
		return NULL;
	}
	size_t leftStrPtr = 0;
//	bool isExternalDtdPart = false;
	if(m_pDtdExternal){
//		isExternalDtdPart = true;
		value.append(m_InnerHTML(leftStrPtr, m_pDtdExternal->GetStartLocationParent() - leftStrPtr));
		value.append(" [");
		entityChar = m_pDtdExternal->entityFoldSelf(entityStr, error);
		leftStrPtr = m_pDtdExternal->GetEndLocationParent();
//		delete m_pDtdExternal;
//		m_pDtdExternal = NULL;
		if(!entityChar)
			return NULL;
		value.append(entityChar);
	}
	if(m_pDtdInternal){
		entityStr.remove(0);
		if(!m_pDtdExternal)
			value.append(m_InnerHTML(leftStrPtr, m_pDtdInternal->GetStartLocationParent() - leftStrPtr));
		entityChar = m_pDtdInternal->entityFoldSelf(entityStr, error);
		leftStrPtr = m_pDtdInternal->GetEndLocationParent();
		if(!entityChar)
			return NULL;
		else
			value.append(entityChar);
	}

	if(!m_pDtdInternal && m_pDtdExternal)
		value.append(']');
	if(m_pDtdInternal != NULL){
		delete m_pDtdInternal;
		m_pDtdInternal = NULL;
	}
	if(m_pDtdExternal != NULL){
		delete m_pDtdExternal;
		m_pDtdExternal = NULL;
	}
	if(m_InnerHTML.length() - leftStrPtr > 0)
		value.append(m_InnerHTML(leftStrPtr, m_InnerHTML.length() - leftStrPtr - (m_pDtdInternal || m_pDtdExternal ? 1 : 0)));
	return value.data();
}


void CWLXmlDocument::printAll()
{
	cout<<"ID:       "<<m_Id.data()<<'\n';
	cout<<"SRC:      "<<m_Src.data()<<'\n';
	cout<<"InnerHTML:     "<<'\n'<<m_InnerHTML.data()<<'\n'<<'\n';
}
