/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include <windows.h>
#include <winsock.h>

#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
#include <tchar.h>
#include <io.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "chaphttpauth.h"

// globals

#define MAX_TRY			2

#define HD_AUTHENTICATE         "WWW-Authenticate:"
#define HD_LENGTH               "Content-Length:"
#define HD_CONNECTION           "Connection:"


// Helper functions

void SkipWhite(PSTR *ppS )
{
    PSTR pS = *ppS;

    while ( *pS && isspace(*pS) )
        ++pS;

    *ppS = pS;
}


void SkipNonWhite(PSTR *ppS )
{
    PSTR pS = *ppS;

    while ( *pS && !isspace(*pS) )
        ++pS;

    *ppS = pS;
}



BOOL
HttpGetSocket(
    char * Verb,
    char * Server,
    char * URL,
    BOOL   DisplayHeaders,
    DWORD  ClientDataSize,
    PSTR   pchUserName,
    PSTR   pchPassword,
    PSTR   pszStore,
    PSTR   pszPref
    )
{

	char               ReceiveBuffer[8*1024];
    int                Error;
    char               Request[1024];
    int                RequestSize;
    char *             AcceptTypes[2] = {"*/*", NULL};
    SOCKET             Socket = INVALID_SOCKET;
    WSADATA            WsaData;
    struct sockaddr_in Address;
    struct hostent *   HostEntry;
    char               Headers[] = 
                           "HTTP/1.0\r\n"
                           "User-Agent: AuthClient\r\n"
                           "Accept: */*\r\n";
    char               CrLf[] = "\r\n";
    char               ClientData[64*1024];
	int                cRec;
    DWORD              cLen;
    BOOL               fInHeader;
	PSTR               pH;
    PSTR               pN;
    BOOL               fStatusLine;
    int                Status = -1;
    DWORD              cToRead;
    BOOL               fSt = FALSE;
    int                hnd = EOF;

    BOOL fNeedAuthenticate = FALSE;
	BOOL fServerKeepAlive = FALSE; //server side
	int	 nCount = 0;			//counts the number of tries
    PSTR pchAuthData = NULL;	//will use to hold the random number data
	BOOL fNeedMoreData;
	long i;
	
	
	Error = WSAStartup (0x101, &WsaData);

    if (Error == SOCKET_ERROR) 
    {
        fprintf(stderr, "Error in WSAStartup = %d\n", GetLastError());
        return FALSE;
    }
	
	//
    // Connect to the server
    //

    if  ( pszStore != NULL )
        if ( (hnd = _open( pszStore, _O_BINARY | _O_CREAT | _O_TRUNC | _O_RDWR, S_IREAD|S_IWRITE )) == EOF )
        {
            fprintf( stderr, "Can't create file %s\n", pszStore );
            return FALSE;
        }

	//Create Socket
	if ( Socket == INVALID_SOCKET )
    {
        Socket = socket(AF_INET, SOCK_STREAM, 0);

        if (Socket == INVALID_SOCKET) 
        {
            fprintf(stderr, "Error creating socket = %d\n", GetLastError());
            fSt = FALSE;
            goto ex;
        }

        Address.sin_family = AF_INET;
        Address.sin_port = 0;
        Address.sin_addr.s_addr = INADDR_ANY;

        Error = 
        bind(
            Socket,
            (struct sockaddr *) &Address,
            sizeof(Address));

        if (Error) 
        {
            fprintf(stderr, "Error in bind = %d\n", GetLastError());
            fSt = FALSE;
            goto ex;
        }

        Address.sin_family = AF_INET;
        Address.sin_port = htons(80);
        Address.sin_addr.s_addr = inet_addr(Server);

        if (Address.sin_addr.s_addr == -1) 
        {
            //
            // Must be a server name
            //
            HostEntry = gethostbyname(Server);
            if (HostEntry == NULL) 
            {
                printf("unable to resolve %s\n", Server);
                fSt = FALSE;
                goto ex;
            } else 
            {
                Address.sin_addr.s_addr = *((unsigned long *) HostEntry->h_addr);
            }
        }

        Error =
        connect(
            Socket,
            (struct sockaddr *) &Address,
            sizeof(Address));

        if (Error) 
        {
            fprintf(stderr, "Error connecting to %s = %d\n", Server, GetLastError());
            fSt = FALSE;
            goto ex;
        }
    }

	//
    // Send the client request
    //

    strcpy(Request, Verb);
    strcat(Request, " ");
	strcat(Request, URL);
    strcat(Request, " ");
    strcat(Request, Headers);
    if (ClientDataSize) 
    {
        sprintf(ClientData, "Content-Length: %d\r\n", ClientDataSize);
        strcat(Request, ClientData);
    }
	
	strcat(Request, CrLf);

    RequestSize = strlen(Request);

    Error =
    send(
        Socket,
        Request,
        RequestSize,
        0);

	printf("%s\n",Request);

    if (Error != RequestSize) 
    {
        printf("Error in client send = %d, %d\n", Error, GetLastError());
        fSt = FALSE;
        goto ex;
    }

	if (ClientDataSize) 
    {
        memset( ClientData, ' ', ClientDataSize );

        //
        // Send the client data
        //

        Error =
        send(
            Socket,
            ClientData,
            ClientDataSize,
            0);

        if ( (DWORD)Error != ClientDataSize) 
        {
            printf("Error in client send = %d, %d\n", Error, GetLastError());
            fSt = FALSE;
            goto ex;
        }
    }

	// parse status & header

    cLen = (DWORD)-1;
    fInHeader = TRUE;

	for ( pH = ReceiveBuffer, fStatusLine = TRUE ; fInHeader ; )
    {
        cRec = recv( Socket, pH, ReceiveBuffer+sizeof(ReceiveBuffer)-pH, 0 );
        if ( cRec <= 0 )
        {
            closesocket( Socket );
            Socket = INVALID_SOCKET;
            break;
        }
        pH[ cRec ] = '\0';

        // Iterate on header fields

        while ( pN = strstr(pH, "\r\n" ) )
        {
            *pN = '\0';

            if ( DisplayHeaders )
            {
                printf( "%s\n", pH );
            }

            if ( fStatusLine )
            {
                // This is the status line, decode status

                SkipNonWhite( &pH );
                SkipWhite( &pH );
                Status = atoi( pH );
                if ( Status == 401 )
                {
                    fNeedAuthenticate = TRUE;
                }
                fStatusLine = FALSE;
            }
            else if ( pN == pH )    // end of header fields
            {
                if ( hnd != EOF )
                    write( hnd, pH+2, ReceiveBuffer+cRec-pH-2 );

                cLen -= ( ReceiveBuffer+cRec-pH-2 );
                fInHeader = FALSE;
                break;
            }
            else if ( !strnicmp( pH, HD_AUTHENTICATE, sizeof(HD_AUTHENTICATE)-1 ) )
            {
                SkipNonWhite( &pH );
                SkipWhite( &pH );

                // check if we are already in the authentication sequence

                if (i==0)
                {
                    // store pointer to authenticate blob

                    SkipNonWhite( &pH );
                    SkipWhite( &pH );
                    pchAuthData = pH;
                }
            }
            else if ( !strnicmp( pH, HD_LENGTH, sizeof(HD_LENGTH)-1 ) )
            {
                // get content length

                SkipNonWhite( &pH );
                SkipWhite( &pH );
                cLen = atoi( pH );
            }
            else if ( !strnicmp( pH, HD_CONNECTION, sizeof(HD_CONNECTION)-1 ) )
            {
                // check for keep-alive flag

                SkipNonWhite( &pH );
                SkipWhite( &pH );
                if ( !strnicmp( pH, "Keep-Alive", sizeof("Keep-Alive")-1 ) )
                    fServerKeepAlive = TRUE;
            }

            pH = pN + 2;
        }
    }

	// read message body

    if ( Socket != INVALID_SOCKET )
    {
        for ( ; cLen ; )
        {
            if ( (cToRead = sizeof(ReceiveBuffer)) > cLen )
                cToRead = cLen;
            cRec = recv( Socket, ReceiveBuffer, cToRead, 0 );

            if ( cRec <= 0 )
            {
                closesocket( Socket );
                Socket = INVALID_SOCKET;
                break;
            }
            if ( hnd != EOF )
                write( hnd, ReceiveBuffer, cRec );

            cLen -= cRec;
        }
    }


	//Code for Starting Handshake...

	//close the socket
	closesocket( Socket );
    Socket = INVALID_SOCKET;

	//start a new socket similar to the last one (connecting to the same server as before)
	//Create Socket
	if ( Socket == INVALID_SOCKET )
    {
        Socket = socket(AF_INET, SOCK_STREAM, 0);

        if (Socket == INVALID_SOCKET) 
        {
            fprintf(stderr, "Error creating socket = %d\n", GetLastError());
            fSt = FALSE;
            goto ex;
        }

        Address.sin_family = AF_INET;
        Address.sin_port = 0;
        Address.sin_addr.s_addr = INADDR_ANY;

        Error = 
        bind(
            Socket,
            (struct sockaddr *) &Address,
            sizeof(Address));

        if (Error) 
        {
            fprintf(stderr, "Error in bind = %d\n", GetLastError());
            fSt = FALSE;
            goto ex;
        }

        Address.sin_family = AF_INET;
        Address.sin_port = htons(80);
        Address.sin_addr.s_addr = inet_addr(Server);

        if (Address.sin_addr.s_addr == -1) 
        {
            //
            // Must be a server name
            //
            HostEntry = gethostbyname(Server);
            if (HostEntry == NULL) 
            {
                printf("unable to resolve %s\n", Server);
                fSt = FALSE;
                goto ex;
            } else 
            {
                Address.sin_addr.s_addr = *((unsigned long *) HostEntry->h_addr);
            }
        }

        Error =
        connect(
            Socket,
            (struct sockaddr *) &Address,
            sizeof(Address));

        if (Error) 
        {
            fprintf(stderr, "Error connecting to %s = %d\n", Server, GetLastError());
            fSt = FALSE;
            goto ex;
        }
    }

	//
    // Send the client request
    //

    strcpy(Request, Verb);
    strcat(Request, " ");
	strcat(Request, URL);
    strcat(Request, " ");
    strcat(Request, Headers);
    if (ClientDataSize) 
    {
        sprintf(ClientData, "Content-Length: %d\r\n", ClientDataSize);
        strcat(Request, ClientData);
    }
	strcat(Request, "Connection: Keep-Alive\r\n" );
	
	//perfom only one loop when UserName and Password are set to NULL
	if ( ((*pchUserName == '\0') &&(*pchPassword == '\0')) )
		nCount++;

	while (fNeedAuthenticate)
	{
		pchAuthData = NULL;
		nCount++;
		/*
		call the following function in order to load the security dll which 
		belongs to the current's machine Windows system
		*/
printf(" -------------------------- Initializing\n");
		if ( !InitAuthorizationHeader() )
			{
				fprintf(stderr, "Cannot initialize security module\n" );
				return FALSE;
			}

		if (nCount <= (int)MAX_TRY)
			for (i=0 ; i<2 ; i++)
			{
				if (nCount == 2)
				{
					*pchUserName = '\0';
					*pchPassword = '\0';
				}

				// The User Name and Password should be taken from the Java Script

				/*
				Make the request headers similar to the previous request and then:->
				add the "Connection: Keep-Alive\r\n" header, then call the following
				function.
				
				The parameters:

				1. Pointer to the end of the last request header
				2. Pointer to the Authtentication method to use
				3. Pointer to the server sent authentication data (Random Number)
				4. Pointer to the User Name under whose behalf the NTLM method would work,
				if set to NULL the name of the user under whose security context the process
				is executing would be taken.
				5. Pointer to the User's Password.
				** If either the User Name nor the User's Password is set to NULL both would be
				taken from the user under whose security context the process is executing.
				*/
//				if ( !AddAuthorizationHeader( Request + strlen(Request), "NTLM", pchAuthData,
//					pchUserName, pchPassword, &fNeedMoreData ) )

				char value[1024];
				if ( !GetAuthorizationValue( value, "NTLM", (const char*)pchAuthData,
					(const char*)pchUserName, (const char*)pchPassword, (bool*)&fNeedMoreData ) )
				{
					printf( "Authentication failed\n" );
				}
				strcat(Request, "Authorization: ");
				strcat(Request, value);
				strcat(Request, "\r\n");

				fServerKeepAlive = FALSE;	//server side (for current response)
				fNeedAuthenticate = FALSE;

				strcat(Request, CrLf);

				RequestSize = strlen(Request);

				Error =
				send(
					Socket,
					Request,
					RequestSize,
					0);

				printf("%s\n",Request);

				if (Error != RequestSize) 
				{
					printf("Error in client send = %d, %d\n", Error, GetLastError());
					fSt = FALSE;
					goto ex;
				}

				if (ClientDataSize) 
				{
					memset( ClientData, ' ', ClientDataSize );

					//
					// Send the client data
					//

					Error =
					send(
						Socket,
						ClientData,
						ClientDataSize,
						0);

					if ( (DWORD)Error != ClientDataSize) 
					{
						printf("Error in client send = %d, %d\n", Error, GetLastError());
						fSt = FALSE;
						goto ex;
					}
				}

				
				cLen = (DWORD)-1;
				fInHeader = TRUE;
				fServerKeepAlive = FALSE;
				fNeedAuthenticate = FALSE;

				for ( pH = ReceiveBuffer, fStatusLine = TRUE ; fInHeader ; )
    {
        cRec = recv( Socket, pH, ReceiveBuffer+sizeof(ReceiveBuffer)-pH, 0 );
        if ( cRec <= 0 )
        {
            closesocket( Socket );
            Socket = INVALID_SOCKET;
            break;
        }
        pH[ cRec ] = '\0';

        // Iterate on header fields

        while ( pN = strstr(pH, "\r\n" ) )
        {
            *pN = '\0';

            if ( DisplayHeaders )
            {
                printf( "%s\n", pH );
            }

            if ( fStatusLine )
            {
                // This is the status line, decode status

                SkipNonWhite( &pH );
                SkipWhite( &pH );
                Status = atoi( pH );
                if ( Status == 401 )
                {
                    fNeedAuthenticate = TRUE;
                }
				else
					break;
                fStatusLine = FALSE;
            }
            else if ( pN == pH )    // end of header fields
            {
                if ( hnd != EOF )
                    write( hnd, pH+2, ReceiveBuffer+cRec-pH-2 );

                cLen -= ( ReceiveBuffer+cRec-pH-2 );
                fInHeader = FALSE;
                break;
            }
            else if ( !strnicmp( pH, HD_AUTHENTICATE, sizeof(HD_AUTHENTICATE)-1 ) )
            {
                SkipNonWhite( &pH );
                SkipWhite( &pH );

                // check if we are already in the authentication sequence

                if (i==0)
                {
                    // store pointer to authenticate blob

                    SkipNonWhite( &pH );
                    SkipWhite( &pH );
                    pchAuthData = pH;
                }
            }
            else if ( !strnicmp( pH, HD_LENGTH, sizeof(HD_LENGTH)-1 ) )
            {
                // get content length

                SkipNonWhite( &pH );
                SkipWhite( &pH );
                cLen = atoi( pH );
            }
            else if ( !strnicmp( pH, HD_CONNECTION, sizeof(HD_CONNECTION)-1 ) )
            {
                // check for keep-alive flag

                SkipNonWhite( &pH );
                SkipWhite( &pH );
                if ( !strnicmp( pH, "Keep-Alive", sizeof("Keep-Alive")-1 ) )
                    fServerKeepAlive = TRUE;
            }

            pH = pN + 2;
        }
    }
				/*
				Send the request and parse the response headers
				If the status code returned is 401 then:->
				*/
				//	fNeedAuthenticate = TRUE;
				//else:->
				//	fNeedAuthenticate = FALSE;
				//	break;

				
				/*
				Search if there exists an header - "WWW-Authenticate" take the right-most
				data token (= dataToken) then :->
				*/
				/*if (i == 0) // First iteration
					pchAuthData = dataToken;
				else if (dataToken != "NTLM")
				{
					fNeedAuthenticate = FALSE;
					break;
				}*/
					

				/*
				Search if there exist an header  - "Connection: Keep-Alive" (must 
				appear on the first iteration) then:->
				*/
				//	fServerKeepAlive = TRUE;

				//handle "Content-Length" header , if exists (as usual)

				// read message body

				if ( Socket != INVALID_SOCKET )
				{
					for ( ; cLen ; )
					{
						if ( (cToRead = sizeof(ReceiveBuffer)) > cLen )
							cToRead = cLen;
						cRec = recv( Socket, ReceiveBuffer, cToRead, 0 );

						if ( cRec <= 0 )
						{
							closesocket( Socket );
							Socket = INVALID_SOCKET;
							break;
						}
						if ( hnd != EOF )
							write( hnd, ReceiveBuffer, cRec );

						cLen -= cRec;
					}
				}

				if (!fServerKeepAlive)
					/*
					then:->
					1) close the current socket
					2) start a new socket similar to the last one (connecting to the
					same server as before)
					*/
					{
						closesocket( Socket );
						Socket = INVALID_SOCKET;
						//Create Socket
						if ( Socket == INVALID_SOCKET )
						{
							Socket = socket(AF_INET, SOCK_STREAM, 0);

							if (Socket == INVALID_SOCKET) 
							{
								fprintf(stderr, "Error creating socket = %d\n", GetLastError());
								fSt = FALSE;
								goto ex;
							}

							Address.sin_family = AF_INET;
							Address.sin_port = 0;
							Address.sin_addr.s_addr = INADDR_ANY;

							Error = 
							bind(
								Socket,
								(struct sockaddr *) &Address,
								sizeof(Address));

							if (Error) 
							{
								fprintf(stderr, "Error in bind = %d\n", GetLastError());
								fSt = FALSE;
								goto ex;
							}

							Address.sin_family = AF_INET;
							Address.sin_port = htons(80);
							Address.sin_addr.s_addr = inet_addr(Server);

							if (Address.sin_addr.s_addr == -1) 
							{
								//
								// Must be a server name
								//
								HostEntry = gethostbyname(Server);
								if (HostEntry == NULL) 
								{
									printf("unable to resolve %s\n", Server);
									fSt = FALSE;
									goto ex;
								} else 
								{
									Address.sin_addr.s_addr = *((unsigned long *) HostEntry->h_addr);
								}
							}

							Error =
							connect(
								Socket,
								(struct sockaddr *) &Address,
								sizeof(Address));

							if (Error) 
							{
								fprintf(stderr, "Error connecting to %s = %d\n", Server, GetLastError());
								fSt = FALSE;
								//goto ex;
								break;
							}
						}
				}
						//
						// Send the client request
						//

						strcpy(Request, Verb);
						strcat(Request, " ");
						strcat(Request, URL);
						strcat(Request, " ");
						strcat(Request, Headers);
						if (ClientDataSize) 
						{
							sprintf(ClientData, "Content-Length: %d\r\n", ClientDataSize);
							strcat(Request, ClientData);
						}
						strcat(Request, "Connection: Keep-Alive\r\n" );

				//	}

			} //End of for loop
		else
			fNeedAuthenticate = FALSE;
printf(" ------------------------- Terminating\n");
		TerminateAuthorizationHeader();
	} //End of While loop

	// Treate the response the usual way and if the status = 401 then:->
	if ( Status == 401 )
		fprintf( stderr, "Authentication rejected by server\n" );
	
	fSt = Status;

	if ( Socket != INVALID_SOCKET )
        closesocket(Socket);

ex:

	if ( hnd != EOF )
        close( hnd );

    return fSt;

	
}
