/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLSslCiphersCollection.h: interface for the CWLSslCiphersCollection class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_WLSSLCIPHERSCOLLECTION_H__76063FB1_349C_403B_ACD0_9B79EA76BDAD__INCLUDED_)
#define AFX_WLSSLCIPHERSCOLLECTION_H__76063FB1_349C_403B_ACD0_9B79EA76BDAD__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <limits.h>


#define MAX_CIPHER_SUITES   128
#define NO_CIPHER_SUITE     ((MAX_CIPHER_SUITES)+1)
#define DEFAULT_ENCRYPTION_STRENGTH	INT_MAX



class CWLConnectionsPool;
class RVCriticalSection;

struct ssl_st;
struct ssl_ctx_st;
struct ssl_method_st;

typedef struct ssl_st SSL;
typedef struct ssl_ctx_st SSL_CTX;
typedef struct ssl_method_st SSL_METHOD;


class CWLSslCiphersCollection  
{
public:
	CWLSslCiphersCollection( CWLConnectionsPool* cp );
	virtual ~CWLSslCiphersCollection();


public:
	int Init( int strength, bool verifyStrength );
	int DoCommand( int command, const char* cipherName );
	int SetStrength( int strength );

	void EnableNamed( const char* cipherName, bool& wasChanged );
	void DisableNamed( const char* cipherName, bool& wasChanged );
	void EnableId( int cipherSuite, bool& wasChanged );
	void DisableId( int cipherSuite, bool& wasChanged );

	bool Update( CWLGenericConnection* pConnection );

	int NameToId( const char * name );
	const char* IdToName( int id );

	int GetCipherCount();
	void GetStrengthList(char** list);

private:
	int InitTemplate();
	int EnableAll(bool& wasChanged);
	int DisableAll(bool& wasChanged);
	void ShowEnabled();
	void ShowAll();

	bool isInList (int bits, int* list, int listLength);
	int getEnabledCiphersCount( void* ciphersStack );

public:
typedef enum
{	SSL_Version_Undetermined = 0,
	SSL_Version_3_0_With_2_0_Hello = 100,
	SSL_Version_3_0_Only = 101,
	TLS_Version_1_0_Only = 102,
	TLS_Version_1_0_With_2_0_Hello = 103,
	SSL_Version_2_0 = 0x0002,
	SSL_Version_3_0 = 0x0300,
	TLS_Version_1_0 = 0x0301
}SSLProtocolVersion;

// cipher commands
enum {CIPHER_NULL_COMMAND,
      CIPHER_SHOW_ALL, 
      CIPHER_SHOW_ENABLED, 
      CIPHER_ENABLE_ALL, 
      CIPHER_DISABLE_ALL, 
      CIPHER_ENABLE, 
      CIPHER_DISABLE
};

typedef enum
{	SSL_AllCrypto,
	SSL_StrongCryptoOnly,
	SSL_ExportCryptoOnly,
	SSL_ServerGatedCrypto
} SSLCryptoStrength;


private:
	CWLConnectionsPool* m_Cp;

	bool m_Init;
	int m_Strength;	// ciphersuites strength limit
	unsigned m_RequestedStrength;	// requested ciphersuites strength limit (used for error messages)
	unsigned m_Count;	// number of enabled cipher suites

	SSL* m_Ssl; // this is the template object for all SSL sessions.
	SSL_CTX* m_Context; // context
	SSL_METHOD* m_Method; // SSL method ( depends on CWLWorkspace::m_SslVersion )
	int m_SslVersion;


private:
	unsigned long * m_IDs;
	int m_numOfCiphersInList;

	RVCriticalSection* m_SCCLock;
};

#endif // !defined(AFX_WLSSLCIPHERSCOLLECTION_H__76063FB1_349C_403B_ACD0_9B79EA76BDAD__INCLUDED_)
