/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : CommandManager.h
 *
 *	Author : Zion Menahem
 *
 *	Description :
 *
 *	History :
 *
 *	Modifications :
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/
#ifndef _CommandManager_h_
#define _CommandManager_h_

/** include files **/
#include <stdio.h>
#include <string.h>
#include <fstream.h>
#include <ordered.h>
#include <collstr.h>
#include <thr.h>
#include <thr.h>

#include <platform.h>
#include <tvpsync.h>
#include "wlleprotocol.h"

#include "wlhtmlvirtualqueue.h"
#include "callbackdata.h"

#include "gt_unicode.h"

#include "rvjsapi.h"

#include "httphost.h"
#include "wlerrorlib.h"

#include "jshttpnames.h"
#include "wlhttpcache.h"

/** local definitions **/
#define RELEASE_STRING "RELEASE"

// some debug messages
#ifdef _DEBUG
#define DEBUG_SET_END_OF_ROUND 1
#endif


#define CWLPage frameContainer
#define CWLHttpValue RVCString


/** default settings **/

/** external functions **/
//extern void  ParseCharForHttp();

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** public functions **/

/** private functions **/

/** classes definitions **/

class CWLHttpPrivateData;
class CWLWorkspace;
class CWLConnectionsPool;
class CWLHttpRequest;
class CWLCookies;
class frameContainer;
class CWLDocument;
class CWLQueue;
class CWLTransactionData;
class CWLTransactionsObject;
class CWLDomItem;


//===================================================================
class DllExport CWLCommandManager : public iLEProtocolVCInMix {
private:
	bool                 m_Initialized;
	unsigned short      m_MaxThreads;

	CWLConnectionsPool* m_Connections;	// connection pool

  CWLHttpCache        m_HttpCache;
	CLEVariablesData*   m_VariablesParent;
	CWLWorkspace*       m_Workspace;
  CWLCookies*         m_Cookie;
	CWLPage*            m_Page;	// frameContainer

	CWLHttpRequest**    m_Request;
	RVThread**          m_RequestThread;
	CWLQueue*           m_UrlQueue;

	JSObject*           m_JSDocument;


	// save for transaction end severity & reason
	RVCriticalSection*      m_LastErrorLock;
	WLLogMessage::Severity	m_LastSeverity;
	RVCString								m_LastError;
  long                    m_lActiveDomId ;
  bool                    m_bErrorInTransTree;

  int                     m_RequestOnConnectionCounter;
  RVCriticalSection       m_RequestOnConnectionCounterGuard;

  RVCString           url;
public:
  RVCString			      m_sTransName ;
	RVCriticalSection*				m_SharedDataLock; //V.K.
	RVCString						m_AuthenticateBasicHeaders; // Authenticate header will be handled for all threads
	RVCString						m_AuthenticateNTHeaders; // Authenticate header will be handled for all threads
	RVCString						m_AuthenticateBasicProxyHeaders; // Authenticate header will be handled for all threads
	RVCString						m_AuthenticateNTProxyHeaders; // Authenticate header will be handled for all threads

	unsigned            srcIndex;

	bool                m_ReadFromFile;
	RVCString           m_Path;
  long                m_lExpectedDomId ;

  bool InsertTransactions(CWLDocument *frame, bool bReport);
  bool InsertFrameTransactions(CWLDocument *frame, bool bReport);
  void SendBeginTransaction(CWLTransactionData *pTrans, bool bTopFrame=false);
  void UpdateParentTransaction(CWLTransactionData *pParentTrans, CWLTransactionData *pTrans);
  bool UpdateTransactions(CWLDocument *frame);
  bool UpdateFrameTransactions(CWLDocument *frame);
  bool InsertAllTransactions();
  void UpdateAllTransactions();
  bool EndTransaction(CWLTransactionData *pTrans, bool bSendTimers);
  void UpdateGetTime();
	void UpdateObjectsBeginTimer(CWLTransactionData *pParentTrans, CWLTransactionData *pTrans);
  double GetPageTime();
  void IncrementRequestOnConnectionCounter();
  void ResetRequestOnConnectionCounter();
  int GetRequestOnConnectionCounter();
  void CloseCBData();

#ifdef _DEBUG
	unsigned long m_RoundNum;
#endif

public:
	CWLCommandManager(iLEProtocolAgendaInMix& father, bool bCallback=false);
	virtual ~CWLCommandManager();

	int            DoCommand(CWLHttpPrivateData* HttpVars, HttpFuncs method, JSContext* cx=NULL);

	CWLWorkspace*  GetWorkspace() { return m_Workspace; }
	CWLCookies*    GetCookies() { return m_Cookie; }
	CWLPage*       GetPage() { return m_Page; }
	CWLQueue*      UrlQueue() { return m_UrlQueue; }
	CWLHttpCache&  GetHttpCache() { return m_HttpCache; }

	void        ClearDnsCache();
	void        ClearSslCache();
	void        ConnectionClose();
  bool	      IsParsed();
  RVCString&  GetPageName();
	CWLDocument*       GetParsedHtmlTree();

	int					SetCookie(char* name, char* value, char* domain, char* path, char* expire);
	int					DeleteCookie(const char* name, const char* domain, const char* path);
  void        ClearAllCookies();

//	void        SetUrl(const char* URL) { url = URL; }
	void        ResetProperties();

	unsigned short ThreadsNumber() { return m_MaxThreads; }

	CLEVariablesData*         GetVariablesParent();
	void                      SetVariablesParent(CLEVariablesData*);


	void        SetJSDocument(JSObject*);
	JSObject*   GetJSDocument();

	CWLConnectionsPool* GetConnPool() { return m_Connections; }

	void        SetLastError(WLLogMessage::Severity sev, const char* err);
 

	int         ParseFile(RVCString& path, CWLDocument* frame);
	int         CreateDom(CWLHttpPrivateData* HttpVars, const char* file, CWLDocument** dom, int mode);
        int         CreateExpectedDom(CWLHttpPrivateData* HttpVars, CWLDomItem* pDomItem, CWLDocument** dom);

#ifdef _DEBUG
	void ParseHtmlFile(const char* file);
	void DebugFunction(int msg);
	void ReadResponseFromFile(char* file, int size);
	void CreateTable(const char* name, int col, int rows, int cols);
#endif

private:
	bool           UpdateMaxThreads();
	bool           AddThread(int num);
	void           RemoveThread(int num);

	virtual void AtEndOfRound();

  //perveiw parameters and functions
protected:
  bool        m_bCallback ;
  bool        m_bWasHttpCommand;
  JSContext*  m_cx ;
  CUrlDataEx  m_PageCBData ;

public:
  void SetJScx(JSContext* cx){ m_cx = cx;}
  JSContext* GetJScx()  {return m_cx;}
  bool IsCallbackEnabled() {return m_bCallback;}
  bool WasHttpCommand() { return m_bWasHttpCommand; }
  bool IsTopFrame(CWLTransactionsObject* pTransObj);
  CUrlDataEx* GetCallbackData(){return &m_PageCBData;}  


// Chaim
  void SetBackup (bool sw, RVCollectableString* key);
  bool UseBackup (RVCollectableString* key);
  bool BackupExists (RVCollectableString* key);

  // the functions of the class iLEProtocolVCInMix
	virtual bool JSInitializeForClient(JSContext* cx, JSObject* obj, CLEVariablesData* variables);
	virtual bool OnThreadActivation();
	virtual bool BeforeInitClient();
	virtual bool AfterInitClient();
	virtual bool BeforeRound();
	virtual bool AfterRound();
	virtual bool BeforeTerminateClient();
	virtual bool AfterTerminateClient();
	virtual bool OnThreadTermination();
        virtual bool OnThreadRestart();
	virtual bool ExceptionHandler(std::exception&) { return true; }
        virtual bool BeforeSleep(long int){return true;}
	virtual bool AfterSleep(long int);
        virtual bool SleepDisabled(long int recordedSleepTime);
        virtual bool BeginTransaction(const char* transname, double time, const char* extraDataType, void* extraData, unsigned long len);
};

extern CWLCommandManager* WLGetCommandManagerPtr(JSContext *cx);

#endif _CommandManager_h_
