/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************************
*
*   Author  : Itaj Sherman & Yossi Nakash
*   Date    : 10.07.20000
*   File    : HttpCallbacks.h
*   Purpose : Classes implementing callbacks from HttpLoad.
*   Classes : iHttpProbingClient - interface for user to inherit & implement.
*
*******************************************************************************/
#ifndef _HTTP_CALLBACK_H
#define _HTTP_CALLBACK_H

#include "loadengineglobals.h"
#include "rtamscript.h"
#include "callbackdata.h"


class DllExport iHttpProbingClientGeneral;
class DllExport iHttpProbingClient;

/******************************************************************************
*
*   Class   : iHttpProbingClient
*   Purpose : Interface for http-related callbacks.
*
*******************************************************************************/
class iHttpProbingClientGeneral: public iLoadProbingClient
{
public:

  explicit iHttpProbingClientGeneral( cSingleClientProcess&, bool IsDebuggerActive );
	virtual ~iHttpProbingClientGeneral();

  void fInit( iSingleClientMachineThread&, const char** pDisabledProtocols = NULL );

  // pure virtual methods.
  virtual tSCActionReporterReturn pageStartDownload(iReportThreadState& threadState, CUrlData* pUrlData, long lExpectedDomId, long &lActiveDomId)=0;
  virtual tSCActionReporterReturn pageAbortDownload(iReportThreadState& threadState, CUrlData* pUrlData)=0;
  virtual tSCActionReporterReturn pageEndDownload(iReportThreadState& threadState, CUrlData* pUrlData)=0;
  virtual tSCActionReporterReturn objectStartDownload(iReportThreadState& threadState, CUrlData* pUrlData)=0;
  virtual tSCActionReporterReturn objectAbortDownload(iReportThreadState& threadState, CUrlDataEx* pUrlDataEx)=0;
  virtual tSCActionReporterReturn objectEndDownload(iReportThreadState& threadState, CallbackData* pData)=0;
  virtual tSCActionReporterReturn UseCachedObject(iReportThreadState& threadState, CUrlDataEx* pUrlDataEx)= 0 ;
};


class iHttpProbingClient: public iHttpProbingClientGeneral
{
public:
  explicit iHttpProbingClient( cSingleClientProcess& );
  virtual ~iHttpProbingClient(){}

private:
	void fJSReporter( WLLogMessage::Severity, const char* ) {}
};


#endif // _HTTP_CALLBACK_H

