//*************************************************************
//**
//**  Table Compare Library
//**  
//**  
//**  Copyright (c) 1996-1999, Radview Software Inc.
//**  
//**  
//**  Change Log:
//**  6/1999  - Initial version. By Uri Har'el
//**  7/1999  - Add the ability to initialize during InitAgenda. - Uri.
//**  9/1999  - Integrate the TableReporter instead of MessageReporter (for v4.0). - Uri.
//**  10/1999 - Comparison is done on innerText striped out of spaces - Uri.
//**  
//*************************************************************


//=============================================================
// Range functions
//=============================================================
function num2char(d)
{
  var a2z="ABCDEFGHIJKLMNOPQRSTUVWXYZ";
  if( d > 26 )
    return a2z.charAt(d/26)+a2z.charAt(d%26);
  else
    return a2z.charAt(d);
}
function makeRange(r1,c1,r2,c2)
{
  var range;
  if( r2 == undefined )
    range = String(r1)+","+String(c1)+","+String(r1)+","+String(c1);
  else
    range = String(r1)+","+String(c1)+","+String(r2)+","+String(c2);
  return range;
}

//function makeRange(r1,c1,r2,c2)
//{
//	var range=num2char(c1)+String(r1+1)
//	if( r2 != undefined )
//	  range += ":" + num2char(c2)+String(r2+1)
//	else
//	  range += ":" + range
//
//	return range
//}

function row2range(r)
{
  return makeRange(r.rowIndex,0,r.rowIndex,r.cells.length-1);
}


function rows2range(r1,r2)
{
  return makeRange(r1.rowIndex,0,r2.rowIndex,r2.cells.length-1);
}


//============================================================
// Error Reporting
//============================================================

// Add an informative message
function myAddMessage(msg)
{
  this.numMessages++;
  this.reporter.AddMessage(msg);
  if( this.useInfoMessage )
    InfoMessage(msg);
}

// Add an error
function myAddError(msg,range1,range2)
{
  this.errorFlag = true;
  if(! (this.parent.Details || this.parent.alwaysReport) )
    throw "abort compare";
  this.numErrors++;
  if( range1 == undefined && range2 == undefined )
  {
    this.reporter.AddMessage(msg);
    if( this.useInfoMessage )
    	InfoMessage(msg);
  }
  else
  {
    if( range2 != undefined )
      this.reporter.AddError(range1,range2,msg);
    else
      this.reporter.AddError(range1,"",msg);
    
    if( this.useInfoMessage )
      InfoMessage(msg+" Range1="+range1+((range2 != undefined)?" Range2="+range2 : ""));
  }
  if(this.parent.StopOnFailure)
  {
  	throw "Stop on failure";
  }
}

function myReport()
{
  if( this.parent.table1 != undefined )
    this.reporter.Table1 = "<html><body>"+this.parent.table1.table.outerHTML+"</body></html>";
  else
    this.reporter.Table1 = "";
  
  if( this.parent.table2 != undefined )
    this.reporter.Table2 = "<html><body>"+this.parent.table2.table.outerHTML+"</body></html>";
  else
    this.reporter.Table2 = "";
  
  if(this.frame == undefined)
  {
  	this.reporter.Report();
  }
  else
  {
  	var severity;
  	var testName;
  	
  	if(this.parent.saveProperties != undefined)
  	{
  		if(this.IdentificationError) //identification error
  		{
  			if(this.parent.saveProperties.IdentificationSeverity != undefined)
  				severity = this.parent.saveProperties.IdentificationSeverity;
  			else
  				severity = WLMinorError;
  		}	
  		else if(this.parent.saveProperties.VerificationSeverity != undefined) //verification error
  			severity = this.parent.saveProperties.VerificationSeverity;
  		else
  			severity = WLMinorError;
  			
  		if(this.parent.Name != undefined && this.parent.Name != "")
				testName = this.parent.Name;
			else
  			testName = "Table Compare";
  	}
  	else //no saveProperties
  	{
  		severity = WLMinorError;
  		testName = "Table Compare";
  	}
  	
  	if(this.onSuccess)
  		severity = WLSuccess;
  		
  	
  	wlHttp.SetSeverity(this.frame,severity);
	wlHttp.SetFailureReason(this.frame, testName,"")
  	wlHttp.AddEvent(this.frame,this.reporter,severity);
  }
  this.reporter.Clear();
  this.errorFlag = false;
  this.numErrors = 0;
  this.numMessages = 0;
}

// constructor function for my reporter
function MyReporter(o,name,onSuccess)
{
  var testName;
  var result;
  
  if(onSuccess)
  {
  	this.onSuccess = true;
  	result = " succeeded.";
  }
  else
  	result = " failed.";	
  if(o.Name != undefined && o.Name != "")
  	testName = o.Name;
  else
  	testName = "Table compare";
  this.reporter = new TableCompareReporter(name,testName+result);
  this.useInfoMessage = false; // a flag indicating whether to use InfoMessage as well as AddMessage
  this.errorFlag = false;
  this.numErrors = 0;
  this.numMessages = 0;
  this.AddMessage = myAddMessage;
  this.AddError = myAddError;
  this.Report = myReport;
  this.parent = o;
  
}

//=============================================================


//=============================================================
// default cell comparing
//=============================================================
function compareCellsByText(c1,c2)
{
  var cc1 = c1.innerText.replace(/[ \xA0]+/g,"");
  var cc2 = c2.innerText.replace(/[ \xA0]+/g,"");
  return this.innerCompareFunc(cc1,cc2);
}

function compareCellsByText_Inner(c1,c2)
{
  if( c1 != c2)
    return false;
  else
    return true;
}

function compareCellsByText_Verbose(c1,c2)
{
  if( c1 != c2)
  {
    cc1 = escape(c1);
    cc2 = escape(c2);
    for(var i = 0 ; i < cc1.length ; i++)
      if( i >= cc2.length || cc1.charAt(i) != cc2.charAt(i) )
        break;
    InfoMessage("C1="+cc1.substr(0,min(cc1.length,i+5)));
    InfoMessage("C2="+cc2.substr(0,min(cc2.length,i+5)));
    
    return false;
  }
  else
    return true;
}

function turnOnVerbose()
{
  this.innerCompareFunc = compareCellsByText_Verbose;
}


function compareCellsByHtml(c1,c2)
{
  return (c1.innerHTML == c2.innerHTML);
}

//=============================================================

// generates a list of column names according to the value of cells in the first
// row of a table
function createColumnNames(t)
{
  var ind;
  if( t.rows.length > 0 ) // there are rows
  {
    names = new Array(t.rows[0].cells.length);
    for(ind = 0 ; ind < t.rows[0].cells.length ; ind++)
      if( t.rows[0].cells[ind].innerText != "" )
        names[t.rows[0].cells[ind].innerText.replace(/[ \xA0]+/g,"")] = ind;
    
    return names;
  }
  else
    return;  // return undefined
}

function compareNumbers(a, b)
{
  return a - b;
}


// Takes a list of columns which may contain column indexes, column ranges and column names
// and returns a list of column indexes.
// Input:
//   columnList    - list of columns (using all above formats)
//   names         - list of column names and their prespective indexes for a table
//   tableName     - Used for reporting
//   abortFlag     - whether to abort is non-existing name is found
//   reporter      - for error reporting
// Output:
//   an array of indexex  corespounding to the column definitions in ColumnList
//
function columnListToIndex(columnsList,names,tableName,reporter)
{
  var na;
  if( typeof(columnsList) != "object" )
    return;  // illegal list
  
  indexes = new Array();
  for(n in columnsList)
    if( isNaN(columnsList[n]) )  // not a Number
      if( columnsList[n].indexOf("-") >= 0 ) // a range
      {
        var range = columnsList[n].split("-");
        if( ! ( isNaN(range[0]) || isNaN(range[1]) ) )
          for(ind = range[0] ; ind <= range[1] ; ind++)
            indexes.push(ind);
      }
      else
        if( (na = names[columnsList[n].replace(/[ \xA0]+/g,"")]) != undefined  )
          indexes.push(na);
        else
        {
          if( reporter != undefined )  // reporter exists
            reporter.AddError("Column name \""+na+"\" does not exist in "+tableName+".");
          return;  // error. name does not exists
        }
    else // a number
      indexes.push(columnsList[n]);
  return indexes;
}


// Translate a row list, that may contain ranges, into a sorted list of row indexes
function rowListToIndex(rowsList,reporter)
{
  var msg;
  indexes = new Array();
  for(n in rowsList)
    if( isNaN(rowsList[n]) )  // not a Number
      if( rowsList[n].indexOf("-") > 0 ) // a range
      {
        var range = rowsList[n].split("-");
        if( ! ( isNaN(range[0]) || isNaN(range[1]) ) )
          for(ind = range[0] ; ind <= range[1] ; ind++)
            indexes.push(ind);
      }
      else
        return;
    else
      indexes.push(rowsList[n]);
  
  indexes.sort(compareNumbers);
  return indexes;
}



// comparing two rows
function compareTwoRows(r1,r1_name,compareList1,r2,r2_name,compareList2,reporter)
{
  var ind, limit, diff;
  if( compareList1 == "all" ) // comapre all rows
  {
    // first check to see if the rows contains the same number of cells
    if( r1.cells.length > r2.cells.length ) // more cells in table 1
    {
      diff = r1.cells.length-r2.cells.length;
      reporter.AddError(r2_name+" is missing "+diff+" cell(s) in row "+r2.rowIndex+".",
                        makeRange(r1.rowIndex,r2.cells.length,r1.rowIndex,r1.cells.length-1),row2range(r2));
    }
    else if( r1.cells.length < r2.cells.length ) // more cells in table 2
    {
      diff = r2.cells.length-r1.cells.length;
      reporter.AddError(r1_name+" is missing "+diff+" cell(s) in row "+r1.rowIndex+".",row2range(r1),
                        makeRange(r2.rowIndex,r1.cells.length,r2.rowIndex,r2.cells.length-1));
    }
    
    limit = min(r1.cells.length,r2.cells.length);
    for(ind = 0 ; ind < limit ; ind++)
      if( ! this.CompareFunction(r1.cells[ind],r2.cells[ind]) ) // cells are not equal
      {
        reporter.AddError("Cell Comparison: Value mismatch in "+r1_name+"["+r1.rowIndex+","+ind+"] and "+r2_name+"["+r2.rowIndex+","+ind+"]",makeRange(r1.rowIndex,ind),makeRange(r2.rowIndex,ind));
      }
  }
  else // compare by a list of columns
  {
    for(ind = 0 ; ind < compareList1.length ; ind++)  // go over the list
    {
      if(compareList1[ind] < 0 || compareList2[ind] < 0)  // missing column in one of the tables
        continue;
      if( compareList1[ind] >= r1.cells.length && compareList2[ind] >= r2.cells.length)
      {
        reporter.AddError("Cell Not Found in both tables: "+r1_name+"["+r1.rowIndex+","+compareList1[ind]+"] and "+r2_name+"["+r2.rowIndex+","+compareList2[ind]+"].",row2range(r1),row2range(r2));
        continue;
      }
      else if(compareList1[ind] >= r1.cells.length) // cell does not exist in table 1
      {
        reporter.AddError("Cell Not Found: "+r2_name+"["+r2.rowIndex+","+compareList1[ind]+"] does not exist in "+r1_name+"["+r1.rowIndex+"].",row2range(r1),makeRange(r2.rowIndex,compareList2[ind]));
        continue;
      }
      else if(compareList2[ind] >= r2.cells.length) // cell does not exist in table 2
      {
        reporter.AddError("Cell Not Found: "+r1_name+"["+r1.rowIndex+","+compareList2[ind]+"] does not exist in "+r2_name+"["+r2.rowIndex+"].",makeRange(r1.rowIndex,compareList1[ind]),row2range(r2));
        continue;
      }
      
      if( ! this.CompareFunction(r1.cells[compareList1[ind]],r2.cells[compareList2[ind]]) ) // cells are not equal
      {
        reporter.AddError("Cell Comparison: Value mismatch in "+r1_name+"["+r1.rowIndex+","+compareList1[ind]+"] and "+r2_name+"["+r2.rowIndex+","+compareList2[ind]+"]",
                          makeRange(r1.rowIndex,compareList1[ind]),
                          makeRange(r2.rowIndex,compareList2[ind]));
      }
    }
  }
}


function RowPointer(key,row)
{
  this.key = key;
  this.row = row;
  this.toString = rowPointerToString;
}

function rowPointerToString()
{
  return this.key;
}


function arrangeRowsForCompare(rows,matchList,reporter)
{
  indexes = new Array;
  var ind, ind1, ind2;
  for(ind1 = 0 ; ind1 < rows.length ; ind1++)
  {
    key = "";
    for(ind2 = 0 ; ind2 < matchList.length ; ind2++)
    {
      ind = matchList[ind2];
      if( ind < rows[ind1].cells.length )
        key += rows[ind1].cells[ind].innerText.replace(/[ \xA0]+/g,"") + ",";
      else
        key += ",";
    }
    
    indexes.push(new RowPointer(key,rows[ind1]));
  }
  
  indexes.sort();
  return indexes;
}

// A Wrap around CompareTables
function compareTables(t,frame)
{
  if( wlGlobals.TableCompare && wlGlobals.TableCompareRound)
  {
    if(frame != undefined)
    	this.reporter.frame = frame;
    if( t != undefined )
    {
      if( this.table2 != undefined )
      {
        this.reporter.AddMessage("Compare(): The Actual Table is already defined. New table definition is ignored.");
        res = CompareTables(this);
      }
      else
        res = CompareTables(this,t);
      
      if( ! res ) // an error - report
        this.reporter.Report();		
      else if(this.Action.ReportOnSuccess) 
      	ReportOnSuccess(this);
  	
      delete this.table2;  // clear the second table for further comparisons
    }
    else
    {
      res = CompareTables(this);
      if( ! res ) // an error - report
        this.reporter.Report();
      else if(this.Action.ReportOnSuccess) 
      	ReportOnSuccess(this);
    }
  
    alwaysReport = true;
    return res;
  }
  else
    return true; // always succeed
}



// function for comparing tables.
function CompareTables(obj,t)
{
  
  try
  {
    with (obj)
    {
      if( t != undefined ) // set as actual
        obj.table2 = new TableDetails(t,"Actual Table");
      else if( obj.table2 == undefined ) //the actual table is missing
      {
        if(table1 == undefined)
        	reporter.AddError("Expected table could not be identified.");
        reporter.AddError("Actual table could not be identified.");
        reporter.IdentificationError = true;
        return false;
      }
      if(table1 == undefined) //the expected table is missing
			{
			 	reporter.AddError("Expected table could not be identified.");
				reporter.IdentificationError = true;
			  return false;
      }
      if( ! Prepare(true) ) // call Prepare and indicate that it is called from the compare function
      {
        reporter.AddError("Compare can not be performed due to previous errors.");
        return false;
      }
      alwaysReport = false;
      
      if( CompareRows != "all" ) // compare only some of the rows
      {
				// extruct the selected rows from the tables
        for(ind = 0 ; ind < rowIndexes.length ; ind++)
          if(rowIndexes[ind] >= table1.table.rows.length) // missing rows
          {
            reporter.AddError("Row "+rowIndexes[ind]+" does not exist in "+table1.name+". Only "+table1.table.rows.length+" rows in the table (starting from #0).");
            break;
          }
          else if(rowIndexes[ind] >= table2.table.rows.length) // missing rows
          {
            reporter.AddError("Row "+rowIndexes[ind]+" does not exist in "+table2.name+". Only "+table2.table.rows.length+" rows in the table (starting from #0).");
            break;
          }
          else
            obj.CompareTwoRows(table1.table.rows[rowIndexes[ind]],
                           table1.name,
                           table1.compareList,
                           table2.table.rows[rowIndexes[ind]],
                           table2.name,
                           table2.compareList,
                           reporter);
        
        return ! reporter.errorFlag;
                }
      else if( matchByRe.test(MatchBy) )
      {
        if( table1.table.rows.length > table2.table.rows.length)
        {
          diff = table1.table.rows.length-table2.table.rows.length;
          reporter.AddError("Missing "+diff+" row(s) in "+table2.name+".",
                            rows2range(table1.table.rows[table2.table.rows.length],
                                       table1.table.rows[table1.table.rows.length-1]),"");
        }
        else if( table1.table.rows.length < table2.table.rows.length && ReportUnexpectedRows)
        {
          diff = table2.table.rows.length-table1.table.rows.length;
          reporter.AddError(diff+" unexpected row(s) in "+table2.name+".","",
                            rows2range(table2.table.rows[table1.table.rows.length],
                                       table2.table.rows[table2.table.rows.length-1]));
        }
	
        limit = min(table1.table.rows.length,table2.table.rows.length);
        for(ind = 0 ; ind < limit ; ind++)
          obj.CompareTwoRows(table1.table.rows[ind],
                         table1.name,
                         table1.compareList,
                         table2.table.rows[ind],
                         table2.name,
                         table2.compareList,
                         reporter);
        
        return ! reporter.errorFlag;
      }
      else
      {
        index1 = 0;
        index2 = 0;
        
        while( index1 < table1.rows.length || index2 < table2.rows.length )
        {
          if( index1 < table1.rows.length && 
              (index2 >= table2.rows.length || table1.rows[index1].key < table2.rows[index2].key) ) // missing row in table2
          {
            reporter.AddError("Row not found in "+table2.name+".",row2range(table1.rows[index1].row),"");
            //reporter.AddError("Row not found:")
            //reporter.AddMessage("  Row "+table1.rows[index1].row.rowIndex+" in "+table1.name)
            //reporter.AddMessage("  does not exist in "+table2.name+" according to the MatchBy criteria.")
            index1++;
          }
          else if( index2 < table2.rows.length &&
                   (index1 >= table1.rows.length || table1.rows[index1].key > table2.rows[index2].key) ) // extra row in table2
          {
            if( ReportUnexpectedRows)
            {
              reporter.AddError("Row not found in "+table1.name+".","",row2range(table2.rows[index2].row));
              //reporter.AddError("Row not found:")
              //reporter.AddMessage("  Row "+table2.rows[index2].row.rowIndex+" in "+table2.name)
              //reporter.AddMessage("  does not exist in "+table1.name+" according to the MatchBy criteria.")
            }
            index2++;
          }
          else // found a match
          {
            obj.CompareTwoRows(table1.rows[index1].row,
                           table1.name,
                           table1.compareList,
                           table2.rows[index2].row,
                           table2.name,
                           table2.compareList,
                           reporter);
            index1++;
            index2++;
          }
        }
        
        return ! reporter.errorFlag;
      }
    }
  }
  catch(e)
  {
    return ! reporter.errorFlag;
  }
}

// Prepare the two tables (if necesssary).
// inCompare - a flag indicating whether the function was called from the Compare method
function prepareCompare(inCompare)
{
  try
  {
    with (this)
    {
      // if this is the first time prepare is called, save all properties to make sure noone will
      // change them before calling compare. If it's not te first time, retrieve the saved properties.
      if( this.saveProperties == undefined )
      {
        this.saveProperties = new TableCompare();
        this.saveProperties.CompareRows = CompareRows;
        this.saveProperties.ReportUnexpectedRows = ReportUnexpectedRows;
        this.saveProperties.MatchBy = MatchBy;
        this.saveProperties.CompareColumns = CompareColumns;
        this.saveProperties.Details = Details;
        if(this.Name != undefined && this.Name != "")
        	this.saveProperties.Name = this.Name;
        if(this.Action.VerificationSeverity != undefined && this.Action.VerificationSeverity >= WLSuccess && this.Action.VerificationSeverity <= WLSevereError)
        	this.saveProperties.VerificationSeverity = this.Action.VerificationSeverity;
        if(this.Action.IdentificationSeverity != undefined && this.Action.IdentificationSeverity >= WLSuccess && this.Action.IdentificationSeverity <= WLSevereError)
        	this.saveProperties.IdentificationSeverity = this.Action.IdentificationSeverity;
        this.reporter = new MyReporter(this,"Table Compare");
      }
      else
      {
        CompareRows = this.saveProperties.CompareRows;
        ReportUnexpectedRows = this.saveProperties.ReportUnexpectedRows;
        MatchBy = this.saveProperties.MatchBy;
        CompareColumns = this.saveProperties.CompareColumns;
        Details = this.saveProperties.Details;
        
      }
      
      if( CompareRows != "all" ) // compare only some of the rows
      {
        this.rowIndexes = rowListToIndex(CompareRows,reporter);
        if( this.rowIndexes == undefined )
        {
          reporter.AddError("Error in \"CompareRows\" definition");
          return false;
        }
      }
      if( this.table1 != undefined && this.table1.valid )
      {
        table1.prepare(this);
        if( ! table1.valid )
        {
          reporter.AddError("Can not compare "+table1.name);
          return false;
        }
      }
      
      if( this.table2 != undefined && this.table2.valid )
      {
        table2.prepare(this);
        if( ! table2.valid )
        {
          reporter.AddError("Can not compare "+table2.name);
          return false;
        }
      }
      
    }
    return true;
  }
  catch(e)
  {
    return ! reporter.errorFlag;
  }
}

// Perform any available preperation for compare using information currently stored in
// the object. This allows to create the object and prepare for compare during InitClient.
//
// co - the TableComapre object
function prepareTable(co)
{
  with (this)
  {
    if( ! prepared && valid)
    {
      this.names = createColumnNames(table);
      if(names == undefined)
      {
        valid = false; // make sure no compare will be done
        co.reporter.AddError("Can not find column names in "+name+". Check table.");
        return;
      }
      
      if( co.CompareColumns == "all" )
        this.compareList = co.CompareColumns;
      else
      {
        this.compareList = columnListToIndex(co.CompareColumns,this.names,name,co.reporter);
        if( this.compareList == undefined )
        {
          valid = false; // make sure no compare will be done 
          co.reporter.AddError("Error in \"CompareColumns\" definition. Column not found in "+name+".");
          return;
        }
      }
      
      
      if( ! co.matchByRe.test(co.MatchBy) )
      {
        this.matchColumns = columnListToIndex(co.MatchBy,this.names,name,co.reporter);
        if( this.matchColumns == undefined )
        {
          valid = false; // make sure no compare will be done 
          co.reporter.AddError("Error in \"MatchBy\" definition. Column not found in "+name+".");
          return;
        }
        
        this.rows = arrangeRowsForCompare(table.rows,this.matchColumns,co.reporter);
      }
      
      prepared = true;
    }
  }
  return;
}



// an object for holding information on a table, required during the compare operation:
// table         - the table it self
// name          - a name use for reporting
// names         - a list of column indexes according to column's names
// compareList   - a list of column indexes generated from compareColumn property
// matchColumns  - a list of column indexes generated from matchBy property
// rows          - a list of rows sorted by matchBy property
// prepared      - a flag indicating that all information regarding the table has been prepared
// valid         - indicates that there where no errors during preparetion for compare
//
// prepare()     - a method for preparing all the table information
function TableDetails(t,name)
{
  this.table = t;
  this.name = name;
  this.prepared = false;
  if( isTable(this.table) )
    this.valid = true;
  else
    this.valid = false;
  // all other properties will be generated during the prepare phase
  this.prepare = prepareTable;
}

// returns true is the object is a table
function isTable(t)
{
  if(t == undefined || t.cols == undefined)
    return false;
  else
    return true;
}


function min(a,b)
{
  return (a < b)?a:b;
}

// constructor function for the TableCompare class
function TableCompare(t1,t2)
{
  //for the test cases
  this.Identification = new Object;
	this.Verification = new Object;
  this.Action = new Object;
  
  if(t1 == undefined) //no exr - do nothing
  {
  	this.Prepare = doNothing;
	this.Compare = doNothing;
  	return;
  }
  if(t1 == "wrong path" || t1 == "wrong domID")
  	this.table1 = undefined;
  else
  	this.table1 = new TableDetails(t1,"Expected Table");
  if( t2 != undefined )
    this.table2 = new TableDetails(t2,"Actual Table");
  //this.reporter = new MyReporter(this,"Table Compare");
  this.CompareRows = "all";
  this.ReportUnexpectedRows = true;
  this.MatchBy = "row numbers";
  this.CompareColumns = "all";
  this.Details = true;
  this.CompareTwoRows = compareTwoRows
  this.CompareFunction = compareCellsByText;
  this.Compare = compareTables;
  this.Prepare = prepareCompare;
  this.matchByRe = /[rR][oO][wW]/;
  this.alwaysReport = true;
  if( wlGlobals.AsmDiagnostics != undefined && wlGlobals.AsmDiagnostics == true )
    this.innerCompareFunc = compareCellsByText_Verbose;
  else
    this.innerCompareFunc = compareCellsByText_Inner;
  this.TurnOnTableVerbose = turnOnVerbose;
}

function doNothing()
{
}


// a wrap arround the TableCompare constractor
/*function CreateTableCompareObject(domId,path)
{
	var table;
	
	if(wlGlobals.ExpectedDOMs != undefined && wlGlobals.ExpectedDOMs[domId] != undefined)
	{
		table = eval( "wlGlobals.ExpectedDOMs["+ domId +"]." + path );
		if(table == undefined)
		{
			WarningMessage("Corrupted expected DOM file. No verification will take place.");
			wlGlobals.Verification = false;
			wlGlobals.TableCompare = false;
		}
		return (new TableCompare(table));
	}
	else  //return an empty object
	{
		return (new TableCompare());
		
	}
	
}
*/



// a wrap arround the TableCompare constractor
function CreateTableCompareObject(domId,path)
{
	var table;
	
	if(wlGlobals.ExpectedDOMs != undefined)
	{
		if(wlGlobals.ExpectedDOMs[domId] != undefined)
		{
			table = eval( "wlGlobals.ExpectedDOMs["+ domId +"]." + path );
			if(table == undefined) //wrong path
			{
				return (new TableCompare("wrong path"));
			}
			else
			{
				return (new TableCompare(table));
			}
		}
		else //wrong domId
		{
			return (new TableCompare("wrong domID"));
		}
	}
	else  //return an empty object - no exr
	{
		return (new TableCompare());
		
	}
}


function ReportOnSuccess(obj)
{
	var frame;
	frame = obj.reporter.frame;
	obj.reporter = new MyReporter(obj,"Table Compare",true);
	obj.reporter.AddMessage("Table compare succeeded.");
	obj.reporter.frame = frame;
	obj.reporter.Report();
}