/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLGenericConnection.cpp: implementation of the CWLGenericConnection class.
//
//////////////////////////////////////////////////////////////////////

#include "WLGenericConnection.h"
#include "WLSecureSslConnection.h"
#include "WLInsecureConnection.h"

#include "commandmanager.h"
#include "connpool.h"
#include "workspace.h"
#include "httprequest.h"
#include "wltransactiondata.h"
//#include "jshttputils.h"
#include "httpdefines.h"
#include "wldocument.h"





//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWLGenericConnection::CWLGenericConnection( CWLHttpRequest* pHttpReq )
: m_pHttpRequest( pHttpReq )
, m_Socket( -1 )
, m_TcpConnected( false )
, m_Port( 0 )
, m_Cm( NULL )
, m_Cp( NULL )
, m_pTransData( NULL )
, m_Ws( NULL )
, m_pTransObj( NULL )
, m_httpStatus( 0 )
, m_UrlType( 0 )
{
	m_UrlType = FRAME;
}

CWLGenericConnection::~CWLGenericConnection()
{
	m_Socket = -1;
	CloseConnection(true); // Chaim - Call non virtual overloading
//	m_TcpConnected = false;
}

CWLGenericConnection* CWLGenericConnection::GetConnection( int protocol, const char* host, int port, int threadNum, CWLHttpRequest* pHttpReq )
{
	CWLGenericConnection* tmpConn = NULL;
/*	size_t i;
	for( i = 0; 0 < m_Session.entries(); i++ ){

		tmpConn = (CWLGenericConnection*)m_Session[i];
		if( NULL == tmpConn )
			continue;

		//IsMatching
		m_Session.remove( tmpConn );
		return tmpConn;
	}

	if( MAX_SESSIONS < i )
		return NULL;

	switch( protocol ){
		case SSL_PROTOCOL:
			tmpConn = new CWLSecureSslConnection;
			break;
		default:
			tmpConn = new CWLInsecureConnection;
	}

	if( NULL == tmpConn )
		return NULL;

	tmpConn->Init( host, port, threadNum, pHttpReq );
*/
	return tmpConn;
}

bool CWLGenericConnection::Init( CWLCommandManager* cm, CWLConnectionsPool* cp, sockaddr_in& addr, const char* host, int port )
{
	m_Cm = cm;
	m_Cp = cp;
	m_Ws = cm->GetWorkspace();

	RVCString aHost(host);
	memmove((void*)&m_SockAddress, (void*)&addr, sizeof(sockaddr_in));
	m_Host =  aHost;

	return true;
}

int CWLGenericConnection::EstablishConnection( RVCString& host, int port, bool usingProxy, int threadNum )
{
	if( isActive( threadNum ) )
		return KEPT_ALIVE;

	sockaddr_in addr;
	if( !m_Cp->GetSockAddr( addr, host, port,true, true ) )
		return RET_ERROR;

	int ret = RET_RETRY_FETCH;
	bool timedOut = false;
	for (int retryNum = 0; retryNum < m_Cp->m_Ws->m_RequestRetries +1 && ret == RET_RETRY_FETCH && !timedOut; retryNum++){
        if (m_Cm->isAbort())
          return IS_ABORT;

		ret= EstablishTcpConnection( addr, host, port, usingProxy, timedOut );
	}
	return ret;
}

int CWLGenericConnection::OpenConnection( RVCString& host, int port, bool usingProxy, int threadNum )
{
	//WlTraceMsg("try to open a connection...");
	if ( m_pTransObj )
	{
		m_pTransObj->AddSubTransaction("Open connection", TVPSysTime(), CWLTransactionData::TRANS_OPEN_CONNECTION);
		m_pTransObj->GetLastSubTransation()->SetFirstSubTrans();
		SetTransactionData(m_pTransObj->GetLastSubTransation());
	}

	int res = EstablishConnection( host, port, usingProxy, threadNum );

	if (res == RET_OK || res == KEPT_ALIVE)
		m_TcpConnected = true;
	else
		m_TcpConnected = false;

	if( m_pTransObj )
	{
		m_pTransObj->GetLastSubTransation()->SetEndTransaction(TVPSysTime(), WLLogMessage::Success);//open connection

		if (res == IS_ABORT || m_Cm->isAbort())
			ReportError(HTTP_ERROR_FILE, 106);
		else if( !m_TcpConnected)
				ReportError(HTTP_ERROR_FILE, 107);

		SetTransactionData(NULL);
	}

	return res;
}

int CWLGenericConnection::SelectForRead(int *errnum, int inputTimeout)
{
  *errnum = 0;
#ifdef __TV_WINDOWS__
  fd_set  readfds, exceptfds;
  struct  timeval timeout;
  timeout.tv_sec = 0;
  timeout.tv_usec = inputTimeout*1000;
  FD_ZERO(&readfds);
  FD_ZERO(&exceptfds);
  FD_SET(m_Socket, &readfds);
  FD_SET(m_Socket, &exceptfds);
  int ret = select(FD_SETSIZE, &readfds, NULL, &exceptfds, &timeout);
#else
  struct   pollfd fds = {m_Socket, POLLIN|POLLPRI, 0};
  int ret = poll(&fds, 1, inputTimeout);
#endif

#ifdef _DEBUG
	int e = errno;
#endif

  if (ret < 0)
    *errnum = errno;
  else if (ret > 0) {
#ifdef __TV_WINDOWS__
    if (FD_ISSET(m_Socket, &exceptfds))
#else
    if (fds.revents&(POLLERR|POLLHUP))
#endif
      *errnum = -1;
  }
  return ret;

}

int CWLGenericConnection::SelectForWrite(int *errnum, int inputTimeout)
{
  *errnum = 0;

#ifdef __TV_WINDOWS__
  fd_set  writefds, exceptfds;
  struct  timeval timeout;
  timeout.tv_sec = 0;
//  timeout.tv_usec = POLLTIMEOUT*1000;
  timeout.tv_usec = inputTimeout*1000;
  FD_ZERO(&writefds);
  FD_ZERO(&exceptfds);
  FD_SET(m_Socket, &writefds);
  FD_SET(m_Socket, &exceptfds);

	int ret = select(FD_SETSIZE, NULL, &writefds, &exceptfds, &timeout);
#else
  struct   pollfd fds = {m_Socket, POLLOUT, 0};
  int ret = poll(&fds, 1, inputTimeout);
#endif

  if (ret < 0)
    *errnum = errno;
  else if (ret > 0) {
#ifdef __TV_WINDOWS__
    if (FD_ISSET(m_Socket, &exceptfds))
#else
    if (fds.revents&(POLLERR|POLLHUP))
#endif
    {
      *errnum = ETIMEDOUT;
			int len = sizeof(errnum);
			if (getsockopt(m_Socket, SOL_SOCKET, SO_ERROR, (char*)errnum, &len)==SOCKET_ERROR)
			{
				//
				// return a special error num indicating getsockopt failed here.
				NULL;
			}
			//if (*errnum == 0)
			//	*errnum=EADDRINUSE;

			ret = -1;
    }
  }
  return ret;
}

bool CWLGenericConnection::isActive(int threadNum)
{
	if ( !m_TcpConnected )
		return false;

	int errNum;
	int res = SelectForWrite(&errNum, m_Ws->m_SelectWriteTimeout);
	if (res >= 1) {
		char test[4];
		int mode = SetSocketMode(NON_BLOCKING);
		int lastRes = 0;	// -1 (SOCKET_ERROR) if error or number of bytes received
		int len = m_Cp->Recv(m_Socket, test, 1, MSG_PEEK, lastRes);
		int err = lastRes == SOCKET_ERROR ? LastError() : 0;
		SetSocketMode(mode);
		if ((len == -1 && err == EWOULDBLOCK))
		return true;
	}

	CloseConnection(true, threadNum);
	return false;
}


void CWLGenericConnection::setSocket( int socket )
{ 
	m_Socket = socket;
	if(-1 != socket )
		m_TcpConnected = true;
	else
		m_TcpConnected = false;
}


void CWLGenericConnection::CloseConnection(bool forceClose, int threadNum)
{
	if ( !m_TcpConnected )
		return;

WlTraceMsg( "----------------------------------------------------- Closing socket %d\n", m_Socket);
WlTraceMsg( "Closing connection %d", m_Index);

	CloseConnection();

	CloseSocket();
	
	m_TcpConnected = false;
}

int CWLGenericConnection::LastError()
{
  return GetLastError();
}

int CWLGenericConnection::ReportError(const char* frmName, int frmId, ...)
{
  WLLogMessage::Severity sev = WLERRGetErrorSeverity(frmName, frmId);
	
	if (m_UrlType == IMAGE) {
	  WLLogMessage::Severity sevImg = WLERRGetErrorSeverity(HTTP_ERROR_FILE, 250);
		if (sevImg < sev)
			sev = sevImg;
	}
	
/*
	if (m_Ssl && !m_Ssl->m_ErrorString.isNull())
		sev = m_Ssl->m_ErrorSeverity;
*/
  char    buf[10240];
  va_list ap;

  va_start(ap, frmId);
  vsprintf(buf, WLERRGetErrorFormat(frmName,frmId), ap);
  va_end(ap);
/*
	if (m_Ssl && !m_Ssl->m_ErrorString.isNull()) {
		strcat(buf, " ");
		strcat(buf, m_Ssl->m_ErrorString.data());
	}
*/
	// add the Error code string
	if (!m_ErrorCode.isNull()) {
		strcat(buf, m_ErrorCode.data());
		m_ErrorCode.remove(0);
	}

	//  const char* sErrMsg = "HTTP ERROR";

	if (sev > WLLogMessage::Note)
  {
    if (!m_Ws->m_AutoNamedTrans)
    {
      m_Cm->reportMultiThread(sev, (const char*)buf);
    }
    else
    {
      if (m_pTransData)
      {
        m_pTransData->SetSeverity(sev);
        m_pTransData->SetFailureReason((const char*)buf);
        m_pTransData->AddEvent((const char*)buf, "", sev);
      }
      else
      {
        m_pTransObj->GetTransaction()->SetSeverity(sev);
        m_pTransObj->GetTransaction()->SetFailureReason((const char*)buf);
        m_pTransObj->AddEvent((const char*)buf, "", sev);
      }

      CWLPage* pPage =  m_Cm->GetPage();
      if (pPage)
      {
        CWLDocument* pFrame = pPage->GetPageTransaction();
        if (pFrame)
          pFrame->SetSeverity(sev);
      }
    }
  }
/*
	if (m_Ssl) {
		m_Ssl->m_ErrorSeverity = WLLogMessage::Success;
		m_Ssl->m_ErrorString.remove(0);
	}
*/
	// update last error
	m_Cm->SetLastError(sev, (const char*)buf);

//  return ( (sev >= WLLogMessage::Error) ? RET_ERROR : RET_OK);
  return RET_OK;
}

void CWLGenericConnection::SetErrorCode(int err)
{
	if (!m_ErrorCode.isNull())
		m_ErrorCode.remove(0);

	m_ErrorCode = " Error code: ";
	char code[16];
	sprintf(code, "%d", err);
	m_ErrorCode += code;
}

// set to blocking/nonblocking mode
int CWLGenericConnection::SetSocketMode(int mode)
{
  unsigned long val = mode;

#ifdef __TV_WINDOWS__
  int ret = ioctlsocket( m_Socket, FIONBIO, &val );
  //int ret = WSAAsyncSelect(m_Socket,0,0,FD_CONNECT|FD_CLOSE);
#else
  int ret = ioctl( m_Socket, FIONBIO, &val );
#endif

	val = m_BlockMode;	// previous mode

	if (ret == 0)
		m_BlockMode = mode;	// update current mode
  else
    WlTraceMsg( "Could not make connection non-blocking, error: %d", LastError() );

	return (int)val;
}

bool CWLGenericConnection::IsMatching(sockaddr_in& address, const char* host, int port)
{
  //We use the same connection only for the same IP:port and the same host name
  //This is how the browser is acting.
  if (strcmpi(host, m_Host.data()))
		return false;


  if (m_SockAddress.sin_port != address.sin_port) {
    return false;
  }
#ifdef __TV_WINDOWS__
  if (  m_SockAddress.sin_addr.s_addr == address.sin_addr.s_addr )
    return true;
#else
  if ( inet_netof(m_SockAddress.sin_addr) == inet_netof(address.sin_addr) ) {
    return true;
  }
#endif //__TV_WINDOWS__
  return false;

}

int CWLGenericConnection::SetCryptographicStrength( int strength )
{

	return RET_OK;
}

int CWLGenericConnection::EstablishTcpConnection(sockaddr_in& address, RVCString& host, int port, bool usingProxy, bool& timedOut )
{
  WlTraceMsg( "----------------------------------------------------- Trying to open socket %d\n", m_Socket);
	int err = RET_ERROR;
	int retrycounter = 0;
	timedOut = false;

	m_Socket = socket(AF_INET, SOCK_STREAM, 0);
	if (m_Socket == -1)  {
		SetErrorCode(LastError());
		ReportError(HTTP_ERROR_FILE, 10);
		return RET_ERROR;
	}

  // Make the socket non-blocking, so the connect can be canceled.
	SetSocketMode(NON_BLOCKING);

  #ifndef __TV_aix__
	  if (m_Cm->IsLoadServer() && m_Ws->m_MultiIP) {
		  sockaddr_in   bind_addr;
		bind_addr.sin_family = AF_INET;
		if (CWLConnectionsPool::m_gLocalAddr.GetNum() <= 0)
			bind_addr.sin_addr.s_addr = INADDR_ANY;
		else {
			// selecting IP address to Thread.
			// Chaim - MultiIP
			int i;
			if (m_Ws->m_MultiIPType == CWLWorkspace::PerClient){
				i = m_Cm->GetLGThreadNum() % CWLConnectionsPool::m_gLocalAddr.GetNum();
			}
			else {
				i = CWLConnectionsPool::GetNextMultiIPAddressIndex();
			}
//			int i = ((m_Cm->GetCurrentScriptRoundNum() + 1) * (m_Cm->GetLGThreadNum() + 1)) % CWLConnectionsPool::m_gLocalAddr.GetNum();
//			int i = m_Cm->GetLGThreadNum() % CWLConnectionsPool::m_gLocalAddr.GetNum();
			bind_addr.sin_addr.s_addr = CWLConnectionsPool::m_gLocalAddr.GetAt(i)->s_addr;
		}
		bind_addr.sin_port = 0;

      #ifdef _DEBUG
		  WlTraceMsg( "----   binding to ip : %s   --------- \n", ip(&bind_addr));
	  #endif
		  err = bind(m_Socket, (sockaddr *)&bind_addr, sizeof(sockaddr_in));
		  if (err != 0) {
			  ReportError(HTTP_ERROR_FILE, 12, LastError());
			  return RET_ERROR;
		  }
	  }
  #endif

  //
  // Setting the No-Delay option for data written to the  socket
  // in order to Improve (!?) performence.
  BOOL opt = TRUE;
  err = setsockopt(m_Socket, IPPROTO_TCP, TCP_NODELAY, (char*)&opt, sizeof(opt));
  if (err != 0) {
    ReportError(HTTP_ERROR_FILE, 14, LastError()/*h_errno*/);
    return RET_ERROR;
  }
#ifdef __TV_WINDOWS__
	struct linger lingopt;
	lingopt.l_onoff = 1;
	lingopt.l_linger = 0;
  err = setsockopt(m_Socket, SOL_SOCKET, SO_LINGER, (char*)&lingopt, sizeof(lingopt));

//  opt=TRUE;
//  err = setsockopt(m_Socket, SOL_SOCKET, SO_DONTLINGER, (char*)&opt, sizeof(opt));
#else
	struct linger lingopt;
	lingopt.l_onoff = 0;
	//memset(&lingopt,0,sizeof(struct linger)); // PROTOCOL!!!
  err = setsockopt(m_Socket, SOL_SOCKET, SO_LINGER, (char*)&lingopt, sizeof(lingopt));
#endif
  if (err != 0) {
    ReportError(HTTP_ERROR_FILE, 14, LastError()/*h_errno*/);
    return RET_ERROR;
  }

  // Try to connect without blocking, usualy it will returned with -1
  // status, because the connection can't completed immediatly.

  WlTraceMsg("Connecting to Host=%s, Port=%d", check_null(host.data()), port);
	err = connect(m_Socket, (sockaddr *)&m_SockAddress, sizeof(sockaddr_in));
	int save_err = err;
	
	// Hryvach: extending error handling
	if( 0 != err ){
		int wsaLastError = WSAGetLastError();
		
		timeval waitTime;
//		waitTime.tv_sec = m_Ws->m_SelectWriteTimeout;
		waitTime.tv_sec = 1;
		waitTime.tv_usec = 0;
		fd_set sock_set;

		int i;
		
		switch( wsaLastError ) {
		case WSAETIMEDOUT:
			CloseSocket();
//			printf( "\nERROR Connection timed out is handled.\n" );
			timedOut = true;
			return RET_RETRY_FETCH;

		case WSAEWOULDBLOCK:


			FD_ZERO( & sock_set );
			FD_SET( m_Socket, &sock_set);
			if (m_Cm->isAbort())
				return IS_ABORT;
			err = select( 0, NULL, &sock_set, NULL, &waitTime );
			FD_ZERO( & sock_set );

			if( 1 > err ) {
				CloseSocket();
				return RET_RETRY_FETCH;
			}
		}
	}

  if (err < 0) {
		int errCode = LastError();
	  WlTraceMsg("Connect had returned with the error=%d",errCode);

    if (errCode == EINPROGRESS  ||
        errCode == EWOULDBLOCK)  {
      int SelectForWriteRetValue = 0;
		  int SelectForWriteRetries  = 1;
      while (SelectForWriteRetValue <= 0) {

        do {

        if (m_Cm->isAbort())
          return IS_ABORT;

          if (SelectForWriteRetries>1) {
            if (SelectForWriteRetries > m_Ws->m_SelectWriteSwitchNum ) {
              m_Cm->sleep(m_Ws->m_SelectWriteTimeout);
           	  WlTraceMsg("Select for write had waited for timeout=%d", m_Ws->m_SelectWriteTimeout);
            }
            else {
              m_Cm->sleep(m_Ws->m_SelectWriteSecondTimeout);
           	  WlTraceMsg("Select for write had waited for second timeout=%d", m_Ws->m_SelectWriteSecondTimeout);
            }
          }

          if (SelectForWriteRetries <= m_Ws->m_SelectWriteSwitchNum) {
         	  WlTraceMsg("Activating Select for write for the %d time with time interval=%d",SelectForWriteRetries,m_Ws->m_SelectWriteTimeout);
            SelectForWriteRetValue = SelectForWrite(&m_httpStatus,m_Ws->m_SelectWriteTimeout);
          }
          else {
         	  WlTraceMsg("Activating Select for write for the %d time with time interval=%d",SelectForWriteRetries,m_Ws->m_SelectWriteSecondTimeout);
            SelectForWriteRetValue = SelectForWrite(&m_httpStatus,m_Ws->m_SelectWriteSecondTimeout);
          }

       	  WlTraceMsg("Select for write had returned with the following return code=%d and status=%d",SelectForWriteRetValue,m_httpStatus);

          SelectForWriteRetries++;

        } while (SelectForWriteRetValue<=0 && m_httpStatus==0 && SelectForWriteRetries*m_Ws->m_SelectWriteTimeout > m_Ws->m_ConnectTimeout);

        WlTraceMsg("Select for write loop had terminated with ret=%d, m_httpStatus=%d, SelectForWriteRetries=%d, m_Ws->m_SelectWriteTimeout=%d, m_Ws->m_ConnectTimeout=%d", SelectForWriteRetValue, m_httpStatus, SelectForWriteRetries, m_Ws->m_SelectWriteTimeout, m_Ws->m_ConnectTimeout);

        if ((SelectForWriteRetValue < 0)&&(m_httpStatus != EALREADY)&&(m_httpStatus != EWOULDBLOCK)) {
          err = SelectForWriteRetValue;
          break;
        }
        else if (SelectForWriteRetValue > 0) {
#ifdef __TV_UNIX__
          // dobule check, if the connection completed.
          // if connect failed and the errno is EISCONN,
          // it's mean that the socket is already connected.
          err = connect(m_Socket, (sockaddr *)&m_SockAddress, sizeof(sockaddr_in));
          if ((err < 0)&&(LastError() == EISCONN))
            err = 0;
          else if (err < 0)
            m_httpStatus = LastError();
          break;
#else
          err = 0;
          break;
#endif  // __TV_UNIX__
        }
#ifdef __TV_UNIX__
        else {
          // if the select system call returned with 0, it's mean that
          // the connection not completed yet,
          // check the connection again.
          err = connect(m_Socket, (sockaddr *)&m_SockAddress, sizeof(sockaddr_in));
          if ((err < 0) && (LastError() != EALREADY)&&
              (LastError() != EISCONN) &&
              (LastError() != EWOULDBLOCK)) {
            m_httpStatus = LastError();
            break;
          }
        }
#endif // __TV_UNIX__
      }
    }
    else  {// on if LastError() == EINPROGRESS.
      m_httpStatus = LastError();
    }
  }

  // Make the socket blocking again.
  if (err >= 0) {
		SetSocketMode(BLOCKING);
  }
  else {
	  CloseSocket();
    int        errsev;
    RVCString  errmsg;
    if (usingProxy)
      errmsg = "proxy";
    else
      errmsg = "server";

    switch (m_httpStatus)  {
			case EADDRINUSE:
      case 0:
        if (m_httpStatus==EADDRINUSE) {
          WlTraceMsg("Retrying (%d) open connection... (m_httpStatus=%d) Addr in use retry", retrycounter+1, m_httpStatus);
          }
        else {
          WlTraceMsg("Retrying (%d) open connection... (m_httpStatus=%d)", retrycounter+1, m_httpStatus);
          }
        if (retrycounter++<20) {
          m_Cm->sleep(50);
          return RET_NONE;// replased "goto start;";
        }
				errsev = 105;
        break;

      case ECONNREFUSED:
				errsev = 100;
				break;

			case EINVAL:
				errsev = 101;
				break;

			case ETIMEDOUT:
				errsev = 102;
				break;

			case ENOBUFS:
				errsev = 103;
				break;

			default:
        WlTraceMsg("Error in open connection (104) , (Errmsg=%s), (Host=%d), (Port=%d), (m_httpStatus=%d)", check_null(errmsg.data()), check_null(host.data()), port, m_httpStatus);
				ReportError(HTTP_ERROR_FILE, 104, check_null(errmsg.data()), check_null(host.data()), port, m_httpStatus );
				return RET_ERROR;
    }
    WlTraceMsg("Error in open connection (%d) , (Errmsg=%s), (Host=%d), (Port=%d), (m_httpStatus=%d)", errsev, check_null(errmsg.data()), check_null(host.data()), port, m_httpStatus);
    ReportError( HTTP_ERROR_FILE, errsev, check_null(errmsg.data()), check_null(host.data()), port );
		return RET_ERROR;
  }

  WlTraceMsg( "----------------------------------------------------- Opening socket %d\n", m_Socket);
  return RET_OK;
}

char* CWLGenericConnection::ip()
{
	static char ip[64];
	strcpy(ip, inet_ntoa(m_SockAddress.sin_addr));
	return ip;
}

char* CWLGenericConnection::ip(sockaddr_in* addr)
{
	static char ip[64];
	strcpy(ip, inet_ntoa(addr->sin_addr));
	return ip;
}

void CWLGenericConnection::CloseSocket()
{
	if( -1 != m_Socket ){
#ifdef __TV_WINDOWS__
		closesocket(m_Socket);
#else
		//(void)shutdown(m_Socket,SHUT_WR);	// PROTOCOL!!!
		close(m_Socket);
#endif
		m_Socket = -1;
	}
}


bool CWLGenericConnection::SetCiphers( int strength/* = SSL_AllCrypto*/ )
{
	return true;
}


unsigned CWLGenericConnection::GetHostByName(const char *host, char* *addresses, short* length, int* err)
{
	*err = 0;
	*length = 4;

#ifdef __TV_WINDOWS__
  struct hostent      *result;
  unsigned long inetAddr = inet_addr(host);

  if (inetAddr != INADDR_NONE) {
		*addresses = (char*)malloc(16);
    memcpy(*addresses, (void*)&inetAddr, sizeof(inetAddr));
    return 1;
  }

  if ((result = gethostbyname(host)) != NULL)
  {
		*length = result->h_length;
		for (unsigned num = 0; (result->h_addr_list)[num]; num++);
		*addresses = (char*)malloc(num * (*length));
		for (unsigned i = 0; i < num; i++)
			memmove(*addresses + (i * (*length)), (void*)(result->h_addr_list)[i], (*length));
		return num;
  }

#elif (defined __TV_solaris__)
  struct hostent   result;
  int    thd_errno;
  char   buffer[HOSTENT_MAX];

  unsigned long inetAddr = inet_addr(host);
  if (inetAddr != INADDR_NONE)  {
		*addresses = (char*)malloc(16);
    memcpy(*addresses, (void*)&inetAddr, sizeof(inetAddr));
		return 1;
  }

  if (gethostbyname_r(host, &result, buffer, HOSTENT_MAX, &thd_errno) != NULL) {
		// for UNIX systems without DNS caching - the above function return strange values
		// (except for the first ip address), so in any case (caching or not), I'm returning
		// only the first ip address (caching should be system's).
		unsigned total = strlen(result.h_addr);
		*addresses = (char*)malloc(total);
		*length = result.h_length;
		memmove(*addresses, result.h_addr, *length);
		return 1;
	}

#elif (defined __TV_aix__)
  struct hostent   result;
  struct hostent_data hd;

  unsigned long inetAddr = inet_addr(host);
  if (inetAddr != INADDR_NONE)  {
		*addresses = (char*)malloc(16);
    memcpy(*addresses, (void*)&inetAddr, sizeof(inetAddr));
		return 1;
  }

  if (gethostbyname_r(host, &result, &hd) == 0) {
		// for UNIX systems without DNS caching - the above function return strange values
		// (except for the first ip address), so in any case (caching or not), I'm returning
		// only the first ip address (caching should be system's).
		unsigned total = strlen(result.h_addr);
		*addresses = (char*)malloc(total);
		*length = result.h_length;
		memmove(*addresses, result.h_addr, *length);
		return 1;
	}


#elif (defined __TV_linux__)
  struct hostent   result;
  struct hostent* presult = &result;
  char buffer[1024];
  int lerrno = 0;
  int rval = 0;

  unsigned long inetAddr = inet_addr(host);
  if (inetAddr != INADDR_NONE) {
		*addresses = (char*)malloc(16);
    memcpy(*addresses, (void*)&inetAddr, sizeof(inetAddr));
    return 1;
  }

  rval = gethostbyname_r(host, &result, buffer, sizeof(buffer), &presult, &lerrno);

  if (rval == 0) {
    // for UNIX systems without DNS caching - the above function return strange values
    // (except for the first ip address), so in any case (caching or not), I'm returning
    // only the first ip address (caching should be system's).
    WlTraceMsg("gethostbyname: host[%s], presult->h_name[%s]",  host, result.h_name);
    unsigned total = strlen(result.h_addr);
    *length = result.h_length;
    unsigned num = 0;
    while ((result.h_addr_list)[num]) { num++; }
    *addresses = (char*)malloc(num * (*length));
    for (unsigned i = 0; i < num; i++) {
			memmove(*addresses + (i * (*length)), (void*)(result.h_addr_list)[i], (*length));
    }
    return 1;
  }
#endif

	*err = LastError();

  return 0;
}



//////////////////////////////////////////////////////////////////////////
//						Static data member
//////////////////////////////////////////////////////////////////////////

RVOrdered CWLGenericConnection::m_Session;

