/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLInsecureConnection.cpp: implementation of the CWLInsecureConnection class.
//
//////////////////////////////////////////////////////////////////////

//#include <Winsock2.h>

#include "WLInsecureConnection.h"
#include "commandmanager.h"
#include "workspace.h"
#include "connpool.h"
#include "httprequest.h"
#include "httpdefines.h"

#define max( a, b ) ( ( (a) > (b) ) ? (a) : (b) )
#define min( a, b ) ( ( (a) < (b) ) ? (a) : (b) )


//////////////////////////////////////////////////////////////////////////
// Macros Definitions
//////////////////////////////////////////////////////////////////////////
#define addTimer       if (!m_Cm->isAbort()) m_Cm->addTimer
#define WAIT_TIME            "Time To First Byte"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWLInsecureConnection::CWLInsecureConnection( CWLHttpRequest* pHttpReq )
: CWLGenericConnection( pHttpReq )
{
	m_Protocol = HTTP_PROTOCOL;

	//////////////////////////////////////////////////////////////////////////
#ifdef TRANSPORT_LAYER_TRACE
	char fname[256] = {'\0'};
	sprintf( fname, "v:\\Log\\http_resp_%d.htm", m_FileNum );
	m_FileNum++;
	m_File = fopen( fname, "w" );
#endif // TRANSPORT_LAYER_TRACE
	//////////////////////////////////////////////////////////////////////////	
}

CWLInsecureConnection::CWLInsecureConnection( CWLGenericConnection* pConnection )
: CWLGenericConnection( *pConnection )
{
	m_Protocol = HTTP_PROTOCOL;

	//////////////////////////////////////////////////////////////////////////
#ifdef TRANSPORT_LAYER_TRACE
	char fname[256] = {'\0'};
	sprintf( fname, "V:\\Log\\http_resp_%d.htm", m_FileNum );
	m_FileNum++;
	m_File = fopen( fname, "w" );
#endif // TRANSPORT_LAYER_TRACE
	//////////////////////////////////////////////////////////////////////////	
}

CWLInsecureConnection::~CWLInsecureConnection()
{
  CWLGenericConnection::CloseConnection(true);
	//////////////////////////////////////////////////////////////////////////
#ifdef TRANSPORT_LAYER_TRACE
	if( m_File )
		fclose( m_File );
#endif // TRANSPORT_LAYER_TRACE
	//////////////////////////////////////////////////////////////////////////
}

bool CWLInsecureConnection::Init( CWLCommandManager* cm, CWLConnectionsPool* cp, int socketDescriptor, bool createNewSession/*= false*/, const char* certFileName/*=NULL*/ )
{
	m_Cm = cm;
	m_Ws = m_Cm->GetWorkspace();
	m_Cp = cp;
	m_Socket = socketDescriptor;
	return true;
}

int CWLInsecureConnection::Connect()
{
	return RET_OK;
}

int CWLInsecureConnection::Send( const char* buffer, int length, int threadNum )
{
	int sentBytes = 0;
	int done = 0;
	int lastRes = 0;
	int nSendBytesCnt = 0;
	int nMaxSendSize = max( m_Ws->m_SendBufferSize, MIN_BUFFER_SIZE );
	bool bUseFixedBufSize = m_Ws->m_SendBufferSize != DEFAULT_SEND_BUFFER_SIZE;

if( !m_TcpConnected )
		return RET_ERROR;

	do {
		if (m_Cm->isAbort())
			return IS_ABORT;

		if( bUseFixedBufSize )
			nSendBytesCnt = min( nMaxSendSize, (length-done) );
		else
			nSendBytesCnt = length-done;

		sentBytes = m_Cp->Send(m_Socket, (const char*)(buffer+done), nSendBytesCnt, 0, lastRes);

		if (sentBytes > 0 && length > 0)
			done += sentBytes;
	}while( sentBytes > 0 && done < length );

	if ( done != length){
		m_httpStatus = lastRes == SOCKET_ERROR ? LastError() : 0;
		ReportError( HTTP_ERROR_FILE, 20 );
		return RET_ERROR;
	}

	return RET_OK;
}

int CWLInsecureConnection::Recv( char* buffer, int* buffsize, int flags, int* err_code, bool* closed, int threadNum )
{
	int length = 0, done = 0;
	*closed = false;
	int lastRes = 0;	// -1 (SOCKET_ERROR) if error or number of bytes received
	int wsaLastError = 0;
	*err_code = 0;

	if( !m_TcpConnected )
		return RET_ERROR;

	do{
		lastRes = 0;
		length = m_Cp->Recv( m_Socket, buffer + done, *buffsize - done, flags, lastRes );

		//////////////////////////////////////////////////////////////////////////
#ifdef TRANSPORT_LAYER_TRACE
		if( m_File && length > 0 ){
			fwrite( buffer, sizeof(char), length, m_File );
		}
#endif // TRANSPORT_LAYER_TRACE
		//////////////////////////////////////////////////////////////////////////

		if ( m_pHttpRequest->IsFirstTimeWait() )
		{
			if (!m_Cm->isAbort())
				addTimer(WAIT_TIME, TVPSysTime() - m_pHttpRequest->m_dStartWait);

			if (m_pTransObj)
			{
				m_pHttpRequest->m_dStartReceive = TVPSysTime();
				
				if (m_Ws->m_AutoNamedTrans)
				{
					m_pTransObj->GetLastSubTransation()->SetEndTransaction(m_pHttpRequest->m_dStartReceive, WLLogMessage::Success);//Time To First Byte
				}
				if (length <= 0)
				{
					SetTransactionData(m_pTransObj->GetLastSubTransation());
					ReportError( HTTP_ERROR_FILE, 108, m_pTransObj->GetTransaction()->GetName() );
					SetTransactionData(NULL);
				}
				if (m_Ws->m_AutoNamedTrans)
				{
					m_pTransObj->AddSubTransaction("Receive", m_pHttpRequest->m_dStartReceive, CWLTransactionData::TRANS_RECEIVE);
				}
			}
			m_pHttpRequest->SetFirstTimeWait(false) ;
		}
		if (length > 0)
			done += length;
	}while ( lastRes > 0 && length > 0 && done < *buffsize );

	// set *err_code
	if( err_code ) // != NULL
			if (lastRes == SOCKET_ERROR){ // if( socket error )
				*err_code = LastError(); // errno
			}
			else
				*err_code = 0; // no error
	
	// look if there is end of data
	if( 0 < done ){ // data received successfully
		*buffsize = done;

		if( lastRes == 0 ) // end of data
			*closed = true;

		return RET_OK;
	}

	if( 0 == done ){ // connection closed
		*buffsize = 0;
		*closed = true;
		return RET_OK;
	}

	return RET_ERROR;
}

void CWLInsecureConnection::CloseConnection()
{
	if ( !m_TcpConnected )
		return;

	WlTraceMsg( "----------------------------------------------------- Closing socket %d\n", m_Socket);
	WlTraceMsg( "Closing connection %d", m_Index);

	m_TcpConnected = false;
}

void CWLInsecureConnection::CleanUp()
{
}


//////////////////////////////////////////////////////////////////////////
// Static data members
//////////////////////////////////////////////////////////////////////////
#ifdef TRANSPORT_LAYER_TRACE
int CWLInsecureConnection::m_FileNum = 0;
#endif // TRANSPORT_LAYER_TRACE

