/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvjsapi.h"

#include "associativearray.h"
#include "loadengineglobals.h"
#include "commandmanager.h"
//#include "loadexceptions.h"
#include "wlerrorlib.h"
#include "loadreportinfo.h"
#include "jsreplynames.h"
#include "wlfuncs.h"

#define WLJSNAME_FUNCTIONALTEST_REPORTER_NAME                "Name"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_DESCRIPTION         "Description"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_HTML1               "Html1"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_HTML2               "Html2"

#define WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION1           "Location1"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION2           "Location2"

#define WLJSNAME_FUNCTIONALTEST_REPORTER_NUMLINES            "NumLines"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_EXPECTED_DATA       "ExpectedData"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_ACTUAL_DATA         "ActualData"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_VERSION             "Version"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_REPOSITORY_NAME     "RepositoryName"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_EXPECTED_ITEM_ID    "ExpectedItemId"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_FRAME_NAME          "FrameName"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_ ""
#define WLJSNAME_FUNCTIONALTEST_REPORTER_ADD_MESSAGE         "AddMessage"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_ADD_ERROR           "AddError"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_ISDIRTY             "IsDirty"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_CLEAR               "Clear"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_REPORT              "Report"
#define WLJSNAME_FUNCTIONALTEST_REPORTER_TO_STRING           "ToString"


#define WLJSNAME_FUNCTIONALTEST_OBJDETAILS_CLEAR			       "Clear"
#define WLJSNAME_FUNCTIONALTEST_OBJDETAILS_SET				       "Set"
#define WLJSNAME_FUNCTIONALTEST_OBJDETAILS_OBJ_TYPE          "ObjectType"
#define WLJSNAME_FUNCTIONALTEST_OBJDETAILS_TEST_TYPE         "TestType"
#define WLJSNAME_FUNCTIONALTEST_OBJDETAILS_INDEX1            "Index1"
#define WLJSNAME_FUNCTIONALTEST_OBJDETAILS_INDEX2            "Index2"
#define WLJSNAME_FUNCTIONALTEST_OBJDETAILS_X1                "X1"
#define WLJSNAME_FUNCTIONALTEST_OBJDETAILS_X2                "X2"
#define WLJSNAME_FUNCTIONALTEST_OBJDETAILS_Y1                "Y1"
#define WLJSNAME_FUNCTIONALTEST_OBJDETAILS_Y2                "Y2"
#define WLJSNAME_FUNCTIONALTEST_OBJDETAILS_URL               "Url"


static JSBool
FunctionalTestReporterCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

	if ((argc != 1) && (argc != 2)) {
    WLErrorException e(JS_DOM_ERROR_FILE, 121, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS, 1, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(JS_DOM_ERROR_FILE, 122, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if ((argc == 2))
    if (!JSVAL_IS_STRING(argv[1])) {
    WLErrorException e(JS_DOM_ERROR_FILE, 122, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

  CAssociativeArray *array = new CAssociativeArray;
  array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_NUMLINES, 0L);
	array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_VERSION,1L);


  JS_SetPrivate(cx, obj, array);

  JS_DefineProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_NAME,argv[0],NULL,NULL,JSPROP_ENUMERATE);
  if (argc == 2)
    JS_DefineProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_DESCRIPTION,argv[1],NULL,NULL,JSPROP_ENUMERATE);
  else
    JS_DefineProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_DESCRIPTION,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);

  JS_DefineProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_HTML1,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);
  JS_DefineProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_HTML2,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);
	JS_DefineProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION1,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);
	JS_DefineProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION2,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);
  JS_DefineProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_REPOSITORY_NAME,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);
  JS_DefineProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_EXPECTED_ITEM_ID,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);
  JS_DefineProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_FRAME_NAME,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);



  *rval = OBJECT_TO_JSVAL(obj);
	return JS_TRUE;
}

static JSBool
FunctionalTestReporterAddMessage(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	long numLines;

	if (argc != 1) {
    WLErrorException e(JS_DOM_ERROR_FILE, 120, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS"."WLJSNAME_FUNCTIONALTEST_REPORTER_ADD_MESSAGE, 3);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
	if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(JS_DOM_ERROR_FILE, 122, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS"."WLJSNAME_FUNCTIONALTEST_REPORTER_ADD_MESSAGE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);
	numLines = array->GetLongValueOfParameter(WLJSNAME_FUNCTIONALTEST_REPORTER_NUMLINES);
	numLines++;
	array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_NUMLINES, numLines);
  array->AddArrayCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_EXPECTED_DATA, "", numLines);
	array->AddArrayCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_ACTUAL_DATA, "", numLines);
	array->AddArrayCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_DESCRIPTION, JS_GetStringBytes(JSVAL_TO_STRING(argv[0])), numLines);

	return JS_TRUE;
}

static JSBool
FunctionalTestReporterAddError(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	long numLines;
	int i;

	if (argc != 3) {
    WLErrorException e(JS_DOM_ERROR_FILE, 120, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS"."WLJSNAME_FUNCTIONALTEST_REPORTER_ADD_ERROR, 3);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	for(i=1;i<3;i++){
		if(!JSVAL_IS_OBJECT(argv[i])){
			WLErrorException e(JS_DOM_ERROR_FILE, 124, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS"."WLJSNAME_FUNCTIONALTEST_REPORTER_ADD_ERROR, i+1);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
		}
		if (!WLIsObjectOfClass(cx, argv[i],WLJSNAME_FUNCTIONALTEST_OBJDETAILS_CLASS)) {
			WLErrorException e(JS_DOM_ERROR_FILE, 125, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS"."WLJSNAME_FUNCTIONALTEST_REPORTER_ADD_ERROR, i+1,WLJSNAME_FUNCTIONALTEST_OBJDETAILS_CLASS);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
		}
	}

	if (!JSVAL_IS_STRING(argv[0])) {
		WLErrorException e(JS_DOM_ERROR_FILE, 122, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS"."WLJSNAME_FUNCTIONALTEST_REPORTER_ADD_ERROR, i+1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);
	CAssociativeArray* expArray = (CAssociativeArray*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[1]));
	CAssociativeArray* actArray = (CAssociativeArray*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[2]));

	numLines = array->GetLongValueOfParameter(WLJSNAME_FUNCTIONALTEST_REPORTER_NUMLINES);
	numLines++;
	array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_NUMLINES, numLines);
	array->AddArrayCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_EXPECTED_DATA, expArray->ToString(), numLines);
	array->AddArrayCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_ACTUAL_DATA, actArray->ToString(), numLines);
	array->AddArrayCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_DESCRIPTION, JS_GetStringBytes(JSVAL_TO_STRING(argv[0])), numLines);

	return JS_TRUE;
}

static JSBool
FunctionalTestReporterClear(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	jsval v=JSVAL_VOID;

	if (argc != 0) {
    WLErrorException e(JS_DOM_ERROR_FILE, 120, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS"."WLJSNAME_FUNCTIONALTEST_REPORTER_CLEAR, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);

  array->clearAndDestroy();
	array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_NUMLINES, 0L);
	array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_VERSION,1L);

	JS_SetProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_HTML1,&v);
	JS_SetProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_HTML2,&v);
	JS_SetProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION1,&v);
	JS_SetProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION2,&v);
	JS_SetProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_REPOSITORY_NAME,&v);
	JS_SetProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_EXPECTED_ITEM_ID,&v);
	JS_SetProperty(cx,obj,WLJSNAME_FUNCTIONALTEST_REPORTER_FRAME_NAME,&v);

	return JS_TRUE;
}

static JSBool
FunctionalTestReporterReport(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	jsval nameProperty, descriptionProperty, html1Property, html2Property, repositoryNameProperty, expectedItemIdProperty, frameNameProperty,location1Property,location2Property;
  const char *name="Undefined", *description="";

	if (argc != 0) {
    WLErrorException e(JS_DOM_ERROR_FILE, 120, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS"."WLJSNAME_FUNCTIONALTEST_REPORTER_REPORT, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);

  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_NAME, &nameProperty)==JS_TRUE)
    if (JSVAL_IS_STRING(nameProperty))
			name = JS_GetStringBytes(JSVAL_TO_STRING(nameProperty));

  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_DESCRIPTION, &descriptionProperty)==JS_TRUE)
    if (JSVAL_IS_STRING(descriptionProperty))
      description = JS_GetStringBytes(JSVAL_TO_STRING(descriptionProperty));

  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_HTML1, &html1Property)==JS_TRUE)
    if (JSVAL_IS_STRING(html1Property))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_HTML1, JS_GetStringBytes(JSVAL_TO_STRING(html1Property)));

  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_HTML2, &html2Property)==JS_TRUE)
    if (JSVAL_IS_STRING(html2Property))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_HTML2, JS_GetStringBytes(JSVAL_TO_STRING(html2Property)));

	if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION1, &location1Property)==JS_TRUE)
    if (JSVAL_IS_STRING(location1Property))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION1, JS_GetStringBytes(JSVAL_TO_STRING(location1Property)));

	if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION2, &location2Property)==JS_TRUE)
    if (JSVAL_IS_STRING(location2Property))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION2, JS_GetStringBytes(JSVAL_TO_STRING(location2Property)));


  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_REPOSITORY_NAME, &repositoryNameProperty)==JS_TRUE)
    if (JSVAL_IS_STRING(repositoryNameProperty))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_REPOSITORY_NAME, JS_GetStringBytes(JSVAL_TO_STRING(repositoryNameProperty)));

  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_EXPECTED_ITEM_ID, &expectedItemIdProperty)==JS_TRUE)
    if (JSVAL_IS_INT(expectedItemIdProperty))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_EXPECTED_ITEM_ID, (long)JSVAL_TO_INT(expectedItemIdProperty));

  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_FRAME_NAME, &frameNameProperty)==JS_TRUE)
    if (JSVAL_IS_STRING(frameNameProperty))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_FRAME_NAME, JS_GetStringBytes(JSVAL_TO_STRING(frameNameProperty)));


  const char* data = array->ToString();

	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
  if (ptrHttp) {
    ptrHttp->VerificationRecordEvent(name,
                                description,
																TVPSysTime(),
                                WLLogMessage::Note,
                                "FunctionalTest",
                                (void*)data,
                                strlen(data));
  }

  FunctionalTestReporterClear(cx, obj, 0, NULL, NULL);

  return JS_TRUE;
}


static JSBool
FunctionalTestReporterToString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	jsval html1Property, html2Property, repositoryNameProperty, expectedItemIdProperty, frameNameProperty,location1Property,location2Property;

	if (argc != 0) {
    WLErrorException e(JS_DOM_ERROR_FILE, 120, WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS"."WLJSNAME_FUNCTIONALTEST_REPORTER_TO_STRING, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);


  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_HTML1, &html1Property)==JS_TRUE)
    if (JSVAL_IS_STRING(html1Property))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_HTML1, JS_GetStringBytes(JSVAL_TO_STRING(html1Property)));

  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_HTML2, &html2Property)==JS_TRUE)
    if (JSVAL_IS_STRING(html2Property))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_HTML2, JS_GetStringBytes(JSVAL_TO_STRING(html2Property)));

	if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION1, &location1Property)==JS_TRUE)
    if (JSVAL_IS_STRING(location1Property))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION1, JS_GetStringBytes(JSVAL_TO_STRING(location1Property)));

	if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION2, &location2Property)==JS_TRUE)
    if (JSVAL_IS_STRING(location2Property))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_LOCATION2, JS_GetStringBytes(JSVAL_TO_STRING(location2Property)));

  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_REPOSITORY_NAME, &repositoryNameProperty)==JS_TRUE)
    if (JSVAL_IS_STRING(repositoryNameProperty))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_REPOSITORY_NAME, JS_GetStringBytes(JSVAL_TO_STRING(repositoryNameProperty)));

  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_EXPECTED_ITEM_ID, &expectedItemIdProperty)==JS_TRUE)
    if (JSVAL_IS_INT(expectedItemIdProperty))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_EXPECTED_ITEM_ID, (long)JSVAL_TO_INT(expectedItemIdProperty));

  if (JS_GetProperty(cx, obj, WLJSNAME_FUNCTIONALTEST_REPORTER_FRAME_NAME, &frameNameProperty)==JS_TRUE)
    if (JSVAL_IS_STRING(frameNameProperty))
      array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_FRAME_NAME, JS_GetStringBytes(JSVAL_TO_STRING(frameNameProperty)));


  const char* data = array->ToString();

	JSString* str = JS_NewStringCopyZ(cx,data);
	if (str)
		*rval = STRING_TO_JSVAL(str);
	return JS_TRUE;
}


static void
FunctionalTestReporterFinalize(JSContext *cx, JSObject *obj)
{
	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);
  if (array)
  {
    array->clearAndDestroy();
	  array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_NUMLINES, 0L);
	  array->AddCouple(WLJSNAME_FUNCTIONALTEST_REPORTER_VERSION,1L);
    delete array;
  }
}

JSClass FunctionalTestReporterClass = {
  WLJSNAME_FUNCTIONALTEST_REPORTER_CLASS,
  JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,	// add Property
  JS_PropertyStub,	// del Property
  JS_PropertyStub,	// get Property
  JS_PropertyStub,	// set Property
  JS_EnumerateStub,
  JS_ResolveStub,
  JS_ConvertStub,
  FunctionalTestReporterFinalize
};

static JSFunctionSpec FunctionalTestReporter_methods[] = {
	{WLJSNAME_FUNCTIONALTEST_REPORTER_ADD_MESSAGE,	FunctionalTestReporterAddMessage,	1},
	{WLJSNAME_FUNCTIONALTEST_REPORTER_ADD_ERROR,	  FunctionalTestReporterAddError,	  3},
	{WLJSNAME_FUNCTIONALTEST_REPORTER_CLEAR,		    FunctionalTestReporterClear,	    0},
	{WLJSNAME_FUNCTIONALTEST_REPORTER_REPORT,		    FunctionalTestReporterReport,	    0},
	{WLJSNAME_FUNCTIONALTEST_REPORTER_TO_STRING,    FunctionalTestReporterToString,   0},
	{0}
};


bool AddFunctionalTestReporterClass(JSContext *cx, JSObject *obj)
{
  if (!JS_InitClass(cx, obj, NULL, &FunctionalTestReporterClass, FunctionalTestReporterCtor, 1, NULL, FunctionalTestReporter_methods, NULL, NULL))
		return false;

  return true;
}

static JSBool
FunctionalTestObjDetailsCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  if (argc != 0) {
    WLErrorException e(JS_DOM_ERROR_FILE, 120, WLJSNAME_FUNCTIONALTEST_OBJDETAILS_CLASS, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }


  CAssociativeArray *array = new CAssociativeArray;
  JS_SetPrivate(cx, obj, array);

  *rval = OBJECT_TO_JSVAL(obj);
  return JS_TRUE;
}



static JSBool
FunctionalTestObjDetailsClear(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 0) {
    WLErrorException e(JS_DOM_ERROR_FILE, 120, WLJSNAME_FUNCTIONALTEST_OBJDETAILS_CLASS"."WLJSNAME_FUNCTIONALTEST_OBJDETAILS_CLEAR, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);
  array->clearAndDestroy();

  return JS_TRUE;
}


static JSBool
FunctionalTestObjDetailsSet(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	int i;

	if (argc != 9) {
    WLErrorException e(JS_DOM_ERROR_FILE, 120, WLJSNAME_FUNCTIONALTEST_OBJDETAILS_CLASS"."WLJSNAME_FUNCTIONALTEST_OBJDETAILS_SET, 8);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

  CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);

  //first, second and third arguments are strings - ObjectType,TestType and Url, and the other 6 numbers
  for (i=0;i<3;i++) {
	  if (!JSVAL_IS_STRING(argv[i])) {
      WLErrorException e(JS_DOM_ERROR_FILE, 122, WLJSNAME_FUNCTIONALTEST_OBJDETAILS_CLASS"."WLJSNAME_FUNCTIONALTEST_OBJDETAILS_SET, i+1);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	  }
	}

	for (i=3;i<9;i++) {
	  if (!JSVAL_IS_INT(argv[i])) {
      WLErrorException e(JS_DOM_ERROR_FILE, 123, WLJSNAME_FUNCTIONALTEST_OBJDETAILS_CLASS"."WLJSNAME_FUNCTIONALTEST_OBJDETAILS_SET, i+1);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	  }
	}

  array->AddCouple(WLJSNAME_FUNCTIONALTEST_OBJDETAILS_OBJ_TYPE,JS_GetStringBytes(JS_ValueToString(cx, argv[0])));
  array->AddCouple(WLJSNAME_FUNCTIONALTEST_OBJDETAILS_TEST_TYPE,JS_GetStringBytes(JS_ValueToString(cx, argv[1])));
	array->AddCouple(WLJSNAME_FUNCTIONALTEST_OBJDETAILS_URL,JS_GetStringBytes(JS_ValueToString(cx, argv[2])));
  array->AddCouple(WLJSNAME_FUNCTIONALTEST_OBJDETAILS_INDEX1,(long)JSVAL_TO_INT(argv[3]));
  array->AddCouple(WLJSNAME_FUNCTIONALTEST_OBJDETAILS_INDEX2,(long)JSVAL_TO_INT(argv[4]));
  array->AddCouple(WLJSNAME_FUNCTIONALTEST_OBJDETAILS_X1,(long)JSVAL_TO_INT(argv[5]));
  array->AddCouple(WLJSNAME_FUNCTIONALTEST_OBJDETAILS_Y1,(long)JSVAL_TO_INT(argv[6]));
  array->AddCouple(WLJSNAME_FUNCTIONALTEST_OBJDETAILS_X2,(long)JSVAL_TO_INT(argv[7]));
  array->AddCouple(WLJSNAME_FUNCTIONALTEST_OBJDETAILS_Y2,(long)JSVAL_TO_INT(argv[8]));

  return JS_TRUE;
}


static void
FunctionalTestObjDetailsFinalize(JSContext *cx, JSObject *obj)
{
	CAssociativeArray* array = (CAssociativeArray*)JS_GetPrivate(cx, obj);
  if (array)
    delete array;
}

JSClass FunctionalTestObjDetailsClass = {
  WLJSNAME_FUNCTIONALTEST_OBJDETAILS_CLASS,
  JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,	// add Property
  JS_PropertyStub,	// del Property
  JS_PropertyStub,	// get Property
  JS_PropertyStub,	// set Property
  JS_EnumerateStub,
  JS_ResolveStub,
  JS_ConvertStub,
  FunctionalTestObjDetailsFinalize
};

static JSFunctionSpec FunctionalTestObjDetails_methods[] = {
	{WLJSNAME_FUNCTIONALTEST_OBJDETAILS_CLEAR,		    FunctionalTestObjDetailsClear,	    0},
	{WLJSNAME_FUNCTIONALTEST_OBJDETAILS_SET,		    FunctionalTestObjDetailsSet,	    0},
	{0}
};


bool AddFunctionalTestObjDetailsClass(JSContext *cx, JSObject *obj)
{
  if (!JS_InitClass(cx, obj, NULL, &FunctionalTestObjDetailsClass, FunctionalTestObjDetailsCtor, 1, NULL, FunctionalTestObjDetails_methods, NULL, NULL))
		return false;

  return true;
}




