/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/*****************************************************************************

    File : GetHostIP.cpp
    Implementation of class GetHostIP.
    For multi platform use, Unix, Windows, Linux.
    Author: Yossi Nakash

*****************************************************************************/

#include <ace/OS.h>
#include "gethostip.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#ifdef __TV_WINDOWS__
#include <winsock2.h>
#include "mibaccess.h"
#else

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/utsname.h>
#include <netdb.h>
#endif


#ifdef __TV_UNIX__
#include <net/if.h>
#if !defined(__TV_linux__)
#include <sys/sockio.h>
#endif
#endif
#include "tracedll.h"

#define LOCAL_LOOP_IP "127.0.0.1"
#define GET_HOST_MAX_IPS 40 //old way gethostbyname, gets only 35 IPs

GetHostIP::GetHostIP()
{
  m_pIPList = NULL ;
  m_nIPNum = 0 ;
//  GetIPAddress() ;
}

GetHostIP::~GetHostIP()
{
  for (int nIndex =0 ; nIndex < GetNum() ; nIndex++ )
    delete m_pIPList[nIndex] ;
  delete []m_pIPList;
}

int GetHostIP::GetIPAddress()
{
WlTraceMsg("inside GetHostIP::GetIPAddress()");

  struct in_addr in;
#ifndef __TV_WINDOWS__  /*__TV_UNIX__ , __TV_linux__ etc. */

  int nIndex, nSocket ;
  char buff[SIZE_OF_BUFFER];
  struct ifconf sockConf;   /* structs defined in "if.h" */
  struct ifreq *p_sockAddr;
  struct sockaddr_in *sin;

  sockConf.ifc_len = sizeof(buff);
  sockConf.ifc_ifcu.ifcu_buf = buff;

  if ((nSocket = socket(AF_INET, SOCK_DGRAM, 0)) < 0) /* open socket */
  {
    WlTraceMsg("failed to open socket");
    return 0;
  }

  if (ioctl(nSocket, SIOCGIFCONF, &sockConf) < 0) /* get socket configuration*/
  {
    WlTraceMsg("failed to get socket configurations");
    return 0;
  }

  p_sockAddr = sockConf.ifc_ifcu.ifcu_req;

  /* get IP addresses */
  m_pIPList = new struct in_addr*[sockConf.ifc_len / sizeof(struct ifreq)+1];

  for (nIndex = sockConf.ifc_len / sizeof(struct ifreq); --nIndex >= 0 ; p_sockAddr++)
  {
    if (ioctl(nSocket, SIOCGIFADDR, (char*)p_sockAddr) < 0)
    {
      /* unknown interface */
      break;
    }
    else
    {
      sin = (struct sockaddr_in *) &(p_sockAddr->ifr_addr);

      AddIPToList(sin->sin_addr);
    }
  }

  DeleteAddr(LOCAL_LOOP_IP);
  (void) close(nSocket);

  //we failed to get IP using snmp, so we're trying the old way.
  if (m_nIPNum <= 0)
  {
    WlTraceMsg("failed to find any IP using ioctl, trying the old way (gethostbyname)");
    char  name[1024];
    struct  utsname sname;
    if (uname(&sname) == -1)
    {
      WlTraceMsg("uname function failed.");
      return 0;
    }
    strcpy(name, sname.nodename);

    struct hostent* hp = gethostbyname(name);
    if (hp == NULL)
    {
      WlTraceMsg("gethostbyname function failed.");
      return 0;
    }

    int i = 0 ;
    for (char** p = hp->h_addr_list; *p != 0; p++, i++)
    {
      memcpy((void*)&in.s_addr, *p,(sizeof(in.s_addr)));
		  AddIPToList(in);
    }
  }

  WlTraceMsg("found %d IPs", m_nIPNum);

  return m_nIPNum ;

#else /*__TV_UNIX__ , __TV_linux__ etc. */

  MibII mObj;
	UINT IPArray[SIZE_OF_BUFFER/4];
	UINT IPArraySize = SIZE_OF_BUFFER/4;


  mObj.Init();
	if(mObj.GetIPAddress(IPArray,IPArraySize))
  {
			m_pIPList = new struct in_addr*[IPArraySize+1];

			for(UINT i = 0 ; i < IPArraySize ; i++ )
      {
        in.s_addr = IPArray[i];
        // @@Chaim - I added this check because I saw in Orange that sometimes the system returns 0.0.0.0
        if (in.s_net || in.s_host || in.s_lh || in.s_impno){
			  AddIPToList(in);
			}
  }
  }
  DeleteAddr(LOCAL_LOOP_IP);

  //we failed to get IP using snmp, so we're trying the old way.
  if (m_nIPNum <= 0)
  {
    WlTraceMsg("failed to find any IP using snmp, trying the old way (gethostbyname)");
    char  name[1024];
		m_pIPList = new struct in_addr*[GET_HOST_MAX_IPS];

    if (gethostname(name, sizeof(name)) == -1 )
    {
      WlTraceMsg("gethostname function failed.");
      return 0;
    }

    struct hostent* hp = gethostbyname(name);
    if (hp == NULL)
    {
      WlTraceMsg("gethostbyname function failed");
      return 0;
    }

    int i = 0 ;
    for (char** p = hp->h_addr_list; *p != 0; p++, i++)
    {
      memcpy((void*)&in.s_addr, *p,(sizeof(in.s_addr)));
		  AddIPToList(in);
      IPArraySize++;
    }
  }

  WlTraceMsg("found %d IPs", m_nIPNum);

  return m_nIPNum ;

#endif
}


void GetHostIP::AddIPToList(const struct in_addr& pAddr)
{
    m_pIPList[m_nIPNum] = new struct in_addr;
    memcpy((char*)m_pIPList[m_nIPNum], (char*)&pAddr, sizeof(struct in_addr));
    m_pIPList[++m_nIPNum] = NULL ;
}



int GetHostIP::DeleteAddr(const char *pcAddr)
{
  struct in_addr delAddr = StrToAddr(pcAddr) ;

  return DeleteAddr(delAddr);
}


int GetHostIP::DeleteAddr(const struct in_addr& pAddr)
{
  for (int i = 0 ; i < m_nIPNum ; i++)
  {
    if ( m_pIPList[i]->s_addr == pAddr.s_addr )
    {
      memcpy((char*)m_pIPList[i], (char*)m_pIPList[--m_nIPNum], sizeof(struct in_addr));
      delete m_pIPList[m_nIPNum] ;
      m_pIPList[m_nIPNum] = NULL ;
      return 1 ;
    }
  }
  return 0;
}


struct in_addr* GetHostIP::GetAt(int nPos)
{
  return IsExist(nPos) ? m_pIPList[nPos] : NULL ;
}

struct in_addr* GetHostIP::GetNext(int &nPos)
{
   return IsExist(nPos) ? m_pIPList[nPos++] : NULL ;
}

struct in_addr* GetHostIP::GetPrev(int &nPos)
{
  return IsExist(nPos) ? m_pIPList[nPos--] : NULL ;
}



char* GetHostIP::GetAtStr(int nPos)
{
  return IsExist(nPos) ? AddrToStr(*m_pIPList[nPos]) : NULL ;
}

char* GetHostIP::GetNextStr(int &nPos)
{
   return IsExist(nPos) ? AddrToStr(*m_pIPList[nPos++]) : NULL ;
}

char* GetHostIP::GetPrevStr(int &nPos)
{
  return IsExist(nPos) ? AddrToStr(*m_pIPList[nPos--]) : NULL ;
}

//static function
struct in_addr GetHostIP::StrToAddr(const char *pcAddr)
{
  struct my_in_addr{
    union{
      unsigned char uc_Addr[4];
      struct in_addr  ia_Addr ;
  }Addr;
  }convertAddr;

  char pcNum[4][4] = {0};
  int  nIndex, nTotal = 0, nPIndex = 0;

  while (pcAddr[nTotal])
  {
    for (nIndex=0 ; pcAddr[nTotal] != '.' && pcAddr[nTotal] ; nIndex++, nTotal++)
    {
      pcNum[nPIndex][nIndex] = pcAddr[nTotal];
    }

    pcNum[nPIndex][nIndex] = '\0';
    convertAddr.Addr.uc_Addr[nPIndex] = (unsigned char)atoi(pcNum[nPIndex]);

    nPIndex++;
    if ( pcAddr[nTotal] == '.' )
      nTotal++;
  }

  return convertAddr.Addr.ia_Addr ;
}

//static function
char* GetHostIP::AddrToStr(const struct in_addr &Addr)
{
  return (char*)inet_ntoa(Addr);
}
