/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : HttpRequest.cpp
 *
 *	Author : Zion Menahem
 *
 *	Description : Based on the HTTPUAR module, this file implements
 *	              the HTTP protocol for JavaScript module.
 *
 *	History : 
 *
 *	Modifications : 
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

/** include files **/
#include <math.h>
#include "rvjsapi.h"
#include <RVCString.h>
#include "tvpnumconversions.h"
#include "wlqueue.h"
#include "httpdefines.h"
#include "jshttpnames.h"
#include "commandmanager.h"
#include "connpool.h"
//#include "httpssl.h"
//#include "sslerrs.h"
#include "httprequest.h"
#include "wljsreply.h"
#include "workspace.h"
#include "httpcookies.h"
#include "httptools.h"
#include "htmlvalues.h"
#include "wlmultipart.h"
#include "httpheaderhandler.h"
#include "gt_strwrapper.h"
#include "tvenv.h"
#ifdef __TV_WINDOWS__
#include "authorization.h"
#endif __TV_WINDOWS__
#include "tracedll.h"
#include "httpembedderreport.h"
#include "wlxml.h"
#ifdef __TV_solaris__
#include <ctype.h>
#endif
#include "wlhttpcache.h"
#include <RVCTokenizer.h>

#include "WLGenericConnection.h"
#include "WLSecureSslConnection.h"
#include "WLInsecureConnection.h"

/** local definitions **/
#define MAX_ASCII       256
#define CONNTIMEOUT     500
#define ACCEPT_VALUE		"*/*"
#define ACCEPT_ENCODING_GZIP      "gzip, deflate"
#define URLENCODED_CONTENT_TYPE		"application/x-www-form-urlencoded"
#define MULTIPART_CONTENT_TYPE		"multipart/form-data"
#define BUFFSIZE				8192

#define HTTP_STRING				"http://"
#define HTTP_STRING_LEN		7
#define HTTPS_STRING			"https://"
#define HTTPS_STRING_LEN	8
#define ABOUT_STRING      "about:"
#define JAVASCRIPT_STRING "javascript:"


#define WL_CONN_TIMEOUT		"__CONNECTION_TIMEOUT"

// Timers & counters
#define CONNECTION_TRIALS    "Attempted Connections"
#define SUCCESS_CONNECTION   "Successful Connections"
#define FAILED_CONNECTION    "Failed Connections"
#define CONNECT_TIME         "Connect Time"

#define SEND_TIME            "Send Time"

#define RESPONSES_COUNT      "Responses"
#define RESPONSE_TIME        "Response Time"
#define RESPONSE_SIZE        "Response Data Size"

#define PROCESS_TIME         "Process Time"
#define TRANSACTION_TIME     "Hit Time"

#define SUCCESS_TRANSACTIONS "Successful Hits"
#define FAILED_TRANSACTIONS  "Failed Hits"
#define TOTAL_TRANSACTIONS   "Hits"
#define HTTP_STATUS          "HTTP Response Status"

#define CHECK_ABORT    if (m_Cm->isAbort()) return IS_ABORT;


#define ENC(c) six2pr[c]
#define IntToString TVPIntToString


// Macros
#define SetErrorCode m_Connection->SetErrorCode
#define ConnectError m_Connection->ReportError
#define ThreadError  m_ConnPool->ReportError
#define addCounter      if (!m_Cm->isAbort()) m_Cm->addCounter
#define addTimer        if (!m_Cm->isAbort()) m_Cm->addTimer
#define addAccumulator  if (!m_Cm->isAbort()) m_Cm->addAccumulator

#define JS7_ISHEX(c)    ((c) < 128 && isxdigit(c))
#define JS7_UNHEX(c)    (uintN)(isdigit(c) ? (c) - '0' : 10 + tolower(c) - 'a')

#define C_No_Authentication							0
#define C_Server_Basic_Authentication   1
#define C_Server_Nt_Authentication			2
#define C_Proxy_Basic_Authentication    3
#define C_Proxy_Nt_Authentication				4

#define IS_REDIRECTION(stat) ((stat) >= 300  &&  (stat) <= 305  &&  (stat) != 304)


/** default settings **/

/** external functions **/
extern void WLParse_AddParseFunction();

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/
static char six2pr[64] = {
  'A','B','C','D','E','F','G','H','I','J','K','L','M',
  'N','O','P','Q','R','S','T','U','V','W','X','Y','Z',
  'a','b','c','d','e','f','g','h','i','j','k','l','m',
  'n','o','p','q','r','s','t','u','v','w','x','y','z',
  '0','1','2','3','4','5','6','7','8','9','+','/'
};


static char ParseChar[MAX_ASCII][4];

/** public functions **/

/** private functions **/


// Replace the ascii code to code that known to
// HTTP protocol, and store it in ParseChar array.
void  ParseCharForHttp()
{
  for (int i = 0; i < MAX_ASCII; i++) {
    char c = (char)i;
    if (c == '@' || c == '*' || c == '_' || c == '-' ||
        c == '.' || /*c == '=' || */(c >= '0' && c <= '9') ||
        (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z')) {
      ParseChar[i][0] = c;
      ParseChar[i][1] = '\0';
      continue;
    }
    
    if (c == ' ') {
      ParseChar[i][0] = '+';
      ParseChar[i][1] = '\0';
      continue;
    }
    
    ParseChar[i][0] = '%';
    int a = i / 16;
    if (a < 10)
      ParseChar[i][1] = '0' + (char)a;
    else 
      ParseChar[i][1] = 'A' + (char)(a - 10);
    int b = i - a*16;
    if (b < 10)
      ParseChar[i][2] = '0' + (char)b;
    else 
      ParseChar[i][2] = 'A' + (char)(b - 10);
      ParseChar[i][3] = '\0';
  }
}

void EncodeString(const char* val, RVCString& result, bool encode)
{
  if (val == NULL)
    return;

	if (!encode) {
		result += val;
		return;
	}

  for (int i = 0; i < (int)strlen(val); i++) {
    unsigned char a = (unsigned char)val[i];
    result += ParseChar[a];
  }
  return;
}

static void HTTPEncode(RVCString& bufin, RVCString& bufcoded)
{
  register char outptr[1024]="";
  int i, j = 0;
  int nbytes = bufin.length();
  // Chaim - bug 19474
  bufin += "   ";
  bufin += "\x00\x00\x00";
 
  for (i=0; i<nbytes; i += 3) {
    outptr[j++] = ENC(bufin(i) >> 2);            /* c1 */
    outptr[j++] = ENC(((bufin(i) << 4) & 060) | ((bufin(i+1) >> 4) & 017)); /*c2*/
    outptr[j++] = ENC(((bufin(i+1) << 2) & 074) | ((bufin(i+2) >> 6) & 03));/*c3*/
    outptr[j++] = ENC(bufin(i+2) & 077);         /* c4 */
  }
 
  /* If nbytes was not a multiple of 3, then we have encoded too
   * many characters.  Adjust appropriately.
   */
  if(i == nbytes+1) {
    /* There were only 2 bytes in that last group */
    outptr[j-1] = '=';
  } else if(i == nbytes+2) {
    /* There was only 1 byte in that last group */
    outptr[j-1] = '=';
    outptr[j-2] = '=';
  }
  outptr[j] = '\0';
  bufcoded = outptr;
}

static void str_unescape(const char* fromStr, RVCString& toStr)
{
    size_t  fromStrLen = strlen (fromStr);
    size_t  i = 0;
    size_t  newPos = 0;
    char    ch;
    char*   toCStr;

    toCStr = (char*) malloc (fromStrLen+1);

    while (i < fromStrLen) {
      ch = fromStr[i];
    	if (ch == '%') {
	      if ((i + 2 < fromStrLen) && 
            (JS7_ISHEX(fromStr[i+1])) && 
            (JS7_ISHEX(fromStr[i+2]))) {
          ch = JS7_UNHEX(fromStr[i+1]) * 16 + JS7_UNHEX(fromStr[i + 2]);
          i += 3;
        }
	          // Chaim - handle double %%
		  else if (fromStr[i+1] == '%'){
			i += 2;
		  }
        else if ((i + 5 < fromStrLen) && 
                 (fromStr[i+1] == 'u') &&
		             (JS7_ISHEX(fromStr[i + 2])) && 
                 (JS7_ISHEX(fromStr[i + 3])) &&
		             (JS7_ISHEX(fromStr[i + 4])) && 
                 (JS7_ISHEX(fromStr[i + 5]))) {
		      ch = (((((JS7_UNHEX(fromStr[i + 2]) << 4)
      			    + JS7_UNHEX(fromStr[i + 3])) << 4)
		            + JS7_UNHEX(fromStr[i + 4])) << 4)
		            + JS7_UNHEX(fromStr[i + 5]);
           i += 6;
        }
      }
      else {
        // Chaim - increment outside of loop (19562)
//        i++;
      }
      toCStr[newPos] = ch;
      newPos++;
      i++;
    }

    toCStr[newPos] = 0;

    toStr.append(toCStr, newPos);

    free (toCStr); 
}


// Functions for myCollectableStr
bool myCollectableStr::isEqual(const RVCollectable* c) const
{
  RVCString *bfr = (RVCString *)(myCollectableStr *)c;
  if (RVCString::compareTo(*bfr, RVCString::ignoreCase) == 0)
    return TRUE;
  else
    return FALSE;
}





// 
// Functions for CWLHttpRequest class.
CWLHttpRequest::CWLHttpRequest()
  : m_headers((RVSize_T)1024), m_Url((RVSize_T)112), m_Frame(NULL)
{
	m_Version = VERSION10;
	m_RequestKeepAlive = false;
	m_ResponseKeepAlive = false;

	m_ThreadMode = Slave;

	m_WaitForNTResponse = false;

	m_Authorization = NULL;
  m_bFirstTimeWait = true ;
  m_dStartWait = m_dStartReceive = 0;
  m_pMultiPart = NULL;
  m_bAuthDone  = false ;
  m_bPAuthDone  = false ;
  m_pRecvBuff = NULL ;
	m_Cm = NULL ;
  m_previousProtocol = UNKNOWN_PROTOCOL;
  
  m_TmpSslConnection = NULL;
  // Init CB Data as closed
  m_CBDataClosed = true;


#ifdef __TV_WINDOWS__
	authorizationReset();
	authorizationClean();
#endif

}

CWLHttpRequest::~CWLHttpRequest()
{
	if (m_pRecvBuff)
		delete []m_pRecvBuff;

  if (m_pMultiPart)
    delete m_pMultiPart ;

#ifdef __TV_WINDOWS__
	authorizationTerminate();
#endif
}

bool CWLHttpRequest::Init(CWLCommandManager* cm, CWLConnectionsPool* pool, int num)
{
	m_ThreadNum = num;
	m_Cm = cm;
	m_Ws = m_Cm->GetWorkspace();
	m_ConnPool = pool;
	m_Connection = NULL;
  m_TmpSslConnection = NULL;
//  m_Ssl = NULL;
	m_Cookies = m_Cm->GetCookies();
	m_Page = m_Cm->GetPage();

  m_Response.SetCallback(m_Cm->IsCallbackEnabled());
  m_Response.SetCheckFileContent(m_Ws->m_bCheckFileContent);
  m_nRecvBufferSize = m_Ws->m_RecvBufferSize;
  if (m_pRecvBuff)
  {
		delete []m_pRecvBuff;
		m_pRecvBuff = NULL ;
  }
  m_pRecvBuff  = new char[m_nRecvBufferSize];
  m_bAuthDone = false ;
  m_bPAuthDone = false ;
  m_previousProtocol = UNKNOWN_PROTOCOL;

	return true;
}

int CWLHttpRequest::acquireConnection()
{
	// If no connection is allocated then we try to acquire a new one
	if (!m_Connection) {
    m_Connection = m_ConnPool->AcquireConnection( m_protocol, m_host.data(), m_port, m_ThreadNum, this );
	}

	// Check if connection was acquired successfully
	if (!m_Connection) {
		return RET_ERROR;
	}
	else {
    m_Connection->SetHttpRequest(this);
		m_Connection->SetUrlType(m_UrlType);
    m_Connection->SetTransactionObj(m_pTransObj);

		return RET_OK;
	}
}

void CWLHttpRequest::releaseConnection()
{
	if (m_Connection) {
#ifdef _DEBUG
	WlTraceMsg("Releasing connection %d, ip:%s, port:%u\n", m_Connection->m_Index, m_Connection->ip(), m_Connection->port());
#endif
		m_ConnPool->ReleaseConnection(m_Connection, m_ThreadNum);
		m_Connection->SetUrlType(FRAME);	// kind of reseting the url type
    m_Connection->SetTransactionObj(NULL);
		m_Connection = NULL;
	}
}

int CWLHttpRequest::openConnection()
{
	int Status = RET_ERROR;
	// Check if we have a connection to open and then open it
	if (m_Connection) {
		// If we are using a proxy and SSL then we set the protocol 
		// of the connection to HTTP
		if ((m_usingProxy) && (m_protocol == SSL_PROTOCOL)) {
		// we need new object to establish only TCP connection with proxy-server
		m_TmpSslConnection = m_Connection;
		m_Connection = new CWLInsecureConnection( m_Connection );
		}
	
		Status = m_Connection->OpenConnection(m_host, m_port, m_usingProxy, m_ThreadNum);
    m_Connection->ClearHttpStatus();

	// if TCP connection with proxy-server was established successfully
	// set created socket descriptor to SSL connection object
	if( ( m_protocol == SSL_PROTOCOL ) && m_TmpSslConnection && m_usingProxy && ( RET_OK == Status || KEPT_ALIVE == Status ) ){
		// SSL connection uses created socket
		m_TmpSslConnection->setSocket( m_Connection->sock() );
		// bug # 172 fix
		m_TmpSslConnection->SetTransactionData( m_Connection->GetTransactionData() );
		m_Connection->setSocket( -1 );
		delete m_Connection;
		m_Connection = m_TmpSslConnection;
		m_TmpSslConnection = NULL;
			}

	}
	return Status;
}

void CWLHttpRequest::closeConnection(bool forceClose)
{

// Chaim - Bug 19474 - If the server actually sends a "Connection: Close", then we close even if we're in the
// middle of a challenge-response session. This will generally happen from what I've seen if the resource
// we're trying to fetch is redirected,
//  if ( m_DuringChallengeResponse ) {
  if ( m_DuringChallengeResponse && (!m_Response.m_bHeaderConnection || strcmp(m_Response.m_HeaderConnectionValue, TOKEN_CLOSE)) ) {
    return;
  }

//	if (!m_usingProxy && m_ResponseKeepAlive && !forceClose)
// Shahar - Patch 5.0.226.001 - We are not supposed to close proxy connection
// In any case. The close of the connection should be done only if the server
// had asked to close the connection...
	if (m_ResponseKeepAlive && !forceClose)
		return;

  if (m_Connection){
    m_Cm->ResetRequestOnConnectionCounter();
		m_Connection->CloseConnection(forceClose, m_ThreadNum);
}
}

int CWLHttpRequest::send(const char* buff, int len, int protocol)
{
	double sendTime = TVPSysTime();
//WlTraceMsg( "sending len: %d, buff:\n%s\n", len, buff);
  
  if (m_pTransObj)
  {
    m_pTransObj->AddSubTransaction("Send", TVPSysTime(), CWLTransactionData::TRANS_SEND);
  }

  int ret = m_Connection->Send(buff, len, /*protocol,*/ m_ThreadNum);

  if (m_pTransObj)
  {
    m_pTransObj->GetLastSubTransation()->SetEndTransaction(TVPSysTime(), WLLogMessage::Success);//Send
    if (ret == RET_ERROR)
    {
      m_Connection->SetTransactionData(m_pTransObj->GetLastSubTransation());     
      ConnectError(HTTP_ERROR_FILE, 109);
      m_Connection->SetTransactionData(NULL);
    }

    m_dStartWait = TVPSysTime();
    SetFirstTimeWait(true);
    m_pTransObj->AddSubTransaction("Time To First Byte", m_dStartWait, CWLTransactionData::TRANS_WAIT);
  }

	double dt = TVPSysTime() - sendTime;
	addTimer(SEND_TIME, dt);
	return ret;
}

int CWLHttpRequest::recv(char* buff, int* len, int protocol, bool* closed, int* errnum)
{
  return m_Connection->Recv(buff, len, /*protocol,*/ 0, errnum, closed, m_ThreadNum);
}

int CWLHttpRequest::selectForRead(int* errnum, int inputTimeout)
{
	return m_Connection->SelectForRead(errnum, inputTimeout);
}

int CWLHttpRequest::NewselectForRead(int inputTimeout)
{
	int			 SelectStatus;
	int			 errSelect = 0;
	int			 ReturnStatus = RET_OK;	

	SelectStatus = m_Connection->SelectForRead(&errSelect, inputTimeout);
	WlTraceMsg("Select for read returned status = %d (error-%d)", SelectStatus, errSelect);
	if (SelectStatus >= 1) {
		if (errSelect == -1) {
			ReturnStatus = RET_ERROR;
		}
		else if (errSelect == WSAECONNRESET) {
			#ifdef _DEBUG
			addCounter("Retry fetch - ECONNRESET");
			#endif
			ReturnStatus = RET_RETRY_FETCH;
		}
		else {
			ReturnStatus = RET_OK;
		}
	}
	else if (SelectStatus == SOCKET_ERROR){
			ReturnStatus = RET_ERROR;
	}
	else {
			ReturnStatus = RET_OK;
	}

	return ReturnStatus;
}

void CWLHttpRequest::ParseCharFunc(const char* val, RVCString& result)
{
	EncodeString(val, result, m_Ws->m_EncodeFormdata);
/*
  if (val == NULL)
    return;
  for (int i = 0; i < (int)strlen(val); i++) {
    unsigned char a = (unsigned char)val[i];
    result += ParseChar[a];
  }
  return;
*/
}


CWLHttpCache& CWLHttpRequest::getHttpCache()
{
	return m_Cm->GetHttpCache();
}

void CWLHttpRequest::updateCachedHeadersAndContent( CWLHttpCacheEntry* p_pCacheEntry )
{
	if ( p_pCacheEntry )
	{
		p_pCacheEntry->UpdateContent( m_Response.GetHeader( CS_HEADER_CONTENT_TYPE ),
			                            m_Response.GetHeader( CS_HEADER_EXPIRES ),
			                            m_Response.GetHeader( CS_HEADER_DATE ),
			                            m_Response.GetHeader( CS_HEADER_ETAG ),
			                            m_Response.GetHeader( CS_HEADER_LAST_MODIFIED ),
																	m_Response.GetOutputData(), m_Response.GetOutputLength() );
	}
}

int CWLHttpRequest::retrieveAndSetCachedContent( CWLHttpCacheEntry* p_pCacheEntry )
{
	// update m_Output that will later be used to retrieve content
	if ( p_pCacheEntry )
	{
		// response content should be empty (content-lenth == 0)
		m_Response.SetServerStatus( 200 ); // simulate successful request

		// in case the cached content is zipped
		ZipTypeCode eZipCode = p_pCacheEntry->GetEZipCode();

		if ( ( eZipCode == ZipCode_GZIP ) || ( eZipCode == ZipCode_DEFLATE ) )
		{
			CWLGZip GZipObj;
			RVCString outBuff;
			if ( eZipCode == ZipCode_GZIP )
			{
				GZipObj.UNGzip ( p_pCacheEntry->GetContent(), outBuff );
			}

			if ( !outBuff.isNull() )
			{
				// save unzipped data for better performance
				p_pCacheEntry->SetContent( outBuff );
				p_pCacheEntry->SetEZipCode( ZipCode_NO );
			}
		}

          m_Response.GetOutputBuff().Write(p_pCacheEntry->GetContent().data(), p_pCacheEntry->GetByteCount());
          m_Response.SetContentType ( p_pCacheEntry->GetContentType() );
		  m_pTransObj->SetCached(true);
	}

	return RET_OK;
}

int CWLHttpRequest::ParseHeader()
{
	bool requestKeepAlive = false;

	bool bHostHeader = false;
	bool bAcceptHeader = false;
  bool bAcceptEncodingGzip = GetCm()->GetWorkspace()->m_bAcceptEncodingGzip;
	bool bAgentHeader = false;
  bool bReferer = false ;
	bool bConnectionHeader = false;
	bool bAuthorizationHeader = false;
	bool bCookieHeader = false;

	// Temporary variables for encoding basic authorization
	// for server and for proxy
//  RVCString  temp;
//	RVCString	 encodeUsernamePassword;
//	RVCString	 encodeProxyUsernamePassword;

  // Cookies variable
	RVCString  cookies;

	char lpszPort[32]="";

  m_headers.remove(0);

  if (m_Ws->HeaderBlock) {
		for (size_t i = 0; i < m_Ws->HeaderBlock->entries(); i++) {
			const char* key = m_Ws->HeaderBlock->Key(i);
			const char* val = m_Ws->HeaderBlock->Value(i);

      switch(CHLHeaderHandler::GetHeaderId(key))
      {
      case CHLHeaderHandler::EN_HEADER_AUTHORIZATION :
      {
  			// Authentication add header is not handled
        continue;
			}

      case CHLHeaderHandler::EN_HEADER_ACCEPT :
      {
        // mark the headers we're supposed to add later
				bAcceptHeader = true;

				if (!strcmp(val, VOID_STRING)) {
	        continue;
				}

        break ;
      }

      case CHLHeaderHandler::EN_HEADER_ACCEPT_ENCODING :
        {
          bAcceptEncodingGzip= false;
				  if (!strcmp(val, VOID_STRING)) {
	          continue;
				  }

            // Add new header line to header string
            m_headers += CS_HEADER_ACCEPT_ENCODING ;
            m_headers += ": ";
            m_headers += val;
            m_headers += CRandLF;

          break ;
        }

      case CHLHeaderHandler::EN_HEADER_CONNECTION :
      {
				bConnectionHeader = true;

				if (!strcmp(val, VOID_STRING)) {
	        continue;
				}

				// if connection should be closed then we
				// update our member epon
				if ( CHLHeaderHandler::GetHeaderId(val) == CHLHeaderHandler::EN_TOKEN_CLOSE) {
					// Make a warning if value of header is different from property
					if (m_Ws->m_UseKeepAlive) {
						ConnectError(HTTP_ERROR_FILE, 94, check_null(val));
					}
					// If we are during a challenge response we SHOULD NOT
					// close this connection
					if (!m_DuringChallengeResponse)
						m_RequestKeepAlive = false;
					else {
						m_RequestKeepAlive = true;
						continue;
					}
				}
				// else if connection should be keep alive then we
				// update our member upon
				else if (CHLHeaderHandler::GetHeaderId(val) == CHLHeaderHandler::EN_TOKEN_KEEP_ALIVE) {
					m_RequestKeepAlive = true;
					// Make a warning if value of header is different from property
					if (!m_Ws->m_UseKeepAlive) {
						ConnectError(HTTP_ERROR_FILE, 94, check_null(val));
					}
				}
				else {
					// If we have an unknown token then we check for the version
					// of http and whether we are on challenge response phase
					if ((m_Version == VERSION11) || m_DuringChallengeResponse)
						m_RequestKeepAlive = true;
					else
						m_RequestKeepAlive = false;
				}
        break;
			}

			// If we find an User agent header then we check if it is
			// equel to the property User agent, if not then we use the
			// User agent from the header and send warning message to user
      case CHLHeaderHandler::EN_HEADER_USER_AGENT : 
      {
				bAgentHeader = true;

				if (!strcmp(val, VOID_STRING)) {
	        continue;
				}

        // Chaim - deprecated warning
//				if (m_Ws->m_UserAgent) {
//					if (strcmpi(val,m_Ws->m_UserAgent)) {
//						ConnectError(HTTP_ERROR_FILE, 93, check_null(val));
//					}
//				}
        break;
			}

      // if added Referer header 
      case CHLHeaderHandler::EN_HEADER_REFERER :
      {
				if (!strcmp(val, VOID_STRING)) {
          bReferer = true ;
	        continue;
				}

				if ( m_Cm->IsTopFrame(m_pTransObj) )              
        {
          bReferer = true ;
          // if val has a value, and that value is not equal to the property one.
					if (*m_Ws->m_sReferer && strcmpi(val,m_Ws->m_sReferer)) 
          {
						ConnectError(HTTP_ERROR_FILE, 89, check_null(val));
					}
			    // Add new header line to header string
			    m_headers += key ;
			    m_headers += ": ";
			    m_headers += val;
			    m_headers += CRandLF;
				}
        continue;
			}

			// the following headers are ignored and will not be added into
			// the request
      
				// don't use the 'content-length' header if exist in the script - not relevant for GET/HEAD
				// and computed for POST method
      case CHLHeaderHandler::EN_HEADER_CONTENT_LENGTH :
        continue;

				// the header is added (and relevant) for POST method only
      case CHLHeaderHandler::EN_HEADER_CONTENT_TYPE :
        continue;

				// ignore 'cookie' header
      case CHLHeaderHandler::EN_HEADER_COOKIE :
				{
					if (!strcmp(val, VOID_STRING)) {
						bCookieHeader = true;
						continue;
					}
					if (!m_Ws -> m_OverrideCookieDefault) {
						continue;
					}
					else {
						bCookieHeader = true;
						break;
					}
				}

				// if a proxy connection header exist, it is ignored 
				// and will be added later as neccecery
      case CHLHeaderHandler::EN_HEADER_PROXY_CONNECTION :
        continue;

				// if a proxy authorization header exist, it is ignored 
				// and will be added later as neccecery
      case CHLHeaderHandler::EN_HEADER_PROXY_AUTHORIZATION :
        continue;

				// retrieve the url regardless of it's date
      case CHLHeaderHandler::EN_HEADER_IF_MODIFIED_SINCE :
				{

					if (!strcmp(val, VOID_STRING)) {
							continue;
					}

					if (!m_Ws -> m_OverrideIfModifiedSinceDefault)
		        continue;
					else
						break;
				}
				
				// the header must be generated 
      case CHLHeaderHandler::EN_HEADER_HOST :
				{
					if (!strcmp(val, VOID_STRING)) {
							bHostHeader = true;
							continue;
					}
					if (!m_Ws -> m_OverrideHostDefault) {
        continue;
					}
					else {
						bHostHeader = true;
						break;
					}
				}

      default:;
			}

			// Add new header line to header string
			m_headers += key ;
			m_headers += ": ";
			m_headers += val;
			m_headers += CRandLF;
		}
	}

  // add the following headers only if they where not added on
  // the headers
	// Add the kind of data we accepting to get
	if (!bAcceptHeader) {
		m_headers += CS_HEADER_ACCEPT;
		m_headers += ": ";
		m_headers += ACCEPT_VALUE;
		m_headers += CRandLF;
	}

  // Add the if we get gzip data
  if (bAcceptEncodingGzip) {
    m_headers += CS_HEADER_ACCEPT_ENCODING;
    m_headers += ": ";
    m_headers += ACCEPT_ENCODING_GZIP;
    m_headers += CRandLF;
  }

	// Add user agent header we are representing (default Radview)
	if (!bAgentHeader) {
		m_headers += CS_HEADER_USER_AGENT;
		m_headers += ": ";

		// If the property user agent contain some value
		// then we add it else we add the default rad-view 
		// user agent
		if (m_Ws->m_UserAgent) 
				m_headers += m_Ws->m_UserAgent;
		else
				m_headers += DEFAULT_USER_AGENT;

		m_headers += CRandLF;
	}

	// Add host header and port if needed
	if (!bHostHeader) {
		m_headers += CS_HEADER_HOST;
		m_headers += ": ";
		m_headers += m_server_host;
		if ((m_protocol == SSL_PROTOCOL && m_server_port != SSL_PORT) ||
				(m_protocol != SSL_PROTOCOL && m_server_port != HTTP_PORT)) {
			itoa(m_server_port, lpszPort, 10);
			m_headers += ":";
			m_headers += lpszPort;
		}
		m_headers += CRandLF;
	}

	// if connection header was not added from headers,
	// we add the connection(:Keep-Alive or :Close) 
	// header epon the http version

	// If it is vesrion 1.1 then we check if specific close connection
	// was asked or if we are not during challenge response phase
	// and then we add the close connection header
	if (!bConnectionHeader) {
		if (m_Version == VERSION11) {
			if (!m_Ws->m_UseKeepAlive && !m_DuringChallengeResponse) {
					m_headers += CS_HEADER_CONNECTION;
					m_headers += ": ";
					m_headers += TOKEN_CLOSE;
					m_headers += CRandLF;
					m_RequestKeepAlive = false;
			}
			else {	// request to keep-alive
					m_RequestKeepAlive = true;

        // Chaim - I saw browsers send it
        m_headers += CS_HEADER_CONNECTION;
        m_headers += ": ";
        m_headers += TOKEN_KEEP_ALIVE;
        m_headers += CRandLF;
        m_RequestKeepAlive = true;
			}
		}
    // else it is version 1.0 then we check if specific keep alive
		// was asked or if we are during challenge response phase
		// and then we add the keep alive connection header
		else {
			if (m_Ws->m_UseKeepAlive || m_DuringChallengeResponse) {
				m_headers += CS_HEADER_CONNECTION;
				m_headers += ": ";
				m_headers += TOKEN_KEEP_ALIVE;
				m_headers += CRandLF;
				m_RequestKeepAlive = true;
			}
			else {	// request not to keep-alive
				m_RequestKeepAlive = false;
			}
		}
	}


	// add referer header if required
  if (!bReferer)
  {
    const char *sReferer = GetReferer() ;

    if (sReferer)
    {
		  m_headers += CS_HEADER_REFERER;
		  m_headers += ": ";
		  m_headers += sReferer;
		  m_headers += CRandLF;
	  }
  }
	if (!bCookieHeader) {
  if (m_Cookies->makeCookieRequest(m_server_host, 
                                   m_server_uri, 
                                   cookies,
                                   m_Ws->m_CookiePath, 
                                   m_Ws->m_CookieDomain, 
                                   m_Ws->m_CookieExpiration) == RET_ERROR) {
    ConnectError(HTTP_ERROR_FILE, 28);
    return RET_ERROR;
  }
  else if (!cookies.isNull())  {
    m_headers += cookies;
    m_headers += CRandLF;
  }
	}

	if ((m_usingProxy) && (m_protocol != SSL_PROTOCOL)) {
		// Add proxy connection header if not added before
		m_headers += CS_HEADER_PROXY_CONNECTION;
		m_headers += ": ";
		m_headers += TOKEN_KEEP_ALIVE;
		m_headers += CRandLF;
  }

	{
		RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock);
		if (!m_Cm->m_AuthenticateNTHeaders.isNull()) {
			m_headers += m_Cm->m_AuthenticateNTHeaders;
		} 
		else if (!m_Cm->m_AuthenticateBasicHeaders.isNull()) {
			m_headers += m_Cm->m_AuthenticateBasicHeaders;
		}
	}

	{
		RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock);
		if (!m_Cm->m_AuthenticateNTProxyHeaders.isNull()) {
			m_headers += m_Cm->m_AuthenticateNTProxyHeaders;
		}
		else if (!m_Cm->m_AuthenticateBasicProxyHeaders.isNull()) {
			m_headers += m_Cm->m_AuthenticateBasicProxyHeaders;
		}
	}

  if (m_Cm->GetWorkspace()->m_sAcceptLanguage && m_Cm->GetWorkspace()->m_sAcceptLanguage[0]){
    // Chaim - RiskClick - accept-language
    m_headers += CS_HEADER_ACCEPT_LANGUAGE;
    m_headers += ": ";
    m_headers += m_Cm->GetWorkspace()->m_sAcceptLanguage;
    m_headers += CRandLF;
  }

  return RET_OK;
}

int CWLHttpRequest::ParseProxyHeader(RVCString& Headers)
{
	bool bAgentHeader = false;
  bool bReferer = false ;

	// Temporary variables for encoding proxy authorization
  RVCString  temp;
	RVCString	 encodeProxyUsernamePassword;

  if (m_Ws->HeaderBlock) {
		for (size_t i = 0; ((i < m_Ws->HeaderBlock->entries()) && (!bAgentHeader)); i++) {
			const char* key = m_Ws->HeaderBlock->Key(i);
			const char* val = m_Ws->HeaderBlock->Value(i);

			// If we find an User agent header then we check if it is
			// equel to the property User agent, if not then we use the
			// User agent from the header and send warning message to user
			if (CHLHeaderHandler::GetHeaderId(key) == CHLHeaderHandler::EN_HEADER_USER_AGENT) {
				bAgentHeader = true;

				if (!strcmp(val, VOID_STRING)) {
	        continue;
				}

        // Chaim - deprecated warning
//				if (m_Ws->m_UserAgent) {
//					if (strcmpi(val,m_Ws->m_UserAgent)) {
//						ConnectError(HTTP_ERROR_FILE, 93, val);
//					}
//				}
				// Add new header line to header string
				Headers += key ;
				Headers += ": ";
				Headers += val;
				Headers += CRandLF;
			}

			// If we find an Referer header then we check if it is
			// equel to the property Referer, if not then we use the
			// Referer from the header and send warning message to user
			if (CHLHeaderHandler::GetHeaderId(key) == CHLHeaderHandler::EN_HEADER_REFERER) {

				if (!strcmp(val, VOID_STRING)) {
          bReferer = true ;
	        continue;
				}

        if ( m_Cm->IsTopFrame(m_pTransObj) )
        {
          bReferer = true ;
					if (*m_Ws->m_sReferer && strcmpi(val,m_Ws->m_sReferer)) 
          {
						ConnectError(HTTP_ERROR_FILE, 89, val);
					}

				  // Add new header line to header string
				  Headers += key ;
				  Headers += ": ";
				  Headers += val;
				  Headers += CRandLF;
        }
			}

      // All other headers are ignored and will not be added into
			// the request
			else 
					continue;
		}
	}

	// add the following headers only if they where not added epon
	// the headers 

	// Add user agent header we are representing (default Radview)
	if (!bAgentHeader) {
		Headers += CS_HEADER_USER_AGENT;
		Headers += ": ";

		// If the property user agent contain some value
		// then we add it else we add the default rad-view 
		// user agent
		if (m_Ws->m_UserAgent) 
				Headers += m_Ws->m_UserAgent;
		else
				Headers += DEFAULT_USER_AGENT;

		Headers += CRandLF;
	}

	// add referer header if required
  if (!bReferer)
  {
    const char *sReferer = GetReferer() ;

    if (sReferer)
    {
		  m_headers += CS_HEADER_REFERER;
		  m_headers += ": ";
		  m_headers += sReferer;
		  m_headers += CRandLF;
	  }
  }

	// Add proxy connection header if not added before
	Headers += CS_HEADER_PROXY_CONNECTION;
	Headers += ": ";
	Headers += TOKEN_KEEP_ALIVE;
	Headers += CRandLF;

	{
		RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock);
		if (!m_Cm->m_AuthenticateNTProxyHeaders.isNull()) {
			Headers += m_Cm->m_AuthenticateNTProxyHeaders;
		}
		else if (!m_Cm->m_AuthenticateBasicProxyHeaders.isNull()) {
			Headers += m_Cm->m_AuthenticateBasicProxyHeaders;
		}
	}

  return RET_OK;
}

int CWLHttpRequest::CollectFormData(RVCString& query)
{
	if (m_Ws->FormdataBlock) {
		int ampersandFlag = 0;
		for (size_t i = 0; i < m_Ws->FormdataBlock->entries(); i++) {
			if (!ampersandFlag)
				ampersandFlag = 1;
			else
				query += "&";
      const char* key = m_Ws->FormdataBlock->Key(i);
			if (strcmp(key, VOID_STRING) && strcmp(key, EMPTY_STRING))
				ParseCharFunc(key, m_form_data);
      const char* val = m_Ws->FormdataBlock->Value(i);

			if (strcmp(val, VOID_STRING)) {
				query += "=";
				if (strcmp(val, EMPTY_STRING))
		  //Galtech
          ParseCharFunc(ConvertToServerEncoding(val), query);
			}
		}
	}

  return RET_OK;
}


int CWLHttpRequest::ParseFormData()
{
	// Clear the form data, content type and Multipart data...
	// which will be calculated in this method
  m_form_data.remove(0);
  m_content_type.remove(0);
  if (m_pMultiPart) {
      delete m_pMultiPart ;
      m_pMultiPart = NULL;
  }

	// if it's not the top most frame then we don't need to continue and
	// build any from data or multipart data...
	// Shahar - 26-11-02 - Patch 226.003
	if (!m_Cm->IsTopFrame(m_pTransObj)) {
      return 0;
	}

  RVCString sContentType(m_Ws->m_sContentType);
  RVCString sContentMultiPartBoundary(m_Ws->m_RequestContentMultiPartBoundary);

  if (sContentType.contains(MULTIPART_CONTENT_TYPE, RVCString::ignoreCase))
  {
    if (!m_Ws->FormdataBlock){
      ConnectError(HTTP_ERROR_FILE, 318, sContentType.data());
      return RET_ERROR ;
    }
    if (m_pMultiPart) {
      delete m_pMultiPart ;
      m_pMultiPart = NULL;
    }
    m_pMultiPart = new CWLMultiPart (this);

    int nRet = m_pMultiPart->CreateMultiPartData(m_Ws->FormdataBlock, m_Cm->GetLGThreadNum(), m_Cm->GetRoundNumScriptInSlave(), sContentMultiPartBoundary, GetServerEncoding());
    m_content_type = MULTIPART_CONTENT_TYPE ;
		if (sContentMultiPartBoundary.isNull()) {
    m_content_type += "; boundary=\"" + m_pMultiPart->GetBoundary();
	m_content_type +="\"";
		}
		else {
			m_content_type += "; boundary=\"" + sContentMultiPartBoundary;
          	m_content_type +="\"";
		}
    if (nRet != 0)
    {
      ConnectError(HTTP_ERROR_FILE, nRet, m_pMultiPart->GetFileName());
      return RET_ERROR ;
    }
      
  }
  else
  {
//	This was comment out by Shahar 26-11-02 - Patch 226.003 - This checking 
//	is done earlier...
/*
    if (m_ThreadNum != 0 || m_Cm->srcIndex > 0)	// only the master can send a formdata
      return 0;
*/
    if (m_Ws->FormdataBlock)  {
      int ampersandFlag = 0;
		  for (size_t i = 0; i < m_Ws->FormdataBlock->entries(); i++) {
        if (!ampersandFlag)
          ampersandFlag = 1;
        else
          m_form_data += "&";
        const char* key = m_Ws->FormdataBlock->Key(i);
			  if (strcmp(key, VOID_STRING) && strcmp(key, EMPTY_STRING))
			  //Galtech
          ParseCharFunc(ConvertToServerEncoding(key), m_form_data);
        const char* val = m_Ws->FormdataBlock->Value(i);

			  if (strcmp(val, VOID_STRING)) {
	        m_form_data += "=";
				  if (strcmp(val, EMPTY_STRING))
		        ParseCharFunc(val, m_form_data);
			  }
      }

      if (!m_form_data.isNull()) {
        m_content_type = URLENCODED_CONTENT_TYPE;
        return (0);
      }
    }
  }

	return 0;
}

// called for POST method only
int CWLHttpRequest::ParseDataBlocks()
{
	// We Init the data file name to null 
	// and the data length to zero (no data from file)
	m_dataFilename.remove(0);
	m_dataLength = 0;

	// if there is data block then we get its values into
	// form data structure
  RVCString FormattedValue;
	if (m_Ws->DataBlock  &&  m_Ws->DataBlock->entries() != 0) {

    m_form_data.remove(0);
		const char* value = m_Ws->DataBlock->Value(WLJSNAME_VALUE);
		if (!value)  {
			ConnectError(HTTP_ERROR_FILE, 45);
			return RET_ERROR;
		}
     EncodeString(value, FormattedValue, m_Ws->m_EncodeData);

	  //Galtech
	RVCString strValue = ConvertToServerEncoding(FormattedValue.data());

	// Chaim - Comverse - If the encoding is at low level, skip it here (so we can handle
	// nulls right at the very end
  m_form_data = strValue;

		// Set content type
		value = m_Ws->DataBlock->Value(WLJSNAME_TYPE);
		if (value)
			m_content_type = value;

	}
	// Else if we have data to send from file then we take the name and 
	// the type of the data and put it into the content type
	else if (m_Ws->DatafileBlock && m_Ws->DatafileBlock->entries() != 0) {

		const char* value = m_Ws->DatafileBlock->Value(WLJSNAME_FILENAME);
		if (!value)  {
			ConnectError(HTTP_ERROR_FILE, 90);
			return RET_ERROR;
		}

		// We Init the data file name to the value from workspace
		// open the file and add set the content type according to
		// the file content type

    //Concatinate LoadEngine temporary path name to the file name.
  	//Galtech: 
    m_dataFilename = CArrangeMachineTempFiles::GetWlTempFullPath (m_Cm->GetJScx() , WLECopiedFile, ToANSI(value)) ;
    TVEnvContainerClass tvpc;
		if (!tvpc.TVPFileExist(m_dataFilename.data())) {
			ConnectError(HTTP_ERROR_FILE, 91, check_null(m_dataFilename.data()));
			return RET_ERROR;
		}

		FILE* df = fopen(m_dataFilename.data(), "rb");
		if (!df) {
			ConnectError(HTTP_ERROR_FILE, 92, check_null(m_dataFilename.data()));
			return RET_ERROR;
		}

		// get file length
		fseek(df, 0, SEEK_END);
		m_dataLength = ftell(df);
		fclose(df);

		// Set content type
		value = m_Ws->DatafileBlock->Value(WLJSNAME_TYPE);
		if (value)
			m_content_type = value;
	}

	return RET_OK;
}

int CWLHttpRequest::Head_Get(HttpFuncs method)
{
  // Chaim
  RVCString buff((RVSize_T) 512);
//  RVCString buff("",512);
	int				ret;

  switch(method)
  {
  case HTTP_FUNCS_HEAD : buff = WLJSNAME_HTTP_HEAD; break;
  case HTTP_FUNCS_GET  : buff = WLJSNAME_HTTP_GET ; break;
  case HTTP_FUNCS_POST : buff = WLJSNAME_HTTP_POST; break;
  default:;
  }

	buff.toUpper();
  buff += " ";
  buff += m_uri;
  if (!m_form_data.isNull() && m_NumRedirections == 0) {
    buff += "?";
    buff += m_form_data;
  }
  buff += " ";
  buff += HTTPPROTOCOL;
  buff += m_Version == VERSION11 ? HTTP_VERSION_11 : HTTP_VERSION_10;
  buff += CRandLF;

	// Add headers to the buf
  buff += m_headers;

  // Chaim - another AJAX bug. Add content type from wlGlobals only for GET
  if (method == HTTP_FUNCS_GET && strlen(m_Cm->GetWorkspace()->m_sContentType) > 0){
	  buff += CS_HEADER_CONTENT_TYPE;
    buff += ": ";
    buff += m_Cm->GetWorkspace()->m_sContentType;
    buff += CRandLF;
  }

  buff += CRandLF;


  //add headers to callbackData object
  if (m_Cm->IsCallbackEnabled())
  {
    m_CBData.AddRequestHeader(buff.data());
  }

	// inform the connection object on start and end of send action
	CWLGeneralGuard guard(m_ConnPool);

	ret = send(buff.data(), buff.length(), m_protocol);
  return (ret);
}


// NOTE : this method is not used for redirections !
int CWLHttpRequest::Post(HttpFuncs method)
{
  RVCString buff;
  RVCString buffForCallabck;
	RVCString StrSize;
	int				ret = RET_OK ;

	buff = WLJSNAME_HTTP_POST;
	buff.toUpper();
  buff += " ";
	
  buff += m_uri;
  buff += " ";
  buff += HTTPPROTOCOL;
  buff += m_Version == VERSION11 ? HTTP_VERSION_11 : HTTP_VERSION_10;
  buff += CRandLF;

	// Add headers to the buf
  buff += m_headers;
  buffForCallabck = buff;

	// Build the content type and size according
	// to data block size or file size
  if ((m_form_data.isNull()) && (m_dataFilename.isNull()) && !m_pMultiPart) {
		//buff += CRandLF;

    //  [5/24/2005] Alon
    // The fix added Content-Length
    // to the header line with the length which is 0
    // when content length is missing it caused problems in some servers

    // Chaim - another AJAX bug. Because of Alon's fix, we didn't send the content type when the length was 0
    if (!m_content_type.isNull()) {
			buff += CS_HEADER_CONTENT_TYPE;
      buff += ": ";
      buff += m_content_type;
      buff += CRandLF;
      buffForCallabck = buff;
    }

    buff += CS_HEADER_CONTENT_LENGTH;
    buff += ": 0";
    buff += CRandLF;
    buff += CRandLF;
    buffForCallabck = buff;
	}
	else {
    if (!m_content_type.isNull()) {
			buff += CS_HEADER_CONTENT_TYPE;
      buff += ": ";
      buff += m_content_type;
      buff += CRandLF;
      buffForCallabck = buff;
    }
		buff += CS_HEADER_CONTENT_LENGTH;
    buff += ": ";
    if (m_pMultiPart)
    {
			char temp[64]="";
			itoa(m_pMultiPart->GetDataLength()-strlen(CRandLF), temp, 10);
			buff += temp;
      buff += CRandLF;
      buff += CRandLF;
      buffForCallabck = buff;
    }
		else if (m_dataFilename.isNull()) {
      //The content length is only the length of the data without the CrLf that we add later
      //to mark the end of the data block. This is how the browser does it 
      //in content-type=application/x-www-for-urlencoded and there are some applications
      //that expects this behavior.
      buff += IntToString(m_form_data.length(), StrSize);

      buff += CRandLF;
      buff += CRandLF;

      buffForCallabck = buff;

      buffForCallabck += CRandLF;
      buffForCallabck += CRandLF;

      buff += m_form_data;
      RVCString formattedFormData;
      FormatFormdataForDisplay(m_form_data, formattedFormData);
	    buffForCallabck += formattedFormData; // Add formatting
		}
		else 
    {
			char temp[64]="";
      //The content length is only the length of the data without the CrLf that we add later
      //to mark the end of the data block. This is how the browser does it 
      //in content-type=application/x-www-for-urlencoded and there are some applications
      //that expects this behavior.
			itoa(m_dataLength, temp, 10);
			buff += temp;
      buff += CRandLF;
      buff += CRandLF;
      buffForCallabck = buff;
		}
	}

  //add headers to callbackData object
  if (m_Cm->IsCallbackEnabled())
  {

    // Chaim - code for sending formdata to IDE. Actually, IDE gets it anyways and parses the data only
    // it misses the line with the formdata in it because it looks for a "\r\n" after each line and the formdata
    // isn't followed by this string. So I add it but to make sure I don't break the formatting of the
    // request that is sent to the server, I first make a copy for this formatting.

    buffForCallabck += CRandLF;
    buffForCallabck += CRandLF;

    buffForCallabck += CRandLF;
    buffForCallabck += CRandLF;
  }

	// inform the connection object on start and end of send action
	CWLGeneralGuard guard(m_ConnPool);
/*
	// first send the request and headers
	ret = send(buff.data(), buff.length(), m_protocol);
	if (ret != RET_OK)
		return ret;
*/	
	// If we dont have to send a file then we just return here
	// else we send the file data
	if (m_dataFilename.isNull()) 
  {
    if (m_pMultiPart)
    {
      ret = send(buff.data(), buff.length(), m_protocol);
      if (ret == RET_OK){
        ret = send(m_pMultiPart->GetData(), m_pMultiPart->GetDataLength(), m_protocol);
        if (m_Cm->IsCallbackEnabled())
        {
          RVCString formattedFormData = m_pMultiPart->GetData();
//          FormatFormdataForDisplay(m_pMultiPart->GetData(), formattedFormData);
	        buffForCallabck += formattedFormData; // Add formatting
          buffForCallabck += CRandLF;
          buffForCallabck += CRandLF;
        }
      }

      delete m_pMultiPart; m_pMultiPart = NULL ;
    }
    else {
		  ret = send(buff.data(), buff.length(), m_protocol);
    }
    if (m_Cm->IsCallbackEnabled()){
      m_CBData.AddRequestHeader(buffForCallabck.data());
		}
    return ret;
	}
	else 
  {
    //in any other case than this (multipart), we send first headers.
    if (!m_content_type.contains(URLENCODED_CONTENT_TYPE, RVCString::ignoreCase))
    {
    	ret = send(buff.data(), buff.length(), m_protocol);
      buff.remove(0);
	    if (ret != RET_OK)
		    return ret;
    }
		// send parts of the file
		FILE* df = fopen(m_dataFilename.data(), "rb");
		if (!df) {
			ConnectError(HTTP_ERROR_FILE, 92, check_null(m_dataFilename.data()));
			return RET_ERROR;
		}

		char* filebuff = NULL;
		size_t buffsize = 1;
    size_t fileLen = m_dataLength;
		// allocate memory for read from file, if could not allocate whole
    // buffer size, allocate half of it, and so on.
    while (!filebuff && buffsize > 0) 
    {
			buffsize = m_dataLength;
			filebuff = (char*)malloc(buffsize + 8);	// 8 extra for CRandLF
			m_dataLength /= 2;
		}

    bool allRead = false ;
    while ((ret == RET_OK) && !feof(df) && !ferror(df) && !allRead ) 
    {
			size_t nRead = fread(filebuff, 1, buffsize, df);
			if (ferror(df)) {
				ConnectError(HTTP_ERROR_FILE, 92, check_null(m_dataFilename.data()));
				return RET_ERROR;
			}
			else 
      {
        if (feof(df) || nRead == fileLen ) 
        {
					memmove(filebuff+nRead, CRandLF, strlen(CRandLF));
					nRead += strlen(CRandLF);
          allRead = true ; 
				}
        buff.append(filebuff,nRead) ;
				ret = send(buff.data(), buff.length(), m_protocol);
        buff.remove(0);
        if (m_Cm->IsCallbackEnabled())
        {
          buffForCallabck.append(filebuff,nRead) ;

          buffForCallabck += CRandLF;
          buffForCallabck += CRandLF;
        }
      }
      }
    if (m_Cm->IsCallbackEnabled()){
      m_CBData.AddRequestHeader(buffForCallabck.data());
		}
		free(filebuff);
		fclose(df);
		return ret;
  }
}

#define LENGTH_FOR_DISPLAY 256
void CWLHttpRequest::FormatFormdataForDisplay(RVCString formdata, RVCString& retBuff){
	char inBuff[5];
	char ch;
  for (int i=0;i<formdata.length();i++)
  {
    // Break long lines for display
//    if ((i % LENGTH_FOR_DISPLAY) == 0){
//      retBuff += CRandLF;
//      retBuff += CRandLF;
//    }
	  ch = formdata(i);
	  if ((ch>=0) && (ch<=31))
	  {
      // Chaim - I wanted to format this as %xx but for some reason this makes the display in IDE display it as the
      // actual non-print char instead.
		  sprintf(inBuff,"\\x%02x",ch);
		  for (int j=2; j<4; j++)
			  inBuff[j] = toupper(inBuff[j]);
		  retBuff += inBuff;
	  }
	  else
		  retBuff += ch;
  }
}

int CWLHttpRequest::Connect(const char* method)
{

  // Chaim
  RVCString buff((RVSize_T) 512);
//  RVCString buff("",512);
	RVCString Headers ("");
	char			lpszPort[32]="";
	int				ret;

	buff = method;
	buff.toUpper();
  buff += " ";
  buff += m_server_host;
  buff += ":";
	itoa(m_server_port, lpszPort, 10);
	buff += lpszPort;
  buff += " ";
  buff += HTTPPROTOCOL;

  buff += HTTP_VERSION_10;
  buff += CRandLF;

	// Add the connect headers to the buf
	ParseProxyHeader (Headers);
  buff += Headers;

  buff += CRandLF;

  //add headers to callbackData object
  if (m_Cm->IsCallbackEnabled())
  {
    m_CBData.AddRequestHeader(buff.data());
  }

	// inform the connection object on start and end of send action
	CWLGeneralGuard guard(m_ConnPool);

	ret = send(buff.data(), buff.length(), m_protocol);

  return (ret);
}


char* CWLHttpRequest::TraceRecvBuff(int Len, char* Buff)
{
  char FmtString[100] =""; 
  sprintf(FmtString,"===== Received data buffer:\n%%.%ds",  Len); 
  WlTraceMsg(FmtString, Buff);
  return "\n=== End buffer =====";
}

int CWLHttpRequest::ReadResponse(HttpFuncs method, double* procTime)
{
	// since 'recv_buff' is now allocated dynamically, if should be freed before each return (zion)
  int      responseSize = 0, SelectForReadRetValue, selectErrNum = 0, errNum = 0;
  int      cont_size = 0;
	int      contentLen = INIT_CONTENT_LENGTH;
  double   res_time = 0;
	double   processTime = 0;
	double   processTimeTotal = 0;
	bool     connClosed = false;
	int			 mode = m_Connection->SetSocketMode(NON_BLOCKING);	//set non-blocking socket mode

  double     ReceiveTimeout = (double) m_Ws->m_ReceiveTimeout/1000 ; //Timeout in seconds
  double     ReceiveStartTime ;
  int      SelectForReadRetries = 1;
  RVCString sBuffer;
	// inform the connection object on start and end of recv action
	CWLGeneralGuard guard(m_ConnPool);

	// Shahar - Add the ability to set the size of the buffer according
	// to the property for each request (was needed for SiteMinder bug)
  if (m_nRecvBufferSize != m_Ws->m_RecvBufferSize) {
		m_nRecvBufferSize = m_Ws->m_RecvBufferSize;
		if (m_pRecvBuff)
		{
			delete []m_pRecvBuff;
			m_pRecvBuff = NULL ;
		}
		m_pRecvBuff  = new char[m_nRecvBufferSize];
	}

	// Init the Response
	m_Response.Init(this);

	m_ResponseKeepAlive = false;

	// force entering calling the selectForRead (recv will be called only when there's data
	// available on the socket).
	errNum = EWOULDBLOCK;

  ReceiveStartTime = TVPSysTime();
  do {
    CHECK_ABORT

		if (errNum == EWOULDBLOCK) {

      if (SelectForReadRetries > 1) {
        if (SelectForReadRetries > m_Ws->m_SelectSwitchNum ) {
          m_Cm->sleep(m_Ws->m_SelectTimeout);
       	  WlTraceMsg("Select for read had waited for timeout=%d", m_Ws->m_SelectTimeout);
        }
        else {
          m_Cm->sleep(m_Ws->m_SelectSecondTimeout);
       	  WlTraceMsg("Select for read had waited for second timeout=%d", m_Ws->m_SelectSecondTimeout);
        }
      }

      if (SelectForReadRetries <= m_Ws->m_SelectSwitchNum) {
        WlTraceMsg("Activating Select for read for the %d time with time interval=%d",SelectForReadRetries,m_Ws->m_SelectTimeout);
        SelectForReadRetValue = selectForRead(&selectErrNum,m_Ws->m_SelectTimeout);
		}
		else {
        WlTraceMsg("Activating Select for read for the %d time with time interval=%d",SelectForReadRetries,m_Ws->m_SelectSecondTimeout);
        SelectForReadRetValue = selectForRead(&selectErrNum,m_Ws->m_SelectSecondTimeout);
      }
      WlTraceMsg("Select for read had returned with the following return code=%d and status=%d, read by now: %d bytes", SelectForReadRetValue, selectErrNum, cont_size);

      SelectForReadRetries++;
		}
		else {
			SelectForReadRetValue = 1;
			errNum = 0;
		}
    if (SelectForReadRetValue >= 1) {
      ReceiveStartTime = TVPSysTime();
      if (selectErrNum == -1)
      {
        break;
      }

	  CHECK_ABORT
      int len = m_nRecvBufferSize;
      int res = recv(m_pRecvBuff, &len, m_protocol, &connClosed, &errNum);

WlTraceMsg("recv returned %4d bytes, total: %8u, error: %d", len, cont_size, errNum);
WlTraceMsg(TraceRecvBuff((len==m_nRecvBufferSize?len-1:len), m_pRecvBuff));

      if (errNum != 0 && errNum != EWOULDBLOCK && errNum != EINTR) { //&& !connClosed) {
        if (responseSize+len == 0 && (errNum == WSAESHUTDOWN || errNum == WSAECONNRESET || errNum == SSL_ERROR_SSL)) {
					#ifdef _DEBUG
						addCounter("Retry fetch");
					#endif
					return RET_RETRY_FETCH;
  			}
//        if (m_Ssl && m_Ssl->SetError(errNum))
//          ConnectError(HTTP_ERROR_FILE, 50, check_null(m_Url.data()));
//        else
				{
					SetErrorCode(errNum);
	        ConnectError(HTTP_ERROR_FILE, 50, check_null(m_Url.data()));
				}
        return RET_ERROR;
      }

      if (len == 0 && (errNum == EWOULDBLOCK || errNum == EINTR ) )
				continue;

			if (len > 0) {
				responseSize += len;
				double tmpProcessTime = TVPSysTime();

				int errCode;
				int done = 0;
				CHECK_ABORT
				int res = m_Response.ParseResponse(m_pTransObj, m_Frame, (const char*)m_pRecvBuff, len, &done,
																	m_Cookies, method, m_Version,
																	m_Ws->doParse, m_Ws->m_SaveSource, m_Ws->m_SaveHeaders,
                                  m_Ws->m_ParseOnce,
																	m_server_host, m_server_uri,
																	m_UrlType, m_ThreadNum, m_Cm, m_ExplicitUrl, m_pCacheEntry, errCode, sBuffer);

				processTime = TVPSysTime() - tmpProcessTime;
				processTimeTotal += processTime;

				// If error was received then we send an error message
				// to user
				// if not then we check whetere we had some warning message in 
				// the error code and if so send the suitable message
				if (res == RET_ERROR) {
					if (errCode == 0)
						ConnectError(HTTP_ERROR_FILE, 52);
					else
						ConnectError(HTTP_ERROR_FILE, errCode);
					return RET_ERROR;
				}
				else {
					if (errCode != 0)
						ConnectError(HTTP_ERROR_FILE, errCode);
				}

				if (res == RET_HEAD_DONE)
					break;

				cont_size += done;
			}
    }
		else {	// SelectForReadRetValue <= 0
			if (selectErrNum == WSAECONNRESET) {
#ifdef _DEBUG
				addCounter("Retry fetch - ECONNRESET");
#endif
				return RET_RETRY_FETCH;
			}
      else if (SelectForReadRetValue < 0) { //Socket error
      	WlTraceMsg("Socket reading error, Return Code:%d, Error number:%d\n", SelectForReadRetValue, selectErrNum);
        ConnectError(HTTP_ERROR_FILE, 317, SelectForReadRetValue, selectErrNum);
        return RET_RETRY_FETCH;
      }
      else if (SelectForReadRetValue == 0) { //returned by Timeout
        if ((ReceiveStartTime + ReceiveTimeout) < TVPSysTime()) { //Timeout expired
       	  WlTraceMsg("SelectForRead Timeout: %d milliseconds" , m_Ws->m_ReceiveTimeout);
					ConnectError(HTTP_ERROR_FILE, 316, m_Ws->m_ReceiveTimeout);
					return RET_RETRY_FETCH;
        }
      }
		}
		contentLen = m_Response.GetContentLen();
    WlTraceMsg( "contentLen: %d", contentLen);

      // Chaim - FileMaker bug - server sends redirection without content-length (legal thing
    // to do). Content length is -1 and connection is still open (keep-alive!) so we
    // have to go through all the retries until we exit and actually perform the redirection.
    if ( (contentLen==INIT_CONTENT_LENGTH) && IS_REDIRECTION(m_Response.GetServerStatus()) && m_Response.m_bHeaderLocation )
		{
			WlTraceMsg( "INIT_CONTENT_LENGTH & redirection with location" );
      break;
    }

    // Chaim - not sure of this should be the final solution but this fixes the slowing
    // down problem caused by Zion's caching mechanism: the server returns 304 w/o content length
    // and since all this is done in Http 1.1, keeps the connection open and like the FileMaker
    // bug, we would loop and retry until finally closing the connection.
    if ( (contentLen==INIT_CONTENT_LENGTH) && m_Response.GetServerStatus() == 304){
			WlTraceMsg( "INIT_CONTENT_LENGTH & NOT MODIFIED response" );
      break;
    }

} while ((contentLen == INIT_CONTENT_LENGTH || cont_size < contentLen
          || (IS_REDIRECTION(m_Response.GetServerStatus()) && !m_Response.m_bHeaderLocation )) && !connClosed);
  
  CHECK_ABORT
  // support Gzip
  ZipTypeCode ZTC = m_Response.GetEZipCode();
  if ((ZTC==ZipCode_GZIP) || (ZTC==ZipCode_DEFLATE) || (m_Ws->m_bParseFullBuffer))
  {
    if (m_Frame && !sBuffer.isNull())
    {
      // Chaim - Prpearing unzipped content for IDE is handled in httpresponse. I think it's because zipped content is
      // also chunked and we should unzip afterwards but I'm not sure. Right now it's a good solution as most
      // zipped content is frames anyway (images and such are by their nature usually deflated)
      if (ZTC==ZipCode_GZIP)
      {
        WlTraceMsg("ReadResponse - Perform UnGzip");
        RVCString sUnzippedBuffer;
  	CWLGZip GZipObj;
        GZipObj.UNGzip (sBuffer, sUnzippedBuffer);
        m_Frame->parse(sUnzippedBuffer.data(), sUnzippedBuffer.length());
        if (m_Response.IsCallbackEnabled()){
          m_Response.SetOutputData(sUnzippedBuffer);
        }
      }
      else if (ZTC==ZipCode_DEFLATE)
      {
        WlTraceMsg("ReadResponse - Perform Inflate");
        RVCString sUnzippedBuffer;
        CWLGZip GZipObj;
        GZipObj.Inflate(sBuffer, sUnzippedBuffer);
        m_Frame->parse(sUnzippedBuffer.data(), sUnzippedBuffer.length());
        if (m_Response.IsCallbackEnabled()){
          m_Response.SetOutputData(sUnzippedBuffer);
        }
    }
      else //ParseFullBuffer
      {
        WlTraceMsg("ReadResponse - Copy full buffer");
        m_Frame->parse(sBuffer.data(), sBuffer.length());
      }
    }
  }

  //Patch for Direct Xml parsing -- Added by aviad
  if (m_UrlType == FRAME) 
      m_Frame->EndXmlDirectParsing() ; 

  WlTraceMsg( "ReadResponse, total of %d bytes", cont_size);

	m_ResponseKeepAlive = m_Response.GetKeepAlive();

	if (!connClosed) {
		m_Connection->SetSocketMode(mode);	// restore socket mode
	}
	else {
		m_ResponseKeepAlive = false;
		// If the socket was closed gracefully
		// during the receive - then we need to close the connection
    if (errNum == SSL_ERROR_ZERO_RETURN) {
			closeConnection(true);
		}
	}

	addTimer(PROCESS_TIME, processTimeTotal);
	addTimer("Receive Time", TVPSysTime() - m_dStartReceive);

	*procTime = processTimeTotal;

  return (responseSize);
}

int	CWLHttpRequest::ParseResponse (const char* recv_buff, 
																	 const int&  ReceiveSize, 
																	 HttpFuncs method,
																	 int&				 ContentCurrentLen,
																	 bool&			 Finished, 
																	 double&		 ProcessTime,
                                   RVCString&  sBuffer)
{
	int			 ParseStatus;
	int			 errCode;
	int			 ParseLen			  = 0;
	int			 ReturnStatus   = RET_OK;
	int      contentLen			= INIT_CONTENT_LENGTH;
	double	 tmpProcessTime = TVPSysTime();

	// We parse the received data and calculate process time for that
	// operation
	ParseStatus = m_Response.ParseResponse (m_pTransObj, 
                                          m_Frame, 
																					(const char*)recv_buff, 
																					ReceiveSize, 
																					&ParseLen, 
																					m_Cookies, 
																					method,
																					m_Version,
																					m_Ws->doParse, 
																					m_Ws->m_SaveSource, 
                                          m_Ws->m_SaveHeaders,
                                          m_Ws->m_ParseOnce,
																					m_server_host, 
																					m_server_uri,
																					m_UrlType, 
																					m_ThreadNum, 
																					m_Cm, 
                                          m_ExplicitUrl,
																					m_pCacheEntry,
																					errCode,
                                          sBuffer);

	ProcessTime += (TVPSysTime() - tmpProcessTime);
	
	// if we have some warning message in the error code 
	// then we send the suitable message.

	if (errCode != 0) {
			ConnectError(HTTP_ERROR_FILE, errCode);
	}

	// Here we are checking the parse status. Could be one of
	// The following:
	// OK - Get the content length if apears in message
	//      Check whether we have read all message
	// HEAD_DONE - If we stated only header command then we
	//             Parsing is stoped after reaching the body
	// Else (ERROR) - if the parsing operation produced an error 
	// then we send a general error message to user
	// and we stop the recieving and raise the error
	// to the activating function
	if (ParseStatus == RET_OK) {
		contentLen = m_Response.GetContentLen();
		ContentCurrentLen += ParseLen;
		if ((contentLen != INIT_CONTENT_LENGTH) && (ContentCurrentLen >= contentLen)) {
			Finished = true;
		}
	}
	else if (ParseStatus == RET_HEAD_DONE) {
		Finished = true;
	}
	else {
		if (errCode == 0) {
			ConnectError(HTTP_ERROR_FILE, 52);
		}
		ReturnStatus = RET_ERROR;
		Finished = true;
	}
	return ReturnStatus;
}


int CWLHttpRequest::GetConnTimeout()
{
////  int timeout = (int)myWorkSpace.GetDoubleValue(WL_CONN_TIMEOUT);
  int timeout = 30000;
  if (!timeout)
    timeout = CONNTIMEOUT;
  return (timeout);
}


int CWLHttpRequest::RedirectionHandler()
{
  int stat = m_Response.GetServerStatus();
  if (stat >= 300  &&  stat <= 305  &&  stat != 304) {

    // Chaim Apr 2001
//    if (m_Response.GetHeader(HEADER_LOCATION) == NULL)  {
    if (!m_Response.m_bHeaderLocation)  
    {
      return ( (ConnectError(HTTP_ERROR_FILE, 30)==RET_ERROR) ? RET_ERROR : RET_OK );
    }


    // Chaim Apr 2001
//    RVCString  location(m_Response.GetHeader(HEADER_LOCATION));
    RVCString  location(m_Response.m_HeaderLocationValue);
    m_Url = m_Base.calcUrl(location); // calculate it

    if (m_pTransObj->GetObjType() == IMAGE && m_Url.compareTo(m_Base.returnBase()) == 0){
      WlTraceMsg("Ending redirection because the server sent back the same location: %s", location.data());
      return RET_NONE;
    }
    if (!ShouldNavigate(m_Url, m_Ws->m_pcFilterUrl)){
      WlTraceMsg("Ending redirection because the url is filtered: %s", m_Url.data());
      return RET_NONE;
    }
    
    m_Base.setBase(m_Url);  // and set it as the new base
    return RET_OK;
  }
  else {
    return RET_NONE;
  }
}

//Galtech
#ifndef _ANSI_ONLY
int	CWLHttpRequest::GetServerEncoding()
{
	if (m_Frame)
		return m_Frame->GetCharset();
	else 
		return GetMonitorCharSet();
}

RVCString CWLHttpRequest::ConvertToServerEncoding(const char* str)
{
	return ToANSI(str, GetServerEncoding());
}
#endif

void CWLHttpRequest::GetHostAndUri(const RVCString& Url, RVCString& lhost, RVCString& luri, int& lport)
{
	RVCString	TempUrl;
	size_t		ProtocolPos = RV_NPOS;

  //Strip leading and trailing blanks and tabs (Aviad - 2/1/2000)
  TempUrl = Url.strip(RVCString::both, ' ') ;
  TempUrl = TempUrl.strip(RVCString::both, '\t') ;
  
 // Extract out the protocol from the url (if it exists in it)
	// First we check if we have the http protocol header and after words
	// we check if we have SSL enabled the appear of https protocol
	ProtocolPos = TempUrl.index(HTTP_STRING, 0, RVCString::ignoreCase);
	if (ProtocolPos == 0) {
		TempUrl = TempUrl (HTTP_STRING_LEN,TempUrl.length()-HTTP_STRING_LEN);
  }
#ifdef __OPEN_SSL__
  else {
		ProtocolPos = TempUrl.index(HTTPS_STRING, 0, RVCString::ignoreCase);
		if (ProtocolPos == 0) {
			TempUrl = TempUrl (HTTPS_STRING_LEN,TempUrl.length()-HTTPS_STRING_LEN);
		}
	}
#endif

  size_t u_loc = TempUrl.first('/');
  if (u_loc == RV_NPOS)
    u_loc = TempUrl.length();

  size_t p_loc = TempUrl.first(':');
  if (p_loc == RV_NPOS  ||  p_loc > u_loc)
    p_loc = u_loc;

  lhost = TempUrl(0, p_loc);
  
  if (p_loc != u_loc)  {
    RVCString p = TempUrl(p_loc + 1, u_loc - p_loc - 1);
    lport = atoi(p.data());
  }
  else if (lport != PROXY_PORT) {
		if (m_protocol == SSL_PROTOCOL)  {
			// If the protocol is SSL we set to the defualt port 443 
			lport = SSL_PORT;
		}
	}
	if (u_loc == TempUrl.length())
    luri = "/";
  else  // e.g u_loc != temp.length()
    luri = TempUrl(u_loc, TempUrl.length() - u_loc);
}


// Checking if the url is a http url, if it not return with
//  status -1.
int CWLHttpRequest::DetermineProtocol(RVCString& url, int *errnum)
{
  if (url.index(HTTP_STRING, 0, RVCString::ignoreCase) == 0) {
    m_protocol = HTTP_PROTOCOL;
    return (HTTP_STRING_LEN);
  }
#ifdef __OPEN_SSL__
  if (url.index(HTTPS_STRING, 0, RVCString::ignoreCase) == 0)  {
    m_protocol = SSL_PROTOCOL;
    return (HTTPS_STRING_LEN);
  }
#else
  if (url.index(HTTPS_STRING, 0, RVCString::ignoreCase) == 0)  {
    *errnum = 80;
    return RET_ERROR;
  }
#endif
  if (url.index(ABOUT_STRING, 0, RVCString::ignoreCase) == 0)  {
    m_protocol = HTTP_PROTOCOL;
    return RET_NONE;
  }
  if (url.index(JAVASCRIPT_STRING, 0, RVCString::ignoreCase) == 0)  {
    m_protocol = HTTP_PROTOCOL;
    return RET_NONE;
  }

  *errnum = 81;
  return RET_ERROR;
}

void parseUrl(RVCString& url)
{
  // exactly the same as stripUrl in htmlparse
  size_t i;

  while (url.length() > 0) {
    if (url(0) == ' ' || url(0) == '\t' || url(0) == '\n' || url(0) == '\r')
	  url.remove(0, 1);
	else
	  break;
  }

  while ((i = url.length() - 1) >= 0) {
    if (url(i) == ' ' || url(i) == '\t' || url(i) == '\n' || url(i) == '\r')
	  url.remove(i, 1);
	else
	  break;
  }

  for (i = 0; i < url.length(); ) {
    if (url(i) == '\n' || url(i) == '\r' || url(i) == '\t') {
//		if (atoi (url(i)) <= 31) {
  	  url.remove(i, 1);
	    continue;
  	}
    else if (url(i) == ' '){
	    url.replace(i, 1, "%20");
    }

    // Patch for &amp;. @gt; etc
    else if (url(i) == '&'){
      int leftLen = (url.length()-i -1);
      if (leftLen >= 4 && url(i + 1, 4) == "amp;")
        url.replace (i, 5, "&");
      else if (leftLen >= 3 && url (i + 1, 3) == "lt;")
        url.replace (i, 4, "<");
      else if (leftLen >= 3 && url (i + 1, 3) == "gt;")
        url.replace (i, 4, ">");
      else if (leftLen >= 4 && url (i + 1, 4) == "quot;")
        url.replace (i, 5, "\"");
    }

  	++i;	
  }
}

int CWLHttpRequest::ParseUri()
{
  RVCString  temp;
  int        offset;
  int        errnum;

  m_Url  = m_Url.strip(RVCString::both, ' ');
  offset = DetermineProtocol(m_Url, &errnum);

  if (offset == RET_ERROR) {
    ThreadError(HTTP_ERROR_FILE, errnum, check_null(m_Url.data()));
    return RET_ERROR;
  }
  else if (offset == RET_NONE) 
  {
    return RET_NONE;
  }

  const char *proxyToUse = NULL;
  if (m_protocol == SSL_PROTOCOL && (!m_Ws->m_UseSameProxyForSSL) )
  {
			proxyToUse = m_Ws->m_HttpsProxyName;
	}
  else
  {
   		proxyToUse = m_Ws->m_ProxyName;
  }
	
  
  if (proxyToUse && proxyToUse[0])  {
    m_usingProxy = true;
    m_port = PROXY_PORT;
	  GetHostAndUri(proxyToUse, m_host, m_uri, m_port);
    m_server_port = HTTP_PORT;
    GetHostAndUri(m_Url, m_server_host, m_server_uri, m_server_port);
		if (m_protocol == SSL_PROTOCOL) {
			m_uri = m_server_uri;
		}
		else {
			m_uri = m_Url;
		}
  }
  else  {
	  m_usingProxy = false;
    m_port = HTTP_PORT;
	  GetHostAndUri(m_Url, m_host, m_uri, m_port);
    m_server_host	= m_host;
    m_server_uri	= m_uri;
		m_server_port	= m_port;
  }

  return (RET_OK);
}


#ifdef __TV_WINDOWS__
AUTH_STATUS CWLHttpRequest::authorizationInit()
{
	if (!m_Authorization)
	{
		try{
			WlTraceMsg(" -------------------------------------- init");
			m_Authorization = new CWLClientAuthentication();
		}
		catch(AUTH_STATUS status){
			WlTraceMsg(" -------------------------------------- init failed");
			m_Authorization = NULL;
			return status;
		}
		catch(...){
			WlTraceMsg(" -------------------------------------- init failed");
			m_Authorization = NULL;
			return AUTH_FAILED;
		}

	}
	return m_Authorization ? AUTH_SUCCESS : AUTH_FAILED;
}

void CWLHttpRequest::authorizationReset()
{
	m_Iteration = 1;
	m_Phase = 1;
	m_Exhausted = false;

	// After we did the request we initialize the authentication headers
	// for the NT authentication because they are sent only epon specific
	// http authentication requests (not like the basic authentication...).
  if (m_Cm != NULL) {
		RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock);
		m_Cm -> m_AuthenticateNTHeaders = "";
		m_Cm -> m_AuthenticateNTProxyHeaders = "";
	}
}

void CWLHttpRequest::authorizationClean()
{
	if (m_Authorization) {
WlTraceMsg(" -------------------------------------- cleaning");
		m_Authorization->Clean();
	}
}

void CWLHttpRequest::authorizationTerminate()
{
	if (m_Authorization) {
WlTraceMsg(" -------------------------------------- terminate");
	/*	m_Authorization->Terminate();*/
		delete m_Authorization;
		m_Authorization = NULL;
	}
}


bool CWLHttpRequest::AuthorizationNTLM(int* err, const char* authValue, const char* username, const char* password, const char* Header, RVCString& Authentication)
{
	bool bRet = false;
  char val[2048] = "";
	const char* pcUsername = NULL ;
	const char* pcPassword = NULL ;
	const char* pcAuthData = NULL;
	bool more = false;

  if (m_Iteration == 1 && username && password ) 
  {
		pcUsername = username ;
		pcPassword = password ;
	}
	else 
  {
		m_Iteration = 2;
		pcUsername = "";
		pcPassword = "";
	}

	// check and update current phase
	if (m_Phase == 2 ) 
  {
		// second phase - get the random number value
		m_Phase = 1;
		if (m_Iteration == 1) 
    {
			m_Iteration = 2;
		}
		else 
    {
			m_Iteration = 1;
			m_Exhausted = true;
		}
		more = false;
		pcAuthData = authValue + strlen(TOKEN_NTLM) + 1;
    WlTraceMsg("Got authData: %s", pcAuthData);
	}
	else if (m_Phase == 1 ) 
  {
		// first phase - check for "WWW-Authenticate: NTLM" header
		m_Phase = 2;
	}
	else 
  {
		// unknown authorization response
		*err = 152;
		return false;
	}

	// set the additional headers for the next requset
  WlTraceMsg("Using username: %s, password: %s", pcUsername, pcPassword);
	if (m_Authorization->GetValue(val, TOKEN_NTLM, pcAuthData, pcUsername, pcPassword, &more)) 
  {
    WlTraceMsg("Authorization value: %s", val);
		Authentication += Header;
		Authentication += ": ";
		Authentication += val;
		Authentication += CRandLF;

		bRet = true;

		// clean authentication module when iteration has done
		if (m_Phase == 1)
    {
			authorizationClean();
    }
	}
	else 
  {
		// authorization value not generated
		*err = 153;
		bRet = false;
	}
  return bRet;
}

#endif //__TV_WINDOWS__


int CWLHttpRequest::AuthorizationInitialization(const char*& authValue, bool* exhaust, int* err)
{
  int nRet  = AUTH_NONE;
  
  // Chaim Apr 2001
//	  authValue = m_Response.GetHeader(HEADER_AUTHENTICATE);
  authValue = m_Response.m_HeaderAuthenticateValue.data();
  
  // Chaim Apr 2001
//  if (!authValue)
  if (!m_Response.m_bHeaderAuthenticate){
//  authValue = m_Response.GetHeader(HEADER_PROXY_AUTHENTICATE);
    authValue = m_Response.m_HeaderProxyAuthenticateValue.data();

    // Chaim Apr 2001
    if (!m_Response.m_bHeaderProxyAuthenticate){
      //  if (!authValue)
		  // unknown authorization response
		  *err = 152;
		  return nRet;
    }
  }

#ifdef __TV_WINDOWS__

	if (m_Exhausted) {
		// authentication exhausted in previous negotiation
		m_Exhausted = false;
		*exhaust = true;
		return nRet;
	}

	// initial state - reset m_Exhausted
	if (m_Iteration == 1 && m_Phase == 1)
		m_Exhausted = false;

	AUTH_STATUS status = authorizationInit();
	if (status == AUTH_NOT_INIT) {
		// Cannot initialize security module
		*err = 150;
		return nRet;
	}
	else if (status == AUTH_FAILED) {
		// Cannot load security module
		*err = 151;
		return nRet;
	}
#endif //__TV_WINDOWS__

  RVCString sAuth(authValue);

  if (sAuth.contains(TOKEN_NTLM, RVCString::ignoreCase))
    nRet = AUTH_NTLM ;
  if (sAuth.contains(TOKEN_BASIC, RVCString::ignoreCase))
  {
    if (nRet == AUTH_NTLM )
      nRet = AUTH_BASIC_NTLM ;
    else
      nRet = AUTH_BASIC ;
  }

  return nRet ;
}


bool CWLHttpRequest::AuthorizationBASIC(const char* username, const char* password, const char* Header, RVCString& Authentication)
{
  RVCString  temp;
	RVCString	 encodeUsernamePassword;
  bool bRet = false ;

  if (username && *username && password && *password ) 
  {
		temp  = username;
		temp += ":";
		temp += password;
		HTTPEncode(temp, encodeUsernamePassword);
		Authentication += Header;
		Authentication += ": ";
		Authentication += BASIC_AUTHENTICATION;
		Authentication += " ";
		Authentication += encodeUsernamePassword;
		Authentication += CRandLF;
    bRet = true ;
	}
	else 
  {
		if (username && *username && (!password || !(*password))) 
    {
			ConnectError(HTTP_ERROR_FILE, 97);//proxy 98
		}
		else if (password && *password && (!username || !(*username))) 
    {
			ConnectError(HTTP_ERROR_FILE, 96);//proxy 99
		}
	}

  return bRet ;
}


/*********************************************************************
  AuthorizationHandler : 
  handling proxy and not proxy NTLM Authorization.
*********************************************************************/
int CWLHttpRequest::AuthorizationHandler(bool* exhaust, int* err)
{
	bool bRet = false;
	int  RetVal = C_No_Authentication;

	*err = 0;
	*exhaust = false;

  char val[2048] = "";
	bool more = false;
	const char* username = NULL;
	const char* password = NULL;
	const char* authData = NULL;
	const char* authValue = NULL ;
  int nAuth;


  switch (m_Response.GetServerStatus())
  {
    case 401 :
	  {
				nAuth = AuthorizationInitialization(authValue, exhaust, err);

				switch (nAuth)
				{
					case AUTH_BASIC:
					{	
						{
							RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock);
							m_Cm -> m_AuthenticateBasicHeaders.remove(0);
							bRet = AuthorizationBASIC(m_Ws->m_Username, m_Ws->m_Password, CS_HEADER_AUTHORIZATION, m_Cm -> m_AuthenticateBasicHeaders);
						}
						if (bRet) { RetVal = C_Server_Basic_Authentication;}
						m_bAuthDone = true ;
						m_bPAuthDone = true ;
						break;
					}

#ifdef __TV_WINDOWS__ //NTLM only in windows

					case AUTH_BASIC_NTLM:
					case AUTH_NTLM:
					{
						{
							RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock);
							m_Cm -> m_AuthenticateNTHeaders.remove(0);
							bRet = AuthorizationNTLM(err, authValue, m_Ws->m_NTUsername, m_Ws->m_NTPassword, CS_HEADER_AUTHORIZATION, m_Cm -> m_AuthenticateNTHeaders);
						}
						if (bRet) { RetVal = C_Server_Nt_Authentication;}
						if (m_Phase == 1)
						{
							m_bAuthDone = true ;
							m_bPAuthDone = true ;
						}
					}
#endif

					default: break;
				}
				break;
    }

    case 407 : //Proxy
	  {
				nAuth = AuthorizationInitialization(authValue, exhaust, err);

				switch (nAuth)
				{
					case AUTH_BASIC:
					{
						{
							RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock);
							m_Cm -> m_AuthenticateBasicProxyHeaders.remove(0);
						}
            const char *basicUserName = NULL;
            const char *basicPassword = NULL;
            if (m_protocol == SSL_PROTOCOL && (!m_Ws->m_UseSameProxyForSSL) )
            {
              basicUserName = m_Ws->m_HttpsProxyUsername;
              basicPassword = m_Ws->m_HttpsProxyPassword;
            }
            else
            {
              basicUserName = m_Ws->m_ProxyUsername;
              basicPassword = m_Ws->m_ProxyPassword;
            }

			{            
				RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock);
				bRet = AuthorizationBASIC(basicUserName, basicPassword, CS_HEADER_PROXY_AUTHORIZATION, m_Cm -> m_AuthenticateBasicProxyHeaders);
			}
						if (bRet) { RetVal = C_Proxy_Basic_Authentication;}
						m_bPAuthDone = true ;
						break;
					}

#ifdef __TV_WINDOWS__ //NTLM only in windows

					case AUTH_BASIC_NTLM:
					case AUTH_NTLM:
					{
						{
							RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock);
							m_Cm -> m_AuthenticateNTProxyHeaders.remove(0);
						}
            const char *ntlmUserName = NULL;
            const char *ntlmPassword = NULL;
            if (m_protocol == SSL_PROTOCOL && (!m_Ws->m_UseSameProxyForSSL) )
            {
              ntlmUserName = m_Ws->m_HttpsProxyNTUsername;
              ntlmPassword = m_Ws->m_HttpsProxyNTPassword;
            }
            else
            {
              ntlmUserName = m_Ws->m_ProxyNTUsername;
              ntlmPassword = m_Ws->m_ProxyNTPassword;
            }
			{
				RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock);
				bRet = AuthorizationNTLM(err, authValue, ntlmUserName, ntlmPassword, CS_HEADER_PROXY_AUTHORIZATION, m_Cm -> m_AuthenticateNTProxyHeaders);
			}
						if (bRet) { RetVal = C_Proxy_Nt_Authentication;}
						if (m_Phase == 1)
						{
							m_bPAuthDone = true ;
						}
					}
#endif

					default: break;
				}
				break;
    }

    default: break;
  }

  return RetVal;
}


int CWLHttpRequest::FetchUrl(HttpFuncs method, bool inRetry)
{
  // Reading the response.
	// For New response  int			responseStatus;
  int			responseSize;
	double	responseTime;
	double	processTime;
  bool    keptAlive = false ;

	// build all parts of the request
  if (ParseHeader() == RET_ERROR)  // handle the Header block if there is one.
    return RET_ERROR;

	// add conditional request headers (currently, relevant for GET only)
	if ( method == HTTP_FUNCS_GET && m_pCacheEntry )
		m_pCacheEntry->GenerateConditionalHeaders( m_Version, m_headers );

  if (!inRetry) 
  {
    m_form_data.remove(0);

	if (m_NumRedirections == 0) {	// must not add query string when redirecting.
		if (ParseFormData() == RET_ERROR)	// handle the FormData (search-string for GET/HEAD) block if exist
			return RET_ERROR;
    }
  }

  if (m_NumRedirections == 0){
	// Chaim - remove the "if". There is no reason not to allow the user to manually code the use of Data block in Get,
	// even though we don't record it.
//		if (method == HTTP_FUNCS_POST )
			ParseDataBlocks();
	}

	// m_RequestKeepAlive is updated in ParseHeader
  // Checkpoint request - allow user intervention in Keep-Alive
  // Chaim - LSE bug fix - close connection if protocl, has changed and we are
  // using proxy
  if (!m_RequestKeepAlive ||
      (m_usingProxy && m_previousProtocol != UNKNOWN_PROTOCOL && m_previousProtocol != m_protocol)){
		closeConnection(true);
  }
  else {
    if (m_Ws->m_Persistancy != IGNORE_PERSISTANCY){
      if (m_protocol != SSL_PROTOCOL && !m_DuringChallengeResponse && m_Cm->GetRequestOnConnectionCounter() >= m_Ws->m_Persistancy ){
        WlTraceMsg("Persistancy limit reached, closing connection after %d requests", m_Cm->GetRequestOnConnectionCounter());
        // Counter is reset in CWLHttpRequest::closeConenction
//        m_Cm->ResetRequestOnConnectionCounter();
        closeConnection(true);
      }
      else {
        m_Cm->IncrementRequestOnConnectionCounter();
        WlTraceMsg("Request %d on open connection", m_Cm->GetRequestOnConnectionCounter());
      }
    }
  }

  m_previousProtocol = m_protocol;

	//get a connection(socket) object
	double connectTime = TVPSysTime();
  
  // the SetFirstTimeWait is a patch and is made because that in OpenConnection in SSL,
  // read is being made, so the 'Wait' and 'Receive' time are being corrupted in SocketRead
  SetFirstTimeWait(false);

  int ret = openConnection();

  SetFirstTimeWait(true);

  if (ret == RET_ERROR || ret == RET_RETRY_FETCH) {
		closeConnection(true);
    addCounter(CONNECTION_TRIALS);
    addCounter(FAILED_CONNECTION);
#ifdef _DEBUG
		addTimer("Failed Connection Time", TVPSysTime() - connectTime);
#endif
    return ret;
  }
  else if (ret == IS_ABORT)	// isAbort retured TRUE
  {
    return IS_ABORT;
  }


  if (ret == KEPT_ALIVE ) 
  {
    keptAlive = true ; 
    // keep-alive, so connect time should be 0
    addTimer(CONNECT_TIME, 0);
  }

  // If a new connection was created we report a timer on it
  if ( !keptAlive || m_DuringChallengeResponse ) 
  {	
    if (!keptAlive)
    {
      addCounter(CONNECTION_TRIALS);
      addCounter(SUCCESS_CONNECTION);
    }
		// if we are using proxy and ssl then we issue the special connect command
		// and check for the status received from this command.
		// We replace for that period of time the protocol to a HTTP PROTOCOL
		// and then set it back to SSL PROTOCOL
	if( m_usingProxy && ( m_protocol == SSL_PROTOCOL ) ){
			// Activate the connect command and check the response


		// store current SSL connection in temporary variable
		m_TmpSslConnection = m_Connection;

			m_protocol = HTTP_PROTOCOL;
		// create the insecure (TCP) connection which uses the same socket as existing SSL connection
		m_Connection = new CWLInsecureConnection( m_TmpSslConnection );
		if( NULL == m_Connection ){
			m_Connection = m_TmpSslConnection;
m_protocol = SSL_PROTOCOL;
			return RET_ERROR;
		}

			ret = Connect (WLJSNAME_HTTP_CONNECT);
			if (ret == RET_ERROR)
				return RET_ERROR;
			// Call New Read response 
			// responseStatus = ReadResponse(WLJSNAME_HTTP_HEAD, processTime, responseSize);
			// if (responseStatus != RET_OK)
		  //  return responseStatus;

			responseSize = ReadResponse(HTTP_FUNCS_HEAD, &processTime);

      // Chaim
      if (m_Cm->IsCallbackEnabled())
        m_CBData.AddResponseHeader(m_Response.GetAndResetResponseHeaderString().data());

		if( ( responseSize == IS_ABORT ) || ( responseSize == RET_ERROR ) || ( responseSize == RET_RETRY_FETCH ) ){
			m_TmpSslConnection->SetTransactionData( m_Connection->GetTransactionData() );
			m_Connection->setSocket( -1 );
			delete m_Connection; // insecure (TCP) connection
			m_Connection = m_TmpSslConnection;
			m_TmpSslConnection = NULL;
m_protocol = SSL_PROTOCOL;
				return responseSize;
      }

		if (m_Response.GetServerStatus() == 407 ){
			m_TmpSslConnection->SetTransactionData( m_Connection->GetTransactionData() );
			m_Connection->setSocket( -1 );
			delete m_Connection; // insecure (TCP) connection
			m_Connection = m_TmpSslConnection;
			m_TmpSslConnection = NULL;
        return RET_OK ; 
      }
      
		if( m_Response.GetServerStatus() == 404 ){
			m_TmpSslConnection->SetTransactionData( m_Connection->GetTransactionData() );
			m_Connection->setSocket( -1 );
			delete m_Connection;
			m_Connection = m_TmpSslConnection;
			m_TmpSslConnection = NULL;
			WlTraceMsg( "The requested URL could not be retrieved by proxy server." );
			return RET_ERROR;
      }

			m_protocol = SSL_PROTOCOL;
		// Return the SSL connection
		if( NULL != m_Connection && NULL != m_TmpSslConnection ){
			m_TmpSslConnection->SetTransactionData( m_Connection->GetTransactionData() );
			m_Connection->setSocket( -1 );
			delete m_Connection; // insecure (TCP) connection
			m_Connection = m_TmpSslConnection;
			m_TmpSslConnection = NULL;

			m_Connection->SetSocketMode( BLOCKING );
			if( RET_OK != m_Connection->OpenConnection( m_host, m_port, m_usingProxy, m_ThreadNum  ) )
				return RET_ERROR;
		}
		else{
			if( m_Connection ){
				m_Connection->setSocket( -1 );
				delete m_Connection; // insecure (TCP) connection
			}
			m_TmpSslConnection = NULL;
			ret = acquireConnection();
			if( RET_OK != ret )
				return ret;
		}
		}


    if (!keptAlive) {
      addTimer(CONNECT_TIME, TVPSysTime() - connectTime);
    }
	}

	int err = RET_OK;
  if ( method == HTTP_FUNCS_POST) {
		err = Post(HTTP_FUNCS_POST);
	}
	else if ((method == HTTP_FUNCS_HEAD) || 
					 (method == HTTP_FUNCS_GET)) {
		err = Head_Get(method);
	}
	if (err != RET_OK) 
		return err;
  
  responseSize	= 0;
	responseTime	= TVPSysTime();
	processTime		= 0;

// call New read response
//	responseStatus = ReadResponse(method, processTime, responseSize);
//  if (responseStatus != RET_OK)
//    return responseStatus;

  responseSize = ReadResponse(method, &processTime);

  // Chaim - for some reason, if this isn't performed, we have a leak
//  if (m_Cm->IsCallbackEnabled())
  m_CBData.AddResponseHeader(m_Response.GetAndResetResponseHeaderString().data());

  if (m_pTransObj)
  {
    m_pTransObj->GetLastSubTransation()->SetEndTransaction(TVPSysTime(), WLLogMessage::Success);//Receive
    if (responseSize == 0)
    {
      m_Connection->SetTransactionData(m_pTransObj->GetLastSubTransation());     
      ConnectError(HTTP_ERROR_FILE, 110);
      m_Connection->SetTransactionData(NULL);
    }
  }

  if ((responseSize == IS_ABORT) ||
			(responseSize == RET_ERROR) ||
			(responseSize == RET_RETRY_FETCH))
    return responseSize;

	// add response timers
	addTimer(RESPONSE_TIME, TVPSysTime() - responseTime - processTime);
  addAccumulator(RESPONSE_SIZE, responseSize);
  addCounter(RESPONSES_COUNT);
//  m_Cm->FlushStatsCache();

	int stat = m_Response.GetServerStatus();
	if (stat != INIT_STATUS) {
		char replyStatus[32]="";
		sprintf(replyStatus, "%s %d", HTTP_STATUS, stat);
		addCounter(replyStatus);
	}
#ifdef _DEBUG
	else {
		char replyStatus[32]="";
		sprintf(replyStatus, "Debug only ! %s %d", HTTP_STATUS, stat);
		addCounter(replyStatus);
	}
#endif

  return RET_OK;

}

bool CWLHttpRequest::CacheHandler( HttpFuncs p_eMethod, RVCString& p_sUrl, HtmlQueueItemType p_uObjType, bool& p_bFreshObject, bool& p_bPartialImage )
{
	m_pCacheEntry = NULL;
	CWLHttpCache& pCache = getHttpCache();
	// currently (maybe for good), don't cache POST replied objects
	const char* psUrl = p_sUrl.data();
	if ( m_Ws->m_HttpCacheType != HTTP_CACHE_TYPE_NONE && p_eMethod == HTTP_FUNCS_GET && pCache.IsCachedType( p_uObjType, psUrl ) )
	{
		// check for cached object
		RVCollectableString* psKey = new RVCollectableString( psUrl );
		RVCollectableString* psReferer = new RVCollectableString( GetReferer() );

		unsigned uCacheEntryStatus;
		m_pCacheEntry = pCache.CreateNewEntry( m_Cm->GetRoundNumScriptInSlave(), psKey, psReferer, p_uObjType, uCacheEntryStatus );
		if ( m_pCacheEntry )
		{
			if ( uCacheEntryStatus == CACHE_ENTRY_FOUND )
			{
				//WlTraceMsg("HTTP cache: got cache entry: %s, %x", psKey->data(), m_pCacheEntry );
				// Chaim - for image, check if we're on the same page using referers and if so, skip navigation
				// @@Chaim - note that isrefresh always returns false right now since expiration date is never update (updateCachedHeadersAndContent is never called)
				if ((p_uObjType  == IMAGE && !m_pCacheEntry->GetReferer().compareTo(psReferer, RVCString::ignoreCase)) || m_pCacheEntry->IsFresh( m_Ws->m_HttpCacheType ) )
				{
					p_bFreshObject = true;
					WlTraceMsg( "HTTP cache: object %s is fresh in cache, skip", psKey->data() );
				}
				else if ( !m_pCacheEntry->IsReady() )
				{
					// if a cache entry exists but not ready (currently retrieved by another thread),
					// it is retrieved by the current thread also, but not saved
					m_pCacheEntry = NULL;
					WlTraceMsg( "HTTP cache: object %s exist but not ready, ignore caching mechanism", psKey->data() );

					// image that is partially exist will not be fetched
					if ( p_uObjType == IMAGE )
						p_bPartialImage = true;
				}
				else
					WlTraceMsg( "HTTP cache: object %s needs to be validated", psKey->data() );

				// Chaim - update refer in cache
				if ((p_uObjType  == IMAGE && m_pCacheEntry->GetReferer().compareTo(psReferer, RVCString::ignoreCase)) /*|| m_pCacheEntry->IsFresh( m_Ws->m_HttpCacheType )*/ )
				{
					m_pCacheEntry->SetReferer(psReferer->data());
					pCache.UpdateCacheEntry(m_pCacheEntry);
					WlTraceMsg( "HTTP cache: update refere for object %s ", psKey->data() );
				}
			}
			else // if ( uCacheEntryStatus == CACHE_ENTRY_NEW )
			{
				WlTraceMsg( "HTTP cache: object %s typed %d needs to be fetched", psKey->data(), p_uObjType );
			}
		}
		delete psKey;
	}
	else
		WlTraceMsg( "HTTP cache: non cachable object: %s, type: %d", psUrl, p_uObjType );


	//WlTraceMsg("cache size: %d, content total size: %u", getHttpCache().entries(), /*getHttpCache().ContentSize(), */getHttpCache().GetTotal() );

	return true;
}

int CWLHttpRequest::DoHttpCommand( HttpFuncs method, HtmlQueueItemType p_uObjType )
{
	bool bFreshObject = false;
	bool bPartialImage = false;

	int retStatus = RET_OK;
	int authErr = 0;
	int AuthStatus = C_No_Authentication;
	int AuthServerCounter=0;
	int AuthProxyCounter=0;
	bool authenticationExhausted = false;

  m_NumRedirections = 0;

	// loop on redirections and authentications
	do {
    if ((retStatus = ParseUri()) == RET_ERROR)  {
			retStatus = RET_ERROR;
			break;
		}
    else if (retStatus == RET_NONE)
    {
      retStatus = RET_OK;
      break;
    }

		double transactionTime = TVPSysTime();

		if (acquireConnection() != RET_OK) {
			return RET_ERROR;
		}
#ifdef _DEBUG
		WlTraceMsg("Acquired connection %d, ip:%s, m_port:%u\n", m_Connection->m_Index, m_Connection->ip(), m_Connection->port());
#endif
		if (m_ThreadNum == 0 && m_Cm->srcIndex == 0 && m_Ws->m_OutFilename && m_Ws->m_OutFilename[0])  {
      //Concatinate LoadEngine temporary path name to the file name.
  	  //Galtech:
      RVCString leOutFilename = CArrangeMachineTempFiles::GetWlTempFullPath (m_Cm->GetJScx(), WLEOutputFile, ToANSI(m_Ws->m_OutFilename)) ;
			if (m_Response.setOutput(leOutFilename) == RET_ERROR)  {
			  const char* fn =(const char *)leOutFilename;
				ConnectError(HTTP_ERROR_FILE, 68, fn);
				return RET_ERROR;
			}
		}
		else
			m_Response.resetOutput();
    
    if (m_Cm->IsCallbackEnabled())//reset & set output of m_Response
      m_Response.SetOutputData();

    if (/*m_NumRedirections > 0 && */m_UrlType == FRAME)
			m_Frame->redirectionHandler(m_Url);

		int err;
		int retryNum = 0;
		do {

			// cache handler
			// =============
			CacheHandler( method, m_Url, p_uObjType, bFreshObject, bPartialImage );

			// break loop on cached resources
			if ( bFreshObject )
			{
				// need to retrieve object content from cache
				err = retrieveAndSetCachedContent( m_pCacheEntry );
				// Chaim - for images, don't do anything otherwise IDE will still take a long time to process repeated images
				break;
			}
			else if ( bPartialImage )
			{
				// subject to change: don't bother with an image requested by another thread, it'll be available later
				err = RET_OK;
				break;
			}
			else
			{
				// need to retrieve object content from server
				if ( m_pCacheEntry ){
					m_pCacheEntry->ContentBegin();
				}

      	err = FetchUrl(method, (retryNum > 0));

			  if ( m_pCacheEntry && IS_REDIRECTION( m_Response.GetServerStatus() ) )
				{
					m_pCacheEntry = NULL;
				}

				if ( m_pCacheEntry )
					m_pCacheEntry->ContentDone();
			}

      if (err == RET_RETRY_FETCH) {
				retryNum++;
        if (retryNum < (m_Ws->m_RequestRetries +1)) {
       	  WlTraceMsg("Server not responding, retrying (%d)... - %s", retryNum, (const char *)m_Url);
				  ConnectError(HTTP_ERROR_FILE, 314, retryNum, (const char *)m_Url);
  			  closeConnection(true);
        } 
//				releaseConnection();
//				if (acquireConnection() != RET_OK) {
//					err = RET_ERROR;
//				}
			}
		} while (err == RET_RETRY_FETCH && retryNum < m_Ws->m_RequestRetries +1);

    //For Fix04
    //The loop will be exited with RET_RETRY_FETCH only when all retries failed.
    if (err == RET_RETRY_FETCH) {
     	WlTraceMsg("%d Retries failed", m_Ws->m_RequestRetries);
			ConnectError(HTTP_ERROR_FILE, 315, m_Ws->m_RequestRetries);
			err = RET_ERROR;
    }

//		if (err == RET_ERROR || err == RET_RETRY_FETCH || err == IS_ABORT) {
    //RET_RETRY_FETCH will always be converted to RET_ERROR
    if (err == RET_ERROR || err == IS_ABORT) {
			closeConnection(true);
      if (!bFreshObject && ! bPartialImage) addCounter(FAILED_TRANSACTIONS);
		}
		else {
			closeConnection();
      if (!bFreshObject && ! bPartialImage) addCounter(SUCCESS_TRANSACTIONS);
		}
		addTimer(TRANSACTION_TIME, TVPSysTime() - transactionTime);
    if (!bFreshObject && ! bPartialImage) addCounter(TOTAL_TRANSACTIONS);

		if (err == RET_ERROR) {
			retStatus = RET_ERROR;
			break;
		}

		if (m_Cm->isAbort())
			return IS_ABORT;

		// headers does not exist on cached resources -> break loop
		if ( bPartialImage || bFreshObject )
			break;

		//====================================================================

		// check for Authentication response
		authErr = 0;
		{
			RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock); 
			m_Cm -> m_AuthenticateNTHeaders.remove(0);
			m_Cm -> m_AuthenticateNTProxyHeaders.remove(0);
		}
		AuthStatus = C_No_Authentication;
		AuthStatus = AuthorizationHandler(&authenticationExhausted, &authErr);		

		if (((AuthStatus == C_Server_Basic_Authentication) && (AuthServerCounter < 1)) || 
		    ((AuthStatus == C_Server_Nt_Authentication)    && (AuthServerCounter < 3)) || 
			  ((AuthStatus == C_Proxy_Basic_Authentication)  && (AuthProxyCounter  < 1)) ||
			  ((AuthStatus == C_Proxy_Nt_Authentication)     && (AuthProxyCounter  < 3))) {

      

			if(  (AuthStatus == C_Server_Nt_Authentication) 
        || (AuthStatus == C_Server_Basic_Authentication)){
        AuthServerCounter++;
      }

			if(  (AuthStatus == C_Proxy_Nt_Authentication)  
        || (AuthStatus == C_Proxy_Basic_Authentication)){
        AuthProxyCounter++;
      }

			m_DuringChallengeResponse = true;
			continue;
		}
		else {
#ifdef __TV_WINDOWS__
			authorizationReset();
			authorizationClean();
#endif
      m_bAuthDone = false ;
      m_bPAuthDone = false ;
			m_DuringChallengeResponse = false;
			authenticationExhausted = true;
			// this one fixes a bug that caused premature authentication abort when after authentication
			// and redirection another authentication was needed (401, 401, 302, 401, 401...)
			AuthServerCounter = 0;
			AuthProxyCounter = 0;

			if (authErr != 0) {
				{
					RVCriticalSection::LockGuard lock(*m_Cm->m_SharedDataLock); 
					m_Cm->m_AuthenticateBasicHeaders.remove(0);
					m_Cm->m_AuthenticateNTHeaders.remove(0);
					m_Cm->m_AuthenticateBasicProxyHeaders.remove(0);
					m_Cm->m_AuthenticateNTProxyHeaders.remove(0);
				}

				ConnectError(HTTP_ERROR_FILE, authErr, check_null(m_Url.data()));
				return RET_ERROR;
			}
		}
		
		int nStatus = m_Response.GetServerStatus();
		//if (m_Response.ErrorHandler(&sever, m_Url) == -1)  {

		if ( nStatus == 304 )
		{
			// need to retrieve the object content from the cache
			if ( m_pCacheEntry )
				retrieveAndSetCachedContent( m_pCacheEntry );
		}
    else if ( nStatus >= 300 && nStatus <= 600 && m_Ws->m_bReportHttpErrors )
		{
      if ( ConnectError( RESPONSE_FILE, nStatus, check_null( m_Url.data() ) ) == RET_ERROR )
				return RET_ERROR;
		}

		//====================================================================

    int redirected = RedirectionHandler();
    if (redirected == RET_ERROR) 
    {
      retStatus = RET_ERROR;
			m_NumRedirections = 0;
      break;
    }
		else if (redirected == RET_OK) 
    {
			m_NumRedirections++;
      if (m_NumRedirections > m_Ws->m_RedirectionLimit && m_Ws->m_RedirectionEnabled)
				ConnectError(HTTP_ERROR_FILE, 130, check_null(m_Url.data()));
			releaseConnection();
			// method might be POST for an explicit script command only !
			// POST turns into GET for redirections
			if (method == HTTP_FUNCS_POST )
				method = HTTP_FUNCS_GET;
		}
		else
			m_NumRedirections = 0;
  } while ((m_Ws->m_RedirectionEnabled && m_NumRedirections > 0 && m_NumRedirections <= m_Ws->m_RedirectionLimit) || !authenticationExhausted);

  if (m_NumRedirections > m_Ws->m_RedirectionLimit && m_Ws->m_RedirectionEnabled)
		return RET_ERROR;

	return (retStatus);
}


void CWLHttpRequest::initLocalVariables()
{
	m_Version = (strcmp(m_Ws->m_Version, HTTP_VERSION_11) == 0) ? VERSION11 : VERSION10;
	m_DuringChallengeResponse = false;
}

void CWLHttpRequest::SlaveThread()
{
	CTraceMsg::RegisterThread (m_Cm ->GetLGThreadNum());
	try {
		startSlave();
	}
	catch (...) {
WlTraceMsg("Thread blown away !!!");
		char message[1024]="";
		WLERRGetErrorString( HTTP_ERROR_FILE, 210, message, 1024, (const char*)m_Url.data() );
		m_Cm->reportMultiThread( WLERRGetErrorSeverity( HTTP_ERROR_FILE, 210 ), message );

		m_Cm->UrlQueue()->terminateAllThreads();
	}
}

void CWLHttpRequest::startSlave()
{
#ifdef __TV_WINDOWS__
  WlTraceMsg("Slave thread : %x", GetCurrentThreadId());
#endif
  // wait on queue
	int retStatus = 0;
	m_Path = "";	// for creating dom from files

	do {
    int ret = CONTINUE;
		HtmlQueueItemType urlType = m_Cm->UrlQueue()->slaveRead((RVCollectable**)&m_pTransObj);
    if (!m_pTransObj)
      break;

    m_pTransObj->GetTransaction()->SetBeginTransaction(TVPSysTime());
    ret = CondDoCommand(HTTP_FUNCS_GET, false, m_pTransObj, urlType, m_Path, retStatus);

    WLLogMessage::Severity err = m_pTransObj->GetTransaction()->GetSeverity();
    m_pTransObj->GetTransaction()->SetEndTransaction(TVPSysTime(), err, m_Ws->m_bSaveTransaction);

    if (ret == BREAK)
			break;
	} while (true);
}

int CWLHttpRequest::CondDoCommand(HttpFuncs method, bool first, CWLTransactionsObject* pTransObj, 
                                  HtmlQueueItemType objType, RVCString& path, int& retStatus)
{

  bool bGetTheUrl = false;

  m_ExplicitUrl = false ;
  if (first) m_ExplicitUrl = true ;
  m_pTransObj = pTransObj ;

  if (!m_pTransObj)
    return BREAK;

  //*************************************************************
  // The following section will execute only if TRACE IS ON!!!
  //*************************************************************
  if (WlTrace) {
     "%d: %s", objType, m_pTransObj->GetObjUrl();
  }
	if (objType == NONE || objType == TERMINATE || objType == RELEASE) {
		if (objType == TERMINATE)
			retStatus = RET_ERROR;
		return BREAK;
	}

	if (m_Cm->isAbort() || retStatus == RET_ERROR)
		return CONTINUE;

	if (m_Cm->m_ReadFromFile) {	// source should be retrieved from a file
		if (objType != FRAME)	// nothing to do when file is not a frame
			return CONTINUE;

		if (path.isNull() && !m_Cm->m_Path.isNull())
			path = m_Cm->m_Path;
		retStatus = m_Cm->ParseFile(path, (CWLDocument*)m_pTransObj->GetObject());

    if (retStatus == RET_ERROR)
    {
      ConnectError(HTTP_ERROR_FILE, 111);
      CWLPage *pPage = m_Cm->GetPage();
      if (!pPage)
      {
        CWLDocument *frame = pPage->GetPageTransaction();
        if (frame)
          frame->SetSeverity(WLLogMessage::Error);
      }
    }
	}

  // The comment is obsolete 10/10/2000  Uri.

  // if there was a URL in the queue, then getContents is true
	// if getContents is true then images should be retrieved
	// if type is frame then it should be retrieved only if getSourcesONLY is false
	// resource should be retrieved if :
	// it's the one requested in the JS command
	// GetContents == "yes"
	// it's an image and GetFrameContents != "no"
  //else if ( first || m_Ws->m_GetFrames || objType == XML || objType == STYLE ||
	//	((objType == APPLET || objType == SCRIPT || objType == LINK || objType == IMAGE) && m_Ws->m_GetImages))

  else if( first                                                           || // the url from the agenda
           ( objType == GENERAL                                          ) || // Chaim - for ParseOnce
           ( objType == FRAME                      && m_Ws->m_GetFrames  ) || // frames
           ( (objType == XML || objType == DTD)  && m_Ws->m_GetXmls    ) || // XML islands
           ( (objType == STYLE || objType == LINK) && m_Ws->m_GetCss     ) || // Style Sheets
           ( objType == APPLET                     && m_Ws->m_GetApplets ) || // Java Applets
           ( objType == EMBED                      && m_Ws->m_GetEmbeds  ) || // Embeded objects
           ( objType == SCRIPT                     && m_Ws->m_GetScripts ) || // JS Scriplets
           ( objType == IMAGE                      && m_Ws->m_GetImages  )   )// Images 
  
  {
     retStatus = DoCommand( method, objType );

     WlTraceMsg("Status returned: %d", retStatus);

		 if (m_Cm->isAbort() || retStatus == RET_ERROR)
		 	 m_Cm->GetPage()->SetInvalidPage(m_Ws->pageName);
  }

  return CONTINUE;
}

bool CWLHttpRequest::ShouldNavigate(RVCString url, const char* filter){
  if (url.first("#") != RV_NPOS) {
    return false;
  }

  if (filter && filter[0])
  {
    //there is some kind of url or domain the user wants to filter
    RVCTokenizer next(filter);
    RVCString sFilterUrl;

    while (!(sFilterUrl=next(";")).isNull())
    {
      size_t index = url.index(sFilterUrl);
      // Chaim - I'm removing this check so FilterUrl will catch suffixes as well.
//      if ( (index!=RV_NPOS) && (index<=8) ) //https://
      if ( (index!=RV_NPOS) ) //https://
      {
        return false;
      }
    }
  }
  return true;
}

void CWLHttpRequest::CloseCBData(){
  // Check if previous object callback was closed
  if (m_Cm->IsCallbackEnabled() && !m_CBDataClosed){
    cHttpEmbedderReportInfoObjectAbortDownload AbortDwnld((CUrlDataEx*)&m_CBData);
    AbortDwnld.fReport(m_Cm->GetJScx());
    m_CBDataClosed = true;
  }
}

int CWLHttpRequest::DoCommand( HttpFuncs method, HtmlQueueItemType p_uObjType )
{

  CloseCBData();

  m_UrlType = m_pTransObj->GetObjType();
  switch(m_UrlType) {
		case FRAME:
  		m_Frame = (CWLDocument*)m_pTransObj->GetObject();
			break;

		case XML:
			m_Frame = (CWLXmlDocument*)m_pTransObj->GetObject();

    default:
      m_Frame = m_Cm->GetPage()->GetGlobalDoc();
  }

  //Galtech
  m_Url = ToANSI(m_pTransObj->GetObjUrl(), GetServerEncoding());

	parseUrl(m_Url);
  m_Base.setBase(m_Url);  // set the base for the request

	initLocalVariables();

  //There cases when a frame or image hold null SRC. In some HTML pages the SRC 
  //tag is assigned by a java script in the HTML and since webload does not execute scripts 
  //we get a frame/image with no source. In this case we just ignore it. 
	if (m_Url.isNull()) {
    ThreadError(HTTP_ERROR_FILE, 305);
    return RET_OK;
  }

	// is this part needed now ???
	int anchorPos = m_Url.first('#');
	if (anchorPos != RV_NPOS)	
		m_Url.remove(anchorPos);

	m_FullUrl = m_Url;

  WlTraceMsg("*** Round: %d, executing: %d(\"%s\")\n", m_Ws->lastRoundNum, method, m_Url.data());

  // Chaim
  if (m_Cm->IsCallbackEnabled()){
    m_CBData.SetUrl(m_FullUrl) ;
    m_CBData.SetOriginalUrl(m_pTransObj->GetOriginalUrl()) ;
    m_CBData.SetReferer(m_pTransObj->GetReferer()) ;
    m_CBData.SetTarget(m_Ws->m_Target) ;
  }

  /// call object callbacks if necessary
  if (m_Cm->IsCallbackEnabled() && method != HTTP_FUNCS_HEAD )
  {
    cHttpEmbedderReportInfoObjectStartDownload StartDwnld((CUrlData*)&m_CBData);
    StartDwnld.fReport(m_Cm->GetJScx());
    m_CBDataClosed = false;
  }

  RVCString sUrl(m_pTransObj->GetObjUrl());

  int retStatus = RET_OK;

  // Chaim - Bug 19341 if page was filtered, show an empty page with user message
  if (!ShouldNavigate(sUrl, m_Ws->m_pcFilterUrl)){
    if (m_Cm->IsCallbackEnabled() && m_UrlType == FRAME){
      RVCString tempOutput("<html><body><b>This page is not displayed because the URL: ");
      tempOutput += sUrl;
      tempOutput += " was screened by the agenda.</b> </body></html>";
      m_CBData.SetUrlType(m_UrlType) ;
      m_CBData.SetHttpStatus(RET_OK) ;
      m_CBData.SetObjContent((void*)tempOutput.data()) ;
      m_CBData.SetObjContentLength(tempOutput.length()) ;
      m_CBData.SetMimeType("text/html") ;
      m_CBData.SetFileSize(0);
      m_CBData.SetCheckSum(0);

      cHttpEmbedderReportInfoObjectEndDownload EndDwnld(&m_CBData);
      EndDwnld.fReport(m_Cm->GetJScx());
      m_CBDataClosed = true;
	}
	}
  else {
	retStatus = DoHttpCommand( method, p_uObjType );

	m_Response.resetOutput();

  // Chaim
  if (m_Cm->IsCallbackEnabled()){
    m_CBData.SetUrlType(m_UrlType) ;
    m_CBData.SetHttpStatus(retStatus) ;
  }

	if (retStatus == RET_ERROR || retStatus == IS_ABORT)
  {
		closeConnection(true);

    if (m_Cm->IsCallbackEnabled() && method != HTTP_FUNCS_HEAD )
    {
      cHttpEmbedderReportInfoObjectAbortDownload AbortDwnld((CUrlDataEx*)&m_CBData);
      AbortDwnld.fReport(m_Cm->GetJScx());
      m_CBDataClosed = true;
    }
  }
  else if (m_Cm->IsCallbackEnabled() && method != HTTP_FUNCS_HEAD )
  {

    // Chaim - bug 19486 - When we download a .cab, it sometimes has a null byte at the start which screws up
    // the browser display in IDE. Since its content type is "text/plain", I simple take its actual length
    // as the object length for the callback, which strips the actual text for display but avoids a crash.
    const char* mimeType = m_Response.GetContentType().data();
    const char* ObjContent = m_Response.GetOutputData();
    int contentLength;
    if (!strcmp(mimeType, "text/plain")){
      contentLength = strlen(ObjContent);
    }
    else {
      contentLength = m_Response.GetOutputLength();
    }
    m_CBData.SetObjContent((void*)ObjContent) ;
    m_CBData.SetObjContentLength(contentLength) ;
    m_CBData.SetMimeType(mimeType) ;
    m_CBData.SetFileSize(m_pTransObj->GetFileSize());
    m_CBData.SetCheckSum(m_pTransObj->GetCheckSum());

// Chaim - bug 19462 - not sure why we add locations for XML. What happens here is we add the locations of the
// frame that contains the XML to the XML itself and that screws up the browser preview in the IDE
//    if ( m_UrlType == FRAME || m_UrlType == XML )
    if ( m_UrlType == FRAME )
    {
      m_CBData.SetFinalUrl(m_Frame->GetDocUrl());
      CWLTransactionsObject *pObjectTransObj ;

      if (m_Frame->m_Frames && m_Ws->m_GetFrames)
      {
        //go through all frames in frame.
        unsigned int unNumOfEntries = m_Frame->m_Frames->entries();
        for (unsigned int unIndex = 0 ; unIndex < unNumOfEntries ; ++unIndex)
        {
          CWLDocument* pFrame = (CWLDocument*)m_Frame->m_Frames->at(unIndex);
          if (!pFrame->GetOriginalUrl().isNull())
            m_CBData.AddLocation(pFrame->GetOriginalUrl(), pFrame->getUrlOffset(), pFrame->GetOriginalUrl().length());
        }
      }

      //go through all objects in frame
      for ( pObjectTransObj = m_Frame->GetFirstTransactionObj() ; 
            pObjectTransObj = m_Frame->GetNextTransactionObj(); )
      {
        const char* pcOriginalUrl = pObjectTransObj->GetOriginalUrl();
        int nUrlLength = 0 ;
        if ( pcOriginalUrl && (nUrlLength=strlen(pcOriginalUrl)) > 0 ) // ( isNull() )
          m_CBData.AddLocation(pcOriginalUrl, pObjectTransObj->GetUrlOffset(), nUrlLength ,
							   (pObjectTransObj->GetObjType() == IMAGE));
      }
    }
    else
    {
      m_CBData.SetFinalUrl(m_pTransObj->GetObjUrl());
    }

    // Chaim - Bug 19341 if page was filtered, show an empty page with user message, so actually the callback was already
    // called
    if (ShouldNavigate(sUrl, m_Ws->m_pcFilterUrl) && !m_pTransObj->IsCached()){
      cHttpEmbedderReportInfoObjectEndDownload EndDwnld(&m_CBData);
      EndDwnld.fReport(m_Cm->GetJScx());
      m_CBDataClosed = true;
    }
  }

	releaseConnection();
  }

  m_CBData.Reset();

  return retStatus;
}

const char* CWLHttpRequest::GetReferer() const
{
  if ( m_Cm->IsTopFrame(m_pTransObj) )
  {
    if ( *m_Ws->m_sReferer )
    {
      return m_Ws->m_sReferer ;
    }
    else{}
  }
  else if ( m_pTransObj->GetReferer() && strlen(m_pTransObj->GetReferer()) &&
            m_pTransObj->GetObjType() != APPLET &&
            m_pTransObj->GetObjType() != EMBED )
  {
    return m_pTransObj->GetReferer();
  }

  return NULL;
}
