/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/******************************************************************\
 *	File name : HTTPSSL.CPP
 *
 *	Author : Moshe Sheked
 *
 *	Description :
 *
 *	History :
 *
 *	Modifications :
 *		Who				When				What
 *		---				----				----
 *
\******************************************************************/

/** include files **/
#include "commandmanager.h"
#include "connpool.h"
#include "httpssl.h"
#include "tracedll.h"
#include "tvenv.h"

#ifdef __SSL__

#include <stdlib.h>
#include <time.h>

#ifdef __TV_WINDOWS__
#include <winsock.h>
#else
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#endif

#include "asn1oids.h" //stav

#include "wlsslcache.h"
#include "httprequest.h"
#include "httpdefines.h"

#include "gt_strwrapper.h"


#ifndef __TV_WINDOWS__
#define SOCKET int
#endif

/** default settings **/

/** external functions **/
extern SSLErr InitDatabase(void **dbRef);
extern SSLErr SSLAddSession(SSLBuffer key, SSLBuffer sessionData, void *sessionRef);
extern SSLErr SSLGetSession(SSLBuffer key, SSLBuffer *sessionData, void *sessionRef);
extern SSLErr SSLDeleteSession(SSLBuffer key, void *sessionRef);

/** external data **/

/** internal functions **/

/** public data **/

/** private data **/

/** classes definitions **/

/** public functions **/

/** private functions **/

void FreeSSLBuffer(SSLBuffer* buffer)
{
	if (buffer->data)
		CD_free(buffer->data);
}

SSLErr SocketRead(SSLBuffer buffer, uint32 *processed, void *state)
{
  if (!state) {
  	WlTraceMsg("SSL: read got a null state and returned SSLConnectionClosedError");
		return SSLConnectionClosedError;
  }

	CWLConnection*          conn          = (CWLConnection*)state;
	SOCKET                  sock          = (SOCKET)conn->GetSocket();
  CWLTransactionsObject*  pTransObj     = conn->GetTransObj();
  CWLHttpRequest*         pHttpRequest  = conn->GetHttpRequest();
  CWLCommandManager*      pCM           = conn->GetCM();
	uint32                  totalLength   = 0;
	int                     length        = 0;
	int                     lastRecvLength= 0;

  do {
		length = conn->GetConnPool()->Recv(sock, (char*)buffer.data + totalLength, (int)(buffer.length - totalLength), 0, lastRecvLength);

    // If we read more then one byte then we add it to our totalLength
    if (length > 0) {
      totalLength += length;
    }

  	WlTraceMsg("SSL: read %d encrypted bytes (total: %d) into buffer (length=%d), lastRecvLength: %d", length, totalLength, buffer.length, lastRecvLength);

    //adding timers and transactions wait and receive

    if (pHttpRequest->IsFirstTimeWait())
    {
	    if (!pCM->isAbort())
        pCM->addTimer("Time To First Byte", TVPSysTime() - pHttpRequest->m_dStartWait);

      pHttpRequest->m_dStartReceive = TVPSysTime();
      pTransObj->GetLastSubTransation()->SetEndTransaction(pHttpRequest->m_dStartReceive, WLLogMessage::Success);//Time To First Byte
      if (length <= 0)
      {
        conn->SetTransactionData(pTransObj->GetLastSubTransation());     
        conn->ReportError(HTTP_ERROR_FILE, 108);
        conn->SetTransactionData(NULL);
      }

      pTransObj->AddSubTransaction("Receive", pHttpRequest->m_dStartReceive, CWLTransactionData::TRANS_RECEIVE);

      pHttpRequest->SetFirstTimeWait(false) ;
    }

#ifndef __TV_WINDOWS__
    if ((lastRecvLength == SOCKET_ERROR) && (errno == EAGAIN)) {
    	  WlTraceMsg("SSL: read got an EAGAIN error - continue reading");
        continue;
    }
#endif
	} while ((lastRecvLength > 0) && (totalLength < buffer.length));

 	WlTraceMsg("SSL: read (Ended) %d encrypted bytes (total: %d) into buffer (length=%d), lastRecvLength: %d", length, totalLength, buffer.length, lastRecvLength);

	*processed = totalLength;

  // If we got an error while reading the data then we check
  // for the specific error the EWOULDBLOCK is the only status
  // that is valid to continue the reading from the socket
#ifdef __TV_WINDOWS__
  if (lastRecvLength == SOCKET_ERROR) {
    if (WSAGetLastError() == EWOULDBLOCK) {
  	  WlTraceMsg("SSL: read got to SSLWouldBlockErr");
		  return SSLWouldBlockErr;
    }
    else if ((WSAGetLastError() == WSAESHUTDOWN) || (WSAGetLastError() == WSAECONNRESET)) {
  	  WlTraceMsg("SSL: read got to SSLConnectionClosedError");
		  return SSLConnectionClosedError;
    }
    else {
      int err = WSAGetLastError();
		  WlTraceMsg("SSL: read IO error %4d", err);
		  return SSLIOErr;
    }
  }
#else
  if (lastRecvLength == SOCKET_ERROR) {
    if (errno == EWOULDBLOCK) {
  	  WlTraceMsg("SSL: read got to SSLWouldBlockErr");
		  return SSLWouldBlockErr;
    }
    else if ((errno == WSAESHUTDOWN) || (errno == WSAECONNRESET)) {
  	  WlTraceMsg("SSL: read got to SSLConnectionClosedError");
		  return SSLConnectionClosedError;
    }
    else {
		  WlTraceMsg("SSL: read IO error %4d", errno);
		  return SSLIOErr;
    }
  }
#endif

  if (lastRecvLength == 0) {
     if (totalLength == 0) {
        // Connection closed at start of read; don't necessarily treat as an error
  	    WlTraceMsg("SSL: read got to SSLIOClosedOverrideGoodbyeKiss");
  		  return SSLIOClosedOverrideGoodbyeKiss;
     }
     else {
		    WlTraceMsg("SSL: read IO error because lastRecvLength=0 and totalLength=%d", totalLength);
		    return SSLIOErr;
     }
  }

  // This if statment wan't be executed ... (left only because of unclear with Certicom)
  if (totalLength != buffer.length) {
  	WlTraceMsg("SSL: read got to SSLWouldBlockErr because done(%d) is not equal buffer.length(%d)", totalLength, buffer.length);
		return SSLWouldBlockErr;
  }

	return SSLNoErr;
}

SSLErr SocketWrite(SSLBuffer buffer, uint32 *processed, void *state)
{
  if (!state) {
  	WlTraceMsg("SSL: write got a null state and returned SSLConnectionClosedError");
		return SSLConnectionClosedError;
  }

	CWLConnection* conn         = (CWLConnection*)state;
	SOCKET         sock         = (SOCKET)conn->GetSocket();
	uint32         totalLength  = 0;
	int            length;
	int            lastSendLength;

	do {
		int lengthToSend = (int)(buffer.length - totalLength);
		length = conn->GetConnPool()->Send(sock, (char*)buffer.data + totalLength, lengthToSend, 0, lastSendLength, true);

    // If we read more then one byte then we add it to our totalLength
    if (length > 0) {
      totalLength += length;
    }

    WlTraceMsg("SSL: wrote %d encrypted bytes (total: %d) from buffer (length=%d), lastSendLength: %d", length, totalLength, buffer.length, lastSendLength);
#ifndef __TV_WINDOWS__
    if ((lastSendLength == SOCKET_ERROR) && (errno == EAGAIN)) {
    	WlTraceMsg("SSL: write got an EAGAIN error");
      continue;
    }
#endif
	} while ((lastSendLength > 0) && (totalLength < buffer.length));

 	WlTraceMsg("SSL: wrote (Ended) %d encrypted bytes (total: %d) from buffer (length=%d), lastSendLength: %d", length, totalLength, buffer.length, lastSendLength);

	*processed = totalLength;

#ifdef __TV_WINDOWS__
	if (lastSendLength == SOCKET_ERROR) {
		int err = WSAGetLastError();
		WlTraceMsg("SSL: IO write error %4d", err);
		return SSLIOErr;
	}
#else
	if (lastSendLength == SOCKET_ERROR) {
		WlTraceMsg("SSL: IO write error %4d", errno);
		return SSLIOErr;
  }
#endif

  if (lastSendLength == 0) {
     if (totalLength == 0) {
        // Connection closed at start of read; don't necessarily treat as an error
  	    WlTraceMsg("SSL: write got to SSLIOClosedOverrideGoodbyeKiss");
  		  return SSLIOClosedOverrideGoodbyeKiss;
     }
     else {
		    WlTraceMsg("SSL: write IO error because lastRes=0 and totalLength=%d",totalLength);
		    return SSLIOErr;
     }
  }
  // This if statment wan't be executed ... (left only because of unclear with Certicom)
  if (totalLength != buffer.length) {
		WlTraceMsg("SSL: write IO error because done not equel buffer.length(%d)",buffer.length);
		return SSLIOErr;
	}

	return SSLNoErr;
}

SSLErr
CertificateCallback(SSLCertificateChain *certs, uint32 trustedCert, SSLErr validateErr, void *checkCertificateRef)
{
	char                    *caName;
	SSLCertificateChain     *peer;
//	SSLErr                  err;
//	SSLAVA                  commonName, nameElem;
//	uint32                  elemCount;
	RVCString               level;

	if (!checkCertificateRef)
		return SSLNoErr;

  level = (const char*)checkCertificateRef;

    if ( level == SSL_CERT_VERIFY_ALL ) {
      if (certs == 0) {
//				printf(" - No certificate supplied\n");
        return X509NoCertSupplied;
      }
      switch (trustedCert)
        {   case 0:
                caName = "Unknown CA";
                break;
            case 1:
                caName = "VeriSign Class 1";
                break;
            case 2:
                caName = "VeriSign Class 2";
                break;
            case 3:
                caName = "VeriSign Class 3";
                break;
            case 4:
                caName = "VeriSign Class 4";
                break;
            case 10:
                caName = "RSA Server CA";
        }
//        printf(" - CA is %s\n", caName);

        /* Last cert is peer */
        peer = certs;
        while (peer->next != 0) {
            peer = peer->next;

//        err = SSLExtractSubjectDNField(peer->certData, OID_commonName, &commonName);
//        if (err != SSLNoErr) {
//            printf(" - Couldn't get common name (%d)\n", err);
//            return err;
//        }
//        else
//        {
//        		printf(" - Common name is \"");
//            fwrite(commonName.avaData.data, 1, (size_t) commonName.avaData.length, stdout);
//            printf("\"\n");
//        }
//
//        err = SSLCountSubjectDNFields(certs->certData, &elemCount);
//        if (err != SSLNoErr) {
//            printf(" - Couldn't count elements (%d)\n", err);
//            return err;
//        }
//        else
//        {
//        		printf(" - CA has %d subject name fields\n", elemCount);
//            err = SSLExtractSubjectDNFieldIndex(certs->certData, elemCount - 1, &nameElem);
//            if (err != SSLNoErr) {
//                printf(" - Couldn't get indexed element (%d)\n", err);
//              return err;
//            }
//            else
//            {
//            		printf(" - Last element (OID = %d) is \"", nameElem.oid);
//                fwrite(nameElem.avaData.data, 1, (size_t) nameElem.avaData.length, stdout);
//                printf("\"\n");
//            }
        }
    }
    //no need 2 verify all, check if need to validate
	  else if ( level == SSL_CERT_VALIDATE_ONLY ) {
      if (validateErr != 0) {
//				printf(" - Certificate chain didn't validate: %d\n", validateErr);
        return validateErr;
      }
    }
    return SSLNoErr;
}

static SSLErr ConfigureSSLSession(SSLContext *ctx, void* *random)
{

  /////////////////////////////////////////////////////////////////////
  // The load of the trusted certificate is in remark
  // because it is not needed for the functionallity to work
  // and we need to find a better way to load it because of performance
  /////////////////////////////////////////////////////////////////////
//  int   trustedCertCount;
//  char* trustedCertFile;
//  RVCString RWtrustedCertFile (TVEGetEnv("TV_BIN"));
//  RWtrustedCertFile += "\\trusted.txt";
//  trustedCertFile = (char*)malloc (RWtrustedCertFile.length());
//  trustedCertFile = strdup (RWtrustedCertFile);

  SSLInitContext(ctx);
//  SSLSetAllocFunc(ctx, SSLAlloc);
//  SSLSetFreeFunc(ctx, SSLFree);
//  SSLSetReallocFunc(ctx, SSLRealloc);
  SSLSetRandomFunc(ctx, WLSSLRandomFunc);
//  SSLSetTimeFunc(ctx, SSLTime);
//  SSLSetConvertTimeFunc(ctx, SSLConvertTime);
  SSLSetReadFunc(ctx, SocketRead);
  SSLSetWriteFunc(ctx, SocketWrite);

// Load the trusted certificate file
//  SSLLoadTrustedCertificateFile(ctx, trustedCertFile, &trustedCertCount);

  WLSeedRandomAndSet(ctx, random);
  return SSLNoErr;
}


/* 1024-bit time = 499 sec.  (8:19) */
static unsigned char prime1024[1024/8] = {
    0xF4, 0x88, 0xFD, 0x58, 0x4E, 0x49, 0xDB, 0xCD,
    0x20, 0xB4, 0x9D, 0xE4, 0x91, 0x07, 0x36, 0x6B,
    0x33, 0x6C, 0x38, 0x0D, 0x45, 0x1D, 0x0F, 0x7C,
    0x88, 0xB3, 0x1C, 0x7C, 0x5B, 0x2D, 0x8E, 0xF6,
    0xF3, 0xC9, 0x23, 0xC0, 0x43, 0xF0, 0xA5, 0x5B,
    0x18, 0x8D, 0x8E, 0xBB, 0x55, 0x8C, 0xB8, 0x5D,
    0x38, 0xD3, 0x34, 0xFD, 0x7C, 0x17, 0x57, 0x43,
    0xA3, 0x1D, 0x18, 0x6C, 0xDE, 0x33, 0x21, 0x2C,
    0xB5, 0x2A, 0xFF, 0x3C, 0xE1, 0xB1, 0x29, 0x40,
    0x18, 0x11, 0x8D, 0x7C, 0x84, 0xA7, 0x0A, 0x72,
    0xD6, 0x86, 0xC4, 0x03, 0x19, 0xC8, 0x07, 0x29,
    0x7A, 0xCA, 0x95, 0x0C, 0xD9, 0x96, 0x9F, 0xAB,
    0xD0, 0x0A, 0x50, 0x9B, 0x02, 0x46, 0xD3, 0x08,
    0x3D, 0x66, 0xA4, 0x5D, 0x41, 0x9F, 0x9C, 0x7C,
    0xBD, 0x89, 0x4B, 0x22, 0x19, 0x26, 0xBA, 0xAB,
    0xA2, 0x5E, 0xC3, 0x55, 0xE9, 0x2F, 0x78, 0xC7
};

unsigned char generator[1] = { 2 };
#ifdef __TV_WINDOWS__
/* extern arrays are in verisign.c; sizes are copied by hand from those definitions */
#define CLASS1_ROOT_SUBJECT_NAME_SIZE 97
extern unsigned char CLASS1_ROOT_SUBJECT_NAME[CLASS1_ROOT_SUBJECT_NAME_SIZE];
#define CLASS2_ROOT_SUBJECT_NAME_SIZE 97
extern unsigned char CLASS2_ROOT_SUBJECT_NAME[CLASS2_ROOT_SUBJECT_NAME_SIZE];
#define CLASS3_ROOT_SUBJECT_NAME_SIZE 97
extern unsigned char CLASS3_ROOT_SUBJECT_NAME[CLASS3_ROOT_SUBJECT_NAME_SIZE];
#define CLASS4_ROOT_SUBJECT_NAME_SIZE 97
extern unsigned char CLASS4_ROOT_SUBJECT_NAME[CLASS4_ROOT_SUBJECT_NAME_SIZE];

#define CLASS1_ROOT_SELF_SIGNED_CERT_SIZE 565
extern unsigned char CLASS1_ROOT_SELF_SIGNED_CERT[CLASS1_ROOT_SELF_SIGNED_CERT_SIZE];
#define CLASS2_ROOT_SELF_SIGNED_CERT_SIZE 565
extern unsigned char CLASS2_ROOT_SELF_SIGNED_CERT[CLASS2_ROOT_SELF_SIGNED_CERT_SIZE];
#define CLASS3_ROOT_SELF_SIGNED_CERT_SIZE 565
extern unsigned char CLASS3_ROOT_SELF_SIGNED_CERT[CLASS3_ROOT_SELF_SIGNED_CERT_SIZE];
#define CLASS4_ROOT_SELF_SIGNED_CERT_SIZE 565
extern unsigned char CLASS4_ROOT_SELF_SIGNED_CERT[CLASS4_ROOT_SELF_SIGNED_CERT_SIZE];
#endif

static
unsigned char RSA_COMMERCIAL_CA_ROOT_SUBJECT_NAME[] =
{   0x30, 0x5F, 0x31, 0x0B, 0x30, 0x09, 0x06, 0x03, 0x55, 0x04, 0x06, 0x13, 0x02, 0x55, 0x53, 0x31,
    0x20, 0x30, 0x1E, 0x06, 0x03, 0x55, 0x04, 0x0A, 0x13, 0x17, 0x52, 0x53, 0x41, 0x20, 0x44, 0x61,
    0x74, 0x61, 0x20, 0x53, 0x65, 0x63, 0x75, 0x72, 0x69, 0x74, 0x79, 0x2C, 0x20, 0x49, 0x6E, 0x63,
    0x2E, 0x31, 0x2E, 0x30, 0x2C, 0x06, 0x03, 0x55, 0x04, 0x0B, 0x13, 0x25, 0x53, 0x65, 0x63, 0x75,
    0x72, 0x65, 0x20, 0x53, 0x65, 0x72, 0x76, 0x65, 0x72, 0x20, 0x43, 0x65, 0x72, 0x74, 0x69, 0x66,
    0x69, 0x63, 0x61, 0x74, 0x69, 0x6F, 0x6E, 0x20, 0x41, 0x75, 0x74, 0x68, 0x6F, 0x72, 0x69, 0x74,
    0x79
};

static
unsigned char RSA_COMMERCIAL_CA_ROOT_SELF_SIGNED_CERT[] =
{   0x30, 0x82, 0x02, 0x29, 0x30, 0x82, 0x01, 0x96, 0x02, 0x05, 0x02, 0x41, 0x00, 0x00, 0x01, 0x30,
    0x0D, 0x06, 0x09, 0x2A, 0x86, 0x48, 0x86, 0xF7, 0x0D, 0x01, 0x01, 0x02, 0x05, 0x00, 0x30, 0x5F,
    0x31, 0x0B, 0x30, 0x09, 0x06, 0x03, 0x55, 0x04, 0x06, 0x13, 0x02, 0x55, 0x53, 0x31, 0x20, 0x30,
    0x1E, 0x06, 0x03, 0x55, 0x04, 0x0A, 0x13, 0x17, 0x52, 0x53, 0x41, 0x20, 0x44, 0x61, 0x74, 0x61,
    0x20, 0x53, 0x65, 0x63, 0x75, 0x72, 0x69, 0x74, 0x79, 0x2C, 0x20, 0x49, 0x6E, 0x63, 0x2E, 0x31,
    0x2E, 0x30, 0x2C, 0x06, 0x03, 0x55, 0x04, 0x0B, 0x13, 0x25, 0x53, 0x65, 0x63, 0x75, 0x72, 0x65,
    0x20, 0x53, 0x65, 0x72, 0x76, 0x65, 0x72, 0x20, 0x43, 0x65, 0x72, 0x74, 0x69, 0x66, 0x69, 0x63,
    0x61, 0x74, 0x69, 0x6F, 0x6E, 0x20, 0x41, 0x75, 0x74, 0x68, 0x6F, 0x72, 0x69, 0x74, 0x79, 0x30,
    0x1E, 0x17, 0x0D, 0x39, 0x34, 0x31, 0x31, 0x30, 0x39, 0x32, 0x33, 0x35, 0x34, 0x31, 0x37, 0x5A,
    0x17, 0x0D, 0x39, 0x39, 0x31, 0x32, 0x33, 0x31, 0x32, 0x33, 0x35, 0x34, 0x31, 0x37, 0x5A, 0x30,
    0x5F, 0x31, 0x0B, 0x30, 0x09, 0x06, 0x03, 0x55, 0x04, 0x06, 0x13, 0x02, 0x55, 0x53, 0x31, 0x20,
    0x30, 0x1E, 0x06, 0x03, 0x55, 0x04, 0x0A, 0x13, 0x17, 0x52, 0x53, 0x41, 0x20, 0x44, 0x61, 0x74,
    0x61, 0x20, 0x53, 0x65, 0x63, 0x75, 0x72, 0x69, 0x74, 0x79, 0x2C, 0x20, 0x49, 0x6E, 0x63, 0x2E,
    0x31, 0x2E, 0x30, 0x2C, 0x06, 0x03, 0x55, 0x04, 0x0B, 0x13, 0x25, 0x53, 0x65, 0x63, 0x75, 0x72,
    0x65, 0x20, 0x53, 0x65, 0x72, 0x76, 0x65, 0x72, 0x20, 0x43, 0x65, 0x72, 0x74, 0x69, 0x66, 0x69,
    0x63, 0x61, 0x74, 0x69, 0x6F, 0x6E, 0x20, 0x41, 0x75, 0x74, 0x68, 0x6F, 0x72, 0x69, 0x74, 0x79,
    0x30, 0x81, 0x9B, 0x30, 0x0D, 0x06, 0x09, 0x2A, 0x86, 0x48, 0x86, 0xF7, 0x0D, 0x01, 0x01, 0x01,
    0x05, 0x00, 0x03, 0x81, 0x89, 0x00, 0x30, 0x81, 0x85, 0x02, 0x7E, 0x00, 0x92, 0xCE, 0x7A, 0xC1,
    0xAE, 0x83, 0x3E, 0x5A, 0xAA, 0x89, 0x83, 0x57, 0xAC, 0x25, 0x01, 0x76, 0x0C, 0xAD, 0xAE, 0x8E,
    0x2C, 0x37, 0xCE, 0xEB, 0x35, 0x78, 0x64, 0x54, 0x03, 0xE5, 0x84, 0x40, 0x51, 0xC9, 0xBF, 0x8F,
    0x08, 0xE2, 0x8A, 0x82, 0x08, 0xD2, 0x16, 0x86, 0x37, 0x55, 0xE9, 0xB1, 0x21, 0x02, 0xAD, 0x76,
    0x68, 0x81, 0x9A, 0x05, 0xA2, 0x4B, 0xC9, 0x4B, 0x25, 0x66, 0x22, 0x56, 0x6C, 0x88, 0x07, 0x8F,
    0xF7, 0x81, 0x59, 0x6D, 0x84, 0x07, 0x65, 0x70, 0x13, 0x71, 0x76, 0x3E, 0x9B, 0x77, 0x4C, 0xE3,
    0x50, 0x89, 0x56, 0x98, 0x48, 0xB9, 0x1D, 0xA7, 0x29, 0x1A, 0x13, 0x2E, 0x4A, 0x11, 0x59, 0x9C,
    0x1E, 0x15, 0xD5, 0x49, 0x54, 0x2C, 0x73, 0x3A, 0x69, 0x82, 0xB1, 0x97, 0x39, 0x9C, 0x6D, 0x70,
    0x67, 0x48, 0xE5, 0xDD, 0x2D, 0xD6, 0xC8, 0x1E, 0x7B, 0x02, 0x03, 0x01, 0x00, 0x01, 0x30, 0x0D,
    0x06, 0x09, 0x2A, 0x86, 0x48, 0x86, 0xF7, 0x0D, 0x01, 0x01, 0x02, 0x05, 0x00, 0x03, 0x7E, 0x00,
    0x88, 0xD1, 0xD1, 0x79, 0x21, 0xCE, 0xE2, 0x8B, 0xE8, 0xF8, 0xC1, 0x7D, 0x34, 0x53, 0x3F, 0x61,
    0x83, 0xD9, 0xB6, 0x0B, 0x38, 0x17, 0xB6, 0xE8, 0xBE, 0x21, 0x8D, 0x8F, 0x00, 0xB8, 0x8B, 0x53,
    0x7E, 0x44, 0x67, 0x1E, 0x22, 0xBD, 0x97, 0x27, 0xE0, 0x9C, 0x85, 0xCC, 0x4A, 0xF6, 0x85, 0x3B,
    0xB2, 0xE2, 0xBE, 0x92, 0xD3, 0xE5, 0x0D, 0xE9, 0xAF, 0x5C, 0x0E, 0x0C, 0x46, 0x95, 0xFF, 0xA1,
    0x1C, 0x5E, 0x3E, 0xE8, 0x36, 0x58, 0x7A, 0x73, 0xA6, 0x0A, 0xF8, 0x22, 0x11, 0x6B, 0xC3, 0x09,
    0x38, 0x7E, 0x26, 0xBB, 0x73, 0xEF, 0x00, 0xBD, 0x02, 0xA4, 0xF3, 0x14, 0x0D, 0x30, 0x3F, 0x61,
    0x70, 0x7B, 0x20, 0xFE, 0x32, 0xA3, 0x9F, 0xB3, 0xF4, 0x67, 0x52, 0xDC, 0xB4, 0xEE, 0x84, 0x8C,
    0x96, 0x36, 0x20, 0xDE, 0x81, 0x08, 0x83, 0x71, 0x21, 0x8A, 0x0F, 0x9E, 0xA9
};


#ifdef __TV_WINDOWS__
static SSLErr
AddTrustedCerts(SSLContext *ctx)
{   SSLErr      err;
    SSLBuffer   certDN;

    certDN.data = RSA_COMMERCIAL_CA_ROOT_SELF_SIGNED_CERT;
    certDN.length = sizeof(RSA_COMMERCIAL_CA_ROOT_SELF_SIGNED_CERT);
    if ((err = SSLAddTrustedCertificate(ctx, certDN, 10)) != 0)
        return err;
    certDN.data = CLASS1_ROOT_SELF_SIGNED_CERT;
    certDN.length = CLASS1_ROOT_SELF_SIGNED_CERT_SIZE;
    if ((err = SSLAddTrustedCertificate(ctx, certDN, 1)) != 0)
        return err;
    certDN.data = CLASS2_ROOT_SELF_SIGNED_CERT;
    certDN.length = CLASS2_ROOT_SELF_SIGNED_CERT_SIZE;
    if ((err = SSLAddTrustedCertificate(ctx, certDN, 2)) != 0)
        return err;
    certDN.data = CLASS3_ROOT_SELF_SIGNED_CERT;
    certDN.length = CLASS3_ROOT_SELF_SIGNED_CERT_SIZE;
    if ((err = SSLAddTrustedCertificate(ctx, certDN, 3)) != 0)
        return err;
    certDN.data = CLASS4_ROOT_SELF_SIGNED_CERT;
    certDN.length = CLASS4_ROOT_SELF_SIGNED_CERT_SIZE;
    if ((err = SSLAddTrustedCertificate(ctx, certDN, 4)) != 0)
        return err;

    return SSLNoErr;
}
#endif


#include <RVCString.h>


int SetPeer(SSLBuffer* peerID, in_addr inAddr, int port)
{
	peerID->length = 2 * sizeof(long);
	peerID->data = (unsigned char*)CD_malloc(peerID->length);
	if (peerID->data == 0)
		return SSLMemoryErr;

	/* Peer ID should contain a unique identifier for this server; in this case, IP
	address + port number */
	((unsigned long*)peerID->data)[0] = inAddr.s_addr;
	((long*)peerID->data)[1] = port;

	return SSLNoErr;
}


CWLSslConnection::CWLSslConnection(CWLSSLContext* ctx, in_addr addr, int port)
{
	m_Closed = true;
	m_Ctx = NULL;
	Init(ctx, addr, port);
}

CWLSslConnection::~CWLSslConnection()
{
	Delete();
}


void CWLSslConnection::Init(CWLSSLContext* ctx, in_addr addr, int port)
{
	m_Template = ctx;
	memmove((void*)&m_Address, (void*)&addr, sizeof(in_addr));
	m_Port = port;
}


int CWLSslConnection::AfterConnection(void *ioRef)
{
	int err = 0;
#ifdef __SSL__
	// Allocate a new context to handle this connection
	m_Ctx = (SSLContext*)malloc(SSLContextSize());;
	if (m_Ctx) {
		// Copy our template context into this context; this also sets ioRef for us
		err = SSLDuplicateChildContext(m_Template->GetContext(), m_Ctx, ioRef);
		if (err == SSLNoErr) {
			if (m_Template->isResumable()) {
				// Set PeerID to the identifier for this peer
				SSLBuffer peerID;
				err = SetPeer(&peerID, m_Address, m_Port);
				if (err == SSLNoErr) {
					err = SSLSetPeerID(m_Ctx, peerID);
					FreeSSLBuffer(&peerID);
				}
			}
			if (err == SSLNoErr) {
				m_Closed = false;
			  int res = SSLHandshake(m_Ctx);
				if (res != SSLNoErr) {
					WlTraceMsg("SSL: error in handshake [%d]\n", res);
				}
        else {
				  // get the negotiated ciphersuite
				  uint16              cipher;
				  SSLCiphersuiteInfo  csInfo;
          int                 cipherStatus;

				  cipherStatus = SSLGetNegotiatedCipher(m_Ctx, &cipher) ;

				  if (cipherStatus == SSLNoErr ) {
            cipherStatus = SSLGetCiphersuiteInfo( cipher, &csInfo );
					  if (cipherStatus == SSLNoErr ) {
              WlTraceMsg("Negotiated cipher is 0x%04X, [%s]\n", cipher, csInfo.suiteName);
					  }
            else {
              WlTraceMsg("SSL: Error calling SSLGetCiphersuiteInfo [%d]",cipherStatus);
            }
				  }
          else {
              WlTraceMsg("SSL: Error calling SSLGetNegotiatedCipher [%d]",cipherStatus);
          }
        }
 				return res;
			}
		}
		// m_Ctx created and error occurred
		Close();
		Delete();
  }
#endif
	return err;
}

void CWLSslConnection::Close()
{
	if (m_Ctx && !m_Closed)
		SSLClose(m_Ctx);
	m_Closed = true;
}

void CWLSslConnection::Delete()
{
	Close();
	if (m_Ctx) {
		SSLDeleteContext(m_Ctx);
		free((char*)m_Ctx);
		m_Ctx = NULL;
	}
}







// =======================================================================================

CWLSSLContext::CWLSSLContext()
{
	m_Cp = NULL;
	m_Ctx = NULL;
	m_DBRef = NULL;
	m_Resumable = false;
	m_CertificateFile = "";
	m_Version = SSL_Version_Undetermined;
	m_Random = NULL;
  m_CryptographicStrength = SSL_AllCrypto;
}

CWLSSLContext::CWLSSLContext(CWLConnectionsPool* cp)
{
	m_Cp = cp;
	m_Ctx = NULL;
	m_DBRef = NULL;
	m_Resumable = false;
	m_CertificateFile = "";
	m_Version = SSL_Version_Undetermined;
  m_CryptographicStrength = SSL_AllCrypto;
}

CWLSSLContext::~CWLSSLContext()
{
	SSLDeleteContext(m_Ctx);
	free((void*)m_Ctx);
	if (m_Random) {
		WLSSLDestroyRandom(&m_Random);
	}
	if (m_DBRef)
		delete m_DBRef;
}

int CWLSSLContext::Init(int         version,
                        bool        resumable,
                        const char* certFile,
                        const char* certPassword,
                        const char* certVerification,
                        int*        errCode,
                        int         CryptoStrength)
{
	int err = SSLNoErr;

	m_Version = version;
  m_CryptographicStrength = CryptoStrength;

  m_Resumable = resumable;
	m_CertificateFile = certFile ? certFile : "";

  m_Ctx = (SSLContext *)malloc(SSLContextSize());
  ConfigureSSLSession(m_Ctx, &m_Random);
//	AddDHParams(m_Ctx, &m_DHParams);

  SSLSetProtocolVersion(m_Ctx, (SSLProtocolVersion)version);

  // Set the cryptographic strength
  SSLSetCryptographicStrength(m_Ctx, (SSLCryptoStrength)m_CryptographicStrength);

  SSLSetProtocolSide(m_Ctx, SSL_ClientSide);

	// If this context will use the cache then we create a
  // new Session Reference to the connection pool which
  // holds in one of his members the ssl session database
  if (m_Resumable && m_Cp != NULL) {
    // Just to be on the safe side ....
    if (m_DBRef)
			delete m_DBRef;

    // Creating the new pointer to the connection pool
    m_DBRef = new CWLDBRef(m_Cp, this);

    // Make the ssl context aware that there is session caching
    // for that context
    SSLSetAddSessionFunc(m_Ctx, SSLAddSession);
	  SSLSetGetSessionFunc(m_Ctx, SSLGetSession);
	  SSLSetDeleteSessionFunc(m_Ctx, SSLDeleteSession);

    // Set the context to have the pointer to the session
    // holded in the connection pool
  	SSLSetSessionRef(m_Ctx, (void*)m_DBRef);
	}

	SSLSetCheckCertificateChainFunc(m_Ctx, CertificateCallback);
	SSLSetCheckCertificateRef(m_Ctx, (void *)certVerification);

	// add a certificate is exist
	if (certFile && *certFile) {
		if (!certPassword)
			certPassword = "";

    //Concatinate LoadEngine temporary path name to the file name.
	  //Galtech:
    RVCString leCertificateFile =  CArrangeMachineTempFiles::GetWlTempFullPath (m_Cp->GetCm()->GetJScx(), WLECopiedFile, ToANSI(certFile)) ;  

    FILE* fp = fopen(leCertificateFile, "r");
		if (!fp)
			err = X509CertificateFile;
		else {
			fclose(fp);
			int trials = 3;
			do {
				err = SSLLoadLocalIdentity(m_Ctx, (char*)leCertificateFile.data(), (char*)certPassword);
				trials--;
			} while (err != SSLNoErr && trials > 0);

			if (err == SSLNoErr)
				m_CertificateFile = certFile;
			else
				if (err > 0) {
					*errCode = err;
					err = X509BadCertificate;
				}
		}
	}

	return err;
}

bool CWLSSLContext::isMatching(int         version,
                               bool        resumable,
                               const char* certFile,
                               int         CryptoStrength)
{
	if (!certFile)
		certFile = "";

	if (m_Version == version &&
			m_Resumable == resumable &&
			!m_CertificateFile.compareTo(certFile) && // case sensitive for unix-like OS
      m_CryptographicStrength == CryptoStrength)
		return true;

	return false;
}




int CWLSSLContext::DetermineVersion(const char* version)
{
	if (!version)
		return (SSL_Version_Undetermined);

	RVCString sslVerStr(version);

  // Chaim - check default value first - igonre case not needed here
	if (sslVerStr.compareTo("SSL_Version_3_0_With_2_0_Hello", sslVerStr.ignoreCase) == 0)
		return (SSL_Version_3_0_With_2_0_Hello);
	else if (sslVerStr.compareTo("SSL_Version_2_0", sslVerStr.ignoreCase) == 0)
		return (SSL_Version_2_0);
	else if (sslVerStr.compareTo("SSL_Version_3_0", sslVerStr.ignoreCase) == 0)
		return (SSL_Version_3_0);
	else if (sslVerStr.compareTo("SSL_Version_3_0_Only", sslVerStr.ignoreCase) == 0)
		return (SSL_Version_3_0_Only);
	else if (sslVerStr.compareTo("TLS_Version_1.0_With_2_0_Hello", sslVerStr.ignoreCase) == 0)
		return (TLS_Version_1_0_With_2_0_Hello);
	else if (sslVerStr.compareTo("TLS_Version_1_0_Only", sslVerStr.ignoreCase) == 0)
		return (TLS_Version_1_0_Only);
	else if (sslVerStr.compareTo("TLS_Version_1_0", sslVerStr.ignoreCase) == 0)
		return (TLS_Version_1_0);
        else if (sslVerStr.compareTo("SSL_Version_Undetermined", sslVerStr.ignoreCase) == 0)
		return (SSL_Version_Undetermined);

	return (SSL_Version_Undetermined);
}

int CWLSSLContext::DetermineCryptographicStrength(const char* CryptoStrength)
{
	if (!CryptoStrength)
		return (SSL_AllCrypto);

	RVCString CryptoStrengthStr(CryptoStrength);
  if (CryptoStrengthStr.compareTo("SSL_AllCrypto", RVCString::ignoreCase) == 0)
		return (SSL_AllCrypto);
  if (CryptoStrengthStr.compareTo("SSL_StrongCryptoOnly", RVCString::ignoreCase) == 0)
		return (SSL_StrongCryptoOnly);
  if (CryptoStrengthStr.compareTo("SSL_ExportCryptoOnly", RVCString::ignoreCase) == 0)
		return (SSL_ExportCryptoOnly);
  if (CryptoStrengthStr.compareTo("SSL_ServerGatedCrypto", RVCString::ignoreCase) == 0)
		return (SSL_ServerGatedCrypto);

	return (SSL_Version_Undetermined);
}

#endif // __SSL__
