/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wlframecontainer.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

static JSBool
action_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside action_getProperty");
/*	
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
 	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLocation* location = (CWLLocation*)prv->GetObj();
	if (!location)
		return JS_TRUE;

	CWLFrame* frame = (CWLFrame*)prv->GetParent();
	if (!frame)
		return JS_TRUE;

	if (location && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_NAME :{	// return the name property of the form
			const char* name = location->Name();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}

    case PropertiesHandler::EN_ID : {	// return the id property of the form
			const char* id = location->Id();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_TARGET : {	// return the target property of the form
			const char* target = location->Target();
			return WLSetStringValue(cx, obj, prop, target, vp);
		}

    default:;
    }
	}
	
//???	return JS_GetProperty(cx, obj, id, vp);
	return JS_TRUE;
}


static JSBool
action_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside action_setProperty");

	return JS_TRUE;
}

JSBool action_ClassEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ID, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_TARGET, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}


static void
action_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside action_finalize");
	WLFinalize(cx, obj);
}



JSClass wl_ActionClass = {
	WLJSNAME_CLASS_ACTION,
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	action_getProperty,
	action_setProperty,
	action_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	action_finalize
};



static JSBool
action_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLocation* location = (CWLLocation*)prv->GetObj();
	if (!location)
		return JS_TRUE;

	// set rval to Href property
	const char* href = location->Href();
	JSString* str = JS_NewStringCopyZ(cx, href);
	if (str)
		*rval = STRING_TO_JSVAL(str);
	return JS_TRUE;
}

static JSBool
action_valueOf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLocation* location = (CWLLocation*)prv->GetObj();
	if (!location)
		return JS_TRUE;

	// set rval to Href property
	const char* href = location->Href();
	JSString* str = JS_NewStringCopyZ(cx, href);
	if (str)
		*rval = STRING_TO_JSVAL(str);
	return JS_TRUE;
}


static JSFunctionSpec action_methods[] = {
	{wl_toString_str,	action_toString,	0},
	{wl_valueOf_str,	action_valueOf,	0},
	{0}
};



/*
static JSBool
InitActionObject(JSContext *cx, JSObject *obj, jsint length, jsval *vector)
{
WlTraceMsg("initializing action object properties");
	// create the CPP object


	// add it's properties
	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_DOM_SCOPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_PROTOCOL))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HOSTNAME))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_PORT))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HOST))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_SEARCH))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_KVP))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HASH))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HREF))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_PROTOCOL))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_HOSTNAME))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_PORT))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_HOST))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_SEARCH))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_KVP))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_SEARCH_PAIRS))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_HASH))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_HREF))
		return JS_FALSE;

	return JS_TRUE;
}


static JSBool
WLAction(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	JSBool ok;

	ok = InitActionObject(cx, obj, 0, NULL);
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}


bool DllExport WLAddActionObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddActionObject(%x, %x)", cx, obj);
	// create an instance of type WLDocument
	JSObject* action = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_ACTION, &wl_ActionClass, NULL, JSPROP_PERMANENT);
	if (!action)
		return false;

	JS_SetPrivate(cx, action, NULL);
	//JS_SetParent(cx, dom, obj);
	
	// define it's properties & structures
	if (!InitActionObject(cx, action, 0, NULL))
		return false;

	return true;
}
*/


JSObject* WLNewActionObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewActionObject(%x, %x)", cx, obj);
	// create an instance of type WLLink
	JSObject* proto = NULL;
	JSObject* action = WLNewObject(cx, &wl_ActionClass, proto, obj);
	if (!action)
		return NULL;

	return action;
}


bool DllExport WLAddActionClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddActionClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_ActionClass, NULL, 1, NULL, action_methods, NULL, NULL))
		return false;

	return true;
}
