/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <ace/OS.h>
#ifdef __TV_WINDOWS__
#include "wlxmlfuncs.h"
#endif
#include <string.h>
#include "rvjsapi.h"
#include "wlframecontainer.h"
#include "wlarray.h"
#include "wltable.h"
#include "wldocument.h"
#include "wlxml.h"
#include "wlelement.h"
#include "wlform.h"
#include "wloption.h"
#include "wllocation.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "jsprivate.h"
#include "jshttpnames.h"
#include "wlfuncs.h"
#include "tracedll.h"

#include "propertieshandler.h"

class CWLXmlDocument;

extern CWLElement*   WLGetElementItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLForm*      WLGetFormItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLFrame*     WLGetFrameItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLEmbed*     WLGetEmbedItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLUrl*       WLGetCSSItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLXmlDocument*  WLGetXmlItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLKvp*       WLGetKvpItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLLink*      WLGetLinkItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLLocation*  WLGetLocationItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLMeta*      WLGetMetaItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLOption*    WLGetOptionItemPtr(RVOrdered* array, size_t index);
extern CWLScript*    WLGetScriptItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLApplet*    WLGetAppletItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLTable*     WLGetTableItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLTableRow*  WLGetRowItemPtr(RVOrdered* array, size_t index, const char* key);
extern CWLTableCell* WLGetCellItemPtr(RVOrdered* array, size_t index, const char* key);
extern RVCollectableString* WLGetStringItemPtr(RVOrdered* array, size_t index, const char* key);


static JSBool
array_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside array_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
	jsint index = -1;
	char* key = NULL;
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
  if (!prv){
		return JS_TRUE;
  }
	CWLFrame* frame = (CWLFrame*)prv->GetParent();
	if (!frame)
		return JS_TRUE;
	frameContainer* frameCont = frame->GetFrameContainer();
	if (!frameCont)
		return JS_TRUE;
	JSFrameMode	  frameMode = frameCont->GetFrameMode();

	RVOrdered* array = (RVOrdered*)prv->GetObj();
	if (!array)
		return JS_TRUE;

	if (array && (JSVAL_IS_STRING(id) || JSVAL_IS_INT(id))) {
		if (JSVAL_IS_INT(id))
			index = JSVAL_TO_INT(id);
		else
			key = JS_GetStringBytes(JSVAL_TO_STRING(id));

		// get length property
		if (key)
			if(!strcmp(key, WLJSNAME_PROP_LENGTH)) {
				*vp = INT_TO_JSVAL(array->entries());
				return JS_TRUE;
			}

		const char* name = prv->name();

    switch (PropertiesHandler::GetClassId(name))
    {
    case PropertiesHandler::EN_CLASS_KVP :
    case PropertiesHandler::EN_CLASS_HEADER :{
			CWLKvp* kvp = WLGetKvpItemPtr(array, index, key);
      return WLSetKvpValue(cx, obj, frame, kvp, vp);}
	
    case PropertiesHandler::EN_CLASS_FRAME :{
			CWLFrame* frame = WLGetFrameItemPtr(array, index, key);
			frameCont->SetFrameMode(JS_Frame);
      return WLSetFrameValue(cx, obj, frame, frame, vp);}

    case PropertiesHandler::EN_CLASS_WINDOW :{
			CWLFrame* window = WLGetFrameItemPtr(array, index, key);
      return WLSetWindowValue(cx, obj, frame, window, vp);}

    case PropertiesHandler::EN_CLASS_ELEMENT :{
			CWLElement* element = WLGetElementItemPtr(array, index, key);
      return WLSetElementValue(cx, obj, frame, element, vp);}

    case PropertiesHandler::EN_CLASS_OPTION :{
			CWLOption* option = WLGetOptionItemPtr(array, index);
      return WLSetOptionValue(cx, obj, frame, option, vp);}

    case PropertiesHandler::EN_CLASS_FORM :{
			CWLForm* form = WLGetFormItemPtr(array, index, key);
      return WLSetFormValue(cx, obj, frame, form, vp);}

    case PropertiesHandler::EN_CLASS_LINK :{
			CWLLink* link = WLGetLinkItemPtr(array, index, key);
      return WLSetLinkValue(cx, obj, frame, (CWLUrl*)link, vp);}

    case PropertiesHandler::EN_CLASS_IMG :{
			CWLEmbed* img = WLGetEmbedItemPtr(array, index, key);
      return WLSetImgValue(cx, obj, frame, img, vp);}

    case PropertiesHandler::EN_CLASS_EMBED :{
			CWLEmbed* embed = WLGetEmbedItemPtr(array, index, key);
      return WLSetEmbedValue(cx, obj, frame, embed, vp);}

    case PropertiesHandler::EN_CLASS_CSS :{
			CWLUrl* css = WLGetCSSItemPtr(array, index, key);
      return WLSetCSSValue(cx, obj, frame, css, vp);}

    case PropertiesHandler::EN_CLASS_SCRIPT :{
			CWLScript* script = WLGetScriptItemPtr(array, index, key);
      return WLSetScriptValue(cx, obj, frame, script, vp);}

    case PropertiesHandler::EN_CLASS_APPLET :{
			CWLApplet* applet = WLGetAppletItemPtr(array, index, key);
      return WLSetAppletValue(cx, obj, frame, applet, vp);}

    case PropertiesHandler::EN_CLASS_LOCATION :{
			CWLLocation* location = WLGetLocationItemPtr(array, index, key);
      return WLSetLocationValue(cx, obj, frame, location, vp);}

    case PropertiesHandler::EN_CLASS_META :{
			CWLMeta* meta = WLGetMetaItemPtr(array, index, key);
      return WLSetMetaValue(cx, obj, frame, meta, vp);}

    case PropertiesHandler::EN_CLASS_TABLE :{
			//have to add support for document.wlTables.myTable
			if(key)
			{
				CWLTable* table = ((CWLTableCollection*)array)->findByID(key);
				return WLSetTableValue(cx, obj, frame, table, vp);
			}
			CWLTable* table = WLGetTableItemPtr(array, index, key);
      return WLSetTableValue(cx, obj, frame, table, vp);}

    case PropertiesHandler::EN_CLASS_ROW :{
			CWLTableRow* row= WLGetRowItemPtr(array, index, key);
      return WLSetRowValue(cx, obj, frame, row, vp);}

    case PropertiesHandler::EN_CLASS_CELL :{
			if(key)
			{
				CWLTable* table = ( (CWLCellCollection*)array )->getOwnerTable();
				if (table) {
					CWLCellCollection* cells = table->getCellRange(key);
					return WLSetCellArrayValue(cx, obj, frame, cells, vp);
				}
			}
			else {
				CWLTableCell* cell= WLGetCellItemPtr(array, index, key);
				return WLSetCellValue(cx, obj, frame, cell, vp);
			}}

      
    case PropertiesHandler::EN_CLASS_XML :{
			CWLXmlDocument* xml = WLGetXmlItemPtr(array, index, key);
			if(!xml)
				return JS_TRUE;
#ifdef __TV_WINDOWS__
			xml->SetDom(frame);
      return WLXmlSetDocumentInterfaceValue(cx, obj, frame, xml, true, vp);}
#else
      return WLSetXmlValue(cx, obj, frame, xml, vp);}
#endif

    case PropertiesHandler::EN_CLASS_STRING :{
			RVCollectableString* string = WLGetStringItemPtr(array, index, key);
			const char* key = NULL;
			if (string)
				key = string->data();
      return WLSetStringValue(cx, obj, NULL, key, vp);}

    default:;
    }
   }

	return JS_TRUE;
}

static JSBool
array_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside array_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_FALSE;

	RVOrdered* array = (RVOrdered*)prv->GetObj();
	if (array && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_KEY)) {	// set the Key property
			array->setKey(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_VALUE)) {	// return the Value property
			array->setValue(WLStringValue(*vp));
			return JS_TRUE;
		}
	}
*/

	return JS_TRUE;
}

static JSBool
array_Get(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{

  CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	const char* name = prv->name();

  if (argc != 1) {
    ReportJSError(cx, JS_DOM_ERROR_FILE, 100, name);
    return JS_FALSE ;
  }

  return array_getProperty(cx, obj, argv[0] , rval) ;

}

JSBool array_ClassEnumerate(JSContext *cx, JSObject *obj ){
  JS_DefineProperty(cx, obj, "length", JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  // Private of array calls is RVOrdered
  CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);

  if (!prv){
    return JS_TRUE;
  }

  RVOrdered* arrayValue = (RVOrdered*)(prv->GetObj());
  for (int i = arrayValue->entries() - 1; i >= 0; i--){
    JS_DefineProperty(cx, obj, (const char*)(i), JSVAL_VOID, NULL, NULL, JSPROP_ENUMERATE | JSPROP_INDEX);
  }

  return JS_TRUE;
}


static void
array_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside array_finalize(%x, %x)", cx, obj);
	WLFinalize(cx, obj);
}



JSClass wl_ArrayClass = {
	WLJSNAME_CLASS_ARRAY,
	JSCLASS_HAS_PRIVATE,//ptr to c++ structure that info regarding the object it represents
	JS_PropertyStub,   //callback to C++
	JS_PropertyStub,	//--"--
	array_getProperty,
	array_setProperty,
	array_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	array_finalize
};

static JSFunctionSpec array_methods[] = {
	{WLJSNAME_HTTP_ARRAY_GET,	array_Get,	  1, JSPROP_READONLY, 0},
	{0,0,0,0,0}
};

/*
static JSBool
array_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("array_toString");
	char name[128];
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	sprintf(name, "[array of class %s]", prv->name());
	JSString* str = JS_NewStringCopyZ(cx, name);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}

static JSBool
array_valueOf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("array_valueOf");
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	RVOrdered* array = (RVOrdered*)prv->GetObj();
	if (!array)
		return JS_TRUE;

	*rval = INT_TO_JSVAL(array->entries());

	return JS_TRUE;
}


*/

/*
static JSBool
InitArrayObject(JSContext *cx, JSObject *obj, jsint argc, jsval *argv)
{
WlTraceMsg("initializing Array object properties");

	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_CLASS_NAME))
		return JS_FALSE;

	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_DOM_SCOPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_LENGTH))
		return JS_FALSE;

	int index = 0;

	for (int i = 0; i < argc; i++) {
		if (JSVAL_IS_STRING(argv[0])) {	// array of strings
			RVOrdered* array;
			if (i == 0) {
				array = new RVOrdered;
				JS_SetPrivate(cx, obj, (void*)array);
			}
			if (!array)
				return JS_FALSE;

			if (JSVAL_IS_STRING(argv[i])) {
				JSString* str = JSVAL_TO_STRING(argv[i]);
				array->insert(new RVCollectableString(JS_GetStringBytes(str)));
				WLInitProperty(cx, obj, (const char*)index);
				index++;
			}
		}
		else if (WLIsObjectOfClass(cx, argv[0], WLJSNAME_CLASS_KVP)) {	// array of CWLKvp
			CWLArray* array;
			if (i == 0) {
				array = new CWLArray;
				JS_SetPrivate(cx, obj, (void*)array);
			}
			if (!array)
				return JS_FALSE;

			if (WLIsObjectOfClass(cx, argv[i], WLJSNAME_CLASS_KVP)) {
				CWLKvp* kvp = (CWLKvp*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[i]));
				array->addKVP(kvp);
				// alias the item's key value with an index
				WLInitProperty(cx, obj, kvp->Key());
				JS_AliasElement(cx, obj, kvp->Key(), index);
				index++;
			}
		}
		else if (WLIsObjectOfClass(cx, argv[0], WLJSNAME_CLASS_ELEMENT)) {	// array of CWLElement
			RVOrdered* array;
			if (i == 0) {
				array = new RVOrdered;
				JS_SetPrivate(cx, obj, (void*)array);
			}
			if (!array)
				return JS_FALSE;

			if (WLIsObjectOfClass(cx, argv[i], WLJSNAME_CLASS_ELEMENT)) {
				CWLElement* element = (CWLElement*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[i]));
				array->insert(element);
				// alias the item's name value with an index
				const char* name = element->Name();
				if (name && *name) {
					WLInitProperty(cx, obj, name);
					JS_AliasElement(cx, obj, name, index);
				}
				index++;
			}
		}
		else if (WLIsObjectOfClass(cx, argv[0], WLJSNAME_CLASS_FRAME)) {	// array of CWLFrame
			RVOrdered* array;
			if (i == 0) {
				array = new RVOrdered;
				JS_SetPrivate(cx, obj, (void*)array);
			}
			if (!array)
				return JS_FALSE;

			if (WLIsObjectOfClass(cx, argv[i], WLJSNAME_CLASS_FRAME)) {
				CWLFrame* frame = (CWLFrame*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[i]));
				array->insert(frame);
				// alias the item's name value with an index
				const char* name = frame->GetName().data();
				if (name && *name) {
					WLInitProperty(cx, obj, name);
					JS_AliasElement(cx, obj, name, index);
				}
				index++;
			}
		}
		else if (WLIsObjectOfClass(cx, argv[0], WLJSNAME_CLASS_LINK)) {	// array of CWLLink
			RVOrdered* array;
			if (i == 0) {
				array = new RVOrdered;
				JS_SetPrivate(cx, obj, (void*)array);
			}
			if (!array)
				return JS_FALSE;

			if (WLIsObjectOfClass(cx, argv[i], WLJSNAME_CLASS_LINK)) {
				CWLLink* link = (CWLLink*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[i]));
				array->insert(link);
				WLInitProperty(cx, obj, (const char*)index);
				index++;
			}
		}
		else if (WLIsObjectOfClass(cx, argv[0], WLJSNAME_CLASS_FORM)) {	// array of CWLForm
			RVOrdered* array;
			if (i == 0) {
				array = new RVOrdered;
				JS_SetPrivate(cx, obj, (void*)array);
			}
			if (!array)
				return JS_FALSE;

			if (WLIsObjectOfClass(cx, argv[i], WLJSNAME_CLASS_FORM)) {
				CWLForm* form = (CWLForm*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[i]));
				array->insert(form);
				// alias the item's name value with an index
				WLInitProperty(cx, obj, form->Name());
				JS_AliasElement(cx, obj, form->Name(), index);
				index++;
			}
		}
	}

	return JS_TRUE;
}


static JSBool
WLArray(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("WLArray constructor");
	JSBool ok;

	ok = InitArrayObject(cx, obj, argc, argv);
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}


//add array object
bool WLAddArrayObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddArrayObject(%x, %x)", cx, obj);
	// create an instance of type WLArray
	JSObject* array = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_ARRAY, &wl_ArrayClass, NULL, JSPROP_PERMANENT);
	if (!array)
		return false;

	JS_SetPrivate(cx, array, NULL);

	// define it's properties & structures
	if (!InitArrayObject(cx, array, 0, NULL))
		return false;

	return true;
}
*/



JSObject* WLNewArrayObject(JSContext *cx, JSObject *obj)
{
	// create an instance of type WLArray
	JSObject* proto = NULL;
	JSObject* array = WLNewObject(cx, &wl_ArrayClass, proto, obj);
	if (!array)
		return NULL;

WlTraceMsg("WLNewArrayObject(%x, %x) ==> %x", cx, obj, array);
	return array;
}


bool DllExport WLAddArrayClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddArrayClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_ArrayClass, NULL, 1, NULL, array_methods, NULL, NULL))
		return false;

	return true;
}
