/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "wllocation.h"
#include "jsprivate.h"
#include "TraceDll.h"
#include "wlstyle.h"

#include "propertieshandler.h"

static JSBool
css_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside css_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
 	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLStyle* location = (CWLStyle*)prv->GetObj();
	if (!location)
		return JS_TRUE;

	CWLFrame* frame = (CWLFrame*)prv->GetParent();
	if (!frame)
		return JS_TRUE;

	if (location && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_ID : {	// return the id property
			const char* id = location->GetId();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_INNERHTML : {	// return the innerHTML property
			const char* innerHtml = location->GetCssText();
			return WLSetStringValue(cx, obj, prop, innerHtml, vp);
		}

    case PropertiesHandler::EN_WLSOURCE_INDEX : { // return the WLSourceIndex property
			int index = location->GetIndex();
			return WLSetIntValue(cx, obj, prop, index, vp);
		}

    case PropertiesHandler::EN_FILE_SIZE : { // return the FileSize property
			int nFileSize = location->GetTransactionObj()->GetFileSize();
			return WLSetIntValue(cx, obj, prop, nFileSize, vp);
		}

    case PropertiesHandler::EN_CHECKSUM : { // return the CheckSum property
			int nCheckSum = location->GetTransactionObj()->GetCheckSum();
			return WLSetIntValue(cx, obj, prop, nCheckSum, vp);
		}

    case PropertiesHandler::EN_OPEN_TIME : { // return the OpenTime property
			double dTime = location->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_OPEN_CONNECTION);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_SEND_TIME : { // return the SendTime property
			double dTime = location->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_SEND);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_WAIT_TIME : { // return the WaitTime property
			double dTime = location->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_WAIT);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_RECEIVE_TIME : { // return the ReceiveTime property
			double dTime = location->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_RECEIVE);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    default:;
    }
	}
	return JS_TRUE;
}


static JSBool
css_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside css_setProperty");
	return JS_TRUE;
}

JSBool css_ClassEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_INNERHTML, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ID, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}

static void
css_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside css_finalize");
	WLFinalize(cx, obj);
}



JSClass wl_CSSClass = {
	WLJSNAME_CLASS_CSS,
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	css_getProperty,
	css_setProperty,
	css_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	css_finalize
};


JSObject* WLNewCSSObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewCSSObject(%x, %x)", cx, obj);
	// create an instance of type WLLink
	JSObject* proto = NULL;
	JSObject* css = WLNewObject(cx, &wl_CSSClass, proto, obj);
	if (!css)
		return NULL;

	return css;
}


bool DllExport WLAddCSSClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddCSSClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_CSSClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
