/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvjsapi.h"

#include "wlerrorlib.h"
#include "domitem.h"
#include "LoadReportInfo.h"
#include "wlfuncs.h"
#include "loadengineglobals.h"
#include "gt_strwrapper.h"


#define WLJSNAME_DOM_REPOSITORY_OPEN           "Open"
#define WLJSNAME_DOM_REPOSITORY_LENGTH         "length"
#define WLJSNAME_DOM_REPOSITORY_GET_ITERATOR   "GetIterator"

static JSBool
DomRepositoryCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  if (argc != 0)  {
    WLErrorException e(JS_DOM_ERROR_FILE, 110);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  CWLDomRepository *rep = new CWLExpectedDomRepository();

  JS_DefineProperty(cx,obj,WLJSNAME_DOM_REPOSITORY_LENGTH,INT_TO_JSVAL(0),NULL,NULL,JSPROP_ENUMERATE);

  JS_SetPrivate(cx, obj, rep);

  *rval = OBJECT_TO_JSVAL(obj);
	return JS_TRUE;
}

static JSBool
DomRepositoryOpen(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  char *name;
  RVCString leName ; 

	if (argc != 1) {
    WLErrorException e(JS_DOM_ERROR_FILE, 111);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
	if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(JS_DOM_ERROR_FILE, 111);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
  else {
    name = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
    //Concatinate LoadEngine temporary path name to the file name.
	  //Galtech: we need ANSI not UTF
    leName =  CArrangeMachineTempFiles::GetWlTempFullPath (cx, WLECopiedFile, ToANSI(name)) ;
  }

	CWLDomRepository *rep = (CWLDomRepository*)JS_GetPrivate(cx, obj);
  if( ! rep )
    return JS_TRUE;

  if( !rep->open(leName) )
  {
    WLErrorException e(JS_DOM_ERROR_FILE, 112, name);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

 	jsval v = INT_TO_JSVAL(rep->length());
  JS_SetProperty(cx,obj,WLJSNAME_DOM_REPOSITORY_LENGTH,&v);

	return JS_TRUE;
}


static JSBool
DomRepositoryGetIterator(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLDomRepository *pRep = (CWLDomRepository*)JS_GetPrivate(cx, obj);
  if (!pRep)
    return JS_TRUE;

  JSBool rtn = JS_FALSE;
  if (pRep->GetItemHashDictionary()) {
     JSObject* iter = IteratorNewObject( cx, *pRep->GetItemHashDictionary() );
     if( iter ) {
       *rval = OBJECT_TO_JSVAL(iter);
       rtn = JS_TRUE;
     }
  }

  return rtn;
}


static void
DomRepositoryFinalize(JSContext *cx, JSObject *obj)
{
	CWLDomRepository *rep = (CWLDomRepository*)JS_GetPrivate(cx, obj);
  if( rep )
    delete rep;
}


JSClass DomRepositoryClass = {
  WLJSNAME_DOM_REPOSITORY_CLASS,
  JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,	// add Property
  JS_PropertyStub,	// del Property
  JS_PropertyStub,	// get Property
  JS_PropertyStub,	// set Property
  JS_EnumerateStub,	
	JS_ResolveStub,
	JS_ConvertStub,
	DomRepositoryFinalize
};

static JSFunctionSpec DomRepository_methods[] = {
	{WLJSNAME_DOM_REPOSITORY_OPEN,	DomRepositoryOpen,	1},
  {WLJSNAME_DOM_REPOSITORY_GET_ITERATOR, DomRepositoryGetIterator, 0},
	{0}
};


bool AddDomRepositoryClass(JSContext *cx, JSObject *obj)
{
  if (!JS_InitClass(cx, obj, NULL, &DomRepositoryClass, DomRepositoryCtor, 1, NULL, DomRepository_methods, NULL, NULL))
		return false;
  
  return true;
}
