/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wlfuncs.h"
#include "wlelement.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

static JSBool
element_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside element_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

  CWLFrame* frame = prv->GetParent();

	CWLElement* element = (CWLElement*)prv->GetObj();
	if (!element)
		return JS_TRUE;

	RVCString type = element->Type();
	type.toLower();

	if (element && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);
		
		
		// get length property
    PropertiesHandler::PropertyNameId PropertyId = PropertiesHandler::EN_NONE ;
		if (prop && ((PropertyId=PropertiesHandler::GetPropertyId(prop)) == PropertiesHandler::EN_LENGTH)) {
			if(type == "select")
				*vp = INT_TO_JSVAL(element->Length());
			return JS_TRUE;
		}


    switch(PropertyId)
    {
    case PropertiesHandler::EN_NAME :{// return the Name property
			const char* name = element->Name();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}

    case PropertiesHandler::EN_TYPE :{	// return the Type property
			const char* type = element->Type();
			return WLSetStringValue(cx, obj, prop, type, vp);
		}

    case PropertiesHandler::EN_VALUE : {	// return the Value property
			if(type != "select")
			{
				const char* value = element->Value();
				return WLSetStringValue(cx, obj, prop, value, vp);
			}
			else
				return JS_TRUE;
		}

    case PropertiesHandler::EN_DEFAULT_VALUE : {	// return the default Value property
			if(type == "text" || type == "textarea" || type == "password")
			{
				const char* value = element->DefaultValue();
				return WLSetStringValue(cx, obj, prop, value, vp);
			}
			else
				return JS_TRUE;
		}

    case PropertiesHandler::EN_SELECTED_INDEX : {	// return the selected index property - only for SELECT
			if(type == "select")
			{
				int value = element->SelectedIndex();
				return WLSetIntValue(cx, obj, prop, value, vp);
			}
			else
				return JS_TRUE;
		}

    case PropertiesHandler::EN_OPTIONS : {	// return the Options property
			if(type == "select")
			{
				RVOrdered* options = element->Options();
				return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_OPTION, (void**)&options, vp);
			}
			else
				return JS_TRUE;
		}

    case PropertiesHandler::EN_CHECKED : {	// return the checked property - only for SELECT
			if(type == "radio" || type == "checkbox")
			{
				bool value = element->Checked();
				return WLSetBoolValue(cx, obj, prop, value, vp);
			}
			else
				return JS_TRUE;
		}	

    case PropertiesHandler::EN_DEFAULT_CHECKED : {	// return the length property - only for SELECT
			if(type == "radio" || type == "checkbox")
			{
				bool value = element->DefaultChecked();
				return WLSetBoolValue(cx, obj, prop, value, vp);
			}
			else
				return JS_TRUE;
		}
    case PropertiesHandler::EN_ID : {	// return the id property
			const char* id = element->Id();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    default:;
    }
  }
	return JS_TRUE;
}

static JSBool
element_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside element_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLElement* element = (CWLElement*)prv->GetObj();
	if (element && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_NAME)) {	// return the Name property
			element->setName(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_TYPE)) {	// return the Type property
			element->setType(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_VALUE)) {	// return the Value property
			element->setValue(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_OPTIONS)) {	// return the Options property
			element->setOptions(WLArrayValue(cx, *vp));
			return JS_TRUE;
		}
	}
*/

	return JS_TRUE;
}

JSBool element_ClassEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ID, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_TAG_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_INNERHTML, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_CHECKED, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_DEFAULT_CHECKED, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_SELECTED_INDEX, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_DEAFAULT_VALUE, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_OPTIONS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_TYPE, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}

static void
element_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside element_finalize");
	WLFinalize(cx, obj);
}

JSClass wl_ElementClass = {
	WLJSNAME_CLASS_ELEMENT,
	JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,
	JS_PropertyStub,
	element_getProperty,
	element_setProperty,
  element_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	element_finalize
};


/*
static JSBool
element_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	// set rval to name
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLElement* element = (CWLElement*)prv->GetObj();
	if (!element)
		return JS_TRUE;

	const char* name = element->Name();
	JSString* str = JS_NewStringCopyZ(cx, name);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}


static JSFunctionSpec element_methods[] = {
	{wl_toString_str,	element_toString,	0},
	{0}
};
*/


/*
static JSBool
InitElementObject(JSContext *cx, JSObject *obj, jsint length, jsval *vector)
{
WlTraceMsg("initializing element object properties");

	CWLElement* element = new CWLElement;
	JS_SetPrivate(cx, obj, (void*)element);

	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_DOM_SCOPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_NAME))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_VALUE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_TYPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_OPTIONS))
		return JS_FALSE;

	return JS_TRUE;
}


static JSBool
WLElement(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	JSBool ok;

	ok = InitElementObject(cx, obj, 0, NULL);
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}


bool WLAddElementObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddElementObject(%x, %x)", cx, obj);
	// create an instance of type WLHttp
	JSObject* element = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_FORM, &wl_ElementClass, NULL, JSPROP_PERMANENT);
	if (!element)
		return false;

	// define it's properties & structures
	if (!InitElementObject(cx, element, 0, NULL))
		return false;

	return true;
}
*/

JSObject* WLNewElementObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewElementObject(%x, %x)", cx, obj);
	// create an instance of type WLElement
	JSObject* proto = NULL;
	JSObject* element = WLNewObject(cx, &wl_ElementClass, proto, obj);
	if (!element)
		return NULL;

	return element;
}


bool DllExport WLAddElementClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddElementClass(%x, %x)", cx, obj);
	// define a class of type WLHttp
	//if (!JS_InitClass(cx, obj, NULL, &wl_ElementClass, WLElement, 1, NULL, element_methods, NULL, NULL))
	if (!JS_InitClass(cx, obj, NULL, &wl_ElementClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
