/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <ace/OS.h>
#ifdef __TV_WINDOWS__
#include "wlxmlfuncs.h"
#endif
#include <string.h>
#include "rvjsapi.h"
#include "wlxml.h"
#include "wlframecontainer.h"
#include "commandmanager.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "jshttpnames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "tracedll.h"
#include "httpdefines.h"
#include "workspace.h"

#include "propertieshandler.h"

#define CWLFrame CWLDocument
#define CWLPage frameContainer

static JSBool
frame_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	WlTraceMsg("Inside frame_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLFrame* doc = (CWLFrame*)prv->GetParent();
	if (!doc)
		return JS_TRUE;
	frameContainer* frameCont = doc->GetFrameContainer();
	if (!frameCont)
		return JS_TRUE;
	JSFrameMode	  frameMode = frameCont->GetFrameMode();

	CWLFrame* frame = (CWLFrame*)prv->GetObj();
	if (frame && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_IMAGES : {	// return the Images property
			RVOrdered* images = frame->m_SrcList;
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_IMG, (void**)&images, vp);
		} 

    case PropertiesHandler::EN_EMBEDS : {	// return the Embeds property
			RVOrdered* embeds = frame->m_EmbedList;
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_EMBED, (void**)&embeds, vp);
		} 

    case PropertiesHandler::EN_STYLESHEETS : {	// return the CSSs property
			RVOrdered* css = frame->m_StyleSheets;
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_CSS, (void**)&css, vp);
		} 

    case PropertiesHandler::EN_LINKS : {	// return the Links property
			RVOrdered* links = frame->m_Links;
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_LINK, (void**)&links, vp);
		}

    case PropertiesHandler::EN_FORMS : {	// return the Forms property
			RVOrdered* forms = frame->m_Forms;
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_FORM, (void**)&forms, vp);
		}

    case PropertiesHandler::EN_SCRIPTS : {	// return the Scripts property
			RVOrdered* scripts = frame->m_Scripts;
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_SCRIPT, (void**)&scripts, vp);
		}

    case PropertiesHandler::EN_APPLETS : {	// return the Applets property
			RVOrdered* applets = frame->m_AppletList;
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_APPLET, (void**)&applets, vp);
		}

    case PropertiesHandler::EN_WLMETAS : {	// return the wlMetas property
			RVOrdered* metas = frame->m_MetaTags;
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_META, (void**)&metas, vp);
		}

    case PropertiesHandler::EN_FRAMES : {	// return the Frames property
			//document.frames and document.frames[0].frames -- webload/browser mode
			//reset JSFrameMode so if you get document.forms next you won't change
			//the domMode -- document.forms works in both modes
			RVOrdered* frames = frame->m_Frames;

      return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_WINDOW, (void**)&frames, vp);
		}

    case PropertiesHandler::EN_LOCATION : {	// return the Location property
			//document.frames[0].location - frames[0] could be document\window
			//reset JSFrameMode so if you get document.location next you won't change
			//the domMode -- document.location works in both modes
			frameCont->SetFrameMode(JS_Document);
			CWLLocation* location = &(frame->GetLocation());
			return WLSetLocationValue(cx, obj, frame, location, vp);
		}

    case PropertiesHandler::EN_TITLE : {	// return the title property
			const char* title = frame->GetTitle();
			return WLSetStringValue(cx, obj, prop, title, vp);
		}

    case PropertiesHandler::EN_URL : {	// return the URL property
			const char* url = frame->GetDocUrl().data();
			return WLSetStringValue(cx, obj, prop, url, vp);
		}

    case PropertiesHandler::EN_DOCUMENT : {	// return the Document property

      // @@Chaim - PIDE
      // I don't know how important this check is but it seems we create a document.document (as far as I can make
      // out it's the proto) in the DOM and so when we watch a 'document' in the watch window, we get an error message. So I'm commenting it out.
      /*
			if(frameMode != JS_Frame)
			{//error
				if(!ReportJSError(cx, JS_DOM_ERROR_FILE, 10, prop, prop))
					return JS_TRUE;
				return JS_FALSE;
			 //the user used document.document which is forbidden in both doms
			 //	or document.frames[0].document  which is forbidden in WEBLOAD_DOM

			}
      */
      
			//The mode is either BROWSER_DOM or ALL_DOM and --
			//we have something like document.frames[0].document
			//which is possible in BROWSER mode only
			//note that document.document is also permissible
			//and therfore we can only assume that the user wrote frames[0].document
			frameCont->SetFrameMode(JS_None);
			return  WLSetFrameValue(cx, obj, frame, frame, vp);
		}

/**********************add webload aux's to browser dom***************/
    case PropertiesHandler::EN_WLXMLS : {
			RVOrdered* xmls = frame->m_Xmls;
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_XML, (void**)&xmls, vp);
		}

    case PropertiesHandler::EN_WLVERSION : {
			const char* version = frame->GetProtocolVersion();
			return WLSetStringValue(cx, obj, prop, version, vp);
		}

    case PropertiesHandler::EN_WLSTATUS_NUMBER : {
			int statusNum = (int)frame->GetStatusNumber();
			return WLSetIntValue(cx, obj, prop, statusNum, vp);
		}

    case PropertiesHandler::EN_WLSTATUS_LINE : {
			const char* statusLine = frame->GetStatusLine();
			return WLSetStringValue(cx, obj, prop, statusLine, vp);
		}

    case PropertiesHandler::EN_WLHEADERS : {
			CWLArray* headers = &frame->GetHeader();
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_HEADER, (void**)&headers, vp);
		}

    case PropertiesHandler::EN_WLSOURCE : {	// return the Source property
			const char* source = frame->m_HtmlPagePtr ? frame->m_HtmlPagePtr->data() : NULL;
			size_t length = frame->m_HtmlPagePtr ? frame->m_HtmlPagePtr->length() : 0;
			return WLSetStringValue(cx, obj, prop, source, vp, length);
		}

    case PropertiesHandler::EN_WLTABLES : {	// return the wlTables property
			RVOrdered* tables = (RVOrdered*)frame->m_Tables;
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_TABLE, (void**)&tables, vp);
		}

    case PropertiesHandler::EN_NAME : {	// return the Name property
			const char* name = frame->GetFrameName();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}

    case PropertiesHandler::EN_ID : {	// return the id property
			const char* id = frame->Id();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_TARGET : {	// return the target property
			const char* target = frame->Target();
			return WLSetStringValue(cx, obj, prop, target, vp);
		}

    case PropertiesHandler::EN_TAGNAME : {	// return the tagName property
			return WLSetStringValue(cx, obj, prop, "FRAME", vp);
		}

    case PropertiesHandler::EN_WLSOURCE_INDEX: { // return the WLSourceIndex property
			int index = frame->GetIndex();
			return WLSetIntValue(cx, obj, prop, index, vp);
		}

    case PropertiesHandler::EN_FILE_SIZE : { // return the FileSize property
			int nFileSize = frame->GetTransactionObj()->GetFileSize();
			return WLSetIntValue(cx, obj, prop, nFileSize, vp);
		}

    case PropertiesHandler::EN_CHECKSUM : { // return the CheckSum property
			int nCheckSum = frame->GetTransactionObj()->GetCheckSum();
			return WLSetIntValue(cx, obj, prop, nCheckSum, vp);
		}

    case PropertiesHandler::EN_OPEN_TIME : { // return the OpenTime property
			double dTime = frame->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_OPEN_CONNECTION);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_SEND_TIME : { // return the SendTime property
			double dTime = frame->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_SEND);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_WAIT_TIME : { // return the WaitTime property
			double dTime = frame->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_WAIT);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_RECEIVE_TIME : { // return the ReceiveTime property
			double dTime = frame->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_RECEIVE);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    default:;
    }
	}

	return JS_TRUE;
}


static JSBool
frame_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside frame_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLFrame* frame = (CWLFrame*)prv->GetObj();
	if (frame && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_FRAMES)) {	// return the Frames property
// 			frame->setFrames(WLArrayValue(cx, *vp));
 			frame->m_Frames = WLArrayValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_LINKS)) {	// return the Links property
// 			frame->setLinks(WLArrayValue(cx, *vp));
 			frame->m_Links = WLArrayValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_FORMS)) {	// return the Forms property
// 			frame->setForms(WLArrayValue(cx, *vp));
 			frame->m_Forms = WLArrayValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_LOCATION)) {	// return the Location property
// 			frame->setLocation(WLLocationValue(cx, *vp));
 			frame->GetLocation() = *WLLocationValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_NAME)) {	// return the Name property
 			frame->setName(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_REFERRER)) {	// return the Referer property
 			frame->setReferrer(WLLocationValue(cx, *vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SPIDERSESSION)) {	// return the SPIDERSESSION property
 			frame->setSPIDERSESSION(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PROTOCOL_VERSION)) {	// return the protocol version
 			frame->setProtocol(WLDoubleValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_STATUS_NUMBER)) {	// return the reply's status number
 			frame->setStatusNumber(WLIntValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_STATUS_LINE)) {	// return the reply status description
 			frame->setStatusLine(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HEADER)) {	// return the reply's headers array
 			frame->setHeaders(cx, WLArrayValue(*vp));
			return JS_TRUE;
		}
	}
*/

	return JS_TRUE;
}

JSBool frame_ClassEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_BROWSER_SOURCE, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_BROWSER_HEADER, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_APPLETS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_SCRIPTS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_CSS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_TABLES, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_EMBEDS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_IMAGES, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_FORMS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_LINKS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_FRAMES, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_LOCATION, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_URL, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_TITLE, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}

static void
frame_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside frame_finalize(%x, %x)", cx, obj);
	WLFinalize(cx, obj);
}


JSClass wl_FrameClass = {
	WLJSNAME_CLASS_FRAME,
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	frame_getProperty,
	frame_setProperty,
	frame_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	frame_finalize
};




static JSBool
frame_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("frame_toString");
	// set rval to location.href
		CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	CWLFrame* doc = (CWLFrame*)prv->GetParent();
	if (!doc)
		return JS_TRUE;

	const char* location = ((CWLFrame*)(prv->GetObj()))->GetLocation().Href();

/*
	char buff[128];
	domModeType   domMode = frameCont->GetDomMode();
	JSFrameMode	  frameMode = frameCont->GetFrameMode();
	if(domMode == BROWSER_DOM)
		if(frameMode == JS_Frame)
			sprintf(buff, "[object of class WLWindow]");
		else
			sprintf(buff, "[object of class WLDocument]");
	else
		sprintf(buff, "[object of class WLFrame]");
	frameCont->SetFrameMode(JS_None);
	JSString* str = JS_NewStringCopyZ(cx, buff);
*/
	JSString* str = JS_NewStringCopyZ(cx, location);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}


/*
static JSBool
frame_followAllLinks(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	// for each link/form(and within frames) use method/GET
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLFrame* frame = (CWLFrame*)prv->GetObj();
	if (!frame)
		return JS_TRUE;

	bool recurse = FALSE;
	if (argc > 0 && JSVAL_IS_BOOLEAN(argv[0])) {
		if (JSVAL_TO_BOOLEAN(argv[0]) == JS_TRUE)
			recurse = true;
	}
//???	frame->followAllLinks(recurse);
	return JS_TRUE;
}
*/


typedef enum {
	FRAMES,
	FORMS,
	LINKS
} ArrayType;

static void addToArray(CWLPage* page, CWLFrame* frame, ArrayType type)
{
	switch (type) {
		case FRAMES:
			page->AddFrameToArray(frame);
			break;

		case FORMS:
			if (frame->m_Forms)
				for (size_t i = 0; i < frame->m_Forms->entries(); i++)
					page->AddFormToArray((CWLForm*)(*(frame->m_Forms))[i]);
			break;

		case LINKS:
			if (frame->m_Links)
				for (size_t i = 0; i < frame->m_Links->entries(); i++)
					page->AddLinkToArray((CWLLink*)(*(frame->m_Links))[i]);
			break;
	}
}

static int getChildFrames(CWLPage* page, CWLFrame* frame, ArrayType type)
{
	if (!frame)
		return 0;

	int total = 1;
	addToArray(page, frame, type);

	// recursively, for each frame, go over all it's frames array and collect the frame's pointers
	if (frame->m_Frames)
		for (size_t i = 0; i < (frame->m_Frames)->entries(); i++)
			total += getChildFrames(page, (CWLFrame*)(*(frame->m_Frames))[i], type);

	return total;
}


void GetTopPageAndFrame(JSContext *cx, JSObject *obj, CWLPage** page, CWLFrame** frame)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
	if (ptrHttp)
		*page = ptrHttp->GetPage();

	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return;

	*frame = (CWLDocument*)prv->GetObj();
}

static JSBool
frame_browser_GetAllFrames(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)

{
	CWLFrame* topFrame = NULL;
	CWLPage* topPage = NULL;

	GetTopPageAndFrame(cx, obj, &topPage, &topFrame);
	if (!topPage || !topFrame)
		return JS_FALSE;

	if (topPage->GetAllFrames() && topPage->GetAllFrames()->entries() > 0)
		topPage->GetAllFrames()->clear();	// can't use clearAndDestroy since those are not my object

	// start from top frame
	int total = getChildFrames(topPage, topFrame, FRAMES);
	RVOrdered* arrayPtr = topPage->GetAllFrames();
	return WLSetArrayValue(cx, obj, topFrame, WLJSNAME_CLASS_FRAME, (void**)&arrayPtr, rval);
}

static JSBool
frame_browser_GetAllForms(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLFrame* topFrame = NULL;
	CWLPage* topPage = NULL;

	GetTopPageAndFrame(cx, obj, &topPage, &topFrame);
	if (!topPage || !topFrame)
		return JS_FALSE;

  if (topPage->GetAllForms() && topPage->GetAllForms()->entries() > 0)
		topPage->GetAllForms()->clear();	// can't use clearAndDestroy since those are not my object

	// start from top frame
	int total = getChildFrames(topPage, topFrame, FORMS);
	RVOrdered* arrayPtr = topPage->GetAllForms();
	return WLSetArrayValue(cx, obj, topFrame, WLJSNAME_CLASS_FORM, (void**)&arrayPtr, rval);
}

static JSBool
frame_browser_GetAllLinks(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLFrame* topFrame = NULL;
	CWLPage* topPage = NULL;

	GetTopPageAndFrame(cx, obj, &topPage, &topFrame);
	if (!topPage || !topFrame)
		return JS_FALSE;

  if (topPage->GetAllLinks() && topPage->GetAllLinks()->entries() > 0)
		topPage->GetAllLinks()->clear();	// can't use clearAndDestroy since those are not my object

	// start from top frame
	int total = getChildFrames(topPage, topFrame, LINKS);
	RVOrdered* arrayPtr = topPage->GetAllLinks();
	return WLSetArrayValue(cx, obj, topFrame, WLJSNAME_CLASS_LINK, (void**)&arrayPtr, rval);
}

#define TYPE_LINK  1
#define TYPE_FORM  2
#define TYPE_IMG   3
#define TYPE_FRAME 4

static JSBool
frame_identifyObject(int objType, JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLUrl *searcher = NULL, *resultUrl = NULL;
  CWLForm * resultForm = NULL;
  CWLEmbed *resultImg = NULL;
  CWLFrame *resultFrame = NULL;
	int32     expectedIndex = 0;

	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_FALSE;

	CWLFrame *frame = (CWLDocument*)prv->GetObj();

	*rval = JSVAL_VOID;  // initial value
	if (argc > 0)
	{
		if (WLIsObjectOfClass(cx, argv[0], WLJSNAME_CLASS_LINK))
		{
			JSObject* obj = JSVAL_TO_OBJECT(argv[0]);
			CWLPrivate* prv1 = (CWLPrivate*)JS_GetPrivate(cx, obj);
			if (prv1)
				searcher = (CWLUrl*)prv1->GetObj();
			else
				return JS_TRUE;  // do nothing
		}
		else
			return JS_TRUE;  // do nothing
	}
	else
		return JS_TRUE;  // do nothing

	if( argc > 1 )  // there is an index
		if (!JS_ValueToECMAInt32(cx, argv[1], &expectedIndex))
			expectedIndex = 0;

  switch( objType )
  {
  case TYPE_LINK:
	  if( frame->m_Links && frame->m_Links->entries())  // there is a collection and it's not empty
		  resultUrl = (CWLUrl *)frame->m_Links->unifyObject(searcher,expectedIndex);

	  if( resultUrl )
		  return WLSetLinkValue(cx, obj, frame, resultUrl, rval);
    break;

  case TYPE_FORM:
  	if( frame->m_Forms && frame->m_Forms->entries())  // there is a collection and it's not empty
	  	resultForm = (CWLForm *)frame->m_Forms->unifyObject(searcher,expectedIndex);

	  if( resultForm )
	  	return WLSetFormValue(cx, obj, frame, resultForm, rval);
    break;

  case TYPE_IMG:
  	if( frame->m_SrcList && frame->m_SrcList->entries())  // there is a collection and it's not empty
	  	resultImg = (CWLEmbed *)frame->m_SrcList->unifyObject(searcher,expectedIndex);

	  if( resultImg )
	  	return WLSetImgValue(cx, obj, frame, resultImg, rval);
    break;

  case TYPE_FRAME:
  	if( frame->m_Frames && frame->m_Frames->entries())  // there is a collection and it's not empty
	  	resultFrame = (CWLFrame *)frame->m_Frames->unifyObject(searcher,expectedIndex);

	  if( resultFrame )
	  	return WLSetWindowValue(cx, obj, frame, resultFrame, rval);
			//return WLSetFrameValue(cx, obj, frame, resultFrame, rval);
    break;
	}

	return JS_TRUE;
}

static JSBool
frame_identifyLink(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  return frame_identifyObject(TYPE_LINK, cx, obj, argc, argv, rval);
}



static JSBool
frame_identifyForm(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  return frame_identifyObject(TYPE_FORM, cx, obj, argc, argv, rval);
}

static JSBool
frame_identifyImg(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  return frame_identifyObject(TYPE_IMG, cx, obj, argc, argv, rval);
}

static JSBool
frame_identifyFrame(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  return frame_identifyObject(TYPE_FRAME, cx, obj, argc, argv, rval);
}


static JSBool
frame_GetFrameByLocation(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
  CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
  if (!prv)
    return JS_FALSE;
  
  CWLFrame *Doc = (CWLDocument*)prv->GetObj();
  
  CWLFrame*       retFrame = NULL;
  RVCString       Target;
  int             errId = RET_OK;
  const char*     val = NULL;
  
  findFrameInDocStatus retStatus;
  
  if (argc > 0)
  {
    if (JSVAL_IS_STRING(argv[0]))
    {
		  val = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
      Target = val;
      if (Doc -> m_Frames)
      {
        retStatus = Doc->GetFrameByLocation(Target, retFrame);
        if (retStatus == findFrameInDocStatus_ERROR)
          errId = 215;
        else if (retStatus == findFrameInDocStatus_NAME_MATCH_NUMBER_MISMATCH_WARNING)
          if (ptrHttp->GetWorkspace()->m_AsmDiagnostics)
            errId = 216;
          else if (retStatus == findFrameinDocStatus_NUMBER_MATCH_NAME_MISMATCH_WARNING)
            if (ptrHttp->GetWorkspace()->m_AsmDiagnostics)
              errId = 217;
      }
      else
        errId = 214;
    }
    else
      errId = 212;
  }
  else
    errId = 211;
  
  if (errId != RET_OK) {
		if(ReportJSError(cx, HTML_ERROR_FILE, errId, check_null(Target.data())))
  		return JS_FALSE;
  }
  
  if( retFrame )
    return WLSetWindowValue(cx, obj, retFrame, retFrame, rval);
  else
    return JS_TRUE;
}


static JSBool
frame_GetIdentLocation(JSContext *cx, JSObject *obj, uintN argc, jsval* argv, jsval *rval)
{
	CWLCommandManager* ptrHttp = WLGetCommandManagerPtr(cx);
  CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
  if (!prv)
    return JS_TRUE;
  
  CWLFrame *Doc = (CWLDocument*)prv->GetObj();
	if(!Doc)
		return JS_TRUE;
	CWLIdentLocation identLocation;
  identLocation = Doc->GetFrameTarget();
	if(identLocation.isEmpty())
		return JS_TRUE;
	RVCString rwStr;
  rwStr = identLocation.getStr();
	JSString* str = JS_NewStringCopyZ(cx,(const char *)rwStr);
	if (str)
		*rval = STRING_TO_JSVAL(str);

  return JS_TRUE;
}



static JSFunctionSpec frame_methods[] = {
	//{wl_toString_str,		frame_toString,		0},
	//{"followAllLinks",	frame_followAllLinks,	1},
	{WLJSNAME_BROWSER_DOM_GET_ALL_FRAMES, frame_browser_GetAllFrames, 0},
	{WLJSNAME_BROWSER_DOM_GET_ALL_FORMS,	frame_browser_GetAllForms	, 0},
	{WLJSNAME_BROWSER_DOM_GET_ALL_LINKS,	frame_browser_GetAllLinks	, 0},
	{WLJSNAME_IDENTIFY_LINK,				 	    frame_identifyLink,         2},
	{WLJSNAME_IDENTIFY_FORM,					    frame_identifyForm,         2},
	{WLJSNAME_IDENTIFY_IMG,				  	    frame_identifyImg,          2},
	{WLJSNAME_IDENTIFY_FRAME,					    frame_identifyFrame,        2},
	{WLJSNAME_HTML_GET_FRAME_BY_LOCATION,	frame_GetFrameByLocation, 	1},
	{WLJSNAME_GET_IDENT_LOCATION,					frame_GetIdentLocation,		 	0},
	{0}
};
/*
static JSFunctionSpec browser_frame_methods[] = {
	{wl_toString_str,		frame_toString,		0},
	{wl_valueOf_str,		frame_valueOf,		0},
	{0}
};
*/



static JSBool
InitFrameObject(JSContext *cx, JSObject *obj, jsint length, jsval *vector)
{
WlTraceMsg("initializing frame object properties");
/*???!!!
	CWLFrame* frame = new CWLFrame;
	JS_SetPrivate(cx, obj, (void*)frame);
*/

	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_DOM_SCOPE))
		return JS_FALSE;

  // Chaim - skip the initialization since we're using the enumerated callback
  return JS_TRUE;

  if (!WLInitProperty(cx, obj, WLJSNAME_PROP_REFERRER))
		return JS_FALSE;

//	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_SPIDERSESSION))
//		return JS_FALSE;

	// the following properties represent the http reply for the frame
	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_PROTOCOL_VERSION))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_STATUS_NUMBER))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_STATUS_LINE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HEADER))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_SOURCE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_PROTOCOL_VERSION))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_STATUS_NUMBER))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_STATUS_LINE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_SEARCH_PAIRS))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_HEADER))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_APPLETS))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_SCRIPTS))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_CSS))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_EMBEDS))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_IMAGES))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_FORMS))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_LINKS))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_FRAMES))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_LOCATION))
		return JS_FALSE;

  // Chaim - I think we need it to show the document properties of a frame in the collection but it looks redundant
  // on the top level document
	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_DOCUMENT))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_URL))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_TITLE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_NAME))
		return JS_FALSE;


	return JS_TRUE;
}


/*
static JSBool
WLFrame(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("WLFrame constructor");
	JSBool ok;

	ok = InitFrameObject(cx, obj, 0, NULL);
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}
*/


JSObject* WLNewFrameObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewFrameObject(%x, %x)", cx, obj);
	// create an instance of type WLFrame
	JSObject* proto = NULL;
	JSObject* frame = WLNewObject(cx, &wl_FrameClass, proto, obj);
	if (!frame)
		return NULL;

	return frame;
}



bool DllExport WLAddFrameClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddFrameClass(%x, %x)", cx, obj);
	// define a class of type WLFrame
	//if (!JS_InitClass(cx, obj, NULL, &wl_FrameClass, WLFrame, 1, NULL, frame_methods, NULL, NULL))
	if (!JS_InitClass(cx, obj, NULL, &wl_FrameClass, NULL, 1, NULL, frame_methods, NULL, NULL))
		return false;

	return true;
}


bool DllExport WLAddDocumentClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddDocumentClass(%x, %x)", cx, obj);
	// define a class of type WLDocument
	return WLNewFrameObject(cx, obj) != NULL ? true : false;
}


bool WLAddFrameObject(JSContext *cx, JSObject *obj)
{
	// create an instance of type WLFrame
	JSObject* frame = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_FRAME, &wl_FrameClass, NULL, JSPROP_PERMANENT);
	if (!frame)
		return false;

  // define it's properties & structures
	if (!InitFrameObject(cx, frame, 0, NULL))
		return false;

WlTraceMsg("WLAddFrameObject(%x, %x) ==> %x", cx, obj, frame);
	return true;
}


bool DllExport WLAddDomObject(JSContext *cx, JSObject *obj, JSObject** jsdoc)
{
WlTraceMsg("WLAddDomObject(%x, %x)", cx, obj);
	// create an instance of type WLDocument
	JSObject* dom = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_DOCUMENT, &wl_FrameClass, NULL, JSPROP_PERMANENT);
	if (!dom)
		return false;

	*jsdoc = dom;
	JS_SetPrivate(cx, dom, NULL);

	// define it's properties & structures
	if (!InitFrameObject(cx, dom, 0, NULL))
		return false;

	return true;
}
