/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wlframecontainer.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "TraceDll.h"

extern CWLFrame*     WLGetFrameItemPtr(RVOrdered* array, size_t index, const char* key);


static JSBool
frames_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside frames_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;
		jsint index = -1;
	char* key = NULL;
	RVOrdered* frames = (RVOrdered*)prv->GetObj();
	if(!frames)
		return JS_TRUE;
	CWLFrame* doc = (CWLFrame*)prv->GetParent();
	if (!doc)
		return JS_TRUE;

	if (frames && (JSVAL_IS_STRING(id) || JSVAL_IS_INT(id))) {
		if (JSVAL_IS_INT(id)){
			index = JSVAL_TO_INT(id);
			CWLFrame* frame = WLGetFrameItemPtr(frames, index, key);
				return WLSetWindowValue(cx, obj, doc, frame, vp);
		}
		else
			key = JS_GetStringBytes(JSVAL_TO_STRING(id));
		// get length property
		if (key && !strcmp(key, WLJSNAME_PROP_LENGTH)) 
			*vp = INT_TO_JSVAL(frames->entries());
		return JS_TRUE;
	}
	return JS_TRUE;
}


static JSBool
frames_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside frames_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_FALSE;

	RVOrdered* array = (RVOrdered*)prv->GetObj();
	if (array && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_KEY)) {	// set the Key property
			array->setKey(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_VALUE)) {	// return the Value property
			array->setValue(WLStringValue(*vp));
			return JS_TRUE;
		}
	}
*/

	return JS_TRUE;
}

static void
frames_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside frames_finalize(%x, %x)", cx, obj);
	WLFinalize(cx, obj);
}



JSClass wl_FramesClass = {
	WLJSNAME_CLASS_FRAMES,
	JSCLASS_HAS_PRIVATE,//ptr to c++ structure that info regarding the object it represents
	JS_PropertyStub,   //callback to C++
	JS_PropertyStub,	//--"--
	frames_getProperty,
	frames_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	frames_finalize
};


/*
static JSBool
frames_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("frames_toString");

	char name[128];
	sprintf(name, "[array of class WLWindow]");

	JSString* str = JS_NewStringCopyZ(cx, name);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}


static JSBool
frames_valueOf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("frames_valueOf");
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	RVOrdered* frames = (RVOrdered*)prv->GetObj();
	if (!frames)
		return JS_TRUE;

	*rval = INT_TO_JSVAL(frames->entries());

	return JS_TRUE;
}


static JSFunctionSpec frames_methods[] = {
	{wl_toString_str,		frames_toString,	0},
	{wl_valueOf_str,		frames_valueOf,	0},
	{0}
};
*/


static JSBool
InitFramesObject(JSContext *cx, JSObject *obj, jsint argc, jsval *argv)
{
WlTraceMsg("initializing Frames object properties");

	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_CLASS_NAME))
		return JS_FALSE;

	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_DOM_SCOPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_LENGTH))
		return JS_FALSE;

	int index = 0;
	for (int i = 0; i < argc; i++) 
	{
		if (!WLIsObjectOfClass(cx, argv[0], WLJSNAME_CLASS_FRAME)) 
			return JS_FALSE;
			// array of CWLFrame
		RVOrdered* array;
		if (i == 0) 
		{
			array = new RVOrdered;
			JS_SetPrivate(cx, obj, (void*)array);				
			if (!array)
				return JS_FALSE;
		}
		CWLFrame* frame = (CWLFrame*)JS_GetPrivate(cx, JSVAL_TO_OBJECT(argv[i]));
		if(frame)
			array->insert(frame);
		else 
			return JS_FALSE;
		// alias the item's name value with an index
		const char* name = frame->GetName().data();
		if (name && *name) 
		{
			WLInitProperty(cx, obj, name);
			JS_AliasElement(cx, obj, name, index);
		}				
		index++;
	}
	
	return JS_TRUE;
}


/*
static JSBool
WLFrames(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("WLFrame constructor");
	JSBool ok;

	ok = InitFramesObject(cx, obj, argc, argv);
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}


JSObject* WLNewFramesObject(JSContext *cx, JSObject *obj)
{
	// create an instance of type WLArray (should inherit from class 'Array')
	JSObject* proto = NULL;
	JSObject* frames = WLNewObject(cx, &wl_FramesClass, proto, obj);
	if (!frames)
		return NULL;

WlTraceMsg("WLNewFramesObject(%x, %x) ==> %x", cx, obj, frames);
	return frames;
}
*/


//add frames object
bool WLAddFramesObject(JSContext *cx, JSObject *obj, JSObject** jsframes)
{
WlTraceMsg("WLAddFramesObject(%x, %x)", cx, obj);
	// create an instance of type WLArray
	JSObject* frames = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_FRAMES, &wl_FramesClass, NULL, JSPROP_PERMANENT);
	if (!frames)
		return false;

	*jsframes = frames;

	JS_SetPrivate(cx, frames, NULL);
	
	// define it's properties & structures
	if (!InitFramesObject(cx, frames, 0, NULL))
		return false;

	return true;
}


bool DllExport WLAddFramesClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddFramesClass(%x, %x)", cx, obj);
//	if (!JS_InitClass(cx, obj, NULL, &wl_ArrayClass, WLArray, 1, NULL, array_methods, NULL, NULL))
	if (!JS_InitClass(cx, obj, NULL, &wl_FramesClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
