/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wlframecontainer.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

static JSBool
link_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	WlTraceMsg("Inside link_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
  CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;
	
	CWLFrame* frame = (CWLFrame*)prv->GetParent();

	CWLLink* link = (CWLLink*)prv->GetObj();
	if (!link)
		return JS_TRUE;

	if (link && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);


    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_PROTOCOL : {	// return the Protocol property
			const char* protocol = link->Protocol();
			return WLSetStringValue(cx, obj, prop, protocol, vp);
		}

    case PropertiesHandler::EN_HOSTNAME : {	// return the Hostname property
			const char* hostname = link->Hostname();
			return WLSetStringValue(cx, obj, prop, hostname, vp);
		}

    case PropertiesHandler::EN_PORT : {	// return the Port property
			int port = link->Port();
			return WLSetIntValue(cx, obj, prop, port, vp);
		}

    case PropertiesHandler::EN_HOST : {	// return the Host property
			const char* host = link->Host();
			return WLSetStringValue(cx, obj, prop, host, vp);
		}

    case PropertiesHandler::EN_PATHNAME : {	// return the pathname property
			const char* uri = link->Uri();
			return WLSetStringValue(cx, obj, prop, uri, vp);
		}

    case PropertiesHandler::EN_HASH : {	// return the Hash property
			const char* hash = link->Hash();
			return WLSetStringValue(cx, obj, prop, hash, vp);
		}

    case PropertiesHandler::EN_WLSEARCHPAIRS : {	// return the KVP property
			if (!frame)     // only for objects from the DOM
				return JS_TRUE;
			CWLArray* kvp = &(link->Array());
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_KVP, (void**)&kvp, vp);
		}

    case PropertiesHandler::EN_SEARCH : {	// return the Search property
			const char* search = link->Search();
			return WLSetStringValue(cx, obj, prop, search, vp);
		}

    case PropertiesHandler::EN_HREF : {	// return the Href property
			const char* href = link->Href();
			return WLSetStringValue(cx, obj, prop, href, vp);
		}

    case PropertiesHandler::EN_WLORIGURL : {	// return the wlOrigUrl property
			const char* origUrl = link->GetOriginalUrl();
			return WLSetStringValue(cx, obj, prop, origUrl, vp);
		}

    case PropertiesHandler::EN_TARGET : {	// return the Target property
			const char* target = link->Target();
			return WLSetStringValue(cx, obj, prop, target, vp);
		}

    case PropertiesHandler::EN_NAME : {	// return the Name property
			const char* name = link->Name();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}

    case PropertiesHandler::EN_ID : {	// return the id property
			const char* id = link->Id();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_INNERHTML : {	// return the innerHTML property
			RVCString innerHTML = link->GetInnerHTML();
			return WLSetStringValue(cx, obj, prop, innerHTML.data(), vp);
		}

    case PropertiesHandler::EN_INNERTEXT : {	// return the innerText property
			const char* innerText = link->GetInnerText();
			return WLSetStringValue(cx, obj, prop, innerText, vp);
		}

    case PropertiesHandler::EN_TAGNAME : {	// return the tagName property
      RVCString innerHTML = link->GetInnerHTML();
			const char* typeOfAnchor = (!strcmp(innerHTML.data(),"AREA_ELEMENT") ? "AREA" : "A");
			return WLSetStringValue(cx, obj, prop, typeOfAnchor, vp);
		}

    case PropertiesHandler::EN_WLSOURCE_INDEX : { // return the WLSourceIndex property
			int index = link->GetIndex();
			return WLSetIntValue(cx, obj, prop, index, vp);
		}

    case PropertiesHandler::EN_INNERIMG : { // return the WLSourceIndex property
			CWLEmbed *img = link->GetInnerImg();
			return WLSetImgValue(cx, obj, frame, img, vp);
		}

    default:;
    }
	}
	
//???	return JS_GetProperty(cx, obj, id, vp);
	return JS_TRUE;
}

static JSBool
link_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside link_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLink* link = (CWLLink*)prv->GetObj();
	if (link && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_PROTOCOL)) {	// set the Protocol property
			link->setProtocol(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HOSTNAME)) {	// return the Hostname property
			link->setHostname(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PORT)) {	// return the Port property
			link->setPort(WLIntValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HOST)) {	// return the Host property
			link->setHost(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PATHNAME)) {	// return the Uri property
			link->setUri(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HASH)) {	// return the Hash property
			link->setHash(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_KVP)) {	// return the KVP property
//!!!???			link->setKvp(WLArrayValue(cx, *vp));	???
//			link->Array() = *WLArrayValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SEARCH)) {	// return the Search property
			link->setSearch(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HREF)) {	// return the Href property
			link->setHref(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SPIDERSESSION)) {	// return the SPIDERSESSION property
			link->setSpiderSession(WLStringValue(*vp));
			return JS_TRUE;
		}
	}
*/

	return JS_TRUE;
}

JSBool link_ClassEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_SEARCH, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_TARGET, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ID, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_TAG_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_INNER_TEXT, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_INNERHTML, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_PROTOCOL, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_PORT, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_HOST, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_PATHNAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_HREF, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}

static void
link_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside link_finalize");
	WLFinalize(cx, obj);
}


JSClass wl_LinkClass = {
	WLJSNAME_CLASS_LINK,
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	link_getProperty,
	link_setProperty,
	link_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	link_finalize
};



static JSBool
link_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLink* link = (CWLLink*)prv->GetObj();
	if (!link)
		return JS_TRUE;

	// set rval to Href property
	const char* href = link->Href();
	JSString* str = JS_NewStringCopyZ(cx, href);
	if (str)
		*rval = STRING_TO_JSVAL(str);
	return JS_TRUE;
}

static JSBool
link_valueOf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLink* link = (CWLLink*)prv->GetObj();
	if (!link)
		return JS_TRUE;

	// set rval to Href property
	const char* href = link->Href();
	JSString* str = JS_NewStringCopyZ(cx, href);
	if (str)
		*rval = STRING_TO_JSVAL(str);
	return JS_TRUE;
}


static JSBool
link_urlCompare(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLink* link = (CWLLink*)prv->GetObj();
	if (!link)
		return JS_TRUE;

	if(argc != 1) 
		return JS_TRUE;

	CWLLink* link1;
	//getting the WLLink parameter
	if (WLIsObjectOfClass(cx, argv[0], WLJSNAME_CLASS_LINK))
		{
			JSObject* linkObj = JSVAL_TO_OBJECT(argv[0]);
			CWLPrivate* prv1 = (CWLPrivate*)JS_GetPrivate(cx, linkObj);
			if (!prv1)
				return JS_TRUE;

			link1 = (CWLLink*)prv1->GetObj();
			if (!link1)
				return JS_TRUE;
		}
	else
		return JS_TRUE;
	
	
	CWLUrlCmp result;
	if(link->urlCompare(*link1,result))
		*rval = INT_TO_JSVAL(result.rank()); //1 to 12, 1 is perfect match 
	else if(result.propRank() > 0)
		*rval = INT_TO_JSVAL(0); //the url didn't match but at least 1 property matched
	else
		*rval = INT_TO_JSVAL(-1); //no match

	return JS_TRUE;
}


static JSBool
link_follow(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	// use Get on url
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_FALSE;

	CWLLink* link = (CWLLink*)prv->GetObj();
	if (!link)
		return JS_FALSE;

//???	link->Follow();
	return JS_TRUE;
}


static JSFunctionSpec link_methods[] = {
	{wl_toString_str,		link_toString,		0},
	{wl_valueOf_str,		link_valueOf,			0},
	{wl_urlCompare_str,	link_urlCompare,	1},
	//{"follow",				link_follow,		0},
	{0}
};



static JSBool
WLLink(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	char  *name = 0, *id = 0, *target = 0, *innerHtml = 0;
  RVCString original_url, referer;
	//check that number pf parameters is correct
	if(argc < 1 || argc > 6)
		if(!ReportJSError(cx, JS_DOM_ERROR_FILE, 90))
			return JS_TRUE;
		else
			return JS_FALSE;
	//convert args to parameters
	RVCString url = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
	if( argc > 1)
		innerHtml = JS_GetStringBytes(JS_ValueToString(cx, argv[1]));
	if( argc > 2)
		name	  = JS_GetStringBytes(JS_ValueToString(cx, argv[2]));
	if( argc > 3)
		id		  = JS_GetStringBytes(JS_ValueToString(cx, argv[3]));
	if( argc > 4)
		target	  = JS_GetStringBytes(JS_ValueToString(cx, argv[4]));
	if( argc > 5)
  	original_url = JS_GetStringBytes(JS_ValueToString(cx, argv[5]));
	if( argc > 6)
	  referer	= JS_GetStringBytes(JS_ValueToString(cx, argv[6]));  

  //construct the c++ object
	CWLLink* link = new CWLLink(url, original_url, referer, name, innerHtml, id, target);//oooo
	//create CWLPrivate 
	CWLPrivate* prv = new CWLPrivate(WLJSNAME_CLASS_LINK, link, NULL, true);
	JS_SetPrivate(cx, obj, prv);
	*rval = OBJECT_TO_JSVAL(obj);
	return JS_TRUE;

}


JSObject* WLNewLinkObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewLinkObject(%x, %x)", cx, obj);
	// create an instance of type WLLink
	JSObject* proto = NULL;
	JSObject* link = WLNewObject(cx, &wl_LinkClass, proto, obj);
	if (!link)
		return NULL;

	return link;
}


bool DllExport WLAddLinkClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddLinkClass(%x, %x)", cx, obj);
	// define a class of type WLLink
	if (!JS_InitClass(cx, obj, NULL, &wl_LinkClass, WLLink, 1, NULL, link_methods, NULL, NULL))
		return false;

	return true;
}
