/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wllocation.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

static JSBool
location_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside location_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
 	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLocation* location = (CWLLocation*)prv->GetObj();
	if (!location)
		return JS_TRUE;
	CWLFrame* frame;
	bool IsObjectJSCreated = prv->IsObjectJSCreated();
	if(!IsObjectJSCreated){
  	frame = (CWLFrame*)prv->GetParent();
		if (!frame)
			return JS_TRUE;
	}

	if (location && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_PROTOCOL : {	// return the Protocol property
			const char* protocol = location->Protocol();
			return WLSetStringValue(cx, obj, prop, protocol, vp);
		}

    case PropertiesHandler::EN_HOSTNAME : {	// return the Hostname property
			const char* hostname = location->Hostname();
			return WLSetStringValue(cx, obj, prop, hostname, vp);
		}

    case PropertiesHandler::EN_PORT : {	// return the Port property
			int port = location->Port();
			return WLSetIntValue(cx, obj, prop, port, vp);
		}

    case PropertiesHandler::EN_HOST : {	// return the Host property
			const char* host = location->Host();
			return WLSetStringValue(cx, obj, prop, host, vp);
		}

    case PropertiesHandler::EN_PATHNAME : {	// return the Uri property
			const char* uri = location->Uri();
			return WLSetStringValue(cx, obj, prop, uri, vp);
		}

    case PropertiesHandler::EN_HASH : {	// return the Hash property
			const char* hash = location->Hash();
			return WLSetStringValue(cx, obj, prop, hash, vp);
		}

    case PropertiesHandler::EN_SEARCH : {	// return the Search property
			const char* search = location->Search();
			return WLSetStringValue(cx, obj, prop, search, vp);
		}

    case PropertiesHandler::EN_HREF : {	// return the Href property
			const char* href = location->Href();
			return WLSetStringValue(cx, obj, prop, href, vp);
		}

    case PropertiesHandler::EN_WLSEARCHPAIRS : {	// return the KVP property
			CWLArray* kvp = &(location->Array());
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_KVP, (void**)&kvp, vp);
		}

    case PropertiesHandler::EN_ID : {	// return the id property
			const char* id = location->Id();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_NAME : {	// return the "name" of form through the action property
			const char* name = location->Name();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}

    case PropertiesHandler::EN_TARGET : {	// return the target property
			const char* target = location->Target();
			return WLSetStringValue(cx, obj, prop, target, vp);
		}
    default:;
    }
  }
	
//???	return JS_GetProperty(cx, obj, id, vp);
	return JS_TRUE;
}

static JSBool
browser_location_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside browser_location_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
 	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLocation* location = (CWLLocation*)prv->GetObj();
	if (!location)
		return JS_TRUE;

	CWLFrame* frame;
	bool IsObjectJSCreated = prv->IsObjectJSCreated();
	if(!IsObjectJSCreated){
  	frame = (CWLFrame*)prv->GetParent();
		if (!frame)
			return JS_TRUE;
	}
	if (location && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);
		
    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_PROTOCOL : {	// return the Protocol property
			const char* protocol = location->Protocol();
			return WLSetStringValue(cx, obj, prop, protocol, vp);
		}

    case PropertiesHandler::EN_HOSTNAME : {	// return the Hostname property
			const char* hostname = location->Hostname();
			return WLSetStringValue(cx, obj, prop, hostname, vp);
		}

    case PropertiesHandler::EN_PORT : {	// return the Port property
			int port = location->Port();
			return WLSetIntValue(cx, obj, prop, port, vp);
		}

    case PropertiesHandler::EN_HOST : {	// return the Host property
			const char* host = location->Host();
			return WLSetStringValue(cx, obj, prop, host, vp);
		}

    case PropertiesHandler::EN_PATHNAME : {	// return the Uri property
			const char* uri = location->Uri();
			return WLSetStringValue(cx, obj, prop, uri, vp);
		}

    case PropertiesHandler::EN_HASH : {	// return the Hash property
			const char* hash = location->Hash();
			return WLSetStringValue(cx, obj, prop, hash, vp);
		}

    case PropertiesHandler::EN_KVP : {	// return the KVP property			
		//error - cannot get to this prperty from the browser object "location"
			return JS_TRUE;
//			CWLArray* kvp = &(location->Array());
//			return WLSetArrayValue(cx, obj, WLJSNAME_CLASS_KVP, (void**)&kvp, vp);
		}

    case PropertiesHandler::EN_WLSEARCHPAIRS : {	// return the KVP property
			CWLArray* kvp = &(location->Array());
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_KVP, (void**)&kvp, vp);
		}

    case PropertiesHandler::EN_SEARCH : {	// return the Search property
			const char* search = location->Search();
			return WLSetStringValue(cx, obj, prop, search, vp);
		}

    case PropertiesHandler::EN_HREF : {	// return the Href property
			const char* href = location->Href();
			return WLSetStringValue(cx, obj, prop, href, vp);
		}

    default:;
    }

/*
		if (!strcmp(prop, WLJSNAME_PROP_PROTOCOL)) {	// return the Protocol property
			const char* protocol = location->Protocol();
			return WLSetStringValue(cx, obj, prop, protocol, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HOSTNAME)) {	// return the Hostname property
			const char* hostname = location->Hostname();
			return WLSetStringValue(cx, obj, prop, hostname, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PORT)) {	// return the Port property
			int port = location->Port();
			return WLSetIntValue(cx, obj, prop, port, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HOST)) {	// return the Host property
			const char* host = location->Host();
			return WLSetStringValue(cx, obj, prop, host, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PATHNAME)) {	// return the Uri property
			const char* uri = location->Uri();
			return WLSetStringValue(cx, obj, prop, uri, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HASH)) {	// return the Hash property
			const char* hash = location->Hash();
			return WLSetStringValue(cx, obj, prop, hash, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_KVP)) {	// return the KVP property			
		//error - cannot get to this prperty from the browser object "location"
			return JS_TRUE;
//			CWLArray* kvp = &(location->Array());
//			return WLSetArrayValue(cx, obj, WLJSNAME_CLASS_KVP, (void**)&kvp, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_BROWSER_SEARCH_PAIRS)) {	// return the KVP property
			CWLArray* kvp = &(location->Array());
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_KVP, (void**)&kvp, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SEARCH)) {	// return the Search property
			const char* search = location->Search();
			return WLSetStringValue(cx, obj, prop, search, vp);
		}
/*not documented
		else if (!strcmp(prop, WLJSNAME_PROP_COMMANDS)) {	// return the commands property
			const char* search = location->Commands();
			return WLSetStringValue(cx, obj, prop, search, vp);
		}
*//*
		else if (!strcmp(prop, WLJSNAME_PROP_HREF)) {	// return the Href property
			const char* href = location->Href();
			return WLSetStringValue(cx, obj, prop, href, vp);
		}
*/
	}
	
//???	return JS_GetProperty(cx, obj, id, vp);
	return JS_TRUE;
}

static JSBool
location_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside location_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLocation* location = (CWLLocation*)prv->GetObj();
	if (location && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_PROTOCOL)) {	// set the Protocol property
			location->setProtocol(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HOSTNAME)) {	// return the Hostname property
			location->setHostname(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PORT)) {	// return the Port property
			location->setPort(WLIntValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HOST)) {	// return the Host property
			location->setHost(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PATHNAME)) {	// return the Uri property
			location->setUri(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HASH)) {	// return the Hash property
			location->setHash(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_KVP)) {	// return the KVP property
//!!!???			location->setKvp(WLArrayValue(cx, *vp));	???
//			location->Array() = *WLArrayValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SEARCH)) {	// return the Search property
			location->setSearch(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HREF)) {	// return the Href property
			location->setHref(WLStringValue(*vp));
			return JS_TRUE;
		}
	}
*/

	return JS_TRUE;
}

JSBool location_ClassEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_SEARCH, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_PROTOCOL, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_PORT, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_HOST, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_PATHNAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_HREF, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}

static void
location_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside location_finalize");
	WLFinalize(cx, obj);
}



JSClass wl_LocationClass = {
	WLJSNAME_CLASS_LOCATION,
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	location_getProperty,
	location_setProperty,
	location_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	location_finalize
};

JSClass wl_BrowserLocationClass = {
	WLJSNAME_CLASS_BROWSER_LOCATION,
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	browser_location_getProperty,
	location_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	location_finalize
};



static JSBool
location_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLocation* location = (CWLLocation*)prv->GetObj();
	if (!location)
		return JS_TRUE;

	// set rval to Href property
	const char* href = location->Href();
	JSString* str = JS_NewStringCopyZ(cx, href);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}

static JSBool
location_valueOf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLLocation* location = (CWLLocation*)prv->GetObj();
	if (!location)
		return JS_TRUE;

	// set rval to Href property
	const char* href = location->Href();
	JSString* str = JS_NewStringCopyZ(cx, href);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}


static JSFunctionSpec location_methods[] = {
	{wl_toString_str,	location_toString,	0},
	{wl_valueOf_str, location_valueOf, 0},
	{0}
};



static JSBool
InitLocationObject(JSContext *cx, JSObject *obj, jsint length, jsval *vector)
{
WlTraceMsg("initializing location object properties");
	// create the CPP object


	// add it's properties
	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_DOM_SCOPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_PROTOCOL))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HOSTNAME))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_PORT))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HOST))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_SEARCH))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_KVP))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_BROWSER_SEARCH_PAIRS))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HASH))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_HREF))
		return JS_FALSE;

	return JS_TRUE;
}



static JSBool
WLLocation(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	//CWLPrivate* prv = new CWLPrivate();
	//check that number pf parameters is correct
	if(argc != 6)
		if(!ReportJSError(cx, JS_DOM_ERROR_FILE, 90))
			return JS_TRUE;
		else
			return JS_FALSE;
	//convert args to parameters
	RVCString url			= JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
	char*     name		= JS_GetStringBytes(JS_ValueToString(cx, argv[1]));
	char*			id			= JS_GetStringBytes(JS_ValueToString(cx, argv[2]));
	char*			target	= JS_GetStringBytes(JS_ValueToString(cx, argv[3]));
	RVCString original_url = JS_GetStringBytes(JS_ValueToString(cx, argv[4]));
	RVCString referer	= JS_GetStringBytes(JS_ValueToString(cx, argv[5]));  
	
	//construct the c++ object
	CWLLocation* location = new CWLLocation(url, original_url, referer, name, id, target);
	//create CWLPrivate 
	CWLPrivate* prv = new CWLPrivate(WLJSNAME_CLASS_LOCATION, location, NULL, true);
	JS_SetPrivate(cx, obj, prv);
	*rval = OBJECT_TO_JSVAL(obj);
	return JS_TRUE;

}



JSObject* WLNewLocationObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewLocationObject(%x, %x)", cx, obj);
	// create an instance of type WLLink
	JSObject* proto = NULL;
	JSObject* location = WLNewObject(cx, &wl_LocationClass, proto, obj);
	if (!location)
		return NULL;

	return location;
}


bool DllExport WLAddLocationClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddLocationClass(%x, %x)", cx, obj);
	// define a class of type WLLocation
	if (!JS_InitClass(cx, obj, NULL, &wl_LocationClass, WLLocation, 1, NULL, location_methods, NULL, NULL))
		return false;

	return true;
}

bool DllExport WLAddBrowserLocationClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddBrowserLocationClass(%x, %x)", cx, obj);
	// define a class of type WLLocation
	if (!JS_InitClass(cx, obj, NULL, &wl_BrowserLocationClass, WLLocation, 1, NULL, location_methods, NULL, NULL))
		return false;

	return true;
}

// for browser mode
bool DllExport WLAddLocationObject(JSContext *cx, JSObject *obj, JSObject** jsloc)
{
WlTraceMsg("WLAddLocationObject(%x, %x)", cx, obj);
	// create an instance of type WLDocument
	JSObject* location = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_LOCATION, &wl_BrowserLocationClass, NULL, JSPROP_PERMANENT);
	if (!location)
		return false;

	*jsloc = location;

	JS_SetPrivate(cx, location, NULL);
	
	// define it's properties & structures
	if (!InitLocationObject(cx, location, 0, NULL))
		return false;

	return true;
}
