/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "wltable.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

static JSBool
row_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside row_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	char* key = NULL;
  CWLFrame* frame = prv->GetParent();
	CWLTableRow* row = (CWLTableRow*)prv->GetObj();
	if (!row)
		return JS_TRUE;

	if (JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!prop)
			return JS_TRUE;

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_ROWINDEX : {	// return the rowIndex property
			int index = row->getRowIndex();
			return WLSetIntValue(cx, obj, prop, index, vp);
		}

    case PropertiesHandler::EN_CELLS : {	// return the cells property
			return WLSetArrayValue(cx, obj, frame, WLJSNAME_CLASS_CELL, (void**)&row, vp);
			//return WLSetArrayValue(cx, obj, WLJSNAME_CLASS_ROW_CELL, (void**)&row, vp);
		}

    default:;
    }
	}
	return JS_TRUE;
}


static JSBool
row_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside row_setProperty");
	return JS_TRUE;
}

JSBool row_ClassEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_ROW_INDEX, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_CELLS, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}
static void
row_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside row_finalize(%x, %x)", cx, obj);
	WLFinalize(cx, obj);
}



JSClass wl_RowClass = {
	WLJSNAME_CLASS_ROW,
	JSCLASS_HAS_PRIVATE,	//ptr to c++ structure that info regarding the object it represents
	JS_PropertyStub,			//callback to C++
	JS_PropertyStub,			//--"--
	row_getProperty,
	row_setProperty,
	row_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	row_finalize
};


/*
static JSBool
row_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("row_toString");
	// set rval to ???
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLTableRow* row = (CWLTableRow*)prv->GetObj();
	if (!row)
		return JS_TRUE;

	return JS_TRUE;
}


static JSBool
row_valueOf(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("row_valueOf");
	// set rval to rowIndex
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLTableRow* row = (CWLTableRow*)prv->GetObj();
	if (!row)
		return JS_TRUE;

	*rval = INT_TO_JSVAL(row->getRowIndex());

	return JS_TRUE;
}


static JSFunctionSpec row_methods[] = {
	{wl_toString_str,		row_toString,	0},
	{wl_valueOf_str,		row_valueOf,	0},
	{0}
};
*/


static JSBool
InitRowObject(JSContext *cx, JSObject *obj, jsint argc, jsval *argv)
{
WlTraceMsg("initializing Row object properties");

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_LENGTH))
		return JS_FALSE;

	return JS_TRUE;
}


JSObject* WLNewRowObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewRowObject(%x, %x)", cx, obj);
	// create an instance of type WLRow
	JSObject* proto = NULL;
	JSObject* row = WLNewObject(cx, &wl_RowClass, proto, obj);
	if (!row)
		return NULL;

	return row;
}


bool DllExport WLAddRowClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddRowClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_RowClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
