/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <string.h>
#include "rvjsapi.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "TraceDll.h"
#include "wlscript.h"

#include "propertieshandler.h"
static JSBool
script_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside script_getProperty");
/*	
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
 	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLScript* script = (CWLScript*)prv->GetObj();
	if (!script)
		return JS_TRUE;

	CWLFrame* frame = (CWLFrame*)prv->GetParent();
	if (!frame)
		return JS_TRUE;

	if (script && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_SRC : {	// return the src property
			const char* src = script->GetSrc();
			return WLSetStringValue(cx, obj, prop, src, vp);
		}

    case PropertiesHandler::EN_EVENT : {	// return the event property
			const char* event = script->GetEvent();
			return WLSetStringValue(cx, obj, prop, event, vp);
		}

    case PropertiesHandler::EN_ID : {	// return id property
			const char* id = script->GetId();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_INNERHTML : {	// return the innerHTML property
			const char* innerHtml = script->GetText();
			return WLSetStringValue(cx, obj, prop, innerHtml, vp);
		}

    case PropertiesHandler::EN_LANGUAGE : {	// return the language property
			const char* language = script->GetLanguage();
			return WLSetStringValue(cx, obj, prop, language, vp);
		}

    case PropertiesHandler::EN_WLSOURCE_INDEX : { // return the WLSourceIndex property
			int index = script->GetIndex();
			return WLSetIntValue(cx, obj, prop, index, vp);
		}

    case PropertiesHandler::EN_FILE_SIZE : { // return the FileSize property
			int nFileSize = script->GetTransactionObj()->GetFileSize();
			return WLSetIntValue(cx, obj, prop, nFileSize, vp);
		}

    case PropertiesHandler::EN_CHECKSUM : { // return the CheckSum property
			int nCheckSum = script->GetTransactionObj()->GetCheckSum();
			return WLSetIntValue(cx, obj, prop, nCheckSum, vp);
		}

    case PropertiesHandler::EN_OPEN_TIME : { // return the OpenTime property
			double dTime = script->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_OPEN_CONNECTION);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_SEND_TIME : { // return the SendTime property
			double dTime = script->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_SEND);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_WAIT_TIME : { // return the WaitTime property
			double dTime = script->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_WAIT);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_RECEIVE_TIME : { // return the ReceiveTime property
			double dTime = script->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_RECEIVE);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    default:;
    }
	}
	
//???	return JS_GetProperty(cx, obj, id, vp);
	return JS_TRUE;
}


static JSBool
script_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside script_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLScript* script = (CWLScript*)prv->GetObj();
	if (script && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_SRC)) {	// set the src property
			script->setProtocol(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_EVENT)) {	// set the event property
			script->SetEvent(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_ID)) {	// set the id  property
			script->SetId(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_LANGUAGE)) {	// set the language property
			script->SetLanguage(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_INNERHTML)) {	// set the innerHTML property
			script->setText(WLStringValue(*vp));
			return JS_TRUE;
		}

*/
	return JS_TRUE;
}

static void
script_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside script_finalize");
	WLFinalize(cx, obj);
}



JSClass wl_ScriptClass = {
	WLJSNAME_CLASS_SCRIPT,
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	script_getProperty,
	script_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	script_finalize
};


/*
static JSBool
script_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLScript* script = (CWLScript*)prv->GetObj();
	if (!script)
		return JS_TRUE;

	// set rval to Href property
	const char* id = script->GetText();
	JSString* str = JS_NewStringCopyZ(cx, id);
	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}


static JSFunctionSpec script_methods[] = {
	{wl_toString_str,	script_toString,	0},
	{0}
};
*/


/*
static JSBool
InitScriptObject(JSContext *cx, JSObject *obj, jsint length, jsval *vector)
{
WlTraceMsg("initializing script object properties");
	// create the CPP object


	// add it's properties
	if (!WLInitHiddenProperty(cx, obj, WLJSNAME_PROP_DOM_SCOPE))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_SRC))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_EVENT))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_ID))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_INNERHTML))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_LANGUAGE))
		return JS_FALSE;

	return JS_TRUE;
}


static JSBool
WLScript(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	JSBool ok;

	ok = InitScriptObject(cx, obj, 0, NULL);
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}


bool DllExport WLAddScriptObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddScriptObject(%x, %x)", cx, obj);
	// create an instance of type WLDocument
	JSObject* script = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_SCRIPT, &wl_ScriptClass, NULL, JSPROP_PERMANENT);
	if (!script)
		return false;

	JS_SetPrivate(cx, script, NULL);
	
	// define it's properties & structures
	if (!InitScriptObject(cx, script, 0, NULL))
		return false;

	return true;
}
*/


JSObject* WLNewScriptObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewScriptObject(%x, %x)", cx, obj);
	// create an instance of type WLLink
	JSObject* proto = NULL;
	JSObject* script = WLNewObject(cx, &wl_ScriptClass, proto, obj);
	if (!script)
		return NULL;

	return script;
}


bool DllExport WLAddScriptClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddScriptClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_ScriptClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
