/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvjsapi.h"
#include "wlframecontainer.h"
#include "commandmanager.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "TraceDll.h"

#include "propertieshandler.h"

#define CWLFrame CWLDocument
#define CWLPage frameContainer



static JSBool
window_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	WlTraceMsg("Inside window_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
  if (JS_TypeOfValue(cx,*vp) == JSTYPE_FUNCTION) 
    return JS_TRUE;

	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLFrame* doc = (CWLFrame*)prv->GetParent();
	if (!doc)
		return JS_TRUE;
	frameContainer* frameCont = doc->GetFrameContainer();
	if(!frameCont)
		return JS_TRUE;

  CWLFrame* frame = (CWLFrame*)prv->GetObj();	
	if (frame && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_FRAMES : {	// return the Frames property
			RVOrdered* windows = frame->m_Frames;
			return WLSetArrayValue(cx, obj, doc, WLJSNAME_CLASS_WINDOW, (void**)&windows, vp);
		}		

    case PropertiesHandler::EN_LOCATION : {	// return the Location property
			CWLLocation* location = &(frame->GetLocation());
			return WLSetLocationValue(cx, obj, doc, location, vp);
		}

    case PropertiesHandler::EN_WLSOURCE_INDEX : { // return the WLSourceIndex property
			int index = frame->GetIndex();
			return WLSetIntValue(cx, obj, prop, index, vp);
		}

    case PropertiesHandler::EN_DOCUMENT : {	// return the Document property
			return  WLSetFrameValue(cx, obj, doc, frame, vp);
		}

    case PropertiesHandler::EN_NAME : {	// return the Name property)
			const char* frameName = (frame->GetFrameName()).data();
			return WLSetStringValue(cx, obj, prop, frameName, vp);
		}

    case PropertiesHandler::EN_ID : {	// return the id property
			const char* id = frame->Id();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_FILE_SIZE : { // return the FileSize property
			int nFileSize = frame->GetTransactionObj()->GetFileSize();
			return WLSetIntValue(cx, obj, prop, nFileSize, vp);
		}

    case PropertiesHandler::EN_CHECKSUM : { // return the CheckSum property
			int nCheckSum = frame->GetTransactionObj()->GetCheckSum();
			return WLSetIntValue(cx, obj, prop, nCheckSum, vp);
		}

    case PropertiesHandler::EN_OPEN_TIME : { // return the OpenTime property
			double dTime = frame->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_OPEN_CONNECTION);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_SEND_TIME : { // return the SendTime property
			double dTime = frame->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_SEND);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_WAIT_TIME : { // return the WaitTime property
			double dTime = frame->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_WAIT);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_RECEIVE_TIME : { // return the ReceiveTime property
			double dTime = frame->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_RECEIVE);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    default: {
//here we allow access to frame by frame name window.frameName will return a frame if exists a frame
//by this name.
			CWLFrame* frame_ = frame->m_Frames ? frame->m_Frames->frame(prop) : NULL;
			return WLSetWindowValue(cx, obj, doc, frame_,  vp);
		}
    }
	}
	
	return JS_TRUE;
}


static JSBool
window_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside window_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLFrame* frame = (CWLFrame*)prv->GetObj();
	if (frame && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_FRAMES)) {	// return the Frames property
// 			frame->setFrames(WLArrayValue(cx, *vp));
 			frame->m_Frames = WLArrayValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_LINKS)) {	// return the Links property
// 			frame->setLinks(WLArrayValue(cx, *vp));
 			frame->m_Links = WLArrayValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_FORMS)) {	// return the Forms property
// 			frame->setForms(WLArrayValue(cx, *vp));
 			frame->m_Forms = WLArrayValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_LOCATION)) {	// return the Location property
// 			frame->setLocation(WLLocationValue(cx, *vp));
 			frame->GetLocation() = *WLLocationValue(cx, *vp);
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_NAME)) {	// return the Name property
 			frame->setName(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_REFERRER)) {	// return the Referer property
 			frame->setReferrer(WLLocationValue(cx, *vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SPIDERSESSION)) {	// return the SPIDERSESSION property
 			frame->setSPIDERSESSION(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PROTOCOL_VERSION)) {	// return the protocol version
 			frame->setProtocol(WLDoubleValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_STATUS_NUMBER)) {	// return the reply's status number
 			frame->setStatusNumber(WLIntValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_STATUS_LINE)) {	// return the reply status description
 			frame->setStatusLine(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_HEADER)) {	// return the reply's headers array
 			frame->setHeaders(cx, WLArrayValue(*vp));
			return JS_TRUE;
		}
	}
*/

	return JS_TRUE;
}

JSBool window_ClassEnumerate(JSContext *cx, JSObject *obj ){

  JS_DefineProperty(cx, obj, WLJSNAME_PROP_DOCUMENT, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_LOCATION, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);
  JS_DefineProperty(cx, obj, WLJSNAME_PROP_NAME, JSVAL_VOID, NULL, NULL, JSPROP_PERMANENT | JSPROP_ENUMERATE | JSPROP_READONLY);

  return JS_TRUE;
}

static void
window_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside window_finalize(%x, %x)", cx, obj);
	WLFinalize(cx, obj);
}


JSClass wl_WindowClass = {
	WLJSNAME_CLASS_WINDOW,
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	window_getProperty,
	window_setProperty,
	window_ClassEnumerate,
	JS_ResolveStub,
	JS_ConvertStub,
	window_finalize
};


/*
static JSBool
window_toString(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("window_toString");
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLFrame* frame = (CWLFrame*)prv->GetObj();	
	if (!frame)
		return JS_TRUE;

	CWLLocation* location = &(frame->GetLocation());
	if (!location)
		return JS_TRUE;

	JSString* str = JS_NewStringCopyZ(cx, location->Href());

	if (str)
		*rval = STRING_TO_JSVAL(str);

	return JS_TRUE;
}


static JSFunctionSpec window_methods[] = {
	{wl_toString_str,		window_toString,		0},
	{0}
};
*/


/*
static JSBool
WLWindow(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
WlTraceMsg("WLWindow constructor");
	JSBool ok;

	ok = InitWindowObject(cx, obj, 0, NULL);
	if (ok)
		*rval = OBJECT_TO_JSVAL(obj);

	return ok;
}
*/


static JSBool
InitWindowObject(JSContext *cx, JSObject *obj, jsint length, jsval *vector)
{
WlTraceMsg("initializing window object properties");
	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_FRAMES))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_LOCATION))
		return JS_FALSE;

	if (!WLInitProperty(cx, obj, WLJSNAME_PROP_DOCUMENT))
		return JS_FALSE;

	return JS_TRUE;
}


//add window object
bool DllExport WLAddWindowObject(JSContext *cx, JSObject *obj, JSObject** jswin)
{
WlTraceMsg("WLAddWindowObject(%x, %x)", cx, obj);
	// create an instance of type WLDocument
	JSObject* window = JS_DefineObject(cx, obj, WLJSNAME_OBJECT_WINDOW, &wl_WindowClass, NULL, JSPROP_PERMANENT);
	if (!window)
		return false;

	*jswin = window;

	JS_SetPrivate(cx, window, NULL);
	
	// define it's properties & structures
	if (!InitWindowObject(cx, window, 0, NULL))
		return false;

	return true;
}


JSObject* WLNewWindowObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewWIndowObject(%x, %x)", cx, obj);
	// create an instance of type WLFrame
	JSObject* proto = NULL;
	JSObject* frame = WLNewObject(cx, &wl_WindowClass, proto, obj);
	if (!frame)
		return NULL;

	return frame;
}


bool DllExport WLAddWindowClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddWindowClass(%x, %x)", cx, obj);
	// define a class of type WLFrame
	//if (!JS_InitClass(cx, obj, NULL, &wl_FrameClass, WLFrame, 1, NULL, frame_methods, NULL, NULL))
	if (!JS_InitClass(cx, obj, NULL, &wl_WindowClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
