/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include "wlxml.h"
#include <string.h>
#include "rvjsapi.h"
#include "replyprops.h"
#include "jsreplynames.h"
#include "wlfuncs.h"
#include "jsprivate.h"
#include "tracedll.h"

#include "propertieshandler.h"

/**************** NOTES **********************************/
//	both work:
//	document.all.myxml(0).documentElement.nodeType
//	document.all.myxml(0).XMLDocument.documentElement.nodeType
/*********************************************************/

static JSBool
xml_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside xml_getProperty");
/*
  // if object exists, we do not create a new one.
  if ( !JSVAL_IS_VOID(*vp) )	
  {
    WlTraceMsg("Using an existing object, not creating a new one.");
		return JS_TRUE;
  }
*/
 	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLXmlDocument* xml = (CWLXmlDocument*)prv->GetObj();
	if (!xml)
		return JS_TRUE;

	CWLFrame* frame = (CWLFrame*)prv->GetParent();
	if (!frame)
		return JS_TRUE;

	if (xml && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

    switch (PropertiesHandler::GetPropertyId(prop))
    {
    case PropertiesHandler::EN_SRC : {	// return the src property
			const char* src = xml->GetSrc();
			return WLSetStringValue(cx, obj, prop, src, vp);
		}

    case PropertiesHandler::EN_ID : {	// return id property
			const char* id = xml->GetId();
			return WLSetStringValue(cx, obj, prop, id, vp);
		}

    case PropertiesHandler::EN_INNERHTML : {	// return the innerHTML property - not documented with ms
			const char* innerHtml = xml->GetInnerHTML();
			return WLSetStringValue(cx, obj, prop, innerHtml, vp);
		}

    case PropertiesHandler::EN_WLSOURCE_INDEX : { // return the WLSourceIndex property
			int index = xml->GetIndex();
			return WLSetIntValue(cx, obj, prop, index, vp);
		}

    case PropertiesHandler::EN_NAME : { // return the FileSize property
			const char* name = xml->GetName();
			return WLSetStringValue(cx, obj, prop, name, vp);
		}

    case PropertiesHandler::EN_FILE_SIZE : { // return the FileSize property
			int nFileSize = xml->GetTransactionObj()->GetFileSize();
			return WLSetIntValue(cx, obj, prop, nFileSize, vp);
		}

    case PropertiesHandler::EN_CHECKSUM : { // return the CheckSum property
			int nCheckSum = xml->GetTransactionObj()->GetCheckSum();
			return WLSetIntValue(cx, obj, prop, nCheckSum, vp);
		}

    case PropertiesHandler::EN_OPEN_TIME : { // return the OpenTime property
			double dTime = xml->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_OPEN_CONNECTION);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_SEND_TIME : { // return the SendTime property
			double dTime = xml->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_SEND);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_WAIT_TIME : { // return the WaitTime property
			double dTime = xml->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_WAIT);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    case PropertiesHandler::EN_RECEIVE_TIME : { // return the ReceiveTime property
			double dTime = xml->GetTransactionObj()->GetSubTransTime(CWLTransactionData::TRANS_RECEIVE);
			return WLSetDoubleValue(cx, obj, prop, dTime, vp);
		}

    default:;
    }
	}

	return JS_TRUE;
}


static JSBool
xml_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
WlTraceMsg("Inside xml_setProperty");
/*
	CWLPrivate* prv = (CWLPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return JS_TRUE;

	CWLXmlDocument* xml = (CWLXmlDocument*)prv->GetObj();
	if (xml && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);

		if (!strcmp(prop, WLJSNAME_PROP_SRC)) {	// set the src property
			xml->setProtocol(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_ID)) {	// set the id  property
			xml->SetId(WLStringValue(*vp));
			return JS_TRUE;
		}
		else if (!strcmp(prop, WLJSNAME_PROP_INNERHTML)) {	// set the innerHTML property
			xml->setText(WLStringValue(*vp));
			return JS_TRUE;
		}

*/
	return JS_TRUE;
}

static void
xml_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside xml_finalize");
	WLFinalize(cx, obj);
}



JSClass wl_XmlClass = {
	WLJSNAME_CLASS_XML,
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	xml_getProperty,
	xml_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	xml_finalize
};





JSObject* WLNewXmlObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewXmlObject(%x, %x)", cx, obj);
	// create an instance of type WLLink
	JSObject* proto = NULL;
	JSObject* xml = WLNewObject(cx, &wl_XmlClass, proto, obj);
	if (!xml)
		return NULL;

	return xml;
}


bool DllExport WLAddXmlClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddXmlClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_XmlClass, NULL, 1, NULL, NULL, NULL, NULL))
		return false;

	return true;
}
