/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// this code is included only in the SSL 40 & 128 versions !
#ifdef __SSL__

#ifdef __TV_UNIX__
#include "windows.h"
#endif
#include "sslciphers.h"
#include "httpdefines.h"
#include "connpool.h"
#include "tracedll.h"


#define reportError m_Cp->ReportError

#define   MIN_STRENGTH  40
#define   MAX_STRENGTH 168

#define min(a,b) (((a) < (b)) ? (a) : (b))
#define max(a,b) (((a) > (b)) ? (a) : (b))


/*
To change the enabled cipher suites:
1. change the m_Suites array to consist of only the desired cipher suites.
2. for each template (SSLContext) exist and created, set the enabled cipher suites for it.
*/

/*
This is the cipherSuiteInfo structure and values. Each of it's values can be retrieved
by index (SSLGetIndexedCiphersuiteInfo) or by id (SSLGetCiphersuiteInfo).

			Cipher Suites	
			ordered by strength (keysize) - strongest	to weakest

Name and Alias	                          Key	  Export	Description	
==============                            ===   ======  ===========
			
SSL_RSA_WITH_3DES_EDE_CBC_MD5	            128	  no	    RSA key exchange with MD5 hashing and 128-bit triple DES encryption using EDE in CBC mode
SSL_RSA_WITH_IDEA_CBC_MD5	                128	  no	    RSA key exchange with MD5 hashing and IDEA encryption in CBC mode
TLS_DHE_DSS_WITH_RC4_128_SHA	            128	  no	    Ephemeral Diffie-Hellman key exchange with SHA-1 hashing and 128-bit ARC4/RC4 encryption
TLS_ECDH_anon_WITH_3DES_EDE_CBC_SHA	      128	  no	    Elliptic Curve Anonymous Diffie-Hellman with SHA-1 hashing and 128-bit triple DES encryption using EDE in CBC mode
TLS_ECDH_anon_WITH_RC4_128_SHA	          128	  no	    Elliptic Curve Anonymous Diffie-Hellman with SHA-1 hashing and 128-bit ARC4/RC4 encryption
TLS_ECMQV_ECNRA_WITH_3DES_EDE_CBC_SHA	    128	  no	    Elliptic Curve Menezes-Qu-Vanstone and Elliptic Curve NRA analog with SHA-1 hashing and 128-bit triple DES encryption using EDE in CBC mode
TLS_ECMQV_ECNRA_WITH_RC4_128_SHA	        128	  no	    Elliptic Curve Menezes-Qu-Vanstone and Elliptic Curve NRA analog with SHA-1 hashing and 128-bit ARC4/RC4 encryption
TLS_ECMQV_ECDSA_WITH_3DES_EDE_CBC_SHA	    128	  no	    Elliptic Curve Menezes-Qu-Vanstone and Elliptic Curve DSA key exchange with SHA-1 hashing and 128-bit triple DES encryption using EDE in CBC mode
TLS_ECMQV_ECDSA_WITH_RC4_128_SHA	        128	  no	    Elliptic Curve Menezes-Qu-Vanstone key exchange with SHA-1 hashing and 128-bit ARC4/RC4 encryption
TLS_ECDH_ECNRA_WITH_3DES_EDE_CBC_SHA	    128	  no	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve NRA analog with SHA-1 hashing and 128-bit triple DES encryption using EDE in CBC mode
TLS_ECDH_ECNRA_WITH_RC4_128_SHA	          128	  no	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve NRA analog with SHA-1 hashing and 128-bit ARC4/RC4 encryption
TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA	    128	  no	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing and 128-bit triple DES encryption using EDE in CBC mode
TLS_ECDH_ECDSA_WITH_RC4_128_SHA	          128	  no	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing and 128-bit ARC4/RC4 encryption
TLS_ECDHE_ECNRA_WITH_3DES_EDE_CBC_SHA	    128	  no	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve NRA analog with SHA-1 hashing and 128-bit triple DES encryption using EDE in CBC mode
TLS_ECDHE_ECNRA_WITH_RC4_128_SHA	        128	  no	    Ephemeral Elliptic Curve Diffie-Hellman analog and Elliptic Curve NRA analog with SHA-1 hashing and 128-bit ARC4/RC4 encryption
TLS_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA	    128	  no	    Ephemeral Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing and 128-bit triple DES encryption using EDE in CBC mode
TLS_ECDHE_ECDSA_WITH_RC4_128_SHA	        128	  no	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing and 128-bit ARC4/RC4 encryption
TLS_ECES_ECNRA_WITH_3DES_EDE_CBC_SHA	    128	  no	    Elliptic Curve encryption scheme and Elliptic Curve NRA analog with SHA-1 hashing and 128-bit triple DES encryption using EDE in CBC mode
TLS_ECES_ECNRA_WITH_RC4_128_SHA	          128	  no	    Elliptic Curve encryption scheme and Elliptic Curve NRA analog with SHA-1 hashing and 128-bit ARC4/RC4 encryption
TLS_ECES_ECDSA_WITH_3DES_EDE_CBC_SHA	    128	  no	    Elliptic Curve encryption scheme and Elliptic Curve DSA analog with SHA-1 hashing and 128-bit triple DES encryption using EDE in CBC mode
TLS_ECES_ECDSA_WITH_RC4_128_SHA	          128	  no	    Elliptic Curve encryption scheme and Elliptic Curve DSA analog with SHA-1 hashing and 128-bit ARC4/RC4 encryption
TLS_DH_anon_WITH_3DES_EDE_CBC_SHA	        128	  no	    Anonymous Diffie-Hellman key exchange with SHA-1 hashing and triple DES 128-bit encryption using EDE in CBC mode
TLS_DH_anon_WITH_RC4_128_MD5	            128	  no	    Anonymous Diffie-Hellman key exchange with MD5 hashing and 128-bit ARC4/RC4 encryption
TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA	        128	  no	    Ephemeral Diffie-Hellman with RSA key exchange with SHA-1 hashing and triple DES 128-bit encryption using EDE in CBC mode
TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA	        128	  no	    Ephemeral Diffie-Hellman key exchange with SHA-1 hashing and triple DES 128-bit encryption using EDE in CBC mode
TLS_DH_RSA_WITH_3DES_EDE_CBC_SHA	        128	  no	    Diffie-Hellman with RSA key exchange with SHA-1 hashing and triple DES 128-bit encryption using EDE in CBC mode
TLS_DH_DSS_WITH_3DES_EDE_CBC_SHA	        128	  no	    Diffie-Hellman key exchange with SHA-1 hashing and triple DES 128-bit encryption using EDE in CBC mode
TLS_RSA_WITH_3DES_EDE_CBC_SHA	            128	  no	    RSA key exchange with SHA-1 hashing and triple DES 128-bit encryption using EDE in CBC mode
TLS_RSA_WITH_IDEA_CBC_SHA	                128	  yes	    RSA key exchange with SHA-1 hashing and IDEA encryption in CBC mode
TLS_RSA_WITH_RC4_128_SHA	                128	  no	    RSA key exchange with SHA-1 hashing and ARC4/RC4 128-bit encryption
TLS_RSA_WITH_RC4_128_MD5	                128	  no	    RSA key exchange with MD5 hashing and ARC4/RC4 128-bit encryption
                                                        
TLS_ECDH_ECDSA_EXPORT_WITH_RC4_56_SHA	    56	  yes	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing and 56-bit ARC4/RC4 encryption
SSL_RSA_WITH_DES_CBC_MD5	                56	  no	    RSA key exchange with MD5 hashing and 56-bit DES encryption in CBC mode
TLS_DHE_DSS_EXPORT1024_WITH_RC4_56_SHA	  56	  yes	    Ephemeral Diffie-Hellman key exchange with SHA-1 hashing and 56-bit ARC4/RC4 encryption
TLS_RSA_EXPORT1024_WITH_RC4_56_SHA	      56	  yes	    RSA key exchange with SHA-1 hashing and 56-bit ARC4/RC4 encryption
TLS_DHE_DSS_EXPORT1024_WITH_DES_CBC_SHA	  56	  yes	    Ephemeral Diffie-Hellman key exchange with SHA-1 hashing and 56-bit DES encryption in CBC mode
TLS_RSA_EXPORT1024_WITH_DES_CBC_SHA	      56	  yes	    RSA key exchange with SHA-1 hashing and 56-bit DES encryption in CBC mode
TLS_ECDH_anon_WITH_DES_CBC_SHA	          56	  no	    Elliptic Curve Anonymous Diffie-Hellman with SHA-1 hashing and 56-bit DES encryption in CBC mode
TLS_ECMQV_ECNRA_WITH_DES_CBC_SHA	        56	  no	    Elliptic Curve Menezes-Qu-Vanstone and Elliptic Curve NRA analog with SHA-1 hashing and 56-bit DES encryption in CBC mode
TLS_ECMQV_ECDSA_WITH_DES_CBC_SHA	        56	  no	    Elliptic Curve Menezes-Qu-Vanstone and Elliptic Curve DSA key exchange with SHA-1 hashing and 56-bit DES encryption in CBC mode
TLS_ECDH_ECNRA_WITH_DES_CBC_SHA	          56	  no	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve NRA analog with SHA-1 hashing and 56-bit DES encryption in CBC mode
TLS_ECDH_ECDSA_WITH_DES_CBC_SHA	          56	  no	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing and 56-bit DES encryption in CBC mode
TLS_ECDHE_ECNRA_WITH_DES_CBC_SHA	        56	  no	    Ephemeral Elliptic Curve Diffie-Hellman analog and Elliptic Curve NRA analog with SHA-1 hashing and 56-bit DES encryption in CBC mode
TLS_ECDHE_ECDSA_WITH_DES_CBC_SHA	        56	  no	    Ephemeral Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing and 56-bit DES encryption in CBC mode
TLS_ECES_ECNRA_WITH_DES_CBC_SHA	          56	  no	    Elliptic Curve encryption scheme and Elliptic Curve NRA analog with SHA-1 hashing and 56-bit DES encryption in CBC mode
TLS_ECES_ECDSA_WITH_DES_CBC_SHA	          56	  no	    Elliptic Curve encryption scheme and Elliptic Curve DSA analog with SHA-1 hashing and 56-bit DES encryption in CBC mode
TLS_DH_anon_WITH_DES_CBC_SHA	            56	  no	    Anonymous Diffie-Hellman key exchange with SHA-1 hashing and DES 56-bit encryption in CBC mode
TLS_DHE_RSA_WITH_DES_CBC_SHA	            56	  no	    Ephemeral Diffie-Hellman with RSA key exchange with SHA-1 hashing and DES 56-bit encryption in CBC mode
TLS_DHE_DSS_WITH_DES_CBC_SHA	            56	  no	    Ephemeral Diffie-Hellman key exchange with SHA-1 hashing and DES 56-bit encryption in CBC mode
TLS_DH_RSA_WITH_DES_CBC_SHA	              56	  no	    Diffie-Hellman with RSA key exchange with SHA-1 hashing and DES 56-bit encryption in CBC mode
TLS_DH_DSS_WITH_DES_CBC_SHA	              56	  no	    Diffie-Hellman key exchange with SHA-1 hashing and DES 56-bit encryption in CBC mode
TLS_RSA_WITH_DES_CBC_SHA	                56	  no	    RSA key exchange with SHA-1 hashing and DES 56-bit encryption in CBC mode
                                                        
TLS_ECDH_ECDSA_EXPORT_WITH_RC4_40_SHA	    40	  yes	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing and 40-bit ARC4/RC4 encryption
SSL_RSA_WITH_RC2_CBC_MD5	                40	  no	    RSA key exchange with MD5 hashing and RC2 40-bit encryption in CBC mode
TLS_ECDH_anon_EXPORT_WITH_RC4_40_SHA	    40	  yes	    Elliptic Curve Anonymous Diffie-Hellman with SHA-1 hashing and 40-bit ARC4/RC4 encryption
TLS_ECDH_anon_EXPORT_WITH_DES40_CBC_SHA	  40	  yes	    Elliptic Curve Anonymous Diffie-Hellman with SHA-1 hashing and 40-bit DES encryption in CBC mode
TLS_ECDHE_ECNRA_EXPORT_WITH_RC4_40_SHA	  40	  yes	    Ephemeral Elliptic Curve Diffie-Hellman analog and Elliptic Curve NRA analog with SHA-1 hashing and 40-bit ARC4/RC4 encryption
TLS_ECDHE_ECNRA_EXPORT_WITH_DES40_CBC_SHA	40	  yes	    Ephemeral Elliptic Curve Diffie-Hellman analog and Elliptic Curve NRA analog with SHA-1 hashing and 40-bit DES encryption in CBC mode
TLS_ECDHE_ECDSA_EXPORT_WITH_RC4_40_SHA	  40	  yes	    Ephemeral Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing and 40-bit ARC4/RC4 encryption
TLS_ECDHE_ECDSA_EXPORT_WITH_DES40_CBC_SHA	40	  yes	    Ephemeral Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing and 40-bit DES encryption in CBC mode
TLS_DH_anon_EXPORT_WITH_DES40_CBC_SHA	    40	  yes	    Anonymous Diffie-Hellman key exchange with SHA-1 hashing and DES 40-bit encryption in CBC mode
TLS_DH_anon_EXPORT_WITH_RC4_40_MD5	      40	  yes	    Anonymous Diffie-Hellman key exchange with MD5 hashing and 40-bit ARC4/RC4 encryption
TLS_DHE_RSA_EXPORT_WITH_DES40_CBC_SHA	    40	  yes	    Ephemeral Diffie-Hellman with RSA key exchange with SHA-1 hashing and DES 40-bit encryption in CBC mode
TLS_DHE_DSS_EXPORT_WITH_DES40_CBC_SHA	    40	  yes	    Ephemeral Diffie-Hellman key exchange with SHA-1 hashing and DES 40-bit encryption in CBC mode
TLS_DH_RSA_EXPORT_WITH_DES40_CBC_SHA	    40	  yes	    Diffie-Hellman with RSA key exchange with SHA-1 hashing and DES 40-bit encryption in CBC mode
TLS_DH_DSS_EXPORT_WITH_DES40_CBC_SHA	    40	  yes	    Diffie-Hellman key exchange with SHA-1 hashing and DES 40-bit encryption in CBC mode
TLS_RSA_EXPORT_WITH_DES40_CBC_SHA	        40	  yes	    RSA key exchange with SHA-1 hashing and DES 40-bit encryption in CBC mode
TLS_RSA_EXPORT_WITH_RC2_CBC_40_MD5	      40	  yes	    RSA key exchange with MD5 hashing and RC2 40-bit encryption in CBC mode
TLS_RSA_EXPORT_WITH_RC4_40_MD5	          40	  yes	    RSA key exchange with MD5 hashing and ARC4/RC4 40-bit encryption
                                                        
TLS_ECDH_anon_NULL_WITH_SHA	              0	    yes	    Elliptic Curve Anonymous Diffie-Hellman with SHA-1 hashing	
TLS_ECMQV_ECNRA_NULL_SHA	                0	    yes	    Elliptic Curve Menezes-Qu-Vanstone and Elliptic Curve NRA analog with SHA-1 hashing	
TLS_ECMQV_ECDSA_NULL_SHA	                0	    yes	    Elliptic Curve Menezes-Qu-Vanstone and Elliptic Curve DSA key exchange with SHA-1 hashing	
TLS_ECDH_ECNRA_NULL_SHA	                  0	    yes	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve NRA analog with SHA-1 hashing	
TLS_ECDH_ECDSA_NULL_SHA	                  0	    yes	    Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing	
TLS_ECDHE_ECNRA_NULL_SHA	                0	    yes	    Ephemeral Elliptic Curve Diffie-Hellman analog and Elliptic Curve NRA analog with SHA-1 hashing	
TLS_ECDHE_ECDSA_NULL_SHA	                0	    yes	    Ephemeral Elliptic Curve Diffie-Hellman analog and Elliptic Curve DSA analog with SHA-1 hashing	
TLS_ECES_ECNRA_NULL_SHA	                  0	    yes	    Elliptic Curve encryption scheme and Elliptic Curve NRA analog with SHA-1 hashing	
TLS_ECES_ECDSA_NULL_SHA	                  0	    yes	    Elliptic Curve encryption scheme and Elliptic Curve DSA analog with SHA-1 hashing	
TLS_RSA_WITH_NULL_SHA	                    0	    yes	    RSA key exchange with SHA-1 hashing	
TLS_RSA_WITH_NULL_MD5	                    0	    yes	    RSA key exchange with MD5 hashing	
TLS_NULL_WITH_NULL_NULL	                  0	    yes	    No key exchange, hashing or encryption	

*/

int  knownCipherSuites[] = {
	SSL_NULL_WITH_NULL_NULL,
	SSL_RSA_WITH_RC4_128_MD5,
	SSL_RSA_WITH_RC4_128_SHA,
	SSL_RSA_EXPORT_WITH_RC2_CBC_40_MD5,
	SSL_RSA_WITH_IDEA_CBC_SHA,
	SSL_RSA_EXPORT_WITH_DES40_CBC_SHA,
	SSL_RSA_WITH_DES_CBC_SHA,
	SSL_RSA_WITH_3DES_EDE_CBC_SHA,
	SSL_DH_DSS_EXPORT_WITH_DES40_CBC_SHA,
	SSL_DH_DSS_WITH_DES_CBC_SHA,
	SSL_DH_DSS_WITH_3DES_EDE_CBC_SHA,
	SSL_DH_RSA_EXPORT_WITH_DES40_CBC_SHA,
	SSL_DH_RSA_WITH_DES_CBC_SHA,
	SSL_DH_RSA_WITH_3DES_EDE_CBC_SHA,
	SSL_DHE_DSS_EXPORT_WITH_DES40_CBC_SHA,
	SSL_DHE_DSS_WITH_DES_CBC_SHA,
	SSL_DHE_DSS_WITH_3DES_EDE_CBC_SHA,
	SSL_DHE_RSA_EXPORT_WITH_DES40_CBC_SHA,
	SSL_DHE_RSA_WITH_DES_CBC_SHA,
	SSL_DHE_RSA_WITH_3DES_EDE_CBC_SHA,
	SSL_DH_anon_EXPORT_WITH_RC4_40_MD5,
	SSL_DH_anon_WITH_RC4_128_MD5,
	SSL_DH_anon_EXPORT_WITH_DES40_CBC_SHA,
	SSL_DH_anon_WITH_DES_CBC_SHA,
	SSL_DH_anon_WITH_3DES_EDE_CBC_SHA,
	SSL_FORTEZZA_DMS_WITH_NULL_SHA,
	SSL_FORTEZZA_DMS_WITH_FORTEZZA_CBC_SHA,
	TLS_ECDH_ECDSA_WITH_NULL_SHA,
  TLS_ECDH_ECDSA_WITH_RC4_128_SHA	          ,
  TLS_ECDH_ECDSA_WITH_DES_CBC_SHA	          ,
	TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA,
	TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA,
	TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA,
  TLS_ECDH_ECDSA_EXPORT_WITH_RC4_40_SHA	    ,
	TLS_ECDH_ECDSA_EXPORT_WITH_RC4_56_SHA,
	TLS_ECDH_RSA_WITH_NULL_SHA,
	TLS_ECDH_RSA_WITH_RC4_128_SHA,
	TLS_ECDH_RSA_WITH_DES_CBC_SHA,
	TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA,
	TLS_ECDH_RSA_WITH_AES_128_CBC_SHA,
	TLS_ECDH_RSA_WITH_AES_256_CBC_SHA,
	TLS_ECDH_RSA_EXPORT_WITH_RC4_40_SHA,
	TLS_ECDH_RSA_EXPORT_WITH_RC4_56_SHA,
  TLS_ECDH_anon_NULL_WITH_SHA	              ,
  TLS_ECDH_anon_WITH_RC4_128_SHA	          ,
  TLS_ECDH_anon_WITH_DES_CBC_SHA	          ,
	TLS_ECDH_anon_WITH_3DES_EDE_CBC_SHA,
	TLS_ECDH_anon_EXPORT_WITH_DES40_CBC_SHA,
	TLS_ECDH_anon_EXPORT_WITH_RC4_40_SHA,
	TLS_RSA_EXPORT1024_WITH_DES_CBC_SHA,
	TLS_DHE_DSS_EXPORT1024_WITH_DES_CBC_SHA,
	TLS_RSA_EXPORT1024_WITH_RC4_56_SHA,
	TLS_DHE_DSS_EXPORT1024_WITH_RC4_56_SHA,
	TLS_DHE_DSS_WITH_RC4_128_SHA,
	SSL_RSA_WITH_RC2_CBC_MD5,
	SSL_RSA_WITH_IDEA_CBC_MD5,
	SSL_RSA_WITH_DES_CBC_MD5,
	SSL_RSA_WITH_3DES_EDE_CBC_MD5,
	TLS_ECDH_ECDSA_EXPORT_WITH_RC4_40_SHA,
	TLS_ECDH_ECDSA_EXPORT_WITH_RC4_56_SHA,
	TLS_RSA_WITH_AES_128_CBC_SHA,
	TLS_RSA_WITH_AES_256_CBC_SHA,
	SSL_RSA_WITH_NULL_MD5,
	SSL_RSA_WITH_NULL_SHA,
	SSL_RSA_EXPORT_WITH_RC4_40_MD5
};
int  supportedCipherSuites[] = {
	SSL_NULL_WITH_NULL_NULL,
	SSL_RSA_WITH_RC4_128_MD5,
	SSL_RSA_WITH_RC4_128_SHA,
	SSL_RSA_EXPORT_WITH_RC2_CBC_40_MD5,
	SSL_RSA_WITH_IDEA_CBC_SHA,
	SSL_RSA_EXPORT_WITH_DES40_CBC_SHA,
	SSL_RSA_WITH_DES_CBC_SHA,
	SSL_RSA_WITH_3DES_EDE_CBC_SHA,
	SSL_DH_DSS_EXPORT_WITH_DES40_CBC_SHA,
	SSL_DH_DSS_WITH_DES_CBC_SHA,
	SSL_DH_DSS_WITH_3DES_EDE_CBC_SHA,
	SSL_DH_RSA_EXPORT_WITH_DES40_CBC_SHA,
	SSL_DH_RSA_WITH_DES_CBC_SHA,
	SSL_DH_RSA_WITH_3DES_EDE_CBC_SHA,
	SSL_DHE_DSS_EXPORT_WITH_DES40_CBC_SHA,
	SSL_DHE_DSS_WITH_DES_CBC_SHA,
	SSL_DHE_DSS_WITH_3DES_EDE_CBC_SHA,
	SSL_DHE_RSA_EXPORT_WITH_DES40_CBC_SHA,
	SSL_DHE_RSA_WITH_DES_CBC_SHA,
	SSL_DHE_RSA_WITH_3DES_EDE_CBC_SHA,
	SSL_DH_anon_EXPORT_WITH_RC4_40_MD5,
	SSL_DH_anon_WITH_RC4_128_MD5,
	SSL_DH_anon_EXPORT_WITH_DES40_CBC_SHA,
	SSL_DH_anon_WITH_DES_CBC_SHA,
	SSL_DH_anon_WITH_3DES_EDE_CBC_SHA,
	SSL_FORTEZZA_DMS_WITH_NULL_SHA,
	SSL_FORTEZZA_DMS_WITH_FORTEZZA_CBC_SHA,
	TLS_ECDH_ECDSA_WITH_NULL_SHA,
	TLS_ECDH_ECDSA_WITH_RC4_128_SHA,
	TLS_ECDH_ECDSA_WITH_DES_CBC_SHA,
	TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA,
	TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA,
	TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA,
	TLS_ECDH_ECDSA_EXPORT_WITH_RC4_40_SHA,
	TLS_ECDH_ECDSA_EXPORT_WITH_RC4_56_SHA,
	TLS_ECDH_RSA_WITH_NULL_SHA,
	TLS_ECDH_RSA_WITH_RC4_128_SHA,
	TLS_ECDH_RSA_WITH_DES_CBC_SHA,
	TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA,
	TLS_ECDH_RSA_WITH_AES_128_CBC_SHA,
	TLS_ECDH_RSA_WITH_AES_256_CBC_SHA,
	TLS_ECDH_RSA_EXPORT_WITH_RC4_40_SHA,
	TLS_ECDH_RSA_EXPORT_WITH_RC4_56_SHA,
	TLS_ECDH_anon_NULL_WITH_SHA,
	TLS_ECDH_anon_WITH_RC4_128_SHA,
	TLS_ECDH_anon_WITH_DES_CBC_SHA,
	TLS_ECDH_anon_WITH_3DES_EDE_CBC_SHA,
	TLS_ECDH_anon_EXPORT_WITH_DES40_CBC_SHA,
	TLS_ECDH_anon_EXPORT_WITH_RC4_40_SHA,
	TLS_RSA_EXPORT1024_WITH_DES_CBC_SHA,
	TLS_DHE_DSS_EXPORT1024_WITH_DES_CBC_SHA,
	TLS_RSA_EXPORT1024_WITH_RC4_56_SHA,
	TLS_DHE_DSS_EXPORT1024_WITH_RC4_56_SHA,
	TLS_DHE_DSS_WITH_RC4_128_SHA,
	SSL_RSA_WITH_RC2_CBC_MD5,
	SSL_RSA_WITH_IDEA_CBC_MD5,
	SSL_RSA_WITH_DES_CBC_MD5,
	SSL_RSA_WITH_3DES_EDE_CBC_MD5,
	TLS_ECDH_ECDSA_EXPORT_WITH_RC4_40_SHA,
	TLS_ECDH_ECDSA_EXPORT_WITH_RC4_56_SHA,
	TLS_RSA_WITH_AES_128_CBC_SHA,
	TLS_RSA_WITH_AES_256_CBC_SHA,
	SSL_RSA_WITH_NULL_MD5,
	SSL_RSA_WITH_NULL_SHA,
	SSL_RSA_EXPORT_WITH_RC4_40_MD5,
};
  

unsigned  gSizeKnown = sizeof(knownCipherSuites)/sizeof(knownCipherSuites[0]);
unsigned  gSizeSupported = sizeof(supportedCipherSuites)/sizeof(supportedCipherSuites[0]);


int*      gSuitesUsed = supportedCipherSuites;
unsigned  gSuitesSize = gSizeSupported;



/*
   =================================================================
	   local & global help functions
   =================================================================
*/
static bool isKnown(int cipherId)
{
	for (unsigned i = 0; i < gSizeKnown; i++)
		if (knownCipherSuites[i] == cipherId)
			return true;

	return false;
}

static bool isSupported(int cipherId)
{
	for (unsigned i = 0; i < gSizeSupported; i++)
		if (supportedCipherSuites[i] == cipherId)
			return true;

	return false;
}

static const char* getCipherName(int id)
{
	SSLCiphersuiteInfo  csInfo;
	for (unsigned i = 0; i < gSuitesSize; i++) {
		SSLErr err = SSLGetCiphersuiteInfo(gSuitesUsed[i], &csInfo);
		if (csInfo.id == id)
			return csInfo.suiteName;
	}

	return "unknown";
}

// check if id corresponds to "NULL_WITTH_NULL_NULL" suite
// actually, if a name was not found, then it's unknown suite.
static bool isNameFound(int id, const char* name)
{
	if (id == SSL_NULL_WITH_NULL_NULL) {
		// check if the requested suite was not NULL_WITH_NULL_NULL
		SSLCiphersuiteInfo  csInfo;
		for (unsigned i = 0; i < gSuitesSize; i++) {
			SSLErr err = SSLGetCiphersuiteInfo(gSuitesUsed[i], &csInfo);
			if (csInfo.id == id)
				if (strcmp(name, csInfo.suiteName))
					return false;
		}
	}

	return true;
}

static unsigned getMaxStrength()
{
	unsigned maxStrength = 0;
	SSLCiphersuiteInfo  csInfo;

	for (unsigned scan = 0; scan < gSuitesSize; scan++ ) {
		SSLErr err = SSLGetCiphersuiteInfo(gSuitesUsed[scan], &csInfo);
		if (err == SSLNoErr) {
			maxStrength = max(maxStrength, csInfo.keyBits);
		}
	}
	if (maxStrength == 0)
		maxStrength = MAX_STRENGTH;	// max known todate

	return maxStrength;
}

static unsigned getMinStrength()
{
	unsigned minStrength = MIN_STRENGTH;	// min known todate
	SSLCiphersuiteInfo  csInfo;

	for (unsigned scan = 0; scan < gSuitesSize; scan++ ) {
		SSLErr err = SSLGetCiphersuiteInfo(gSuitesUsed[scan], &csInfo);
		if (err == SSLNoErr) {
			if (csInfo.keyBits < minStrength &&
					csInfo.keyBits > 0)
				minStrength = csInfo.keyBits;
		}
	}

	return minStrength;
}

bool isInList (int bits, int* list)
{
	for (int i = 0; list[i]; i ++) {
		if (list[i] == bits)
			return true;
	}

	return false;
}

// return a list of legal strength values
void SSLGetStrengthList(char** list)
{
	int* intList = (int*)calloc(gSuitesSize, sizeof(int));
	if (!intList)
		return;

	int emptySlot = 0;

	SSLCiphersuiteInfo  csInfo;
	for ( unsigned scan = 0; scan < gSuitesSize; ++scan ) {
		int cipherSuite = gSuitesUsed[scan];
		SSLErr err = SSLGetCiphersuiteInfo(cipherSuite, &csInfo);
		if (err == SSLNoErr) {
			if (csInfo.keyBits == 0 || (scan > 0 && isInList(csInfo.keyBits, intList)))
				continue;
			else
				intList[emptySlot++] = csInfo.keyBits;
		}
	}

	// allocate 8 bytes for each strength found
	*list = (char*)calloc(emptySlot * 8, sizeof(char));
	if (!list) {
		free(intList);
		return;
	}

	bool first = true;
	char num[16];
	strcat(*list, "0,");
	for (int i = 0; i < emptySlot; i++) {
		if (!first)
			strcat(*list, ",");
		else
			first = false;
		sprintf(num, "%d", intList[i]);
		strcat(*list, num);
	}

	free(intList);
}





// ---------------------------------------------------------------------------------
// CWLSSLCiphers class

CWLSSLCiphers::CWLSSLCiphers(CWLConnectionsPool* cp)
{
	m_Cp = cp;
	m_Init = false;
	m_Count = 0;
	m_Strength = 0;
	setAll();	// set all cipher suites to be active
}

SSLErr CWLSSLCiphers::Init(unsigned strength, bool verifyStrength)
{

	SSLErr err = SSLNoErr;

	if (m_Init) {
		if (strength != DEFAULT_CIPHER_STRENGTH &&
				strength != m_RequestedStrength)
			reportError(CIPHER_ERROR_FILE, 50, m_Strength);
		return err;
	}

	m_RequestedStrength = strength;

	m_Init = true;

	if (strength == DEFAULT_CIPHER_STRENGTH)	{ // this is the default, so do nothing (2^32 - 1)
		err = SSLNoErr;
		strength = m_Strength;	// here m_Strength is the max
	}

	else if (strength == 0) {
		resetAll();
		err = SSLNoErr;
	}

	else if (strength > 0) {
		resetAll();
		err = setStrength(strength, verifyStrength);
	}

	return err;
}

// update the enabled cipher suites to a context
SSLErr CWLSSLCiphers::Update(SSLContext* ctx)
{
	return setSuites(ctx);
}

// remove all cipher suites
void CWLSSLCiphers::resetAll()
{
	for (int i = 0; i < MAX_CIPHER_SUITES; i++)
		m_Suites[i] = 0;
	m_Count = 0;
}

// set all cipher suites
void CWLSSLCiphers::setAll()
{
	// set available cipher suites to all
	m_Strength = getMaxStrength();
	m_Count = gSuitesSize;
	unsigned  scan;
	int cipherSuite;
	for ( scan = 0; scan < m_Count; ++scan ) {
		cipherSuite = gSuitesUsed[scan];
		m_Suites[scan] = cipherSuite;
		SSLCiphersuiteInfo  csInfo;
		SSLErr err = SSLGetCiphersuiteInfo(cipherSuite, &csInfo);
    WlTraceMsg("* Added [%4d ] %s (0x%0.4hX)", csInfo.keyBits, csInfo.suiteName, cipherSuite);
	}
  WlTraceMsg("Total of %d ciphers added", m_Count);
}


// set available cipher suites to those equal or less than strength
SSLErr CWLSSLCiphers::setStrength(unsigned strength, bool verifyStrength)
{
	// use a strength that is within the available boundaries
	unsigned maxStrength = getMaxStrength();
	if (strength > maxStrength) {
		if (verifyStrength)
			reportError(CIPHER_ERROR_FILE, 51, strength, maxStrength);
		m_Strength = maxStrength;
	}
	else {
		unsigned minStrength = getMinStrength();
		if (strength < minStrength) {
			if (verifyStrength)
				reportError(CIPHER_ERROR_FILE, 52, strength, minStrength);
			m_Strength = minStrength;
		}
		else {
			m_Strength = strength;
		}
	}

	SSLErr err = SSLNoErr;

	m_Count = 0;
	unsigned  size = gSuitesSize;
	unsigned  scan;
	unsigned  maxStrengthUsed = 0;
	int       cipherSuite;
	SSLCiphersuiteInfo  csInfo;
	for ( scan = 0; scan < size; ++scan ) {
		cipherSuite = gSuitesUsed[scan];
		err = SSLGetCiphersuiteInfo(cipherSuite, &csInfo);
		if (err == SSLNoErr) {
			if (csInfo.keyBits <= m_Strength) {
				m_Suites[m_Count] = cipherSuite;
				m_Count++;
				maxStrengthUsed = max(maxStrengthUsed, csInfo.keyBits);
        WlTraceMsg("* Added [%4d ] %s (0x%0.4hX)", csInfo.keyBits, csInfo.suiteName, cipherSuite);
			}
		}
		else {
			// error retrieving cipher suite information
		}
	}

	// set m_Strength to max exist (if strength == 130, then m_Strength will be 128)
	if (maxStrengthUsed > 0 &&
			maxStrengthUsed != m_Strength) {
		m_Strength = maxStrengthUsed;
		if (verifyStrength)
			reportError(CIPHER_ERROR_FILE, 53, m_RequestedStrength, m_Strength);
	}

  WlTraceMsg("Total of %d ciphers added", m_Count);
	return err;
}



// enable the cipher with the given name
void CWLSSLCiphers::EnableNamed( const char *cipherName, bool& wasChanged )
{
	int cipherSuite = SSLCiphersuiteNameToNumber( cipherName );
	if (isNameFound(cipherSuite, cipherName)) {
  	EnableId( cipherSuite, wasChanged );
  }
  else {
		reportError(CIPHER_ERROR_FILE, 22, cipherName);
	}
}


// disable the cipher with the given name
void CWLSSLCiphers::DisableNamed( const char *cipherName, bool& wasChanged )
{
	int cipherSuite = SSLCiphersuiteNameToNumber( cipherName );
	if (isNameFound(cipherSuite, cipherName)) {
  	DisableId( cipherSuite, wasChanged );
  }
  else {
		reportError(CIPHER_ERROR_FILE, 22, cipherName);
	}
}


// enable all known/supported cipher suites
SSLErr CWLSSLCiphers::EnableAll(bool& wasChanged)
{
  SSLErr              retStatus = SSLNoErr;
	SSLErr              sslStatus = SSLNoErr;
	SSLCiphersuiteInfo  csInfo;

	for (unsigned i=0; i<gSuitesSize; i++) {

    // Check if this Cipher is not yet enabled 
    // then we know that the enabled ciphers will
    // be changed
    if (!isEnabled(gSuitesUsed[i]))
			wasChanged = true;

		m_Suites[i] = gSuitesUsed[i];
		sslStatus = SSLGetCiphersuiteInfo(m_Suites[i], &csInfo);

    if (sslStatus != SSLNoErr)
      retStatus = sslStatus;

    WlTraceMsg("* Added cipher %s [KeySize:%4d] (SSL Id:0x%0.4hX)", csInfo.suiteName, csInfo.keyBits, m_Suites[i]);
	}
  return retStatus;
}

// disable all known/supported cipher suites
SSLErr CWLSSLCiphers::DisableAll(bool& wasChanged)
{
  SSLErr              retStatus = SSLNoErr;
	SSLErr              sslStatus = SSLNoErr;
	SSLCiphersuiteInfo  csInfo;

	if (m_Count > 0)
		wasChanged = true;

	for (unsigned i=0; i<m_Count; i++) {

    // First we take the info for the Cipher which is going to be disabled
    sslStatus = SSLGetCiphersuiteInfo(m_Suites[i], &csInfo);

    if (sslStatus != SSLNoErr)
      retStatus = sslStatus;

    WlTraceMsg("* Removed cipher %s [KeySize:%4d] (SSL Id:0x%0.4hX)", csInfo.suiteName, csInfo.keyBits, m_Suites[i]);

    // And now we disable it ...
    m_Suites[i] = 0;
	}
	// reset enabled suites counter
	m_Count = 0;

  return retStatus;
}


// enable the cipher with the given suite identifier
void CWLSSLCiphers::EnableId( int cipherSuite, bool& wasChanged )
{
	if (!isKnown(cipherSuite)) {
		reportError(CIPHER_ERROR_FILE, 21, getCipherName(cipherSuite));
		return;
	}

	// first make certain it's not already enabled
	if ( isEnabled( cipherSuite ) ) {
		reportError(CIPHER_ERROR_FILE, 34, getCipherName(cipherSuite));
		return;
	}

	// add it at the last cell - This is not good we should add it at 
    // the right place according to the strength
	m_Suites[m_Count] = cipherSuite;
	m_Count++;


	wasChanged = true;
	SSLCiphersuiteInfo  csInfo;
	SSLGetCiphersuiteInfo(cipherSuite, &csInfo);
	// not doing anything about a potential error here - debug only
  WlTraceMsg("* Enabled cipher %s (SSL Id:0x%0.4hX)",csInfo.suiteName, cipherSuite);
}

// disable the cipher with the given suite identifier
void CWLSSLCiphers::DisableId( int cipherSuite, bool& wasChanged )
{
	if (!isKnown(cipherSuite)) {
		reportError(CIPHER_ERROR_FILE, 21, getCipherName(cipherSuite));
		return;
	}

	if ( m_Count <= 0 ) {
		reportError(CIPHER_ERROR_FILE, 33);
		return;
	}

	// first find the occupied cell, if any
  // m_Count might change inside the loop
	wasChanged = false;
	unsigned size = m_Count;
	for (unsigned i=0; i < size; i++ ) {
		if ( m_Suites[i] == cipherSuite ) {
			SSLCiphersuiteInfo  csInfo;
			SSLGetCiphersuiteInfo(cipherSuite, &csInfo);
      WlTraceMsg("* Disabled cipher %s (SSL Id:0x%0.4hX)",csInfo.suiteName, cipherSuite);
			// move all ciphers down one cell 
			for (unsigned j = i; j < m_Count-1; j++ ) {
				m_Suites[j] = m_Suites[j+1];
			}

			m_Suites[m_Count] = 0;	// clear the last cell
			m_Count--;
			wasChanged = true;
			break;
		}
	}

	if (!wasChanged) {
		reportError(CIPHER_ERROR_FILE, 35, getCipherName(cipherSuite));
	}
}


// tell whether or not the given cipher is enabled
bool CWLSSLCiphers::isEnabled( int cipherSuite )
{
	for (unsigned i=0; i< m_Count; i++ ) {
		if ( m_Suites[i] == cipherSuite ) {
			return true;
		}
	}

	return false;
}


// set all available suites to the given context
SSLErr CWLSSLCiphers::setSuites(SSLContext* ctx)
{
	SSLErr     err;
	SSLBuffer  ciphersBuf;

	ciphersBuf.data = (unsigned char*)m_Suites;
	ciphersBuf.length = m_Count * sizeof(m_Suites[0]);
	SSLCiphersuiteInfo  csInfo;
	for (unsigned i = 0; i < m_Count; i++) {
		SSLGetCiphersuiteInfo(m_Suites[i], &csInfo);
    WlTraceMsg("* Setting cipher %s (SSL Id:0x%0.4hX)",csInfo.suiteName, m_Suites[i]);
	}
	err = SSLSetCipherSuites(ctx, &ciphersBuf);
	return err;
}


// main function implementing the client's "cipher" command
SSLErr CWLSSLCiphers::DoCommand( int command, const char *cipherName)
{
	SSLErr     retStatus = SSLNoErr;
	bool       wasChanged = false;
	/*
	possible commands:
	all:                 enables all ciphers
	none:                disables all ciphers
	enable <cipher>:     enables the specified cipher
	disable <cipher>:    disables the specified cipher
	showall:             displays all known ciphers
	showallenabled:      displays all enabled ciphers
	*/

	if ( command == CIPHER_ENABLE_ALL ) {
		retStatus = EnableAll(wasChanged);
	}
	else if ( command == CIPHER_DISABLE_ALL ) {
		retStatus = DisableAll(wasChanged);
	}
	else if ( command == CIPHER_ENABLE ) {
		EnableNamed(cipherName, wasChanged);
	}
	else if ( command == CIPHER_DISABLE ) {
		DisableNamed(cipherName, wasChanged);
	}
	else if ( command == CIPHER_SHOW_ALL ) {
		ShowAll();
	}
	else if ( command == CIPHER_SHOW_ENABLED ) {
		ShowEnabled();
	}
  return (retStatus);
}


// show all enabled ciphersuites
void CWLSSLCiphers::ShowEnabled()
{
	if ( m_Count > 0 ) {
		SSLCiphersuiteInfo  csInfo;
		SSLErr              sslStatus;
		short               whichCipher;

    WlTraceMsg("* Show all enabled ciphers (%d):", m_Count);
		for (unsigned i=0; i< m_Count; ++i ) {
			whichCipher = m_Suites[i];
			sslStatus = SSLGetCiphersuiteInfo(whichCipher, &csInfo);
			/* not doing anything about potential error here */
      WlTraceMsg("Name: %s, SSL Id:0x%0.4hX : ", csInfo.suiteName, whichCipher);
		}
	}
	else {
    WlTraceMsg("* There are not any enabled ciphers (%d):", m_Count);
	}
}


/* show all known ciphersuites */
void CWLSSLCiphers::ShowAll()
{
	unsigned            ciphersuiteCount = SSLGetCiphersuiteCount();
  if (ciphersuiteCount > 0) {
	  SSLCiphersuiteInfo  csInfo;
	  SSLErr              err;

    WlTraceMsg("* Show all known ciphers (%d):", ciphersuiteCount);
	  for (unsigned i=0; i<ciphersuiteCount; ++i) {
		  err = SSLGetIndexedCiphersuiteInfo(i,&csInfo);
		  /* not doing anything about potential error here */
      WlTraceMsg("Name: %s, SSL Id:0x%0.4hX : ", csInfo.suiteName, csInfo.id);
	  }
  }
  else {
    WlTraceMsg("* There are not any known ciphers (%d):", ciphersuiteCount);
  }
}

SSLErr CWLSSLCiphers::SetStrength(unsigned strength)
{
	resetAll();
	return setStrength(strength, true);
}

#endif __SSL__
