/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include "BPLoadUI.h"

#include <stdlib.h>
#include <fstream.h>
#include <memory.h>
#include "tvenv.h"

#ifdef __TV_WINDOWS__
  #define SafeStringVal(x) x
#else
  #define SafeStringVal(x) (x) ? (x) : "(null)"
#endif // __TV_WINDOWS__

#ifdef __TV_WINDOWS__
#define IS_OPEN(f) (f).is_open()
#else
#define IS_OPEN(f) (f).rdbuf() && (f).rdbuf()->is_open()
#endif


CScriptFile::CScriptFile(
	const char* Name ):
	m_Name(Name) {}

CScriptFile::~CScriptFile() {}

const char* CScriptFile::fGetName() const {
	return m_Name.data();
	}

void CScriptFile::fSourceExport( 
	iEmbedderCallContext& EC,
	iSourceCommit& Sourcer ) {

	RVCString Source;


	ifstream* pFileInput;
	ifstream X0,X1;

#if defined(__PPC750__) || defined(__PPC405__) || defined(__TV_linux__)
   X0.open( m_Name.data(),ios::in);
#else
  X0.open( m_Name.data(), ios::nocreate|ios::in );
#endif

  if( IS_OPEN(X0)) {
    pFileInput = &X0;
  } else {
    TVEnvContainerClass tvpc;
    const char* tv_Include = tvpc.TVEGetEnv("WL_DEFAULT_INCLUDE");
    RVCString fileName;
    if ( tv_Include ) {
      fileName = tv_Include;
      fileName += FILE_SEP_STR; // defined diffrently in Unix and Windows
    }
    fileName += m_Name;
#if defined(__PPC750__) || defined(__PPC405__) || defined(__TV_linux__)
    X1.open( fileName, ios::in  );
#else
    X1.open( fileName, ios::nocreate|ios::in  );
#endif
    pFileInput = &X1;
  }
	ifstream& FileInput = *pFileInput;

  
  if( !IS_OPEN(*pFileInput) ) {
  	RVCString Message("");
		Message += "can't open ";
		Message += "iFile: ";
		Message +=  m_Name.data();
		Message += "\n";
		cLoadReportInfoAgenda loadRepInf( WLLogMessage::Fatal, Message.data() );
		EC.fReport( loadRepInf );
  }
  
	char ch;
  for ( ch =pFileInput->get();
	pFileInput->good() ; 
	ch =pFileInput->get() ) {
		Source += ch;
	}

	if( Source.length() > 0 ) {
		Sourcer.fTake( Source.length(), Source.data() );
	}	else {
  	RVCString Message("");
		Message += "bad ";
		Message += "iFile: ";
		Message +=  m_Name.data();
		Message += "\n";
		cLoadReportInfoAgenda loadRepInf( WLLogMessage::Fatal, Message.data() );
		EC.fReport( loadRepInf );
	}

}	



RVCString IGeneralProbingClientUITest::fsReportFormat(
	WLLogMessage::Severity Severity, 
	const char* Message ) {

	RVCString Rtn;

	switch( Severity ) {
	case WLLogMessage::Note:
		Rtn += "N";
		break;
	case WLLogMessage::Warning:
		Rtn += "W";
		break;
	case WLLogMessage::Error:
		Rtn += "E";
		break;
	case WLLogMessage::Fatal:
		Rtn += "F";
		break;
	default:
		WlAssert(0); // bad switch
		break;
		}
	Rtn += " ";
	Rtn += Message;

	return Rtn;
	}




ILoadProbingClientUITest::ILoadProbingClientUITest(
  cSingleClientProcess& SCP,
  int nRoundsToDo/*0*/)
:
  iHttpProbingClient(SCP),
	m_RoundsToDo(nRoundsToDo) 
{
}

ILoadProbingClientUITest::~ILoadProbingClientUITest() 
{
  while(!m_FileList.isEmpty())
      delete m_FileList.get();     // Remove & delete list items  
}

iAgendaFile* ILoadProbingClientUITest::fGetFile( 
	bool& IsNew, 
	iEmbedderCallContext&ecx, 
	const char* Name) {
	
	CScriptFile* File = new CScriptFile( Name );
	m_FileList.insert( File );
	IsNew = true;
	return File;
	}




void ILoadProbingClientUITest::fCopyFile( 
	iEmbedderCallContext& EC, 
	const char* Source, 
	const char* Target ) {

	
	if( Source ) {
    printf( "CopyFile( %s, %s )\n", 
	    SafeStringVal(Source),
	    SafeStringVal(Target) );
	}	else {
		cLoadReportInfoAgenda loadRepInf( WLLogMessage::Warning, "no source file" );
		EC.fReport( loadRepInf );
	}		

}


void ILoadProbingClientUITest::fScriptHook( 
	iScriptHookable& Script,
	bool Creating ) 
{}


void ILoadProbingClientUITest::fExecReporter(
	iReportThreadState* rThreadState, 
	iReportInfo& rInfo )
{
	if( iLoadReportInfo* aInfo = dynamic_cast<iLoadReportInfo*>(&rInfo) ) {
		RVCString X0 = fsReportFormat( aInfo->fGetSeverity(), aInfo->fGetMessage() );
	printf( "Non JS report: %s\n", X0.data() );
	}
}


	
tDebugReporterReturn ILoadProbingClientUITest::fDebugReporter(
	iReportThreadState& ThreadState,
	iReportInfo& ReportInfo ) 
{

	RVCString Message;

	if( iLoadReportInfo* NativeInfo = dynamic_cast<iLoadReportInfo*>(&ReportInfo) ) {
		Message += " " + fsReportFormat( NativeInfo->fGetSeverity(), NativeInfo->fGetMessage() );
		}
	else if( cInternalReportInfo* InternalInfo = dynamic_cast<cInternalReportInfo*>(&ReportInfo) ) {
		Message += InternalInfo->fIsWarning() ? 'W' : 'E';
		Message += InternalInfo->fIsException() ? 'X' : '-';
		Message += InternalInfo->fIsStrict() ? 'S' : '-';
		Message += " ";
		Message += InternalInfo->fGetMessage();
		}
	else {
		WlAssert(0); // unknown report info type;
		}
		
		
	{
		iFile* File = ThreadState.fGetFile();
    const char *fName = NULL;
    if ( File  ) {
      fName = static_cast<CScriptFile*>(File)->fGetName();
    }
    else {
      fName = ThreadState.fGetName();
    }
    char sMessage[10000];
    sprintf(sMessage,"%s, %s %d:%d %s",
			Message.data(),
	    SafeStringVal(fName),
			ThreadState.fGetLineNumber()+1,
			ThreadState.fGetColumn()+1,
	    SafeStringVal(ThreadState.fGetLine()));

    printf( "JSD rep: %s\n",sMessage);
    RVCString rwsMessage = sMessage;
    if(rwsMessage(0,9) != " N @Dgnx:")
      WlTraceMsg(rwsMessage.data());
	}

	tDebugReporterReturn Rtn = sDebugReporterReturnApply;
	bool Continue = (false);
	while(Continue) {
		Continue = false;

		char X0[100];
		scanf( "%s", X0 );
		RVCString Str(X0);

		if( Str == "deb" ) {
			Rtn = sDebugReporterReturnDM;
			}
		else if( Str == "clr" ) {
			Rtn = sDebugReporterReturnClear;
			}
		else if( Str == "apl" ) {
			Rtn = sDebugReporterReturnApply;
			}
		else if( Str == "con" ) {
			Rtn = sDebugReporterReturnContinue;
			}
		else{
			Continue = true;
			printf( "rep>\n" );
			}
		} 


	return(Rtn);
}




void ILoadProbingClientUITest::fReportDMHook(
  tInterruptionControl& InterruptionControl, bool& oIsHookPosition, tStepping Stepping, iDMThreadState& ThreadState) 
{}


tNormalDMHookReturn ILoadProbingClientUITest::vInterruptDMHook_iLoadProbingClient(
  tInterruptionControl& InterruptionControl, bool& oIsHookPosition, tStepping Stepping, iDMThreadState& ThreadState) 
{
		return( sNormalDMHookReturnContinue );
}

tNormalDMHookReturn ILoadProbingClientUITest::fBreakPointDMHook(
  tInterruptionControl& InterruptionControl, bool& oIsHookPosition, tStepping Stepping, iDMThreadState& ThreadState) 
{
		return( sNormalDMHookReturnContinue );
}

tNormalDMHookReturn ILoadProbingClientUITest::fKeywordDMHook(
  tInterruptionControl& InterruptionControl, bool& oIsHookPosition, tStepping Stepping, iDMThreadState& ThreadState) 
{
		return( sNormalDMHookReturnContinue );
}

tThrowDMHookReturn ILoadProbingClientUITest::fThrowDMHook(
  tInterruptionControl& InterruptionControl, bool& oIsHookPosition, tStepping Stepping, iDMThreadState& ThreadState) 
{
		return( sThrowDMHookReturnApply );
	}

tNormalDMHookReturn ILoadProbingClientUITest::fDMHookX2( 
	tInterruptionControl& InterruptionControl,
	const char* DMHookMsg,
	iDMThreadState& ThreadState ) 
{
		return( sNormalDMHookReturnContinue );
		}


bool ILoadProbingClientUITest::fDMHookX1( 
	tInterruptionControl& InterruptionControl,
	const char* DMHookMsg,
	iDMThreadState& ThreadState ) 
{
		return( sNormalDMHookReturnContinue );
}

bool ILoadProbingClientUITest::fDMHookX0(
	const char* DMHookMsg,
	iDMThreadState& ThreadState,
	tInterruptionControl& InterruptionControl ) 
{
		return( sNormalDMHookReturnContinue );
}


void ILoadProbingClientUITest::fBeforeInitAgenda( iInterruptionControlCommit& ) {}
void ILoadProbingClientUITest::fBeforeTerminateAgenda( iInterruptionControlCommit& ) {}
void ILoadProbingClientUITest::fBeforeOnErrorTerminateAgenda( iInterruptionControlCommit& ) {}
void ILoadProbingClientUITest::fBeginClient( iInterruptionControlCommit& ) {}
void ILoadProbingClientUITest::fBeforeInitClient( iInterruptionControlCommit& ) {}
void ILoadProbingClientUITest::fBeforeTerminateClient( iInterruptionControlCommit& ) {}
void ILoadProbingClientUITest::fBeforeOnErrorTerminateClient( iInterruptionControlCommit& ) {}
void ILoadProbingClientUITest::fEndClient( iInterruptionControlCommit& ) {}


bool ILoadProbingClientUITest::fBeforeRound(
	unsigned long RoundNum,
	iInterruptionControlCommit& ) {
/*
	bool Rtn = true;

	if( 0 == m_RoundsToDo ) {

		printf( "How many rounds: " );
		int IntX0;
		scanf( "%d", &IntX0 );
	
		if( 0 < IntX0 ) {
			m_RoundsToDo = IntX0;
    } else {
      Rtn = false;
    }

	} else {
		--m_RoundsToDo;
	}
	
	return( Rtn );
*/
  return (m_RoundsToDo-- > 0);
}

void ILoadProbingClientUITest::fBeforeOnScriptAbort( iInterruptionControlCommit& ) {}






// ~~ tSCActionReporterReturn fStubEmbedderReporter(iFile&,tLineNumber,tColumn,const char*,iSCActionReportInfo&);



