/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "testcallback.h"

#include <fstream.h>


cHttpProbingClientUI::~cHttpProbingClientUI()
{
  while(!m_FileList.isEmpty())
      delete m_FileList.get();     // Remove & delete list items  
}


RVCString fsReportFormat(
	WLLogMessage::Severity Severity, 
	const char* Message ) {

	RVCString Rtn;

	switch( Severity ) {
	case WLLogMessage::Note:
		Rtn += "N";
		break;
	case WLLogMessage::Warning:
		Rtn += "W";
		break;
	case WLLogMessage::Error:
		Rtn += "E";
		break;
	case WLLogMessage::Fatal:
		Rtn += "F";
		break;
	default:
		assert(0); // bad switch
		break;
		}
	Rtn += " ";
	Rtn += Message;

	return Rtn;
	}


 

void cHttpProbingClientUI::fReporter(
	WLLogMessage::Severity Severity, 
	const char* Message ) {
	RVCString X0 = fsReportFormat( Severity, Message );
	printf( "Non JS report: %s\n", X0.data() );
	}



	
tReporterReturn cHttpProbingClientUI::fReporter(
	iFile& File,
	tLineNumber LineNumber,
	tColumn Column,
	const char* Line,
	iReportInfo& ReportInfo ) {

	RVCString Message;

	cLoadReportInfo* NativeInfo;
	cInternalReportInfo* InternalInfo;
	if( NativeInfo = dynamic_cast<cLoadReportInfo*>(&ReportInfo) ) {
		Message += " " + fsReportFormat( NativeInfo->fGetSeverity(), NativeInfo->fGetMessage() );
		}
	else if( InternalInfo = dynamic_cast<cInternalReportInfo*>(&ReportInfo) ) {
		Message += InternalInfo->fIsWarning() ? 'W' : 'E';
		Message += InternalInfo->fIsException() ? 'X' : '-';
		Message += InternalInfo->fIsStrict() ? 'S' : '-';
		Message += " ";
		Message += InternalInfo->fGetMessage();
		}
	else {
		assert(0); // unknown report info type;
		}
		
		
	printf( "JSD rep: %s, %s %d:%d %s\n",
		Message.data(),
		static_cast<cScriptFile&>(File).fGetName(),
		LineNumber+1,
		Column+1,
		Line  );

	tReporterReturn Rtn = sReporterReturnApply;
  return Rtn;
}


tEmbedderReporterReturn cHttpProbingClientUI::pageStartDownload(
                               iFile& file,
                               tLineNumber lineN,
                               tColumn col,
                               const char* line,
                               const char* url,  
                               const char* original_url,
                               const char* referer,
                               const char* target)
{
  printf("*Pg Start Dwnld* the url is : %s, The Target is : %s\n",url, target);
  printf("Original Url: %s Referer: %s\n", original_url, referer);
  return sEmbedderReporterReturnContinue;
}

tEmbedderReporterReturn cHttpProbingClientUI::pageAbortDownload(
                               iFile& file,
                               tLineNumber lineN,
                               tColumn col,
                               const char* line,
                               const char *url,
                               const char* original_url,
                               const char* referer,
                               const char *target)
{
  printf("*Pg Abort Dwnld* the url is : %s, The Target is : %s\n",url, target);
  printf("Original Url: %s Referer: %s\n", original_url, referer);
  return sEmbedderReporterReturnContinue;
}

tEmbedderReporterReturn cHttpProbingClientUI::pageEndDownload(
                               iFile& file,
                               tLineNumber lineN,
                               tColumn col,
                               const char* line,
                               const char* url,
                               const char* original_url,
                               const char* referer,
                               const char *target)
{
  printf("*Pg End Dwnld* the url is : %s, The Target is : %s\n",url, target);
  printf("Original Url: %s Referer: %s\n", original_url, referer);
  return sEmbedderReporterReturnContinue;
}

tEmbedderReporterReturn cHttpProbingClientUI::objectStartDownload(
                               iFile& file,
                               tLineNumber lineN,
                               tColumn col,
                               const char* line,
                               const char *url,
                               const char* original_url,
                               const char* referer)
{
  printf("*Obj Start Dwnld* the url is : %s\n",url);
  printf("Original Url: %s Referer: %s\n", original_url, referer);
  return sEmbedderReporterReturnContinue;
}

tEmbedderReporterReturn cHttpProbingClientUI::objectAbortDownload(
                               iFile& file,
                               tLineNumber lineN,
                               tColumn col,
                               const char* line,
                               const char *url,
                               const char* original_url,
                               const char* referer,
                               int httpStatus)
{
  printf("*Obj Abort Dwnld* the url is : %s, The status is : %d\n",url, httpStatus);
  printf("Original Url: %s Referer: %s\n", original_url, referer);
  return sEmbedderReporterReturnContinue;
}

tEmbedderReporterReturn cHttpProbingClientUI::objectEndDownload(
                               iFile& file,
                               tLineNumber lineN,
                               tColumn col,
                               const char* line,
                               const char *url,
                               const char* original_url,
                               const char* referer,
                               HtmlQueueItemType urlType,
                               void *objContent, 
                               int objContentLength)
{
  printf("*Obj End Dwnld* the url is : %s, The content length is : %d\n",url, objContentLength);
  printf("Original Url: %s Referer: %s Type : %d\n", original_url, referer, urlType);
    
  FILE *outputFile = fopen("d:\\output.txt", "a+");
  fwrite((char*)objContent, objContentLength,1,outputFile);
	fclose(outputFile); 
  
  return sEmbedderReporterReturnContinue;
}

tEmbedderReporterReturn cHttpProbingClientUI::UseCachedObject(
                               iFile& file,
                               tLineNumber lineN,
                               tColumn col,
                               const char* line,
                               const char *url,
                               const char* original_url,
                               const char* referer,
                               HtmlQueueItemType urlType)
{
  printf("*Obj Load from cache* the url is : %s\n",url);
  printf("Original Url: %s Referer: %s, Type : %d\n", original_url, referer, urlType);
  return sEmbedderReporterReturnContinue;
}

 

iFile* cHttpProbingClientUI::fGetFile( bool& IsNew, iEmbedderContext& ecx, const char* Name ) 
{
  IsNew = true; // LoadEngine is currently checking for unique include file
  cScriptFile* pFile = new cScriptFile(Name);
	m_FileList.append(pFile);
	return pFile;
}

void cHttpProbingClientUI::fBeforeInitAgenda( iInterruptionControlCommit& ) {}
void cHttpProbingClientUI::fBeforeTerminateAgenda( iInterruptionControlCommit& ) {}
void cHttpProbingClientUI::fBeforeOnErrorTerminateAgenda( iInterruptionControlCommit& ) {}
void cHttpProbingClientUI::fBeginClient( iInterruptionControlCommit& ) {}
void cHttpProbingClientUI::fBeforeInitClient( iInterruptionControlCommit& ) {}
void cHttpProbingClientUI::fBeforeTerminateClient( iInterruptionControlCommit& ) {}
void cHttpProbingClientUI::fBeforeOnErrorTerminateClient( iInterruptionControlCommit& ) {}
void cHttpProbingClientUI::fEndClient( iInterruptionControlCommit& ) {}


bool cHttpProbingClientUI::fBeforeRound(
	unsigned long RoundNum,
	iInterruptionControlCommit& ) 
{
	return (RoundNum<=1);
}

void cHttpProbingClientUI::fBeforeOnScriptAbort( iInterruptionControlCommit& ) {}
	


/////////  cScriptFile implementation

cScriptFile::cScriptFile(const char* Name)
{
  if (Name)
    m_Name = Name ;
}

cScriptFile::~cScriptFile() {}

const char* cScriptFile::fGetName() const 
{
	return m_Name.data();
}

void cScriptFile::fSourceExport(iEmbedderContext&ecx, iSourceCommit& Sourcer ) 
{
	RVCString sSource;
  ifstream FileInput;
  FileInput.open(m_Name.data(),ios::in | ios::nocreate);

  //sSource.readFile(FileInput);
  readFile(sSource,FileInput);

  if (!sSource.isNull())
    Sourcer.fTake( sSource.length(), sSource.data() );
}

