/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <ace/OS.h>
#include "wlxmlcom.h"
#include "wlxmlfuncs.h"
#include <string.h>
#include <RVCString.h>
#include "rvjsapi.h"

#include "xmlreplyprops.h"
#include "jsxmlreplynames.h"
#include "jsxmlprivate.h"
#include "jsxmlwrappers.h"
#ifdef _DEBUG
#include <iostream.h>
#endif

#include "xmlerrorreporter.h"

JSBool
xmlcharacterdata_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp, IXMLDOMNodePtr& pNodePtr)
{
	TRY_COM {
		WlTraceMsg("Inside xmlcharacterdata_getProperty");	
		IXMLDOMCharacterDataPtr charDataPtr = pNodePtr;
		if ((pNodePtr != NULL) && JSVAL_IS_STRING(id)) {
			JSString* str = JSVAL_TO_STRING(id);
			char* prop = JS_GetStringBytes(str);
			if (!strcmp(prop, WLJSNAME_PROP_DATA)) {	
				_bstr_t data;
				data = charDataPtr->Getdata();
				WLXmlSetStringValue( cx,  obj, NULL, (char*)data, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_LENGTH)) {	
				long length;
				length = charDataPtr->Getlength();
				return WLXmlSetIntValue(cx, obj, NULL, length, vp);
			}		
		}
		return JS_TRUE;
	}
	CATCH_COM(cx)
}	

JSBool
xmlcharacterdata_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp, IXMLDOMNodePtr& pNodePtr)
{
	TRY_COM {
		WlTraceMsg("Inside xmlcharacterdata_setProperty");	
		IXMLDOMCharacterDataPtr charDataPtr = pNodePtr;
		if ((pNodePtr != NULL) && JSVAL_IS_STRING(id)) {
			JSString* str = JSVAL_TO_STRING(id);
			char* prop = JS_GetStringBytes(str);
			if (!strcmp(prop, WLJSNAME_PROP_DATA) && JSVAL_IS_STRING(*vp)) {	
				_bstr_t data = JS_GetStringBytes(JSVAL_TO_STRING(*vp));
				charDataPtr->Putdata(data);				
				return WLXmlSetStringValue( cx,  obj, NULL, (char*)data, vp);
			}
		}
		return JS_TRUE;
	}
	CATCH_COM(cx)
}


JSBool
xmlcharacterdata_appendData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlcharacterdata_appendData");	
    if(argc != 1) {
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12,"appendData", 1);
    }
    if(!JSVAL_IS_STRING(argv[0])){
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "appendData");
    }
		IXMLDOMCharacterDataPtr charDataPtr = pNodePtr;
		_bstr_t data = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
		HRESULT hr = charDataPtr->appendData(data);
		return succeeded(cx, hr);
	}
	CATCH_COM_METHOD(cx, "appendData")
}

JSBool
xmlcharacterdata_deleteData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlcharacterdata_deleteData");
    if(argc != 2) {//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "deleteData", 2);
    }
    if(!JSVAL_IS_INT(argv[0]) || !JSVAL_IS_INT(argv[1])){
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "deleteData");
    }
		IXMLDOMCharacterDataPtr charDataPtr = pNodePtr;	
		jsint offset = JSVAL_TO_INT(argv[0]);
		jsint count = JSVAL_TO_INT(argv[1]);
		HRESULT hr = charDataPtr->deleteData(offset, count);
		return succeeded(cx, hr);
	}
	CATCH_COM_METHOD(cx, "deleteData")
}

JSBool
xmlcharacterdata_insertData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlcharacterdata_insertData");	
    if(argc != 2){//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "insertData", 2);
    }
    if(!JSVAL_IS_INT(argv[0]) || !JSVAL_IS_STRING(argv[1])){
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "insertData");
    }
		IXMLDOMCharacterDataPtr charDataPtr = pNodePtr;	
		jsint		offset = JSVAL_TO_INT(argv[0]);
		_bstr_t data = JS_GetStringBytes(JSVAL_TO_STRING(argv[1]));
		HRESULT hr = charDataPtr->insertData(offset, data);
		return succeeded(cx, hr);
	}
	CATCH_COM_METHOD(cx, "insertData")
}

JSBool
xmlcharacterdata_replaceData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmlcharacterdata_replaceData");	
    if(argc != 3){//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "replaceData", 3);
    }
    if(!JSVAL_IS_INT(argv[0]) || !JSVAL_IS_INT(argv[1]) ||!JSVAL_IS_STRING(argv[2])){
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "replaceData");
    }
		IXMLDOMCharacterDataPtr charDataPtr = pNodePtr;	
		jsint		offset = JSVAL_TO_INT(argv[0]);
		jsint		count = JSVAL_TO_INT(argv[1]);
		_bstr_t data = JS_GetStringBytes(JSVAL_TO_STRING(argv[2]));
		HRESULT hr = charDataPtr->replaceData(offset, count, data);
		return succeeded(cx, hr);
	}
	CATCH_COM_METHOD(cx, "replaceData")
}

JSBool
xmlcharacterdata_substringData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval, IXMLDOMNodePtr& pNodePtr)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		WlTraceMsg("Inside xmltext_setProperty");	
    if(argc != 2){//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "substringData", 2);
    }
    if(!JSVAL_IS_INT(argv[0]) || !JSVAL_IS_INT(argv[1])){
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "substringData");
    }
		IXMLDOMCharacterDataPtr charDataPtr = pNodePtr;	
		jsint offset = JSVAL_TO_INT(argv[0]);
		jsint count = JSVAL_TO_INT(argv[1]);
		_bstr_t data = charDataPtr->substringData(offset, count);
		return WLXmlSetStringValue(cx, obj, NULL, (char*)data, rval);
	}
	CATCH_COM_METHOD(cx, "substringData")
}
