/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <ace/OS.h>
#include "wlxmlcom.h"
#include "wlxmlfuncs.h"
#include "wlxml.h"
#include <RVCString.h>
#include "rvjsapi.h"

#include "wljsobjects.h"
#include "xmlreplyprops.h"
#include "xmlerrorreporter.h"
#include "jsxmlreplynames.h"
#include "jsxmlprivate.h"
#include "jsxmlwrappers.h"
#include "tracedll.h"

#include <comdef.h>
#include <atlbase.h>
//#include <atlcom.h>

//#include <atlconv.h>
#include <stdio.h>
#include <string.h>
#ifdef _DEBUG
#include <iostream.h>
#define __WL_TRACE__
#include "tracedll.h"
#endif


CWLXmlDocumentWrapper::CWLXmlDocumentWrapper(IXMLDOMDocumentPtr* pXMLDOMDocumentPtr) {
	m_pXMLDOMDocumentPtr = pXMLDOMDocumentPtr;
	m_type = wlInterfaceDocumentType;
}

IXMLDOMDocumentPtr* CWLXmlDocumentWrapper::GetInterfacePtr() {
	return m_pXMLDOMDocumentPtr;
}

CWLXmlDocumentWrapper::~CWLXmlDocumentWrapper() {
		if(m_pXMLDOMDocumentPtr)
		{	
//			WlTraceMsg("deleting IXMLDOMDocumentPtr*: 0x%x",m_pXMLDOMDocumentPtr );
//			m_pXMLDOMDocumentPtr->Release();
			delete m_pXMLDOMDocumentPtr;
	}
}

//change to ptr  so  u can return null
bool getDoc(JSContext* cx, JSObject* obj,IXMLDOMDocumentPtr& docPtr, bool& isNode)
{
	CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return false;
	CWLXmlInterfaceParent* parent = (CWLXmlInterfaceParent*)(prv->GetObj());
	if(!parent)
		return false;
	if(parent->GetType() == wlInterfaceNodeType)
	{
		isNode = true;
		CWLXmlNodeWrapper* wrapper	= (CWLXmlNodeWrapper*)(prv->GetObj());
		if(!wrapper)
			return false;
		docPtr				= *(wrapper->GetInterfacePtr());
	}
	else if(parent->GetType() == wlInterfaceDocumentType)
	{
		CWLXmlDocumentWrapper* wrapper	= (CWLXmlDocumentWrapper*)(prv->GetObj());
		if(!wrapper)
			return false;
		docPtr				= *(wrapper->GetInterfacePtr());
	}
	else
		return false;
	return true;
}

bool getDoc(JSContext* cx, JSObject* obj,IXMLDOMDocumentPtr& docPtr)
{
	CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return false;
	CWLXmlInterfaceParent* parent = (CWLXmlInterfaceParent*)(prv->GetObj());
	if(!parent)
		return false;
	if(parent->GetType() == wlInterfaceNodeType)
	{	
		CWLXmlNodeWrapper* wrapper	= (CWLXmlNodeWrapper*)(prv->GetObj());
		if(!wrapper)
			return false;
		docPtr				= *(wrapper->GetInterfacePtr());
	}
	else if(parent->GetType() == wlInterfaceDocumentType)
	{
		CWLXmlDocumentWrapper* wrapper	= (CWLXmlDocumentWrapper*)(prv->GetObj());
		if(!wrapper)
			return false;
		docPtr				= *(wrapper->GetInterfacePtr());
	}
	else
		return false;
	return true;
}



//in ie5 the document holds 'html dom' properties  and 'xml dom' properties
//we get the object by asking for "xmlObj = document.all.myxml(0)"
//then we can ask for xmlObj.id -->"html" prop
//or   we can ask for xmlObj.documentElement -->"xml" prop
//also xmlObj.XMLDocument return itself

JSBool
xmldocument_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	TRY_COM {
		WlTraceMsg("Inside xmldocument_getProperty");	
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(!JSVAL_IS_STRING(id))
			return JS_TRUE;
		JSString* str = JSVAL_TO_STRING(id);
		char* prop = JS_GetStringBytes(str);
		//first check for html props
		CWLXmlDocument* xmlPrv = prv->GetXmlObj();
		bool enableHtmlProps = false;
		if(xmlPrv)
			enableHtmlProps = prv->enableHtmlProps();
		if(enableHtmlProps) {
			if(!strcmp(prop, WLJSNAME_PROP_SRC)) {
					const char* src = xmlPrv->GetSrc();
					return WLXmlSetStringValue(cx, obj, prop, src, vp);
			}
			if(!strcmp(prop, WLJSNAME_PROP_ID)) {
					const char* src = xmlPrv->GetId();
					return WLXmlSetStringValue(cx, obj, prop, src, vp);
			}
			if(!strcmp(prop, WLJSNAME_PROP_INNERHTML)) {
					const char* src = xmlPrv->GetInnerHTML();
					return WLXmlSetStringValue(cx, obj, prop, src, vp);
			}
		}
		//second we look for xml props
		IXMLDOMDocumentPtr pXMLDocumentInterface;
		bool isNode = false;
		if (getDoc(cx, obj, pXMLDocumentInterface, isNode)) {			
			if(!strcmp(prop, WLJSNAME_PROP_XMLDOCUMENT)) {
				IXMLDOMDocumentPtr* doc = new IXMLDOMDocumentPtr;
				*doc = pXMLDocumentInterface;
				CWLXmlDocumentWrapper* wrapper = new CWLXmlDocumentWrapper(doc);
				return WLXmlSetDocumentInterfaceValue(cx, obj, wrapper, vp);					
			}
			else if (!strcmp(prop, WLJSNAME_PROP_ASYNC)) {	
				VARIANT_BOOL VBaSync;
				VBaSync = pXMLDocumentInterface->Getasync();
				bool aSync = false;
				if(VBaSync != 0)
					aSync = true;
				return WLXmlSetBoolValue(cx, obj, NULL, aSync, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_DOCTYPE )) {	
				IXMLDOMNodePtr*  pXmlDocumentType  = new IXMLDOMNodePtr;	
				*pXmlDocumentType = pXMLDocumentInterface->Getdoctype();		
				CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(pXmlDocumentType);
				return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, vp); 
				}
			
			else if (!strcmp(prop, WLJSNAME_PROP_DOCUMENTELEMENT)) {	
				IXMLDOMNodePtr* pXmlDocElementPtr = new IXMLDOMNodePtr;
				*pXmlDocElementPtr = pXMLDocumentInterface->GetdocumentElement();
				CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(pXmlDocElementPtr);
				return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, vp);
			}		
			else if (!strcmp(prop, WLJSNAME_PROP_IMPLEMENTATION)) {	
				IXMLDOMImplementationPtr * pXmlImplementationPtr = new IXMLDOMImplementationPtr;
				*pXmlImplementationPtr = pXMLDocumentInterface->Getimplementation();
				CWLXmlImplementationWrapper* pImplementationWrapper = new CWLXmlImplementationWrapper(pXmlImplementationPtr);			
				return WLXmlSetImplementationInterfaceValue(cx, obj, pImplementationWrapper, vp); 
			}
			//next three props have to do with events
			//they specify a handler function to be executed when the event is fired
			else if (!strcmp(prop, WLJSNAME_PROP_ONDATAAVAILABLE)) {	
				return JS_TRUE;
			}
			else if (!strcmp(prop, WLJSNAME_PROP_ONREADYSTATECHANGE)) {	
				return JS_TRUE;
			}
			else if (!strcmp(prop, WLJSNAME_PROP_ONTRANSFORMNODE)) {	
				return JS_TRUE;
			}
			//seems that Get/PutpreserveWhiteSpace is faulty
			else if (!strcmp(prop, WLJSNAME_PROP_PRESERVEWHITESPACE)) {	
				VARIANT_BOOL VBisPreserving;
				VBisPreserving = pXMLDocumentInterface->GetpreserveWhiteSpace();
				bool isPreserving = false;
				if(VBisPreserving != 0)
					isPreserving = true;
				return WLXmlSetBoolValue(cx, obj, NULL, isPreserving, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_PARSEERROR)) {	
				IXMLDOMParseErrorPtr* pXmlParseError = new IXMLDOMParseErrorPtr;
				*pXmlParseError = pXMLDocumentInterface->GetparseError();
				CWLXmlParseErrorWrapper* pParseErrorWrapper = new CWLXmlParseErrorWrapper(pXmlParseError);
				return WLXmlSetParseErrorInterfaceValue(cx, obj, pParseErrorWrapper, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_READYSTATE)) {	
				long ready;
				ready = pXMLDocumentInterface->GetreadyState();
				return WLXmlSetIntValue(cx, obj, NULL, (unsigned)ready, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_RESOLVEEXTERNALS)) {	
				bool isResolving = false;
				if(pXMLDocumentInterface->GetresolveExternals() != 0)
					isResolving = true;
				return WLXmlSetBoolValue(cx, obj, NULL, isResolving, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_URL)) {				
				_bstr_t url;
				url = pXMLDocumentInterface->Geturl();
				return WLXmlSetStringValue(cx, obj, prop, (char*)url, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_VALIDATEONPARSE)) {	
				bool isValidating = false;
				if(pXMLDocumentInterface->GetvalidateOnParse() != 0)
					isValidating = true;
				return WLXmlSetBoolValue(cx, obj, NULL, isValidating, vp);
			}
			else if(!isNode)
				return xmlnode_getProperty(cx, obj, id, vp);
		}
		return JS_TRUE;
	}
	CATCH_COM(cx)
}


JSBool
xmldocument_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp) 
{
	TRY_COM {
		WlTraceMsg("Inside xmldocument_setProperty");
		IXMLDOMDocumentPtr docPtr;
		bool isNode = false;
		if (getDoc(cx, obj, docPtr, isNode) && JSVAL_IS_STRING(id)) {
			JSString* str = JSVAL_TO_STRING(id);
			char* prop = JS_GetStringBytes(str);
			if (!strcmp(prop, WLJSNAME_PROP_ASYNC) && JSVAL_IS_BOOLEAN(*vp)) {	
				VARIANT_BOOL VBaSync = (JSVAL_TO_BOOLEAN(*vp) == JS_TRUE ? -1 : 0);
				docPtr->Putasync(VBaSync);
				bool aSync = VBaSync != 0 ? true : false;
				return WLXmlSetBoolValue(cx, obj, NULL, aSync, vp);
			}		
			else if (!strcmp(prop, WLJSNAME_PROP_DOCUMENTELEMENT) && JSVAL_IS_OBJECT(*vp)) {	
					JSObject* obj = JSVAL_TO_OBJECT(*vp);
					if(!obj)
						return JS_TRUE;
				CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
				if(!prv)
						return JS_TRUE;
					CWLXmlNodeWrapper* wrapper = (CWLXmlNodeWrapper*)(prv->GetObj());
					if(!wrapper) 
						return JS_TRUE;
					IXMLDOMElementPtr pIElement = *(wrapper->GetInterfacePtr());
					docPtr->PutRefdocumentElement(&(*pIElement));							
					return JS_TRUE;
			}

			//events
			if (!strcmp(prop, WLJSNAME_PROP_ONDATAAVAILABLE)) {	
				return JS_TRUE;
			}
			if (!strcmp(prop, WLJSNAME_PROP_ONREADYSTATECHANGE)) {	
				return JS_TRUE;
			}
			if (!strcmp(prop, WLJSNAME_PROP_ONTRANSFORMNODE)) {	
				return JS_TRUE;
			}
			if (!strcmp(prop, WLJSNAME_PROP_PRESERVEWHITESPACE) && JSVAL_IS_BOOLEAN(*vp)) {	
				VARIANT_BOOL VBisPreserving = (JSVAL_TO_BOOLEAN(*vp) == JS_TRUE) ? -1 : 0;
				docPtr->PutpreserveWhiteSpace(VBisPreserving);
				bool isPreserving = VBisPreserving != 0 ? true : false;
				return WLXmlSetBoolValue(cx, obj, NULL, isPreserving, vp);
			}
			if (!strcmp(prop, WLJSNAME_PROP_RESOLVEEXTERNALS) && JSVAL_IS_BOOLEAN(*vp)) {				
				VARIANT_BOOL VBisResolving = (JSVAL_TO_BOOLEAN(*vp) == JS_TRUE) ? -1 : 0;
				docPtr->PutresolveExternals(VBisResolving);
				bool isResolving = VBisResolving != 0 ? true : false;
				return WLXmlSetBoolValue(cx, obj, NULL, isResolving, vp);
			}
			else if (!strcmp(prop, WLJSNAME_PROP_VALIDATEONPARSE) && JSVAL_IS_BOOLEAN(*vp)) {	
				VARIANT_BOOL VBisValidating = JSVAL_TO_BOOLEAN(*vp) == JS_TRUE ? -1 : 0;
				docPtr->put_validateOnParse(VBisValidating);
				bool isValidating = VBisValidating != 0 ? true : false;
				return WLXmlSetBoolValue(cx, obj, NULL, isValidating, vp);
			}
			else if(!isNode)
				return xmlnode_setProperty(cx, obj, id, vp);
		}
		return JS_TRUE;
	}
	CATCH_COM(cx)
}

JSBool
xmldocument_abort(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
    if(argc){//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "abort", 0);
    }
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr)) {
			HRESULT hr = docPtr->abort();
			return succeeded(cx, hr);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "abort")
}

JSBool
xmldocument_createAttribute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
    if(argc != 1){
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "createAttribute", 1);
    }
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr)){ 
      if(!JSVAL_IS_STRING(argv[0])){
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "createAttribute");
      }
			_bstr_t name(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
			IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
			*nodePtr = docPtr->createAttribute(name);
			CWLXmlNodeWrapper* pNodeWrapper = new CWLXmlNodeWrapper(nodePtr);
			return WLXmlSetNodeInterfaceValue( cx, obj, pNodeWrapper, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "createAttribute")
}

JSBool
xmldocument_createComment(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
    if(argc != 1){//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "createComment", 1);
    }
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr)){
      if(!JSVAL_IS_STRING(argv[0])){
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "createComment");
      }
			_bstr_t comment(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
			IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
			*nodePtr = docPtr->createComment(comment);
			CWLXmlNodeWrapper* pNodeWrapper = new CWLXmlNodeWrapper(nodePtr);
			return WLXmlSetNodeInterfaceValue( cx, obj, pNodeWrapper, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "createComment")
}

JSBool
xmldocument_createCDATASection(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
    if(argc != 1){//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "createCDATASection", 1);
    }
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr)){
      if(!JSVAL_IS_STRING(argv[0])){
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "createCDATASection");
      }
			_bstr_t cdata(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
			IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
			*nodePtr = docPtr->createCDATASection(cdata);
			CWLXmlNodeWrapper* pNodeWrapper = new CWLXmlNodeWrapper(nodePtr);
			return WLXmlSetNodeInterfaceValue( cx, obj, pNodeWrapper, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "createCDATASection")
}

JSBool
xmldocument_createDocumentFragment(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		IXMLDOMDocumentPtr docPtr;
    if(argc){
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "createDocumentFragment", 0);
    }
		if(getDoc(cx, obj, docPtr))
		{
			IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
			*nodePtr = docPtr->createDocumentFragment();
			CWLXmlNodeWrapper* pNodeWrapper = new CWLXmlNodeWrapper(nodePtr);
			return WLXmlSetNodeInterfaceValue( cx, obj, pNodeWrapper, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "createDocumentFragment")
}

JSBool
xmldocument_createElement(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
    if(argc != 1){//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "createElement", 1);
    }
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr)){
      if(!JSVAL_IS_STRING(argv[0])){
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "createElement");
      }
			_bstr_t name(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
			IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
			*nodePtr = docPtr->createElement(name);
			CWLXmlNodeWrapper* pNodeWrapper = new CWLXmlNodeWrapper(nodePtr);
			return WLXmlSetNodeInterfaceValue( cx, obj, pNodeWrapper, rval);

		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "createElement")
}

JSBool
xmldocument_createEntityReference(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
    if(argc != 1){//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "createEntityReference", 1);
    }
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr)){
      if(!JSVAL_IS_STRING(argv[0])){
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "createEntityReference");
      }
			_bstr_t name(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
			IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
			*nodePtr = docPtr->createEntityReference(name);
			CWLXmlNodeWrapper* pNodeWrapper = new CWLXmlNodeWrapper(nodePtr);
			return WLXmlSetNodeInterfaceValue( cx, obj, pNodeWrapper, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "createEntityReference")
}
//createNode - the function takes 3 params
//for:
//NODE_CDATA_SECTION, NODE_COMMENT, NODE_DOCUMENT, NODE_DOCUMENT_FRAGMENT, NODE_TEXT
//the last parameter must be an empty string the 2nd parameter(name) is ignored
JSBool
xmldocument_createNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 3)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "createNode", 3);
		if(!(JSVAL_IS_STRING(argv[0]) || JSVAL_IS_INT(argv[0])) || !JSVAL_IS_STRING(argv[1]) || !JSVAL_IS_STRING(argv[2]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "createNode");
		if(JSVAL_IS_STRING(argv[0])){
			char* type = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
			if(checkNodeTypeByString(type, "document", "documenttype", "notation", "entity", NULL))
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 31, "createNode", type);
		}
		else if(JSVAL_IS_INT(argv[0])){
			long type = JSVAL_TO_INT(argv[0]);
			if(checkNodeTypeByInt(type, 9, 10, 12, 6, -1))
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 31, "createNode", ConvertNodeTypeToString(type));
		}
		_variant_t type;
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr)){
			 if(!WLXmlJsvalToVariant(argv[0], type))
				 return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 15);
			_bstr_t name(JS_GetStringBytes(JSVAL_TO_STRING(argv[1])));
			_bstr_t nameSpaceURI(JS_GetStringBytes(JSVAL_TO_STRING(argv[2])));
			IXMLDOMNodePtr* pNodePtr = new IXMLDOMNodePtr;
			*pNodePtr = docPtr->createNode(type, name, nameSpaceURI);
			CWLXmlNodeWrapper* pNodeWrapper = new CWLXmlNodeWrapper(pNodePtr);
			return WLXmlSetNodeInterfaceValue(cx, obj, pNodeWrapper, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "createNode")
}

JSBool
xmldocument_createProcessingInstruction(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 2)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "createProcessingInstruction", 2);
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr)){
			if(!JSVAL_IS_STRING(argv[0]) || !JSVAL_IS_STRING(argv[1]))
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "createProcessingInstruction");
			_bstr_t target(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
			_bstr_t data(JS_GetStringBytes(JSVAL_TO_STRING(argv[1])));
			IXMLDOMNodePtr* pNodePtr = new IXMLDOMNodePtr;
			*pNodePtr = docPtr->createProcessingInstruction(target, data);
			CWLXmlNodeWrapper* pNodeWrapper = new CWLXmlNodeWrapper(pNodePtr);
			return WLXmlSetNodeInterfaceValue(cx, obj, pNodeWrapper, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "createProcessingInstruction")
}

JSBool
xmldocument_createTextNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "createTextNode", 1);
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr)){
			if(!JSVAL_IS_STRING(argv[0]))
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "createTextNode");
			_bstr_t data(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
			IXMLDOMNodePtr* pNodePtr = new IXMLDOMNodePtr;
			*pNodePtr = docPtr->createTextNode(data);
			CWLXmlNodeWrapper* pNodeWrapper = new CWLXmlNodeWrapper(pNodePtr);
			return WLXmlSetNodeInterfaceValue(cx, obj, pNodeWrapper, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "createTextNode")
}

JSBool
xmldocument_getElementsByTagName(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "getElementsByTagName", 1);
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr)){
			if(!JSVAL_IS_STRING(argv[0]))
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "getElementsByTagName");
			_bstr_t tagName(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
			IXMLDOMNodeListPtr* pNodeListPtr				= new IXMLDOMNodeListPtr;
			*pNodeListPtr = docPtr->getElementsByTagName(tagName);
			CWLXmlNodeListWrapper* pNodeListWrapper = new CWLXmlNodeListWrapper(pNodeListPtr);
			return WLXmlSetNodeListInterfaceValue(cx, obj, pNodeListWrapper, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx,"getElementsByTagName")
}

//document.load("t:\\xmls\\hello.xml")


//load parameter is: 1) string(url) - in our case file name, should we GET a url ?
//									 2) II2 request object - we do not support that
JSBool
xmldocument_load(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		if(argc != 1)//error
			return JS_TRUE;
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr) && JSVAL_IS_STRING(argv[0]))
		{
			/*
			char* urlStr = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
			CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
			CWLXmlDocument* xmlDocPtr = prv->GetXmlObj();
			xmlDocPtr->loadFromScript(urlStr);		
			*/
			//VARIANT_BOOL VBaSync = (JSVAL_TO_BOOLEAN(*vp) == JS_TRUE ? -1 : 0);
			VARIANT_BOOL VBaSync = 0;
			docPtr->Putasync(VBaSync);
			_bstr_t			BSTRsource(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
			_variant_t  source(BSTRsource);
			bool loadSuccess = false;			
			if((docPtr->load(source))== -1)
				loadSuccess = true;
			return WLXmlSetBoolValue(cx, obj, NULL, loadSuccess, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "load")
}

JSBool
xmldocument_loadXML(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "load", 1);
		IXMLDOMDocumentPtr docPtr;	
		if(getDoc(cx, obj, docPtr)){
			if(!JSVAL_IS_STRING(argv[0]))
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "load");;
			bool loadSuccess = false;
			_bstr_t source(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
			if((docPtr->loadXML(source)))
				loadSuccess = true;
			return WLXmlSetBoolValue(cx, obj, NULL, loadSuccess, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "loadXML")
}


//doesnt work here as well as in the browser
JSBool
xmldocument_nodeFromID(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "nodeFromID", 1);
		IXMLDOMDocumentPtr docPtr;
		if(getDoc(cx, obj, docPtr))
		{
			if(!JSVAL_IS_STRING(argv[0]))
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "nodeFromID");
			_bstr_t id(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
			IXMLDOMNodePtr* pNode = new IXMLDOMNodePtr;			
			if(*pNode = docPtr->nodeFromID(id)){
				CWLXmlNodeWrapper* pNodeWrapper = new CWLXmlNodeWrapper(pNode);
				return WLXmlSetNodeInterfaceValue(cx, obj, pNodeWrapper, rval);
			}
			else
				delete pNode;
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "nodeFromID")
}


//we allow only saving into a document produced by the user for now we do not save into
//a file.
JSBool
xmldocument_save(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		IXMLDOMDocumentPtr docPtr;
		if(argc != 1)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "save", 1);
		if(getDoc(cx, obj, docPtr)){			
			if(JSVAL_IS_OBJECT(argv[0])){
//				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "save");
				JSObject* saveToObj = JSVAL_TO_OBJECT(argv[0]);
				bool isNode = false;
				IXMLDOMDocumentPtr newDocPtr;
				if(getDoc(cx, saveToObj, newDocPtr, isNode)){
					if(!isNode)
					{
						_variant_t destination(&(*newDocPtr));
						HRESULT hr = docPtr->save(destination);
						if(SUCCEEDED(hr))
							return JS_TRUE;		
						else
							return JS_FALSE;
					}
				}
				else 
					return JS_TRUE;						
			}
			else if(JSVAL_IS_STRING(argv[0])) {//file name only
				char* file = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
				FILE* fp = fopen(file, "w");
				IXMLDOMNodePtr docNodePtr = docPtr;
				//char* xmlStr = (char*)
				_bstr_t w_xmlStr = 	(docNodePtr->Getxml());
				char* xmlStr = (char*)w_xmlStr;
				fwrite(xmlStr, sizeof(char), strlen(xmlStr), fp);
				fclose(fp);
			}			
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "save")
}

//node methods
JSBool
xmldocument_node_appendChild(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	return xmlnode_appendChild(cx, obj, argc, argv, rval);
}

JSBool
xmldocument_node_cloneNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	return xmlnode_cloneNode(cx, obj, argc, argv, rval);
}


JSBool
xmldocument_node_hasChildNodes(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)											
{
	return xmlnode_hasChildNodes(cx, obj, argc, argv, rval);
}


JSBool	
xmldocument_node_insertBefore(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	return xmlnode_insertBefore(cx, obj, argc, argv, rval);
}


JSBool
xmldocument_node_removeChild(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)											
{
	return xmlnode_removeChild(cx, obj, argc, argv, rval);
}


JSBool
xmldocument_node_replaceChild(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)													
{
	return xmlnode_replaceChild(cx, obj, argc, argv, rval);
}


JSBool
xmldocument_node_selectNodes(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)													
{
	return xmlnode_selectNodes(cx, obj, argc, argv, rval);
}


JSBool
xmldocument_node_selectSingleNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	return xmlnode_selectSingleNode(cx, obj, argc, argv, rval);
}


JSBool
xmldocument_node_transformNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	return xmlnode_transformNode(cx, obj, argc, argv, rval);
}


JSBool
xmldocument_node_transformNodeToObject(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	return xmlnode_transformNodeToObject(cx, obj, argc, argv, rval);
}






void
xmldocument_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside xmldocument_finalize(%x, %x)", cx, obj);
	WLXmlFinalize(cx, obj);
}																																																												




JSClass wl_XmldocumentClass = {
	WLJSNAME_CLASS_XMLDOCUMENT,
	JSCLASS_HAS_PRIVATE,	//ptr to msxml interface structure that info regarding the object it represents
	JS_PropertyStub,			//callback to C++
	JS_PropertyStub,			//--"--
	xmldocument_getProperty,
	xmldocument_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	xmldocument_finalize
};

static JSFunctionSpec xmldoc_methods[] = {

	{"abort"												,xmldocument_abort,															0},
	{"createAttribute"							,xmldocument_createAttribute,										0},
	{"createComment"								,xmldocument_createComment,											0},
	{"createCDATASection"						,xmldocument_createCDATASection,								0},
	{"createDocumentFragment"				,xmldocument_createDocumentFragment,						0},
	{"createElement"								,xmldocument_createElement,											0},
	{"createEntityReference"				,xmldocument_createEntityReference,							0},
	{"createNode" 									,xmldocument_createNode,												0},
	{"createProcessingInstruction"	,xmldocument_createProcessingInstruction,				0},
	{"createTextNode" 							,xmldocument_createTextNode,										0},
	{"getElementsByTagName"					,xmldocument_getElementsByTagName,							0},
	{"load"													,xmldocument_load,															0},
	{"loadXML"											,xmldocument_loadXML,														0},
	{"nodeFromID"										,xmldocument_nodeFromID,												0},		
	{"save"													,xmldocument_save,															0},
	//node methods
	{"appendChild", 								xmldocument_node_appendChild,														0},
	{"cloneNode",										xmldocument_node_cloneNode,															0},
	{"hasChildNodes",								xmldocument_node_hasChildNodes,													0},
	{"insertBefore",								xmldocument_node_insertBefore,																		0},
	{"removeChild",								  xmldocument_node_removeChild,														0},
	{"replaceChild",								xmldocument_node_replaceChild,													0},
	{"selectNodes",									xmldocument_node_selectNodes,														0},
	{"selectSingleNode",						xmldocument_node_selectSingleNode,											0},
	{"transformNode",								xmldocument_node_transformNode,													0},
	{"transformNodeToObject",				xmldocument_node_transformNodeToObject,									0},
	{0}
};


//constructor

static JSBool
WLXmlDocument(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		IXMLDOMDocumentPtr *pDocumentPtr = new IXMLDOMDocumentPtr(L"{2933BF90-7B36-11D2-B20E-00C04F983E60}");//DOMDocument CoClass		
		CWLXmlDocumentWrapper* wrapper = new CWLXmlDocumentWrapper(pDocumentPtr) ;
		CWLXmlPrivate* prv = new CWLXmlPrivate((void*)wrapper);
		JS_SetPrivate(cx, obj, prv);
		if(argc > 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 16);	
		if(argc == 1) {//load an xml string
			if(!JSVAL_IS_STRING(argv[0]))
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 17);
			else{
				char* str = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
				USES_CONVERSION;
				BSTR bs = A2BSTR(str);
				//_bstr_t bstrXML(bs);
				(*pDocumentPtr)->loadXML(bs);
				SysFreeString(bs);
			}
		}
			*rval = OBJECT_TO_JSVAL(obj);
		return JS_TRUE;
	}
	CATCH_COM(cx)
}


//from xml dom
JSObject* WLNewXmlDocumentObject(JSContext *cx, JSObject *obj)
{
	YieldRequest(cx);
WlTraceMsg("WLNewXmlDocumentObject For Dom(%x, %x)", cx, obj);
	// create an instance 
	JSObject* proto = NULL;
	JSObject* xmldoc = WLNewXmlObject(cx, &wl_XmldocumentClass, proto, obj);
	if (!xmldoc)
		return NULL;
	// define it's methods
//	if (!JS_DefineFunctions(cx, xmldoc, xmldoc_methods))
	//	return NULL;
	
	return xmldoc;
}


//from html dom
JSObject* WLNewXmlDocumentObject(JSContext *cx, JSObject *obj, CWLXmlDocument* xml, CWLDocument* parent)
{
	YieldRequest(cx);
WlTraceMsg("WLNewXmlDocumentObject For Html object(%x, %x)", cx, obj);
	// create an instance 

  WlAssert(piHLErrorReporter);

TRY_COM{
	JSObject* proto = NULL;
	JSObject* xmldoc = WLNewXmlObject(cx, &wl_XmldocumentClass, proto, obj);
	if (!xmldoc)
		return NULL;
	// define it's methods
//	if (!JS_DefineFunctions(cx, xmldoc, xmldoc_methods))
//		return NULL;
	CWLXmlPrivate* prv = NULL;
	//check whether the com object for this xml exists - if it doesnt:
//	WlTraceMsg("xml object 0x%x",xml);
//	WlTraceMsg("WLNewXmlDocumentObject Privates Count is: %d", xml->GetPrivatesCount());
	if(!(xml->GetPrivateInterface())){
		IXMLDOMDocumentPtr *pDocumentPtr_ForReference = new IXMLDOMDocumentPtr(L"{2933BF90-7B36-11D2-B20E-00C04F983E60}");//DOMDocument CoClass		
		CWLXmlInterfacePtr* newInterfacePtr = new CWLXmlInterfacePtr(pDocumentPtr_ForReference);
		xml->SetPrivateInterface(newInterfacePtr);
		IXMLDOMDocumentPtr *pDocumentPtr = new IXMLDOMDocumentPtr(*pDocumentPtr_ForReference);
		CWLXmlDocumentWrapper* wrapper = new CWLXmlDocumentWrapper(pDocumentPtr) ;
		prv = new CWLXmlPrivate((void*)wrapper, parent, xml, true);//xml now points to prv
    if(parent) {
			parent->incRefCount();	
      WlTraceMsg("Increase reference count to parent frame (by new xml document object & com object just created): %d", (int)parent);
    }
		JS_SetPrivate(cx, xmldoc, prv);
		//*********incorporate serialization of xml
		RVCString xmlString, error;
		const char* str = xml->GetXmlStandAloneStr(xmlString, error);
		if(!error.isNull())
			throw new wlXmlError(error);
		else{
			USES_CONVERSION;
			BSTR bs = A2BSTR(str);
			//_bstr_t bstrXML(bs);
			(*pDocumentPtr)->loadXML(bs);	
			SysFreeString(bs);
		}
		//****************
	}
	else //the com object already exists
	{	
		prv  = new CWLXmlPrivate(xml, parent);
    if(parent) {
      WlTraceMsg("Increase reference count to parent frame (by new xml document object & com object already exists): %d", (int)parent);
			parent->incRefCount();	
    }
		JS_SetPrivate(cx, xmldoc, prv);
	}
	return xmldoc;
	}
	catch(wlXmlError* e){
		piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 20, e->ErrorMessage());
		delete e;
		return NULL;
	}
	CATCH_COM_RETURN(cx, NULL);

}

/*
JSObject* WLNewXmlDocumentObject(JSContext *cx, JSObject *obj, CWLXmlDocument* xml, CWLDocument* doc)
{
WlTraceMsg("WLNewXmlDocumentObject For Html object(%x, %x)", cx, obj);
	// create an instance 

TRY_COM{
	JSObject* proto = NULL;
	JSObject* xmldoc = WLNewXmlObject(cx, &wl_XmldocumentClass, proto, obj);
	if (!xmldoc)
		return NULL;
	// define it's methods
	if (!JS_DefineFunctions(cx, xmldoc, xmldoc_methods))
		return NULL;
	CWLXmlPrivate* prv;
	//check whether the com object for this xml exists - if it doesnt:
	WlTraceMsg("xml object 0x%x",xml);
	WlTraceMsg("WLNewXmlDocumentObject Privates Count is: %d", xml->GetPrivatesCount());
	if(xml->GetPrivatesCount() < 1){
		IXMLDOMDocumentPtr *pDocumentPtr = new IXMLDOMDocumentPtr(L"{2933BF90-7B36-11D2-B20E-00C04F983E60}");//DOMDocument CoClass		
		CWLXmlDocumentWrapper* wrapper = new CWLXmlDocumentWrapper(pDocumentPtr) ;
		xml->SetPrivateInterface(pDocumentPtr);
		prv = new CWLXmlPrivate((void*)wrapper, doc, xml, true);//xml now points to prv
		JS_SetPrivate(cx, xmldoc, prv);
		//*********incorporate serialization of xml
		RVCString xmlString, error;
		const char* str = xml->GetXmlStandAloneStr(xmlString, error);
		if(!error.isNull())
			throw new wlXmlError(error);
		else{
			_bstr_t bstrXML(str);
			(*pDocumentPtr)->loadXML(bstrXML);	
		}
		//****************
	}
	else //the com object already exists
	{
		prv  = new CWLXmlPrivate(xml);
		JS_SetPrivate(cx, xmldoc, prv);
	}
	return xmldoc;
	}
	catch(wlXmlError* e){
		piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 20, e->ErrorMessage());
		return NULL;
	}
	CATCH_COM_RETURN(NULL);

}
/*
JSObject* WLNewXmlDocumentObject(JSContext *cx, JSObject *obj, CWLXmlDocument* xml, CWLDocument* doc)
{
WlTraceMsg("WLNewXmlDocumentObject For Html object(%x, %x)", cx, obj);
	// create an instance
	TRY_COM{
		JSObject* proto = NULL;
		JSObject* xmldoc = WLNewXmlObject(cx, &wl_XmldocumentClass, proto, obj);
		if (!xmldoc)
			return NULL;
		// define it's methods
		if (!JS_DefineFunctions(cx, xmldoc, xmldoc_methods))
			return NULL;
		CWLXmlPrivate* prv = new CWLXmlPrivate(NULL, doc, xml, true);
		RVCString xmlString, error;
		const char* str = xml->GetXmlStandAloneStr(xmlString, error);
		if(!error.isNull())
			throw new wlXmlError(error);
		else{
			_bstr_t bstrXML(str);
			CWLXmlDocumentWrapper* wrapper =  (CWLXmlDocumentWrapper*)(prv->GetObj());
			IXMLDOMDocumentPtr* docPtr = wrapper->GetInterfacePtr();
			(*docPtr)->loadXML(bstrXML);	
		}
		JS_SetPrivate(cx, xmldoc, prv);
		return xmldoc;
		}
		catch(wlXmlError* e){
			piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 20, e->ErrorMessage());
			return NULL;
		}
	CATCH_COM_RETURN(NULL);

}
*/

bool DllExport WLAddXmlDocumentClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddXmlDocumentClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_XmldocumentClass, WLXmlDocument, 1, NULL, xmldoc_methods,  NULL, NULL))
		return false;

	return true;
}
