/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <ace/OS.h>
#include "wlxmlcom.h"
#include "wlxmlfuncs.h"
#include <string.h>
#include <RVCString.h>
#include "rvjsapi.h"

#include "xmlreplyprops.h"
#include "jsxmlreplynames.h"
#include "jsxmlprivate.h"
#include "jsxmlwrappers.h"
#ifdef _DEBUG
#include <iostream.h>
#endif

#include "xmlerrorreporter.h"
#include "tracedll.h"


CWLXmlNamedNodeMapWrapper :: CWLXmlNamedNodeMapWrapper(IXMLDOMNamedNodeMapPtr* pXMLDOMNamedNodePtr){
	m_pXMLNamedNodeMap = pXMLDOMNamedNodePtr;
}

IXMLDOMNamedNodeMapPtr* CWLXmlNamedNodeMapWrapper:: GetInterfacePtr(){
		return m_pXMLNamedNodeMap;
}

CWLXmlNamedNodeMapWrapper :: ~CWLXmlNamedNodeMapWrapper(){
	if(m_pXMLNamedNodeMap)
		delete m_pXMLNamedNodeMap;
}

static JSBool
xmlnamednodemap_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	TRY_COM {
		WlTraceMsg("Inside xmlnamednodemap_getProperty");
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		IXMLDOMNamedNodeMapPtr pNamedNodeMapInterface = *(((CWLXmlNamedNodeMapWrapper*)(prv->GetObj()))->GetInterfacePtr());
		if ((pNamedNodeMapInterface != NULL) && JSVAL_IS_STRING(id)) {
			JSString* str = JSVAL_TO_STRING(id);
			char* prop = JS_GetStringBytes(str);
			if (!strcmp(prop, WLJSNAME_PROP_LENGTH)) {	
				long length;
				length = pNamedNodeMapInterface->Getlength();
				return WLXmlSetIntValue(cx, obj, NULL, (unsigned)length, vp);
			}
		}
		return JS_TRUE;
	}
	CATCH_COM(cx)
}

static JSBool
xmlnamednodemap_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp) {
	WlTraceMsg("Inside xmlnamednodemap_setProperty");
	return JS_TRUE;
}

static JSBool
xmlnamednodemap_getNamedItem(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "getNamedItem", 1);
		if(!JSVAL_IS_STRING(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "getNamedItem");
		CWLXmlNamedNodeMapWrapper* wrapper	= (CWLXmlNamedNodeMapWrapper*)(prv->GetObj());
		if(!wrapper)
			return JS_TRUE;//error
		IXMLDOMNamedNodeMapPtr mapPtr				= *(wrapper->GetInterfacePtr());
		IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
		_bstr_t name(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
		*nodePtr = mapPtr->getNamedItem(name);
		CWLXmlNodeWrapper* nodeWrapper = new CWLXmlNodeWrapper(nodePtr);
		return WLXmlSetNodeInterfaceValue(cx, obj, nodeWrapper, rval);
	}
	CATCH_COM_METHOD(cx, "getNamedItem")
}

static JSBool
xmlnamednodemap_getQualifiedItem(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(argc != 2)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "getQualifiedItem", 2);
		if(!JSVAL_IS_STRING(argv[0]) || !JSVAL_IS_STRING(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "getQualifiedItem");
		CWLXmlNamedNodeMapWrapper* wrapper	= (CWLXmlNamedNodeMapWrapper*)(prv->GetObj());
		if(!wrapper)
			return JS_TRUE;//error
		IXMLDOMNamedNodeMapPtr mapPtr				= *(wrapper->GetInterfacePtr());
		IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
		_bstr_t baseName(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
		_bstr_t namespaceURI(JS_GetStringBytes(JSVAL_TO_STRING(argv[1])));
		*nodePtr = mapPtr->getQualifiedItem(baseName, namespaceURI);
		CWLXmlNodeWrapper* nodeWrapper = new CWLXmlNodeWrapper(nodePtr);
		return WLXmlSetNodeInterfaceValue(cx, obj, nodeWrapper, rval);
	}
	CATCH_COM_METHOD(cx, "getQualifiedItem")
}

static JSBool
xmlnamednodemap_Getitem(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "item", 1);
		if(!JSVAL_IS_INT(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "item");
		CWLXmlNamedNodeMapWrapper* wrapper	= (CWLXmlNamedNodeMapWrapper*)(prv->GetObj());
		if(!wrapper)
			return JS_TRUE;//error
		IXMLDOMNamedNodeMapPtr mapPtr				= *(wrapper->GetInterfacePtr());
		IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
		jsint index = JSVAL_TO_INT(argv[0]);
		*nodePtr = mapPtr->Getitem((long)index);
		CWLXmlNodeWrapper* nodeWrapper = new CWLXmlNodeWrapper(nodePtr);
		return WLXmlSetNodeInterfaceValue(cx, obj, nodeWrapper, rval);	
	}
	CATCH_COM_METHOD(cx, "item")
}

static JSBool
xmlnamednodemap_nextNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(argc)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "nextNode", 0);
		CWLXmlNamedNodeMapWrapper* wrapper	= (CWLXmlNamedNodeMapWrapper*)(prv->GetObj());
		if(!wrapper)
			return JS_TRUE;//error
		IXMLDOMNamedNodeMapPtr mapPtr				= *(wrapper->GetInterfacePtr());
		IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
		*nodePtr = mapPtr->nextNode();
		CWLXmlNodeWrapper* nodeWrapper = new CWLXmlNodeWrapper(nodePtr);
		return WLXmlSetNodeInterfaceValue(cx, obj, nodeWrapper, rval);
	}
	CATCH_COM_METHOD(cx, "nextNode")
}

static JSBool
xmlnamednodemap_removeNamedItem(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "removeNamedItem", 2);
		if(!JSVAL_IS_STRING(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "removeNamedItem");
		CWLXmlNamedNodeMapWrapper* wrapper	= (CWLXmlNamedNodeMapWrapper*)(prv->GetObj());
		if(!wrapper)
			return JS_TRUE;//error
		IXMLDOMNamedNodeMapPtr mapPtr				= *(wrapper->GetInterfacePtr());
		IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
		_bstr_t name(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
		*nodePtr = mapPtr->removeNamedItem(name);
		CWLXmlNodeWrapper* nodeWrapper = new CWLXmlNodeWrapper(nodePtr);
		return WLXmlSetNodeInterfaceValue(cx, obj, nodeWrapper, rval);
	}
	CATCH_COM_METHOD(cx, "removeNamedItem")
}

static JSBool
xmlnamednodemap_removeQualifiedItem(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(argc != 2)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "removeQualifiedItem", 2);
		if(!JSVAL_IS_STRING(argv[0]) || !JSVAL_IS_STRING(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "removeQualifiedItem");
		CWLXmlNamedNodeMapWrapper* wrapper	= (CWLXmlNamedNodeMapWrapper*)(prv->GetObj());
		if(!wrapper)
			return JS_TRUE;//error
		IXMLDOMNamedNodeMapPtr mapPtr				= *(wrapper->GetInterfacePtr());
		IXMLDOMNodePtr* nodePtr = new IXMLDOMNodePtr;
		_bstr_t baseName(JS_GetStringBytes(JSVAL_TO_STRING(argv[0])));
		_bstr_t namespaceURI(JS_GetStringBytes(JSVAL_TO_STRING(argv[1])));
		*nodePtr = mapPtr->removeQualifiedItem(baseName, namespaceURI);
		CWLXmlNodeWrapper* nodeWrapper = new CWLXmlNodeWrapper(nodePtr);
		return WLXmlSetNodeInterfaceValue(cx, obj, nodeWrapper, rval);
	}
	CATCH_COM_METHOD(cx, "removeQualifiedItem")
}

static JSBool
xmlnamednodemap_reset(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(argc)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "reset", 0);
		CWLXmlNamedNodeMapWrapper* wrapper	= (CWLXmlNamedNodeMapWrapper*)(prv->GetObj());
		if(!wrapper)
			return JS_TRUE;//error
		IXMLDOMNamedNodeMapPtr mapPtr				= *(wrapper->GetInterfacePtr());
		HRESULT hr = mapPtr->reset();
		return succeeded(cx, hr);
	}
	CATCH_COM_METHOD(cx, "reset")
}

static JSBool
xmlnamednodemap_setNamedItem(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
		if (!prv)
			return JS_TRUE;
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "setNamedItem", 1);
		if(!JSVAL_IS_OBJECT(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "setNamedItem");
		CWLXmlNamedNodeMapWrapper* wrapper	= (CWLXmlNamedNodeMapWrapper*)(prv->GetObj());
		if(!wrapper)
			return JS_TRUE;//error
		IXMLDOMNamedNodeMapPtr mapPtr				= *(wrapper->GetInterfacePtr());
		//extract the node 
		JSObject* nodeObj = JSVAL_TO_OBJECT(argv[0]);
		CWLXmlPrivate* nodePrv = (CWLXmlPrivate*)JS_GetPrivate(cx, nodeObj);
		CWLXmlNamedNodeMapWrapper* nodeWrapper	= (CWLXmlNamedNodeMapWrapper*)(nodePrv->GetObj());
		IXMLDOMNodePtr nodePtr = *(nodeWrapper->GetInterfacePtr());
		IXMLDOMNodePtr* newNodePtr = new IXMLDOMNodePtr;
		*newNodePtr = mapPtr->setNamedItem(&(*nodePtr));
		CWLXmlNodeWrapper* newNodeWrapper = new CWLXmlNodeWrapper(newNodePtr);
		return WLXmlSetNodeInterfaceValue(cx, obj, newNodeWrapper, rval);
	}
	CATCH_COM_METHOD(cx, "setNamedItem")
}


static void
xmlnamednodemap_finalize(JSContext *cx, JSObject *obj)
{
WlTraceMsg("Inside xmlnamednodemap_finalize(%x, %x)", cx, obj);
	WLXmlFinalize(cx, obj);
}				

JSClass wl_XmlnamednodemapClass = {
	WLJSNAME_CLASS_XMLNAMED_NODEMAP,
	JSCLASS_HAS_PRIVATE,	//ptr to msxml interface structure that info regarding the object it represents
	JS_PropertyStub,			//callback to C++
	JS_PropertyStub,			//--"--
	xmlnamednodemap_getProperty,
	xmlnamednodemap_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	xmlnamednodemap_finalize
};	



static JSFunctionSpec namednodemap_methods[] = {
	{"getNamedItem",				xmlnamednodemap_getNamedItem,				 0},
	{"getQualifiedItem", 		xmlnamednodemap_getQualifiedItem,		 0},
	{"setNamedItem",				xmlnamednodemap_setNamedItem,				 0},
	{"removeNamedItem", 		xmlnamednodemap_removeNamedItem,		 0},
	{"item",								xmlnamednodemap_Getitem,						 0},
	{"reset"				,				xmlnamednodemap_reset,							 0},
	{"removeQualifiedItem", xmlnamednodemap_removeQualifiedItem, 0},
	{"nextNode",				    xmlnamednodemap_nextNode,						 0},
	{0}
};


//constructor
static JSBool
WLNamedNodeMap(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	return JS_TRUE;
}

JSObject* WLNewXmlNamedNodeMapObject(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLNewXmlNamedNodeMapObject(%x, %x)", cx, obj);
	// create an instance 
	JSObject* proto = NULL;
	JSObject* namedMap = WLNewXmlObject(cx, &wl_XmlnamednodemapClass, proto, obj);
	if (!namedMap)
		return NULL;

	// define it's methods
//	if (!JS_DefineFunctions(cx, namedMap, namednodemap_methods))
//		return NULL;

	return namedMap;
}

bool DllExport WLAddXmlNamedNodeMapClass(JSContext *cx, JSObject *obj)
{
WlTraceMsg("WLAddXmlNamedNodeMapClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_XmlnamednodemapClass, NULL, 1, NULL, namednodemap_methods, NULL, NULL))
		return false;

	return true;
}
