/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <ace/OS.h>
#include "wlxmlcom.h"
#include "wlxmlfuncs.h"
#include <string.h>
#include <RVCString.h>
#include "rvjsapi.h"

#include "wljsobjects.h"
#include "xmlreplyprops.h"
#include "jsxmlreplynames.h"
#include "jsxmlprivate.h"
#include "jsxmlwrappers.h"
#ifdef _DEBUG
#include <iostream.h>
#endif
#include "tracedll.h"

#include "xmlerrorreporter.h"
#include "tracedll.h"


CWLXmlNodeWrapper :: CWLXmlNodeWrapper(IXMLDOMNodePtr* pNodePtr){
	m_pXMLNode		= pNodePtr;
	m_type				= wlInterfaceNodeType;
}

IXMLDOMNodePtr* CWLXmlNodeWrapper:: GetInterfacePtr(){
		return m_pXMLNode;
}

CWLXmlNodeWrapper :: ~CWLXmlNodeWrapper(){
	if(m_pXMLNode)
		delete m_pXMLNode;
}


//change to ptr  so  u can return null
bool getNode(JSContext* cx, JSObject* obj,IXMLDOMNodePtr& nodePtr, bool& isDocument, CWLXmlDocument*& pXmlDoc)
{
	CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return false;
	pXmlDoc = prv->GetXmlObj() ;
	CWLXmlInterfaceParent* parent = (CWLXmlInterfaceParent*)(prv->GetObj());
	if (!parent)
		return false;
	if(parent->GetType() == wlInterfaceNodeType)
	{
		CWLXmlNodeWrapper* wrapper	= (CWLXmlNodeWrapper*)(prv->GetObj());
		if(!wrapper)
			return false;
		nodePtr				= *(wrapper->GetInterfacePtr());
	}
	else if(parent->GetType() == wlInterfaceDocumentType)
	{
		isDocument = true;
		CWLXmlDocumentWrapper* wrapper	= (CWLXmlDocumentWrapper*)(prv->GetObj());
		if(!wrapper)
			return false;
		nodePtr				= *(wrapper->GetInterfacePtr());
	}
	else 
		return false;
	return true;
}

bool getNode(JSContext* cx, JSObject* obj,IXMLDOMNodePtr& nodePtr)
{
	CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, obj);
	if (!prv)
		return false;
	CWLXmlInterfaceParent* parent = (CWLXmlInterfaceParent*)(prv->GetObj());
	if(!parent)
		return false;
	if(parent->GetType() == wlInterfaceNodeType)
	{
		CWLXmlNodeWrapper* wrapper	= (CWLXmlNodeWrapper*)(prv->GetObj());
		if(!wrapper)
			return false;
		nodePtr				= *(wrapper->GetInterfacePtr());
	}
	else if(parent->GetType() == wlInterfaceDocumentType)
	{
		CWLXmlDocumentWrapper* wrapper	= (CWLXmlDocumentWrapper*)(prv->GetObj());
		if(!wrapper)
			return false;
		nodePtr				= *(wrapper->GetInterfacePtr());
	}
	else 
		return false;
	return true;
}


JSBool
xmlnode_getProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp)
{
	long nodeType;
	char* prop;
TRY_COM{
	WlTraceMsg("Inside xmlnode_getProperty");
	IXMLDOMNodePtr pNodeInterface;
	bool isDocument = false;
	CWLXmlDocument* pXmlDoc = NULL;
	if (getNode(cx, obj, pNodeInterface, isDocument, pXmlDoc) && JSVAL_IS_STRING(id)) {
		nodeType = pNodeInterface->GetnodeType();
		JSString* str = JSVAL_TO_STRING(id);
		prop = JS_GetStringBytes(str);
//general node props
		if (getNode(cx, obj, pNodeInterface) && !strcmp(prop, WLJSNAME_PROP_ATTRIBUTES)) {	
			IXMLDOMNamedNodeMapPtr* pNodeMap = new IXMLDOMNamedNodeMapPtr;
			*pNodeMap = pNodeInterface->Getattributes();
			CWLXmlNamedNodeMapWrapper* wrapper = new CWLXmlNamedNodeMapWrapper(pNodeMap);
			return WLXmlSetNamedNodeMapInterfaceValue(cx, obj, wrapper, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_BASENAME)) {
			_bstr_t base = pNodeInterface->GetbaseName();
			return WLXmlSetStringValue(cx, obj, NULL, (char*)base, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_CHILDNODES)) {	
			IXMLDOMNodeListPtr* childs = new IXMLDOMNodeListPtr;
			*childs = pNodeInterface->GetchildNodes();
			CWLXmlNodeListWrapper* wrapper = new CWLXmlNodeListWrapper(childs);
			return WLXmlSetNodeListInterfaceValue(cx, obj, wrapper, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_DATATYPE)) {	
			_variant_t type = pNodeInterface->GetdataType();
			return WLXmlSetVariantValue(cx, obj, type, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_DEFINITION)) {			
			IXMLDOMNodePtr* node = new IXMLDOMNodePtr;
			*node = pNodeInterface->Getdefinition();
			CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(node);
			return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, vp);
		}
		else if(!strcmp(prop, WLJSNAME_PROP_FIRSTCHILD)) {	
			IXMLDOMNodePtr* node = new IXMLDOMNodePtr;
			*node = pNodeInterface->GetfirstChild();
			CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(node);
			return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_LASTCHILD)) {
			if(checkNodeTypeByInt(nodeType, 3, 4, 7, 8, 12, -1))
				return JS_TRUE;			
			IXMLDOMNodePtr* node = new IXMLDOMNodePtr;
			*node = pNodeInterface->GetlastChild();
			CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(node);
			return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, vp);
		}
		if (!strcmp(prop, WLJSNAME_PROP_NEXTSIBLING)) {	
			IXMLDOMNodePtr* node = new IXMLDOMNodePtr;
			*node = pNodeInterface->GetnextSibling();
			CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(node);
			return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_NODETYPE)) {	
			long type = pNodeInterface->GetnodeType();
			return WLXmlSetIntValue(cx, obj, NULL, (unsigned)type, vp);
		}		
		else if (!strcmp(prop, WLJSNAME_PROP_NAMESPACEURI)) {
			_bstr_t nameSpaceURI = pNodeInterface->GetnamespaceURI();
			return WLXmlSetStringValue(cx, obj, NULL, (char*)nameSpaceURI, vp);
		}
		//for
		//NODE_CDATA_SECTION, NODE_COMMENT, NODE_DOCUMENT, NODE_DOCUMENT_FRAGMENT, NODE_TEXT
		//a constant string is returned e.g "#text"
		else if (!strcmp(prop, WLJSNAME_PROP_NODENAME)) {			
			_bstr_t name = pNodeInterface->GetnodeName();
			return WLXmlSetStringValue(cx, obj, NULL, (char*)name, vp);
		}		
		else if (!strcmp(prop, WLJSNAME_PROP_NODETYPESTRING)) {
			_bstr_t name = pNodeInterface->GetnodeTypeString();
			return WLXmlSetStringValue(cx, obj, NULL, (char*)name, vp);			
		}
		else if (!strcmp(prop, WLJSNAME_PROP_NODETYPEDVALUE)) {	
			_variant_t typedvalue = pNodeInterface->GetnodeTypedValue();
			return WLXmlSetVariantValue(cx, obj, typedvalue, vp);			
		}
		else if (!strcmp(prop, WLJSNAME_PROP_NODEVALUE)) {	
			_variant_t value = pNodeInterface->GetnodeValue();
			return WLXmlSetVariantValue(cx, obj, value, vp);			
		}
		else if (!strcmp(prop, WLJSNAME_PROP_OWNERDOCUMENT)) {	
			IXMLDOMDocumentPtr* doc = new IXMLDOMDocumentPtr;
			*doc = pNodeInterface->GetownerDocument();
			CWLXmlDocumentWrapper* wrapper = new CWLXmlDocumentWrapper(doc);
			return WLXmlSetDocumentInterfaceValue(cx, obj, wrapper, vp);					
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PARENTNODE)) {	
			IXMLDOMNodePtr* node = new IXMLDOMNodePtr;
			*node = pNodeInterface->GetparentNode();
			CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(node);
			return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PARSED)) {	
			bool parsed = false;
			if(pNodeInterface->Getparsed() != 0)
				parsed = true;
			return WLXmlSetBoolValue( cx, obj, NULL, parsed, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PREFIX)) {	
			_bstr_t prefix = pNodeInterface->Getprefix();
			return WLXmlSetStringValue(cx, obj, NULL, (char*)prefix, vp);			
		}
		else if (!strcmp(prop, WLJSNAME_PROP_PREVIOUSSIBLING)) {	
			IXMLDOMNodePtr* node = new IXMLDOMNodePtr;
			*node = pNodeInterface->GetpreviousSibling();
			CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(node);
			return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_SPECIFIED)) {	
			bool specified = false;
			if(pNodeInterface->Getspecified() != 0)
				specified = true;
			return WLXmlSetBoolValue( cx, obj, NULL, specified, vp);
		}
		else if (!strcmp(prop, WLJSNAME_PROP_TEXT)) {	
			_bstr_t text = pNodeInterface->Gettext();
			return WLXmlSetStringValue(cx, obj, NULL, (char*)text, vp);		
		}
		else if (!strcmp(prop, WLJSNAME_PROP_XML)) {	
			_bstr_t xml = pNodeInterface->Getxml();
			return WLXmlSetStringValue(cx, obj, NULL, (char*)xml, vp);		
		}
		else{
			switch(pNodeInterface->GetnodeType()){

			case  MSXML::NODE_ELEMENT		:
				return xmlelement_getProperty(cx, obj, id, vp, pNodeInterface);

			case	MSXML::NODE_ATTRIBUTE	:
				return xmlattribute_getProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_TEXT:
				return xmltext_getProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_CDATA_SECTION:
				return xmlcdata_getProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_ENTITY_REFERENCE :
				return xmlentityref_getProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_ENTITY:
				return xmlentity_getProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_PROCESSING_INSTRUCTION:
				return xmlprocessinginstruction_getProperty(cx, obj, id, vp, pNodeInterface);

			case	MSXML::NODE_COMMENT :
				return xmlcomment_getProperty(cx, obj, id, vp, pNodeInterface);

			case	MSXML::NODE_DOCUMENT :
				if(!isDocument)
					return xmldocument_getProperty(cx, obj, id, vp);
				break;

			case	MSXML::NODE_DOCUMENT_TYPE :
				return xmldocumenttype_getProperty(cx, obj, id, vp, pNodeInterface);

			case	MSXML::NODE_DOCUMENT_FRAGMENT :
				return xmldocumentfragment_getProperty(cx, obj, id, vp, pNodeInterface);

			case	MSXML::NODE_NOTATION:
				return xmlnotation_getProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_INVALID:

			default:
				return JS_TRUE;			
			}
		}
	}
	return JS_TRUE;
	}
CATCH_COM(cx)
}

JSBool
xmlnode_setProperty(JSContext *cx, JSObject *obj, jsval id, jsval *vp) {
TRY_COM{
	WlTraceMsg("Inside xmlnode_setProperty");
  WlAssert(piHLErrorReporter);
	IXMLDOMNodePtr pNodeInterface;
	bool isDocument = false;
	CWLXmlDocument* pXmlDoc;
	if (getNode(cx, obj, pNodeInterface, isDocument, pXmlDoc) && JSVAL_IS_STRING(id)) {
		JSString* str = JSVAL_TO_STRING(id);
		_bstr_t name = pNodeInterface->GetnodeTypeString();
		char* nodeType = (char*)name;
		long  nodeTypeInt = pNodeInterface->GetnodeType();
		char* prop = JS_GetStringBytes(str);
		if (!strcmp(prop, WLJSNAME_PROP_NODETYPEDVALUE)) {	
			_variant_t value;
			if(WLXmlJsvalToVariant(*vp, value))
				pNodeInterface->PutnodeTypedValue(value);
			else 
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 15);

		}
		else if(!strcmp(prop, WLJSNAME_PROP_NODEVALUE)) {	
			if(checkNodeTypeByInt(nodeTypeInt, 1, 5, 6, 9, 10, 11, 12, -1))
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 43, "nodeValue", nodeType);				
			_bstr_t value = JS_GetStringBytes(JSVAL_TO_STRING(*vp));
			pNodeInterface->PutnodeValue(value);
		}
		else if(!strcmp(prop, WLJSNAME_PROP_TEXT)) {	
			if(checkNodeTypeByInt(nodeTypeInt, 5, 6, 10, 11, 12, -1))
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 43,  "text", nodeType);				
			_bstr_t text = JS_GetStringBytes(JSVAL_TO_STRING(*vp));
			pNodeInterface->Puttext(text);
		}
		else if(!strcmp(prop, WLJSNAME_PROP_DATATYPE)) {	
			if(checkNodeTypeByInt(nodeTypeInt, 3, 4, 6, 7, 8, 9, 10, 11, 12, -1))
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 43,  "dataType", nodeType);				
		_bstr_t type =  JS_GetStringBytes(JSVAL_TO_STRING(*vp));
		pNodeInterface->PutdataType(type);
		}
		else
			switch(pNodeInterface->GetnodeType()){

			case  MSXML::NODE_ELEMENT		:
				return xmlelement_getProperty(cx, obj, id, vp, pNodeInterface);

			case	MSXML::NODE_ATTRIBUTE	:
				return xmlattribute_setProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_TEXT:
				return xmltext_setProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_CDATA_SECTION:
				return xmlcdata_setProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_ENTITY_REFERENCE :
				return xmlentityref_setProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_ENTITY:
				return xmlentity_setProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_PROCESSING_INSTRUCTION:
				return xmlprocessinginstruction_setProperty(cx, obj, id, vp, pNodeInterface);

			case	MSXML::NODE_COMMENT :
				return xmlcomment_setProperty(cx, obj, id, vp, pNodeInterface);

			case	MSXML::NODE_DOCUMENT :
				if(!isDocument)
					return xmldocument_setProperty(cx, obj, id, vp);
				break;

			case	MSXML::NODE_DOCUMENT_TYPE :
				return xmldocumenttype_setProperty(cx, obj, id, vp, pNodeInterface);

			case	MSXML::NODE_DOCUMENT_FRAGMENT :
				return xmldocumentfragment_setProperty(cx, obj, id, vp, pNodeInterface);

			case	MSXML::NODE_NOTATION:
				return xmlnotation_setProperty(cx, obj, id, vp, pNodeInterface);

			case  MSXML::NODE_INVALID:

			default:
				return JS_TRUE;			
			}
	}
		return JS_TRUE;
}
CATCH_COM(cx)
}

/*****************************  NODE METHODS  *********************************/

JSBool
xmlnode_appendChild(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM{
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "appendChild", 1);
		IXMLDOMNodePtr pNodeInterface;
		if(!JSVAL_IS_OBJECT(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "appendChild");
		if(!getNode(cx, obj, pNodeInterface))
			return JS_TRUE;//error
		JSObject* newChildObj = JSVAL_TO_OBJECT(argv[0]);
		IXMLDOMNodePtr newChildPtr;
		if(!getNode(cx, newChildObj, newChildPtr))
			return JS_TRUE;
		long parentNodeType = pNodeInterface->GetnodeType();
		long newChildNodeType = newChildPtr->GetnodeType();
		switch(parentNodeType){
			case(1)://element	
				if(checkNodeTypeByInt(newChildNodeType, 2, 5, 6, 9, 10, 12, -1))
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32, "appendChild", "element", ConvertNodeTypeToString(newChildNodeType));
				else
					break;
			case(2)://attribute
				if(checkNodeTypeByInt(newChildNodeType, 1, 2, 4, 6,  7, 8, 9, 10, 12, -1))
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32, "appendChild", "attribute", ConvertNodeTypeToString(newChildNodeType));
				else
					break;
			case(9)://document
				if(checkNodeTypeByInt(newChildNodeType, 2, 3, 4, 5, 6, 9, 10, 12, -1))
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32, "appendChild", "document", ConvertNodeTypeToString(newChildNodeType));
				else
					break;
			case(11)://document fragment
				if(checkNodeTypeByInt(newChildNodeType, 2, 6, 9, 10, 12, -1))
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32, "appendChild", "documentfragment", ConvertNodeTypeToString(newChildNodeType));
				else
					break;
			case(10)://documenttype
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 33, "appendChild", "documenttype");				
			case(3)://text
			case(4)://cdata
			case(7)://PI
			case(8)://comment
			case(12)://notation
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 34, "appendChild", ConvertNodeTypeToString(parentNodeType));
		}
		IXMLDOMNodePtr* pOutNewChildPtr = new IXMLDOMNodePtr;
		*pOutNewChildPtr = pNodeInterface->appendChild(&(*newChildPtr));
		CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(pOutNewChildPtr);
		return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, rval);
	}
	CATCH_COM_METHOD(cx, "appendChild")	
}




JSBool
xmlnode_cloneNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "cloneNode", 1);
		IXMLDOMNodePtr pNodeInterface;
		if(!JSVAL_IS_BOOLEAN(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "cloneNode");
		if(!getNode(cx, obj, pNodeInterface))
			return JS_TRUE;//error
		VARIANT_BOOL deep = 0;
		if(JSVAL_TO_BOOLEAN(argv[0]) == JS_TRUE)
			deep = -1;
		IXMLDOMNodePtr* cloneNode = new IXMLDOMNodePtr;
		*cloneNode = pNodeInterface->cloneNode(deep);	
		CWLXmlNodeWrapper* cloneNodeWrapper = new CWLXmlNodeWrapper(cloneNode);
		return WLXmlSetNodeInterfaceValue(cx, obj, cloneNodeWrapper, rval);
	}
	CATCH_COM_METHOD(cx, "cloneNode")
}

JSBool
xmlnode_hasChildNodes(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "hasChildNodes", 0);
		IXMLDOMNodePtr pNodeInterface;
		if(getNode(cx, obj, pNodeInterface)) {
		bool hasChildNodes = false;
		if(pNodeInterface->hasChildNodes() != 0)
		hasChildNodes = true;
		return WLXmlSetBoolValue(cx, obj, NULL, hasChildNodes, rval);
		}
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "hasChildNode")
}

JSBool
xmlnode_insertBefore(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 2)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "insertBefore", 2);
		if(!JSVAL_IS_OBJECT(argv[0]) || !JSVAL_IS_OBJECT(argv[1]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "insertBefore");
		IXMLDOMNodePtr pNodeInterface;
		if(!getNode(cx, obj, pNodeInterface))
			return JS_TRUE;
		//get new child
		JSObject* childObj = JSVAL_TO_OBJECT(argv[0]);	
		IXMLDOMNodePtr pNewChildPtr ;
		if(!getNode(cx, childObj, pNewChildPtr))
			return JS_TRUE;
		MSXML::IXMLDOMNode* newChild = &(*pNewChildPtr);
		long parentNodeType = pNodeInterface->GetnodeType();
		long newChildNodeType = pNewChildPtr->GetnodeType();
		switch(parentNodeType){
			case(1)://element	
				if(checkNodeTypeByInt(newChildNodeType, 2, 5, 6, 9, 10, 12, -1))
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32, "insertBefore", "element", ConvertNodeTypeToString(newChildNodeType));
				else
					break;
			case(2)://attribute
				if(checkNodeTypeByInt(newChildNodeType, 1, 2, 4, 6,  7, 8, 9, 10, 12, -1))
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32, "insertBefore", "attribute", ConvertNodeTypeToString(newChildNodeType));
				else
					break;
			case(9)://document
				if(checkNodeTypeByInt(newChildNodeType, 2, 3, 4, 5, 6, 9, 10, 12, -1))
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32, "insertBefore", "document", ConvertNodeTypeToString(newChildNodeType));
				else
					break;
			case(11)://document fragment
				if(checkNodeTypeByInt(newChildNodeType, 2, 6, 9, 10, 12, -1))
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32, "insertBefore", "documentfragment", ConvertNodeTypeToString(newChildNodeType));
				else
					break;
			case(10)://documenttype
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 33, "insertBefore", "documenttype");				
			case(3)://text
			case(4)://cdata
			case(7)://PI
			case(8)://comment
			case(12)://notation
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 34, "insertBefore", ConvertNodeTypeToString(parentNodeType));
		}
		//get ref child - the child before which we insert new child
		childObj = JSVAL_TO_OBJECT(argv[1]);	
		IXMLDOMNodePtr pRefChildPtr;
		if(!getNode(cx, childObj, pRefChildPtr))
			return JS_TRUE;
		IDispatchPtr disp = pRefChildPtr;
		_variant_t varRefChild(&(*disp));
		//insertbefore
		MSXML::IXMLDOMNodePtr* returnedNode = new IXMLDOMNodePtr;
		*returnedNode  = pNodeInterface->insertBefore(newChild, varRefChild);
		//release the IDispatchPtr
		disp->Release();
		CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(returnedNode);
		return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, rval);
	}
	CATCH_COM_METHOD(cx, "insertBefore")
}

JSBool
xmlnode_removeChild(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 1)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "removeChild", 1);
		if(!JSVAL_IS_OBJECT(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "removeChild");
		IXMLDOMNodePtr pNodeInterface;
		if(!getNode(cx, obj, pNodeInterface))
			return JS_TRUE;
		long parent = pNodeInterface->GetnodeType();
		if(parent == 5)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 35, "removeChild");
		JSObject* childObj = JSVAL_TO_OBJECT(argv[0]);	
		IXMLDOMNodePtr pRemoveNodePtr;
		if(!getNode(cx, childObj, pRemoveNodePtr))
			return JS_TRUE;
		long removedChild = pRemoveNodePtr->GetnodeType();
		if(checkNodeTypeByInt(removedChild, 6, 10, 12, -1))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 33, "removeChild", ConvertNodeTypeToString(removedChild));
		IXMLDOMNodePtr* returnedNode = new IXMLDOMNodePtr;	
		*returnedNode = pNodeInterface->removeChild(&(*pRemoveNodePtr));
		CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(returnedNode);
		return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, rval);
	}
CATCH_COM_METHOD(cx, "removeChild")

}

JSBool
xmlnode_replaceChild(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 2)//error
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "replaceChild", 2);
		if(!JSVAL_IS_OBJECT(argv[0]) || !JSVAL_IS_OBJECT(argv[1]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "replaceChild");
		IXMLDOMNodePtr pNodeInterface;
		if(!getNode(cx, obj, pNodeInterface))
			return JS_TRUE;
		//get new child interface
		JSObject* newChildObj = JSVAL_TO_OBJECT(argv[0]);		
		IXMLDOMNodePtr pNewChildNodePtr;
		if(!getNode(cx, newChildObj, pNewChildNodePtr))
			return JS_TRUE;
		//get old child interface
		JSObject* oldChildObj = JSVAL_TO_OBJECT(argv[1]);		
		IXMLDOMNodePtr pOldChildNodePtr;
		if(!getNode(cx, oldChildObj, pOldChildNodePtr))
			return JS_TRUE;
		//check for errors
		long parentNodeType = pNodeInterface->GetnodeType();
		long newChildNodeType = pNewChildNodePtr->GetnodeType();
		switch(parentNodeType){
			case(1)://element
				//attribute, entity, entity reference, document, document type, notation
				if(checkNodeTypeByInt(newChildNodeType, 2, 5, 6, 9, 10, 12, -1))
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32, "replaceChild", "element", ConvertNodeTypeToString(newChildNodeType));
				break;
			case(2)://attribute
				if(checkNodeTypeByInt(newChildNodeType, 3, 5, 11, -1))
					break;
				else
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32, "replaceChild", "attribute", ConvertNodeTypeToString(newChildNodeType));
			case(9)://document
				//elment, PI, comment, document fragment - OK
				if(checkNodeTypeByInt(newChildNodeType, 1, 7, 8, 11, -1))
					break;
				//document type is read only
				else if(checkNodeTypeByInt(newChildNodeType, 10, -1))
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 33, "replaceChild", "documenttype");				
				//the rest are invalid as children of document
				else
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32, "replaceChild", "document", ConvertNodeTypeToString(newChildNodeType));
			case(11)://document fragment
				//attribute. entity, document, document type, notation not allowed as children of document fragment
				if(checkNodeTypeByInt(newChildNodeType, 2, 6, 9, 10, 12, -1))
					return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 32,  "replaceChild", "documentfragment", ConvertNodeTypeToString(newChildNodeType));
				else
					break;
			case(3)://text
			case(4)://cdata
			case(7)://PI
			case(8)://comment
			case(12)://notation
				return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 34, "replaceChild", ConvertNodeTypeToString(parentNodeType));
		}
		//replace and return 
		IXMLDOMNodePtr* returnedNode = new IXMLDOMNodePtr;	
		*returnedNode = pNodeInterface->replaceChild(&(*pNewChildNodePtr), &(*pOldChildNodePtr));
		CWLXmlNodeWrapper* wrapper = new CWLXmlNodeWrapper(returnedNode);
		return WLXmlSetNodeInterfaceValue(cx, obj, wrapper, rval);
	}
	CATCH_COM_METHOD(cx, "replaceChild")

}

JSBool
xmlnode_selectNodes(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 1)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "selectNodes", 1);
		if(!JSVAL_IS_STRING(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "selectNodes");
		IXMLDOMNodePtr pNodeInterface;
		if(!getNode(cx, obj, pNodeInterface))
			return JS_TRUE;
		char* selectStr = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));//XSL pattern that is to be matched
		_bstr_t str(selectStr);	
		IXMLDOMNodeListPtr* pNodeListPtr = new IXMLDOMNodeListPtr;	
		*pNodeListPtr = pNodeInterface->selectNodes(str);
		CWLXmlNodeListWrapper* listWrapper = new CWLXmlNodeListWrapper(pNodeListPtr);
		return WLXmlSetNodeListInterfaceValue( cx, obj, listWrapper, rval);
	}
	CATCH_COM_METHOD(cx, "selectNodes")

}

JSBool
xmlnode_selectSingleNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 1)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "selectSingleNode", 1);
		if(!JSVAL_IS_STRING(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "selectSingleNode");
		IXMLDOMNodePtr pNodeInterface;
		if(!getNode(cx, obj, pNodeInterface))
			return JS_TRUE;
		char* selectStr = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));//XSL pattern that is to be matched
		_bstr_t str(selectStr);	
		IXMLDOMNodePtr* pSelectedNodePtr = new IXMLDOMNodePtr;	
		*pSelectedNodePtr = pNodeInterface->selectSingleNode(str);
		CWLXmlNodeWrapper* nodeWrapper = new CWLXmlNodeWrapper(pSelectedNodePtr);
		return WLXmlSetNodeInterfaceValue( cx, obj, nodeWrapper, rval);
	}
	CATCH_COM_METHOD(cx, "selectSingleNode")
}

JSBool
xmlnode_transformNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 1)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "transformNode", 1);
		if(!JSVAL_IS_OBJECT(argv[0]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "transformNode");
		IXMLDOMNodePtr pNodeInterface;
		if(!getNode(cx, obj, pNodeInterface))
			return JS_TRUE;
		//get the object
		JSObject* styleObj = JSVAL_TO_OBJECT(argv[0]);
		IXMLDOMNodePtr stylesheetNodePtr;
		if(!getNode(cx, styleObj, stylesheetNodePtr))
			return JS_TRUE;
		//transform the node 
		IXMLDOMNodePtr* pTransformedNodePtr = new IXMLDOMNodePtr;	
		_bstr_t str;
		str = pNodeInterface->transformNode(&(*stylesheetNodePtr));
		//prepare the object
		return WLXmlSetStringValue(cx, obj, NULL, (char*)str, rval);
	}
	CATCH_COM_METHOD(cx, "transformNode")
	
}

//Note: the Object could be a document interface or an Istream Interface - Msxml
//does not support the Istream interface so we have to limit ourselves to document only
//Should we support it.?
JSBool
xmlnode_transformNodeToObject(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  WlAssert(piHLErrorReporter);
	TRY_COM {
		if(argc != 2)
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 12, "transformNodeToObject", 1);
		if(!JSVAL_IS_OBJECT(argv[0]) || !JSVAL_IS_OBJECT(argv[1]))
			return piHLErrorReporter->ReportXMLError(cx, XML_ERROR_FILE, 13, "transformNodeToObject");
		IXMLDOMNodePtr pNodeInterface;
		if(!getNode(cx, obj, pNodeInterface))
			return JS_TRUE;
		//get the style sheetobject
		JSObject* tempObj = JSVAL_TO_OBJECT(argv[0]);
		IXMLDOMNodePtr stylesheetNodePtr;
		if(!getNode(cx, tempObj, stylesheetNodePtr))
			return JS_TRUE;
		//get the output object - only document interface produced by the user
		tempObj = JSVAL_TO_OBJECT(argv[1]); 
		CWLXmlPrivate* prv = (CWLXmlPrivate*)JS_GetPrivate(cx, tempObj);
		if(!prv)
			return JS_TRUE;
		//first check that the object at hand is a document interface produced by the user "new WLXmlDocument...."
		CWLXmlInterfaceParent* parent = (CWLXmlInterfaceParent*)(prv->GetObj());
		if(parent->GetType() != wlInterfaceDocumentType)
			return JS_TRUE;
		//next get the document interface ptr
		IXMLDOMDocumentPtr* pOutputDocumentNodePtr = 	((CWLXmlDocumentWrapper*)(prv->GetObj()))->GetInterfacePtr() ;
		IDispatch* pDocDisp;
		(*pOutputDocumentNodePtr)->QueryInterface(IID_IDispatch, (void **)&pDocDisp);	
		//produce a variant with the above doc interface
		_variant_t outputObject(/*pDisp*/pDocDisp);
		//complete the transformation
		pNodeInterface->transformNodeToObject(&(*stylesheetNodePtr), outputObject); 
		CWLXmlDocumentWrapper* docWrapper = new CWLXmlDocumentWrapper(pOutputDocumentNodePtr);
		return WLXmlSetDocumentInterfaceValue(cx, obj, docWrapper, rval);					
	}
	CATCH_COM_METHOD(cx, "transformNodeToObject")
}


//text methods
static JSBool
xmlnode_text_splitText(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {		
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmltext_splitText( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "splitText")
}

//characterdata methods
static JSBool
xmlnode_characterdata_appendData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlcharacterdata_appendData( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "appendData")
}

static JSBool
xmlnode_characterdata_deleteData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlcharacterdata_deleteData( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "deleteData")
}

static JSBool
xmlnode_characterdata_insertData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlcharacterdata_insertData( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "insertData")
}

static JSBool
xmlnode_characterdata_replaceData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlcharacterdata_replaceData( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "replaceData")
}

static JSBool
xmlnode_characterdata_substringData(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlcharacterdata_substringData( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "substringData")
}

//element method
static JSBool
xmlnode_element_getAttribute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlelement_getAttribute( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "getAttribute")
}

static JSBool
xmlnode_element_setAttribute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlelement_setAttribute( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "setAttribute")
}

static JSBool
xmlnode_element_removeAttribute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlelement_removeAttribute( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "removeAttribute")
}

static JSBool
xmlnode_element_getAttributeNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlelement_getAttributeNode( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "getAttributeNode")
}

static JSBool
xmlnode_element_setAttributeNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlelement_setAttributeNode( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "setAttributeNode")
}

static JSBool
xmlnode_element_removeAttributeNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlelement_removeAttributeNode( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "removeAttributeNode")
}

static JSBool
xmlnode_element_getElementsByTagName(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlelement_getElementsByTagName( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "getElementsByTagName")
}

static JSBool
xmlnode_element_normalize(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	TRY_COM {
		IXMLDOMNodePtr node;
		if(getNode(cx, obj, node))
			return xmlelement_normalize( cx, obj, argc, argv, rval, node);
		return JS_TRUE;
	}
	CATCH_COM_METHOD(cx, "normalize")
}



static void
xmlnode_finalize(JSContext *cx, JSObject *obj)
{
	WlTraceMsg("Inside xmlnode_finalize(%x, %x)", cx, obj);
	WLXmlFinalize(cx, obj);
}					

JSClass wl_XmlnodeClass = {
	WLJSNAME_CLASS_XML_NODE,
	JSCLASS_HAS_PRIVATE,	//ptr to msxml interface structure that info regarding the object it represents
	JS_PropertyStub,			//callback to C++
	JS_PropertyStub,			//--"--
	xmlnode_getProperty,
	xmlnode_setProperty,
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	xmlnode_finalize
};

static JSFunctionSpec node_methods[] = {
	//node methods
	{"appendChild", 								xmlnode_appendChild,														0},
	{"cloneNode",										xmlnode_cloneNode,															0},
	{"hasChildNodes",								xmlnode_hasChildNodes,													0},
	{"insertBefore",								xmlnode_insertBefore,														0},
	{"removeChild",									xmlnode_removeChild,														0},
	{"replaceChild",								xmlnode_replaceChild,														0},
	{"selectNodes",									xmlnode_selectNodes,														0},
	{"selectSingleNode",						xmlnode_selectSingleNode,												0},
	{"transformNode",								xmlnode_transformNode,													0},
	{"transformNodeToObject",				xmlnode_transformNodeToObject,									0},
	// methods
	{"splitText"										,xmlnode_text_splitText,												0},
	//characterdata methods
	{"appendData"										,xmlnode_characterdata_appendData,							0},
	{"deleteData"										,xmlnode_characterdata_deleteData,							0},
	{"insertData"										,xmlnode_characterdata_insertData,							0},
	{"replaceData"									,xmlnode_characterdata_replaceData,							0},
	{"substringData"								,xmlnode_characterdata_substringData,						0},
	//element methods
	{"substringData"								,xmlnode_element_getAttribute,									0},
	{"setAttribute"									,xmlnode_element_setAttribute,									0},
	{"removeAttribute"							,xmlnode_element_removeAttribute,								0},
	{"getAttributeNode"							,xmlnode_element_getAttributeNode,							0},
	{"setAttributeNode"							,xmlnode_element_setAttributeNode,							0},
	{"removeAttributeNode"					,xmlnode_element_removeAttributeNode,						0},
	{"getElementsByTagName"					,xmlnode_element_getElementsByTagName,					0},
	{"normalize"										,xmlnode_element_normalize,											0},
	
	{0}
};


//constructor
static JSBool
WLXmlNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	return JS_TRUE;
}

JSObject* WLNewXmlNodeObject(JSContext *cx, JSObject *obj)
{
	YieldRequest(cx);
	WlTraceMsg("WLNewXmlNodeObject(%x, %x)", cx, obj);
	// create an instance 
	JSObject* proto = NULL;
	JSObject* node = WLNewXmlObject(cx, &wl_XmlnodeClass, proto, obj);
	if (!node)
		return NULL;

	// define it's methods
//	if (!JS_DefineFunctions(cx, node, node_methods))
//		return NULL;
	return node;
}

bool DllExport WLAddXmlNodeClass(JSContext *cx, JSObject *obj)
{
	WlTraceMsg("WLAddXmlNodeClass(%x, %x)", cx, obj);
	if (!JS_InitClass(cx, obj, NULL, &wl_XmlnodeClass, NULL, 1, NULL, node_methods, NULL, NULL))
		return false;
	WlTraceMsg("WLAddXmlNodeClass - successful");
	return true;
}


