/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef _iSession_H
#define _iSession_H
#include <limits.h>
#include <RVCString.h>
#include "platform.h"
#include "generalprocess.h"
#include "wlleprotocolcontainer.h"
#include <ordered.h>

class DllExport iSession;
class cDll;


class iSession
{
public:

	explicit iSession( CGeneralProcess& );
	virtual ~iSession();

	CGeneralProcess& fGetProcess();
	
	double fGetBeginTime() {return vGetBeginTime();}

	double fGetTimeFromBegin();

	iLEProtocolContainer& GetLEProtocolSessionContainer();
	iLEProtocolContainer& GetLEProtocolProcessContainer();

  bool fInit(const char** pDisabledProtocols = NULL);
  void fFinal();

  virtual int GetErrorCounter() { return m_ErrorCounter;}
  virtual int GetWarningCounter() { return m_WarningCounter;}
  virtual void IncrementErrorCounter();
  virtual void IncrementWarningCounter(); 

	unsigned long GetCompletedRounds();
	void IncrementCompletedRounds();

private:
	CGeneralProcess& mProcess;

	virtual double vGetBeginTime() = 0;

	iLEProtocolContainer m_ProtocolSessionContainer;
	bool InitProtocols(const char** pDisabledProtocols = NULL);
  void ProtocolSessionTermination();

  void ExceptionHandler(std::exception*);

	RVMutexLock		m_ErrorsLock;
	int	m_ErrorCounter;
	int	m_WarningCounter;

	RVMutexLock		m_CompletedRoundsLock;
	unsigned long	m_CompletedRounds;
};


#endif

