/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_AGENDA_MANAGER_H
#define __WL_AGENDA_MANAGER_H

#include <collstr.h>

#include "associativearray.h"

#include "platform.h"
#include "tvpsync.h"
#include "slavecountmanager.h"
#include "loadagendastatisticsmanager.h"
#include "loadgeneratorslaveproxy.h"
#include "loadcopyfileobject.h"
#include "outputfile.h"
#include "loadinputfileinterface.h"
#include "wlloadgeneratorglobalshandler.h"


class CLoadGenerator;
class CLoadAgendaManager;

class CLoadAgendaCommandsManager
{
public:
  typedef enum {OK = 0, Error, Timeout, ErrorActivatingApp} ErrorReason;

private:
  bool            m_Status;
  long            m_Id;
  int             m_Count;
  RVMutexLock     m_Mutex;
  TVPManualEvent  m_Ready;
  long            m_Timeout;
  ErrorReason     m_Reason;

public:
  CLoadAgendaCommandsManager();
  ~CLoadAgendaCommandsManager();

  void SetTimeout(long timeout) {m_Timeout = timeout;};
  long Init(int numSlaves); // returns the Command ID.
  void Terminate();

  bool SlaveDone(int id, bool status);
  bool Cancel(ErrorReason reason = Error);

  bool Wait();

  ErrorReason Reason() {return m_Reason;};
};


class CLoadAgendaManager : public RVCollectableString {
  friend class CLoadGenerator;
  friend class CLoadGeneratorSlaveProxy;
  friend class CLoadAgendaStatisticsManager;
public:

private:
  CLoadGeneratorSlaveProxy     *m_firstLoadGeneratorSlaveProxy;
  RVOrdered                     m_SlaveProxyArray;
  CSlaveCountManager            m_SlaveCountManager;
  CLoadAgendaStatisticsManager  m_AgendaStatisticsManager;

  CLoadAgendaCommandsManager m_CommandsManager;

  CLoadGenerator*     m_pMyLoadGenerator;
  int                 m_NumThreads;
  int                 m_NumRunning;
  RVCString           m_Status;
	bool								m_AgendaPaused;
	bool								m_AgendaThrottled;
  bool                m_SevereErrorOccurred;
  int                 m_ScheduleNumRunning;
  int                 m_ThrottleNumRunning;
  int                 m_NumSlaves;
  long                m_MaxThreadsPerSlave;
	long								m_LGSlaveTimeout;

  RVCString												m_Name;
	RVCString												m_RealName;
	bool														m_OneclientFlag;
  RVCString												m_LoadAgendaMessageString;
  RVCString												m_StartSessionCommand;
  CLoadCopyFileObject							m_CopyFileObject;
  CWLOutputFileHandler						m_OutputFileHandler;
  CWLInputFileHandler						  m_InputFileHandler;
	CWLLoadGeneratorGlobalsHandler  m_LGGlobalsHandler;
  RVMutexLock							  			m_FatalErrorMutex;

public:
  CLoadAgendaManager(const char *name,long MaxThreadsPerSlave);
  virtual ~CLoadAgendaManager();

  bool setNumActive(int n);

  // agenda load functions.
  bool LoadAgenda(int size);
	bool LoadOneClientAgenda();

  bool StartSession(CAssociativeArray& StartSessionCommand);
  bool StartAgenda(int n);
  bool ThrottleAgenda(int n);
  bool UnThrottleAgenda();
  bool UserPauseAgenda();
  bool GoAgenda();
  bool StopAgenda();
  void ReportAgenda();
	bool StopSendingTransactions();
	bool StopSendingLogMessages();
 	bool IsAgendaPaused()           { return m_AgendaPaused; }

  
  bool reportLogMessageString(CAssociativeArray& ErrorMessage);
  bool reportLoadGeneratorMessage(CAssociativeArray& ErrorMessage);

  void SlaveFatalError();

  bool CopyMonitorFile(const char* sourcefilename, const char* targetfilename, const char* agendaPath, RVCString& errormessage);
  bool IncludeMonitorFile(const char* sourcefilename, const char** data, const char* agendaPath, RVCString& errormessage);
	void OutputFilePutMessage(CAssociativeArray* AssoArray) {m_OutputFileHandler.PutMessage(AssoArray);};
  CAssociativeArray* InputFileActions(CAssociativeArray& AssoArray) {return m_InputFileHandler.InputFileActions(AssoArray);};
	CLoadGeneratorSlaveProxy* GetFirstLoadGeneratorSlaveProxy()       {return m_firstLoadGeneratorSlaveProxy;};
	CWLLoadGeneratorGlobalsHandler *GetGlobalsHandler()               {return &m_LGGlobalsHandler;}
  CLoadGenerator* GetLoadGenerator() {return m_pMyLoadGenerator;};
  const char* Name() {return m_Name.data();};
	long GetMaxThreadsPerSlave() {return m_MaxThreadsPerSlave;};
	long LGSlaveTimeout() {return m_LGSlaveTimeout;};
protected:
	bool LoadNumThreads(int num);
};
/*
class CLoadAgendaManagerContainer : public RVHashDictionary {
public:
  CLoadAgendaManagerContainer() {}
  virtual ~CLoadAgendaManagerContainer() { clearAndDestroy(); }
 
  CLoadAgendaManager* AddThreadsManager(const char *name);
  CLoadAgendaManager* GetThreadsManager(const char* name) const;
  CLoadAgendaManager* operator [](const char* name) const { return GetThreadsManager(name); }
  void DeleteThreadsManager(const char *name);
};
*/
#endif //__WL_AGENDA_MANAGER_H

