/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __LoadEngineVariablesData_h
#define __LoadEngineVariablesData_h

class CWLJSScript;

#include "rvjsapi.h"
#include "platform.h"
#include <collstr.h>
#include <RVCString.h>
#include <hashdict.h>
#include <thr.h>

class CLEPropertyAttributes;
class CLEPropertyValue;
#define PROP_SYNTAX_CHECK_NAME "UI_ONLY_DOSYNTAXCHECK"


////////////////////////////////////////////////////////////////////
//  the declaration of class CLEPropWorkSpace   
////////////////////////////////////////////////////////////////////

class DllExport CLEPropWorkSpace {
public :
	explicit CLEPropWorkSpace();
	~CLEPropWorkSpace();
	void ReadData(CWLJSScript&);//will read the LE property from the global 
									// variables vector. And will set the the
									//instance's data members

	static long GetTimeoutBeforeOperation();

private:
  void prepareClientsStatisticsFilter();

public :
	//the value of the properties
	bool m_DisableSleep;
	long m_SleepRandomMin;
	long m_SleepRandomMax;
	long m_SleepDeviation;
	bool m_ExceptionEnabled;

	bool m_ReportCSJSInternalErrors;

	static long m_JTimeoutBeforeOperation;

	bool m_SendClientStatistics;
	RVCString m_SendClientStatisticsFilter;
	RVCString m_JVMType;
	int		  m_CharEncoding;

};




//
// class for handling variables mechanism (i.e.  settings for wlGlobals , 
// wlLOcals, wlHttp).

class DllExport CLEPropertyValue {
public:
  enum Level {PL_GLOBALS=0, PL_LOCALS =1, PL_PRIVATE=2} ;
  enum PropertyType {PT_INT, PT_FLOAT, PT_STRING, PT_BOOLEAN, PT_BOOLEAN_STR, PT_OBJECT} ;

  CLEPropertyValue() ;
  CLEPropertyValue( const long Value) ;  
  CLEPropertyValue( const float Value) ;  
  CLEPropertyValue( const char* Value) ;  
  CLEPropertyValue( const bool Value, PropertyType PropType = PT_BOOLEAN) ;  
  ~CLEPropertyValue() ;
  CLEPropertyValue (const CLEPropertyValue & other) ; //Copy constrauctor
  CLEPropertyValue & operator=(const CLEPropertyValue & Other) ; 


  //type declaration for the Variables array, can be used to define local variables array
  typedef CLEPropertyValue* tVariablesArray ;
  
  bool m_IsSet ;
  PropertyType m_ValueType ;
  union ValueRecType {
    long  LongVal ;
    float FloatVal ;
    char* StringVal ;
    bool  BoolVal ;
  } m_Value ;

  CLEPropertyValue & Set ( long Value) ;  
  CLEPropertyValue & Set ( float Value) ;  
  CLEPropertyValue & Set ( char* Value) ;  
  CLEPropertyValue & Set ( bool Value, PropertyType PropType = PT_BOOLEAN) ;  
};

class DllExport CLEVariablesData {
public:

	typedef enum {OK = 0, WrongLevel, PropertyNotFound} SyntaxCheckStatus;

  CLEVariablesData(CLEVariablesData* Parent, CLEPropertyValue::Level Level); 
  CLEVariablesData (const CLEVariablesData & other) ; //Copy constrauctor
  ~CLEVariablesData() ; 
  
  //Since the VariableDictionary class is not exported through the DLL
  //this is a wrapper function for the registration option. 
  //Returns the index number of the new definition.
  static int RegisterProperty 
    ( char*                            Name, 
      CLEPropertyValue::PropertyType   Type,
      const CLEPropertyValue &         DefaultValue,
      char*                            LegalListOfValues,
      CLEPropertyValue::Level          StartLevel = CLEPropertyValue::PL_GLOBALS,
      CLEPropertyValue::Level          EndLevel = CLEPropertyValue::PL_PRIVATE);

	static void RegisterObjectProperty
  ( char*                             Name,
    CLEPropertyValue::PropertyType    Type);

	static RVCString MakeKeyToPropertiesDictionary(const RVCString & objPropertyName,const RVCString & propertyName );

  //Reset the variables array to its default values.
  void ResetVariablesArray () ;                                   
  void Set ( int Index, const CLEPropertyValue & Val) ;           
  void Set ( char* Name, const CLEPropertyValue & Val) ;          
  void InitializeArrays() ;                                       

  const CLEPropertyValue & Get ( int Index, bool HierarchySearch = true) const ;
  const CLEPropertyValue & Get ( char* Name, bool HierarchySearch = true) const ;
  
  //These function are registered in the Java script for the wlGlobals and wlLocals
  static JSBool JsSet (JSContext* cx, JSObject* obj, jsval id, jsval* vp) ;
  static JSBool JsGet (JSContext* cx, JSObject* obj, jsval id, jsval* vp) ;
  //These function are used by the jsSet & jsGet of the wlHttp
  JSBool JsVarSet (JSContext* cx, JSObject* obj, jsval id, jsval* vp) ;
  JSBool JsVarGet (JSContext* cx, JSObject* obj, jsval id, jsval* vp) ;
  CLEPropertyValue::Level GetVariablesLevel () {return m_VariablesLevel ;}

 
	//These function are registered in the Java script for the wlPropertyObject_class
  static JSBool JsSetPropertyObj (JSContext* cx, JSObject* obj, jsval id, jsval* vp) ;
  static JSBool JsGetPropertyObj (JSContext* cx, JSObject* obj, jsval id, jsval* vp);

	//this function is used by JsGetPropertyObj  
	JSBool CLEVariablesData::JsVarGetPropertyObj (JSContext* cx, JSObject* obj, jsval id, jsval* vp, RVCString & objPropName);
	JSBool CLEVariablesData::JsVarSetPropertyObj (JSContext* cx, JSObject* obj, jsval id, jsval* vp, RVCString & objPropName );

  //This method should be used only by the Load Engine!!!
  static void CreateDefaultValuesArrays() ; 
  static void SetDoSyntaxChecking (JSContext* cx, bool Mode) ; 
private:
  CLEPropertyValue::Level           m_VariablesLevel ;
  CLEPropertyValue::tVariablesArray m_ValuesArray;
  int                               m_ValuesArrayLength ;
  CLEVariablesData*                 m_Parent;
  static bool                       m_DoSyntaxChecking;
  static bool                       m_DoSyntaxCheckingIsSet;


  bool JsValueToPropertyValue 
      ( JSContext *Cx, 
        jsval & JsValue, 
        const CLEPropertyAttributes & PropAttr, 
        CLEPropertyValue & PropVal ) ; 
  bool PropertyValueToJsValue 
      ( JSContext *Cx, 
        jsval & JsValue, 
        const CLEPropertyAttributes & PropAttr, 
        const CLEPropertyValue & PropVal ) ; 
  SyntaxCheckStatus SyntaxCheck (JSContext* cx, JSObject* obj, jsval id, jsval* vp, 
									bool SetMethod, CLEPropertyAttributes* & PropAttr) ;
  static void MessageReport (JSContext* Cx, int ErrorId, ...) ;
};

#endif //__LoadEngineVariablesData_h

