/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// LoadEngineVerification.h: interface for the CWLTransactionManager class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_LOADENGINEVERIFICATIONCLASSES_H__C59C4073_136B_11D3_B5DC_00A0C9458D0D__INCLUDED_)
#define AFX_LOADENGINEVERIFICATIONCLASSES_H__C59C4073_136B_11D3_B5DC_00A0C9458D0D__INCLUDED_

#include <collstr.h>
#include <hashdict.h>
#include <ordered.h>

#include "wlerrorlib.h"

void PrintTransactionTree(const char* input);

class CLoadTransaction;
class CLoadTransactionEvent;
class IGeneralThreadTransactionManager;
class cSCThreadTransactionManager;
class CLoadThreadTransactionManager;


class CLoadTransactionEvent : public RVCollectable
{
  friend class CLoadTransaction;
  friend class IGeneralThreadTransactionManager;
	friend class cSCThreadTransactionManager;
	friend class CLoadThreadTransactionManager;

private:
  RVCString   m_Name;
  RVCString   m_Description;
  double      m_Time;
  long        m_LineNumber;
  RVCString   m_FileName;
  RVCString   m_ExtraDataType;
  RVCString   m_ExtraData;
  WLLogMessage::Severity m_Severity;

  RVCString   m_ToString;

public:
	CLoadTransactionEvent(const char* name, const char* description, WLLogMessage::Severity severity, const char* extraDataType, void* extraData, unsigned long len);
	virtual ~CLoadTransactionEvent();

	void SetScriptData(double time, const char* fileName, int lineNumber);

  const char* toString();
};

class CLoadTransaction : public RVCollectableString
{
public:
  typedef enum {Successful, Marked, Failed} Status;
  static const char* GetStatusName(Status status);

private:
  RVCString     m_ToString;
  RVOrdered     m_EventsList;

public:
  long          m_InstanceID;
  long          m_ParentID;
	long          m_TopInstanceID;        // Eyal 28.03.00
  long          m_Level;
  RVCString     m_TransactionName;
  RVCString     m_Reason;
  double        m_Time;
  double        m_Duration;
  long          m_LoadSize;
	RVCString     m_ExtraDataType;    // Eyal 28.03.00
  RVCString     m_ExtraData;        // Eyal 28.03.00
  WLLogMessage::Severity m_Severity;
  bool          m_Ending;
  Status        m_Status;

  IGeneralThreadTransactionManager *m_MyManager;
  
public:
	CLoadTransaction(const char* name, IGeneralThreadTransactionManager *manager);
	virtual ~CLoadTransaction();

  CLoadTransactionEvent* RecordEvent(const char* name, const char* description, double time, WLLogMessage::Severity severity, const char* extraDataType, void* extraData, unsigned long len);
  const char* toString();
	bool End(WLLogMessage::Severity severity, double time, bool save, bool SendTimers );
};


#endif // !defined(AFX_LOADENGINEVERIFICATIONCLASSES_H__C59C4073_136B_11D3_B5DC_00A0C9458D0D__INCLUDED_)

