/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_LOADGENERATOR_H
#define __WL_LOADGENERATOR_H

#include "platform.h"
#include "tvpeventqueue.h"
#include "associativearray.h"
#include "loadenginestatistics.h"
#include "loadgeneratorcommunicator.h"
#include "wlloadgeneratorglobalshandler.h"

#include "nvsnames.h"


CAssociativeArray* LGStandAlone(CLoadGenerator* lg);

class CLoadAgendaManager;
class CGeneralProcess;

class CLoadGenerator
{
friend class CLoadGeneratorSlaveProxy;

public:
  typedef enum {Regular, StandAlone, Developer} GeneratorType;
  bool      m_ThreadPoolOn;

private:
	const char*	GetCommand();
		
	RVCString	m_MonitorName;
	RVCString	m_LoaderType;
	RVCString	m_RealName;
  RVCString m_LoadGeneratorNo;
  RVCString m_SessionNo;
	RVCString	m_LoaderStatus;
  RVCString m_MonitorUID;
	double    m_StartTime;
  bool      m_Terminated;
  bool      m_StopMeDone;

  CLoadGeneratorCommunicator    m_MessagesCommunicator;
  TVPEventQueue                 m_ControlMessageQueue;

  RVCString                     m_OneClientThreadsManagerName;
  CLoadAgendaManager           *m_AgendaManager;

  //
  // Variables for eliminating sending Messages, Transactions 
  // and Statistical Measurements.
  bool            m_SendingLogMessages;
  int             m_LogMessagesCounter;
  int             m_LogMessagesMax;
	WLLogMessage::Severity
									m_MustLogBaseSeverity;
  bool            m_SendingTransactions;
  int             m_TransactionsCounter;
  int             m_TransactionsMax;
  int             m_StatisticalMesurementsMask;
	
	double					m_StopAllTime;

	int m_MasterPid;

public:
  GeneratorType   m_GeneratorType;
	RVCString	      m_MonitorGlobalVariablesAddress; 
	bool			m_firstcycle;

public:

	CLoadGenerator( const char* MonitorName,
					        const char* LoaderType,
					        const char* RealName,
					        const char* LoadGeneratorNo );

	CLoadGenerator( const char* ScriptFile, int ReportResolution = 0);

  ~CLoadGenerator();

	void MainLoop();
	void ExceptionHandler(std::exception*);
  void StopMe();
	void ForceKillLoadGenerator();

	const char*	WhoAmI();
	CLoadAgendaManager*	GetAgendaManager( const char* ThreadsManagerName );
	CLoadAgendaManager*	GetAgendaManager();
  
  //
  // Communicator functions

	// Send message to specified address:
	void  SendMessageToAddress(CAssociativeArray& Message, const char* netAddress, int priority);
	// Send message to monitor only(defualt):
	void	SendMessage( CAssociativeArray& Message, int priority );
  void  PutMessage( CAssociativeArray* Message ) { m_ControlMessageQueue.putEvent(Message); }
  CAssociativeArray* GetMessage();
  
  //
  // 
	void	ReportLoadGeneratorMessage( const char* Status );

  //
  // LoadGenerator level commands
	void	StartSession( double StartTime, CAssociativeArray& AssoArrayCommand);
	void	StopSession();
	
	void	PauseLoader();
	void	GoLoader();

	void	SetStatus( const char* Status )	{ m_LoaderStatus = Status; }
	const char*	GetStatus()	{ return m_LoaderStatus; }

  //
  // Generator/probing client commands.
	void	LoadThreadsManager(long ThreadsManagerSize, 
                           const char* ThreadsManagerName, 
 													 const char* ThreadsManagerRealName,
                           CAssociativeArray&	ScriptsArray); 

	void	LoadOneClientThreadsManager(const char* ThreadsManagerName, 
																		const char* ThreadsManagerRealName,
                                    CAssociativeArray& ScriptsArray); 

	void	StartThreadsManager( const char*   ThreadsManagerName, int NumRunning );

	void	ThrottleThreadsManager( const char* ThreadsManagerName, int	NumRunning );
	void	UnThrottleThreadsManager( const char* ThreadsManagerName );

	void	PauseThreadsManager( const char* ThreadsManagerName );
	void	UserPauseThreadsManager( const char* ThreadsManagerName );
	void	GoThreadsManager( const char* ThreadsManagerName );
	void	StopThreadsManager( const char* ThreadsManagerName );

  double GetTimeFromBeginning();
  double GetBeginningTime();
  const char* GetLoadGeneratorNo() {return m_LoadGeneratorNo.data();}

	RVCString& GetMonitorName() { return m_MonitorName; }
	RVCString& GetMonitorUID() { return m_MonitorUID; }

  //
  // Functions for eliminating sending Messages, Transactions 
  // and Statistical Measurements.
  bool IsSendLogMessage();
  void SetSendLogMessage(bool send);
  void AddLogMessage();
  bool IsSendTransactions();
  void AddTransactions(int count);
  bool IsSendStatisticalMesurement(CWLStatPointTypeEnum::PointType type);
	void StopSendingLogMessages();
	void StopSendingTransactions();
	long CalcMaxThreadsPerSlave(CAssociativeArray& AssoArray);

	double GetStopAllTime() const { return m_StopAllTime; }

	int GetMasterPid() { return m_MasterPid; }

	WLLogMessage::Severity GetMustLogBaseSeverity() { return m_MustLogBaseSeverity; }

};

#endif

