/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef __WL_LOADGENERATORSLAVE_H
#define __WL_LOADGENERATORSLAVE_H
#include <collint.h>

#include "platform.h"
#include "associativearray.h"
#include "netkernelmessagehandler.h"
#include "loadenginestatistics.h"
#include "wldlls.h"
#include "wljsobjects.h"
#include "loadjsgcthread.h"
#include "isession.h"
#include "wljsjavaapplet.h"
#include "rvjsmachinethread.h"

class CLoadGeneratorSlave;
class CLoadGeneratorSlaveProxy;
class CLoadThreadsManager;

class CLoadGeneratorSlaveCommunicator : public CNetMessageHandler
{
private:
  CLoadGeneratorSlave *m_pMyLoadGeneratorSlave;

public:
	//CLoadGeneratorSlaveCommunicator(CLoadGeneratorSlave *pLG, const char* name);
	CLoadGeneratorSlaveCommunicator(const char* name);
	~CLoadGeneratorSlaveCommunicator();

	void InitParent(CLoadGeneratorSlave *pLG);

	virtual void SendMessageErrorHandler( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, bool status);
	virtual void MessageHandler( CAssociativeArray& AssoArrayCommand );
};

class CGeneralProcess;
class iLEProtocolProcess;
class iLEProtocolSession;

class CLoadGeneratorSlave: public iSession , public iLEJSAgendaMachineThread  {
public:

	CGeneralProcess& fGetProcess() { return iSession::fGetProcess(); }
	
	typedef enum {Regular, InProcess, StandAlone} GeneratorType;
  bool      m_ThreadPoolOn;

private:
	const char*	GetCommand();
		
	RVThread  m_MainLoopThread;

  RVCString m_LoadGeneratorName;
  RVCString m_LoadGeneratorNo;
  RVCString m_ProxyAddress;
  int			  m_StatisticsFrequency;
  double    m_StartTime;
  bool      m_Terminated;
  bool      m_StopMeDone;
  long      m_CurrentRequestId;

  CLoadGeneratorSlaveProxy        *m_MyProxy;
  CLoadGeneratorSlaveCommunicator m_MessagesCommunicator;
  TVPEventQueue                   m_ControlMessageQueue;

  CLoadThreadsManager             *m_MyThreadsManager;
	
  // The Java stream hash table.
  CWLJavaStreamTable  m_JavaStreamTable;

  //
  // Variables for eliminating sending Messages, Transactions 
  // and Statistical Measurements.
  bool            m_SendingLogMessages;
  int             m_LogMessagesCounter;
  int             m_LogMessagesMax;
	WLLogMessage::Severity
									m_MustLogBaseSeverity;
  bool            m_SendingTransactions;
  int             m_TransactionsCounter;
  int             m_TransactionsMax;
  int             m_StatisticalMesurementsMask;

	int							m_MasterPID;

	RVCString				m_MonitorUID;
	
  // @@ChaimTBD - at some point this should be moved to the loadgenerator level
	cLEJSGCThread   mGCThread;
  RVThread        m_JSGCThread;
  TVPManualEvent  m_JSGCTerminateEvent;

public:
  GeneratorType   m_GeneratorType;
  int             m_SlaveBaseThreadId;
	RVCString	      m_MonitorGlobalVariablesAddress; 
	RVCString				m_MonitorAddress;
	bool						m_MonitorAddressInit;

public:
  CLoadGeneratorSlave( const char* ProxyAddress,
                       const char* SlaveNo,
											 CGeneralProcess& rProcess);

  CLoadGeneratorSlave( CLoadGeneratorSlaveProxy* myProxy, 
                       const char* SlaveNo,
											 CGeneralProcess&						rProcess);
											 
  ~CLoadGeneratorSlave();

  void Start();
	void Terminate();
	void MainLoop();
	void ExceptionHandler(std::exception*);
  void StopMe();
	void ForceKillLoadGenerator();
	const char*	WhoAmI();
	
  //
  // Communicator functions

	// Send message to specified address:
	void  SendMessageToAddress( CAssociativeArray& Message, const char* netAddress, int priority);
	// Send message to slave proxy only:
	void	SendMessage( CAssociativeArray& Message, int priority );

	// Send message directly to monitor
	void SendMessageToMonitor( CAssociativeArray& Message, int priority ){
		WlAssert(m_MonitorAddressInit);
		SendMessageToAddress(Message, m_MonitorAddress, priority);
	}

	void InitMonitorAddress(const char* pMonitorName){
		m_MonitorAddressInit = true;
		m_MonitorAddress = pMonitorName;
	}

  void  PutMessage( CAssociativeArray* Message ) { m_ControlMessageQueue.putEvent(Message); }
  CAssociativeArray* GetMessage();
  
  //
  // 
	void	ReportLoadGeneratorSlaveStatus( const char* MessageName, long status, long priority = 1 );

  //
  // Generator/probing client commands.
	void	LoadThreadsManager(bool oneClient, CAssociativeArray& ScriptsArray); 

	void	StartThreadsManager( const char*   ThreadsManagerName,
            								 int				   NumRunning );

	void	StopThreadsManager( const char*   ThreadsManagerName );

  double vGetBeginTime();
	int GetStatisticsFrequency() { return m_StatisticsFrequency; }

  //
  // Functions for eliminating sending Messages, Transactions 
  // and Statistical Measurements.
  bool IsSendLogMessage();
  void SetSendLogMessage(bool send);
  void AddLogMessage();
  bool IsSendTransactions();
  void AddTransactions(int count);
  bool IsSendStatisticalMesurement(CWLStatPointTypeEnum::PointType type);

  const char* GetLoadGeneratorNo() {return m_LoadGeneratorNo.data();}

  CLoadThreadsManager* GetThreadsManager() {return m_MyThreadsManager;};

	int GetMasterPID() { return m_MasterPID; }

  bool AddJavaStream(JSContext*  cx, 
                     const char* streamName,
                     const char* codeBase,
                     RVCString&  javaStream);

  bool JavaStreamExists(const char* streamName);

  void JSGCFunc();

	WLLogMessage::Severity GetMustLogBaseSeverity() { return m_MustLogBaseSeverity; }

	cLEJSGCThread& GetJSGCThread() {return mGCThread;}

  CLoadGeneratorSlaveProxy* GetLoadGeneratorSlaveProxy() { return m_MyProxy; }

};



#endif

