/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __WL_THREADS_MANAGER_H
#define __WL_THREADS_MANAGER_H

#include <collstr.h>

#include "platform.h"
#include "tvpsync.h"
#include "loadgeneratorscript.h"
#include "loadthreadinstance.h"
#include "slaveloadsizemanager.h"

class CLoadGeneratorSlave;
class CThreadsManagerGlobalsRequestHandler;

class DllExport IThreadsManager {

public:

	virtual void reportLogCurrent(WLLogMessage::Severity, const char* );
	virtual void reportLog(iReport&, int threadNum = -1) = 0;
  virtual const char* GetAgendaName() = 0;
  virtual const char* GetLoadGeneratorNo() = 0;
	};


class CLoadThreadsManager:
	public IThreadsManager {
  friend class CLoadThreadsManagerScript;
  friend class CLoadThreadsManagerScriptContainer;
  friend class CLoadThreadInstance;
  friend class CLoadGeneratorSlave;
private:
  CLoadThreadsManagerScriptContainer    m_ScriptManager;
	CThreadsManagerGlobalsRequestHandler* m_ThreadsManagerGlobalsHandler;

  CLoadThreadsVector  m_ThreadsVec;
  
  RVCriticalSection   m_TreatedThreadsMutex;
  TVPManualEvent      m_TreatedThreadsEvent;
  unsigned long       m_TreatedThreadsCounter;
  bool                m_TreatedThreadsOn;

  TVPManualEvent      m_ThreadsStartEvent;
  TVPManualEvent      m_ThreadsRestartEvent;

  bool                m_Terminate;
  long                m_ActiveThreads;

  bool                m_SendStatisticTimers;

//public:
  TVPSafeCounter      m_AliveThreadsCounter;
  
  CLoadGeneratorSlave *m_pMyLoadGeneratorSlave;

	TVPSafeFlag         m_ErrorFlag;
	RVMutexLock					*m_SynchronizationPointMutex;
  bool                m_EventIsTerminating;
  int                 m_InitialThreadId;
  int                 m_NumRunning;
  int                 m_MixNumRunning;

  RVCString           m_Name;     // Agenda unique name: a1::generator1@host
	RVCString						m_RealName; // Agenda file path: *.js or *.mix 
	bool                m_OneclientFlag;

private:
	bool LoadNumThreads(int num);
  
  bool SetNumThreads(int size);
  void IncrementTreatedThreads();
  void WaitToRestart() {m_ThreadsRestartEvent.Wait();}
  void WaitToStart() {m_ThreadsStartEvent.Wait();}

public:
  CLoadThreadsManager(const char *name);
  virtual ~CLoadThreadsManager();

	void Init(const char *name, const char *realName, CLoadGeneratorSlave *slave, bool oneClientFlag);
  //
  // Functions used by the the LoadGenerator Slave.
  bool LoadScript(const char* scriptName, const char* scriptFileName, const char* scriptCode, double weight, const char* InitializationScript, int NumScriptInMix);
  void StartLoad(double startTime, int sliceSize);
  bool setNumActive(int n);
  bool StopAgenda();

  //
  // Functions Used from Threads & Scripts.
  bool IsTerminate() {return m_Terminate;}
  bool IsOneClient();
  bool IsLoadServer();
  void OnSevereError();
  int  GetAgendaNumRunning();
  const char* GetAgendaName()      {return m_Name.data();}
	const char* GetAgendaPath()      {return m_RealName.data();}
  const char* GetLoadGeneratorNo();
  virtual int GetErrorCounter();
  virtual int GetWarningCounter();
  virtual void IncrementErrorCounter();
  virtual void IncrementWarningCounter();
  void reportLog( iReport&, int threadNum = -1 );
	CLoadThreadsManagerScriptContainer& GetScriptManager() {return m_ScriptManager; }
	CLoadGeneratorSlave* GetLoadGeneratorSlave()           {return m_pMyLoadGeneratorSlave;}
	CLoadThreadInstance* GetThread(size_t threadID)        {return m_ThreadsVec[threadID];}
  void setNumRunning(int num) {m_NumRunning = num;}
  void setMyLoadGeneratorSlave(CLoadGeneratorSlave* lgs) {m_pMyLoadGeneratorSlave = lgs;}
  void InputFileData(CAssociativeArray* AssoArrayCommand);
};

#endif //__WL_THREADS_MANAGER_H

