/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef __WL_SINGLECLIENT_H
#define __WL_SINGLECLIENT_H

#include "wlerrorlib.h"
//#include "loadjsshell.h"
#include "loadthreadinstance.h"
#include "managerscriptdata.h"
#include "wlglobalstable.h"
#include "loadthreadsmanager.h"
#include "rvjsmachinethread.h"
#include "rvjs.h"
#include "agendafile.h"

#include "wlleprotocol.h"

class iLEProtocolContainer;
class CloadThreadInstanceScript;

class CWLJSScript;
class CWLJSThread;

class DllExport cSingleClientProcess;
class DllExport iSingleClientMachineThread;
class DllExport cSingleClientMachineThread;
class DllExport iLoadEmbedderReportInfo;
class DllExport iSCActionReportInfo;
class DllExport iLoadSingleClient;
class DllExport iLoadProbingClient;
class DllExport iLoadSAFHeaders;

class DllExport cSCGlobalsRequestHandler;
class DllExport cSCThreadTransactionManager;
class DllExport cSCThreadsManager;
class DllExport cSCStatClient;
class DllExport cSCManagerScriptData;

//####### The declaration of the class cSingleClientProcess  #########

class cSingleClientProcess: public CGeneralProcess {
	friend class iLoadSingleClient;
public:
	explicit cSingleClientProcess();
	~cSingleClientProcess();

	bool fInit();
	void fTerminate();

private:
	RVCString m_ErrorOfProtocols;
	bool m_ExistErrorOfProtocols;
	
};


class iSingleClientMachineThread: public iLEJSAgendaMachineThread
{
public:
	explicit iSingleClientMachineThread();
	~iSingleClientMachineThread();
};

class cSingleClientMachineThread: public iSingleClientMachineThread
{
public:
	explicit cSingleClientMachineThread();
};


enum tSCActionReporterReturn { sSCActionReporterReturnContinue, sSCActionReporterReturnDM };

class iSCActionReportInfo:
	public virtual iDebugExecReportInfo,
	public virtual iActionReportInfo 
{
public:
	explicit iSCActionReportInfo();
	~iSCActionReportInfo();

	void fReport( JSContext* );

	virtual void fApply( tSCActionReporterReturn&, iLoadSingleClient&, iReportThreadState* ) = 0;

	tDebugReporterReturn vApplyDebug( cJSDReportCommit& );
	void vApplyJSExec( iJSExecReportCommit& );
	bool vIsStop() const;

	void vApplyHard( iLEJSEmbedContext& );

private:
	bool mDoneApply;
	};



class cSCMainFileCommit:
	public iFileSourceCommit {

public:
	explicit cSCMainFileCommit( iAgendaFile&, CWLJSScript& );

	bool fGetStatus();


private:
	void fTakeFile( const char*, int, const char* );

	CWLJSScript& mScript;
	bool mStatus;
	};


class cSCFileCommit: public iFileSourceCommit {

public:
	explicit cSCFileCommit( iAgendaFile&, iLoadSingleClient&, bool );
	void fTakeFile( const char*, int, const char* );
	bool fGetStatus() const;

private:
	iLoadSingleClient& mSC;
	bool mExecute;
	bool mStatus;
	};



class cSCGlobalsRequestHandler: public IGeneralGlobalsRequestHandler {

public:
	void Globals_Set(const char* variableName,
									 CGlobalVariableProperties *newProperties,
									 Level level);

	void Globals_Add(const char* variableName, 
									 CGlobalVariableProperties *newProperties,
									 Level level);

	CGlobalVariableProperties*  Globals_SyncAdd(const char* variableName, 
																						  CGlobalVariableProperties *newProperties,
																						  Level level);

	CGlobalVariableProperties *Globals_Get(const char* variableName,
																				 Level level);


private:
	CWLGlobalsTable mGlobalsTable;
	static const char msObjectName[];

	};



class cSCThreadTransactionManager: public IGeneralThreadTransactionManager {

public:
	explicit cSCThreadTransactionManager();
	void InitParent(iLoadSingleClient&);

private:
	iLoadSingleClient* mSC;
	long               mTransactionEventID; 


	CLoadGeneralThreadInstance* GetThreadInstance();
	void SendWhole();
  void SendEvent( CLoadTransactionEvent& );

	virtual void RecordEventCallBack(CLoadTransactionEvent&, long topInstanceID, long instanceID );     
  virtual void BeginTransactionCallBack(CLoadTransaction& );
  virtual void EndTransactionCallBack(CLoadTransaction& );  

  void AddLast(const char*, double, CWLStatPointTypeEnum::PointType);
  bool IsSendTransactions();
  double vGetTimeFromBegin();
	double vGetBeginTime();
	double vGetCurrentTime();
	};


class cSCThreadsManager: public IThreadsManager 
{

public:
	explicit cSCThreadsManager();
	void InitParent(iLoadSingleClient&);

private:
	iLoadSingleClient* mSC;

	void reportLog(iReport&, int threadNum = -1);
  const char* GetAgendaName();
  const char* GetLoadGeneratorNo();
  virtual int GetErrorCounter();
  virtual int GetWarningCounter();
  virtual void IncrementErrorCounter();
  virtual void IncrementWarningCounter();
};


class cSCStatClient
{

private:
	CWLStatGenMeasurementDictionary mDictionary;
	CWLStatGenTimerDictionary mTimingDictionary;
	long mClientId;

public:

	explicit cSCStatClient();
	virtual ~cSCStatClient();

//	RVCollectableInt fGetClientId() {return clientId;}

//  double fGetCurrentEndTime();

	void fAddLast(const char *measurement, double v, CWLStatPointTypeEnum::PointType type);
	void fResetTimer(const char *name);
  void fSendTimer(const char *name);


	CWLStatGenMeasurementDictionary& fGetMeasurmentDictionary() {return mDictionary;}

//  void fGetNewDictionary();
//  void fReleaseDictionary();

};


class cSCManagerScriptData:
	public IGeneralManagerScriptData
{
public:
	explicit cSCManagerScriptData();
	void InitParent(iLoadSingleClient&);

	~cSCManagerScriptData();

	CSCFileDictionary& GetInputFiles();

	virtual int GetNumScriptInMix() ;

  RVCString& ScriptName();
	RVCString& ScriptFileName();

  // iLEAgendaJSEmbedContext
	CWLJSScript& GetJSScript();
  iLEJSMachineThread& vGetMachineThread();
  
  virtual int GetErrorCounter();
  virtual int GetWarningCounter();
  virtual void IncrementErrorCounter();
  virtual void IncrementWarningCounter();
  
private:
	iLoadSingleClient* mSC;


	cSCGlobalsRequestHandler mGlobalsRequestHandler;
	CSCFileDictionary mInputFiles;
	
  // IGeneralManagerScriptData 
  bool CopyFile(const char*, const char* );
  bool vIncludeFile_IGeneralManagerScriptData(const char*, bool );
	
  bool InInitAgenda();
	bool IsSpawn();

  IFileDictionary* GetFileDict();
	void AppendStringForEvaluation(const char*,CWLJSScript::WhenToRunScript whenToRunScript);

	void SendMessage( CAssociativeArray&, int );

	
	void reportLog( iReport& );
	void OnSevereError(); 

	CGeneralProcess& fGetProcess();
	iSession& vGetSession();

	void vEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType );
	void vExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType );

	bool vIsJSGC();
	
	bool IsJavaThreadWrapperCreationScriptExists();
	void SetJavaThreadWrapperCreationScript();

  // iLEAgendaJSEmbedContext
  CWLJSBaseObject& GetJSBaseObject();

public:
	IGeneralGlobalsRequestHandler* GetGlobalsRequestHandler();

//	void fActionReportCommit( iActionReportInfo& );

};


class iLoadSingleClient:
	public iSpaceUi,
	public iSession
{

	friend class cSCFileCommit;
	friend class cMainSCFileCommit;
	friend class cSCThreadTransactionManager;
	friend class CRTAMThreadInstance;
	friend class cSCThreadsManager;
	friend class cSCManagerScriptData;

public:	

  cSCManagerScriptData mManagerScriptData;

	explicit iLoadSingleClient( cSingleClientProcess&, bool IsDebuggerActive );
	virtual ~iLoadSingleClient();

	void fInit( iSingleClientMachineThread&, const char** pDisabledProtocol = NULL);
	bool fExecuteProtoInit( const char*, iAgendaFile&, const char*, bool& exceptionOccured );
	bool fExecuteThread( int, unsigned long );
	void fExecuteProtoTerminate( bool );
	void fFinal();
	bool fIsAbort() const;
	unsigned long fRoundNum() const;
	void OnSevereError();

	virtual void fExecReporter( iReportThreadState*, iReportInfo& ) = 0;

	virtual void fRecordEvent(long eventID,
			  										long topInstanceID,
			  										long instanceID,
														WLLogMessage::Severity severity,
														const char* eventName,
														double time,
														long lineNumber,
														const char* fileName,
														const char* description, 
														const char* extraDataType,
														const char* extraData) = 0;

	virtual void fBeginTransaction(long instanceID, 
																 const char* transactionName, 
																 const char* transactionType) = 0;

	virtual void fEndTransaction(long topInstanceID, 
															 long instanceID, 
															 long parentID, 
															 unsigned long roundNum,
															 const char* reason,
															 const char* transactionName,
															 double time, 
															 WLLogMessage::Severity severity,
															 const char* transactionType, 
															 const char* extraData) = 0;


	virtual bool fBeforeSleep( long int Time );
	virtual void fAfterSleep();

	void fReportApply(iReport&);

private:

	iSingleClientMachineThread* mMThread;
	CRTAMThreadInstance *m_ThreadInstance;

  CRTAMThreadInstanceScript *m_ThreadInstanceScript;
	cSCThreadTransactionManager mTransactionManager;
	cSCThreadsManager mThreadsManager;

	bool mInitAgendaRun;
	bool mIsAbort;
	unsigned long mRoundNum;

	void vAbort_iDebugger();

	RVCString m_AgendaName;


	virtual void fBeforeInitAgenda( iInterruptionControlCommit& ) = 0;
	virtual void fBeforeTerminateAgenda( iInterruptionControlCommit& ) = 0;
	virtual void fBeforeOnErrorTerminateAgenda( iInterruptionControlCommit& ) = 0;
	virtual void fBeginClient( iInterruptionControlCommit& ) = 0;
	virtual void fBeforeInitClient( iInterruptionControlCommit& ) = 0;
	virtual void fBeforeTerminateClient( iInterruptionControlCommit& ) = 0;
	virtual void fBeforeOnErrorTerminateClient( iInterruptionControlCommit& ) = 0;
	virtual bool fBeforeRound( unsigned long, iInterruptionControlCommit& ) = 0;
	virtual void fBeforeOnScriptAbort( iInterruptionControlCommit& ) = 0;
	virtual void fEndClient( iInterruptionControlCommit& ) = 0;

	virtual void fCopyFile( iEmbedderCallContext&, const char*, const char* ) = 0;

// Made these public so they can be accessed by CRTAMThreadInstanceScript
public:
	CWLJSScript *m_jsScript;
  virtual void fProtoAddLast(const char *name, double l, CWLStatPointTypeEnum::PointType type) = 0;
	virtual void fProtoResetTimer(const char *name) = 0;
	virtual void fProtoSendTimer(const char *name) = 0;
  virtual void fThreadAddLast(const char *name, double l, CWLStatPointTypeEnum::PointType type) = 0;
	virtual void fThreadResetTimer(const char *name) = 0;
	virtual void fThreadSendTimer(const char *name) = 0;

	virtual iAgendaFile* fGetFile( bool&, iEmbedderCallContext&, const char* ) = 0;

	tDebugReporterReturn vDebugReporter_iUi( iReportThreadState&, iReportInfo& );
	virtual tDebugReporterReturn fDebugReporter( iReportThreadState&, iReportInfo& ) = 0;

public:
	IGeneralGlobalsRequestHandler* GetGlobalsRequestHandler();

private:
	RVCString&  GetEvaluationStringBeforeInitClient();
	RVCString&  GetEvaluationStringAfterInitAgenda();
//	RVCString&	GetEvaluationStringOnThreadTermination();
	RVCString&  GetEvaluationStringBeforeThreadActivation();
	RVCString&  GetEvaluationStringOnThreadActivation();
	RVCString&	GetEvaluationStringAfterTerminateClient();
	RVCString&	GetEvaluationStringBeforeRound();
	RVCString&	GetEvaluationStringAfterRound();
	RVCString&	GetEvaluationStringAfterTerminateAgenda();

protected:
	bool IsUserSection();


// iSession 
public:
	cSingleClientProcess& fGetProcess();

private: 
	double mBeginTime;

	double vGetBeginTime();
	};



class iLoadProbingClient:	public iLoadSingleClient 
{

public:
	explicit iLoadProbingClient( cSingleClientProcess&, bool IsDebuggerActive );
	virtual ~iLoadProbingClient();

	bool fExecute( const char*, iAgendaFile&, const char*, int, unsigned long );

	tNormalDMHookReturn vInterruptDMHook_iUi( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState&);
		virtual tNormalDMHookReturn vInterruptDMHook_iLoadProbingClient( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& ) = 0;

private:
	cSCStatClient mStatClient;

private:
  void fProtoAddLast(const char *name, double l, CWLStatPointTypeEnum::PointType type);
	void fProtoResetTimer(const char *name);
	void fProtoSendTimer(const char *name);
  void fThreadAddLast(const char *name, double l, CWLStatPointTypeEnum::PointType type);
	void fThreadResetTimer(const char *name);
	void fThreadSendTimer(const char *name);

protected:

	CWLStatGenMeasurementDictionary& fGetMeasurmentDictionary() {return mStatClient.fGetMeasurmentDictionary();}
	

	};



class iLoadSAFHeaders: public iLoadSingleClient 
{
public:
	explicit iLoadSAFHeaders(cSingleClientProcess&);
	virtual ~iLoadSAFHeaders();

	bool fExecute( const char*, iAgendaFile&, const char* );

private:

	virtual void fReporter( iReportInfo& ) = 0;

	void fExecReporter( iReportThreadState*, iReportInfo& );
	tDebugReporterReturn fDebugReporter( iReportThreadState& ,iReportInfo& );

	void fBeforeInitAgenda( iInterruptionControlCommit& );
	void fBeforeTerminateAgenda( iInterruptionControlCommit& );
	void fBeforeOnErrorTerminateAgenda( iInterruptionControlCommit& );
	void fBeginClient( iInterruptionControlCommit& );
	void fBeforeInitClient( iInterruptionControlCommit& );
	void fBeforeTerminateClient( iInterruptionControlCommit& );
	void fBeforeOnErrorTerminateClient( iInterruptionControlCommit& );
	bool fBeforeRound( unsigned long, iInterruptionControlCommit& );
	void fBeforeOnScriptAbort( iInterruptionControlCommit& );
	void fEndClient( iInterruptionControlCommit& );


	void fScriptHook( iScriptHookable&, bool Creating );
	void fReportDMHook(tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState&);

	tNormalDMHookReturn vInterruptDMHook_iUi(tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState&);
	tNormalDMHookReturn fBreakPointDMHook(tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState&);
	tNormalDMHookReturn fKeywordDMHook( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState&);

	tThrowDMHookReturn fThrowDMHook( tInterruptionControl&, bool& oIsHookPosition, tStepping, iDMThreadState& );

	bool fInvokeDMHook( bool rIsFunction, tInvocationDMHookType, iDMThreadState* );


  void fProtoAddLast(const char *name, double l, CWLStatPointTypeEnum::PointType type) {}
	void fProtoResetTimer(const char *name) {}
	void fProtoSendTimer(const char *name) {}
  void fThreadAddLast(const char *name, double l, CWLStatPointTypeEnum::PointType type) {}
	void fThreadResetTimer(const char *name) {}
	void fThreadSendTimer(const char *name) {}

	};

class cVirtualLoadEngine : public iVirtualLoadEngine {
public:
  explicit cVirtualLoadEngine() {};
  virtual ~cVirtualLoadEngine() {};
private:
  virtual bool vShouldReportInternalCSJSErrors(JSContext* Cx);
};




#endif //__CRTAMSCRIPT_H

