/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef __JSEmbed_H
#define __JSEmbed_H
class iReport;
class iSession;
class iLEJSMachineThread;
class iLEProtocolJSEmbedContext;

#include "platform.h"
#include "wlerrorlib.h"
#include "rvjsapi.h"
#include "rvextras_liveconnect.h"
#include "wljsobjects.h"
#include "rvjs.h"
#include "wlleprotocolbasetypes.h"
#include "tracedll.h"


class DllExport iLEJSEmbedContext;
//class DllExport iLEJSEmbedContext::cControlMachineThread;
class DllExport cLEJSEmbedContextBeginEndRequest;
class DllExport cLEJSEmbedContextSuspendResumeRequest;

class DllExport iActionReportInfo;
class DllExport iLEJSEmbedContext;
class DllExport cLEJSEmbedContextBeginEndRequest;
class DllExport cLEJSEmbedContextSuspendResumeRequest;

class iActionReportInfo
{
public:
	virtual ~iActionReportInfo();

	virtual void vApplyHard( iLEJSEmbedContext& ) = 0;
};

struct DllExport iRVJSJSection {};

class iLEJSEmbedContext: public iJSEmbedContext,  public iRVJSJSection
{
	friend class cLEJSEmbedContextBeginEndRequest;
	friend class cLEJSEmbedContextSuspendResumeRequest;
  friend class iLEProtocolAgendaInMix;
  friend class iLEProtocolVCInMix;
public:
	explicit iLEJSEmbedContext();
	virtual ~iLEJSEmbedContext();

	static iLEJSEmbedContext* fsGet( JSContext* rCx );

  bool fIsControlMachineThread();

	iSession& fGetSession() {return vGetSession(); }
	void fReport( iReportInfo& );

	virtual void reportApply( iReport& ) = 0;
	virtual void reportLog( iReport& ) = 0;

	virtual bool IsJSActive() = 0;

//	virtual void fActionReportCommit( iActionReportInfo& );

	void fHardReportCommit( iReportInfo& _ ) { vHardReportCommit( _ ); }

	iRVJSJSection* fEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType );
	void fExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType, iRVJSJSection* );

	bool fIsJSGC() { return vIsJSGC(); }


	class DllExport cControlMachineThread {
	public:
		explicit cControlMachineThread();
		explicit cControlMachineThread(iLEJSEmbedContext&);
		explicit cControlMachineThread(iLEJSEmbedContext&, bool rDoBegin );
		~cControlMachineThread();

		void fBegin(iLEJSEmbedContext&);
		void fBegin();
		void fEnd();

  private:
		iLEJSEmbedContext* mTop;
		iLEJSEmbedContext* mPrev;
	};
	friend class cControlMachineThread;

	iLEJSMachineThread& fGetMachineThread() { return vGetMachineThread(); }
	JSContext* fCurrentJSContext() {return vCurrentJSContext();}

  virtual iLEProtocolJSEmbedContext& GetProtocolJSEmbedContext(WLProtocolIndex pProtocolIndex) = 0;

private:
	virtual iLEJSMachineThread& vGetMachineThread() = 0;

	virtual void vEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType ) = 0;
	virtual void vExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType ) = 0;

	virtual bool vIsJSGC() = 0;

	virtual JSContext* vCurrentJSContext() = 0;

	void fEnterControlMachineThread();
  virtual void vEnterControlMachineThread_iLEJSEmbedContext() {};

	void fExitControlMachineThread();
  virtual void vExitControlMachineThread_iLEJSEmbedContext() {};

	void vJSExecReportCommit( iJSExecReportCommit&, iReportInfo& );
	void vHardReportCommit( iReportInfo& );

	virtual iSession& vGetSession() = 0;

};

typedef iLEJSEmbedContext::cControlMachineThread cLEJSEmbedContextControlMachineThread;


class cLEJSEmbedContextBeginEndRequest: RV_INHERIT_PRIVATE CJSGCRequestBeginEnd 
{
public:
	explicit cLEJSEmbedContextBeginEndRequest( iLEJSEmbedContext& );
};


class cLEJSEmbedContextSuspendResumeRequest: RV_INHERIT_PRIVATE CJSGCRequestSuspendResume 
{
public:
	explicit cLEJSEmbedContextSuspendResumeRequest( iLEJSEmbedContext& );
};

#endif //__JSEmbed_H
