/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#ifndef __WLLEProtocolEmbedClient_H
#define __WLLEProtocolEmbedClient_H

#include "wlleprotocolbasetypes.h"
#include "loadreportinfo.h"
#include "platform.h"
#include "rvjsclient.h"
#include <collint.h>

class DllExport iLEProtocolJSClientThread;
class DllExport cLEProtocolJSClientMachineThread;
class DllExport cLEProtocolVirtualReflectorJSClientEmbedContext;
class DllExport iLEProtocolJSClientEmbedContext;
class DllExport cLEProtocolJSFunctionHandle;
class DllExport cLEProtocolJSScriptHandle;
class DllExport cLEProtocolJSEmbedContextControlMachineThread;
class DllExport cLEProtocolJSClientEmbedContextBeginEndRequest;
class DllExport cLEProtocolJSClientEmbedContextSuspendResumeRequest;


//####### The declaration of the class iLEProtocolJSClientThread  #########

class iLEProtocolJSClientThread {

public:
	explicit iLEProtocolJSClientThread();
	virtual ~iLEProtocolJSClientThread();	

  void fInit( iLEProtocolVCJSThread&);
  void fFinal();

	JSContext* fGetContext();
  iLEJSClientThread& GetLEJSClientThread();

private:
  iLEJSClientThread m_JSClientThread;
};



//####### The declaration of the class cLEProtocolJSClientMachineThread  #########

class cLEProtocolJSClientMachineThread : public iLEProtocolMachineThread, public iLEProtocolVCJSThread
{
public:
	explicit cLEProtocolJSClientMachineThread(iLEProtocolAgendaJSEmbedContext&);
  virtual ~cLEProtocolJSClientMachineThread();

  void fInit();
	void fFinal();

  iLEJSMachineThread& GetJSMachineThread();
	
	// iLEVCJSThread stuff
	iLEVCJSThread& vGetVCJSThread();
  
private:
  cLEJSClientMachineThread          m_JSClientMachineThread;
  CLoadGeneralThreadInstanceScript& m_TIS;
};



//####### The declaration of the class cLEProtocolVirtualReflectorJSClientEmbedContext  #########

class cLEProtocolVirtualReflectorJSClientEmbedContext :  public iLEJSClientEmbedContext {
friend class iLEProtocolJSClientEmbedContext;
public:
  explicit cLEProtocolVirtualReflectorJSClientEmbedContext(iLEProtocolJSClientEmbedContext&);
  virtual ~cLEProtocolVirtualReflectorJSClientEmbedContext();
  void vReporter(WLLogMessage::Severity, const char* rMessage);
  iLEProtocolJSEmbedContext& GetProtocolJSEmbedContext(WLProtocolIndex pProtocolIndex);
private:
  iLEJSClientThread& vGetThread();

  iLEProtocolJSClientEmbedContext&  m_ProtocolJSClientEmbedContext;
};

//####### The declaration of the class iLEProtocolJSClientEmbedContext  #########

class iLEProtocolJSClientEmbedContext : public iLEProtocolJSEmbedContext {
friend class cLEProtocolVirtualReflectorJSClientEmbedContext;
public:
  explicit iLEProtocolJSClientEmbedContext(iLEProtocolProcess*);
  virtual ~iLEProtocolJSClientEmbedContext();

 	JSObject* fSet( JSObject* rGlobal );
	void fClear();
	JSBool fCallFunction( JSObject* rScope, jsval rFuncVal, uintN rArgc, jsval rArgv[], jsval& oRval );
	bool IsJSActive();
	JSContext* fGetContext();
	virtual void vReporter( WLLogMessage::Severity, const char* rMessage) = 0;
  JSContext* fCurrentJSContext();

  bool Report( iLoadReportInfo& pReportInfo );
  
 	void endMultiThread();
	
	iLEJSClientEmbedContext& GetLEJSClientEmbedContext(){
		WlAssert(dynamic_cast<iLEJSClientEmbedContext*>(&m_VirtualReflectorJSClientEmbedContext));
		return dynamic_cast<iLEJSClientEmbedContext&>(m_VirtualReflectorJSClientEmbedContext);
	}

  iLEJSEmbedContext& GetInternalLEJSEmbedContext();
  iLEProtocolAgendaJSEmbedContext& GetProtocolAgendaJSEmbedContext();  
private:
  virtual iLEProtocolJSClientThread& vGetThread() = 0;
	iLEJSMachineThread& vGetMachineThread();
	void vEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType );
	void vExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType );
	bool vIsJSGC();
	JSContext* vCurrentJSContext();
	iSession& vGetSession();

  iLEProtocolProcess* m_ProtocolProcess;
  cLEProtocolVirtualReflectorJSClientEmbedContext
    m_VirtualReflectorJSClientEmbedContext;
};



//####### The declaration of the class cLEProtocolJSFunctionHandle  #########
class cLEProtocolJSFunctionHandle {
public:
  explicit cLEProtocolJSFunctionHandle(cLEJSFunctionStatementInfo&);
  virtual ~cLEProtocolJSFunctionHandle();

	bool fSet( iLEProtocolJSClientEmbedContext& rEmbedContext, cLEJSFunctionStatementCompileInfo& rFunctionCompileInfo);
	bool fSet( iLEProtocolJSClientEmbedContext& rEmbedContext, jsval rFuncVal );
  bool fIsSet();
	JSBool fCall( iLEProtocolJSClientEmbedContext& rEmbedContext, jsval& oRval );
	jsval fGetFuncVal(iLEProtocolJSClientEmbedContext& rEmbedContext);
	const char* fGetToString( JSContext* );
	const char* fGetToString( iLEProtocolJSClientEmbedContext& rEmbedContext);
	void fFinal(iLEProtocolJSClientEmbedContext& rEmbedContext);

private:
  cLEJSFunctionHandle m_JSFunctionHandle;
};


//####### The declaration of the class cLEProtocolJSScriptHandle  #########
class cLEProtocolJSScriptHandle {
public:
	explicit cLEProtocolJSScriptHandle(cLEJSScriptStatementInfo&);
	~cLEProtocolJSScriptHandle();

	JSBool fExecute(iLEProtocolJSClientEmbedContext& rEmbedContext, cLEJSScriptStatementCompileInfo& rScriptCompileInfo, jsval& oRval );

		// Two rules for using the following 2 functions:
	// 1. If you call fExecuteProlog, you must call fExecuteMain before the dtor
	// 2. You can;t call fExecuteMain without calling fExecuteProlog first
	JSBool fExecuteProlog(iLEProtocolJSClientEmbedContext& rEmbedContext, cLEJSScriptStatementCompileInfo& rScriptCompileInfo, jsval& oRval);
	JSBool fExecuteMain(iLEProtocolJSClientEmbedContext& rEmbedContext, jsval& oRval);
	void fFinal(iLEProtocolJSClientEmbedContext& rEmbedContext);
private:
	cLEJSScriptHandle m_JSScriptHandle;
};


//####### The declaration of the class cLEProtocolJSEmbedContextControlMachineThread  #########
class cLEProtocolJSEmbedContextControlMachineThread {
public:
	explicit cLEProtocolJSEmbedContextControlMachineThread();
	explicit cLEProtocolJSEmbedContextControlMachineThread(iLEProtocolJSClientEmbedContext&);
	explicit cLEProtocolJSEmbedContextControlMachineThread(iLEProtocolJSClientEmbedContext&, bool rDoBegin );
	~cLEProtocolJSEmbedContextControlMachineThread();

	void fBegin(iLEProtocolJSEmbedContext&);
	void fBegin();
	void fEnd();

private:
  cLEJSEmbedContextControlMachineThread m_EmbedContextControlMachineThread;

};


//####### The declaration of the class cLEProtocolJSClientEmbedContextBeginEndRequest  #########
class cLEProtocolJSClientEmbedContextBeginEndRequest {

public:
  explicit cLEProtocolJSClientEmbedContextBeginEndRequest(iLEProtocolJSClientEmbedContext&);

private:
    cLEJSEmbedContextBeginEndRequest m_BeginEndRequest;
};

//####### The declaration of the class cLEProtocolJSClientEmbedContextSuspendResumeRequest  #########
class cLEProtocolJSClientEmbedContextSuspendResumeRequest {

public:
  explicit cLEProtocolJSClientEmbedContextSuspendResumeRequest(iLEProtocolJSClientEmbedContext&);

private:
    cLEJSEmbedContextSuspendResumeRequest m_SuspendResumeRequest;
};

#endif // __WLLEProtocolEmbedClient_H
