/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
 // Log.cpp: implementation of the CLog class.
//
//////////////////////////////////////////////////////////////////////

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "Log.h"

#define min(a,b) (((a) < (b)) ? (a) : (b))

const char* KTypeEvents[] = 
{	  "[ COMMAND]"
	, "[   EVENT]"
	, "[   STATS]"
	, "[   START]"
	, "[  FINISH]"
	, "[MSG_USER]"
	, "[  MSG_LG]"
};

const char* KTimeStampType[3] = {"abs","rel","off"};

CLog*	g_Logger = NULL;

//////////////////////////////////////////////////////////////////////
// Construction
//////////////////////////////////////////////////////////////////////
CLog::CLog( const char* filename, const TimestampType a_tsType, const bool a_overvrite, const bool a_console )
	:m_logFileName(NULL)
	,m_tsType(a_tsType)
	,m_consoleDump(a_console)
{
	m_logFileName = new char[strlen(filename)+1];
	strcpy( m_logFileName, filename );
	SYSTEMTIME SystemTime;
	GetSystemTime(&SystemTime);
	SystemTimeToFileTime(&SystemTime, (LPFILETIME)&m_timeStamp);

	if (a_overvrite)
		m_logFile = fopen(m_logFileName, "wt");
	else
		m_logFile = fopen(m_logFileName, "at");
}

//////////////////////////////////////////////////////////////////////
// Destruction
//////////////////////////////////////////////////////////////////////
CLog::~CLog()
{
	if (m_logFileName) {
		delete[] m_logFileName;
		m_logFileName = NULL;
	}


	if (m_logFile){
		fclose(m_logFile);
		m_logFile = NULL;
	}
}

//////////////////////////////////////////////////////////////////////
// Destruction
//////////////////////////////////////////////////////////////////////
//const CLog& CLog::GetLogger( )
//{
//	if(g_Logger)
//		return *g_Logger;
//	else
//		throw 1;
//}

//////////////////////////////////////////////////////////////////////
// void CLog::Write( const char*, const MessageType a_eType )
//////////////////////////////////////////////////////////////////////
void CLog::Write( const char* a_str, const MessageType a_eType ) 
{
	if (!g_Logger)
		return;

	if( g_Logger->m_logFile )
	{
		SYSTEMTIME systemTime;
		GetLocalTime(&systemTime);
		switch(g_Logger->m_tsType) {
		case etAbsolute:
			fprintf( g_Logger->m_logFile, "%0.2d:%0.2d:%0.2d:%0.3d %s %s\n"
				, systemTime.wHour
				, systemTime.wMinute
				, systemTime.wSecond
				, systemTime.wMilliseconds 
				, KTypeEvents[a_eType]
				, a_str);
			break;
		case etRelative:
			{
				__int64 timestamp = 0;
				SystemTimeToFileTime(&systemTime, (LPFILETIME)&timestamp);
				__int64 difftimestamp = timestamp - g_Logger->m_timeStamp;
				FileTimeToSystemTime( (LPFILETIME)&difftimestamp, &systemTime );
				fprintf( g_Logger->m_logFile, "%0.2d:%0.2d:%0.2d:%0.3d %s %s\n"
					, systemTime.wHour-2
					, systemTime.wMinute
					, systemTime.wSecond
					, systemTime.wMilliseconds 
					, KTypeEvents[a_eType]
					, a_str);
			}
			break;
		case etNone:
			if( a_eType == eStats || a_eType == eStart || a_eType == eFinish)
				return;
		default:
			fprintf( g_Logger->m_logFile, "\n%s %s", KTypeEvents[a_eType], a_str );
			break;
		}
		fflush( g_Logger->m_logFile );
	}
	if (g_Logger->m_consoleDump)
		printf("%s\n", a_str);
}

//////////////////////////////////////////////////////////////////////
// void CLog::InitLogFile( const char* filename, const TimestampType a_TOT, const bool a_firstTimeOpen )
//////////////////////////////////////////////////////////////////////
void CLog::InitLog( 
	const char* filename, const TimestampType a_tsType, 
	const bool a_overvrite, const bool a_console )
{
	ResetLog();
	g_Logger = new CLog(filename, a_tsType, a_overvrite, a_console );
}

void CLog::ResetLog()
{
	if (g_Logger) {
		delete g_Logger;
		g_Logger = NULL;
	}
}

//////////////////////////////////////////////////////////////////////
// CTimerEntry::CTimerEntry(const char*)
//////////////////////////////////////////////////////////////////////
CTimerEntry::CTimerEntry(const char* a_strName)
{
	if (g_Logger) {
		//len cannot be more than KMaxTimerEntryNameLen-1, to allow for a terminating null
		size_t len = min( KMaxTimerEntryNameLen-1, strlen(a_strName) );
		memcpy( m_strName, a_strName, len );
		m_strName[len] = 0;

		CLog::Write( m_strName, eStart );
		m_timerStart = GetTickCount();
	}
}

//////////////////////////////////////////////////////////////////////
// CTimerEntry::CTimerEntry(const char*)
//////////////////////////////////////////////////////////////////////
CTimerEntry::~CTimerEntry()
{
	if (g_Logger) {
		unsigned int timerEnd = GetTickCount();
		unsigned int timerDiff = timerEnd - m_timerStart;
		char buff[100];
		sprintf( buff, "%s stopped: %d.%d seconds", 
			m_strName,
			timerDiff/1000, timerDiff%1000);
		CLog::Write( buff, eFinish );
	}
}

/*
//////////////////////////////////////////////////////////////////////
// CTimerEntry::CTimerEntry(const char*)
//////////////////////////////////////////////////////////////////////
CTimerEntry::CTimerEntry(const char* a_strName)
{
	if (g_Logger) {
		size_t len = min( KMaxTimerEntryNameLen, strlen(a_strName) );
		memcpy( m_strName, a_strName, len );
		m_strName[len] = 0;

		CLog::Write( m_strName, eStart );
		SYSTEMTIME SystemTime;
		GetSystemTime(&SystemTime);
		SystemTimeToFileTime(&SystemTime, (LPFILETIME)&m_timestamp);
	}
}

//////////////////////////////////////////////////////////////////////
// CTimerEntry::CTimerEntry(const char*)
//////////////////////////////////////////////////////////////////////
CTimerEntry::~CTimerEntry()
{
	if (g_Logger) {
		__int64 timestamp = 0;
		SYSTEMTIME systemTime;
		GetLocalTime(&systemTime);
		SystemTimeToFileTime(&systemTime, (LPFILETIME)&timestamp);
		__int64 difftimestamp = timestamp - m_timestamp;
		FileTimeToSystemTime( (LPFILETIME)&difftimestamp, &systemTime );
		char buff[100];
		sprintf( buff, "%s stopped:%f", 
			m_strName,
			systemTime.wSecond + 60 * (systemTime.wMinute + (systemTime.wHour) * 60 ) //seconds part
				+ double(systemTime.wMilliseconds) / 1000.0);//milliseconds part
		CLog::Write( buff, eFinish );
	}
}
*/