/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.loadengine;

//##########################################################################
//###          														     ###
//###          class JavaThreadWrappper									 ###
//###          							                                 ###
//##########################################################################

//###########################################################################
//###          							                            	  ###
//###     The javaThreadWrapper wraps part of the functinality of the     ###
//###	WL Engine. It gets a jsObject as a parameter in it's constructor. ###
//###   This jsObject is the JS object that represents the thread of each ###
//###   Agenda that is executed. (The number of the virtual clients * 	  ###
//###   The number of agendas in the mix ). In the constuctor of the      ###
//###   class it keep this jsObject as a private data member 'threadIS'   ###
//###   (thread instance script ).										  ###
//###	  Then it uses the data member 'threadIS' to call the WL engine   ###
//###	functions.														  ###
//###          							                                  ###
//###	  The class imports the JSObject.class. The JSObject class is a   ###
//### 	class of netscape that allows Java to manipulate objects that are ###
//###	defined in JavaScript.											  ###
//###	In order to call a function of the 'threadIS' we use the function ###
//###	of the JSObject ,the function 'call'.	 				 		  ###
//###          							                                  ###
//###	Example of using the function 'call' :						      ###
//###          							                                  ###
//###			public void SetTimer(String timerName)				  	  ###
//###			{														  ###
//###				Object args[] = {timerName};                          ###
//###				threadIS.call("SetTimer", args);                      ###
//###			} 														  ###
//###          							                                  ###
//###	In the first line prepares the array of the parameters.           ###
//###	In the second line using the function 'call' in order to          ###
//###	call the JS function 'SetTimer' of the jsObject 'threadIS'.       ###
//###          							                                  ###
//###          							                                  ###
//###########################################################################

//import netscape.javascript.JSObject;
import com.radview.util.log.Log;

public class JavaThreadWrapper
{
	private String _threadData = null;
	private int _threadNum = -1;

	public JavaThreadWrapper(int threadNum, String threadData)
	{
		_threadNum = threadNum;
		_threadData = threadData;
	}

	public void setThreadData(String threadData)
	{
		_threadData = threadData;
	}

	public String toString()
	{
		return "WebLoad JavaThreadWrapper for thread " + _threadNum;
	}

//##########################	Native functions  ################################

	public native void jniSetTimer(String threadData, String timerName);
	public native void jniSendTimer(String threadData, String timerName);
	public native void jniSendCounter(String threadData, String counterName);

	public native boolean jniIsAbort(String threadData);
	//public native void SendMeasurement(String measurementName, double measurement);

//##########################	statistics functions  ################################

	public void SetTimer(String timerName)
	{
		jniSetTimer(_threadData, timerName);
	}

	public void SendTimer(String timerName)
	{
		jniSendTimer(_threadData, timerName);
	}

	public void SendCounter(String counterName)
	{
		jniSendCounter(_threadData, counterName);
	}

	public boolean IsAbort()
	{
		return jniIsAbort(_threadData);
	}

/*
	public void SendMeasurement(String measurementName, double measurement)
	{
		Double dMeasurement = new Double(measurement);
		Object args[] = {measurementName , dMeasurement};
		threadIS.call("SendMeasurement",args);
	}
	public void DisableJavaDefaultStatistics()
	{
		threadIS.call("DisableJavaDefaultStatistics",null);
	}

	public void EnableJavaDefaultStatistics()
	{
		threadIS.call("EnableJavaDefaultStatistics",null);
	}



//##########################	verification functions  ################################


	public void BeginTransactionEx(String transName, double time)
	{
		Double dTime = new Double(time);
		Object args[] = {transName, dTime};

		threadIS.call("BeginTransactionEx", args);
	}

	public void EndTransactionEx(String transName, int verification, boolean saveFlag,
			  					 double time, boolean sendTimers, boolean isLog)
	{
		Integer nVerification = new Integer(verification);
		Boolean bSaveFlag = new Boolean(saveFlag);
		Double dTime = new Double(time);
		Boolean bSendTimers = new Boolean(sendTimers);
		Boolean bIsLog = new Boolean(isLog);

		Object args[] = {transName, nVerification, bSaveFlag, dTime, bSendTimers, bIsLog };
		threadIS.call("EndTransactionEx", args);
	}

	public void SetFailureReasonEx(String reason, String description, double time, boolean isEvent )
	{
		Double dTime = new Double(time);
		Boolean bIsEvent = new Boolean(isEvent);

		Object args[] = {reason, description, dTime, bIsEvent};
		threadIS.call("SetFailureReasonEx", args);

	}

	public void ReportEventEx(String reason,String description, double time, int verification,String extraDataType , String extraData, boolean isLog )
	{
		Double dTime = new Double(time);
		Integer nVerification = new Integer(verification);
		Boolean bIsLog = new Boolean(isLog);

		Object args[] = {reason, description, dTime, nVerification, extraDataType, extraData ,bIsLog };
		threadIS.call("ReportEventEx", args);
	}

	public void EndTransactionFailureEx(String transname, double time, int verification,String reason, boolean sendTimers, boolean isLog, boolean isFailureEvent)
	{
		Double dTime = new Double(time);
		Integer nVerification = new Integer(verification);
		Boolean bSendTimers = new Boolean(sendTimers);
		Boolean bIsLog = new Boolean(isLog);
		Boolean bIsFailureEvent = new Boolean(isFailureEvent);

		Object args[] = {transname, dTime, nVerification, reason, bSendTimers, bIsLog, bIsFailureEvent };
		threadIS.call("EndTransactionFailureEx", args);
	}


	public String GetCurrentTransaction()
	{
		return  (String)threadIS.call("GetCurrentTransaction",null);
	}

	public int GetTransactionDepth()
	{
		Double intObj = (Double)threadIS.call("GetTransactionDepth",null);

		return intObj.intValue();
	}




//##########################	messages functions  ################################

	public void InfoMessage(String message)
	{
		String args[] = {message};
		threadIS.call("InfoMessage",args);
	}

	public void WarningMessage(String warningMessage)
	{
		Object args[] = { warningMessage };
		threadIS.call("WarningMessage",args);
	}

	public void ErrorMessage(String errorMessage)
	{
		String args[] = { errorMessage };
		threadIS.call("ErrorMessage",args);
	}

	public void SevereErrorMessage(String severeMessage)
	{
		Object args[] = { severeMessage };
		threadIS.call("SevereMessage",args);
	}

//######################  other Engine functions  ################################

	public void Sleep(int time)
	{
		Integer nTime = new Integer(time);

		Object args[] = { nTime };
		threadIS.call("Sleep",args);
	}

*/
}








