/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
package com.radview.loadengine.appletloader;


import java.applet.*;
import java.awt.*;
import java.util.*;
import java.net.*;

// Radview classes
import com.radview.loadengine.appletloader.system.*;
import com.radview.util.log.Log;
import com.radview.httpload.*;


/**
 * public class wlapplet.WebLoadAppletStub
 *
 */
public class WebLoadAppletStub extends Frame implements AppletStub, AppletContext
{
	HashMap		 	  __parametersTable;
	boolean			      __isActive;
	URL				      __documentBase;
	URL 			      __codeBase;

	/**
	 * Constructor
	 *
	 */
	public WebLoadAppletStub(URL documentBase, URL codeBase)
	{
		__isActive			= false;
		__documentBase      = documentBase;
		__codeBase          = codeBase;

		__parametersTable	= new HashMap();
	}

	/**
	 * addParameter
	 *
	 */
	public void addParameter(String name, String value)
	{
		String key = name.toLowerCase();
		//if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.addParameter("+key+","+value+")");
		__parametersTable.put(key, value);
	}

	public int getIntParameter(String key, int defValue)
	{
		String value = getParameter(key);
		int ret = defValue;
		if (value != null) {
			try {
				ret = Integer.parseInt(value);
			} catch (NumberFormatException e) {}
		}

		return ret;
	}
	/**
	 * setActiveStatus
	 *
	 */
	public void setActiveStatus(boolean isActive)
	{
		__isActive = isActive;
	}

	//
	// AppletStub interface....

	/**
	 * isActive
	 *
	 */
    public boolean isActive()
    {
			//if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.isActive()");
			return __isActive;
    }


	/**
	 * getDocumentBase
	 *
	 */
    public URL getDocumentBase()
    {
			if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.getDocumentBase() returned:" + __documentBase);
		  return __documentBase;
	  }

	/**
	 * getCodeBase :  @@Dudu : Need to be improved :
	 *
	 */
	public URL getCodeBase()
	{
		if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.getCodeBase()");
		if(__codeBase != null) {
			return __codeBase;
		}

		try {
			String codeBaseString;

			if( (codeBaseString = getParameter("CODEBASE")) == null ) {
				codeBaseString = InetAddress.getLocalHost().getHostName();
				__codeBase = new URL("file://" + codeBaseString);
			}
			else {
		    	__codeBase = new URL(codeBaseString);
			}
		}
		catch(Exception e) {
			e.printStackTrace();
		}

		return __codeBase;
	}


	/**
	 * getParameter
	 *
	 */
    public String getParameter(String name)
    {
			String key = name.toLowerCase();
			if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.getParameter(" +key+") returned:"+(String)__parametersTable.get(key));
			return (String)__parametersTable.get(key);
    }


	/**
	 * getAppletContext
	 *
	 */
	public AppletContext getAppletContext()
	{
		//if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.getAppletContext()");
		return this;
	}


	/**
	 * appletResize
	 *
	 */
	public void appletResize(int width, int height)
	{
		//
		// ignore this...
		if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.appletResize()");
	}


/**
 * Implementation of AppletContext interface
 *
 */
   //static Hashmap __appletsTable;
   //__appletsTable.put(__applet.getName(), __applet);

	public AudioClip getAudioClip(URL url)
	{
		if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.getAudioClip("+url.toString()+")");
		AudioClip audio = null;
		Object obj = null;

		try
		{
			// first try to get from the jarResources.
			JarResources resManager = JarResources.getSystemJarResources();
			byte audioStream[]  = resManager.getResource(url.toString());
			if (audioStream == null)
			{
				audioStream = HttpServices.jniGetStream(getParameter("__wlThreadData"), url.toString());
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

    return null;
	}

	public Image getImage(URL url)
	{
		if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.getImage("+url.toString()+")");

		Image image = null;
		Object obj = null;

		try
		{
			// first try to get from the jarResources.
			JarResources resManager = JarResources.getSystemJarResources();
			byte imageStream[]  = resManager.getResource(url.toString());
			if (imageStream == null)
			{
				imageStream = HttpServices.jniGetStream(getParameter("__wlThreadData"), url.toString());
			}

			if(imageStream != null)
				image = Toolkit.getDefaultToolkit().createImage(imageStream);

		}
		catch(Exception e)
		{
			e.printStackTrace();
		}

    return image;
	}

	public Applet getApplet(String name)
	{
		if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.getApplet("+name+")");
		Applet applet = (Applet)HttpServices.jniGetApplet(getParameter("WL_CDOMWindow*"),name);
		return applet;
	}

	public Enumeration getApplets()
	{
		if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.getApplets()");
		Object applets[] = HttpServices.jniGetApplets(getParameter("WL_CDOMWindow*"));

		Vector vector = new Vector();
	    for(int index = 0; index < applets.length; index++)
			vector.addElement(applets[index]);

		return vector.elements();
	}

	public void showDocument(URL url)
	{
		if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.showDocument("+url.toString()+")");

		HttpServices.jniShowDocument( getParameter("__wlThreadData"),
																	getParameter("WL_CDOMWindow*"),
																	url.toString());
	}

	public void showDocument(URL url, String target)
	{
		if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.showDocument("+url.toString()+","+target+")");

		HttpServices.jniShowDocumentTarget( getParameter("__wlThreadData"),
																			  getParameter("WL_CDOMWindow*"),
																				url.toString(),
																				target);
	}

	public void showStatus(String status)
	{
		if (Log.isLogging()) Log.logCategory.debug("WLAppletStub.showStatus("+status+")");
		//
		// ignore
	}
}


