/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "loadcopyfileobject.h"
#include "associativearray.h"
#include "loadgenerator.h"
#include "loadagendamanager.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadjsdefs.h"
#include <RVCString.h>
#include <collstr.h>
#include "loadenginedecs.h"
#include "tracedll.h"
#include "tvenv.h"
			


CLoadCopyFileObject::~CLoadCopyFileObject() {
	m_CopyFileHashTable.clearAndDestroy();
	m_IncludeFileList.clearAndDestroy();
}

void CLoadCopyFileObject::SetLoadAgendaManager(CLoadAgendaManager* lam) 
{
  m_LoadAgendaManager = lam;
}

bool CLoadCopyFileObject::CopyMonitorFile(const char* sourcefilename,
																					const char* targetfilename,
																					const char* agendaPath,
																					RVCString&  errorMessage) 
{
  RVCriticalSection::LockGuard lock(m_CopyFileMutex);
	RVCString rwstrKeyToTable;

	RVCString rwSource(sourcefilename);
	//if the source file is a full path the full path will be the key to the table.
#ifdef __TV_WINDOWS__
	if(rwSource.contains("\\")) { //the filename parameter is a path . 
		
#endif
#ifdef __TV_UNIX__
	if(rwSource.contains("/")) { //the filename parameter is a path
#endif
		// the key will be the full path of the file.
		rwstrKeyToTable = sourcefilename;
	} else {
		// the source file isn't a full path, in that case the key
		// of the  hash table will be fileName__fullpathAgenda. ( for example : textFile1.txt__D:\agenda1.js)
		rwstrKeyToTable = sourcefilename;
		rwstrKeyToTable.append("__");
		rwstrKeyToTable.append(agendaPath);
	}

  RVCollectableString key(rwstrKeyToTable);
	RVCollectableString target(targetfilename);


	// each file that is copied will be writen in the hash table 'm_CopyFileHashTable'.	

	RVHashTable* hashTableOfKey = (RVHashTable*)m_CopyFileHashTable.findValue(&key);

	if(hashTableOfKey == NULL) {//the file has never been copied.
		RVHashTable* hashTableForSourceFile = new RVHashTable();
		hashTableForSourceFile->insert(new RVCollectableString(targetfilename));
		RVCollectableString *tempKey = (RVCollectableString*)m_CopyFileHashTable.insertKeyAndValue(new RVCollectableString(rwstrKeyToTable),
																		 hashTableForSourceFile);
	}
	else {// there is an hash table for the source file.
		if(hashTableOfKey->contains(&target)) { // the source file has already copied to that target.
			return true;
		}
		else {//the source file was copied , but not to that target.
			RVHashTableIterator iterator(*hashTableOfKey);
			RVCollectableString* sourceString = (RVCollectableString *)iterator();
				if(TVPCopyFile(sourceString->data(),targetfilename)) {
					hashTableOfKey->insert(new RVCollectableString(targetfilename));
					return true;
			}
			else {
					WlTraceMsg("copyfile failed");
					return true;
			}

		}
	}


	//will ask the monitor to copy file.
	errorMessage = "";
	bool status = false;
  CAssociativeArray	 copyFileMessage;
	CAssociativeArray* copyDoneMessage = NULL;
	CLoadGenerator* lg = m_LoadAgendaManager->GetLoadGenerator();

  copyFileMessage.AddCouple("$COPYMONITORFILE", "");
  copyFileMessage.AddCouple("$REASON", "COPYFILE");
  copyFileMessage.AddCouple("$SRCFILENAME", sourcefilename);
	copyFileMessage.AddCouple("$DSTFILENAME", targetfilename);
	copyFileMessage.AddCouple("$AGENDAPATH", agendaPath);

	WlTraceMsg("LoadEngine ask monitor to copy from %s to %s", sourcefilename, targetfilename);

  lg->SendMessage(copyFileMessage, 2);
	copyDoneMessage = lg->GetMessage();

	if(copyDoneMessage->KeyExists("$COPYMONITORFILEDONE")) {

    if(copyDoneMessage->KeyExists("$COPYMONITORFILEFINDERROR")) {
      errorMessage = "CopyFile - ";
      errorMessage += copyDoneMessage->GetValueOfParameter("$COPYMONITORFILEFINDERROR");
      return false;
	  }

		long copyStatus = copyDoneMessage->GetLongValueOfParameter("$COPYSTATUS");
		const char* copyStatusOnFile = copyDoneMessage->GetValueOfParameter("$COPYSTATUSONFILE");

		switch(copyStatus) {
			case CNetMessageHandler::CopyFileSucceed:
			{
				status = true;
				break;
			}

			case CNetMessageHandler::OpenFileFailed:
			{
				// OPEN ERROR
				WLErrorException e(LoadExceptionFrmFile, 233, WLJSNAME_COPY_MONITOR_FILE, sourcefilename, targetfilename, copyStatusOnFile);
				errorMessage = e.GetErrorMsg();
				break;
			}	
			
			case CNetMessageHandler::CloseFileFailed:
			{
				// CLOSE ERROR
        WLErrorException e(LoadExceptionFrmFile, 235, WLJSNAME_COPY_MONITOR_FILE, sourcefilename, targetfilename, copyStatusOnFile);
				errorMessage = e.GetErrorMsg();
				break;
			}

			case CNetMessageHandler::ReadFileFailed:
			{
				// READ ERROR
				WLErrorException e(LoadExceptionFrmFile, 236, WLJSNAME_COPY_MONITOR_FILE, sourcefilename, targetfilename, copyStatusOnFile);
				errorMessage = e.GetErrorMsg();
				break;
			}
	
			case CNetMessageHandler::WriteFileFailed:
			{
				// WRITE ERROR
				WLErrorException e(LoadExceptionFrmFile, 234, WLJSNAME_COPY_MONITOR_FILE, sourcefilename, targetfilename, copyStatusOnFile);
				errorMessage = e.GetErrorMsg();
				break;
			}

			default: {
				WlTraceMsg( "the Monitor reply after copy file is : %s", copyDoneMessage->ToString() );
				// GENERAL(UNKNOWN) COPY FAILURE -
				WLErrorException e(LoadExceptionFrmFile, 237, WLJSNAME_COPY_MONITOR_FILE, sourcefilename, targetfilename);
				errorMessage = e.GetErrorMsg();
				break;
			}
		}
		delete copyDoneMessage;
	}
	else {
		WlTraceMsg( "the Monitor reply after copy file is : %s", copyDoneMessage->ToString() );
		// the message is none of the messages of the copy file. Thus this might be
		// a stop message or another message sent by the monitor.
		lg->PutMessage(copyDoneMessage);
		//    ???  why it was here
		//    GetThreadsManager()->reportLogMessageString(WLLogMessage::Warning, AssoBuffer->ToString());
    WLErrorException e(LoadExceptionFrmFile, 237, WLJSNAME_COPY_MONITOR_FILE, sourcefilename, targetfilename);
    errorMessage = e.GetErrorMsg();
    return false;
	}

  return status;
}

bool CLoadCopyFileObject::IncludeMonitorFile(const char* sourcefilename,
																						 const char** data,
																						 const char* agendaPath,
																						 RVCString& errormessage) 
{
  RVCriticalSection::LockGuard lock(m_CopyFileMutex);
  
  RVCollectableString key(sourcefilename);
  RVCollectableString* buffer = (RVCollectableString*)m_IncludeFileList.findValue(&key);
  if (buffer != NULL) {
    *data = buffer->data();
    return true;
  }

  CAssociativeArray	MessageStart;
  MessageStart.AddCouple("$COPYMONITORFILESTART",sourcefilename);
  MessageStart.AddCouple("$REASON","INCLUDE");
	MessageStart.AddCouple("$AGENDAPATH", agendaPath);
  CAssociativeArray	MessageNext;
  MessageNext.AddCouple("$COPYMONITORFILENEXT",sourcefilename);
	CAssociativeArray* AssoBuffer = NULL;

  CLoadGenerator* lg = m_LoadAgendaManager->GetLoadGenerator();
  
  lg->SendMessage(MessageStart, 2);
	AssoBuffer = lg->GetMessage();

  bool status = true;
  buffer = new RVCollectableString("");
  while (AssoBuffer->KeyExists("$COPYMONITORFILEBUFFER")) {
		buffer->append(AssoBuffer->GetValueOfParameter("$COPYMONITORFILEBUFFER"));
	  if (AssoBuffer->KeyExists("$COPYMONITORFILEEOF"))
      break;
		delete AssoBuffer;
    AssoBuffer = NULL;
		lg->SendMessage(MessageNext, 2);
		AssoBuffer = lg->GetMessage();
	}
  if (AssoBuffer->KeyExists("$COPYMONITORFILEERROR")) {
    status = false;
    WLErrorException e(LoadExceptionFrmFile, 240, WLJSNAME_INCLUDE,sourcefilename,sourcefilename);
    errormessage = e.GetErrorMsg();
  }
  else if (AssoBuffer->KeyExists("$COPYMONITORFILEFINDERROR")) {
    status = false;
    errormessage = "IncludeFile - ";
    errormessage += AssoBuffer->GetValueOfParameter("$COPYMONITORFILEFINDERROR");
  }
	else if (AssoBuffer->KeyExists("$COPYMONITORFILEEOF")) {/* do nothing*/}
	else {
		// the message is none of the messages of the copy file. Thus this might be
		// a stop message or another message sent by the monitor.
		lg->PutMessage(AssoBuffer);
//    GetThreadsManager()->reportLogMessageString(WLLogMessage::Warning, AssoBuffer->ToString());
    WLErrorException e(LoadExceptionFrmFile, 244, WLJSNAME_INCLUDE,sourcefilename);
    errormessage = e.GetErrorMsg();
    return false;
	}

	if (AssoBuffer != NULL) {
		delete AssoBuffer;
		AssoBuffer = NULL;
	}
  if (!status)
    return status;
  m_IncludeFileList.insertKeyAndValue(new RVCollectableString(sourcefilename),buffer);
  *data = buffer->data();
  return true;
}

