/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <stdio.h>
#include <RVCTokenizer.h>
#include "loadengineinifileproperties.h"
#include "tvenv.h"
#include "platform.h"
#include "loadenginedecs.h"
#include "tracedll.h"
#include "JVMAutoDiscovery.h"

#define PROPERTY_FOUND(str) (str != NULL && strlen(str))

long CLEIniFileProperties::LoadGeneratorSlaveTimeOut = 0;
int CLEIniFileProperties::MaxItemsPerChunk = 0;
long CLEIniFileProperties::RuntimeSize = 0;
long CLEIniFileProperties::ContextSize = 0;
long CLEIniFileProperties::StatGarbageTerminateTimeout = 0;
long CLEIniFileProperties::StatPackTerminateTimeout = 0;
long CLEIniFileProperties::JSGCTerminateTimeout = 0;
long CLEIniFileProperties::LogMessagesMax = 0;
int CLEIniFileProperties::TransactionMax = 0;
int CLEIniFileProperties::GlobalsHandlerWaitTime = 0;
int CLEIniFileProperties::WaitForThreadtoTerminate = 0;
RVCString CLEIniFileProperties::Protocols = "";
RVCString CLEIniFileProperties::ProtocolsNames = "";
RVCString CLEIniFileProperties::additionalProtocols = "";
RVCString CLEIniFileProperties::additionalProtocolsNames = "";
RVCString CLEIniFileProperties::ProtocolExtensionsPath = "";
RVCString CLEIniFileProperties::JavaExtensionsPath = "";
RVCString CLEIniFileProperties::JVMExtensionsPath = "";
bool CLEIniFileProperties::GCThreadOnly = false;
bool CLEIniFileProperties::UseWLJVMIni = false;
RVCString CLEIniFileProperties::WebLOADJVMPath = "";
RVCString CLEIniFileProperties::LibJVMPath = "";
RVCString CLEIniFileProperties::WebLOADClasspath = "";
RVCString CLEIniFileProperties::UserClasspath = "";
bool CLEIniFileProperties::DestroyJVM = false;
RVCString CLEIniFileProperties::LGDefaultInitializationScript = "";
bool CLEIniFileProperties::EnableStopgeneratorOnNetErrors = false;
bool CLEIniFileProperties::IgnoreInitScript = false;
bool CLEIniFileProperties::ThreadPoolOn = false;
int CLEIniFileProperties::InitialThreadPoolSize = 0;
long CLEIniFileProperties::LoadGeneratorMaxThreadsPerSlave = 0;
RVCString  CLEIniFileProperties::OptionsBuf[100];
bool CLEIniFileProperties::initPerformed = false;
bool CLEIniFileProperties::tablesInited = false;
bool CLEIniFileProperties::additionalProtocolsSet = false;
TableHandle CLEIniFileProperties::LETableHandle;
TableHandle CLEIniFileProperties::WLTableHandle;
TVEnvContainerClass CLEIniFileProperties::tvpc;
bool CLEIniFileProperties::ErrorCountersOn = false;
bool CLEIniFileProperties::TranslateSystemExceptions = false;
bool CLEIniFileProperties::ListThreadInfo = false;
int CLEIniFileProperties::JSGCInterval = 30;
int CLEIniFileProperties::JSGCSleep = 1000;
int CLEIniFileProperties::JSGCInitialSleep = 60000;
bool CLEIniFileProperties::DisplayUserAgendaPositionInErrors = false;
RVCString CLEIniFileProperties::WebLOADDefaultIncludePath = "";
int CLEIniFileProperties::JVMCreateWaitTime = 5;
bool CLEIniFileProperties::ScriptCacheOn = false;
long CLEIniFileProperties::ScriptCacheWarmupLoadRounds = 1000;
long CLEIniFileProperties::ScriptCacheWarmupAnalysisRounds = 1000;
long CLEIniFileProperties::UseCountThreshold = 1;
bool CLEIniFileProperties::ErrorTraceAll = false;
bool CLEIniFileProperties::ErrorTraceCSJSCompileErrors = false;
bool CLEIniFileProperties::ErrorTraceScriptCache = false;
bool CLEIniFileProperties::UseNewJSGCGuardStyle = false;
bool CLEIniFileProperties::UseAtomClip = false;
bool CLEIniFileProperties::CompilerIntervention = false;
int CLEIniFileProperties::CompilerInterventionCount = 0;
bool CLEIniFileProperties::StandardReferenceErrorBehavior = false;
bool CLEIniFileProperties::EnforceReservedKeywords = false;

typedef int  TableHandle;

//####### The implementation of the class CLEIniFileProperties   #########

void CLEIniFileProperties::Init() {

  initTables();
  
  const char* LGSTimeout = tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"LG_SLAVE_TIMEOUT"));
  if (PROPERTY_FOUND(LGSTimeout))
    CLEIniFileProperties::LoadGeneratorSlaveTimeOut = atol(LGSTimeout);
  else
    CLEIniFileProperties::LoadGeneratorSlaveTimeOut = 900000;

  CLEIniFileProperties::MaxItemsPerChunk =	CLEIniFileProperties::MaxItemsPerChunk = atoi(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"MAX_ITEMS_PER_CHUNK")));
  tvpc.reset();
	CLEIniFileProperties::RuntimeSize = atol(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"RUNTIME_SIZE")));
  tvpc.reset();
	CLEIniFileProperties::ContextSize = atol(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"CONTEXT_SIZE")));
  tvpc.reset();
  CLEIniFileProperties::StatGarbageTerminateTimeout = atol(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"STAT_GARBAGE_TERMINATE_TIMEOUT")));
  tvpc.reset();
  CLEIniFileProperties::StatPackTerminateTimeout = atol(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"STAT_PACK_TERMINATE_TIMEOUT")));
  tvpc.reset();
  CLEIniFileProperties::JSGCTerminateTimeout = atol(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"JSGC_TERMINATE_TIMEOUT")));
  tvpc.reset();
  CLEIniFileProperties::LogMessagesMax = atol(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"LOG_MESSAGES_MAX")));
  tvpc.reset();
  CLEIniFileProperties::TransactionMax = atoi(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"TRANSACTION_MAX")));
  tvpc.reset();
  CLEIniFileProperties::GlobalsHandlerWaitTime = atoi(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"GLOBALS_HANDLER_WAIT_TIME")));
  tvpc.reset();
  CLEIniFileProperties::WaitForThreadtoTerminate = atoi(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"WAIT_FOR_THREAD_TO_TERMINATE")));
  tvpc.reset();
  CLEIniFileProperties::Protocols = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle,"PROTOCOLS"));
  tvpc.reset();
  CLEIniFileProperties::ProtocolsNames = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle,"PROTOCOLS_NAMES"));
  tvpc.reset();

  if (CLEIniFileProperties::additionalProtocolsSet){
    addProtocols();
  }

  {
    const char* temp = tvpc.TVERuntimeResolve( "#(PROTOCOL_EXTENSIONS_PATH)" );
    if (temp)
      CLEIniFileProperties::ProtocolExtensionsPath = temp;
  }
  tvpc.reset();

  {
    const char* temp = tvpc.TVERuntimeResolve( "#(JAVA_EXTENSIONS_PATH)" );
    if (temp)
      CLEIniFileProperties::JavaExtensionsPath = temp;
  }
  tvpc.reset();

  {
    const char* temp = tvpc.TVERuntimeResolve( "#(JVM_EXTENSIONS_PATH)" );
    if (temp)
      CLEIniFileProperties::JVMExtensionsPath = temp;
  }
  tvpc.reset();
  
  CLEIniFileProperties::GCThreadOnly = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"LOAD_GENERATOR_GC_THREAD_ONLY")));
  tvpc.reset();
  CLEIniFileProperties::UseWLJVMIni = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle,"USE_WEBLOAD_JVM")));
  tvpc.reset();
  {
    const char* temp = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle,"WEBLOAD_JVM_PATH"));
    if (temp)
      CLEIniFileProperties::WebLOADJVMPath = temp;
  }
  tvpc.reset();
  {
    const char* temp = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle,"LIBJVM"));
    if (temp)
      CLEIniFileProperties::LibJVMPath = temp;
  }

  for (int indx = 0; indx < 100; indx++){
		RVCString aEnvKey( "JOPTION" );
		char aStrOptionIndex[10];
		aEnvKey += itoa( indx, aStrOptionIndex, 10 );
    tvpc.reset();
    const char* temp = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle, aEnvKey.data()));
    if (temp)
      CLEIniFileProperties::OptionsBuf[indx] = temp;
    else
      CLEIniFileProperties::OptionsBuf[indx] = "";
  }

  tvpc.reset();
  CLEIniFileProperties::WebLOADClasspath = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle,"WEBLOAD_CLASSPATH"));
  tvpc.reset();
  CLEIniFileProperties::UserClasspath = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle,"USER_CLASSPATH"));
  tvpc.reset();
  CLEIniFileProperties::DestroyJVM = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle,"DestroyJVM")));
  tvpc.reset();
  CLEIniFileProperties::LGDefaultInitializationScript = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle,"LOAD_GENERATOR_DEFAULT_INITIALIZATION_SCRIPT"));
  tvpc.reset();
  CLEIniFileProperties::EnableStopgeneratorOnNetErrors = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"ENABLE_STOP_GENERATOR_ON_NET_ERRORS")));
  tvpc.reset();
  CLEIniFileProperties::IgnoreInitScript = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"WL_IGNORE_INIT_SCRIPT")));
  tvpc.reset();
  CLEIniFileProperties::ThreadPoolOn = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"WL_THREADPOOLON")));
  
  tvpc.reset();
  const char* ThreadPoolSize = tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"WL_INITIAL_THREADPOOL_SIZE"));
  if (PROPERTY_FOUND(ThreadPoolSize))
    CLEIniFileProperties::InitialThreadPoolSize = atol(ThreadPoolSize);
  else
    CLEIniFileProperties::InitialThreadPoolSize = 0;

  tvpc.reset();
  CLEIniFileProperties::LoadGeneratorMaxThreadsPerSlave = atol(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"LG_MAX_THREADS_PER_SLAVE")));

  tvpc.reset();
	CLEIniFileProperties::ErrorCountersOn = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"WL_ERROR_COUNTERS_ON")));

	tvpc.reset();
	CLEIniFileProperties::TranslateSystemExceptions = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"WL_TRANSLATE_SYSTEM_EXCEPTIONS")));

	tvpc.reset();
	CLEIniFileProperties::ListThreadInfo = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"WL_LIST_THREAD_INFO")));

	{
	tvpc.reset();
		const char* X0 = tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"JSGC_INTERVAL"));
		if (PROPERTY_FOUND(X0)) {
		  CLEIniFileProperties::JSGCInterval = atol(X0);
		}
	}

	{
		tvpc.reset();
		const char* X0 = tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"JSGC_SLEEP"));
		if (PROPERTY_FOUND(X0)) {
		  CLEIniFileProperties::JSGCSleep = atol(X0);
		}
	}

	{
		tvpc.reset();
		const char* X0 = tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"JSGC_INITIAL_SLEEP"));
		if (PROPERTY_FOUND(X0)) {
		  CLEIniFileProperties::JSGCInitialSleep = atol(X0);
		}
	}

	tvpc.reset();
	CLEIniFileProperties::DisplayUserAgendaPositionInErrors = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"WL_DISPLAY_USER_AGENDA_POSITION")));

  tvpc.reset();
  CLEIniFileProperties::WebLOADDefaultIncludePath = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle,"WL_DEFAULT_INCLUDE"));

	tvpc.reset();
	const char* JVMCreateWaitTimeStr = tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"JVM_CREATE_WAIT_TIME"));
	if (PROPERTY_FOUND(JVMCreateWaitTimeStr))
	  CLEIniFileProperties::JVMCreateWaitTime = atol(JVMCreateWaitTimeStr);
	else
		CLEIniFileProperties::JVMCreateWaitTime = 5;

	tvpc.reset();
	CLEIniFileProperties::ScriptCacheOn = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"SCRIPT_CACHE_ON")));

	tvpc.reset();
	const char* ScriptCacheWarmupLoadRoundsStr = tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"SCRIPT_CACHE_WARMUP_LOAD_ROUNDS"));
	if (ScriptCacheWarmupLoadRoundsStr != NULL)
	  CLEIniFileProperties::ScriptCacheWarmupLoadRounds = atol(ScriptCacheWarmupLoadRoundsStr);
	else
		CLEIniFileProperties::ScriptCacheWarmupLoadRounds = 1000;

	tvpc.reset();
	const char* ScriptCacheWarmupAnalysisRoundsStr = tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"SCRIPT_CACHE_WARMUP_ANALYSIS_ROUNDS"));
	if (ScriptCacheWarmupAnalysisRoundsStr != NULL)
	  CLEIniFileProperties::ScriptCacheWarmupAnalysisRounds = atol(ScriptCacheWarmupAnalysisRoundsStr);
	else
		CLEIniFileProperties::ScriptCacheWarmupAnalysisRounds = 1000;

	tvpc.reset();
	const char* UseCountThresholdStr = tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"USE_COUNT_THRESHOLD"));
	if (UseCountThresholdStr != NULL)
	  CLEIniFileProperties::UseCountThreshold = atol(UseCountThresholdStr);
	else
		CLEIniFileProperties::UseCountThreshold = 5;


	tvpc.reset();
	CLEIniFileProperties::ErrorTraceAll = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"ERROR_TRACE_ALL")));

	tvpc.reset();
	CLEIniFileProperties::ErrorTraceCSJSCompileErrors = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"ERROR_TRACE_CSJS_COMPILE_ERRORS")));

	tvpc.reset();
	CLEIniFileProperties::ErrorTraceScriptCache = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"ERROR_TRACE_SCRIPT_CACHE")));

	if (CLEIniFileProperties::ErrorTraceAll){
		CLEIniFileProperties::ErrorTraceCSJSCompileErrors = true;
		CLEIniFileProperties::ErrorTraceScriptCache = true;
	}

	tvpc.reset();
	CLEIniFileProperties::UseNewJSGCGuardStyle = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"USE_NEW_JSGC_GUARD_STYLE")));

	tvpc.reset();
	CLEIniFileProperties::UseAtomClip = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"USE_ATOM_CLIP")));

	tvpc.reset();
	CLEIniFileProperties::CompilerIntervention = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"COMPILER_INTERVENTION")));

	tvpc.reset();
	const char* CompilerInterventionCountStr = tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"COMPILER_INTERVENTION_COUNT"));
	if (CompilerInterventionCountStr != NULL)
	  CLEIniFileProperties::CompilerInterventionCount = atol(CompilerInterventionCountStr);
	else
		CLEIniFileProperties::CompilerInterventionCount = 5;

	if (CLEIniFileProperties::ErrorTraceScriptCache){
		if (CLEIniFileProperties::ScriptCacheOn){
			WlErrorMsg("Script cache ON");
		}
		else{
			WlErrorMsg("Script cache OFF");
		}
	}

	tvpc.reset();
	CLEIniFileProperties::StandardReferenceErrorBehavior = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"STANDARD_REFERENCE_ERROR_BEHAVIOR")));

	tvpc.reset();
	CLEIniFileProperties::EnforceReservedKeywords = optionToBool(tvpc.TVERuntimeResolve(TVSGetValue(LETableHandle,"ENFORCE_RESERVED_KEYWORDS")));

  CLEIniFileProperties::initPerformed = true;
}

void CLEIniFileProperties::AppendProtocol(const char* pProtocol, const char* pProtocolName){

  if (strchr(pProtocol, ';')){
    WlErrorMsg("AppendProtocol can accept only a single protocol at a time. Sorry.");
    WlAssert(0);
  }
  
  CLEIniFileProperties::additionalProtocols += pProtocol;
  CLEIniFileProperties::additionalProtocolsNames += pProtocolName;
  CLEIniFileProperties::additionalProtocols += ";";
  CLEIniFileProperties::additionalProtocolsNames += ";";
  CLEIniFileProperties::additionalProtocolsSet = true;

  // If we're after init, then add one protocol at a time.
  // If we're before init, we need to maintain the full list so we can
  // append the entire list of additional protocols during the init.
  if (CLEIniFileProperties::initPerformed){
    addProtocols();
    CLEIniFileProperties::additionalProtocols = "";
    CLEIniFileProperties::additionalProtocolsNames = "";
  }
  else {
		initTables();
    tvpc.reset();
	//Galtech: this is wrong
//    RVCString aAdditionalProtocols = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle, pProtocol));
  }
}

bool CLEIniFileProperties::optionToBool(const char* pEnvOption){
  if (PROPERTY_FOUND(pEnvOption)) {
    RVCString EnvOption = pEnvOption;
    if (EnvOption.index("YES",0, RVCString::ignoreCase ) != RV_NPOS)
        return true;
  }
  return false;
}



void CLEIniFileProperties::addProtocols(){
  tvpc.reset();
  {
    RVCTokenizer aDelimiter(CLEIniFileProperties::additionalProtocols);
    RVCString aCurProtocol;
    while (!(aCurProtocol=aDelimiter(";")).isNull()){
      RVCString aAdditionalProtocols = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle, aCurProtocol));
      if (aAdditionalProtocols.length() > 0){
        CLEIniFileProperties::Protocols += ";";
        CLEIniFileProperties::Protocols += aAdditionalProtocols;
      }
    }
  }

  tvpc.reset();
  {
    RVCTokenizer aDelimiter(CLEIniFileProperties::additionalProtocolsNames);
    RVCString aCurProtocolName;
    while (!(aCurProtocolName=aDelimiter(";")).isNull()){
      RVCString aAdditionalProtocolsNames = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle, aCurProtocolName));
      if (aAdditionalProtocolsNames.length() > 0){
        CLEIniFileProperties::ProtocolsNames += ";";
        CLEIniFileProperties::ProtocolsNames += aAdditionalProtocolsNames;
      }
    }
  }
}

void CLEIniFileProperties::initTable(TableHandle& tableHandle, const char* iniFileName) {

	//Init the LE constants from the loadengine.ini file
	tableHandle = TVSCreateTable();
	const char* tv_Bib = tvpc.TVEGetEnv("TV_BIN");
	if(tv_Bib) {
  	char buffer[WL_MAX_PATH];
#ifdef __TV_WINDOWS__
	  sprintf(buffer,"%s\\%s",tv_Bib,iniFileName);
#endif
#ifdef __TV_UNIX__
	  sprintf(buffer,"%s/%s",tv_Bib,iniFileName);
#endif
	  TVSLoadTable(buffer,tableHandle);
  }
}

void CLEIniFileProperties::initTables(){
  if (tablesInited) return;
  initTable(LETableHandle, "loadengine.ini");
	initTable(WLTableHandle, "webload.ini");
  tablesInited = true;
}

const char* CLEIniFileProperties::GetAppendProtocolRealName(const char* pProtocol){
  initTables();
  tvpc.reset();
  const char* aAdditionalProtocolsNames = tvpc.TVERuntimeResolve(TVSGetValue(WLTableHandle,pProtocol));

  if (strchr(aAdditionalProtocolsNames, ';')){
    WlErrorMsg("The entry you requested contained multiple protocols dll\'s. Sorry.");
    WlAssert(0);
  }
  return aAdditionalProtocolsNames;
}

void CLEIniFileProperties::resetJVMPath(const char* JVMType){
  RVCString iniPathName = JVMType;
  iniPathName += "_JVM_PATH";
  CJVMAutoDiscovery autoDiscovery;

  autoDiscovery.discoverPath(CLEIniFileProperties::JVMExtensionsPath, "*.xml", true);

  RVCString RVJVMType = JVMType;

  RVCString* JVMPath = autoDiscovery.findJVMPath(&RVJVMType);

  if (JVMPath){
	CLEIniFileProperties::WebLOADJVMPath = *JVMPath;
  }

}
