/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <math.h>

#include "tracedll.h"
#include "associativearray.h"
#include "loadgeneratorscript.h"
#include "loadthreadsmanager.h"
#include "loadgeneratorslave.h"
#include "send2tml.h"
#include "tvenv.h"
#include "tvpnumconversions.h"

#include "loadenginestatistics.h"
#include "loadengineinifileproperties.h"

#include "CommandLinePropertie.h"

static RVCString StatisticsFileName = "";

void SetStatisticsFileName(const char* name) {

  TVEnvContainerClass tvpc;
  RVCString LogDir = tvpc.TVEGetEnv("LOG_DIR");

  if (LogDir.length() > 0)
    StatisticsFileName = tvpc.TVPConcatDirAndName(LogDir, name);
  else
    StatisticsFileName = name;
}
const char*  GetStatisticsFileName() {
  return StatisticsFileName;
}

//////////////////////////////////////////////////////////////////////////
//RVCString SetTestLogFileName(const char* name) 
//////////////////////////////////////////////////////////////////////////
RVCString GetTestLogFileName(const char* name) 
{
  RVCString res;
	char buff[MAX_PATH],fname[MAX_PATH];
	strcpy( fname, name );
  TVEnvContainerClass tvpc;
  RVCString LogDir = tvpc.TVEGetEnv("LOG_DIR");
	const char* tmpfilename = fname, *tmp;
	while( (tmp = strrchr(tmpfilename, '\\')) && tmpfilename != tmp)
		tmpfilename = tmp;
	const char* placeofpoint = tmpfilename;
	while( (tmp = strrchr(placeofpoint, '.') ) && placeofpoint != tmp)
			placeofpoint = tmp;
	fname[ placeofpoint - fname ] = '\0';
	
	if(getCommandLineProperties().logSuffix)
	{
		SYSTEMTIME systemTime;
		GetLocalTime( &systemTime );
		sprintf( buff, "%s%0.2d_%0.2d_%0.2d_%0.3d.log", tmpfilename, systemTime.wHour, systemTime.wMinute, systemTime.wSecond, systemTime.wMilliseconds );
	}else
		sprintf( buff, "%s.log", tmpfilename );

  if (LogDir.length() > 0)
    res = tvpc.TVPConcatDirAndName(LogDir, buff);
  else
    res = buff;
//	StatisticsFileName = res;
 return res;
}

//
// ***** Agenda Statistics class *****
//
CAgendaStatistics::CAgendaStatistics(const char* name) :
  CWLStatGenMain(name)
{
    m_ExtraStatisticsFirstTime = true;
		m_SendClientStatistics = false;
    m_SendClientStatisticsFilter = "";

    if (StatisticsFileName.length() > 0){
      WlTraceMsg("%s used as statistics file", StatisticsFileName);
      if (!(m_StatisticsFile = fopen(StatisticsFileName, "w")))
        WlTraceMsg("Error opening statistics file");
	  else
		fclose(m_StatisticsFile);
    }
    else
      m_StatisticsFile = NULL;
} 
 
void CAgendaStatistics::Init(CLoadThreadsManagerScript* ag)
{
  m_pMyLoadThreadsManagerScript = ag;
} 

void CAgendaStatistics::Start(double BeginTime, double sliceSizeResolution, bool pSendClientStatistics, const char* pSendClientStatisticsFilter){
	m_SendClientStatistics = pSendClientStatistics;
  m_SendClientStatisticsFilter = pSendClientStatisticsFilter;
	CWLStatGenMain::Start(BeginTime, sliceSizeResolution);
}
 
//void CAgendaStatistics::SetRecipientName(const char* name) {
  //statRecipientName = name;
//}

void CAgendaStatistics::CalculateExtraStatisticsForDictionary( CWLStatGenMeasurementDictionary *pDic, 
																															 double lastSliceSize, 
																															 double lastSliceTime) {
  double averageLoadSize, sliceLoadSize; 
  m_pMyLoadThreadsManagerScript->m_LoadSizeManager.GetLoadSize(lastSliceTime, averageLoadSize, sliceLoadSize);
  pDic->SetValues("Load Size", 0, 0, 1, averageLoadSize, 0 , 0, 0, 1, sliceLoadSize, CWLStatPointTypeEnum::LoadSize);
//  pDic->SetValues("Load Size Delta", 0, 0, 1, 0, 0 , 0, 0, 1, pMyLoadThreadsManagerScript->GetLoadGenerator()->GetTimeFromBeginning()-lastSliceTime, CWLStatPointTypeEnum::Timer);
	char buff[500];
  sprintf(buff,"Time:(%f,%f), Hits: %d, Rounds: %d, Size(%f,%f) Slave first thread (%d)\n"
	  , lastSliceTime
	  , m_pMyLoadThreadsManagerScript->GetLoadGenerator()->fGetTimeFromBegin()
	  , pDic->GetLastsCount("Hits")
	  , pDic->GetLastsCount("Rounds")
	  , averageLoadSize
	  , sliceLoadSize
	  , m_pMyLoadThreadsManagerScript->GetLoadGenerator()->m_SlaveBaseThreadId);
  if(getCommandLineProperties().messagesOn && !getCommandLineProperties().consoleDump)
	  printf( buff );
  WlTraceMsg( buff );
  CLog::Write( buff );

  // Print to file as well
  if (StatisticsFileName.length() > 0 &&
	  (m_StatisticsFile = fopen(StatisticsFileName, "a"))){
    if (m_ExtraStatisticsFirstTime){
      fprintf(m_StatisticsFile, "Time, Hits, Rounds, Load Size(Average), Load Size(Slice)\n");
      m_ExtraStatisticsFirstTime = false;
    }
    fprintf(m_StatisticsFile, "%f, %d, %d, %f, %f\n", lastSliceTime, pDic->GetLastsCount("Hits"), pDic->GetLastsCount("Rounds"), averageLoadSize, sliceLoadSize);
    fclose(m_StatisticsFile);
  }

}
void CAgendaStatistics::CalculateExtraStatisticsForCurrentClientDictionary( 
	CWLStatGenMeasurementDictionary *pDic, 
	int ClientId,
	double SliceEndTime) {

	if (!m_SendClientStatistics || !ShouldSendCurrentSlice(ClientId) ||
		  ClientId >= 1000000 || ClientId == -1)
		return;

	// Format the current slice as string
	RVCString aFmtString ("");
  RVCString buf;
  RVCString num;
	char c=1;						// Seperator

	// Slice data - create a temporary slice and format it
	// Use slice ctor that won't delete its dicts.
	CWLStatGenSlice aTempSlice (0, SliceEndTime, this, false);
	// Create a temporary dictionary otherwise the slice's dtor will destroy the dict
	// Don't delete it, the slice's dtor will delete it
	//CWLStatGenMeasurementDictionary* aTempDict = new CWLStatGenMeasurementDictionary(*pDic);
  //aTempSlice.AddDictionary(aTempDict, ClientId);
  aTempSlice.AddDictionary(pDic, ClientId);
	aTempSlice.DumpSlice(aFmtString);

	// Thread num
  TVPDoubleToString(ClientId, num, 3);
  buf = num;
  buf.append(c);

	// Agenda name
	buf.append(GetAgendaName());
	buf.append(c);

	// Slice size
  TVPDoubleToString(sliceSize, num, 3);
  buf.append(num);
  buf.append(c);

	aFmtString.prepend(buf);

	// Send it
  CAssociativeArray StatisticsMessage;
	CAssociativeArray SubMessage;

	// Wrap the aFmtString as an associative array of #STATSTRING
	SubMessage.AddCouple("$ENTRIES", (long)1);
	SubMessage.AddArrayCouple("$STATSTRING",(const char*)aFmtString.data(), 1);
  StatisticsMessage.AddCouple("$STATISTICSCLIENTDATA",SubMessage.ToString());

  StatisticsMessage.AddCouple("$AGENDANAME", GetAgendaName());
  StatisticsMessage.AddCouple("$TIME", SliceEndTime - GetBeginSystem());
	StatisticsMessage.AddCouple("$STATISTICSFREQUENCY", (long)m_pMyLoadThreadsManagerScript->GetLoadGeneratorSlave()->GetStatisticsFrequency());

  m_pMyLoadThreadsManagerScript->GetLoadGenerator()->SendMessageToMonitor(StatisticsMessage, 1);

}

void CAgendaStatistics::SendDictionaryString( RVCString &str, double sliceTime, const char* agendaName ) {
  WlTraceMsg("CAgendaStatistics::SendDictionaryString(time: %f, slave:%s, angenda:%s", sliceTime, m_pMyLoadThreadsManagerScript->GetLoadGenerator()->WhoAmI(), agendaName);

  CAssociativeArray StatisticsMessage;

  StatisticsMessage.AddCouple("$STATISTICSMANAGERDATA",(const char*)str);
  StatisticsMessage.AddCouple("$AGENDANAME", agendaName);
  StatisticsMessage.AddCouple("$TIME", sliceTime);
	StatisticsMessage.AddCouple("$STATISTICSFREQUENCY", (long)m_pMyLoadThreadsManagerScript->GetLoadGeneratorSlave()->GetStatisticsFrequency());
  m_pMyLoadThreadsManagerScript->GetLoadGenerator()->SendMessage(StatisticsMessage, 1);
}

bool CAgendaStatistics::ShouldSendCurrentSlice( int ClientId ){
	if (!(m_SendClientStatisticsFilter.length() > 0) || m_SendClientStatisticsFilter.data()[0] == '*')
		return true;

  RVCString aThreadNum;

  TVPDoubleToString(ClientId, aThreadNum, 3);

	RVCString aSearchString(";");
	aSearchString += aThreadNum;
	aSearchString += ";";

	return m_SendClientStatisticsFilter.contains(aSearchString);
	
}
