/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// LoadEngineVerification.cpp: implementation of the IGeneralThreadTransactionManager class.
//
//////////////////////////////////////////////////////////////////////

#include "associativearray.h"
#include "loadgeneratorscript.h"
#include "loadexceptions.h"
#include "loadgeneratorslave.h"
#include "loadthreadinstance.h"
#include "loadthreadsmanager.h"
#include "loadengineverificationclasses.h"
#include "loadengineverification.h"
#include "loadjsdefs.h"
#include "loadreportinfo.h"

#define MAX_TRNASNAME_LEN 255
//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

IGeneralThreadTransactionManager::IGeneralThreadTransactionManager() {
	m_TransactionTopInstanceIdBase = 0;
  Reset();
	}

IGeneralThreadTransactionManager::~IGeneralThreadTransactionManager() {
	}
/*
WLLogMessage::Severity IGeneralThreadTransactionManager::reportLogMessage(int errId, ...)
{
  va_list ap;
  va_start(ap,errId);
  WLLogMessage::Severity severity = 
    m_MyThreadInstanceScript->m_MyThreadInstance->reportLogMessageVa(LoadExceptionFrmFile, errId, ap);
  va_end(ap);

  return severity;
}  
*/


CLoadTransaction* IGeneralThreadTransactionManager::findTransaction(const char* transactionName)
{
  RVCollectableString trans(transactionName);
  CLoadTransaction* endTrans = (CLoadTransaction*)m_TransactionsStack.find((RVCollectable*)&trans);
  return endTrans;
}

void IGeneralThreadTransactionManager::Reset()
{
  m_RecordLogMessages = true;
  m_ErrorOccured = false;
  m_Save = false;
  m_FailureReason = WLJSNAME_VERIFICATION_NOT_COMPLETED;
  m_TransactionIdBase = 0;
	m_TransactionTopInstanceIdBase += m_TransactionsList.entries();
  m_EventsList.clear();
  m_TransactionsStack.clear();
  m_TransactionsList.clearAndDestroy();
}

void IGeneralThreadTransactionManager::Send() {

  if (!m_Save || !IsSendTransactions()) {
    SendAllEvents();
		}
	else {
		SendWhole();
		}
  Reset();
	}


bool IGeneralThreadTransactionManager::BeginTransaction(const char* name, 
																												double time, 
																												const char* extraDataType, 
																												void* extraData, 
																												unsigned long len) 
{
  //
  // Cutting the name if > MAX_TRNASNAME_LEN chars.
  char tname[MAX_TRNASNAME_LEN+1];
  const char* transname = name;
  if (strlen(name) > MAX_TRNASNAME_LEN) {
    strncpy(tname, name, MAX_TRNASNAME_LEN);
    tname[MAX_TRNASNAME_LEN]=0;
    transname = tname;
  }
/*
  if (findTransaction(transname)) {
  WLErrorException errExcep( LoadExceptionFrmFile, 201,transname );
    return Report( errExcep );
  }
*/
  CLoadTransaction* currTrans = (CLoadTransaction*)m_TransactionsStack.top();
  CLoadTransaction* newTrans = new CLoadTransaction(transname, this);
  newTrans->m_InstanceID = ++m_TransactionIdBase;
	newTrans->m_TopInstanceID = m_TransactionTopInstanceIdBase + m_TransactionIdBase + 1;
  newTrans->m_LoadSize = GetAgendaNumRunning();
  newTrans->m_Reason = "";
  newTrans->m_Time = time - vGetBeginTime();
  newTrans->m_TransactionName = transname;
  newTrans->m_Severity = WLLogMessage::Success;
  newTrans->m_Status = CLoadTransaction::Successful;
	newTrans->m_ExtraDataType = extraDataType;
	newTrans->m_ExtraData.append((const char*)extraData, len);

  if (currTrans) {
    newTrans->m_Level = currTrans->m_Level+1;
    newTrans->m_ParentID = currTrans->m_InstanceID;
  } else {
    newTrans->m_Level = 1;
    newTrans->m_ParentID = 0;
  }

  m_TransactionsList.insert(newTrans);
  m_TransactionsStack.push(newTrans);

	BeginTransactionCallBack(*newTrans);

  return true;
}

bool IGeneralThreadTransactionManager::EndTransaction(const char* name, double time, WLLogMessage::Severity severity, bool save, bool SendTimers, bool IsLog ) 
{
  //
  // Cutting the name if > MAX_TRNASNAME_LEN chars.
  char tname[MAX_TRNASNAME_LEN+1];
  const char* transname = name;
  if (strlen(name) > MAX_TRNASNAME_LEN) {
    strncpy(tname, name, MAX_TRNASNAME_LEN);
    tname[MAX_TRNASNAME_LEN]=0;
    transname = tname;
  }

  bool retStatus;
/*
  CLoadTransaction* endTrans = findTransaction(transname);
  if (!endTrans) {
  WLErrorException errExcep( LoadExceptionFrmFile, 202, transname );
    return Report( errExcep );
  }
*/
  CLoadTransaction* currTrans = (CLoadTransaction*)m_TransactionsStack.top();
  if (!currTrans) {
    WLErrorException errExcep( LoadExceptionFrmFile, 202, transname );
    return Report( errExcep );
  }
  if (strcmp(transname,currTrans->m_TransactionName.data()) != 0) {
    WLErrorException errExcep( LoadExceptionFrmFile, 203,transname, currTrans->m_TransactionName.data() );
    return Report( errExcep);
  }
  currTrans = (CLoadTransaction*)m_TransactionsStack.pop();

  if (save)
    m_Save = true;

  if (severity != WLLogMessage::Success) {
    m_Save = true;
    for (unsigned int i=0; i<m_TransactionsStack.entries(); i++) {
      CLoadTransaction* trans = (CLoadTransaction*)m_TransactionsStack[i];
      trans->m_Status = CLoadTransaction::Marked;
    }
  } 

  currTrans->End(severity, time, save, SendTimers );
  m_RecordLogMessages = false;
  
	EndTransactionCallBack(*currTrans); 

  switch (severity) {
    case WLLogMessage::Success :  
      m_FailureReason = WLJSNAME_VERIFICATION_GENERAL_REASON;
      retStatus =  true;
      break;

    case WLLogMessage::Warning :  
      {
	WLErrorException errExcep( LoadExceptionFrmFile, 204, transname, (const char *)currTrans->m_Reason);
      ReportCond( IsLog, errExcep );
      m_FailureReason = WLJSNAME_VERIFICATION_GENERAL_REASON;
      retStatus =  true;
      }
      break;

    case WLLogMessage::Error :  
      {
	WLErrorException errExcep( LoadExceptionFrmFile, 205, transname, (const char *)currTrans->m_Reason);
      ReportCond( IsLog, errExcep );
      m_FailureReason = currTrans->m_Reason;
      m_ErrorOccured = true;
      retStatus =  false;
      }
      break;

    case WLLogMessage::Fatal :  // severe
      {
	WLErrorException errExcep( LoadExceptionFrmFile, 206, transname, (const char *)currTrans->m_Reason);
      ReportCond( IsLog, errExcep );
      m_FailureReason = currTrans->m_Reason;
      m_ErrorOccured = true;
      retStatus =  false;
      }
      break;

    default:
      break;
  }

  m_RecordLogMessages = true;

  if (m_TransactionsStack.top() == NULL)
    Send();

  return retStatus;
}

bool IGeneralThreadTransactionManager::EndTransactionFailure(const char* name, double time, WLLogMessage::Severity severity, const char* reason, bool SendTimers, bool IsLog, bool isFailureEvent ) 
{
  //
  // Cutting the name if > MAX_TRNASNAME_LEN chars.
  char tname[MAX_TRNASNAME_LEN+1];
  const char* transname = name;
  if (strlen(name) > MAX_TRNASNAME_LEN) {
    strncpy(tname, name, MAX_TRNASNAME_LEN);
    tname[MAX_TRNASNAME_LEN]=0;
    transname = tname;
  }

/*
  CLoadTransaction* endTrans = findTransaction(transname);
  if (!endTrans) {
  WLErrorException errExcep( LoadExceptionFrmFile, 202,transname);
    return Report( errExcep );
  }

  CLoadTransaction* currTrans = (CLoadTransaction*)m_TransactionsStack.top();
  if (endTrans != currTrans) {
  WLErrorException errExcep( LoadExceptionFrmFile, 203,transname, currTrans->m_TransactionName.data());
    return Report( errExcep );
  }
*/
  CLoadTransaction* currTrans = (CLoadTransaction*)m_TransactionsStack.top();
  if (!currTrans) {
    WLErrorException errExcep( LoadExceptionFrmFile, 202, transname );
    return Report( errExcep );
  }
  if (strcmp(transname,currTrans->m_TransactionName.data()) != 0) {
    WLErrorException errExcep( LoadExceptionFrmFile, 203,transname, currTrans->m_TransactionName.data() );
    return Report( errExcep);
  }

  SetFailureReason(reason, NULL, time, isFailureEvent );
  EndTransaction( transname, time, severity, true, SendTimers, IsLog );
  return true; 
}


void IGeneralThreadTransactionManager::RecordEvent(const char* reason, const char* description, double time, WLLogMessage::Severity severity, const char* extraDataType, void* extraData, unsigned long len, bool IsLog, bool IsReport)
{
  if (!m_RecordLogMessages)
    return;

	bool deleteEvent = false;
	CLoadTransactionEvent* transactionEvent = NULL;
  CLoadTransaction* currTrans = (CLoadTransaction*)m_TransactionsStack.top();

  if (!currTrans) {
    if (!IsReport) {
      SendEvent(reason, description, time, severity, extraDataType, extraData, len, IsLog, IsReport);
		}
  } else {

	  if (!IsSendTransactions()) {
			SendEvent(reason, description, time, severity, extraDataType, extraData, len, IsLog, IsReport);
		}
	  transactionEvent = currTrans->RecordEvent(reason, description, time, severity, extraDataType, extraData, len);
	}
	
	if(transactionEvent == NULL) {
		transactionEvent = new CLoadTransactionEvent(reason, description, severity, extraDataType, extraData, len);
		transactionEvent->m_Time = time - vGetBeginTime();
		transactionEvent->m_LineNumber = 1;
		transactionEvent->m_FileName = "None";
		deleteEvent = true;
	}

	if(currTrans != NULL) {
		RecordEventCallBack(*transactionEvent, currTrans->m_TopInstanceID, currTrans->m_InstanceID);
	}
	else {
		RecordEventCallBack(*transactionEvent, 0, 0);
	}

	if(deleteEvent == true) {
		delete transactionEvent;
	}
}

void IGeneralThreadTransactionManager::SetFailureReason(const char* reason, const char* description, double time, bool isEvent ) {
  //
  // Find the current transaction, set failure reason, and 
  // Report the Description as event.
  CLoadTransaction* currTrans = (CLoadTransaction*)m_TransactionsStack.top();
  if (!currTrans) {
    //reportLogMessage(207,"SetFailureReason");
    return; // false;
  }
  m_FailureReason = reason;
  currTrans->m_Reason = reason;

	if( isEvent ) {
		if (description != NULL) {
			currTrans->RecordEvent(reason, description, time, WLLogMessage::Note, NULL, NULL, 0 );
		}
	}
}

void IGeneralThreadTransactionManager::SendFailureForOpenTransactions(bool Failed, bool SendTimers) 
{
  if (m_TransactionsStack.isEmpty())
    return;

  CLoadTransaction* currTrans;
  m_RecordLogMessages = false;
  WLLogMessage::Severity errorSeverity = GetErrorSeverity();
  while ((currTrans = (CLoadTransaction*)m_TransactionsStack.pop()) != NULL) {

    // If in abort, then don't do anything besides popping the transaction
    if (!GetThreadInstance()->isAbort(false)){
      if (Failed) {
        m_FailureReason = WLJSNAME_VERIFICATION_NOT_COMPLETED;

        if (m_ErrorOccured) {
	  WLErrorException errExcep( LoadExceptionFrmFile,208,currTrans->m_TransactionName.data());
          Report( errExcep );
          m_FailureReason .append(" - Error occured in internal transaction.");
        } else if (errorSeverity >= WLLogMessage::Error) {
	  WLErrorException errExcep( LoadExceptionFrmFile,209,currTrans->m_TransactionName.data());
          Report( errExcep );
          m_FailureReason .append(" - Error occured in current transaction.");
          m_ErrorOccured = true;
        } else if (GetThreadInstance()->isAbort(false)) {
	  WLErrorException errExcep( LoadExceptionFrmFile,210,currTrans->m_TransactionName.data());
          Report( errExcep );
          m_FailureReason .append(" - Thread terminated by external request.");
        } else {
	  WLErrorException errExcep( LoadExceptionFrmFile,211,currTrans->m_TransactionName.data());
          Report( errExcep );
        }
        currTrans->m_Reason = m_FailureReason;
      }
      else {
        currTrans->m_Reason = WLJSNAME_VERIFICATION_NOT_COMPLETED;
          WlTraceMsg("Open transaction %s for thread %d", currTrans->m_TransactionName.data(), GetThreadInstance()->GetLGThreadNum());
	  WLErrorException errExcep( LoadExceptionFrmFile,212,currTrans->m_TransactionName.data());
        Report( errExcep );
      }

      m_Save = true;
      currTrans->End(WLLogMessage::Error, vGetCurrentTime(), true, SendTimers );
// @@Chaim - talk to Dudu
//			EndTransactionCallBack(*currTrans); 
    }
  }
  m_RecordLogMessages = true;

  Send();
}

const char* IGeneralThreadTransactionManager::GetCurrentTransaction()
{
  CLoadTransaction* currTrans = (CLoadTransaction*)m_TransactionsStack.top();
  return currTrans->m_TransactionName;
}

int IGeneralThreadTransactionManager::GetTransactionDepth()
{
  return m_TransactionsStack.entries();
}


void IGeneralThreadTransactionManager::InsertEvent(CLoadTransactionEvent *event)
{
  m_EventsList.insert(event);
}

void IGeneralThreadTransactionManager::SendAllEvents()
{
  if (!IsSendTransactions())
    return;

  for (unsigned int i=0; i<m_EventsList.entries(); i++) {
    CLoadTransactionEvent &event = *(CLoadTransactionEvent*)m_EventsList(i);
    //
    // If the event is not a message, insert it to the menager's events list.
    if (!event.m_ExtraDataType.isNull() && event.m_ExtraDataType.compareTo("Message", RVCString::ignoreCase)!=0)
      SendEvent(event);
  }
}

void IGeneralThreadTransactionManager::SendEvent(const char* reason, const char* description, double time, WLLogMessage::Severity severity, const char* extraDataType, void* extraData, unsigned long len, bool IsLog, bool IsReport )
{
	if (!IsSendTransactions()) {
		if (!IsReport) {
			if( IsLog || (WLLogMessage::Fatal==severity) ) {
				GetThreadInstance()->report( severity, description );
			} else {
			  cLoadReportInfoAgenda loadRepInf( severity, description );
			  cLoadReportHard errRep( loadRepInf );
				GetThreadInstance()->reportApply( errRep ); 
			}

		}
	return;
	}

  CLoadTransactionEvent event(reason, description, severity, extraDataType, extraData, len);
	event.m_Time = time - vGetBeginTime();
  event.m_LineNumber = 1;
  event.m_FileName = "None";

  SendEvent(event);
}




bool IGeneralThreadTransactionManager::Report(
	WLErrorException& e ) {
  return GetThreadInstance()->report( e );
	}

bool IGeneralThreadTransactionManager::ReportCond( 
	bool Cond, 
	WLErrorException& e ) {

  bool rtn;
	if(Cond || (WLLogMessage::Fatal==e.GetErrorSeverity())) {
		rtn = Report( e );
	} else {
    rtn = true; 
    cLoadReportInfoAgenda loadRepInf( e.GetErrorSeverity(), e.GetErrorMsg() );
    cLoadReportHard errRep( loadRepInf );
		GetThreadInstance()->reportApply( errRep ); 
	}

  return rtn;
}


int IGeneralThreadTransactionManager::GetAgendaNumRunning() {
	return GetThreadInstance()->GetAgendaNumRunning();
	}


WLLogMessage::Severity IGeneralThreadTransactionManager::GetErrorSeverity() {
	return GetThreadInstance()->getErrorSeverity();
	}



//////////////////////////////////////////////////////////////////
//
//       Implementation of the class CLoadThreadTransactionManager
//
//////////////////////////////////////////////////////////////////
/*
CLoadThreadTransactionManager::CLoadThreadTransactionManager(
	CLoadThreadInstanceScript* ScriptInstance ):
	m_MyThreadInstanceScript(ScriptInstance) {}
*/

void CLoadThreadTransactionManager::InitParent(CLoadThreadInstanceScript* ScriptInstance) {
		m_MyThreadInstanceScript = ScriptInstance;
}

CLoadGeneratorSlave* CLoadThreadTransactionManager::GetLoadGenerator()
{
  return m_MyThreadInstanceScript->m_MyThreadsManagerScript->GetLoadGenerator();
}
/*
CLoadThreadsManager* IGeneralThreadTransactionManager::GetThreadsManager()
{
  return m_MyThreadInstanceScript->m_MyThreadsManagerScript->GetThreadsManager();
}
*/



CLoadGeneralThreadInstance* CLoadThreadTransactionManager::GetThreadInstance() {
	return m_MyThreadInstanceScript->m_MyThreadInstance;
	}



void CLoadThreadTransactionManager::AddLast(
	const char* name,  
	double l, 
	CWLStatPointTypeEnum::PointType type) {
	
	m_MyThreadInstanceScript->addLast( name, l, type );
	}




double CLoadThreadTransactionManager::vGetTimeFromBegin() {
	return GetLoadGenerator()->fGetTimeFromBegin();
	}

double CLoadThreadTransactionManager::vGetBeginTime() {
	return GetLoadGenerator()->fGetBeginTime();
	}

double CLoadThreadTransactionManager::vGetCurrentTime() {
	return GetLoadGenerator()->fGetProcess().fGetCurrentTime();
}


bool CLoadThreadTransactionManager::IsSendTransactions() {
	return GetLoadGenerator()->IsSendTransactions();
	}



void CLoadThreadTransactionManager::SendWhole() {

  GetLoadGenerator()->AddTransactions(m_TransactionsList.entries());

  CAssociativeArray Array;
  CAssociativeArray Message;

  Array.AddCouple("ThreadNum", (long)m_MyThreadInstanceScript->m_MyThreadInstance->GetLGThreadNum() );
  Array.AddCouple("RoundNum", (long)m_MyThreadInstanceScript->m_RoundNum);
  Array.AddCouple("LoadGeneratorName", m_MyThreadInstanceScript->m_MyThreadInstance->GetAgendaName());
  Array.AddCouple("NumTransactions", (long)m_TransactionsList.entries());

  for (unsigned int i=0; i<m_TransactionsList.entries(); i++) {
    CLoadTransaction *trans = (CLoadTransaction*)m_TransactionsList(i);
    Array.AddArrayCouple("Transaction", trans->toString(), i+1);
  }

  Message.AddCouple("$TRANSACTIONTREE",Array.ToString());
  Message.AddCouple("NumTransactions", (long)m_TransactionsList.entries());

  if (GetLoadGenerator()->m_GeneratorType == CLoadGeneratorSlave::StandAlone) 
    PrintTransactionTree(Message.ToString());
  else
    GetLoadGenerator()->SendMessage(Message, 4);

}


void CLoadThreadTransactionManager::SendEvent(CLoadTransactionEvent &event)
{
  CAssociativeArray Message;

  Message.AddCouple("$STANDALONEEVENT",event.toString());
  Message.AddCouple("ThreadNum", (long)m_MyThreadInstanceScript->m_MyThreadInstance->GetLGThreadNum() );
  Message.AddCouple("RoundNum", (long)m_MyThreadInstanceScript->m_RoundNum);
  Message.AddCouple("LoadGeneratorName", m_MyThreadInstanceScript->m_MyThreadInstance->GetAgendaName());

  if (GetLoadGenerator()->m_GeneratorType == CLoadGeneratorSlave::StandAlone) 
    cout << Message.ToString() << endl;
  else
    GetLoadGenerator()->SendMessage(Message, 4);
}


