/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "send2tml.h"
#include "wlerrorlib.h"
#include "tvpnumconversions.h"

#include "associativearray.h"
#include "loadjsshell.h"
#include "loadenginestatistics.h"
#include "loadengineglobals.h"
#include "loadgeneratorslave.h"
#include "loadthreadinstance.h"
#include "loadthreadsmanager.h"
#include "loadexceptions.h"

#include "loadreportinfo.h"
#include "rtamscript.h"
#include "loadengineexternaldecs.h"

#include "wlleprotocol.h"
#include "rvjsembed.h"

//#include <time.h>

//#include "loadgeneratorscript.h"

//////////////////////////////////////////////////////////////
//
//  The implementation of the class CLoadGeneralThreadInstance
//
//////////////////////////////////////////////////////////////

CLoadGeneralThreadInstance::CLoadGeneralThreadInstance():
m_RoundNum(0),
m_InCriticalSection(false)
{
}

CLoadGeneralThreadInstance::~CLoadGeneralThreadInstance() {

}

bool CLoadGeneralThreadInstance::Terminate() {

	return true;
}


bool CLoadGeneralThreadInstance::report(
	WLLogMessage::Severity Severity, 
	const char* Message) 
{
  return LoadReport( vCurrentJSContext(), Severity, Message );
};


bool CLoadGeneralThreadInstance::report(
	WLErrorException& e ) {

	return report( e.GetErrorSeverity(), e.GetErrorMsg() );
}	



bool CLoadGeneralThreadInstance::reportTime(
	WLLogMessage::Severity Severity, 
	const char* Message,
	double Time,
  WLLogMessage::Severity ErrorInTransTree) {

	return LoadReportTime( vCurrentJSContext(), Severity, Message, Time, ErrorInTransTree);
};


bool CLoadGeneralThreadInstance::reportTime(
	WLErrorException& e,
	double Time ) {

	return reportTime( e.GetErrorSeverity(), e.GetErrorMsg(), Time );
	}	



bool CLoadGeneralThreadInstance::reportMultiThread(
	WLErrorException& e ) {
	cLoadReportInfoAgenda errRep( e.GetErrorSeverity(), e.GetErrorMsg() );
	return reportMultiThread( errRep );
}	

bool CLoadGeneralThreadInstance::reportMultiThread(
	WLLogMessage::Severity Severity, 
	const char* Message) {

	cLoadReportInfoAgenda errRep( Severity, Message );
	return reportMultiThread( errRep );
	}	


void CLoadGeneralThreadInstance::reportApplyEvent( 
	WLLogMessage::Severity Severity, 
	const char* Message ) {

	RVCString temp( Message );
	RVCString Reason;
	Reason.insert( 0, temp, 30 );

	GetTransactionManager().RecordEvent( Reason.data(), Message, fGetProcess().fGetCurrentTime(), Severity, "Message", (void *)"" , 0, true, true);
}

////////////////////////////////////////////////////////////////
// Moved from Thread Data
////////////////////////////////////////////////////////////////

// Chaim - Handle the combined IsAbort and InScript check
bool CLoadGeneralThreadInstance::isAbort(bool CheckInScript){
  return (internalIsAbort() && (!CheckInScript || InScript()));
}

void CLoadGeneralThreadInstance::sleep(int milisec)
{
  TVPTimerObject startTime;
  int timePassed;

  while ((timePassed = (int)(1000 * startTime.get())) < milisec && !isAbort())
 	{
#ifdef __TV_UNIX__
#define __min(x,y) ((x) < (y) ? (x) : (y))
    ::rvSleep(__min(milisec-timePassed,1000));
#else
    Sleep(__min(milisec-timePassed,1000));
#endif
 	}
}

void CLoadGeneralThreadInstance::VerificationRecordEvent(const char* reason, const char* description, double time, WLLogMessage::Severity severity, const char* extraDataType, void* extraData, unsigned long len, bool IsLog)
{
  GetTransactionManager().RecordEvent(reason, description, time, severity, extraDataType, extraData, len, IsLog, false);
}

void CLoadGeneralThreadInstance::VerificationSetFailureReason(const char* reason,const char* description, double time, bool isEvent ) {
  GetTransactionManager().SetFailureReason(reason, description, time, isEvent );
}

bool CLoadGeneralThreadInstance::VerificationBeginTransaction(const char* transname, double time, const char* extraDataType, void* extraData, unsigned long len) {
  return GetTransactionManager().BeginTransaction(transname, time, extraDataType, extraData, len);
};

bool CLoadGeneralThreadInstance::VerificationEndTransaction(const char* transname, double time, WLLogMessage::Severity severity, bool save, bool SendTimers, bool IsLog ) {
  
  return GetTransactionManager().EndTransaction(transname, time, severity, save, SendTimers, IsLog );
};

bool CLoadGeneralThreadInstance::VerificationEndTransactionFailure(const char* transname, double time, WLLogMessage::Severity severity, const char* reason, bool SendTimers, bool IsLog, bool isFailureEvent ) {
  
  return GetTransactionManager().EndTransactionFailure(transname, time, severity, reason, SendTimers, IsLog, isFailureEvent);
};

int  CLoadGeneralThreadInstance::VerificationGetTransactionDepth()
{
  return GetTransactionManager().GetTransactionDepth();
}

const char* CLoadGeneralThreadInstance::VerificationGetCurrentTransaction()
{
  return GetTransactionManager().GetCurrentTransaction();
}

void CLoadGeneralThreadInstance::fEnterControlMachineThread()
{
	JSContext* aCx = vCurrentJSContext();
	GetCurrentScript()->fSignContext( aCx );
	JS_SetGlobalObject( aCx, GetGlobalObject() );
}


void CLoadGeneralThreadInstance::fExitControlMachineThread()
{}


bool CLoadGeneralThreadInstance::inCriticalSection(){
	return m_InCriticalSection;
}

void CLoadGeneralThreadInstance::fEnterCriticalSection(){
	m_InCriticalSection = true;
}

void CLoadGeneralThreadInstance::fExitCriticalSection(){

	m_InCriticalSection = false;
}
