/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <RVCString.h>

#ifdef __TV_UNIX__
#include <pthread.h>
#endif

#include "platform.h"
#include "tracedll.h"
#include "tvenv.h"
#include "tvpsync.h"
#include "socketlib.h"

#include "loadengineglobals.h"
#include "loadexceptions.h"
#include "associativearray.h"
#include "loadagendamanager.h"
#include "loadgenerator.h"
#include "loadenginedecs.h"
#include "loadengineinifileproperties.h"
#include "loadengineselistener.h"
//#include <RVCTokenizer.h>

#include "CommandLinePropertie.h"

CLoadGenerator::CLoadGenerator( const char* MonitorName,
                               const char* LoaderType,
                               const char* RealName,
                               const char* LoadGeneratorNo ) :
//m_MessagesCommunicator(this,LoadGeneratorNo)
m_MessagesCommunicator(LoadGeneratorNo),
m_ThreadPoolOn(false),
m_firstcycle(true)
{
  WlTraceMsg("Entering CLoadGenerator::CLoadGenerator (4 args)");
  
  m_LoadGeneratorNo = LoadGeneratorNo;
  m_AgendaManager   = NULL;
  m_MonitorName     = MonitorName;
  m_LoaderType      = LoaderType;
  m_RealName        = RealName;
  m_SessionNo       = "0";
  m_LoaderStatus    = "RUN";
  m_StartTime       = 0;
  m_Terminated      = false;
  m_StopMeDone      = false;
  
  m_GeneratorType   = Regular;
  
  m_SendingLogMessages = true;
  m_LogMessagesCounter = 0;
  m_LogMessagesMax = CLEIniFileProperties::LogMessagesMax;
	m_MustLogBaseSeverity = WLLogMessage::Fatal;
  m_SendingTransactions = true;
  m_TransactionsCounter = 0;
  m_TransactionsMax = CLEIniFileProperties::TransactionMax;
  m_StatisticalMesurementsMask = 0;
  
  m_MessagesCommunicator.Init();

	m_StopAllTime = -1;

	m_MasterPid = TVPgetpid();

	m_MessagesCommunicator.InitParent(this);
  //
	// @@Chaim - This code is duplicated here and in the other CLoadGenerator ctor
  // because we can't do this in CGeneralProcess. The creation of temporary
  // directories is once per process in the LE and once per session in the PC.
	char path[WL_MAX_PATH];
	strcpy(path,CArrangeMachineTempFiles::GetWlTempPath(m_MasterPid).data());
	RVCString rwPath(path);

	TVPRemoveAllDirectories(rwPath);
	CArrangeMachineTempFiles::LECreateDirectory(path);

  // Init Messages.
  CAssociativeArray InitMessage;
  InitMessage.AddCouple("$REALNAMERUNNING",1L);
  InitMessage.AddCouple("$REALNAME",m_RealName);
  
  SendMessage(InitMessage, 2);
  
  CAssociativeArray* SessionInitMessage = NULL;
  SessionInitMessage = GetMessage();
  
  m_SessionNo = SessionInitMessage->GetValueOfParameter("$SESSIONID");
  m_MonitorUID = SessionInitMessage->GetValueOfParameter("$MONITORUID");
  if( NULL == m_MonitorUID.data() )
		m_MonitorUID = "";
  if( NULL == m_MonitorUID.data() )
		m_MonitorUID = "";

  /*GALTECH:*/
#ifndef _ANSI_ONLY
  if ( SessionInitMessage->KeyExists( "$MONITORCHARSET" ) )
    SetMonitorCharSet(SessionInitMessage->GetLongValueOfParameter("$MONITORCHARSET"));
#endif

  m_MonitorGlobalVariablesAddress = SessionInitMessage->GetValueOfParameter("$MONITORGLOBALSNAME");
  delete SessionInitMessage;
  
  CAssociativeArray InitMessage2;
  InitMessage2.AddCouple("$STATUS","READY");
  InitMessage2.AddCouple("$MONITOR",m_MonitorName);
  SendMessage(InitMessage2, 2);
  
  WLERRGetErrorFormat(LoadExceptionFrmFile, 10);
  WlTraceMsg("Exiting CLoadGenerator::CLoadGenerator (4 args)");

}

CLoadGenerator::CLoadGenerator( const char* ScriptFile, 
                               int ReportResolution) :
//m_MessagesCommunicator(this,"1")
m_MessagesCommunicator("1"),
m_ThreadPoolOn(false),
m_firstcycle(true)
{
  m_LoadGeneratorNo = "1";
  m_AgendaManager   = NULL;
  m_LoaderType      = "loadserver";
  m_RealName        = "blabla";
  m_SessionNo       = "0";
  m_MonitorUID		= "";
  m_MonitorUID		= "";
  m_LoaderStatus    = "RUN";
  m_StartTime       = 0;
  m_Terminated       = false;
  m_StopMeDone      = false;
  
  m_MonitorName     = "";
  m_GeneratorType = StandAlone;
  
  m_SendingLogMessages = false;
  m_LogMessagesCounter = 0;
  m_LogMessagesMax = CLEIniFileProperties::LogMessagesMax;
	m_MustLogBaseSeverity = WLLogMessage::Fatal;
  m_SendingTransactions = true;
  m_TransactionsCounter = 0;
  m_TransactionsMax = CLEIniFileProperties::TransactionMax;
  m_StatisticalMesurementsMask = 0;
  m_MonitorGlobalVariablesAddress = "STGlobalsSender1@eyal:9000";

	m_StopAllTime = -1;

	m_MasterPid = TVPgetpid();

	m_MessagesCommunicator.InitParent(this);
  //
  // Do StartSession and Startagenda.
  CAssociativeArray ScriptsArray;
  //ScriptsArray.AddCouple("$LOADCLIENTAGENDA",1L);
  ScriptsArray.AddCouple("$LOADAGENDA",1L);
  ScriptsArray.AddCouple("$AGENDANAME", m_RealName.data());
  ScriptsArray.AddCouple("$AGENDAPATH", "blablapath");
  ScriptsArray.AddCouple("$NUMSCRIPTS",1L);
  ScriptsArray.AddCouple("$SPAWNING_MECHANISM",1L);
  ScriptsArray.AddCouple("$DEFAULT_MAX_THREADS_PER_SLAVE",CLEIniFileProperties::LoadGeneratorMaxThreadsPerSlave);
  ScriptsArray.AddArrayCouple("$SCRIPTNAME","TestScript",1);
  ScriptsArray.AddArrayCouple("$SCRIPTFILE",ScriptFile,1);
  ScriptsArray.AddArrayCouple("$SCRIPTWEIGHT",100L,1);
#ifdef __TV_UNIX__
  ScriptsArray.AddCouple("$UNIX_MAX_THREADS_PER_SLAVE",200L);
#endif  // __TV_UNIX__
#ifdef __TV_linux__
  ScriptsArray.AddCouple("$LINUX_MAX_THREADS_PER_SLAVE",200L);
#endif // __TV_linux__
  
	char path[WL_MAX_PATH];
	strcpy(path,CArrangeMachineTempFiles::GetWlTempPath(m_MasterPid).data());
	RVCString rwPath(path);

	TVPRemoveAllDirectories(rwPath);
	CArrangeMachineTempFiles::LECreateDirectory(path);

  LoadThreadsManager(1, "TestAgenda", "TestAgendaFile", ScriptsArray);
  //LoadOneClientThreadsManager("TestAgenda", ScriptsArray);
  
  CAssociativeArray StartSessionMessage;
  StartSessionMessage.AddCouple("$STARTSESSION", 1L);
  //StartSessionMessage.AddCouple("$DONTSENDTIMERS", 1L);
  //StartSessionMessage.AddCouple("$DONTSENDTRANSACTIONTIMERS", 1L);
  StartSessionMessage.AddCouple("$LOGMESSAGESMAX", 10000000L);
  StartSessionMessage.AddCouple("$TRANSACTIONSMAX", 12L);
  StartSessionMessage.AddCouple("$REPORTRESOLUTION", (long)ReportResolution);
  StartSession(TVPSysTime(), StartSessionMessage);
  
  // preload message files
  WLERRGetErrorFormat(LoadExceptionFrmFile, 1);
  WLERRGetErrorFormat(JSErrorsFrmFile, 39);
  WLERRGetErrorFormat(JSSyntaxChecks, 1);
}

CLoadGenerator::~CLoadGenerator()
{
  if (m_AgendaManager){
    m_AgendaManager->m_SlaveProxyArray.clearAndDestroy();
    delete m_AgendaManager;
  }
}

CAssociativeArray* CLoadGenerator::GetMessage() 
{ 
  switch (m_GeneratorType) {
  case Regular:
    return (CAssociativeArray*)m_ControlMessageQueue.waitEvent(); 
    
  case StandAlone:
		if( !getCommandLineProperties().batchMode )
		{
    return LGStandAlone(this);
		}	else
		{
			if( m_firstcycle )
			{
				CAssociativeArray* retArray = new CAssociativeArray;
				retArray->AddCouple("$STARTAGENDA", 1L);
				retArray->AddCouple("$AGENDANAME", "TestAgenda");
				retArray->AddCouple("$AGENDAPATH", "TestAgendaPath");
				retArray->AddCouple("$AGENDAACTUALRUNNING", long(getCommandLineProperties().virtualClients));
				SetSendLogMessage(getCommandLineProperties().messagesOn);
				m_firstcycle = false;
				return retArray;
			}else {
				return (CAssociativeArray*)m_ControlMessageQueue.waitEvent(); 
			}
		}
    
  default:
    WlAssert(NULL);
    return NULL;
  }
}

void CLoadGenerator::MainLoop()
{
	CTraceMsg::RegisterThread ();
  
#ifdef __TV_WINDOWS__  
  HANDLE hThread = GetCurrentThread();
  SetThreadPriority(hThread,15);
#endif
#ifdef __TV_UNIX__
  sched_param  sc;
#ifdef __TV_solaris__
  sc.sched_priority = 127;
#else
  sc.sched_priority = 127;
#endif
  pthread_t th = pthread_self();
  
  pthread_setschedparam(th,SCHED_OTHER,&sc);
#endif
  
  // Get ThreadPool flag from ini file here; later, we also check in Associative Array.
  m_ThreadPoolOn = CLEIniFileProperties::ThreadPoolOn;

  CAssociativeArray *AssoArrayCmd = NULL;
  
  while ( true )
  {
	  try { LE_SET_SE_TRANSLATOR();
			// Set structured exception translator
			
      if (AssoArrayCmd)
        delete AssoArrayCmd;
      AssoArrayCmd = GetMessage();
      CAssociativeArray& AssoArrayCommand = *AssoArrayCmd;
      
      WlTraceMsg("CLoadGenerator::MainLoop - Received command (%s)",AssoArrayCommand.ToString());
      if ( AssoArrayCommand.KeyExists( "$STARTSESSION" ) )
      {
        CLog::Write("$STARTSESSION", eCommand );
        // @@Chaim - LoadGeneratorReset
        // Get time from message
        double startTime;
        if (AssoArrayCommand.KeyExists( "$STARTTIME" ) ){
          startTime = AssoArrayCommand.GetDoubleValueOfParameter("$STARTTIME");
        }
        else {
          startTime = TVPSysTime();
        }
        StartSession( 
          startTime, 
          AssoArrayCommand
          );
        m_LoaderStatus = "RUN";
        ReportLoadGeneratorMessage("RUN");
        
      }
      
      else if ( AssoArrayCommand.KeyExists( "$LOADAGENDA" ) )
      {
        CLog::Write("$LOADAGENDA", eCommand );
        // Check for ThreadPool flag in message (only if not already set in ini)
        // Check value of threadpool flag in message only on Linux
#ifdef __TV_linux__ 
        if (!m_ThreadPoolOn)
          m_ThreadPoolOn = (bool)AssoArrayCommand.GetLongValueOfParameter("$THREADPOOLON", 0);
#endif
        
        LoadThreadsManager( 
          AssoArrayCommand.GetLongValueOfParameter("$AGENDASIZE") ,
          AssoArrayCommand.GetValueOfParameter("$AGENDANAME") ,
          AssoArrayCommand.GetValueOfParameter("$AGENDAPATH") ,
          AssoArrayCommand
          );
        
      }
      
      else if ( AssoArrayCommand.KeyExists( "$STARTAGENDA" ) )
      {
        CLog::Write("$STARTAGENDA", eCommand );
        StartThreadsManager( 
          AssoArrayCommand.GetValueOfParameter("$AGENDANAME") ,
          AssoArrayCommand.GetLongValueOfParameter("$AGENDAACTUALRUNNING")
          );
      }
      
      else if ( AssoArrayCommand.KeyExists( "$THROTTLEAGENDA" ) )
      {
        CLog::Write("$THROTTLEAGENDA", eCommand );
        ThrottleThreadsManager( 
          AssoArrayCommand.GetValueOfParameter("$AGENDANAME") ,
          AssoArrayCommand.GetLongValueOfParameter("$THROTTLENUMRUNNING")
          );
      }
      
      else if ( AssoArrayCommand.KeyExists( "$UNTHROTTLEAGENDA" ) )
      {
        CLog::Write("$UNTHROTTLEAGENDA", eCommand );
        UnThrottleThreadsManager( AssoArrayCommand.GetValueOfParameter("$AGENDANAME") );
      }
      
      else if ( AssoArrayCommand.KeyExists( "$PAUSEAGENDA" ) )
      {
        CLog::Write("$PAUSEAGENDA", eCommand );
        PauseThreadsManager( AssoArrayCommand.GetValueOfParameter("$AGENDANAME") );
      }
      
      else if ( AssoArrayCommand.KeyExists( "$USERPAUSEAGENDA" ) )
      {
        CLog::Write("$USERPAUSEAGENDA", eCommand );
        UserPauseThreadsManager( AssoArrayCommand.GetValueOfParameter("$AGENDANAME") );
      }
      
      else if ( AssoArrayCommand.KeyExists( "$GOAGENDA" ) )
      {
        CLog::Write("$GOAGENDA", eCommand );
        GoThreadsManager( AssoArrayCommand.GetValueOfParameter("$AGENDANAME") );
      }
      
      else if ( AssoArrayCommand.KeyExists( "$PAUSELOADER" ) )
      {
        CLog::Write("$PAUSELOADER", eCommand );
        PauseLoader();
        m_LoaderStatus = "PAUSE";
        ReportLoadGeneratorMessage("PAUSE");
      }
      
      else if ( AssoArrayCommand.KeyExists( "$GOLOADER" ) )
      {
        CLog::Write("$GOLOADER", eCommand );
        GoLoader();
        m_LoaderStatus = "RUN";
        ReportLoadGeneratorMessage("RUN");
      }
      
      else if ( AssoArrayCommand.KeyExists( "$STOPAGENDA" ) )
      {
        CLog::Write("$STOPAGENDA", eCommand );
        StopThreadsManager( AssoArrayCommand.GetValueOfParameter("$AGENDANAME") );
      }
      
      else if ( AssoArrayCommand.KeyExists( "$EXIT" ) || AssoArrayCommand.KeyExists( "$EXITTML" ) )
      {
        CLog::Write("$EXIT", eCommand );
				m_StopAllTime = GetTimeFromBeginning();
        WlTraceMsg("CLoadGenerator::MainLoop - Recieved exit command");
        if (m_Terminated)
          break;
        
        m_Terminated = true;
        m_LoaderStatus = "STOP";
        if (m_AgendaManager)
          StopSession();
        ReportLoadGeneratorMessage("STOP");
        m_MessagesCommunicator.Terminate();
        
        if (AssoArrayCmd)
          delete AssoArrayCmd;
       
				char pathTemp[WL_MAX_PATH];
				strcpy(pathTemp,CArrangeMachineTempFiles::GetWlTempPath(m_MasterPid).data());
				RVCString path(pathTemp);

				TVPRemoveAllDirectories(path);

					WlTraceMsg("CLoadGenerator::MainLoop - Exit done");
        
        return;
        
      }
      
      else if ( AssoArrayCommand.KeyExists( "$STOPFORRESET" ) )
      {
        CLog::Write("$STOPFORRESET", eCommand );
				//m_StopAllTime = GetTimeFromBeginning();
        WlTraceMsg("CLoadGenerator::MainLoop - Received STOP FOR RESET command");
        if (m_Terminated)
          break;
        
        m_Terminated = true;
        m_LoaderStatus = "STOP";
        if (m_AgendaManager)
          m_AgendaManager->StopAgenda();
        
        //if (m_AgendaManager)
        //  StopSession();

        //A ReportLoadGeneratorMessage("STOP");
        m_MessagesCommunicator.Terminate();
        
        if (AssoArrayCmd)
          delete AssoArrayCmd;
       
				char pathTemp[WL_MAX_PATH];
				strcpy(pathTemp,CArrangeMachineTempFiles::GetWlTempPath(m_MasterPid).data());
				RVCString path(pathTemp);

				TVPRemoveAllDirectories(path);

			  WlTraceMsg("CLoadGenerator::MainLoop - STOP FOR RESET done");
        
        return;
        
      }
      


      else if ( AssoArrayCommand.KeyExists( "$LOADCLIENTAGENDA" ) )
      {
        CLog::Write("$LOADCLIENTAGENDA", eCommand );
        LoadOneClientThreadsManager( 
          AssoArrayCommand.GetValueOfParameter("$AGENDANAME") ,
          AssoArrayCommand.GetValueOfParameter("$AGENDAPATH") ,
          AssoArrayCommand
          );
      }
      
      else if ( AssoArrayCommand.KeyExists( "$STARTCLIENTAGENDA" ) )
      {
        CLog::Write("$STARTCLIENTAGENDA", eCommand );
        StartThreadsManager(m_OneClientThreadsManagerName,1);
      }
      
      else if ( AssoArrayCommand.KeyExists( "$PAUSECLIENTAGENDA" ) )
      {
        CLog::Write("$PAUSECLIENTAGENDA", eCommand );
        UserPauseThreadsManager(m_OneClientThreadsManagerName);
      }
      
      else if ( AssoArrayCommand.KeyExists( "$GOCLIENTAGENDA" ) )
      {
        CLog::Write("$GOCLIENTAGENDA", eCommand );
        GoThreadsManager(m_OneClientThreadsManagerName);
      }
      
      else if ( AssoArrayCommand.KeyExists( "$STOPSENDINGLOGMESSAGES" ) )
      {
        CLog::Write("$STOPSENDINGLOGMESSAGES", eCommand );
        StopSendingLogMessages();
      }
      
      else if ( AssoArrayCommand.KeyExists( "$STOPSENDINGTRANSACTIONS" ) )
      {
        CLog::Write("$STOPSENDINGTRANSACTIONS", eCommand );
        StopSendingTransactions();
      }
      
      else
      {
        WlTraceMsg("Illegal command (%s)",AssoArrayCmd->ToString());
        //SendMessage( WhoAmI(), "Wrong command: " + AssoArrayString 
      }
      
      } catch (std::exception* e) {
				ExceptionHandler(e);
				delete e;		// Only for catch of pointer

      } catch (std::exception& e) {
				ExceptionHandler(&e);
          
      } catch (...) {
        try {
          WLErrorException e("LoadGenerator::MainLoop - Exception not handled with command (%s)",WLLogMessage::Fatal, AssoArrayCmd->ToString());
          WlTraceMsg(e.GetErrorMsg());
          CAssociativeArray ErrorMessage;
          ErrorMessage.AddCouple("$GENERATORLOGMESSAGE",e.GetErrorMsg());
          ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
          ErrorMessage.AddCouple("$TIME", GetTimeFromBeginning());
          SendMessage(ErrorMessage, 2);
					WlTraceMsg("call stopMe %s",ErrorMessage.ToString());
          StopMe();
        } catch (...) {
          WlAssert(0);
        }
        WlAssert(0);
      }
  }
}

void CLoadGenerator::ExceptionHandler(std::exception* e){

  // Check if WLErrorException and if reported
  WLErrorException* aException;

  if (aException = dynamic_cast<WLErrorException*>(e)){
    if (aException->GetReported()){
      StopMe(); // Already reported, just stop
      return;
    }
  }

	RVCString FullExcName ("");
	WLERRGetErrorMsg(e, FullExcName);
	RVCString StackTrace("");
	if (WLERRGetStackTrace(e, StackTrace) && StackTrace.length() > 0){
	  WlErrorMsg("CLoadGenerator::MainLoop - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	  printf("CLoadGenerator::MainLoop - Exception %s occured\nTrace:\n%s", FullExcName, StackTrace);
	}
	else {
	  WlErrorMsg("CLoadGenerator::MainLoop - Exception %s occured", FullExcName);
	  printf("CLoadGenerator::MainLoop - Exception %s occured", FullExcName);
	}
	CAssociativeArray ErrorMessage;
  ErrorMessage.AddCouple("$GENERATORLOGMESSAGE",FullExcName);
  ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
  ErrorMessage.AddCouple("$TIME", GetTimeFromBeginning());
  SendMessage(ErrorMessage, 2);
 
  WlTraceMsg("call stopMe %s",ErrorMessage.ToString());
  StopMe();
}

double CLoadGenerator::GetTimeFromBeginning() {
  if (m_StartTime == 0)
    return 0;
  else
    return TVPSysTime()-m_StartTime;
}

double CLoadGenerator::GetBeginningTime() 
{
  return m_StartTime;
}

const char* CLoadGenerator::WhoAmI()
{
  return m_MessagesCommunicator.WhoAmI();
}

CLoadAgendaManager* CLoadGenerator::GetAgendaManager( const char* ThreadsManagerName ) 
{
  return m_AgendaManager;
}
CLoadAgendaManager* CLoadGenerator::GetAgendaManager() 
{
  return m_AgendaManager;
}

void CLoadGenerator::SendMessage( CAssociativeArray& Message, int priority )
{
  switch (m_GeneratorType) {
  case Regular:
    Message.AddCouple("$FROM",WhoAmI());
    Message.AddCouple("$SESSIONID",m_SessionNo);
    //WlTraceMsg("**** : %s",Message.ToString());
    m_MessagesCommunicator.SendMessage(Message, m_MonitorName, "", priority);
    break;
  case StandAlone:
    if (Message.KeyExists("$AGENDASTATUS")) 
      cout << "Status: " << Message["$AGENDASTATUS"] << " NumRunning: " << Message["$AGENDARUNNING"] << endl;
    break;
  default:
		WlAssert(0);
    break;
  }
}

void CLoadGenerator::SendMessageToAddress(CAssociativeArray& Message, const char* netAddress, int priority)
{
  switch (m_GeneratorType) {
  case Regular:
    Message.AddCouple("$SESSIONID",m_SessionNo);
    m_MessagesCommunicator.SendMessage(Message, netAddress, "", priority);
    break;
  case StandAlone:
    if (Message.KeyExists("$AGENDASTATUS")) 
      cout << "Status: " << Message["$AGENDASTATUS"] << " NumRunning: " << Message["$AGENDARUNNING"] << endl;
    break;
  default:
		WlAssert(0);
    break;
  }
}

void CLoadGenerator::StopMe()
{
	WlTraceMsg("CLoadGenerator::StopMe(%s) invoked", m_RealName);

  if (m_StopMeDone || m_Terminated)
    return;
  
  m_StopMeDone = true;
  CAssociativeArray *StopMessage = new CAssociativeArray;
  StopMessage->AddCouple("$EXITTML", 1L);
  StopMessage->AddCouple("$FROM",WhoAmI());
  PutMessage(StopMessage);
	
	ForceKillLoadGenerator();
}

void CLoadGenerator::ReportLoadGeneratorMessage( const char* Status )
{
  CAssociativeArray ReportMessage;
  ReportMessage.AddCouple("$STATUS",Status);
  
  if (strcmp(Status,"STOP"))
    SendMessage(ReportMessage, 2);
  else
    SendMessage(ReportMessage, 5);
}

void CLoadGenerator::StartSession( double StartTime, CAssociativeArray& AssoArrayCommand)
{
  m_StartTime = StartTime;
  AssoArrayCommand.AddCouple("$STARTTIME", StartTime);
  m_AgendaManager->StartSession(AssoArrayCommand);
  m_AgendaManager->ReportAgenda();
  
  // New Parameters for transactions, log-messages, and timer sending instructions.
  if ( AssoArrayCommand.KeyExists( "$DONTSENDTIMERS" ) )
    m_StatisticalMesurementsMask |= (int)CWLStatPointTypeEnum::Timer;
  if ( AssoArrayCommand.KeyExists( "$DONTSENDTRANSACTIONTIMERS" ) )
    m_StatisticalMesurementsMask |= (int)CWLStatPointTypeEnum::TransactionTimer;
  if ( AssoArrayCommand.KeyExists( "$LOGMESSAGESMAX" ) )
    m_LogMessagesMax = AssoArrayCommand.GetLongValueOfParameter("$LOGMESSAGESMAX");
  if ( AssoArrayCommand.KeyExists( "$TRANSACTIONSMAX" ) ) {
    m_TransactionsMax = AssoArrayCommand.GetLongValueOfParameter("$TRANSACTIONSMAX");

    if (m_TransactionsMax==0)
      m_SendingTransactions = false;
  }
	if ( AssoArrayCommand.KeyExists( "$ALWAYSLOGERRORS" ) )
		m_MustLogBaseSeverity = WLLogMessage::Error;
}

void CLoadGenerator::StopSession()
{
  m_AgendaManager->StopAgenda();
  m_AgendaManager->ReportAgenda();
}

void CLoadGenerator::PauseLoader()
{
  m_AgendaManager->UserPauseAgenda();
  m_AgendaManager->ReportAgenda();
}

void CLoadGenerator::GoLoader()
{
  m_AgendaManager->GoAgenda();
  m_AgendaManager->ReportAgenda();
}

void CLoadGenerator::LoadThreadsManager( long ThreadsManagerSize, 
                                        const char* ThreadsManagerName,
                                        const char* ThreadsManagerRealName,
                                        CAssociativeArray& ScriptsArray)
{
  if ( m_AgendaManager == NULL ) {
    m_AgendaManager = new CLoadAgendaManager(ThreadsManagerName, CalcMaxThreadsPerSlave(ScriptsArray));
  }
  
  CLoadAgendaManager* pAgendaManager = m_AgendaManager;
  
  pAgendaManager->m_pMyLoadGenerator = this;
  pAgendaManager->m_Name = ThreadsManagerName;
  pAgendaManager->m_RealName = ThreadsManagerRealName;
  
  pAgendaManager->m_LoadAgendaMessageString = ScriptsArray.ToString();
  
  if (m_ThreadPoolOn){

    // Get thread pool size from ini; if not found, try to get from ScriptsArray
    if (!(ThreadsManagerSize = CLEIniFileProperties::InitialThreadPoolSize)){
      long tmSize = (long) ScriptsArray.GetDoubleValueOfParameter("$INITIAL_THREADPOOL_SIZE", 0);
      if (tmSize != 0)
        ThreadsManagerSize = tmSize;
    }
  }

  if (!(pAgendaManager->LoadAgenda(ThreadsManagerSize))){
    throw WLErrorException(true, LoadExceptionFrmFile, 31, ThreadsManagerName);
  }
  
  pAgendaManager->ReportAgenda();
}

void CLoadGenerator::LoadOneClientThreadsManager(const char* ThreadsManagerName,
                                                 const char* ThreadsManagerRealName,
                                                 CAssociativeArray& ScriptsArray)
{
  if ( m_AgendaManager == NULL ) {
    m_AgendaManager = new CLoadAgendaManager(ThreadsManagerName, CalcMaxThreadsPerSlave(ScriptsArray));
  }
  
  CLoadAgendaManager* pAgendaManager = m_AgendaManager;
  
  pAgendaManager->m_pMyLoadGenerator = this;
  pAgendaManager->m_Name = ThreadsManagerName;
  pAgendaManager->m_RealName = ThreadsManagerRealName;
  
  pAgendaManager->m_LoadAgendaMessageString = ScriptsArray.ToString();
  
  if (!(pAgendaManager->LoadOneClientAgenda())) {
    throw WLErrorException(LoadExceptionFrmFile, 31, ThreadsManagerName);
  }
  
  pAgendaManager->ReportAgenda();
}

void CLoadGenerator::StartThreadsManager( const char*  ThreadsManagerName,
                                         int          NumRunning )
{
  if (m_Terminated) return;
  CLoadAgendaManager* pAgendaManager = GetAgendaManager(ThreadsManagerName);
  
  if (!pAgendaManager->StartAgenda(NumRunning)) {
    //throw Error;
  }
  
  pAgendaManager->ReportAgenda();
}

void CLoadGenerator::ThrottleThreadsManager( const char*  ThreadsManagerName,
                                            int           NumRunning )
{
  if (m_Terminated) return;
  CLoadAgendaManager* pAgendaManager = GetAgendaManager(ThreadsManagerName);
  
  if (!pAgendaManager->ThrottleAgenda(NumRunning)) {
    //throw Error;
  }
  
  pAgendaManager->ReportAgenda();
}

void CLoadGenerator::UnThrottleThreadsManager( const char*  ThreadsManagerName )
{
  if (m_Terminated) return;
  CLoadAgendaManager* pAgendaManager = GetAgendaManager(ThreadsManagerName);
  
  if (!pAgendaManager->UnThrottleAgenda()) {
    //throw Error;
  }
  
  pAgendaManager->ReportAgenda();
}

void CLoadGenerator::PauseThreadsManager( const char* ThreadsManagerName )
{
  CLoadAgendaManager* pAgendaManager = GetAgendaManager(ThreadsManagerName);
  
  if (!pAgendaManager->StartAgenda(0)) {
    //throw Error;
  }
  
  pAgendaManager->ReportAgenda();
}


void CLoadGenerator::UserPauseThreadsManager( const char* ThreadsManagerName )
{
  CLoadAgendaManager* pAgendaManager = GetAgendaManager(ThreadsManagerName);
  
  if (!pAgendaManager->UserPauseAgenda()) {
    //throw Error;
  }
  
  pAgendaManager->ReportAgenda();
}


void CLoadGenerator::GoThreadsManager( const char*   ThreadsManagerName )
{
  CLoadAgendaManager* pAgendaManager = GetAgendaManager(ThreadsManagerName);
  
  if (!pAgendaManager->GoAgenda()) {
    //throw Error;
  }
  
  pAgendaManager->ReportAgenda();
}


void CLoadGenerator::StopThreadsManager( const char*   ThreadsManagerName )
{
  CLoadAgendaManager* pAgendaManager = GetAgendaManager(ThreadsManagerName);
  
  if (!pAgendaManager->StopAgenda()) {
    //throw Error;
  }
  
  pAgendaManager->ReportAgenda();
}

void CLoadGenerator::StopSendingLogMessages() {
  m_SendingLogMessages = false;
  CLoadAgendaManager* pAgendaManager = GetAgendaManager();
  pAgendaManager->StopSendingLogMessages();
}
void CLoadGenerator::StopSendingTransactions() {
  m_SendingTransactions = false;
  CLoadAgendaManager* pAgendaManager = GetAgendaManager();
  pAgendaManager->StopSendingTransactions();
}

//
// Functions for eliminating sending Messages, Transactions 
// and Statistical Measurements.
bool CLoadGenerator::IsSendLogMessage()
{
  return m_SendingLogMessages;
}

void CLoadGenerator::SetSendLogMessage(bool send)
{
  m_SendingLogMessages = send;
}

void CLoadGenerator::AddLogMessage()
{
  m_LogMessagesCounter++;

  if (m_LogMessagesMax != -1 && m_LogMessagesCounter >= m_LogMessagesMax)
    m_SendingLogMessages = false;
}

bool CLoadGenerator::IsSendTransactions()
{
  return m_SendingTransactions;
}

void CLoadGenerator::AddTransactions(int count)
{
  m_TransactionsCounter += count;
  if (m_TransactionsCounter >= m_TransactionsMax)
    m_SendingTransactions = false;
}

bool CLoadGenerator::IsSendStatisticalMesurement(CWLStatPointTypeEnum::PointType type)
{
  return (m_StatisticalMesurementsMask & (int)type) == 0;
}

long CLoadGenerator::CalcMaxThreadsPerSlave(CAssociativeArray& AssoArray) {
  long MaxThreadsPerSlave = 0;
  if (AssoArray.KeyExists("$SPAWNING_MECHANISM")) {
    if (AssoArray.GetLongValueOfParameter("$SPAWNING_MECHANISM") == 1) {

      // Kept old name of Solaris spawning for compatability for UNIX platforms
#ifdef __TV_UNIX__
	    MaxThreadsPerSlave = AssoArray.GetLongValueOfParameter("$UNIX_MAX_THREADS_PER_SLAVE");
#endif

#ifdef __TV_linux__
	    MaxThreadsPerSlave = AssoArray.GetLongValueOfParameter("$LINUX_MAX_THREADS_PER_SLAVE");
#endif

#ifdef __TV_WINDOWS__
      SYSTEM_INFO lpSystemInfo;
      GetSystemInfo(&lpSystemInfo);
      int ProcessorsNum = lpSystemInfo.dwNumberOfProcessors;
      char buf[50];
      sprintf(buf,"$NT_%dPROCESSORS_MAX_THREADS_PER_SLAVE",ProcessorsNum);
      if (AssoArray.KeyExists(buf)) {
        MaxThreadsPerSlave = AssoArray.GetLongValueOfParameter(buf);
			}
#endif
      if (MaxThreadsPerSlave == 0) {
        MaxThreadsPerSlave = AssoArray.GetLongValueOfParameter("$DEFAULT_MAX_THREADS_PER_SLAVE");
			}
    }
  }
  if (MaxThreadsPerSlave == 0) {
    MaxThreadsPerSlave = 100; 
	}
  AssoArray.AddCouple("$MAXTHREADSPERSLAVE",MaxThreadsPerSlave);
  SetMaxThreadsPerSlave(MaxThreadsPerSlave);
  return MaxThreadsPerSlave;
}

void CLoadGenerator::ForceKillLoadGenerator() {
	CAssociativeArray killMsg;
	char localHostName[100] = "localhost";

    CSocketLib::GetLocalHostName(localHostName, 100);
	CTestTalkAddress sendTo("theTml", localHostName);

	killMsg.AddCouple("$KILLLOADGENERATOR", "");
	killMsg.AddCouple("$MONITORUID",m_MonitorUID.data() );

	SendMessageToAddress(killMsg, sendTo.Address(), 1);
}

