/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include "loadinputfileinterface.h"
#include <time.h>
#include "loadreportinfo.h"
#include "loadexceptions.h"
#include "gt_unicode.h"


CInputLineBuf::CInputLineBuf():
	mIfLineNumber(false),
	mIfFileRound(false) {}

CInputLineBuf::~CInputLineBuf() {}


void CInputLineBuf::LetLine(
	const char* Value ) {
	mLine = Value;
	}

void CInputLineBuf::LetLine(
	const RVCString& Value ) {
	mLine = Value;
	}

void CInputLineBuf::LetLineNumber(
	long Value ) {
	mIfLineNumber = true;
	mLineNumber = Value;
	}

void CInputLineBuf::LetFileRound(
	long Value ) {
	mIfFileRound = true;
	mFileRound = Value;
	}

const char* CInputLineBuf::GetLine() {
	return mLine.data();
	}

bool CInputLineBuf::IfLineNumber() {
	return mIfLineNumber;
	}

long CInputLineBuf::GetLineNumber() {
	return mLineNumber;
	}

bool CInputLineBuf::IfFileRound() {
	return mIfFileRound;
	}

long CInputLineBuf::GetFileRound() {
	return mFileRound;
	}



IFileDictionary::~IFileDictionary() {}


CLoadFileDictionary::CLoadFileDictionary(
	IGeneralManagerScriptData* ScriptData ):
	m_Files(ScriptData) {}


bool CLoadFileDictionary::AddFile(
	const char* FileName,
	bool Random,
	JSContext* ) {
	m_Files.AddFile( FileName, Random ); // has no bad status
	return true;
	}

bool CLoadFileDictionary::ReadLine(
	const char* FileName,
	IInputLineOutpar& LineBuf,
	iLEAgendaJSEmbedContext* embedContext,
	JSContext* ) {

	bool Rtn;

  CWLInputLine* Line = m_Files.GetNextLine(FileName,embedContext);

  if (Line) {
		Rtn = true;
	  LineBuf.LetLine( Line->GetLine() );
	  LineBuf.LetLineNumber( Line->GetLineNumber() );
	  LineBuf.LetFileRound( Line->GetFileRound() );
		}
	else {
		Rtn = false;
		}

	delete Line;

	return(Rtn);
	}


CWLFileDictionary& CLoadFileDictionary::GetWLFileDict() {
	return m_Files;
	}


CSCFileDictionary::CSCFileDictionary() {}

CSCFileDictionary::~CSCFileDictionary() {
	mFiles.clearAndDestroy();
	}


ISCInputFile* CSCFileDictionary::NewFile(
	bool Random ) {

	ISCInputFile* File;
	if(Random) {
		File = new CSCRandomInputFile();
		}
	else {
		File = new CSCSeqInputFile();
		}

	return File;
	}


bool CSCFileDictionary::Add(
	ISCInputFile* File,
	const char* FileName,
	JSContext* cx ) {

	if( !File ) {	// error
		return false;
	}

	if( File->Open( FileName ) ) {
		if( mFiles.insertKeyAndValue(
				static_cast<RVCollectable*>(new CFileDictKey(FileName) ),
				static_cast<RVCollectable*>(File) ) ) {
			return true;
		}
	}

	// error
	delete File;
	WLErrorException errExcp(LoadExceptionFrmFile, 310, "Input File", FileName);
	return LoadReport( cx, errExcp );
}


bool CSCFileDictionary::AddFile(
	const char* FileName,
	bool Random,
	JSContext* cx ) {

	return Add(NewFile(Random), FileName, cx); // internally deleted if not used
	}


bool CSCFileDictionary::ReadLine(
	const char* FileName,
	IInputLineOutpar& LineBuf,
	iLEAgendaJSEmbedContext*,
	JSContext* cx ) {

	ISCInputFile* File;
#ifdef __TV_UNIX__
	RVCollectable* X = mFiles.findValue( (CFileDictKey *)(FileName) );
#else
	RVCollectable* X = mFiles.findValue( &(CFileDictKey(FileName)) );
#endif
	if(X) {
	  File = dynamic_cast<ISCInputFile*>(X);
		if(!File) {
			return false;
			}
		}
	else {
		File = NewFile( false );
		if(!Add(File, FileName, cx)) { // internally deleted if not used
			return false;
			}
		}

	return File->ReadLine( LineBuf );
	}



IInputFile::~IInputFile() {}


ISCInputFile::ISCInputFile():
	m_File(0),
	m_Round(1),
	m_LineNum(0) {}


ISCInputFile::~ISCInputFile() {
	if(m_File) {
		fclose(m_File);
		}
	}


bool ISCInputFile::Open_(
	const char* FileName ) {
	
	bool Rtn = true;
  if( m_File = fopen(FileName, "r") ) {
		fseek( m_File, 0L, SEEK_END);       // seek to the end of the file
	  if( ftell( m_File ) == 0 ) 	{        // taking its length.
		//empty file
			fclose(m_File);
			Rtn = false;
		} else {
			fseek( m_File, 0L, SEEK_SET);
		}
	} else {
		Rtn = false;
	}

	return Rtn;
	
	}





void ISCInputFile::ReadLine_(
	IInputLineOutpar& LineBuf ) {

	char Bytes[102400]; // 100K, old value was 2K /@@itaj: (intg 50 to 60) who owns the fucking stack?!
#ifndef _ANSI_ONLY
	char utfBytes[102400]; // 100K, old value was 2K /@@itaj: (intg 50 to 60) who owns the fucking stack?!
#endif
	RVCString Line;

	while(!fgets(Bytes, sizeof(Bytes), m_File)) {
		fseek(m_File, 0L, 0);
		m_LineNum = 0;
		++m_Round;
	}

	++m_LineNum;

	//Galtech
	char *pBytes = Bytes;
#ifndef _ANSI_ONLY
	ANSIToUTF8(GetMonitorCharSet(),	Bytes, -1, utfBytes, sizeof(utfBytes));
	pBytes = utfBytes;
#endif

	int Length = strlen(pBytes);
	if (pBytes[Length-1] == '\n') {
		Line.append(pBytes, Length-1);
	}	else {
		Line.append(pBytes, Length);
	}

	LineBuf.LetLine( Line );
}


CSCSeqInputFile::CSCSeqInputFile() {}

bool CSCSeqInputFile::Open(
	const char* FileName ) {

	return Open_( FileName );
	}


bool CSCSeqInputFile::ReadLine(
	IInputLineOutpar& LineBuf ) {

	ReadLine_( LineBuf );
	LineBuf.LetLineNumber( m_LineNum );
	LineBuf.LetFileRound( m_Round );
	return true; // @@Chaim (inherited from Itaj): no error check
	}



CSCRandomInputFile::CSCRandomInputFile() {}

bool CSCRandomInputFile::Open(
	const char* FileName ) {

	if( !Open_( FileName ) ) {
		return false;
		}

  srand( (unsigned int)time( NULL ) );   // set the random
  fseek( m_File, 0L, SEEK_END);       // seek to the end of the file
  m_Size = ftell( m_File );        // taking its length.
  if (m_Size <= 0) {
		m_Size = 1; // don't want it to be zero. 
		}

	return true;
	}


bool CSCRandomInputFile::ReadLine(
	IInputLineOutpar& LineBuf ) {

	{
		// there is double read. First seek to the rand record.
		// And, read this record to the end. We'll use the next record
		// whether it is in next in the file or the first one.
	  long lRandPos;
#ifdef __TV_linux__
		lRandPos = rand() % m_Size;
#else
		long temp = rand();
		lRandPos = temp << 16;
		lRandPos = (lRandPos | rand()) % m_Size;
#endif //__TV_linux__
		fseek( m_File, lRandPos, SEEK_SET);
		char Bytes[102400];
		fgets(Bytes, sizeof(Bytes), m_File);
		}

	ReadLine_( LineBuf );
	LineBuf.LetLineNumber( -1 );
	LineBuf.LetFileRound( m_Round );
	return true; 
	}

