/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include "rvjsapi.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadjsdefs.h"
#include "loadjsfile.h"
#include "loadinputfileinterface.h"
#include "wljsobjects.h"
#include "tracedll.h"
#include "loadthreadsmanager.h"
#include "loadengineglobals.h"
#include <RVCTokenizer.h>
#include "loadgeneratorscript.h"
#include "loadreportinfo.h"
#include "loadenginedecs.h"

//Galtech
#include "gt_unicode.h"
#include "gt_jshelper.h"

static JSBool
file_Open(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (!pMSD) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_FILE_OPEN);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  if (!(pMSD->InInitAgenda())) {
    WLErrorException e(LoadExceptionFrmFile, 230, WLJSNAME_FILE_OPEN);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if ((argc != 1) && (argc != 2)) {
    WLErrorException e(LoadExceptionFrmFile, 221, WLJSNAME_FILE_OPEN, 1, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

//Galtech
  RVCString FileName = ANSIGetStringBytes(JS_ValueToString(cx,argv[0]));

  bool random;
  if (argc == 2) {
    if (!JSVAL_IS_INT(argv[1])) {
      WLErrorException e(LoadExceptionFrmFile, 228, WLJSNAME_FILE_OPEN, 2);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }
    int tempRandom = JSVAL_TO_INT(argv[1]);
    if ( (tempRandom != 0) && (tempRandom != 1) ) {
      WLErrorException e(LoadExceptionFrmFile, 228, WLJSNAME_FILE_OPEN, 2);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }
    random = (tempRandom == 1);
  }
  else
    random = false;

	char FullPathFileName[WL_MAX_PATH];
	strcpy(FullPathFileName ,CArrangeMachineTempFiles::GetWlTempFullPath(cx ,WLECopiedFile, FileName).data() );

  IFileDictionary* FileDict = pMSD->GetFileDict();

  if( !FileDict->AddFile(FullPathFileName, random, cx) ) {
    RVCString Message;
    Message = Message + WLJSNAME_FILE_GET_LINE + " : Internal Error occurred.";
		return LoadReport( cx, WLLogMessage::Fatal, Message.data() );
		}

	return JS_TRUE;

}

#define NULL_STRING_VALUE_WITHOUT_DIEZ "$WL$NULL$STRING$"
#define NULL_STRING_VALUE_WITHOUT_DOLLAR "#WL#NULL#STRING#"

static JSBool
file_GetLine(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  YieldRequest(cx);

  IFileDictionary* FileDict;

	iLEAgendaJSEmbedContext* pAgendaEmbedContext = iLEAgendaJSEmbedContext::fGetAgendaEmbedContext(cx);

	if (pAgendaEmbedContext->isAbort()) return JS_FALSE;
	FileDict = pAgendaEmbedContext->GetFileDict();

  jsval v;
  RVCString delimiter(WLJSNAME_FILE_DEFAULT_DELIMITER);

  if ( !((argc == 1) || (argc == 2)) ) {
    WLErrorException e(LoadExceptionFrmFile, 221, WLJSNAME_FILE_GET_LINE, 1, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

//Galtech
  RVCString FileName = ANSIGetStringBytes(JS_ValueToString(cx,argv[0]));

  if (argc == 2) {
    delimiter = JS_GetStringBytes(JS_ValueToString(cx,argv[1]));
    if (delimiter.length() !=1) {
      WLErrorException e(LoadExceptionFrmFile, 229, WLJSNAME_FILE_GET_LINE, 2);
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }
  }

	char FullPathFileName[WL_MAX_PATH];
	strcpy(FullPathFileName ,CArrangeMachineTempFiles::GetWlTempFullPath(cx ,WLECopiedFile, FileName).data() );
  CInputLineBuf Line;
	if( !FileDict->ReadLine( FullPathFileName, Line, pAgendaEmbedContext, cx) ) {
    return JS_FALSE;
	}

  const char* LineStr = Line.GetLine();
  long LineNum = Line.GetLineNumber();
  long FileRound = Line.GetFileRound();

  jsval Vector;
  JSObject* Arr = JS_NewArrayObject(cx,0,&Vector);

  if (LineNum != -1) {
    if (!JS_DefineProperty(cx, Arr, WLJSNAME_FILE_ITEM_ROUND_NUM, INT_TO_JSVAL(FileRound),
            NULL, NULL, JSPROP_ENUMERATE)) {
      RVCString Message;
      Message = Message + WLJSNAME_FILE_GET_LINE + " : Internal Error occurred. Error in Define Property.";
			return LoadReport( cx, WLLogMessage::Fatal, Message.data() ); // @@Chaim (inherited from Itaj): frm
			}
    if (!JS_DefineProperty(cx, Arr, WLJSNAME_FILE_ITEM_LINE_NUM, INT_TO_JSVAL(LineNum),
            NULL, NULL, JSPROP_ENUMERATE)) {
      RVCString Message;
      Message = Message + WLJSNAME_FILE_GET_LINE + " : Internal Error occurred. Error in Define Property.";
			return LoadReport( cx, WLLogMessage::Fatal, Message.data() ); // @@Chaim (inherited from Itaj): frm
			}
		}

  JSString* str = JS_NewStringCopyZ(cx,LineStr);
  v = STRING_TO_JSVAL(str);
  JS_SetElement(cx,Arr,0,&v);

  // check for 2 delimiters
  RVCString NullStringValue;
  if (delimiter=="$")
    NullStringValue = NULL_STRING_VALUE_WITHOUT_DOLLAR;
  else
    NullStringValue = NULL_STRING_VALUE_WITHOUT_DIEZ;

  RVCString UpdatedLine = LineStr;
  RVCString TwoDelimiters = delimiter + delimiter;
  if (UpdatedLine(0,1) == delimiter)
    UpdatedLine.prepend(NullStringValue);
  size_t indx;
  while ((indx = UpdatedLine.index(TwoDelimiters)) != RV_NPOS) {
    UpdatedLine.insert(indx+1,NullStringValue);
  }
  if (UpdatedLine(UpdatedLine.length()-1,1) == delimiter)
    UpdatedLine += NullStringValue;
  
  RVCTokenizer next(UpdatedLine);
  RVCString    token;
  int i=0;
  while (!(token = next(delimiter)).isNull()) {
    i++;
    if (token == NullStringValue)
      v = STRING_TO_JSVAL(JS_NewStringCopyZ(cx,""));
    else
      v = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, (const char *)token));
    JS_SetElement(cx,Arr,i,&v);
  }

  JS_SetArrayLength(cx,Arr,i+1);
  

  *rval = OBJECT_TO_JSVAL(Arr);

  return JS_TRUE;
}

static JSFunctionSpec global_file_methods[] = {
	{WLJSNAME_FILE_OPEN,	    file_Open,	  	1},
	{WLJSNAME_FILE_GET_LINE,	file_GetLine,		2},
	{0}
};

bool js_AddFileFunctions(JSContext* cx, JSObject* obj)
{

  if (!JS_DefineFunctions(cx,obj,&global_file_methods[0]))
    return false;
  
	return true;
}

