/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifdef __TV_WINDOWS__
//#include <objbase.h>
#endif

#include <RVCString.h>
#include <thr.h>
#include <RVCTokenizer.h>

#include "rvjsapi.h"
#include "tracedll.h"
#include "tvpsync.h"
#include "wljsobjects.h"
#include "wljsruntime.h"
#include "loadengineglobals.h"
#include "wljsglobalvariables.h"
#include "loadjsdefs.h"
#include "loadjserrorfuncs.h"
#include "loadjsshell.h"
#include "loadjsrand.h"
#include "loadjsfile.h"
#include "messagereporter.h"
#include "outputfile.h"
#include "tablecomparereporter.h"
#include "jsjavacore.h"
#include "loadexceptions.h"
#include "tvenv.h"
#include "loadenginedecs.h"
#include "loadengineexternaldecs.h"
#include "wljssystemshell.h"
#include "wljsexception.h"
#include "wlleprotocol.h"
#include "loadengineinifileproperties.h"
#include "rvjsapi.h"
#include "rvjsembed.h"

#include <time.h>

#include "jstestclass.h"
//Galtech
#include "gt_unicode.h"
#include "gt_unicodectype.h"
#ifndef __TV_UNIX__
#include <io.h>
#else
#include "tvpfiles.h"
#include <unistd.h>
#endif
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

//=======================================================




////////////////////////////////////////////////////////////////////
//  the implementation of class CIncludeFileInfo
////////////////////////////////////////////////////////////////////


CIncludeFileInfo::CIncludeFileInfo(const char* filename, int CodeLength, const char* CodeBytes, bool execute) :
	m_FileName(filename),
	m_Script(NULL),
	m_SO(NULL),
	m_ScriptCode( CodeBytes, CodeLength ),
	m_Execute(execute) {
};


/*
JSBool WLBranchCallBack(JSContext* cx, JSScript* script) {
  JS_YieldRequest(cx);
//  JS_EndRequest(cx);
//  ::rvSleep(5000);
//  JS_BeginRequest(cx);
  return JS_TRUE;
}
*/

CWLJSBaseObject::CWLJSBaseObject() :
  m_Scope(NULL),
	m_NumErrors(0),
	m_ActiveCtr(0),
	m_ActiveAntiCtr(0),
	m_UserSectionActiveCtr(0),
	m_Errors(""),
	m_OwnScope( false )
{}

CWLJSBaseObject::~CWLJSBaseObject()
{
//  Terminate();
}

void CWLJSBaseObject::Terminate()
{
  // @@Chaim - 19032
//	if( m_OwnScope ) {
//    if (m_Scope)
//	    delete m_Scope;
//	}
  m_Scope = NULL;

}




JSBool fsJSGCCallback( JSContext* rCx, JSGCStatus rStatus )
{

	JSBool rtn = JS_TRUE;

	if( JSGC_BEGIN == rStatus ) {
		iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet(rCx);

		if( aEmbedContext ) {
			if( aEmbedContext->fIsJSGC() ) {
				rtn = JS_TRUE;
			} else {
				rtn = JS_FALSE;
			}
		} else {
			// JSD dumbContext
			rtn = JS_TRUE;
		}
	}

	return rtn;
}

bool CWLJSBaseObject::Init(CJSLoadRuntime& runtime, JSContext* guardcx,JSObject* proto)
{
	m_Scope = new CJSLoadScope(runtime, guardcx, CLEIniFileProperties::ContextSize, proto);

//Needed only for the first context created in the runtime
	JS_SetGCCallback( GetContext(), fsJSGCCallback );
	m_OwnScope = true;
  return (m_Scope != NULL);
}


bool CWLJSBaseObject::Init( CJSLoadScope& Scope )
{
	m_Scope = &Scope;
	return true;
}


bool CWLJSBaseObject::IsActive() {

	if(0==m_ActiveAntiCtr) {
		if(IsUserSectionActive()) {
			return true;
		} else {
			return(m_ActiveCtr>0);
		}
	} else {
		return false;
	}

	}

void CWLJSBaseObject::Activate() {
	++m_ActiveCtr;
	}

void CWLJSBaseObject::Deactivate() {
	--m_ActiveCtr;
	}



bool CWLJSBaseObject::IsUserSectionActive() {
	return( m_UserSectionActiveCtr>0 );
	}

void CWLJSBaseObject::UserSectionActivate() {
	++m_UserSectionActiveCtr;
	}

void CWLJSBaseObject::UserSectionDeactivate() {
	--m_UserSectionActiveCtr;
	}

void CWLJSBaseObject::ReportException()
{
	if ( JS_IsExceptionPending(GetContext()) )
	{
		jsval exn;
		JSObject *exnObject;
		JS_GetPendingException(GetContext(), &exn);

		iLEJSEmbedContext* aEmbedCx= iLEJSEmbedContext::fsGet(GetContext());
		if( RVJS_IsInstanceOf( GetContext(), exn, GetJsExceptionClass() ) )
		{
      // Get the exception - send the severest to load report. The rest of the
      // elements of the private ordered member are send to report log.
			exnObject = JSVAL_TO_OBJECT(exn);
			CJSExceptionPrivate* pJSException = ( CJSExceptionPrivate* )JS_GetPrivate(GetContext(), exnObject);
			WlAssert( pJSException );


      RVOrdered ExceptionList = pJSException->GetExceptionList();
      for (size_t i=0; i<ExceptionList.entries(); i++)
      {
        if ( i == (size_t)pJSException->GetSevereIndex() )
        {
          if (pJSException->GetMessage())
          {
            if (pJSException->HasTime()) {
 		cLoadReportInfoAgenda errRep( pJSException->GetSeverity(), pJSException->GetMessage(), true, pJSException->GetTime(), &pJSException->GetReportThreadState());
 		LoadReport2(*aEmbedCx, GetContext(), errRep);
	    }
            else {
              cLoadReportInfoAgenda errRep( pJSException->GetSeverity(), pJSException->GetMessage(), false, 0, &pJSException->GetReportThreadState());
              LoadReport2(*aEmbedCx, GetContext(), errRep);
	    }
          }
        }
        else
        {
          CExceptionError *e = (CExceptionError*)ExceptionList(i);
          cLoadReportInfoClient loadRepInf( e->GetSeverity(), e->GetMessage(), false, true, false, e->HasTime(),e->GetTime() );
          cLoadReportHard errRepHard( loadRepInf );
          aEmbedCx->reportLog( errRepHard );
        }
      }

		}
		else
		{
			JSString* str = JS_ValueToString( GetContext(), exn );
			WlAssert(str);
			const char* str2 = JS_GetStringBytes(str);
      RVCString aErrorMsg = "Exception thrown: ";
      aErrorMsg += str2;
			cLoadReportInfoAgenda errRep( WLLogMessage::Error, aErrorMsg );
//			cLoadReportInfoAgenda errRep( WLLogMessage::Error, RVCString("Exception thrown: ")+str2 );
			LoadReport2( *aEmbedCx, GetContext(), errRep);
		}
		JS_ClearPendingException(GetContext());
	}
}



const char* CWLJSBaseObject::Evaluate(const char* command)
{
	jsval v;

  CJSGCRequestBeginEnd GCGuard(GetContext());
	Activate();
  JSBool ret = JS_EvaluateScript(GetContext(), GetObject(), command, strlen(command), "CWLJSBaseObject::Evaluate", 1, &v);
	Deactivate();
  if (!ret) {
		return NULL;
	}

  JSString* str = JS_ValueToString(GetContext(), v);
  m_RetValue = JS_GetStringBytes(str);

  return m_RetValue.data();
}

bool CWLJSBaseObject::Evaluate(RVCString& command)
{
	if(command.length() == 0) {
		return true;
	}

	if(Evaluate(command.data()) != NULL) {
		return true;
	}

	return false;
}

bool CWLJSBaseObject::CallFunctionName( const char* name, uintN argc, jsval* argv, jsval& rval ) {
	bool rtn;

  CJSGCRequestBeginEnd GCGuard(GetContext());
	Activate();
	jsval fval;
	if( JS_GetProperty( GetContext(), GetObject(), name, &fval ) ) {
		if( !JSVAL_IS_VOID( fval ) ) {
			if( JS_CallFunctionValue(GetContext(), GetObject(), fval, argc, argv, &rval) ) {
				rtn = true;
			} else {
				rtn = false;
			}
		}
	} else {
		rtn = true; // @@Chaim (inherited from Itaj): why true? The function is not defined!
	}
	Deactivate();

	return rtn;
}

void CWLJSBaseObject::AddError(const char* error){
	m_NumErrors++;
	m_Errors.append(error);
}

void CWLJSBaseObject::reportApply(
	iReport& rReport )
{}

void CWLJSBaseObject::ClearError()
{
	m_NumErrors=0;
	m_Errors.remove(0);
}

const char* CWLJSBaseObject::GetError()
{
  return m_Errors.data();
}

int CWLJSBaseObject::GetNumErrors()
{
  return m_NumErrors;
}

JSContext* CWLJSBaseObject::GetContext()
{
	return m_Scope->GetContext();
}

JSObject* CWLJSBaseObject::GetObject()
{
	return m_Scope->GetObject();
}

void CWLJSBaseObject::EnterJava()
{
	m_IsRunningJava = true;
	++m_ActiveAntiCtr;
}

void CWLJSBaseObject::ExitJava()
{
	m_IsRunningJava = false;
	--m_ActiveAntiCtr;
}


//=======================================================
CWLJSScript::CWLJSScript(int numScriptInMix):
  m_Script(NULL),
	m_Proto(NULL),
  m_wlGlobal(NULL),
  m_SO(NULL),
  m_VariablesData(NULL),
  m_FileName(),
  m_ScriptCode(),
	m_InInitAgendaCtr(0),
	m_IncludeFiles(),
	m_IsJavaThreadWrapperCreationScriptExists(false),
  m_EvaluationStringBeforeInitClient(""),
  m_EvaluationStringAfterInitAgenda(""),
//	m_EvaluationStringOnThreadTermination(""),
	m_EvaluationStringBeforeThreadActivation(""),
	m_EvaluationStringOnThreadActivation(""),
	m_EvaluationStringAfterTerminateClient(""),
	m_EvaluationStringBeforeRound(""),
	m_EvaluationStringAfterRound(""),
	m_EvaluationStringAfterTerminateAgenda(""),
	m_numScriptInMix (numScriptInMix),
	m_VersionNumber(5.0),
	m_VersionNumberSet(false)
{
//  m_GCNewContextMutex = new RVCriticalSection;
}

CWLJSScript::~CWLJSScript()
{

	{
	  CJSGCRequestBeginEnd GCGuard(GetContext());


	{ // include files
		RVBagIterator iter(m_IncludeFiles);
		while( RVCollectable* key = iter() ) {
			WlAssert(key);
			WlAssert(dynamic_cast<CIncludeFileInfo*>(key));
			CIncludeFileInfo& IncludeFile = static_cast<CIncludeFileInfo&>(*key);
			JS_RemoveRoot( GetContext(), &(IncludeFile.m_SO) );
		}

	  m_IncludeFiles.clearAndDestroy();

	}

  if (m_VariablesData)
    delete m_VariablesData;

  JS_RemoveRoot(GetContext(),&m_SO);
	}
  CWLJSBaseObject::Terminate();

//	JAVASupportDetachThread();

//  delete m_GCNewContextMutex;
}



bool CWLJSScript::IsInInitAgenda() {
	return( m_InInitAgendaCtr>0 );
	}


static JSClass proto_class = {
	"Proto",
  0,
	JS_PropertyStub,	// add
	JS_PropertyStub,	// del
	JS_PropertyStub,	// get
	JS_PropertyStub,	// set
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	JS_FinalizeStub
};



bool CWLJSScript::Init(CJSLoadRuntime& pLoadRuntime, IGeneralManagerScriptData& pManagerScriptData, iLEProtocolContainer& pProtocolSessionContainer, const char* agendaFileName)
{

  //Initialize defaultvaluesArrays.
  CLEVariablesData::CreateDefaultValuesArrays() ;
  //Create variables array for wlGlobals
  m_VariablesData = new CLEVariablesData(NULL, CLEPropertyValue::PL_GLOBALS);

  if (!CWLJSBaseObject::Init(pLoadRuntime, NULL,NULL)) {
    AddError("CWLJSScript::Init Error in Init Base.");
		return false;
  }

	CJSGCRequestBeginEnd GCGuard( GetContext() );

	m_Proto = JS_NewObject( GetContext(), &proto_class, NULL, NULL );

  if (!JS_InitStandardClasses(GetContext(), m_Proto )) {
    AddError("CWLJSScript::Init Error in JS_InitStandardClasses.");
    return false;
  }

  JS_SetGlobalObject(GetContext(), GetObject());
	JS_SetPrototype( GetContext(), GetObject(), m_Proto );

	cLoadReportCommit::fsSignContext( GetContext() );
		// JS_SetErrorReporter(GetContext(), script_ErrorReporter);

//  JS_SetBranchCallback(GetContext(),&WLBranchCallBack);

	if(!AddGlobalsClass( GetContext(), GetObject() )) {
    AddError("CWLJSScript::Init can't initialize wlGlobals class" );
		return false;
	}

  m_wlGlobal = WLJSAddGlobalObject(GetContext(), GetObject(), m_VariablesData);
  if (!m_wlGlobal) {
    AddError("CWLJSScript::Init Error in WLJSAddGlobalObject.");
    return false;
  }
  if (!WLJSAddGlobalFunctions(GetContext(), GetObject(), GetProto() )) {
    AddError("CWLJSScript::Init Error in WLJSAddGlobalFunctions.");
    return false;
  }

	if (!js_AddGeneratorGlobalsObject(GetContext(), GetObject())) {
    AddError("CWLJSScript::Init Error in js_AddGeneratorGlobalsObject.");
    return false;
  }
	if (!js_AddMetaGlobalsObject(GetContext(), GetObject())) {
    AddError("CWLJSScript::Init Error in js_AddMetaGlobalsObject.");
    return false;
  }

  if (!AddOutputFileClass(GetContext(), GetObject())) {
    AddError("CWLJSScript::Init Error in AddOutputFileClass.");
    return false;
  }

  if (!AddMessageReporterClass(GetContext(), GetObject())) {
    AddError("CWLJSScript::Init Error in AddMessageReporterClass.");
    return false;
  }

  if (js_AddFileFunctions(GetContext(), GetObject()) != JS_TRUE) {
    AddError("CWLJSScript::Init Error in js_AddFileFunctions.");
    return false;
  }

  if (!js_AddRandObject(GetContext(),GetObject())) {
    AddError("CWLJSScript::Init Error in js_AddRandObject.");
    return false;
  }


  if (!AddTableCompareReporterClass(GetContext(),GetObject())) {
    AddError("CWLJSScript::Init Error in AddTableCompareReporterClass.");
    return false;
  }

  if (!AddSystemShellObject(GetContext(),GetObject())) {
    AddError("CWLJSScript::Init Error in AddSystemShellObject.");
    return false;
  }

  if (!AddJSExceptionClass(GetContext(),GetObject())) {
    AddError("CWLJSScript::Init Error in AddJSExceptionClass.");
    return false;
  }

#ifdef _DEBUG
  if (!AddTestJSClass(GetContext(), GetObject())) {
    AddError("CWLJSScript::Init Error in AddTestJSClass.");
    return false;
  }
#endif

  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_SUPER_GLOBALS_ALL_AGENDAS,INT_TO_JSVAL(0),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_SUPER_GLOBALS_CURRENT_AGENDA,INT_TO_JSVAL(1),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;

  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_SYNCHRONIZATION_POINT_STATUS_NORMAL,INT_TO_JSVAL(0),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_SYNCHRONIZATION_POINT_STATUS_RESCHEDULE,INT_TO_JSVAL(1),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_SYNCHRONIZATION_POINT_STATUS_TIMEOUT,INT_TO_JSVAL(2),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
//  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_SYNCHRONIZATION_POINT_STATUS_ERROR,INT_TO_JSVAL(3),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
//    return false;

  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_VERIFICATION_SUCCESS,INT_TO_JSVAL(WLLogMessage::Success),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
	if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_VERIFICATION_NOTE,INT_TO_JSVAL(WLLogMessage::Note),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_VERIFICATION_WARNING,INT_TO_JSVAL(WLLogMessage::Warning),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_VERIFICATION_ERROR,INT_TO_JSVAL(WLLogMessage::Error),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_VERIFICATION_SEVERE_ERROR,INT_TO_JSVAL(WLLogMessage::Fatal),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;

  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_GETLINE_SEQUENTIAL,INT_TO_JSVAL(0),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_GETLINE_RANDOM,INT_TO_JSVAL(1),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;

  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_INCLUDE_DONT_EXECUTE_SCRIPT,INT_TO_JSVAL(0),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;

  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_INCLUDE_EXECUTE_SCRIPT,INT_TO_JSVAL(1),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;

  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_EVALUATE_SCRIPT_BEFORE_INIT_CLIENT,INT_TO_JSVAL(CWLJSScript::WLBeforeInitClient),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;

  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_EVALUATE_SCRIPT_AFTER_INIT_AGENDA,INT_TO_JSVAL(CWLJSScript::WLAfterInitAgenda),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
//  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_EVALUATE_SCRIPT_ON_THREAD_TERMINATION,INT_TO_JSVAL(2),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
//    return false;
	if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_EVALUATE_SCRIPT_AFTER_TERMINATE_CLIENT,INT_TO_JSVAL(CWLJSScript::WLAfterTerminateClient),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
	if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_EVALUATE_SCRIPT_BEFORE_ROUND,INT_TO_JSVAL(CWLJSScript::WLBeforeRound),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_EVALUATE_SCRIPT_AFTER_TERMINATE_AGENDA,INT_TO_JSVAL(CWLJSScript::WLAfterTerminateAgenda),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
	if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_EVALUATE_SCRIPT_AFTER_ROUND,INT_TO_JSVAL(CWLJSScript::WLAfterRound),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_EVALUATE_SCRIPT_ON_THREAD_ACTIVATION,INT_TO_JSVAL(CWLJSScript::WLOnThreadActivation),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_EVALUATE_SCRIPT_BEFORE_THREAD_ACTIVATION,INT_TO_JSVAL(CWLJSScript::WLBeforeThreadActivation),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;

  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_SESSION_AVERAGE,INT_TO_JSVAL(WLJSNAME_SESSION_AVERAGE_VALUE),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_SESSION_MINIMUM,INT_TO_JSVAL(WLJSNAME_SESSION_MINIMUM_VALUE),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_SESSION_MAXIMUM,INT_TO_JSVAL(WLJSNAME_SESSION_MAXIMUM_VALUE),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_SESSION_COUNT,INT_TO_JSVAL(WLJSNAME_SESSION_COUNT_VALUE),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_LAST_SLICE_AVERAGE,INT_TO_JSVAL(WLJSNAME_LAST_SLICE_AVERAGE_VALUE),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_LAST_SLICE_MINIMUM,INT_TO_JSVAL(WLJSNAME_LAST_SLICE_MINIMUM_VALUE),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_LAST_SLICE_MAXIMUM,INT_TO_JSVAL(WLJSNAME_LAST_SLICE_MAXIMUM_VALUE),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_LAST_SLICE_COUNT,INT_TO_JSVAL(WLJSNAME_LAST_SLICE_COUNT_VALUE),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;


	//GALTECH
	// Charsets
#ifndef _ANSI_ONLY
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_ARABIC_ISO,INT_TO_JSVAL(CP_MS_ARABIC),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_BALTIC_ISO,INT_TO_JSVAL(CP_MS_BALTIC),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_CENTRAL_EUROPEAN_DOS,INT_TO_JSVAL(CP_IBM_852),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_CENTRAL_EUROPEAN_ISO,INT_TO_JSVAL(CP_MS_LATIN2),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_CHINESE_SIMPLIFIED,INT_TO_JSVAL(CP_MS_CHINESE_SIMPLIFIED),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_CHINESE_TRADITIONAL,INT_TO_JSVAL(CP_MS_CHINESE_TRADITIONAL),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_CYRILLIC_DOS,INT_TO_JSVAL(CP_IBM_866),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_CYRILLIC_ISO,INT_TO_JSVAL(CP_MS_CYRILLIC),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_CYRILLIC_KOI8R,INT_TO_JSVAL(CP_KOI_8),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_GREEK_ISO,INT_TO_JSVAL(CP_MS_GREEK),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_GREEK_DOS,INT_TO_JSVAL(CP_IBM_869),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_HEBREW_DOS,INT_TO_JSVAL(CP_IBM_862),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_HEBREW_ISO,INT_TO_JSVAL(CP_MS_HEBREW),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_HEBREW_VISUAL,INT_TO_JSVAL(CP_MS_HEBREW),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_HEBREW_LOGICAL,INT_TO_JSVAL(CP_MS_HEBREW),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_HEBREW_MAC,INT_TO_JSVAL(CP_MAC_HEBREW),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_ICELANDIC_DOS,INT_TO_JSVAL(CP_IBM_861),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_JAPANESE_SHIFT_JIS,INT_TO_JSVAL(CP_MS_JAPANESE),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_JAPANESE_JIS,INT_TO_JSVAL(CP_JIS),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_JAPANESE_EUC,INT_TO_JSVAL(CP_EUCJP),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_KOREAN_ISO,INT_TO_JSVAL(CP_MS_KOREAN),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_LATIN_1_ISO,INT_TO_JSVAL(CP_MS_LATIN1),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_LATIN_3_ISO,INT_TO_JSVAL(CP_ISO_8859_3),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_TURKISH_ISO,INT_TO_JSVAL(CP_MS_TURKISH),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_UNICODE,INT_TO_JSVAL(CP_UCS2),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_UTF7,INT_TO_JSVAL(CP_UTF7),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_UTF8,INT_TO_JSVAL(CP_UTF8),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_US_ASCII,INT_TO_JSVAL(CP_MS_LATIN1),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_WESTERN_EUROPEAN_DOS,INT_TO_JSVAL(CP_IBM_850),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_WESTERN_EUROPEAN_ISO,INT_TO_JSVAL(CP_MS_LATIN1),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
  if (!JS_DefineProperty(GetContext(),GetObject(),WLJSNAME_CHARSET_WESTERN_EUROPEAN_MAC,INT_TO_JSVAL(CP_MAC_ROMAN),NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY))
    return false;
#endif

  if (!JS_DefineProperty(GetContext(),GetObject(),
		WLJSNAME_THREAD_LE,
		INT_TO_JSVAL( WL_ENUM_TO_JSINT(WLEThreadTypeLE) ),
		NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY)) {
    return false;
	}

  if (!JS_DefineProperty(GetContext(),GetObject(),
		WLJSNAME_THREAD_PC,
		INT_TO_JSVAL(WL_ENUM_TO_JSINT(WLEThreadTypePC)),
		NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY)) {
    return false;
	}

  if (!JS_DefineProperty(GetContext(),GetObject(),
		WLJSNAME_THREAD_BP,
		INT_TO_JSVAL(WL_ENUM_TO_JSINT(WLEThreadTypeBP)),
		NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY)) {
    return false;
	}

  if (!JS_DefineProperty(GetContext(),GetObject(),
		WLJSNAME_AGENDA_FILE_NAME,
		STRING_TO_JSVAL(JS_NewStringCopyZ(GetContext(),agendaFileName)),
		NULL,NULL,JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY)) {
    return false;
	}

	if( !cRVJVMManager::fsJSInit( GetContext(), GetObject() ) ) {
    return false;
	}

  setCurrentManagerScriptData( &pManagerScriptData, GetContext() );

  pManagerScriptData.InsertProtocolAgendaInMix(pProtocolSessionContainer);
  pManagerScriptData.ProtocolJSInitializeForScript(GetContext(),GetObject(),GetProto(), pProtocolSessionContainer);

  return true;
}

void CWLJSScript::Terminate() {

}


void CWLJSScript::JSGC(bool maybe)
{
  if (maybe)
    JS_MaybeGC(GetContext());
  else
    JS_GC(GetContext());
}

bool CWLJSScript::InitAgenda(const char* InitializationScript) {

	char path[WL_MAX_PATH];
	strcpy(path,CArrangeMachineTempFiles::GetWlTempFullPath(fGetProcess().GetMasterPid(),m_numScriptInMix));
	CArrangeMachineTempFiles::LECreateDirectory(path);

	++m_InInitAgendaCtr;

	bool Rtn(true);

	if(Rtn) {
		Rtn = InitAgendaInternal( InitializationScript );
	}

	--m_InInitAgendaCtr;

	if( !Evaluate(m_EvaluationStringAfterInitAgenda) ) {
		return LoadReport(GetContext(), WLLogMessage::Fatal, "Evaluate in time AfterInitAgenda failed");
	}

	m_PropWorkSpace.ReadData(*this);//set the LE properties after running
									//the user script that is written in the InitAgenda function.

	if (m_PropWorkSpace.m_JVMType.compareTo("Default",ignoreCase)){
		CLEIniFileProperties::resetJVMPath(m_PropWorkSpace.m_JVMType.data());
	}

    SetMonitorCharSet(m_PropWorkSpace.m_CharEncoding);

	return Rtn;
}

bool CWLJSScript::InitAgendaInternal(const char* InitializationScript)
{
  jsval v;
  JSBool ok;


  CJSGCRequestBeginEnd GCGuard(GetContext());

  if (strcmp(InitializationScript,"") != 0) {
    JSBool ret = JS_EvaluateScript(GetContext(), GetObject(), InitializationScript, strlen(InitializationScript), "init script", 1, &v);
    if (ret == JS_FALSE) {
      AddError("CWLJSScript::InitAgenda - Internal Error while evaluating default parameters");
      return false;
    }
  }

  JSContext* CurJSContext = GetContext();
  JSObject* CurJSObject = GetObject();

  const char* DefaultInitializationScript = CLEIniFileProperties::LGDefaultInitializationScript;
  if (DefaultInitializationScript){
    if (strcmp(DefaultInitializationScript,"") != 0) {
      JSBool ret = JS_EvaluateScript(CurJSContext, CurJSObject, DefaultInitializationScript, strlen(DefaultInitializationScript), "default init script", 1, &v);
      if (ret == JS_FALSE) {
        AddError("CWLJSScript::InitAgenda - Internal Error while evaluating DefaultInitializationScript");
        return false;
      }
    }
  }

	m_PropWorkSpace.ReadData(*this);//set the LE properties before running
									//the user script that is written in the InitAgenda function.

  ok = JS_LookupProperty(GetContext(),GetObject(),WLJSNAME_INIT_AGENDA_FUNC,&v);
  if (!ok) {
    AddError("CWLJSScript::InitAgenda - Error in lookup-property");
    return false;
  }

  if (!JSVAL_IS_VOID(v)) {
    if (JS_TypeOfValue(GetContext(),v) == JSTYPE_FUNCTION) {
			UserSectionActivate();
			JSBool status = JS_CallFunctionName(GetContext(), GetObject(),WLJSNAME_INIT_AGENDA_FUNC,0,0,&v);
			ReportException();
			UserSectionDeactivate();
      if ( status == JS_TRUE)
        return true;
      else
        return false;
    }
  }
  return true;
}

bool CWLJSScript::TerminateAgenda(IGeneralManagerScriptData& ManagerScriptData)
{
  jsval v;
  JSBool ok;

  CJSGCRequestBeginEnd GCGuard(GetContext());

  ok = JS_LookupProperty(GetContext(),GetObject(),WLJSNAME_TERMINATE_AGENDA_FUNC,&v);
  if (!ok) {
    AddError("CWLJSScript::TerminateAgenda - Error in lookup-property");
    return false;
  }

  if (!JSVAL_IS_VOID(v)) {
    if (JS_TypeOfValue(GetContext(),v) == JSTYPE_FUNCTION) {
			UserSectionActivate();
			JSBool status = JS_CallFunctionName(GetContext(), GetObject(),WLJSNAME_TERMINATE_AGENDA_FUNC,0,0,&v);
			ReportException();
			UserSectionDeactivate();
      if ( status == JS_TRUE) {
				if( !Evaluate(m_EvaluationStringAfterTerminateAgenda) ) {
					return LoadReport(GetContext(), WLLogMessage::Fatal, "Evaluate in time AfterTerminateAgenda failed");
				}

		ManagerScriptData.ProtocolJSTerminateForScript(GetContext(),GetObject(),GetProto());
        ManagerScriptData.ProtocolAfterTerminateAgenda();

        return true;
			} else {
        return false;
			}
    }
  }


  return true;
}

bool CWLJSScript::OnErrorTerminateAgenda(IGeneralManagerScriptData& ManagerScriptData)
{
  jsval v;
  JSBool ok;

  CJSGCRequestBeginEnd GCGuard(GetContext());

  ok = JS_LookupProperty(GetContext(),GetObject(),WLJSNAME_ON_ERROR_TERMINATE_AGENDA_FUNC,&v);
  if (!ok) {
    AddError("CWLJSScript::OnErrorTerminateAgenda - Error in lookup-property");
    return false;
  }

  if (!JSVAL_IS_VOID(v)) {
    if (JS_TypeOfValue(GetContext(),v) == JSTYPE_FUNCTION) {
			UserSectionActivate();
			JSBool status = JS_CallFunctionName(GetContext(), GetObject(),WLJSNAME_ON_ERROR_TERMINATE_AGENDA_FUNC,0,0,&v);
			ReportException();
			UserSectionDeactivate();

			ManagerScriptData.ProtocolJSTerminateForScript(GetContext(),GetObject(),GetProto());
      ManagerScriptData.ProtocolAfterTerminateAgenda();

      if ( status == JS_TRUE)
        return true;
      else
        return false;
    }
  }
  return true;
}

#define AllInitTerminateFunctions "function "WLJSNAME_INIT_AGENDA_FUNC"() {};function "WLJSNAME_TERMINATE_AGENDA_FUNC"() {};function "WLJSNAME_ON_ERROR_TERMINATE_AGENDA_FUNC"() {};function "WLJSNAME_INIT_CLIENT_FUNC"() {};function "WLJSNAME_TERMINATE_CLIENT_FUNC"() {};function "WLJSNAME_ON_ERROR_TERMINATE_CLIENT_FUNC"() {};function "WLJSNAME_ON_SCRIPT_ABORT_FUNC"() {};function UNDOC_BeforeRound(){};function UNDOC_AfterRound(){}"

bool CWLJSScript::LoadScript(const char* filename,int ScriptLength, const char* ScriptCode)
{
	if (!filename || !strlen(filename))
		return false;

  CJSGCRequestBeginEnd GCGuard(GetContext());

  // Eval the default Init/Terminate functions
  Evaluate(AllInitTerminateFunctions);

  if ( (ScriptLength==0) || !ScriptCode || (strcmp(ScriptCode,"") == 0)) {
		UserSectionActivate();

	//Galtech
#ifndef _ANSI_ONLY
#ifdef __TV_WINDOWS__
	int handle = _open(filename, _O_TEXT | _O_RDONLY);
    if(handle == -1)
    {
        RVCString errMsg = "CWLJSScript::LoadScript Error opening script: \"";
        errMsg += filename;
        errMsg += "\"; Error: ";
        errMsg += strerror(errno);

        AddError(errMsg);

        return false;
    }

	int size = filelength(handle) + 1;
	char *buf = new char[size];
	size = _read(handle, buf, size);
	_close(handle);
#else
	int handle = open(filename, O_RDONLY);
	int size = filelength(handle) + 1;
	char *buf = new char[size];
	size = read(handle, buf, size);
	close(handle);
#endif
	buf[size - 1] = '\0';
	gt_wchar_t *wbuf = new gt_wchar_t[size];
	ANSIToUCS2(GetMonitorCharSet(), buf, -1, wbuf, size);
    m_Script = JS_CompileUCScript(GetContext(),GetObject(),wbuf, w_strlen(wbuf), filename, 1);
	delete [] buf;
	delete [] wbuf;
#else
    m_Script = JS_CompileFile(GetContext(),GetObject(),filename);
#endif

		UserSectionDeactivate();
    m_FileName = filename;
  }
  else {
		UserSectionActivate();
    m_Script = JS_CompileScript(GetContext(),GetObject(),ScriptCode,ScriptLength,filename,1);
		UserSectionDeactivate();
    m_ScriptCode = RVCString( ScriptCode, ScriptLength );
  }

  if (!m_Script) {
    AddError("CWLJSScript::LoadScript Error while compiling script.");
		return false;
	}

	m_SO = JS_NewScriptObject(GetContext(),m_Script);
  if (!m_SO) {
    AddError("CWLJSScript::LoadScript Error while creating script object.");
    return false;
  }

  if (!JS_AddNamedRoot(GetContext(),&m_SO, "CWLJSScript::LoadScript m_SO")) {
    AddError("CWLJSScript::LoadScript Error while AddRoot the script object.");
    return false;
  }

{
		jsval rval;
    // No debugger here.
		JSBool status = JS_ExecuteScriptPart(GetContext(), GetObject(), m_Script, JSEXEC_PROLOG, &rval);
		if (!status) {
			AddError("CWLJSScript::LoadScript Error while execute part.");
			return false;
			}
		}


  return true;

}


bool CWLJSScript::IncludeFile(
	const char* FileNameBytes,
	int CodeLength,
	const char* CodeBytes,
	bool Execute ) {

  CIncludeFileInfo* pIncludefile = new CIncludeFileInfo(FileNameBytes,CodeLength, CodeBytes, Execute);
  CWLJSScript::IncludeFileStatus st = IncludeFileCompile(pIncludefile);

	switch(st) {
  case CWLJSScript::IncludeFileErrorCompilingScript:
	case CWLJSScript::IncludeFileErrorExecutingScript:
		delete pIncludefile;
		return false;
	default:
		// OK status
	  m_IncludeFiles.insert(pIncludefile);
		return true;
	}
}


CWLJSScript::IncludeFileStatus CWLJSScript::IncludeFileCompile(
	CIncludeFileInfo* pIncludeFileInfo) {

	CJSGCRequestBeginEnd GCGuard(GetContext());
  if (pIncludeFileInfo->m_ScriptCode == "" ) {
		Activate();
    pIncludeFileInfo->m_Script = JS_CompileFile(GetContext(),GetObject(),pIncludeFileInfo->m_FileName.data());
		Deactivate();
  } else {
		Activate();
    pIncludeFileInfo->m_Script = JS_CompileScript(GetContext(),GetObject(),pIncludeFileInfo->m_ScriptCode.data(),pIncludeFileInfo->m_ScriptCode.length(), pIncludeFileInfo->m_FileName.data(),1);
		Deactivate();
  }
  if (!pIncludeFileInfo->m_Script) {
    AddError("CWLJSScript::IncludeFile Error while compiling script.");
		return IncludeFileErrorCompilingScript;
	}

  pIncludeFileInfo->m_SO = JS_NewScriptObject(GetContext(),pIncludeFileInfo->m_Script);
  if (!pIncludeFileInfo->m_SO) {
    AddError("CWLJSScript::IncludeFile Error while creating script object.");
    return IncludeFileErrorCompilingScript;
  }

  if (!JS_AddNamedRoot(GetContext(),&(pIncludeFileInfo->m_SO), "CWLJSScript::IncludeFile pIncludeFileInfo->m_SO" )) {
    AddError("CWLJSScript::IncludeFile Error while AddRoot the script object.");
    return IncludeFileErrorCompilingScript;
  }


  jsval rval;
  if (pIncludeFileInfo->m_Execute) {
		Activate();
		JSBool status = JS_ExecuteScript(GetContext(), GetObject(), pIncludeFileInfo->m_Script, &rval);
		Deactivate();
    if (!status) {
      AddError("CWLJSScript::IncludeFile Error while executing script.");
      return IncludeFileErrorExecutingScript;
    }
  }
  else {
		Activate();
		JSBool status = JS_ExecuteScriptPart(GetContext(), GetObject(), pIncludeFileInfo->m_Script, JSEXEC_PROLOG, &rval);
		Deactivate();
    if (!status) {
      AddError("CWLJSScript::IncludeFile Error while execute part.");
      return IncludeFileErrorCompilingScript;
    }
  }

  return IncludeFileOK;
}

void CWLJSScript::SetJavaThreadWrapperCreationScript()
{
	m_IsJavaThreadWrapperCreationScriptExists = true;
}

bool CWLJSScript::IsJavaThreadWrapperCreationScriptExists()
{
	return m_IsJavaThreadWrapperCreationScriptExists;
}


void CWLJSScript::AppendStringForEvaluation(
	const char* str,CWLJSScript::WhenToRunScript whenToRunScript ) {
	switch(whenToRunScript)
	{
		case WLAfterInitAgenda: {
			m_EvaluationStringAfterInitAgenda.append(str);
			m_EvaluationStringAfterInitAgenda.append("\n");
		}
		break;

		case WLBeforeThreadActivation: {
			m_EvaluationStringBeforeThreadActivation.append(str);
			m_EvaluationStringBeforeThreadActivation.append("\n");
		}
		break;

		case WLOnThreadActivation: {
			m_EvaluationStringOnThreadActivation.append(str);
			m_EvaluationStringOnThreadActivation.append("\n");
		}
		break;

		case WLBeforeInitClient: {
			m_EvaluationStringBeforeInitClient.append(str);
			m_EvaluationStringBeforeInitClient.append("\n");
		}
		break;

		case WLBeforeRound: {
			m_EvaluationStringBeforeRound.append(str);
			m_EvaluationStringBeforeRound.append("\n");
		}
		break;

		case WLAfterRound: {
			m_EvaluationStringAfterRound.append(str);
			m_EvaluationStringAfterRound.append("\n");
		}
		break;

		case WLAfterTerminateClient: {
			m_EvaluationStringAfterTerminateClient.append(str);
			m_EvaluationStringAfterTerminateClient.append("\n");
		}
		break;

		/*
		case WLOnThreadTermination: { //still not in use
			m_EvaluationStringOnThreadTermination.append(str);
			m_EvaluationStringOnThreadTermination.append("\n");
		}
		break;
*/

		case WLAfterTerminateAgenda: {
			m_EvaluationStringAfterTerminateAgenda.append(str);
			m_EvaluationStringAfterTerminateAgenda.append("\n");
		}
		break;

		default:{
				LoadReport(GetContext(),WLLogMessage::Fatal,"Internal bug : the code in the function 'CWLJSScript::AppendStringForEvaluation' has a mistake");
		}

	}
}

RVCString& CWLJSScript::GetEvaluationStringBeforeInitClient()
{
	return	m_EvaluationStringBeforeInitClient;
}

RVCString& CWLJSScript::GetEvaluationStringAfterInitAgenda()
{
	return	m_EvaluationStringAfterInitAgenda;
}
/*
RVCString&	CWLJSScript::GetEvaluationStringOnThreadTermination()
{
	return m_EvaluationStringOnThreadTermination;
}
*/

RVCString& CWLJSScript::GetEvaluationStringBeforeThreadActivation()
{
	return m_EvaluationStringBeforeThreadActivation;
}

RVCString& CWLJSScript::GetEvaluationStringOnThreadActivation()
{
	return m_EvaluationStringOnThreadActivation;
}

RVCString&	CWLJSScript::GetEvaluationStringAfterTerminateClient()
{
	return m_EvaluationStringAfterTerminateClient;
}

RVCString&	CWLJSScript::GetEvaluationStringBeforeRound()
{
	return m_EvaluationStringBeforeRound;
}

RVCString&	CWLJSScript::GetEvaluationStringAfterRound()
{
	return m_EvaluationStringAfterRound;
}

RVCString&	CWLJSScript::GetEvaluationStringAfterTerminateAgenda()
{
	return m_EvaluationStringAfterTerminateAgenda;
}


CGeneralProcess& CWLJSScript::fGetProcess(){
	iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet(GetContext());
	return aEmbedContext->fGetMachineThread().fGetProcess();
}

CLEVariablesData* CWLJSScript::GetVariablesData(){
	return m_VariablesData;
}

double CWLJSScript::GetVersionNumber(){
	WlAssert(m_VersionNumberSet);
	return m_VersionNumber;
}

void CWLJSScript::InitVersionNumber(){
	if (m_VersionNumberSet) return;

	CJSGCRequestBeginEnd Guard( GetContext() );

	m_VersionNumber = 5.0;
	jsval aFunc;
	JSBool aRet = JS_GetProperty(GetContext(), GetObject(), "RVScriptVersion", &aFunc);
	if (aRet == JS_TRUE && aFunc!= JSVAL_VOID){
		jsval aVersionNumber;
		aRet = JS_CallFunctionValue(GetContext(), GetObject(), aFunc, 0, NULL, &aVersionNumber);
		if (aRet == JS_TRUE && aVersionNumber!= JSVAL_VOID){
			jsdouble aTemp;
			JS_ValueToNumber(GetContext(), aVersionNumber, &aTemp);
			m_VersionNumber = aTemp;
		}
	}
	m_VersionNumberSet = true;
}


//=======================================================
CWLJSThread::CWLJSThread(CWLJSScript& pJSScript, bool OwnScope ) :
  m_JSScript(&pJSScript),
  m_GlobalVariablesData(NULL),
  m_LocalVariablesData(NULL),
	m_IsJavaDefaultStatistics(true),
	m_InRoundCtr(0),
	m_OwnScope(OwnScope)
{
//		srand( (unsigned)time( NULL ) );
	//	int a = rand();
}

CWLJSThread::~CWLJSThread()
{
	// if we will reach the destructor of CWLJSThread,
	// some lines from the function Terminate of this class should be here.
  if (m_GlobalVariablesData)
    delete m_GlobalVariablesData;

  if (m_LocalVariablesData)
    delete m_LocalVariablesData;

  CWLJSBaseObject::Terminate();
}

bool CWLJSThread::IsInRound() {
	return( m_InRoundCtr>0 );
	}


bool CWLJSThread::Init(CJSLoadRuntime& runtime, CLoadGeneralThreadInstanceScript* TIS, IGeneralManagerScriptData& GMSD, int nThread) {

  //RVCriticalSection::LockGuard lock(*(m_JSScript->GetGCNewContextMutex()));
	if( !( m_OwnScope ?
		CWLJSBaseObject::Init(runtime, m_JSScript->GetContext(),m_JSScript->GetObject()) :
		CWLJSBaseObject::Init(*m_JSScript->GetScope() ) ) )
	{
    AddError("CWLJSThread::Init Error in Init Base.");
		return false;
  }

  //Add the wlGlobals in the thread level
  m_GlobalVariablesData = new CLEVariablesData(*(m_JSScript->GetLoadEngineVariablesData()));

  //
  // Creating the VariablesData object for the current thread, and registering it
  // as the PrivateData of the context.
  m_LocalVariablesData = new CLEVariablesData(m_GlobalVariablesData, CLEPropertyValue::PL_LOCALS);
  //JS_SetPrivate(GetContext(), GetObject(), (void*)m_VariablesData);

  CJSGCRequestBeginEnd GCGuard(GetContext());
  JS_SetGlobalObject(GetContext(), GetObject());

	cLoadReportCommit::fsSignContext( GetContext() );
	  // JS_SetErrorReporter(GetContext(), thread_ErrorReporter);
//  JS_SetBranchCallback(GetContext(),&WLBranchCallBack);
/*
  if (m_JSScript->GetFileName() != NULL) {
		Activate();
    m_Script = JS_CompileFile(GetContext(),GetObject(),m_JSScript->GetFileName());
		Deactivate();
		}
  else {
		Activate();
    m_Script = JS_CompileScript(GetContext(),GetObject(),m_JSScript->GetScriptData(),strlen(m_JSScript->GetScriptData()),m_JSScript->GetFileName(),1);
		Deactivate();
		}

  if (!m_Script) {
    AddError("CWLJSScript::LoadScript Error while compiling script.");
		return false;
	}
*/
{
  jsval rval;
	UserSectionActivate();
	JSBool status = JS_ExecuteScriptPart(GetContext(), GetObject(), m_JSScript->GetScript(), JSEXEC_PROLOG, &rval);
	UserSectionDeactivate();
  if (!status) {
    AddError("CWLJSThread::Init Error while execute part.");
    return false;
		}
	}


/*
  m_SO = JS_NewScriptObject(GetContext(),m_Script);
  if (!m_SO) {
    AddError("CWLJSScript::LoadScript Error while creating script object.");
    return false;
  }

  if (!JS_AddNamedRoot(GetContext(),&m_SO, "CWLJSThread::Init")) {
    AddError("CWLJSScript::LoadScript Error while AddRoot the script object.");
    return false;
  }
*/
  if (WLJSAddLocalProperties(GetContext(), GetObject(), nThread) != JS_TRUE) {
    AddError("CWLJSThread::Init Error in WLJSAddLocalProperties.");
		return false;
  }

  JSObject* wlGlobal = WLJSAddGlobalPerThreadObject(GetContext(), GetObject(), m_GlobalVariablesData);
  if (!wlGlobal) {
    AddError("CWLJSScript::Init Error in WLJSAddGlobalObject.");
    return false;
  }
  JS_SetPrototype(GetContext(),wlGlobal,m_JSScript->GetwlGlobal());

  if (!WLJSAddLocalObject(GetContext(), GetObject(), m_LocalVariablesData)) {
    AddError("CWLJSThread::Init Error in WLJSAddLocalObject.");
    return false;
  }

  TIS->InitProtocols(*this, GMSD, GetContext(), GetObject(), m_LocalVariablesData);

	if(!IncludeFiles(*m_JSScript)) {
		return false;
		}

  return true;
}

void CWLJSThread::Terminate() {


	CWLJSBaseObject::Terminate();
}


bool CWLJSThread::IncludeFiles(
	CWLJSScript& Script ) {

  CIncludeFileInfo* pIncludeFileInfo;

	RVBagIterator iter(Script.m_IncludeFiles);
	while(iter()) {
    WlAssert(dynamic_cast<CIncludeFileInfo*>( iter.key() ));
    pIncludeFileInfo = static_cast<CIncludeFileInfo*>( iter.key() );
    if (!IncludeFile(pIncludeFileInfo)) {
      return false;
		}
	}

/*
  for (size_t i = 0 ; i < Script.m_IncludeFiles.length(); i++) {
    pIncludeFileInfo = (CIncludeFileInfo*)(Script.m_IncludeFiles[i]);
    if (!IncludeFile(pIncludeFileInfo)) {
      return false;
		}
  }
*/
  return true;
}


bool CWLJSThread::IncludeFile(CIncludeFileInfo* pIncludeFileInfo) {
  CJSGCRequestBeginEnd GCGuard(GetContext());

  jsval rval;
	Activate();
	JSBool status = JS_ExecuteScriptPart(GetContext(), GetObject(), pIncludeFileInfo->m_Script, JSEXEC_PROLOG, &rval);
	Deactivate();
  if (!status) {
    AddError("CWLJSThread::IncludeFile Error while execute part.");
    return false;
	  }

  return true;
}


bool CWLJSThread::ExecScript(unsigned long RoundNum)
{

  JSBool foundp;

  CJSGCRequestBeginEnd GCGuard(GetContext());

	jsval v = INT_TO_JSVAL(RoundNum);

  JS_SetPropertyAttributes(GetContext(), GetObject(),WLJSNAME_ROUND_NUM,
			 JSPROP_ENUMERATE | JSPROP_PERMANENT, &foundp);
  JS_SetProperty(GetContext(), GetObject(), WLJSNAME_ROUND_NUM,&v);
  JS_SetPropertyAttributes(GetContext(), GetObject(),WLJSNAME_ROUND_NUM,
			 JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_READONLY, &foundp);

  // since there is a bug in JS_ExecuteScriptPart I use the JS_ExecuteScript
	++m_InRoundCtr;
  jsval result;
	UserSectionActivate();
	bool status = ( JS_ExecuteScriptPart(GetContext(),GetObject(),m_JSScript->GetScript(),JSEXEC_MAIN,&result) ? true : false );
	ReportException();
	UserSectionDeactivate();
	--m_InRoundCtr;
  return status;
}

void CWLJSThread::SetJSContext() {
  // Command that should work before any JS command the thread is doing. Since the context was
  // created in the main thread any other thread can do nothing, before the following command
  // is being called.
  JS_SetContextThread(GetContext());
}

bool CWLJSThread::InitClient(CLoadGeneralThreadInstanceScript* TIS)
{
  jsval v;
  JSBool ok;


  TIS->ProtocolBeforeInitClient();

	if (!Evaluate(m_JSScript->GetEvaluationStringBeforeInitClient())){
		return false;
	}

  CJSGCRequestBeginEnd GCGuard(GetContext());

  ok = JS_LookupProperty(GetContext(),GetObject(),WLJSNAME_INIT_CLIENT_FUNC,&v);
  if (!ok) {
    AddError("CWLJSThread::InitClient - Error in lookup-property");
    return false;
  }

  if (!JSVAL_IS_VOID(v)) {
    if (JS_TypeOfValue(GetContext(),v) == JSTYPE_FUNCTION) {

			UserSectionActivate();
			JSBool status = JS_CallFunctionName(GetContext(), GetObject(),WLJSNAME_INIT_CLIENT_FUNC,0,0,&v);
			ReportException();
			UserSectionDeactivate();
      TIS->ProtocolAfterInitClient();
      if ( status == JS_TRUE)
        return true;
      else
        return false;

    }
  }
  return true;
}

bool CWLJSThread::TerminateClient()
{
	bool rtn( true );

  jsval v;
  JSBool ok;

  CJSGCRequestBeginEnd GCGuard(GetContext());

  ok = JS_LookupProperty(GetContext(),GetObject(),WLJSNAME_TERMINATE_CLIENT_FUNC,&v);
  if (!ok) return true;

  if (!JSVAL_IS_VOID(v))
    if (JS_TypeOfValue(GetContext(),v) == JSTYPE_FUNCTION) {
			UserSectionActivate();
      if( JS_CallFunctionName(GetContext(), GetObject(),WLJSNAME_TERMINATE_CLIENT_FUNC,0,0,&v) ) {
				rtn = true;
			} else {
				rtn = false;
			}
			ReportException();
			UserSectionDeactivate();
		}

	return rtn;
}

bool CWLJSThread::OnScriptAbort()
{
	bool rtn( true );
  jsval v;
  JSBool ok;

  CJSGCRequestBeginEnd GCGuard(GetContext());

  ok = JS_LookupProperty(GetContext(),GetObject(),WLJSNAME_ON_SCRIPT_ABORT_FUNC,&v);
  if (!ok) return true;

  if (!JSVAL_IS_VOID(v))
    if (JS_TypeOfValue(GetContext(),v) == JSTYPE_FUNCTION) {
			UserSectionActivate();
      rtn = ( JS_CallFunctionName(GetContext(), GetObject(),WLJSNAME_ON_SCRIPT_ABORT_FUNC,0,0,&v) ? true : false );
	  ReportException();
			UserSectionDeactivate();
			}

	return rtn;
}

bool CWLJSThread::OnErrorTerminateClient()
{
	bool rtn(true);
  jsval v;
  JSBool ok;

  CJSGCRequestBeginEnd GCGuard(GetContext());

  ok = JS_LookupProperty(GetContext(),GetObject(),WLJSNAME_ON_ERROR_TERMINATE_CLIENT_FUNC,&v);
  if (!ok) return true;

  if (!JSVAL_IS_VOID(v))
    if (JS_TypeOfValue(GetContext(),v) == JSTYPE_FUNCTION) {
			UserSectionActivate();
			rtn = (JS_CallFunctionName(GetContext(), GetObject(),WLJSNAME_ON_ERROR_TERMINATE_CLIENT_FUNC,0,0,&v) ? true : false);
			ReportException();
			UserSectionDeactivate();
			}

	return(rtn);
}

void CWLJSThread::ToggleJavaDefaultStatistics( bool rValue )
{
	m_IsJavaDefaultStatistics = rValue;
}

bool CWLJSThread::IsDisabledJavaDefaultStatistics()
{
	return !m_IsJavaDefaultStatistics;
}

CWLJSScript& CWLJSThread::GetJSScript(){
	return *m_JSScript;
}

