/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "loadreportinfo.h"

#include "loadexceptions.h"
#include "loadthreadsmanager.h"
#include "wljsexception.h"
#include "rvjsapi.h"
#include "rvjsclient.h"


iReport::iReport(
	iReportThreadState* rThreadState,
	iReportInfo& rInfo )
:
	mThreadState(rThreadState),
	mInfo(rInfo)
{}

iReport::~iReport() {}

iReportThreadState* iReport::fGetThreadState() const
{
	return mThreadState;
}

iReportInfo& iReport::fGetInfo() const
{
	return mInfo;
}

bool iReport::vIsEvent() const
{
	return true;
}

bool iReport::vHasTime() const
{
	return false;
}

double iReport::vGetTime() const
{
	return 0;
}

iReportThreadState* iReport::vGetifReportThreadState() const
{
	return NULL;
}


void fsCreateErrorString(
	RVCString& oError, 
	iReportThreadState& rThreadState,
	cInternalReportInfo& rInfo )
{

	oError = "";

	// @@Chaim (inherited from Itaj): add roundnum, threadId

	{
		const char* aFileName;
		if( iFile* aFile = rThreadState.fGetFile() ) {
			aFileName = aFile->fGetName();
		} else {
			aFileName = rThreadState.fGetName();
      if( !aFileName ) { // @@Chaim (inherited from Itaj): intg from applets
        aFileName = "unknown";
      }
		}
		oError += aFileName;
		oError += ", ";
	}

	if ( tLineNumber aLineNumber = rThreadState.fGetLineNumber() ) {
		oError += "line ";
		char aBuf[16];
		sprintf(aBuf, "%u: ", aLineNumber);
		oError += aBuf;
	}
	
	oError += " \n";
	oError += rInfo.fGetMessage();


	if( rInfo.fGetJSReport().linebuf ) {

		oError += ":\n";
		oError += rInfo.fGetJSReport().linebuf;
		oError += "\n";

		int i, j, k, n;
		n = rInfo.fGetJSReport().tokenptr - rInfo.fGetJSReport().linebuf;
		for (i = j = 0; i < n; i++) {
			if (rInfo.fGetJSReport().linebuf[i] == '\t') {
				for (k = (j + 8) & ~7; j < k; j++)
					oError += "=";
				continue;
			}
			oError += "=";
			j++;
		}
		oError += "^\n";
	}		
}


cJSInternalReport::cJSInternalReport( 
	iLEJSEmbedContext& rEmbedContext,
	iReportThreadState& rThreadState,
	cInternalReportInfo& rInfo )
:
	iReport( &rThreadState, rInfo )
{
	const char* aSourceMessage = rInfo.fGetMessage();
	WLLogMessage::Severity aSeverity;

	if( rInfo.fIsWarning() ) {
		aSeverity = WLLogMessage::Warning;
	}	else {
		aSeverity = WLLogMessage::Fatal;
		int aId = rInfo.fGetErrorNumber();
    const char* aFormat = WLERRGetifErrorFormat(JSErrorsFrmFile, aId);
		if(!aFormat) {
			aId = 10000;
		}
		aSeverity = WLERRGetErrorSeverity(JSErrorsFrmFile, aId);
	}
	
	if( aSeverity == WLLogMessage::Fatal ) {
		if( iLEJSClientEmbedContext* rEmbedContextX0 = dynamic_cast<iLEJSClientEmbedContext*>(&rEmbedContext) ) {
			aSeverity = WLLogMessage::Error;
		}
	}
		
	RVCString aMessage;
	fsCreateErrorString( aMessage, rThreadState, rInfo );

	mSeverity = aSeverity;
	mMessage = aMessage;
}


WLLogMessage::Severity cJSInternalReport::vGetSeverity() const
{
	return mSeverity;
}


const char* cJSInternalReport::vGetMessage() const
{
	return mMessage.data();
}





iLoadReportInfo::iLoadReportInfo( 
	WLLogMessage::Severity rSeverity, 
	const char* rMessage,
	bool rIsApply,
	bool rIsEvent,
	bool rHasTime,
	double rTime,
	iReportThreadState* rReportThreadState )
:
	mSeverity(rSeverity),
	mMessage(rMessage),
	mIsApply(rIsApply),
	mHasTime(rHasTime),
	mTime(rTime),
	mIsEvent(rIsEvent),
	mReportThreadState(rReportThreadState)
{}


WLLogMessage::Severity iLoadReportInfo::fGetSeverity() const {
	return mSeverity;
	}

const char* iLoadReportInfo::fGetMessage() const {
	return mMessage;
	}

bool iLoadReportInfo::fIsApply() const
{
	return mIsApply;
}

bool iLoadReportInfo::fIsEvent() const 
{
	return mIsEvent;
}

bool iLoadReportInfo::fHasTime() const {
	return mHasTime;
	}

double iLoadReportInfo::fGetTime() const {
	return mTime;
	}

iReportThreadState* iLoadReportInfo::fGetifReportThreadState() const {
	return mReportThreadState;
}

bool iLoadReportInfo::vIsStop() const
{
	bool rtn;

	if( mIsApply ) {
	switch( mSeverity ) {
	case WLLogMessage::Note:
	case WLLogMessage::Warning:
			rtn = false;
		break;
	case WLLogMessage::Error:
	case WLLogMessage::Fatal:
			rtn = true;
		break;
	case WLLogMessage::Success: // why the hell?
	default:
		WlAssert(0);
		break;
		}
	} else {
		rtn = false;
	}

	return rtn;
	}


cLoadReportInfoAgenda::cLoadReportInfoAgenda( 
	WLLogMessage::Severity rSeverity, 
	const char* rMessage )
:
	iLoadReportInfo( rSeverity, rMessage, true, true, false, 0, NULL )
{}

cLoadReportInfoAgenda::cLoadReportInfoAgenda( 
	WLLogMessage::Severity rSeverity, 
	const char* rMessage,
	double rTime )
:
	iLoadReportInfo( rSeverity, rMessage, true, true, true, rTime, NULL )
{}

cLoadReportInfoAgenda::cLoadReportInfoAgenda( 
	WLLogMessage::Severity rSeverity, 
	const char* rMessage,
	bool rHasTime,
	double rTime,
	iReportThreadState* rReportThreadState )
:
	iLoadReportInfo( rSeverity, rMessage, true, true, rHasTime, rTime, rReportThreadState )
{}

cLoadReportInfoClient::cLoadReportInfoClient( 
	WLLogMessage::Severity rSeverity, 
	const char* rMessage,
	bool rIsApply,
	bool rIsLog,
	bool rIsEvent,
	bool rHasTime,
	double rTime )
:
	iLoadReportInfo( rSeverity, rMessage, rIsApply, rIsEvent, rHasTime, rTime, NULL ),
	mIsLog(rIsLog)
{}

cLoadReportInfoClient::cLoadReportInfoClient( 
	WLLogMessage::Severity rSeverity, 
	const char* rMessage,
	bool rIsApply,
	bool rIsLog,
	bool rIsEvent,
	bool rHasTime,
	double rTime,
	iReportThreadState* rReportThreadState )
:
	iLoadReportInfo( rSeverity, rMessage, rIsApply, rIsEvent, rHasTime, rTime, rReportThreadState ),
	mIsLog(rIsLog)
{}



bool cLoadReportInfoClient::fIsLog() const
{
	return mIsLog;
}



iLoadReport::iLoadReport( 
	iReportThreadState* rThreadState,
	iLoadReportInfo& rInfo )
:
	iReport( rThreadState, rInfo )
{}


WLLogMessage::Severity iLoadReport::vGetSeverity() const
{
	WlAssert(dynamic_cast<iLoadReportInfo*>(&fGetInfo()));
	return static_cast<iLoadReportInfo&>(fGetInfo()).fGetSeverity();
}

const char* iLoadReport::vGetMessage() const
{
	WlAssert(dynamic_cast<iLoadReportInfo*>(&fGetInfo()));
	return static_cast<iLoadReportInfo&>(fGetInfo()).fGetMessage();
}

bool iLoadReport::vIsEvent() const
{
	WlAssert(dynamic_cast<iLoadReportInfo*>(&fGetInfo()));
	return static_cast<iLoadReportInfo&>(fGetInfo()).fIsEvent();
}

bool iLoadReport::vHasTime() const
{
	WlAssert(dynamic_cast<iLoadReportInfo*>(&fGetInfo()));
	return static_cast<iLoadReportInfo&>(fGetInfo()).fHasTime();
}

double iLoadReport::vGetTime() const
{
	WlAssert(dynamic_cast<iLoadReportInfo*>(&fGetInfo()));
	return static_cast<iLoadReportInfo&>(fGetInfo()).fGetTime();
}

iReportThreadState* iLoadReport::vGetifReportThreadState() const {
	WlAssert(dynamic_cast<iLoadReportInfo*>(&fGetInfo()));
	return static_cast<iLoadReportInfo&>(fGetInfo()).fGetifReportThreadState();
}


cLoadReport::cLoadReport( 
	iReportThreadState& rThreadState,
	iLoadReportInfo& rInfo )
:
	iLoadReport( &rThreadState, rInfo ),
  mMessage()
{
	const char* aSourceMessage = rInfo.fGetMessage();
	WLLogMessage::Severity aSeverity = rInfo.fGetSeverity();
	bool aHasTime = rInfo.fHasTime();
	double aTime = rInfo.fGetTime();

	RVCString aMessage;
	{
		aMessage = aSourceMessage;

		if( WLLogMessage::Warning <= rInfo.fGetSeverity() ) {
			const char* aFileName = 0;
			tLineNumber aLineNumber;
			iReportThreadState* aThreadState;
			if( !( aThreadState = rInfo.fGetifReportThreadState() ) ) {
				aThreadState = &rThreadState;
			}
			if( iFile* aFile = aThreadState->fGetFile() ) {
				aFileName = aFile->fGetName();
			} else {
				aFileName = aThreadState->fGetName();
			}
			if(!aFileName) {
				aFileName = "unknown";
			}
			aLineNumber = aThreadState->fGetLineNumber();

			if( aFileName ) {
			aMessage += " in ";
			aMessage += aFileName;
			aMessage += " at line ";
			char aBuffer[10];
			sprintf( aBuffer, "%d", (int)aLineNumber );
			aMessage += aBuffer;
		}
	}
	}
  mMessage = aMessage;
}


const char* cLoadReport::vGetMessage() const
{
	return mMessage.data();
}


cLoadReportHard::cLoadReportHard( 
	iLoadReportInfo& rInfo)
:
	iLoadReport(NULL,rInfo)
{}



/*
cLoadReportEvent::cLoadReportEvent( 
	WLLogMessage::Severity rSeverity, 
	const char* rMessage )
:
	mSeverity(rSeverity),
	mMessage(rMessage)
{}


WLLogMessage::Severity cLoadReportEvent::vGetSeverity() const
{
	return mSeverity;
}


const char* cLoadReportEvent::vGetMessage() const
{
	return mMessage.data();
}
*/



iLEEmbedderCallContext::iLEEmbedderCallContext(
	iLEJSEmbedContext& EvalInter )
:
	mEvalInter(EvalInter)
{}



cLEHardEmbedderCallContext::cLEHardEmbedderCallContext(
	iLEJSEmbedContext& EvalInter )
:
	iLEEmbedderCallContext(EvalInter)
{}

bool cLEHardEmbedderCallContext::fGetReturn() {
	return mReturn;
}

void cLEHardEmbedderCallContext::vReport_iEmbedderCallContext(
	iReportInfo& Info )
{

	mEvalInter.fHardReportCommit( Info );

	mReturn = !Info.fIsStop();
}




cLEJSEmbedderCallContext::cLEJSEmbedderCallContext(
	JSContext* Context,
	iLEJSEmbedContext& EvalInter )
:
	iLEEmbedderCallContext(EvalInter),
	cJSEmbedderCallContext(Context)
{}


cReportInfoPosition::cReportInfoPosition(
	coReportThreadState& rReportThreadState)
:
	mDone(false),
	mReportThreadState(rReportThreadState)
{}

cReportInfoPosition::~cReportInfoPosition()
{
	WlAssert(mDone);
}


tDebugReporterReturn cReportInfoPosition::vApplyDebug( 
	cJSDReportCommit& rCommit )
{
	mReportThreadState.fInit( rCommit.fGetJSReport() );
	mDone = true;
	return sDebugReporterReturnApply;
}


void cReportInfoPosition::vApplyJSExec( 
	iJSExecReportCommit& rCommit )
{
	if( !mDone ) {
		mReportThreadState.fInit( rCommit.fGetJSReport() );
		mDone = true;
	}
}

bool cReportInfoPosition::vIsStop() const
{
	return false;
}



bool LoadReport2( 
	iLEJSEmbedContext& rEmbedContext,
	JSContext* Context, 
	iReportInfo& Info	) 
{
	if (dynamic_cast<cLoadReportInfoAgenda*>(&Info)){
		WLLogMessage::Severity Severity = static_cast<cLoadReportInfoAgenda&> (Info).fGetSeverity();
		iLEAgendaJSEmbedContext* aAgendaEmbedContext;
		if (!(aAgendaEmbedContext = dynamic_cast<iLEAgendaJSEmbedContext*>(&rEmbedContext))){
			if (dynamic_cast<iLEJSClientEmbedContext*>(&rEmbedContext)){
				aAgendaEmbedContext = &(static_cast<iLEJSClientEmbedContext&>(rEmbedContext).fGetThreadInstanceScript());
			}
			else{
				WlErrorMsg("Could not get agenda embed context");
				WlAssert(0);
			}
		}
		if (Severity == WLLogMessage::Error || Severity == WLLogMessage::Fatal)
			aAgendaEmbedContext->IncrementErrorCounter();
		else if (Severity == WLLogMessage::Warning)
			aAgendaEmbedContext->IncrementWarningCounter();
	}



	if( rEmbedContext.IsJSActive() ) {
		cLEJSEmbedderCallContext EC( Context, rEmbedContext );
		EC.fReport(Info);
		return EC.fGetJSReturn();
	} else {
		cLEHardEmbedderCallContext EC( rEmbedContext );
		EC.fReport(Info);
		return EC.fGetReturn();
	}
}


bool LoadReportNoExn( 
	JSContext* rContext, 
	iReportInfo& rInfo ) 
{
	CJSGCRequestBeginEnd guard(rContext);
	bool Rtn;
	
	WlAssert(iLEJSEmbedContext::fsGet(rContext));
	Rtn = LoadReport2( *iLEJSEmbedContext::fsGet(rContext), rContext, rInfo );
	return Rtn;
}




bool LoadReport( 
	iLEJSEmbedContext& rEmbedContext,
	JSContext* rContext, 
	iReportInfo& rInfo,
	WLLogMessage::Severity rErrorInTransTree ) 
{
	CJSGCRequestBeginEnd guard(rContext);
	bool aIsExceptions;
	{
		CWLJSScript* aJSScript = NULL;

		if( CLoadGeneralThreadInstanceScript* aTIS = getCurrentThreadInstanceScript(rContext) ) {
			aJSScript = &aTIS->GetJSScript();
		} else if( IGeneralManagerScriptData* aSD = getCurrentManagerScriptData(rContext) ) {
			aJSScript = &aSD->GetJSScript();
		} else {
      //No exceptions for non-agenda code
		}
    if( aJSScript ) {
		  aIsExceptions = aJSScript->m_PropWorkSpace.m_ExceptionEnabled ||
											aJSScript->GetVersionNumber() >= 6.0;
    } else {
      aIsExceptions = false;
    }
	}

	bool aIsCurrentExn = false;
	if( iLoadReportInfo* aInfo = dynamic_cast<iLoadReportInfo*>(&rInfo) ) {
		if( aIsExceptions ) {
			aIsCurrentExn = rEmbedContext.IsJSActive() && aInfo->fIsApply();

			// If we had an error in transaction we start to pend exceptions by ignoring their severity.
			// rErrorInTransTree <= WLLogMessage::Warning  , defaut = -1
			if (rErrorInTransTree <= WLLogMessage::Warning && aInfo->fGetSeverity() <= WLLogMessage::Warning) {
				aIsCurrentExn = false;
			}
		}

		// Throw the exception.
		if(aIsCurrentExn) {
			// If there is a pending exception add the next LoadInfo to it.
			JSObject* aJSExn = NULL;
			if( JS_IsExceptionPending(rContext) ) {
				jsval aJSExnVal;
				JS_GetPendingException(rContext, &aJSExnVal);
				if( RVJS_IsInstanceOf( rContext, aJSExnVal, GetJsExceptionClass() ) ) {
					aJSExn = JSVAL_TO_OBJECT(aJSExnVal);
					CJSExceptionPrivate* aJSException = reinterpret_cast< CJSExceptionPrivate* >( JS_GetPrivate(rContext, aJSExn) );
					WlAssert( aJSException );
					aJSException->SetException(*aInfo);
				}
        // If we ever have a problem in this area, let it be noted that
        // Itaj pondered adding the following line:
        // else WlAssert(0)
			}	else { // create a new exception.
				aJSExn = NewJSExceptionObject(rContext, aInfo->fGetSeverity(), aInfo->fGetMessage(), aInfo->fHasTime(), aInfo->fGetTime());
				JS_SetPendingException(rContext, OBJECT_TO_JSVAL(aJSExn)); 
			}
			aIsCurrentExn = true;
		}
	}

	bool rtn;
	if( aIsCurrentExn ) {
		rtn = false;
	} else {
		rtn = LoadReport2( rEmbedContext, rContext, rInfo);		
	}

	return rtn;
}





// bool DllExport LoadReport( JSContext*, iReportInfo& );
bool LoadReport( 
	JSContext* Context, 
	iReportInfo& rInfo,
	WLLogMessage::Severity rErrorInTransTree ) 
{
	bool Rtn;
	
	//WlAssert(iLEJSEmbedContext::fsGet(Context));
  
  if( iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet(Context) ) {
	  Rtn = LoadReport( *aEmbedContext, Context, rInfo, rErrorInTransTree );
  } else {
     Rtn = true;
  }
	return Rtn;
}





//bool DllExport LoadReport( JSContext*, WLLogMessage::Severity, const char* Message );
//bool DllExport LoadReport( JSContext*, WLErrorException& );

//bool DllExport LoadReport( iLEJSEmbedContext*, JSContext*, WLLogMessage::Severity, const char* Message );
//bool DllExport LoadReport( iLEJSEmbedContext*, JSContext*, WLErrorException& );



bool LoadReport( 
	JSContext* Context, 
	WLLogMessage::Severity Severity, 
	const char* Message,
	WLLogMessage::Severity rErrorInTransTree ) 
{
	
	WlAssert(iLEJSEmbedContext::fsGet(Context));
  
  iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet(Context);
  if (dynamic_cast<iLEJSClientEmbedContext*>(aEmbedContext)){
	  cLoadReportInfoClient loadRepInf( Severity, Message, true, true, false, false, 0, NULL );
	  return LoadReport( Context, loadRepInf, rErrorInTransTree );
  }
  else{
	cLoadReportInfoAgenda loadRepInf( Severity, Message );
	return LoadReport( Context, loadRepInf, rErrorInTransTree );
}
}


bool LoadReportTime( 
	JSContext* Context, 
	WLLogMessage::Severity Severity, 
	const char* Message,
	double Time,
	WLLogMessage::Severity rErrorInTransTree ) 
{
	if (Time){
		WlAssert(iLEJSEmbedContext::fsGet(Context));
		Time -= iLEJSEmbedContext::fsGet(Context)->fGetSession().fGetBeginTime();
	}
  
  iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet(Context);
  if (dynamic_cast<iLEJSClientEmbedContext*>(aEmbedContext)){
	  cLoadReportInfoClient loadRepInf( Severity, Message, true, true, false, true, Time, NULL );
	  return LoadReport( Context, loadRepInf, rErrorInTransTree );
  }
  else{
	cLoadReportInfoAgenda loadRepInf( Severity, Message, Time );
	return LoadReport( Context, loadRepInf, rErrorInTransTree );
}
}

bool LoadReport( 
	JSContext* Context, 
	WLErrorException& E,
	WLLogMessage::Severity rErrorInTransTree ) 
{
	return LoadReport( Context, E.GetErrorSeverity(), E.GetErrorMsg(), rErrorInTransTree );
}

bool LoadReport( 
	iLEJSEmbedContext& rEmbedContext,
	JSContext* Context, 
	WLLogMessage::Severity Severity, 
	const char* Message,
	WLLogMessage::Severity rErrorInTransTree ) 
{
  if (dynamic_cast<iLEJSClientEmbedContext*>(&rEmbedContext)){
	  cLoadReportInfoClient loadRepInf( Severity, Message, true, true, false, false, 0, NULL );
	  return LoadReport( rEmbedContext, Context, loadRepInf, rErrorInTransTree );
  }
  else{
	cLoadReportInfoAgenda loadRepInf( Severity, Message );
	return LoadReport( rEmbedContext, Context, loadRepInf, rErrorInTransTree );
}
}

		
bool LoadReport( 
	iLEJSEmbedContext& rEmbedContext,
	JSContext* Context, 
	WLErrorException& E,
	WLLogMessage::Severity rErrorInTransTree ) 
{
	return LoadReport( rEmbedContext, Context, E.GetErrorSeverity(), E.GetErrorMsg(), rErrorInTransTree );
}






