/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvjsapi.h"

#include "loadjsdefs.h"
#include "loadengineglobals.h"
#include "loadexceptions.h"
#include "wlerrorlib.h"
#include "loadreportinfo.h"
#include "loadgeneratorscript.h"
#include "loadthreadinstance.h"
#include "generalprocess.h"

static JSBool
MessageReporterCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  if ((argc != 1) && (argc != 2)) {
    WLErrorException e(LoadExceptionFrmFile, 221, WLJSNAME_MESSAGE_REPORTER_CLASS, 1, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_MESSAGE_REPORTER_CLASS, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if ((argc == 2))
    if (!JSVAL_IS_STRING(argv[1])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_MESSAGE_REPORTER_CLASS, 2);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }

  JS_SetPrivate(cx, obj, new RVCString(""));

  JS_DefineProperty(cx,obj,WLJSNAME_MESSAGE_REPORTER_NAME,argv[0],NULL,NULL,JSPROP_ENUMERATE);
  if (argc == 2)
    JS_DefineProperty(cx,obj,WLJSNAME_MESSAGE_REPORTER_DESCRIPTION,argv[1],NULL,NULL,JSPROP_ENUMERATE);
  else
    JS_DefineProperty(cx,obj,WLJSNAME_MESSAGE_REPORTER_DESCRIPTION,JSVAL_VOID,NULL,NULL,JSPROP_ENUMERATE);
  
  *rval = OBJECT_TO_JSVAL(obj);
	return JS_TRUE;
}

static JSBool
MessageReporterAddMessage(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_MESSAGE_REPORTER_CLASS"."WLJSNAME_MESSAGE_REPORTER_ADD_MESSAGE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
	if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_MESSAGE_REPORTER_CLASS"."WLJSNAME_MESSAGE_REPORTER_ADD_MESSAGE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	RVCString* data = (RVCString*)JS_GetPrivate(cx, obj);
	*data += JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
	*data += (char)1;
	return JS_TRUE;
}

static JSBool
MessageReporterClear(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_MESSAGE_REPORTER_CLASS"."WLJSNAME_MESSAGE_REPORTER_CLEAR, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	RVCString* data = (RVCString*)JS_GetPrivate(cx, obj);
	*data = "";
	return JS_TRUE;
}

static JSBool
MessageReporterReport(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_MESSAGE_REPORTER_CLASS"."WLJSNAME_MESSAGE_REPORTER_REPORT, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

	RVCString* data = (RVCString*)JS_GetPrivate(cx, obj);
  jsval nameProperty, descriptionProperty;
  const char *name="Undefined", *description="";
  if (JS_GetProperty(cx, obj, WLJSNAME_MESSAGE_REPORTER_NAME, &nameProperty)==JS_TRUE)
    name = JS_GetStringBytes(JSVAL_TO_STRING(nameProperty));
  if (JS_GetProperty(cx, obj, WLJSNAME_MESSAGE_REPORTER_DESCRIPTION, &descriptionProperty)==JS_TRUE)
    if (JSVAL_IS_STRING(descriptionProperty))
      description = JS_GetStringBytes(JSVAL_TO_STRING(descriptionProperty));
  
	CLoadThreadInstanceScript* ti = (CLoadThreadInstanceScript*)getCurrentThreadInstanceScript(cx);
  if (ti) {
    ti->m_MyThreadInstance->VerificationRecordEvent(name, 
                                description, 
																ti->m_MyThreadInstance->fGetProcess().fGetCurrentTime(),
                                WLLogMessage::Note, 
                                "SimpleReport", 
                                (void*)data->data(), 
                                data->length(),
																true ); // @@Chaim (inherited from Itaj): or take parameter?
  }

  *data = "";
	return JS_TRUE;
}

static void
MessageReporterFinalize(JSContext *cx, JSObject *obj)
{
	RVCString* data = (RVCString*)JS_GetPrivate(cx, obj);
	delete data;
}

JSClass MessageReporterClass = {
  WLJSNAME_MESSAGE_REPORTER_CLASS,
  JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,	// add Property
  JS_PropertyStub,	// del Property
  JS_PropertyStub,	// get Property
  JS_PropertyStub,	// set Property
  JS_EnumerateStub,	
	JS_ResolveStub,
	JS_ConvertStub,
	MessageReporterFinalize
};

static JSFunctionSpec MessageReporter_methods[] = {
	{WLJSNAME_MESSAGE_REPORTER_ADD_MESSAGE,	MessageReporterAddMessage,	1},
	{WLJSNAME_MESSAGE_REPORTER_CLEAR,		    MessageReporterClear,	      0},
	{WLJSNAME_MESSAGE_REPORTER_REPORT,		  MessageReporterReport,	    0},
	{0}
};


bool AddMessageReporterClass(JSContext *cx, JSObject *obj)
{
  if (!JS_InitClass(cx, obj, NULL, &MessageReporterClass, MessageReporterCtor, 1, NULL, MessageReporter_methods, NULL, NULL))
		return false;
  
  return true;
}

