/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "outputfile.h"
#include "loadjsdefs.h"
#include "loadexceptions.h"
#include "wlerrorlib.h"
#include "loadgenerator.h"
#include "loadgeneratorscript.h"
#include "loadagendamanager.h"
#include "tracedll.h"
#include "loadreportinfo.h"
#include "loadenginedecs.h"
#include "tvenv.h"
#include "gt_unicode.h"
#include "gt_jshelper.h"

#include "Log.h"

CWLOutputFileHandler::CWLOutputFileHandler() :
  m_ThreadStarted(false)
{
  m_TerminateEvent.Reset();
}

CWLOutputFileHandler::~CWLOutputFileHandler()
{
}

void CWLOutputFileHandler::SetLoadAgendaManager(CLoadAgendaManager* lam) {
  m_LoadAgendaManager = lam;
}
void CWLOutputFileHandler::run() {
	TIMERENTRY("CWLOutputFileHandler::run()");
	CTraceMsg::RegisterThread ();
  TVPManualEvent::SetGuard GCGuard(m_TerminateEvent);
	CAssociativeArray* AssoArray;
	const char* filename;
	CWLOutputFile* f;
	bool st;
	RVCollectableString key;
  const char* ErrorMessageStr;

  AssoArray = (CAssociativeArray*)m_Messages.waitEvent();
	while (!AssoArray->KeyExists( "$TERMINATE" )) {
		filename = AssoArray->GetValueOfParameter("$FILENAME");
		key = filename;
		f = (CWLOutputFile*)m_OutputFileList.findValue(&key);
		if (f == NULL) {
			f = new CWLOutputFile(filename,false,NULL);
			m_OutputFileList.insertKeyAndValue(new RVCollectableString(filename),(RVCollectable*)f);
		}
    if ( AssoArray->KeyExists( "$OPEN" ) ) {
      if ( AssoArray->KeyExists( "$MODE" ) ) {
	const char* openMode = AssoArray->GetValueOfParameter("$MODE");
		    WlTraceMsg("In CWLOutputFileHandler::SetLoadAgendaManager before call to f->Open(openMode)\n");
	st = f->Open(openMode);
      }
      else {
		    WlTraceMsg("In CWLOutputFileHandler::SetLoadAgendaManager before call to f->Open()\n");
      st = f->Open();
      }
      if (!st) {
        WLErrorException e(LoadExceptionFrmFile, 250, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_OPEN, filename);
        ErrorMessageStr =  strdup(e.GetErrorMsg());
      }
    }
    else if ( AssoArray->KeyExists( "$WRITE" ) ) {
			st = f->Write(AssoArray->GetValueOfParameter("$DATA"));
      if (!st) {
        WLErrorException e(LoadExceptionFrmFile, 251, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_WRITE, filename);
        ErrorMessageStr =  strdup(e.GetErrorMsg());
      }
    }
    else if ( AssoArray->KeyExists( "$WRITELN" ) ) {
			st =f->Writeln(AssoArray->GetValueOfParameter("$DATA"));
      if (!st) {
        WLErrorException e(LoadExceptionFrmFile, 251, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_WRITELN, filename);
        ErrorMessageStr =  strdup(e.GetErrorMsg());
      }
    }
    else if ( AssoArray->KeyExists( "$RESET" ) ) {
			st = f->Reset();
      if (!st) {
        WLErrorException e(LoadExceptionFrmFile, 252, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_RESET, filename);
        ErrorMessageStr =  strdup(e.GetErrorMsg());
      }
    }
    else if ( AssoArray->KeyExists( "$CLOSE" ) ) {
			st = f->Close();
      if (!st) {
        WLErrorException e(LoadExceptionFrmFile, 253, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_CLOSE, filename);
        ErrorMessageStr =  strdup(e.GetErrorMsg());
      }
    }
		if (!st) {
      CAssociativeArray ErrorMessage;
		  ErrorMessage.AddCouple("$SEVERITY", (long)WLLogMessage::Fatal);
		  ErrorMessage.AddCouple("$AGENDALOGMESSAGE", ErrorMessageStr);
		  ErrorMessage.AddCouple("$AGENDANAME", m_LoadAgendaManager->Name());
		  ErrorMessage.AddCouple("$TIME", m_LoadAgendaManager->GetLoadGenerator()->GetTimeFromBeginning());
		  m_LoadAgendaManager->reportLogMessageString(ErrorMessage);
			WlTraceMsg("call stopMe %s",ErrorMessage.ToString());
			m_LoadAgendaManager->GetLoadGenerator()->StopMe();
			break;
		}
    delete AssoArray;
    AssoArray = (CAssociativeArray*)m_Messages.waitEvent();
	}
  delete AssoArray;
  RVHashDictionaryIterator i(m_OutputFileList);
  while (i()) {
    CWLOutputFile *f = (CWLOutputFile*)i.value();
		f->Close();
  }
}

void CWLOutputFileHandler::Terminate() {
  if (m_ThreadStarted) {
    CAssociativeArray* terminate = new CAssociativeArray();
    terminate->AddCouple("$TERMINATE","1");
    PutMessage(terminate);
	  m_TerminateEvent.Wait();
  }
}
void CWLOutputFileHandler::PutMessage(CAssociativeArray* Message) {
	if (!m_ThreadStarted) {
		m_ThreadStarted = true;
		Start();
	}

	m_Messages.putEvent(Message);
}

void CWLOutputFileHandler::Start() {
  #if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)            
    m_OutputFileThread = rvMakeThreadFunction(
                        *this,                                        // instance
                        &CWLOutputFileHandler::run // member function
                      );
  #else
    m_OutputFileThread = rvMakeThreadFunction(
                        CWLOutputFileHandler                                   // class
                        *this,                                        // instance
                        void,                                         // return value
                        &CWLOutputFileHandler::run // member function
                      );
  #endif
	m_OutputFileThread.start();
}

CWLOutputFile::CWLOutputFile(const char* filename, bool global, IGeneralManagerScriptData* msd) {
  m_FileName = filename;
  m_File = NULL;
	m_Global = global;
	m_ThreadsManagerScript = msd;
}
CWLOutputFile::~CWLOutputFile() {
}

bool CWLOutputFile::Open(const char* mode /*="w"*/) {
	WlTraceMsg("In CWLOutputFile::Open m_FileName=%s,mode=%s\n",m_FileName.data(),mode);
	if (m_Global) {
		CAssociativeArray Message;
		Message.AddCouple("$OUTPUTFILE","1");
		Message.AddCouple("$OPEN","1");
		Message.AddCouple("$MODE",mode);
		Message.AddCouple("$FILENAME",m_FileName);
		m_ThreadsManagerScript->SendMessage(Message,4);
		return true;
	}
	else {
		if (!m_File)
			return ((m_File = fopen(m_FileName, mode)) != NULL);
		else
			return true;
	}
}

bool CWLOutputFile::Close() {
	if (m_Global) {
		CAssociativeArray Message;
		Message.AddCouple("$OUTPUTFILE","1");
		Message.AddCouple("$CLOSE","1");
		Message.AddCouple("$FILENAME",m_FileName);
		m_ThreadsManagerScript->SendMessage(Message,4);
		return true;
	}
	else {
		if (m_File) {
			FILE* tmpfile = m_File;
			m_File = NULL;
			return (fclose(tmpfile) == 0);
		}
		else
			return true;
	}
}

bool CWLOutputFile::Remove() {
	if (m_Global) {

    // @@Chaim - TBD

    /*
		CAssociativeArray Message;
		Message.AddCouple("$OUTPUTFILE","1");
		Message.AddCouple("$CLOSE","1");
		Message.AddCouple("$FILENAME",m_FileName);
		m_ThreadsManagerScript->SendMessage(Message,4);
    */
		return false;
	}
	else {
		if (m_File) {
			FILE* tmpfile = m_File;
			m_File = NULL;
			if (fclose(tmpfile) == 0){
				return (remove(FileName()) == 0);
			}
			else
				return false;
		}
		else
			return true;
	}
}

bool CWLOutputFile::Write(const char* data) {
	if (m_Global) {
		CAssociativeArray Message;
		Message.AddCouple("$OUTPUTFILE","1");
		Message.AddCouple("$WRITE","1");
		Message.AddCouple("$FILENAME",m_FileName);
		Message.AddCouple("$DATA",data);
		m_ThreadsManagerScript->SendMessage(Message,4);
		return true;
	}
	else {
		if (m_File) {
			int datasize = strlen(data);
			int numwritten = fwrite(data, sizeof( char ), datasize, m_File );
			int ok = fflush(m_File);
			return (ok == 0) && (datasize == numwritten);
		}
		else
			return false;
	}
}

bool CWLOutputFile::Writeln(const char* data) {
	if (m_Global) {
		CAssociativeArray Message;
		Message.AddCouple("$OUTPUTFILE","1");
		Message.AddCouple("$WRITELN","1");
		Message.AddCouple("$FILENAME",m_FileName);
		Message.AddCouple("$DATA",data);
		m_ThreadsManagerScript->SendMessage(Message,4);
		return true;
	}
	else {
		if (m_File) {
			int datasize = strlen(data);
			int numwritten = fwrite( data, sizeof( char ), datasize, m_File );
			if (datasize != numwritten)
				return false;
			numwritten = fwrite( "\n", sizeof( char ), 1, m_File );
			int ok = fflush(m_File);
			return (ok == 0) && (1 == numwritten);
		}
		else
			return false;
	}
}

bool CWLOutputFile::Reset() {
	if (m_Global) {
		CAssociativeArray Message;
		Message.AddCouple("$OUTPUTFILE","1");
		Message.AddCouple("$RESET","1");
		Message.AddCouple("$FILENAME",m_FileName);
		m_ThreadsManagerScript->SendMessage(Message,4);
		return true;
	}
	else {
		if (m_File) {
			return (fseek(m_File, 0L, 0) == 0);
		}
		else
			return false;
	}
}

static JSBool
OutputFileCtor(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	bool GlobalFile = false;
	char fullPath[WL_MAX_PATH];
  IGeneralManagerScriptData* pMSD = getCurrentManagerScriptData(cx);
  if (pMSD != NULL) {
	  if (pMSD->IsSpawn()) {
			GlobalFile = true;
		}
  }

  if (argc != 1 && argc != 2) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_OUTPUT_FILE_CLASS, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

  if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_OUTPUT_FILE_CLASS, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

//Galtech
	sprintf(fullPath,"%s", (char*)ANSIGetStringBytes(JSVAL_TO_STRING(argv[0])) );

	RVCString rwFullPath(CArrangeMachineTempFiles::GetWlTempFullPath(cx,WLEOutputFile, fullPath));
  CWLOutputFile* f = new CWLOutputFile(rwFullPath.data(),GlobalFile, pMSD);


  bool st;
  if ( 2 == argc ) {
    if ( !JSVAL_IS_STRING(argv[1])) {
      WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_OUTPUT_FILE_CLASS, 2);
      return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
    }
    char openMode[10];
    sprintf(openMode,"%s", JS_GetStringBytes(JSVAL_TO_STRING(argv[1])) );
    WlTraceMsg("In OutputFileCtor before call to f->Open(openMode)\n");
    st = f->Open(openMode);
  }
  else {
    WlTraceMsg("In OutputFileCtor before call to f->Open()\n");
    st = f->Open();
  }

  if (!st) {
    WLErrorException e(LoadExceptionFrmFile, 250, WLJSNAME_OUTPUT_FILE_CLASS, f->FileName());
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
  JS_SetPrivate(cx, obj, f);
  return JS_TRUE;
}

static JSBool
OutputFileOpen(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
	if (argc != 0 && argc !=1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_OPEN, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

  CWLOutputFile* f = (CWLOutputFile*)JS_GetPrivate(cx, obj);
  if (!f) return JS_TRUE;

  bool st;
  if ( 1 == argc ) {
    char openMode[10];
    sprintf(openMode,"%s", JS_GetStringBytes(JSVAL_TO_STRING(argv[0])) );
    WlTraceMsg("In OutputFileOpen before call to f->Open(openMode)\n");
    st = f->Open(openMode);
  }
  else {
    WlTraceMsg("In OutputFileOpen before call to f->Open()\n");
    st = f->Open();
  }
  if (st) {
    return JS_TRUE;
  }
  else {
    WLErrorException e(LoadExceptionFrmFile, 250, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_OPEN, f->FileName());
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
}

static JSBool
OutputFileWriteln(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
	if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_WRITELN, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
	if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_WRITELN, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

  CWLOutputFile* f = (CWLOutputFile*)JS_GetPrivate(cx, obj);
  if (!f) return JS_TRUE;

  //Galtech
	RVCString tmp = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
#ifndef _ANSI_ONLY
	int size = UTF8ToANSI(GetMonitorCharSet(), tmp, -1, 0, 0);
	char * ansitmp = new char[size];
	UTF8ToANSI(GetMonitorCharSet(), tmp, -1, ansitmp, size);

  if (f->Writeln(ansitmp))
  {
	  delete [] ansitmp;
    return JS_TRUE;
  }
  else {
	  delete [] ansitmp;
#else
	if (f->Writeln(tmp))
    return JS_TRUE;
  else {
#endif
    WLErrorException e(LoadExceptionFrmFile, 251, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_WRITELN, f->FileName());
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

}

static JSBool
OutputFileWrite(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
	if (argc != 1) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_WRITE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}
	if (!JSVAL_IS_STRING(argv[0])) {
    WLErrorException e(LoadExceptionFrmFile, 222, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_WRITE, 1);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

  CWLOutputFile* f = (CWLOutputFile*)JS_GetPrivate(cx, obj);
  if (!f) return JS_TRUE;

  //Galtech
	RVCString tmp = JS_GetStringBytes(JSVAL_TO_STRING(argv[0]));
#ifndef _ANSI_ONLY
	int size = UTF8ToANSI(GetMonitorCharSet(), tmp, -1, 0, 0);
	char * ansitmp = new char[size];
	UTF8ToANSI(GetMonitorCharSet(), tmp, -1, ansitmp, size);

  if (f->Write(ansitmp))
  {
	  delete [] ansitmp;
    return JS_TRUE;
  }
  else {
	  delete [] ansitmp;
#else
  if (f->Write(tmp))
    return JS_TRUE;
  else {
#endif
    WLErrorException e(LoadExceptionFrmFile, 251, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_WRITELN, f->FileName());
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }

}

static JSBool
OutputFileReset(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
	if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_RESET, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

  CWLOutputFile* f = (CWLOutputFile*)JS_GetPrivate(cx, obj);
  if (!f) return JS_TRUE;

  if (f->Reset())
    return JS_TRUE;
  else {
    WLErrorException e(LoadExceptionFrmFile, 252, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_RESET, f->FileName());
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
}

static JSBool
OutputFileClose(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
	if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_CLOSE, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

  CWLOutputFile* f = (CWLOutputFile*)JS_GetPrivate(cx, obj);
  if (!f) return JS_TRUE;

  if (f->Close())
    return JS_TRUE;
  else {
    WLErrorException e(LoadExceptionFrmFile, 253, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_CLOSE, f->FileName());
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
}

static JSBool
OutputFileRemove(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) {
	if (argc != 0) {
    WLErrorException e(LoadExceptionFrmFile, 220, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_CLOSE, 0);
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
	}

  CWLOutputFile* f = (CWLOutputFile*)JS_GetPrivate(cx, obj);
  if (!f) return JS_TRUE;

  if (f->Remove())
    return JS_TRUE;
  else {
    WLErrorException e(LoadExceptionFrmFile, 253, WLJSNAME_OUTPUT_FILE_CLASS"."WLJSNAME_OUTPUT_FILE_CLOSE, f->FileName());
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );
  }
}

static void
OutputFileFinalize(JSContext *cx, JSObject *obj)
{
	CWLOutputFile* f = (CWLOutputFile*)JS_GetPrivate(cx, obj);
  if (f) {

    // close won't work from dtor (but it was never called anyway because dtor
    // was commented out so I guess it's ok.
//    f->Close();
	  delete f;
    JS_SetPrivate(cx, obj, NULL);
  }
}

JSClass OutputFileClass = {
  WLJSNAME_OUTPUT_FILE_CLASS,
  JSCLASS_HAS_PRIVATE,
  JS_PropertyStub,	// add Property
  JS_PropertyStub,	// del Property
  JS_PropertyStub,	// get Property
  JS_PropertyStub,	// set Property
  JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	OutputFileFinalize
};

static JSFunctionSpec OutputFile_methods[] = {
	{WLJSNAME_OUTPUT_FILE_OPEN,           OutputFileOpen,	    0},
	{WLJSNAME_OUTPUT_FILE_WRITE,          OutputFileWrite,	  1},
	{WLJSNAME_OUTPUT_FILE_WRITELN,        OutputFileWriteln,	1},
	{WLJSNAME_OUTPUT_FILE_RESET,          OutputFileReset,	  0},
	{WLJSNAME_OUTPUT_FILE_CLOSE,          OutputFileClose,	  0},
	{WLJSNAME_OUTPUT_FILE_REMOVE,         OutputFileRemove,	  0},
	{0}
};


bool AddOutputFileClass(JSContext *cx, JSObject *obj)
{
  if (!JS_InitClass(cx, obj, NULL, &OutputFileClass, OutputFileCtor, 1, NULL, OutputFile_methods, NULL, NULL))
		return false;

  return true;
}




