/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvjsclient.h"

#include "gt_unidef.h"


#include "loadthreadinstance.h"
#include "loadjserrorfuncs.h"
#include "generalprocess.h"
#include "loadgeneratorscript.h"
#include "wlleprotocolbasetypes.h"
#include "wlleprotocolclient.h"
#include "loadengineinifileproperties.h"
#include "wljsruntime.h"
#include "rvcsjscompilerintervention.h"


void iClientFile::vPick()
{}

bool iClientFile::vDrop()
{
	delete this;
	return false;
}



const char* FunctionToString(JSContext* rCx, jsval rFuncVal){
	const char* rtn = NULL;

	WlAssert( RVJS_IsFunction( rCx, rFuncVal ) );
	
	if( JSString* aJSStr = JS_ValueToString( rCx, rFuncVal ) ) {
		rtn = JS_GetStringBytes( aJSStr );
	}

	return rtn;
}


cLEJSClientContext::cLEJSClientContext()
:
	mGeneralThreadInstanceScript( NULL ),
	mCx( NULL )
{}


void cLEJSClientContext::fInit(CLoadGeneralThreadInstanceScript& rGeneraThreadInstanceScript )
{
	WlAssert( !mGeneralThreadInstanceScript );
	mGeneralThreadInstanceScript = &rGeneraThreadInstanceScript;

	WlAssert(!mCx);
	mCx = RVJS_NewContext( JS_GetRuntime( mGeneralThreadInstanceScript->m_MyThreadInstance->GetJSScript().GetContext() ), 1000 ); 
	cLoadReportCommit::fsSignContext( mCx );
	mGeneralThreadInstanceScript->m_MyThreadInstance->SignNewJSContext( mCx );
}

void cLEJSClientContext::fFinal()
{
	WlAssert(mCx);

  // Chaim - Final GC performed only on LoadRuntime's context
	JS_DestroyContextNoGC( mCx );
	mCx = 0;
}

cLEJSClientContext::~cLEJSClientContext()
{
	WlAssert(!mCx);
}


CLoadGeneralThreadInstanceScript& cLEJSClientContext::vGetGeneralThreadInstanceScript()
{
	WlAssert(mGeneralThreadInstanceScript);
  return *mGeneralThreadInstanceScript;
}

JSContext* cLEJSClientContext::vGetContext() 
{
	return mCx;
}

cLEJSClientMachineThread::cLEJSClientMachineThread(
	CGeneralProcess& rProcess, 
cLEProtocolJSClientMachineThread& rProtocolJSClientMachineThread )
:
	iLEJSMachineThread( rProcess ),
  m_ProtocolJSClientMachineThread(rProtocolJSClientMachineThread)
{}


cLEJSClientMachineThread::~cLEJSClientMachineThread()
{}

void cLEJSClientMachineThread::fInit( CLoadGeneralThreadInstanceScript& rGeneralThreadInstanceScript )
{
	iLEJSMachineThread::fInit();
	cLEJSClientContext::fInit( rGeneralThreadInstanceScript );
}

void cLEJSClientMachineThread::fFinal()
{
	cLEJSClientContext::fFinal();
	iLEJSMachineThread::fFinal();
}


iLEJSMachineThread& cLEJSClientMachineThread::vGetMachineThread()
{
	return *this;
}

iLEProtocolMachineThread& cLEJSClientMachineThread::GetProtocolMachineThread(WLProtocolIndex pProtocolIndex) {
  return m_ProtocolJSClientMachineThread;
}

iLEJSClientThread::iLEJSClientThread()
:
  mJSThread(NULL)
{}


void iLEJSClientThread::fInit(
	iLEVCJSThread& rJSThread )
{
	WlAssert( !mJSThread );
	mJSThread = &rJSThread;

}

void iLEJSClientThread::fFinal()
{
}

iLEJSClientThread::~iLEJSClientThread()
{
}



CLoadGeneralThreadInstanceScript& iLEJSClientThread::fGetThreadInstanceScript()
{
  WlAssert(mJSThread);
  return mJSThread->fGetGeneralThreadInstanceScript();
}

JSContext* iLEJSClientThread::fGetContext()
{
  WlAssert(mJSThread);
  return mJSThread->fGetContext();
}

iLEVCJSThread& iLEJSClientThread::fGetJSThread()
{
	WlAssert(mJSThread);
  return *mJSThread;
}


CRVCompileErrorInfo::CRVCompileErrorInfo():
  errorNumber(0),
  linebuf(""),
  filename(""),
  currentLine(0)
{}

CRVCompileErrorInfo::~CRVCompileErrorInfo(){
}

iLEJSClientEmbedContext::iLEJSClientEmbedContext(iLEProtocolJSClientEmbedContext& rProtocolJSClientEmbedContext)
:
	iLEJSEmbedContext(),
	mJSGlobal(0),
	mJSActiveCtr(0),
  mProtocolJSClientEmbedContext(rProtocolJSClientEmbedContext),
	mUserSectionActCtr(1),
	mUserSectionActAntiCtr(0)
{}

/*
void iLEJSClientEmbedContext::fInit()
{}

void iLEJSClientEmbedContext::fTerminate()
{}
*/

iLEJSClientEmbedContext::~iLEJSClientEmbedContext()
{
	WlAssert( 1 == mUserSectionActCtr );
	WlAssert( 0 == mUserSectionActAntiCtr );
}


void iLEJSClientEmbedContext::fSetApply()
{
  if( fIsControlMachineThread() ) {
	JSObject* aOldGlobal = JS_GetGlobalObject( fGetJSThread().fGetContext() );
	if( aOldGlobal != mJSGlobal ) {
		JS_SetGlobalObject( fGetJSThread().fGetContext(), mJSGlobal );
	}
	}
}


JSObject* iLEJSClientEmbedContext::fSet( JSObject* rGlobal )
{
	JSObject* rtn = mJSGlobal;
	mJSGlobal = rGlobal;
	fSetApply();
	return rtn;
}

void iLEJSClientEmbedContext::fClear()
{
  // @@Chaim - Bug in reset of global object
  /*
	mJSGlobal = NULL;

	fSetApply();
  */
}

JSBool iLEJSClientEmbedContext::fCallFunction( JSObject* rScope, jsval rFuncVal, uintN rArgc, jsval rArgv[], jsval& oRval )
{
	JSContext* aCx = fGetJSThread().fGetContext();
	++mJSActiveCtr;
	JSBool rtn = JS_CallFunctionValue( aCx, rScope, rFuncVal, rArgc, rArgv, &oRval );
	--mJSActiveCtr;
	return rtn;
}



bool iLEJSClientEmbedContext::IsJSActive()
{
	return mJSActiveCtr > 0;
}


iLEJSMachineThread& iLEJSClientEmbedContext::vGetMachineThread()
{
	return fGetJSThread().fGetMachineThread();
}

iSession& iLEJSClientEmbedContext::vGetSession()
{
	return fGetJSThread().fGetGeneralThreadInstanceScript().fGetSession();
}



JSContext* iLEJSClientEmbedContext::vCurrentJSContext()
{
	return fGetJSThread().fGetContext();
}


void iLEJSClientEmbedContext::reportApply( 
	iReport& rReport )
{

	bool aIsForward( false );
	if( cLoadReportInfoClient* aInfo_Client = ( dynamic_cast<cLoadReportInfoClient*>(&rReport.fGetInfo()) ) ) {
		aIsForward = aInfo_Client->fIsApply();
	}
	if( aIsForward ) {
		fGetThreadInstanceScript().reportApply( rReport );
	}

//	fGetThreadInstanceScript().reportApply( rReport );
}

void iLEJSClientEmbedContext::reportLog( 
	iReport& rReport ) 
{
	WLLogMessage::Severity aSeverity( rReport.fGetSeverity() );
	const char* aMessage = ( rReport.fGetMessage() );

	vReporter( aSeverity, aMessage );

	bool aIsForward( false );
	if( cLoadReportInfoClient* aInfo_Client = ( dynamic_cast<cLoadReportInfoClient*>(&rReport.fGetInfo()) ) ) {
		aIsForward = aInfo_Client->fIsLog();
	} else if( cInternalReportInfo* aInfo_Internal = ( dynamic_cast<cInternalReportInfo*>(&rReport.fGetInfo()) ) ) {
		aIsForward = true;
  }
	if( aIsForward ) {
		fGetThreadInstanceScript().reportLog( rReport );
	}
}

CLoadGeneralThreadInstanceScript& iLEJSClientEmbedContext::fGetThreadInstanceScript()
{
  return fGetThread().fGetThreadInstanceScript();
}

iLEProtocolJSEmbedContext& iLEJSClientEmbedContext::GetProtocolJSEmbedContext(WLProtocolIndex pProtocolIndex){
  // Ignore index
  WlAssert(dynamic_cast<iLEProtocolJSEmbedContext*>(&(mProtocolJSClientEmbedContext)));
  return static_cast<iLEProtocolJSEmbedContext&>(mProtocolJSClientEmbedContext);
}


JSContext* iLEJSClientEmbedContext::fGetContext()
{
	return fGetThread().fGetContext();
}

bool iLEJSClientEmbedContext::Report(iLoadReportInfo& pReportInfo) 
{
  return LoadReport( *this, fGetContext(), pReportInfo );
};

void iLEJSClientEmbedContext::vEnterControlMachineThread_iLEJSEmbedContext()
{
	fSignContext(fGetThread().fGetContext());
	fSetApply();
}

void iLEJSClientEmbedContext::vExitControlMachineThread_iLEJSEmbedContext()
{
	fsUnsignContext(fGetThread().fGetContext());
}

iLEVCJSThread& iLEJSClientEmbedContext::fGetJSThread()
{
	return fGetThread().fGetJSThread();
}

void iLEJSClientEmbedContext::vEnterJava( const char* rClassDescriptionName, const char* rMemberName, tJavaCallType rCallType )
{
	//@@Chaim (inherited from Itaj): think about it
}

void iLEJSClientEmbedContext::vExitJava( JSBool rOK, const char* rClassDescriptionName, const char* rMemberName, tJavaCallType rCallType )
{
}

bool iLEJSClientEmbedContext::vIsJSGC()
{
	return fGetMachineThread().fGetProcess().fIsJSGCInExecutionContext();
}

bool iLEJSClientEmbedContext::vIsUserSection()
{
	WlAssert( 0 <= mUserSectionActCtr );
	WlAssert( 0 <= mUserSectionActAntiCtr );
	bool rtn;
	if( 0 == mUserSectionActAntiCtr ) {
		if( 0 == mUserSectionActCtr ) {
			rtn = false;
		} else {
			rtn = true;
		}
	} else {
		rtn = false;
	}
	return rtn;
}

iLEJSClientEmbedContext::cUserSectionGuard::cUserSectionGuard(
	iLEJSClientEmbedContext& rEmbedContext )
:
	mEmbedContext( rEmbedContext )
{
	++rEmbedContext.mUserSectionActCtr;
}

iLEJSClientEmbedContext::cUserSectionGuard::~cUserSectionGuard()
{
	--mEmbedContext.mUserSectionActCtr;
}



iLEJSClientEmbedContext::cUserSectionOffGuard::cUserSectionOffGuard(
	iLEJSClientEmbedContext& rEmbedContext )
:
	mEmbedContext( rEmbedContext )
{
	++rEmbedContext.mUserSectionActAntiCtr;
}

iLEJSClientEmbedContext::cUserSectionOffGuard::~cUserSectionOffGuard()
{
	--mEmbedContext.mUserSectionActAntiCtr;
}



static long gUncachedCounter = 0;
static long gCachedCounter = 0;
static bool gCacheReadOnly = false;

bool CacheReadOnly(iLEJSClientEmbedContext& rEmbedContext){
  if (!gCacheReadOnly){
	  gCacheReadOnly = (rEmbedContext.fGetSession().GetCompletedRounds()
													>=
										  CLEIniFileProperties::ScriptCacheWarmupLoadRounds);
  }
	return gCacheReadOnly;
}

//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of iLEJSStatement
//////////////////////////////////////////////////////////////////////////////////////////

iLEJSStatement::iLEJSStatement() :
mRefN(0),
mUseCnt(0)
{
}

iLEJSStatement::~iLEJSStatement(){
	// Causes trouble if there's an exception during statement execution
//	WlAssert(!mRefN);
}

//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of iLEJSStatementInfo
//////////////////////////////////////////////////////////////////////////////////////////

iLEJSStatementInfo::iLEJSStatementInfo():
mHashValue(NULL),
mSourceLength(0),
mInit(false)
{
}

iLEJSStatementInfo::~iLEJSStatementInfo(){
}

unsigned iLEJSStatementInfo::hash() const{
	WlAssert(mInit);
	return mHashValue;
}

int iLEJSStatementInfo::fGetSourceLength(){
	WlAssert(mInit);
	return mSourceLength;
}

const char* iLEJSStatementInfo::fGetSource(){
	WlAssert(mInit);
	return fGiveClientFile().fGetSource();
}

void iLEJSStatementInfo::fInit(){
	mInit = true;
	mHashValue = vCalcHashValue();
	mSourceLength = strlen(fGiveClientFile().fGetSource());
}


//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of iLEJSStatementManager
//////////////////////////////////////////////////////////////////////////////////////////

iLEJSStatementManager::iLEJSStatementManager():
mFrozen(false)
{
}

iLEJSStatementManager::~iLEJSStatementManager(){
}

iLEJSStatement* iLEJSStatementManager::FindStatement(const iLEJSStatementInfo* pStatementInfo) {
	RVCollectable* aTemp = findValue(pStatementInfo);
	if (aTemp){
		WlAssert(dynamic_cast<iLEJSStatement*> (aTemp));
		return static_cast<iLEJSStatement*> (aTemp);
	}
	else {
		return NULL;
	}
}

iLEJSStatementInfo* iLEJSStatementManager::AddStatement(iLEJSStatementInfo* pStatementInfo, iLEJSStatement* pStatment){

	RVCollectable* aTemp = insertKeyAndValue(pStatementInfo, pStatment);
	if (aTemp){
		WlAssert(dynamic_cast<iLEJSStatementInfo*> (aTemp));
		pStatment->IncrementRef();
		return static_cast<iLEJSStatementInfo*> (aTemp);
	}
	else {
		return NULL;
	}
}

void iLEJSStatementManager::fFreezeCond(JSContext* rContext, long rCurRound) {
 if (!mFrozen && 
	 (rCurRound >= CLEIniFileProperties::ScriptCacheWarmupLoadRounds + CLEIniFileProperties::ScriptCacheWarmupAnalysisRounds)){
    fFreeze(rContext, rCurRound);
	 }
}

void iLEJSStatementManager::fFreeze(JSContext* rContext, long rCurRound) {
	RVHashDictionaryIterator  iter(*this);

	iLEJSStatement* aCurStatement;

	RVMutexLock::LockGuard lock(mReadWriteLock);

	if (mFrozen) return; // In case someone managed to freeze the cache before we locked it
	
	while (iter()){
		WlAssert(dynamic_cast<iLEJSStatement*>(iter.value()));
		aCurStatement = static_cast<iLEJSStatement*>(iter.value());

		if (aCurStatement->GetUse() < CLEIniFileProperties::UseCountThreshold){
			aCurStatement->DecrementRef();
		}

		if (aCurStatement->GetRef() == 0){
			WlAssert(dynamic_cast<iLEJSStatementInfo*>(iter.key()));
			iLEJSStatementInfo* aCurStatementInfo = static_cast<iLEJSStatementInfo*>(iter.key());
      // @@Chaim: Need trace mechanism for long strings
//			WlTraceMsg("Removing from cache: %s", aCurStatementInfo->vGetIDString());
			
			aCurStatement->vRemoveRoot(rContext);
			iter.remove();
		}
	}

	if (CLEIniFileProperties::ErrorTraceScriptCache){
	WlErrorMsg("Successfully froze cache in round %d", rCurRound);
	}
	WlTraceMsg("Successfully froze cache in round %d", rCurRound);

	mFrozen = true;

}

void iLEJSStatementManager::fFinal(JSContext* rContext) {
	RVHashDictionaryIterator  iter(*this);

	iLEJSStatement* aCurStatement;

	while (iter()){
		WlAssert(dynamic_cast<iLEJSStatement*>(iter.value()));
		aCurStatement = static_cast<iLEJSStatement*>(iter.value());
		aCurStatement->vRemoveRoot(rContext);
		iter.remove();
	}
}

bool iLEJSStatementManager::fIsFrozen(){
	return mFrozen;
}

//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of cLEJSStatementManagerGuard
//////////////////////////////////////////////////////////////////////////////////////////

cLEJSStatementManagerGuard::cLEJSStatementManagerGuard(iLEJSClientEmbedContext& rEmbedContext, iLEJSStatementManager& rStatementManager):
	mGuard(NULL)
{
	if (CLEIniFileProperties::ScriptCacheOn && !rStatementManager.mFrozen){
		cLEJSEmbedContextSuspendResumeRequest aContextSuspendResumeRequest(rEmbedContext);
		mGuard = new RVMutexLock::LockGuard(rStatementManager.mReadWriteLock);
	}
}

cLEJSStatementManagerGuard::~cLEJSStatementManagerGuard() {
	if (mGuard){
		delete mGuard;
	}
}

//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of iLEJSStatementHandle
//////////////////////////////////////////////////////////////////////////////////////////

iLEJSStatementHandle::iLEJSStatementHandle(iLEJSStatementInfo& rInfoSource) :
mStatementInfo(&rInfoSource),
mStatement(NULL),
mOwnStatement(false),
mFinal(false),
mAddedRoot(false),
mCacheFrozen(false)
{
}

iLEJSStatementHandle::~iLEJSStatementHandle(){
	WlAssert(mFinal);
	if (mStatement && mOwnStatement){
		delete mStatement;
	}
}

void iLEJSStatementHandle::fFinal(iLEJSClientEmbedContext& rEmbedContext){
	WlAssert(!mFinal);
	mFinal = true;
//	if (mCompileInfo) delete mCompileInfo;
	if (mStatement && mOwnStatement){
		fRemoveRoot(rEmbedContext.fCurrentJSContext());
	}
}

iLEJSStatement* iLEJSStatementHandle::GetStatement(
																					iLEJSClientEmbedContext& rEmbedContext,
																					iLEJSStatementManager& pStatementManager){

	WlAssert(mCompileInfo);

	cLEJSStatementManagerGuard aGuard(rEmbedContext, pStatementManager);

	if (!mStatement){

		// Cache is not used in RTAM
		if (!CLEIniFileProperties::ScriptCacheOn || rEmbedContext.fGetThreadInstanceScript().IsRTAM()){
			mOwnStatement = CreateStatement(rEmbedContext);
      // @@Chaim: Need trace mechanism for long strings
//			WlTraceMsg("Uncached (cache off): %s", mStatementInfo->vGetIDString());
		}
		else {
			// Start by searching in cache
			if ((mStatement = pStatementManager.FindStatement(mStatementInfo))){
				mOwnStatement = false;
        // @@Chaim: Need trace mechanism for long strings
//				WlTraceMsg("Got from cache: %s", mStatementInfo->vGetIDString());
			}
			else {
				if (CreateStatement(rEmbedContext)){
					if (CacheReadOnly(rEmbedContext)){
						mOwnStatement = true;

						if (CLEIniFileProperties::ErrorTraceScriptCache){
              // @@Chaim: Need trace mechanism for long strings
//							WlErrorMsg("Uncached: %s", mStatementInfo->vGetIDString());
							++gUncachedCounter;
						}

            // @@Chaim: Need trace mechanism for long strings
//						WlTraceMsg("Uncached: %s", mStatementInfo->vGetIDString());
					}
					else {
						mOwnStatement = false;
						pStatementManager.AddStatement(mStatementInfo->fCloneStatementInfo(), mStatement);
            // @@Chaim: Need trace mechanism for long strings
//						WlTraceMsg("Round %d - Added to cache (%d): %s", rEmbedContext.fGetSession().GetCompletedRounds(), ++gCachedCounter, mStatementInfo->vGetIDString());
					}
				}
				else {
					mOwnStatement = false; 
				}
			}
		}
	}
	return mStatement;
}

void iLEJSStatementHandle::fTraceStatement(const char* rStatement){
  try {
    RVCString aTempSource = rStatement;
    long  aIndex = 0;
    long  aPos = 0;
    long aLen = aTempSource.length();
    while (aPos < aTempSource.length()){
      aPos = aTempSource.index("\n", aIndex);
      if (aPos == RV_NPOS) aPos = aTempSource.length();
      if (aPos - aIndex > 512) aPos = aIndex + 512; // @@Chaim
      RVCString aTempSubString = aTempSource(aIndex, aPos - aIndex); 
      WlErrorMsg("%s", aTempSubString.data());
      aIndex = aPos + 1;
    }
  }
  catch (...){
    WlErrorMsg("Error tracing script/function");
  }
}

void iLEJSStatementHandle::fBeforeExecute(iLEJSClientEmbedContext& rEmbedContext, iLEJSStatementManager& rStatementManager){
	if (!(mCacheFrozen = rStatementManager.fIsFrozen())){
		cLEJSStatementManagerGuard aGuard(rEmbedContext, rStatementManager);
		mStatement->IncrementUse();
		mStatement->IncrementRef();
	}
}

void iLEJSStatementHandle::fAfterExecute(iLEJSClientEmbedContext& rEmbedContext, iLEJSStatementManager& rStatementManager){
	if (!mCacheFrozen){
		cLEJSStatementManagerGuard aGuard(rEmbedContext, rStatementManager);
		mStatement->DecrementRef();
	}
}


//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of cLEJSFunctionStatement
//////////////////////////////////////////////////////////////////////////////////////////

cLEJSFunctionStatement::cLEJSFunctionStatement():
mFuncVal(JSVAL_VOID){
}

cLEJSFunctionStatement::~cLEJSFunctionStatement(){
}

jsval cLEJSFunctionStatement::fGetFuncVal() {
	return mFuncVal;
}

const char* cLEJSFunctionStatement::fGetToString(JSContext* rCx)
{
	return FunctionToString (rCx, mFuncVal);
}

const char* cLEJSFunctionStatement::fGetToString(iLEJSClientEmbedContext& rEmbedContext)
{
 	JSContext* aCx = rEmbedContext.fCurrentJSContext();
	return FunctionToString(aCx , mFuncVal);
}


//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of cLEJSFunctionStatementInfo
//////////////////////////////////////////////////////////////////////////////////////////
cLEJSFunctionStatementInfo::cLEJSFunctionStatementInfo(const char* pName):
mName(pName)
{
}

cLEJSFunctionStatementInfo::~cLEJSFunctionStatementInfo(){
}

const char* cLEJSFunctionStatementInfo::vGetIDString() {
	return fGetSource();
}
	
const char* cLEJSFunctionStatementInfo::vGetName() {
	return mName.data() ;
}

bool cLEJSFunctionStatementInfo::isEqual(const RVCollectable* t) const {
	WlAssert(dynamic_cast<const cLEJSFunctionStatementInfo*>(t));
	WlAssert(fGiveClientFile().fGetSource());

	const cLEJSFunctionStatementInfo* aCurStatementInfo = static_cast<const cLEJSFunctionStatementInfo*>(t);
	WlAssert(aCurStatementInfo->fGiveClientFile().fGetSource());

	bool ret = (!mName.compareTo(aCurStatementInfo->mName) &&
						  !strcmp(fGiveClientFile().fGetSource(), aCurStatementInfo->fGiveClientFile().fGetSource()));
	return ret;
}

unsigned cLEJSFunctionStatementInfo::vCalcHashValue() {
	RVCString aSource(fGiveClientFile().fGetSource());
	return mName.hash() ^ aSource.hash();
}

//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of cLEJSFunctionStatementCompileInfo
//////////////////////////////////////////////////////////////////////////////////////////

cLEJSFunctionStatementCompileInfo::cLEJSFunctionStatementCompileInfo(JSObject* rScope, uintN rNargs, jsval* rArgNames, uintN rLineNumber):
  iLEJSStatementCompileInfo(rScope),
	mNargs(rNargs),
	mArgNames(rArgNames),
	mLineNumber(rLineNumber)
{
}

cLEJSFunctionStatementCompileInfo::~cLEJSFunctionStatementCompileInfo(){
}

//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of cLEJSFunctionHandle
//////////////////////////////////////////////////////////////////////////////////////////

cLEJSFunctionHandle::cLEJSFunctionHandle(cLEJSFunctionStatementInfo& pFunctionStatementInfo):
iLEJSStatementHandle(pFunctionStatementInfo),
mFuncCloneObject(NULL),
mFuncVal(JSVAL_VOID)
{
}

cLEJSFunctionHandle::~cLEJSFunctionHandle(){
}

int cLEJSFunctionHandle::fGetSourceLength(){
	WlAssert(dynamic_cast<cLEJSFunctionStatementInfo*>(mStatementInfo));
	cLEJSFunctionStatementInfo* aFunctionStatementInfo = static_cast<cLEJSFunctionStatementInfo*>(mStatementInfo);
	return aFunctionStatementInfo->fGetSourceLength();
}

const char* cLEJSFunctionHandle::fGetSource(){
	WlAssert(dynamic_cast<cLEJSFunctionStatementInfo*>(mStatementInfo));
	cLEJSFunctionStatementInfo* aFunctionStatementInfo = static_cast<cLEJSFunctionStatementInfo*>(mStatementInfo);
	return aFunctionStatementInfo->fGetSource();
}

const char* cLEJSFunctionHandle::fGetName(){
	WlAssert(dynamic_cast<cLEJSFunctionStatementInfo*>(mStatementInfo));
	cLEJSFunctionStatementInfo* aFunctionStatementInfo = static_cast<cLEJSFunctionStatementInfo*>(mStatementInfo);
	return aFunctionStatementInfo->vGetName();
}

bool cLEJSFunctionHandle::fCompile(iLEJSClientEmbedContext& rEmbedContext){
 	JSContext* aCx = rEmbedContext.fCurrentJSContext();

	WlAssert(dynamic_cast<cLEJSFunctionStatementCompileInfo*>(mCompileInfo));
	cLEJSFunctionStatementCompileInfo& aFunctionStatementCompileInfo = static_cast<cLEJSFunctionStatementCompileInfo&>(*mCompileInfo);

	WlAssert(dynamic_cast<cLEJSFunctionStatementInfo*>(mStatementInfo));
	cLEJSFunctionStatementInfo* aFunctionStatementInfo = static_cast<cLEJSFunctionStatementInfo*>(mStatementInfo);

	WlAssert(dynamic_cast<cLEJSFunctionStatement*>(mStatement));
	cLEJSFunctionStatement* aFunctionStatement = static_cast<cLEJSFunctionStatement*>(mStatement);

	bool aStatus = false;

	ecPointerToZTS<iFile> aFileName(mStatementInfo->fGiveClientFile());

	cLEJSEmbedContextBeginEndRequest aGuard(rEmbedContext);
  RVCString aSource = aFunctionStatementInfo->fGetSource();

  //rEmbedContext.SetCharset(aFunctionStatementInfo->fGetCharset());
  
	JSFunction* aFunc = JS_CompileFunction(aCx,
																				 aFunctionStatementCompileInfo.mScope,
																				 aFunctionStatementInfo->vGetName(),
																				 aFunctionStatementCompileInfo.mNargs,
																				 reinterpret_cast<const char**>(aFunctionStatementCompileInfo.mArgNames),
																				 aSource.data(),
																				 aSource.length(),
																				 aFileName,
																				 aFunctionStatementCompileInfo.mLineNumber);
	if( aFunc ) {
		JSObject* aFuncObj = JS_GetFunctionObject( aFunc );
		JS_SetParent( aCx, aFuncObj, NULL );
		aFunctionStatement->mFuncVal = OBJECT_TO_JSVAL( aFuncObj );
		fAddRoot(rEmbedContext.fCurrentJSContext());
		aStatus = true;
	}
	else {
		if (CLEIniFileProperties::ErrorTraceCSJSCompileErrors){
  		WlErrorMsg("Error compiling function %s:", aFunctionStatementInfo->vGetName());
      fTraceStatement(aFunctionStatementInfo->fGetSource());
		}
		aStatus = false;
	}

	bool rtn;
	if(aStatus) {
		rtn = true;
	} else {
		rtn = false;
		mFuncVal = JSVAL_VOID;
	}

	return rtn;
}

bool cLEJSFunctionHandle::fSet(iLEJSClientEmbedContext& rEmbedContext, cLEJSFunctionStatementCompileInfo& rFunctionCompileInfo){

	mFuncVal = JSVAL_VOID;
	
	SetCompileInfo(&rFunctionCompileInfo);
	return (GetStatement(rEmbedContext,
											 rEmbedContext.fGetSession().fGetProcess().GetLoadRuntime().m_FunctionCache) != NULL);


}

bool cLEJSFunctionHandle::fSet( 
	iLEJSClientEmbedContext& rEmbedContext,
	jsval rFuncVal )
{
 	JSContext* aCx = rEmbedContext.fCurrentJSContext();

	WlAssert( RVJS_IsFunction( aCx, rFuncVal ) );

	mFuncVal = rFuncVal;
	mFuncCloneObject = NULL;
	JS_SetParent( aCx, JSVAL_TO_OBJECT( mFuncVal ), NULL );
	return true;
}

bool cLEJSFunctionHandle::fIsSet(){
  return mFuncVal != JSVAL_VOID;
}

JSBool cLEJSFunctionHandle::fCall( iLEJSClientEmbedContext& rEmbedContext, jsval& oRval){

	fBeforeExecute(rEmbedContext, rEmbedContext.fGetSession().fGetProcess().GetLoadRuntime().m_FunctionCache);
	JSBool ret;

	if (mFuncVal != JSVAL_VOID){
		ret = fCall(rEmbedContext, mFuncVal, oRval);
	}
	else {
		cLEJSFunctionStatement* aFunctionStatement = static_cast<cLEJSFunctionStatement*>(mStatement);

		// Logic is different if using cached function
		if (mOwnStatement){
			ret = fCall(rEmbedContext, aFunctionStatement->fGetFuncVal(), oRval);
		}
		else{

			WlAssert(dynamic_cast<cLEJSFunctionStatementCompileInfo*>(mCompileInfo));
			cLEJSFunctionStatementCompileInfo* aFunctionStatementCompileInfo = static_cast<cLEJSFunctionStatementCompileInfo*>(mCompileInfo);

			// Clone function object and call clone
			if (mFuncCloneObject = JS_CloneFunctionObject(rEmbedContext.fCurrentJSContext(), JSVAL_TO_OBJECT(aFunctionStatement->fGetFuncVal()), aFunctionStatementCompileInfo->mScope)){
				JS_AddNamedRoot(rEmbedContext.fCurrentJSContext(), &mFuncCloneObject, "cLEJSFunctionHandle::fCall - Temp func object");
				mStatement->IncrementUse();
				ret = fCall(rEmbedContext, OBJECT_TO_JSVAL(mFuncCloneObject), oRval);
				JS_RemoveRoot(rEmbedContext.fCurrentJSContext(), &mFuncCloneObject);
			}
			mFuncCloneObject = NULL;
		}
	}
	fAfterExecute(rEmbedContext, rEmbedContext.fGetSession().fGetProcess().GetLoadRuntime().m_FunctionCache);
	return ret;
}

JSBool cLEJSFunctionHandle::fCall( iLEJSClientEmbedContext& rEmbedContext, jsval rFuncVal, jsval& oRval){

	WlAssert(dynamic_cast<cLEJSFunctionStatementCompileInfo*>(mCompileInfo));
	cLEJSFunctionStatementCompileInfo* aFunctionStatementCompileInfo = static_cast<cLEJSFunctionStatementCompileInfo*>(mCompileInfo);

	JSContext* aCx = rEmbedContext.fCurrentJSContext();
	WlAssert(RVJS_IsFunction(aCx, rFuncVal));
		
	JS_SetParent(aCx, JSVAL_TO_OBJECT(rFuncVal), aFunctionStatementCompileInfo->mScope);

	++rEmbedContext.mJSActiveCtr;
	JSBool rtn = JS_CallFunctionValue(aCx,
		aFunctionStatementCompileInfo->mScope,
		rFuncVal,
  	aFunctionStatementCompileInfo->mNargs,
		aFunctionStatementCompileInfo->mArgNames,
	  &oRval);

	--rEmbedContext.mJSActiveCtr;
	
	JS_SetParent(aCx, JSVAL_TO_OBJECT(rFuncVal), NULL);

	return rtn;
}

iLEJSStatement* cLEJSFunctionHandle::CreateStatement(iLEJSClientEmbedContext& rEmbedContext){
	mStatementInfo->fCloneClientFile();
	mStatement = new cLEJSFunctionStatement();
	
	if (!fCompile(rEmbedContext)){
		delete mStatement;
		mStatement = NULL;
	}

	return mStatement;
}

jsval cLEJSFunctionHandle::fGetFuncVal(iLEJSClientEmbedContext& rEmbedContext){

  cLEJSFunctionStatement* aFunctionStatement =
  static_cast<cLEJSFunctionStatement*>(mStatement);

	jsval ret;
  jsval unclonedFunc;

	if (mFuncVal != JSVAL_VOID){
		ret = mFuncVal;
    return ret;
	}
	else {
    unclonedFunc = aFunctionStatement->fGetFuncVal();
	}

  JSObject* retObj;

	JSContext* aCx = rEmbedContext.fCurrentJSContext();
	WlAssert(RVJS_IsFunction(aCx, unclonedFunc));

	WlAssert(dynamic_cast<cLEJSFunctionStatementCompileInfo*>(mCompileInfo));
	cLEJSFunctionStatementCompileInfo* aFunctionStatementCompileInfo = static_cast<cLEJSFunctionStatementCompileInfo*>(mCompileInfo);
        
	retObj = JS_CloneFunctionObject(aCx, JSVAL_TO_OBJECT(unclonedFunc), aFunctionStatementCompileInfo->mScope);
  WlAssert(retObj);

  JS_SetParent(aCx, retObj, aFunctionStatementCompileInfo->mScope);

  ret = OBJECT_TO_JSVAL(retObj);
	return ret;
}

const char* cLEJSFunctionHandle::fGetToString(JSContext* rCx){
	const char* ret;
	if (mFuncVal != JSVAL_VOID){
		ret = FunctionToString(rCx , mFuncVal);;
	}
	else {
			cLEJSFunctionStatement* aFunctionStatement =
				static_cast<cLEJSFunctionStatement*>(mStatement);
			ret = FunctionToString(rCx , aFunctionStatement->fGetFuncVal());
		}
	return ret;
}

const char* cLEJSFunctionHandle::fGetToString(iLEJSClientEmbedContext& rEmbedContext){
	return fGetToString(rEmbedContext.fCurrentJSContext());
}


//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of cLEJSScriptStatement
//////////////////////////////////////////////////////////////////////////////////////////

cLEJSScriptStatement::cLEJSScriptStatement():
mCode(""),
mScript(NULL),
mRootObject(NULL)
{
}

cLEJSScriptStatement::~cLEJSScriptStatement(){
}


//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of cLEJSScriptStatementInfo
//////////////////////////////////////////////////////////////////////////////////////////
cLEJSScriptStatementInfo::cLEJSScriptStatementInfo(){
}
	
cLEJSScriptStatementInfo::~cLEJSScriptStatementInfo(){
}

const char* cLEJSScriptStatementInfo::vGetIDString(){
	return fGiveClientFile().fGetSource();
}

bool cLEJSScriptStatementInfo::isEqual(const RVCollectable* t) const {
	WlAssert(dynamic_cast<const cLEJSScriptStatementInfo*>(t));
	WlAssert(fGiveClientFile().fGetSource());

	const cLEJSScriptStatementInfo* aCurStatementInfo = static_cast<const cLEJSScriptStatementInfo*>(t);
	WlAssert(aCurStatementInfo->fGiveClientFile().fGetSource());

	bool ret = (!strcmp(fGiveClientFile().fGetSource(), aCurStatementInfo->fGiveClientFile().fGetSource()));
	return ret;
}

unsigned cLEJSScriptStatementInfo::vCalcHashValue(){
	RVCString aSource(fGiveClientFile().fGetSource());
	return aSource.hash();
 }

//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of cLEJSScriptStatementCompileInfo
//////////////////////////////////////////////////////////////////////////////////////////

cLEJSScriptStatementCompileInfo::cLEJSScriptStatementCompileInfo(JSObject* rScope):
  iLEJSStatementCompileInfo(rScope)
{
}

cLEJSScriptStatementCompileInfo::~cLEJSScriptStatementCompileInfo(){
}


//////////////////////////////////////////////////////////////////////////////////////////
// Implementation of cLEJSScriptHandle
//////////////////////////////////////////////////////////////////////////////////////////

cLEJSScriptHandle::cLEJSScriptHandle(cLEJSScriptStatementInfo& pScriptStatementInfo):
iLEJSStatementHandle(pScriptStatementInfo),
mExecPrologDone(false),
mExecMainDone(false)
{
}

cLEJSScriptHandle::~cLEJSScriptHandle(){
	if (mExecPrologDone)
		WlAssert(mExecMainDone);
}

int cLEJSScriptHandle::fGetSourceLength(){
	WlAssert(dynamic_cast<cLEJSScriptStatementInfo*>(mStatementInfo));
	cLEJSScriptStatementInfo* aScriptStatementInfo = static_cast<cLEJSScriptStatementInfo*>(mStatementInfo);
	return aScriptStatementInfo->fGetSourceLength();
}

const char* cLEJSScriptHandle::fGetSource(){
	WlAssert(dynamic_cast<cLEJSScriptStatementInfo*>(mStatementInfo));
	cLEJSScriptStatementInfo* aScriptStatementInfo = static_cast<cLEJSScriptStatementInfo*>(mStatementInfo);
	return aScriptStatementInfo->fGetSource();
}

bool cLEJSScriptHandle::fCompile(iLEJSClientEmbedContext& rEmbedContext){

	WlAssert(dynamic_cast<cLEJSScriptStatementCompileInfo*>(mCompileInfo));
	cLEJSScriptStatementCompileInfo* aScriptStatementCompileInfo = static_cast<cLEJSScriptStatementCompileInfo*>(mCompileInfo);

	WlAssert(dynamic_cast<cLEJSScriptStatementInfo*>(mStatementInfo));
	cLEJSScriptStatementInfo* aScriptStatementInfo = static_cast<cLEJSScriptStatementInfo*>(mStatementInfo);

	WlAssert(dynamic_cast<cLEJSScriptStatement*>(mStatement));
	cLEJSScriptStatement* aScriptStatement = static_cast<cLEJSScriptStatement*>(mStatement);

 	JSContext* aCx = rEmbedContext.fCurrentJSContext();

	JSBool rtn = JS_FALSE;
	ecPointerToZTS<iFile> aName(mStatementInfo->fGiveClientFile());
   
	cLEJSEmbedContextBeginEndRequest aGuard(rEmbedContext);

  bool aSucessOrGiveUp = false;
  int aFixAttemptCount = 0;
  RVCString aSource = aScriptStatementInfo->fGetSource();

  //rEmbedContext.SetCharset(aScriptStatementInfo->fGetCharset());
  
  while (!aSucessOrGiveUp){
	  JSScript* aScript = JS_CompileScript(aCx,
																			   aScriptStatementCompileInfo->mScope,
																			   aSource.data(),
																			   aSource.length(),
																			   aName,
																			   1 );


	  if (aScript) {
		  JSObject* aRootObject;
		  aRootObject = JS_NewScriptObject(aCx, aScript);
		  WlAssert(aRootObject);
		  if( aRootObject ) {
			  aScriptStatement->mScript = aScript;
			  aScriptStatement->mRootObject = aRootObject;
			  fAddRoot(rEmbedContext.fCurrentJSContext());
			  rtn = JS_TRUE;
        aSucessOrGiveUp = true;
		  }
	  }
	  else {
      CompileErrorType compileErrorType = fsGetCurrentCompileErrorType(aCx);

      if (compileErrorType == CompileErrorUnfixable){
        if (CLEIniFileProperties::ErrorTraceCSJSCompileErrors){
  		    WlErrorMsg("Error compiling script:");
          fTraceStatement(aScriptStatementInfo->fGetSource());
		    }

		    rtn = JS_FALSE;
        aSucessOrGiveUp = true;

      }

      // Try to fix and clear exception
      else {
        if (CLEIniFileProperties::CompilerIntervention) {
          if (++aFixAttemptCount <= CLEIniFileProperties::CompilerInterventionCount) {
            bool aFixerRet = fsFixCompileError(aCx, compileErrorType, aSource);
            WlAssert(aFixerRet);
            JS_ClearPendingException(aCx);
            aSucessOrGiveUp = false;
          }
          else {
            aSucessOrGiveUp = true;
          }
        }
        else {
          aSucessOrGiveUp = true;
        }
      }
    }

  }
	return rtn;
}
	
JSBool cLEJSScriptHandle::fExecute( iLEJSClientEmbedContext& rEmbedContext, cLEJSScriptStatementCompileInfo& rScriptCompileInfo, jsval& oRval ){

	JSBool ret;

	SetCompileInfo(&rScriptCompileInfo);
	if (GetStatement(rEmbedContext,rEmbedContext.fGetSession().fGetProcess().GetLoadRuntime().m_ScriptCache) != NULL){
		fBeforeExecute(rEmbedContext, rEmbedContext.fGetSession().fGetProcess().GetLoadRuntime().m_ScriptCache);

		WlAssert(dynamic_cast<cLEJSScriptStatement*>(mStatement));
		cLEJSScriptStatement* aScriptStatement = static_cast<cLEJSScriptStatement*>(mStatement);

		ret = JS_ExecuteScript(rEmbedContext.fCurrentJSContext(),
													 rScriptCompileInfo.mScope,
													 aScriptStatement->fGetScript(),
													 &oRval);

		fAfterExecute(rEmbedContext, rEmbedContext.fGetSession().fGetProcess().GetLoadRuntime().m_ScriptCache);

	}
	else{
		ret = JS_FALSE;
	}
	return ret;

}
	
JSBool cLEJSScriptHandle::fExecuteProlog( iLEJSClientEmbedContext& rEmbedContext, cLEJSScriptStatementCompileInfo& rScriptCompileInfo, jsval& oRval ){

	JSBool ret;

	SetCompileInfo(&rScriptCompileInfo);
	if (GetStatement(rEmbedContext,
									 rEmbedContext.fGetSession().fGetProcess().GetLoadRuntime().m_ScriptCache) != NULL){
		WlAssert(dynamic_cast<cLEJSScriptStatement*>(mStatement));
		cLEJSScriptStatement* aScriptStatement = static_cast<cLEJSScriptStatement*>(mStatement);

		fBeforeExecute(rEmbedContext, rEmbedContext.fGetSession().fGetProcess().GetLoadRuntime().m_ScriptCache);

		ret = JS_ExecuteScriptPart(rEmbedContext.fCurrentJSContext(),
															 rScriptCompileInfo.mScope,
															 aScriptStatement->fGetScript(),
															 JSEXEC_PROLOG,
															 &oRval);

		mExecPrologDone = true;
	}
	else {
		ret = JS_FALSE;
		mExecPrologDone = false;
	}

	return ret;

}

	
JSBool cLEJSScriptHandle::fExecuteMain( iLEJSClientEmbedContext& rEmbedContext, jsval& oRval ){
	
	WlAssert(mExecPrologDone);

	WlAssert(dynamic_cast<cLEJSScriptStatementCompileInfo*>(mCompileInfo));
	cLEJSScriptStatementCompileInfo* aScriptStatementCompileInfo = static_cast<cLEJSScriptStatementCompileInfo*>(mCompileInfo);

	WlAssert(dynamic_cast<cLEJSScriptStatement*>(mStatement));
	cLEJSScriptStatement* aScriptStatement = static_cast<cLEJSScriptStatement*>(mStatement);

	JSBool ret;

	ret = JS_ExecuteScriptPart(rEmbedContext.fCurrentJSContext(),
														 aScriptStatementCompileInfo->mScope,
														 aScriptStatement->fGetScript(),
														 JSEXEC_MAIN,
														 &oRval);

	fAfterExecute(rEmbedContext, rEmbedContext.fGetSession().fGetProcess().GetLoadRuntime().m_ScriptCache);

	mExecMainDone = true;

	return ret;

}

iLEJSStatement* cLEJSScriptHandle::CreateStatement(iLEJSClientEmbedContext& rEmbedContext){
	mStatementInfo->fCloneClientFile();
	mStatement = new cLEJSScriptStatement();
	
	// JS Stuff
	if (!fCompile(rEmbedContext)){
		delete mStatement;
		mStatement = NULL;
	}

	return mStatement;
}
