/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include <typeinfo>

#include "rvjsmachinethread.h"
#include "wlleprotocol.h"
#include "generalprocess.h"
#include "loadengineinifileproperties.h"
#include "tracedll.h"

cLEJSMachineThreadKey::cLEJSMachineThreadKey()
{}

cLEJSMachineThreadKey::cLEJSMachineThreadKey(
	RVThreadId rId )
:
	RVCollectableInt( ::rvThreadHash( rId ) )
{}


iLEJSMachineThread::iLEJSMachineThread(
	CGeneralProcess& rProcess)
:
	iRVJavaMachineThread(),
	mProcess(rProcess),
	mCurrentEmbedContext(NULL),
	mGCLockDepth(0)
{}

iLEJSMachineThread::~iLEJSMachineThread()
{
}

void iLEJSMachineThread::fInit()
{
	mThreadId = ::rvThreadId();
	mThreadKey = cKey( mThreadId );
	mProcess.fThreadStart( 
		mThreadKey /* caller allocation responsibility */, 
		*this );
	iRVJavaMachineThread::fInit();

	CTraceMsg::RegisterThreadInfo (typeid(*this).name());
}

void iLEJSMachineThread::fFinal()
{

	iRVJavaMachineThread::fFinal();
	mProcess.fThreadEnd( mThreadKey );
}

RVThreadId iLEJSMachineThread::fGetMachineThreadId()
{
	return mThreadId;
}

CGeneralProcess& iLEJSMachineThread::fGetProcess()
{
	return mProcess;
}

cRVJVM& iLEJSMachineThread::vGetJVM() 
{
	return mProcess.fGetJVMManager().fGetJVM();
}


iLEJSEmbedContext* iLEJSMachineThread::fGetCurrentEmbedContext()
{
//	WlAssert(mCurrentEmbedContext); //@@Chaim (inherited from Itaj): why not ?
	return mCurrentEmbedContext;
}

iLEJSEmbedContext* iLEJSMachineThread::fSetCurrentEmbedContext( iLEJSEmbedContext* r )
{
	iLEJSEmbedContext* aPrev = mCurrentEmbedContext;
	mCurrentEmbedContext = r;
	return aPrev;
}

iLEJSAgendaMachineThread::iLEJSAgendaMachineThread(CGeneralProcess& rProcess):
  iLEJSMachineThread(rProcess)
{
}

iLEJSAgendaMachineThread::~iLEJSAgendaMachineThread(){
}

void iLEJSAgendaMachineThread::fInit(WLProtocolThreadType p_ProtocolThreadType)
{
  iLEJSMachineThread::fInit();
  ProtocolAgendaMachineThreadSetUp(mProcess.GetLEProtocolProcessContainer() , p_ProtocolThreadType);
  ProtocolAgendaMachineThreadInitialize();
}

void iLEJSAgendaMachineThread::fFinal()
{
  ProtocolAgendaMachineThreadTermination();
  iLEJSMachineThread::fFinal();
}

iLEProtocolMachineThread& iLEJSAgendaMachineThread::GetProtocolMachineThread(WLProtocolIndex protocolIndex) {
	WlAssert(dynamic_cast<iLEProtocolAgendaMachineThread*>(&(m_ProtocolAgendaMachineThreadContainer.GetProtocolAtIndex(protocolIndex))));	
	return static_cast<iLEProtocolAgendaMachineThread&>(m_ProtocolAgendaMachineThreadContainer.GetProtocolAtIndex(protocolIndex));	
}

void iLEJSAgendaMachineThread::ProtocolAgendaMachineThreadSetUp(iLEProtocolContainer& p_ProtocolProcessContainer, WLProtocolThreadType p_ProtocolThreadType){
	WlTraceMsg("Calling NewProtocolAgendaMachineThread for all protocols");
  int numProtocols = p_ProtocolProcessContainer.NumProtocols();
  m_ProtocolAgendaMachineThreadContainer.InitProtocolContainer(numProtocols);
	for(WLProtocolIndex indx=0; indx< m_ProtocolAgendaMachineThreadContainer.NumProtocols(); indx++) {
    WlAssert(dynamic_cast<iLEProtocolProcess*>(&(p_ProtocolProcessContainer.GetProtocolAtIndex(indx))));
    iLEProtocolAgendaMachineThread& a_ProtocolAgendaMachineThread =
          static_cast<iLEProtocolProcess&>(p_ProtocolProcessContainer.GetProtocolAtIndex(indx)).NewProtocolAgendaMachineThread(p_ProtocolThreadType);
    a_ProtocolAgendaMachineThread.Init(this);
    m_ProtocolAgendaMachineThreadContainer.InsertProtocolAtIndex(indx, a_ProtocolAgendaMachineThread);
  }
}

void iLEJSAgendaMachineThread::ProtocolAgendaMachineThreadInitialize(){
	WlTraceMsg("Calling MachineThreadInitialize for all protocols");
	for(WLProtocolIndex indx=0; indx< m_ProtocolAgendaMachineThreadContainer.NumProtocols(); indx++) {
    if (m_ProtocolAgendaMachineThreadContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolAgendaMachineThread*>(&(m_ProtocolAgendaMachineThreadContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolAgendaMachineThread&>(m_ProtocolAgendaMachineThreadContainer.GetProtocolAtIndex(indx)).MachineThreadInitialize()){
			  WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolAgendaMachineThreadInitialize");
		  }
    }
  }
}

void iLEJSAgendaMachineThread::ProtocolAgendaMachineThreadTermination(){
	WlTraceMsg("Calling MachineThreadTermination for all protocols");
  for (WLProtocolIndex indx = m_ProtocolAgendaMachineThreadContainer.NumProtocols() - 1; indx >= 0; indx--){
    if (m_ProtocolAgendaMachineThreadContainer.ProtocolExists(indx)){
      WlAssert(dynamic_cast<iLEProtocolAgendaMachineThread*>(&(m_ProtocolAgendaMachineThreadContainer.GetProtocolAtIndex(indx))));
      if (!static_cast<iLEProtocolAgendaMachineThread&>(m_ProtocolAgendaMachineThreadContainer.GetProtocolAtIndex(indx)).MachineThreadTermination()){
			  WlTraceMsg("Error 1 in function CLoadGeneralThreadInstanceScript::ProtocolAgendaMachineThreadTermination");
		  }
    }
  }
}

