/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//
// File: WLGlobals.cpp

#include "wlglobalsobject.h"


///////////////////////////////////////////////////////////////////////////////
//
// Implementation of CWLGlobalsObject class
///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// Constructor\Destructor
////////////////////////////////////////////////////////////////////////

CWLGlobalsObject::CWLGlobalsObject(const char *name)
:m_VariablesDictionary(10)
{	
}

CWLGlobalsObject::~CWLGlobalsObject(void)
{	
}


///////////////////////////////////////////////////////////////////////
// Methods
///////////////////////////////////////////////////////////////////////

/*
 * Set Method - assigns new properties to <variableName>,
 * creates a new entry to this variable if needed
 */
void CWLGlobalsObject::Set(const char* variableName, CGlobalVariableProperties *newProperties)
{
	CGlobalVariableProperties *oldProperties;

	oldProperties = m_VariablesDictionary.ReplaceVariableProperties(variableName, newProperties);

	if(!oldProperties)
		m_VariablesDictionary.AddVariableAndProperties(variableName, newProperties);
	else
		delete oldProperties;
}

/*
 * Get Method - retruns the properties of <variableName>.
 */
CGlobalVariableProperties* CWLGlobalsObject::Get(const char* variableName)
{
	CGlobalVariableProperties* properties;

	properties = m_VariablesDictionary.GetVariableProperties(variableName);

	if(!properties) 
		properties = new CGlobalVariableProperties("U", "Undefind");

	return properties;
}

/*
 * Add Method - adds the constant's value to the current
 * value of <variableName> (numbers only). 
 */
void CWLGlobalsObject::Add(const char* variableName, CGlobalVariableProperties *value)
{
	CGlobalVariableProperties *properties;

	properties = m_VariablesDictionary.GetVariableProperties(variableName);

	if(!properties) {
		Set(variableName, value);
		return;
	}
	
	RVCString charValue;
	if(value->IsStringType() || properties->IsStringType()) {
			charValue = properties->GetUnformatedCharValue();
			charValue += value->GetUnformatedCharValue();
			properties->SetValue(charValue.data());
			delete value;
			return;
	}
	double currentDValue, newDValue;
	GlobalsBool currentBoolValue, newBoolValue;

	if(properties->GetNumberValue(currentDValue)) {
		if(value->GetNumberValue(newDValue)) 
			properties->SetValue(currentDValue + newDValue);
		else if(value->GetBoolValue(newBoolValue))
			properties->SetValue(currentDValue + newBoolValue);
	}
	else if(properties->GetBoolValue(currentBoolValue)) {
		if(value->GetBoolValue(newBoolValue))
			properties->SetValue(double(currentBoolValue + newBoolValue));
		else if(value->GetNumberValue(currentDValue)) 
			properties->SetValue(double(currentDValue + currentBoolValue));
	}
	delete value;
}

/** Add in later: **
if(currentBoolValue) 
			charValue = "true";
		else 
			charValue = "false";
		charValue += properties->GetUnformatedCharValue();
		properties->SetValue(charValue.data());
*/

///////////////////////////////////////////////////////////////////////////////
//
// Implementation of CWLGlobalsDictionary class
///////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
// Constructor\Destructor
////////////////////////////////////////////////////////////////////////

CWLGlobalsDictionary::CWLGlobalsDictionary(size_t n)
:RVHashDictionary(n)
{
}

CWLGlobalsDictionary::~CWLGlobalsDictionary()
{
	clearAndDestroy();
}


///////////////////////////////////////////////////////////////////////
// Methods
///////////////////////////////////////////////////////////////////////

CGlobalVariableProperties* CWLGlobalsDictionary::GetVariableProperties(const char* variableName)
{
	RVCollectableString key(variableName);
	
	return (CGlobalVariableProperties*)findValue(&key);
}

CGlobalVariableProperties* CWLGlobalsDictionary::ReplaceVariableProperties(const char* variableName, CGlobalVariableProperties *variableProperties)
{
	RVCollectableString key(variableName);
	RVCollectable *value = (RVCollectable*)variableProperties;

	return (CGlobalVariableProperties*)findValue(&key, value);
}

bool CWLGlobalsDictionary::AddVariableAndProperties(const char* variableName, CGlobalVariableProperties *variableProperties)
{
	RVCollectableString *key = new RVCollectableString(variableName);
	RVCollectable *value = (RVCollectable*)variableProperties;
	
	if(insertKeyAndValue(key, value))
		return true;
	return false;
}
