/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLGlobalsTable.cpp: implementation of the CWLGlobalsTable class.
//
//////////////////////////////////////////////////////////////////////

#include "wlglobalstable.h"
#include "wlglobalsobject.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWLGlobalsTable::CWLGlobalsTable(): 
RVHashDictionary(10)
{	
	
}

CWLGlobalsTable::~CWLGlobalsTable()
{
	clearAndDestroy();
}

void CWLGlobalsTable::Set(const char* globlasObjectName, const char* variableName, 
													CGlobalVariableProperties *newProperties)
{
	CWLGlobalsObject* globals = GetGlobalsObject(globlasObjectName);

	if(!globals) 
		globals = CreateWLGlobalsObject(globlasObjectName);

	globals->Set(variableName, newProperties);
}

CGlobalVariableProperties* CWLGlobalsTable::Get(const char* globlasObjectName, 
																								const char* variableName)
{
	CWLGlobalsObject* globals = GetGlobalsObject(globlasObjectName);

	if(!globals) 
		globals = CreateWLGlobalsObject(globlasObjectName);

	return globals->Get(variableName);
}

void CWLGlobalsTable::Add(const char* globlasObjectName, const char* variableName,
													CGlobalVariableProperties *value)
{
	CWLGlobalsObject* globals = GetGlobalsObject(globlasObjectName);

	if(!globals) 
		globals = CreateWLGlobalsObject(globlasObjectName);

	globals->Add(variableName, value);
}

CGlobalVariableProperties* CWLGlobalsTable::SyncAdd(const char* globlasObjectName, const char* variableName,
																										CGlobalVariableProperties *value)
{
	CWLGlobalsObject* globals = GetGlobalsObject(globlasObjectName);

	if(!globals) 
		globals = CreateWLGlobalsObject(globlasObjectName);

	globals->Add(variableName, value);

	return globals->Get(variableName);
}

CWLGlobalsObject *CWLGlobalsTable::CreateWLGlobalsObject(const char *requestedName) 
{
	RVCollectableString *key = new RVCollectableString(requestedName);
	CWLGlobalsObject *value = new CWLGlobalsObject(requestedName);

	if(insertKeyAndValue(key, value))
		return value;

	delete key;
	delete value;
	return NULL;
}

void CWLGlobalsTable::RemoveGlobalsObject(const char* globlasObjectName)
{
	RVCollectableString key(globlasObjectName);

	removeAndDestroy(&key);
}

inline CWLGlobalsObject *CWLGlobalsTable::GetGlobalsObject(const char* globlasObjectName)
{
	RVCollectableString key(globlasObjectName);
	
	return (CWLGlobalsObject*)findValue(&key); 
}

