/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <RVCString.h>

#include "rvjsapi.h"

#include "loadjsdefs.h"
#include "loadthreadsmanager.h"
#include "loadgeneratorscript.h"
#include "threadglobalshandler.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "wljsglobalvariables.h"
#include "loadengineglobals.h"



/*********************************************************************
* WebLoad JS Global Variables Implementation
**********************************************************************/

#define SUPER_GLOBALS_ERROR_NAME "Global Variables"


static IGeneralGlobalsRequestHandler::Level
GetGlobalsLevel(JSContext *cx, bool isLGLevel, long levelFlag)
{
	if(isLGLevel) {
		if(levelFlag == 0)
			return IGeneralGlobalsRequestHandler::LGLevelPerAgenda;
		else
			return IGeneralGlobalsRequestHandler::LGLevelPerAgendaInMix;
	}

	if(levelFlag == 0)
		return IGeneralGlobalsRequestHandler::MetaLevel;
	else
		return IGeneralGlobalsRequestHandler::MetaLevelPerAgenda;
}

/**********************************************************************
 * WLGlobals Set(name, value, level)
 **********************************************************************/
static JSBool
globals_Set(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	jsval jsValue;
	CGlobalVariableProperties properties;
	IGeneralGlobalsRequestHandler* handler;

	if(argc != 3) {
    WLErrorException e(LoadExceptionFrmFile, 220, "Global Variables - Set", 3);
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
  }

	iLEAgendaJSEmbedContext* pAgendaEmbedContext = iLEAgendaJSEmbedContext::fGetAgendaEmbedContext(cx);
	handler = pAgendaEmbedContext->GetGlobalsRequestHandler();

	const char* variableName = JS_GetStringBytes( JS_ValueToString( cx, argv[0] ) );

	if( JSVAL_IS_OBJECT(argv[1]) ) {
		if( !JS_ConvertValue(cx, argv[1], JSTYPE_VOID, &jsValue) ) {
	    WLErrorException e(LoadExceptionFrmFile, 257, SUPER_GLOBALS_ERROR_NAME, 2);
			return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
		}
	}
	else
		jsValue = argv[1];

	if( JSVAL_IS_BOOLEAN(jsValue) ) {
		GlobalsBool value = (GlobalsBool)JSVAL_TO_BOOLEAN(jsValue);
		properties.SetValue(value);
	}
	else if( JSVAL_IS_NUMBER(jsValue) ) {
		double value;

		if( JSVAL_IS_INT(jsValue) )
			value = (double)JSVAL_TO_INT(jsValue);
		else
			value = (double)(*(JSVAL_TO_DOUBLE(jsValue)));
		properties.SetValue(value);
	}
	else if( JSVAL_IS_STRING(jsValue) ) {
		const char* value = JS_GetStringBytes(JSVAL_TO_STRING(jsValue));
		properties.SetValue(value);
	}
	else {
		const char* value = JS_GetStringBytes(JS_ValueToString(cx, jsValue));
		if(value)
			properties.SetValue(value);
		else
			WlAssert(0); // general failure
	}

	int32 flag;
	if( !JS_ValueToInt32(cx, argv[2], &flag) ) {
		WLErrorException e(LoadExceptionFrmFile, 258, SUPER_GLOBALS_ERROR_NAME, 3);
		return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
	}
	else if( (int)flag != 0 && (int)flag != 1 ) {
			WLErrorException e(LoadExceptionFrmFile, 258, SUPER_GLOBALS_ERROR_NAME, 3);
			return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
	}
	bool isLGLevel = *((bool*)JS_GetPrivate(cx, obj));
	IGeneralGlobalsRequestHandler::Level level = GetGlobalsLevel(cx, isLGLevel, (long)flag);

	handler->Globals_Set(variableName, &properties, level);
	return JS_TRUE;
}

/**********************************************************************
 * WLGlobals Get(name, level)
 **********************************************************************/
static JSBool
globals_Get(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	CGlobalVariableProperties* properties;
	IGeneralGlobalsRequestHandler* handler;

	if(argc != 2) {
    WLErrorException e(LoadExceptionFrmFile, 220, "Global Variables - Get", 2);
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
  }

	iLEAgendaJSEmbedContext* pAgendaEmbedContext = iLEAgendaJSEmbedContext::fGetAgendaEmbedContext(cx);
	handler = pAgendaEmbedContext->GetGlobalsRequestHandler();

	const char* variableName = JS_GetStringBytes( JS_ValueToString( cx, argv[0] ) );

	int32 flag;
	if( !JS_ValueToInt32(cx, argv[1], &flag) ) {
		WLErrorException e(LoadExceptionFrmFile, 258, SUPER_GLOBALS_ERROR_NAME, 2);
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
	}
	else if( (int)flag != 0 && (int)flag != 1 ) {
		WLErrorException e(LoadExceptionFrmFile, 258, SUPER_GLOBALS_ERROR_NAME, 3);
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
	}
	bool isLGLevel = *((bool*)JS_GetPrivate(cx, obj));
	IGeneralGlobalsRequestHandler::Level level = GetGlobalsLevel(cx, isLGLevel, (long)flag);

	properties = handler->Globals_Get(variableName, level);

 	// Eyal - unnecessary?
  //if(handler->IsAbort()) {
	//	delete properties;
	//	return JS_TRUE;
	//}

	if(properties->IsBoolType()) { // boolean
		GlobalsBool value;
		properties->GetBoolValue(value);
		*rval = value ? JSVAL_TRUE : JSVAL_FALSE;
	}
	else if(properties->IsNumberType()) { // number
		double value;
		properties->GetNumberValue(value);
		*rval = DOUBLE_TO_JSVAL(JS_NewDouble(cx, (jsdouble)value));
	}
	else if(properties->IsStringType()) { // string
		const char* value = properties->GetUnformatedCharValue();
		*rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, value));
	}
	else { // Undefined
		*rval = JSVAL_VOID;
	}
	delete properties;

	return JS_TRUE;
}

/**********************************************************************
 * WLGlobals Add(name, value, level)
 **********************************************************************/
static JSBool
globals_Add(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	jsval jsValue;
	CGlobalVariableProperties properties;
	IGeneralGlobalsRequestHandler* handler;

	if(argc != 3) {
    WLErrorException e(LoadExceptionFrmFile, 220, "Global Variables - Add", 3);
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
  }

	iLEAgendaJSEmbedContext* pAgendaEmbedContext = iLEAgendaJSEmbedContext::fGetAgendaEmbedContext(cx);
	handler = pAgendaEmbedContext->GetGlobalsRequestHandler();

	const char* variableName = JS_GetStringBytes( JS_ValueToString( cx, argv[0] ) );

	if( JSVAL_IS_OBJECT(argv[1]) ) {
		if( !JS_ConvertValue(cx, argv[1], JSTYPE_VOID, &jsValue) ) {
	    WLErrorException e(LoadExceptionFrmFile, 257, SUPER_GLOBALS_ERROR_NAME, 2);
			return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
		}
	}
	else
		jsValue = argv[1];

	if( JSVAL_IS_BOOLEAN(jsValue) ) {
		GlobalsBool value = (GlobalsBool)JSVAL_TO_BOOLEAN(jsValue);
		properties.SetValue(value);
	}
	else if( JSVAL_IS_NUMBER(jsValue) ) {
		double value;

		if( JSVAL_IS_INT(jsValue) )
			value = (double)JSVAL_TO_INT(jsValue);
		else
			value = (double)(*(JSVAL_TO_DOUBLE(jsValue)));

		properties.SetValue(value);
	}
	else if( JSVAL_IS_STRING(jsValue) ) {
		const char* value = JS_GetStringBytes(JSVAL_TO_STRING(jsValue));
		properties.SetValue(value);
	}
	else {
		const char* value = JS_GetStringBytes(JS_ValueToString(cx, jsValue));
		if(value)
			properties.SetValue(value);
		else
			WlAssert(0); // general failure
	}

	int32 flag;
	if( !JS_ValueToInt32(cx, argv[2], &flag) ) {
		WLErrorException e(LoadExceptionFrmFile, 258, SUPER_GLOBALS_ERROR_NAME, 3);
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
	}
	else if( (int)flag != 0 && (int)flag != 1 ) {
		WLErrorException e(LoadExceptionFrmFile, 258, SUPER_GLOBALS_ERROR_NAME, 3);
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
	}
	bool isLGLevel = *((bool*)JS_GetPrivate(cx, obj));
	IGeneralGlobalsRequestHandler::Level level = GetGlobalsLevel(cx, isLGLevel, (long)flag);

	handler->Globals_Add(variableName, &properties, level);
	return JS_TRUE;
}

/**********************************************************************
 * WLGlobals AddEx(name, value, level)
 **********************************************************************/
static JSBool
globals_AddEx(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
	jsval jsValue;
	CGlobalVariableProperties properties;
	IGeneralGlobalsRequestHandler* handler;

	if(argc != 3) {
    WLErrorException e(LoadExceptionFrmFile, 220, "Global Variables - Add", 3);
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
  }

	iLEAgendaJSEmbedContext* pAgendaEmbedContext = iLEAgendaJSEmbedContext::fGetAgendaEmbedContext(cx);
	handler = pAgendaEmbedContext->GetGlobalsRequestHandler();

	const char* variableName = JS_GetStringBytes( JS_ValueToString( cx, argv[0] ) );

	if( JSVAL_IS_OBJECT(argv[1]) ) {
		if( !JS_ConvertValue(cx, argv[1], JSTYPE_VOID, &jsValue) ) {
	    WLErrorException e(LoadExceptionFrmFile, 257, SUPER_GLOBALS_ERROR_NAME, 2);
			return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
		}
	}
	else
		jsValue = argv[1];

	if( JSVAL_IS_BOOLEAN(jsValue) ) {
		GlobalsBool value = (GlobalsBool)JSVAL_TO_BOOLEAN(jsValue);
		properties.SetValue(value);
	}
	else if( JSVAL_IS_NUMBER(jsValue) ) {
		double value;

		if( JSVAL_IS_INT(jsValue) )
			value = (double)JSVAL_TO_INT(jsValue);
		else
			value = (double)(*(JSVAL_TO_DOUBLE(jsValue)));

		properties.SetValue(value);
	}
	else if( JSVAL_IS_STRING(jsValue) ) {
		const char* value = JS_GetStringBytes(JSVAL_TO_STRING(jsValue));
		properties.SetValue(value);
	}
	else {
		const char* value = JS_GetStringBytes(JS_ValueToString(cx, jsValue));
		if(value)
			properties.SetValue(value);
		else
			WlAssert(0); // general failure
	}

	int32 flag;
	if( !JS_ValueToInt32(cx, argv[2], &flag) ) {
		WLErrorException e(LoadExceptionFrmFile, 258, SUPER_GLOBALS_ERROR_NAME, 3);
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
	}
	else if( (int)flag != 0 && (int)flag != 1 ) {
		WLErrorException e(LoadExceptionFrmFile, 258, SUPER_GLOBALS_ERROR_NAME, 3);
    return LoadReport(cx, e.GetErrorSeverity(), e.GetErrorMsg());
	}
	bool isLGLevel = *((bool*)JS_GetPrivate(cx, obj));
	IGeneralGlobalsRequestHandler::Level level = GetGlobalsLevel(cx, isLGLevel, (long)flag);

	CGlobalVariableProperties* retProperties;
	retProperties = handler->Globals_SyncAdd(variableName, &properties, level);

	// Eyal - unnecessary?
	//if(handler->IsAbort()) {
	//	delete retProperties;
	//	return JS_TRUE;
	//}

	if(retProperties->IsNumberType()) { // number
		double value;
		retProperties->GetNumberValue(value);
		*rval = DOUBLE_TO_JSVAL(JS_NewDouble(cx, (jsdouble)value));
	}
	else if(retProperties->IsBoolType()) { // boolean
		GlobalsBool value;
		retProperties->GetBoolValue(value);
		*rval = value ? JSVAL_TRUE : JSVAL_FALSE;
	}
	else if(retProperties->IsStringType()) { // string
		const char* value = retProperties->GetUnformatedCharValue();
		*rval = STRING_TO_JSVAL(JS_NewStringCopyZ(cx, value));
	}
	else { // Undefined
		*rval = JSVAL_VOID;
	}

	delete retProperties;

	return JS_TRUE;
}

static void
SuperGlobalsFinalize(JSContext *cx, JSObject *obj)
{
	bool *b = (bool*)JS_GetPrivate(cx, obj);
  if(b) {
    delete b;
    JS_SetPrivate(cx, obj, NULL);
  }
}

///////////////////////////////////////////
// JS Class and Object defenitions
///////////////////////////////////////////

JSClass js_SuperGlobalsClass = {
    WLJSNAME_SUPER_GLOBALS_CLASS,
    JSCLASS_HAS_PRIVATE,
    JS_PropertyStub,	// add Property
		JS_PropertyStub,	// del Property
		JS_PropertyStub,	// get Property
		JS_PropertyStub,	// set Property
    JS_EnumerateStub,
		JS_ResolveStub,
		JS_ConvertStub,
		SuperGlobalsFinalize
};

static JSFunctionSpec globalvariables_methods[] = {
	{WLJSNAME_SUPER_GLOBALS_ADD,	globals_Add,	3},
	{WLJSNAME_SUPER_GLOBALS_ADDEX,	globals_AddEx,3},
	{WLJSNAME_SUPER_GLOBALS_SET,	globals_Set,	3},
	{WLJSNAME_SUPER_GLOBALS_GET,	globals_Get,	2},
	{0}
};

JSObject*
js_AddGeneratorGlobalsObject(JSContext *cx, JSObject *obj)
{
	JSObject* wlGeneratorGlobals =
		JS_DefineObject(cx, obj, WLJSNAME_GENERATOR_GLOBAL_OBJECT, &js_SuperGlobalsClass, NULL, JSPROP_PERMANENT);

	if (!wlGeneratorGlobals)
		return NULL;

	// define it's methods
	if (!JS_DefineFunctions(cx, wlGeneratorGlobals, globalvariables_methods))
		return NULL;

	bool *isLGLevel = new bool(true);

	if(!JS_SetPrivate(cx, wlGeneratorGlobals, isLGLevel))
		return NULL;

	return wlGeneratorGlobals;
}


JSObject*
js_AddMetaGlobalsObject(JSContext *cx, JSObject *obj)
{
	JSObject* wlMetaGlobals =
		JS_DefineObject(cx, obj, WLJSNAME_META_GLOBAL_OBJECT, &js_SuperGlobalsClass, NULL, JSPROP_PERMANENT);

	if (!wlMetaGlobals)
		return NULL;

	// define it's methods
	if (!JS_DefineFunctions(cx, wlMetaGlobals, globalvariables_methods))
		return NULL;

	bool *isLGLevel = new bool(false);

	if(!JS_SetPrivate(cx, wlMetaGlobals, isLGLevel))
		return NULL;

	return wlMetaGlobals;
}

