/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// WLJSJavaApplet.cpp: implementation of the CWLJSJavaApplet class.
//
//////////////////////////////////////////////////////////////////////

#include "tracedll.h"
#include "wljsjavaapplet.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadjsshell.h"
#include <string.h>
#ifdef __TV_WINDOWS__
#include <setupapi.h>
#endif // __TV_WINDOWS__
#include <jni.h>
#include "loadenginedecs.h"
#include "loadengineglobals.h"
#include "loadgeneratorscript.h"
#include "loadthreadinstance.h"
#include "pointertojava.h"
#include "rvjsembed.h"
#include "rvjsclient.h"
#include "tvenv.h"
#include "com_radview_loadengine_javathreadwrapper.h"
#include "com_radview_loadengine_javauiplayeroptions.h"

//Galtech
#include "gt_unicode.h"
#include "gt_jshelper.h"

/* Manually load the required native methods. */
bool DllExport InitLoadEngineJNI(JNIEnv* env)
{

  // commented, till java recorder/player is needed
  // when this comment will be removed, we have to make sure
  // that a failure here, shows an error message to the user.
  // there's no error indication to the user right now.

  // register the native functions
  jclass jcJavaThreadWrapper = env->FindClass("com/radview/system/JavaThreadWrapper");
  WlAssert(jcJavaThreadWrapper);
  static JNINativeMethod nativeMethods[] = {
    {"jniSetTimer", "(Ljava/lang/String;Ljava/lang/String;)V", (void*)&Java_com_radview_loadengine_JavaThreadWrapper_jniSetTimer},
    {"jniSendTimer", "(Ljava/lang/String;Ljava/lang/String;)V", (void*)&Java_com_radview_loadengine_JavaThreadWrapper_jniSendTimer},
    {"jniSendCounter", "(Ljava/lang/String;Ljava/lang/String;)V", (void*)&Java_com_radview_loadengine_JavaThreadWrapper_jniSendCounter},
    {"jniIsAbort", "(Ljava/lang/String;)Z", (void*)&Java_com_radview_loadengine_JavaThreadWrapper_jniIsAbort},
  };

  env->RegisterNatives(jcJavaThreadWrapper, nativeMethods, sizeof(nativeMethods) / sizeof(JNINativeMethod));
  if (env->ExceptionOccurred()) {
    //
    // report the error 
    env->ExceptionDescribe();
    env->ExceptionClear();
    WlAssert(0);
    //return false;
  }

  env->DeleteLocalRef(jcJavaThreadWrapper);


  //
  // Initialize the JNI manager.
  if (!CWLJavaJniManager::fsGetSingleton().Init(env)) {
    return false;
  }


  return true;
}

bool DllExport TerminateLoadEngineJNI(JNIEnv* env) {
  // commented out, till java recorder/player is needed
  // when this comment will be removed, we have to make sure
  // that a failure here, shows an error message to the user.
  // there's no error indication to the user right now.
  CWLJavaJniManager::fsGetSingleton().Terminate(env);
	
	return true;
}


/*****************************************************************
* JavaThreadWrapper functions
*****************************************************************/
/*
 * Class:     com_radview_loadengine_JavaThreadWrapper
 * Method:    jniSetTimer
 * Signature: (Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_com_radview_loadengine_JavaThreadWrapper_jniSetTimer
  (JNIEnv *env, jobject obj, jstring threadData, jstring timerName)
{
  CPointerToJava ptj(env, threadData);
  CLoadGeneralThreadInstanceScript* aTIS = reinterpret_cast<CLoadGeneralThreadInstanceScript*>(ptj.getPointer());

  const char *cTimerName = env->GetStringUTFChars(timerName, 0);
  aTIS->resetTimer(cTimerName);
  env->ReleaseStringUTFChars(timerName, cTimerName);
}

/*
 * Class:     com_radview_loadengine_JavaThreadWrapper
 * Method:    jniSendTimer
 * Signature: (Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_com_radview_loadengine_JavaThreadWrapper_jniSendTimer
  (JNIEnv *env, jobject obj, jstring threadData, jstring timerName)
{
  CPointerToJava ptj(env, threadData);
  CLoadGeneralThreadInstanceScript* aTIS = reinterpret_cast<CLoadGeneralThreadInstanceScript*>(ptj.getPointer());

  const char *cTimerName = env->GetStringUTFChars(timerName, 0);
  aTIS->sendTimer(cTimerName);
  env->ReleaseStringUTFChars(timerName, cTimerName);
}



/*
 * Class:     com_radview_loadengine_JavaThreadWrapper
 * Method:    jniSendCounter
 * Signature: (Ljava/lang/String;Ljava/lang/String;)V
 */
JNIEXPORT void JNICALL Java_com_radview_loadengine_JavaThreadWrapper_jniSendCounter
  (JNIEnv *env, jobject obj, jstring threadData, jstring counterName)
{
  CPointerToJava ptj(env, threadData);
  CLoadGeneralThreadInstanceScript* aTIS = reinterpret_cast<CLoadGeneralThreadInstanceScript*>(ptj.getPointer());

  const char *cCounterName = env->GetStringUTFChars(counterName, 0);
  aTIS->addCounter(cCounterName);
  env->ReleaseStringUTFChars(counterName, cCounterName);
}

/*
 * Class:     com_radview_loadengine_JavaThreadWrapper
 * Method:    jniIsAbort
 * Signature: (Ljava/lang/String;)Z
 */
JNIEXPORT jboolean JNICALL Java_com_radview_loadengine_JavaThreadWrapper_jniIsAbort
  (JNIEnv *env, jobject obj, jstring threadData)
{
  CPointerToJava ptj(env, threadData);
  CLoadGeneralThreadInstanceScript* aTIS = reinterpret_cast<CLoadGeneralThreadInstanceScript*>(ptj.getPointer());

  return aTIS->isAbort();
}


/*****************************************************************
* JavaUIPlayerOptions functions
*****************************************************************/
/*
 * Class:     Java_com_radview_loadengine_JavaUIPlayerOptions_getUIOption
 * Method:    getUIOption
 * Signature: (Ljava/lang/String;)Ljava/lang/String;
 */
JNIEXPORT jstring JNICALL Java_com_radview_loadengine_JavaUIPlayerOptions_getUIOption
(JNIEnv *env, jclass clazz, jstring name)
{
  char value[100];

  const char *key = env->GetStringUTFChars(name, 0);

  if( strcmp(key, "TTBeforeOperation") == 0 ) {
 	long timeout = CLEPropWorkSpace::GetTimeoutBeforeOperation();
	sprintf(value, "%d", timeout);
  }

  jstring jvalue = env->NewStringUTF(value);

  env->ReleaseStringUTFChars(name, key);

  return jvalue;
}


/*****************************************************************
* Class CWLJavaJniManager
*****************************************************************/
// #BEGIN:

//////////////////////////////////////////////////////////////////////
// Static variables.
//////////////////////////////////////////////////////////////////////

CWLJavaJniManager* CWLJavaJniManager::msSingleton = NULL;
static CWLJavaJniManager singletonCWLJavaJniManager;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWLJavaJniManager& CWLJavaJniManager::fsGetSingleton()
{
	WlAssert( msSingleton );
	return *msSingleton;
}

CWLJavaJniManager::CWLJavaJniManager() :
	m_Initialized(false)
{
	WlAssert( !msSingleton );
	msSingleton = this;
}

//////////////////////////////////////////////////////////////////////
// Init/Terminate
//////////////////////////////////////////////////////////////////////
bool CWLJavaJniManager::Init(JNIEnv* env)
{
  if (m_Initialized)
    return true;

	{
		jclass rvPropertiesClass = env->FindClass("com/radview/system/RVProperties");
		WlAssert(rvPropertiesClass);
		jmethodID setTVTOPProperty = env->GetStaticMethodID(rvPropertiesClass, "setTVTOPProperty", "(Ljava/lang/String;)V");
		WlAssert(setTVTOPProperty);

		if( setTVTOPProperty )
		{
			TVEnvContainerClass tvpc;
			RVCString tvTop(tvpc.TVEGetEnv("TV_TOP"));

			jstring jsTVTop = env->NewStringUTF(tvTop.data());

			env->CallStaticObjectMethod(rvPropertiesClass, setTVTOPProperty, jsTVTop);

			env->DeleteLocalRef(jsTVTop);
		}
	}

  //
  // Get the com.radview.loadengine.appletloader.system.JarResources.setstream function.
  m_jarResourcesClass = env->FindClass("com/radview/loader/JarResources");
  WlAssert(m_jarResourcesClass);
  m_midGetSystemJarResources = env->GetStaticMethodID(m_jarResourcesClass, "getSystemJarResources", "()Lcom/radview/loader/JarResources;");
  WlAssert(m_midGetSystemJarResources);

  m_jarResourcesObject = env->CallStaticObjectMethod(m_jarResourcesClass, m_midGetSystemJarResources);
  WlAssert(m_jarResourcesObject);
  m_midSetStream = env->GetMethodID(m_jarResourcesClass, "setStream", "(Ljava/lang/String;Ljava/lang/String;[B)V");
  WlAssert(m_midSetStream);
  m_midIsResourceFullnameExists = env->GetMethodID(m_jarResourcesClass, "isResourceFullnameExists", "(Ljava/lang/String;)Z");
  WlAssert(m_midIsResourceFullnameExists);

  //
  // AppletHandler class.
  m_appletHandlerClass = env->FindClass("com/radview/appletloader/WLAppletHandler");

  if(!m_appletHandlerClass)
	{
    jthrowable exc = env->ExceptionOccurred();

    if (exc) 
		{
			RVCString excString;
			fsGetStackTrace(env, exc, excString);
      env->ExceptionDescribe();
			WlErrorMsg("Error in CWLJavaJniManager::Init %s", excString);
    }

    WlAssert(0);
		return false;
  }

  m_midCreateApplet= env->GetStaticMethodID(m_appletHandlerClass, "createApplet", "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;ILjava/lang/String;)Ljava/applet/Applet;");
  WlAssert(m_midCreateApplet);
  m_midCreateAndRunApplet = env->GetStaticMethodID(m_appletHandlerClass, "createAndRunApplet", "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;ILjava/lang/String;)Ljava/applet/Applet;");
  WlAssert(m_midCreateAndRunApplet);
  m_midInitApplet= env->GetStaticMethodID(m_appletHandlerClass, "initApplet", "(ILjava/applet/Applet;)V");
  WlAssert(m_midInitApplet);
  m_midStartApplet= env->GetStaticMethodID(m_appletHandlerClass, "startApplet", "(ILjava/applet/Applet;)V");
  WlAssert(m_midStartApplet);
  m_midStopApplet= env->GetStaticMethodID(m_appletHandlerClass, "stopApplet", "(ILjava/applet/Applet;)V");
  WlAssert(m_midStopApplet);
  //m_midSetAuthUsrAndPwd = env->GetStaticMethodID(m_appletHandlerClass, "setAuthenticationUsernameAndPassword", "(Ljava/lang/String;Ljava/lang/String;)V");
  //WlAssert(m_midSetAuthUsrAndPwd);
	m_midSetAuthUsrAndPwd = NULL;


  //
  // AppletHandler class.
  m_RVThrowableClass = env->FindClass("com/radview/util/rvexception/RVThrowable");
  if (m_RVThrowableClass == NULL) 
	{
    return false;
  }


  m_midStackTraceToString= env->GetStaticMethodID(m_RVThrowableClass, "stackTraceToString", "(Ljava/lang/Throwable;)Ljava/lang/String;");
  WlAssert(m_midStackTraceToString);


  m_Initialized = true;

  return true;
}

void CWLJavaJniManager::Terminate(JNIEnv* env)
{
  env->DeleteLocalRef(m_jarResourcesClass);
  env->DeleteLocalRef(m_jarResourcesObject);
  env->DeleteLocalRef(m_appletHandlerClass);
  env->DeleteLocalRef(m_RVThrowableClass);

  m_Initialized = false;
}


void CWLJavaJniManager::fsGetStackTrace(JNIEnv* env, jthrowable e, RVCString& s)
{
  jstring ret = (jstring)env->CallStaticObjectMethod(m_RVThrowableClass, m_midStackTraceToString, e);
  const char* str = env->GetStringUTFChars(ret,0);
  s = str;
  env->ReleaseStringUTFChars(ret, str);
}

bool CWLJavaJniManager::fsHandleJVMException(JNIEnv* env, JSContext* cx, int frmErrorNum)
{
	jthrowable exc = env->ExceptionOccurred();

	if(exc) 
	{
		RVCString excString;
		fsGetStackTrace(env, exc, excString);
		WLErrorException e(LoadExceptionFrmFile, frmErrorNum, excString.data());
		LoadReport(cx, WLLogMessage::Error, e.GetErrorMsg());
		env->ExceptionClear();

		return true;
	}

	return false;
}

bool CWLJavaJniManager::fsHandleJVMException(JNIEnv* env, JSContext* cx, const char* className, int frmErrorNum)
{
	jthrowable exc = env->ExceptionOccurred();

	if(exc) 
	{
		RVCString excString;
		fsGetStackTrace(env, exc, excString);
		WLErrorException e(LoadExceptionFrmFile, frmErrorNum, className, excString.data());
		LoadReport(cx, WLLogMessage::Error, e.GetErrorMsg());
		env->ExceptionClear();

		return true;
	}

	return false;
}

//////////////////////////////////////////////////////////////////////
// AddStream
//////////////////////////////////////////////////////////////////////
bool CWLJavaJniManager::fsAddStream(/*[IN]*/JSContext*        cx,
                                    /*[IN]*/const char*       streamName,
                                    /*[IN]*/const char*       codeBase,
									/*[IN]*/const char*       javaStream,
                                    /*[IN]*/size_t            streamLen)

{
  //
  // Get the JNI environment.
  iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet( cx );
  JNIEnv* env = aEmbedContext->fGetMachineThread().fGetJEnv();

  WlAssert(m_Initialized);
  //
  // prepare the parameters.
  jstring jStreamName = env->NewStringUTF(streamName);
  jstring jCodeBase = env->NewStringUTF(codeBase);
  jbyteArray jStream = env->NewByteArray(streamLen);

  env->SetByteArrayRegion(jStream, 0 , streamLen, (jbyte*)javaStream);
  //
  // Call the setStream function.
  env->CallVoidMethod(m_jarResourcesObject, m_midSetStream, jStreamName, jCodeBase, jStream);

	fsHandleJVMException(env, cx, streamName, 405);

  //
  // delete variable references
  env->DeleteLocalRef(jStreamName);
  env->DeleteLocalRef(jCodeBase);
  env->DeleteLocalRef(jStream);

	return true;
}

bool CWLJavaJniManager::fsClassAppletExists( JSContext*     cx,
                                             const char*    appletClassName,
                                             bool&          classAppletExists)
{
  bool rtn = false;
  //
  // Get the JNI environment.
  iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet( cx );
  JNIEnv* env = aEmbedContext->fGetMachineThread().fGetJEnv();

  //
  // prepare the parameter.
  jstring jClassName = env->NewStringUTF(appletClassName);
  //
  // Call the setStream function.
  jboolean exists = env->CallBooleanMethod(m_jarResourcesObject, m_midIsResourceFullnameExists, jClassName);
  classAppletExists = (exists != 0);
  //
  // delete variable references
  env->DeleteLocalRef(jClassName);

  rtn = true;

  return rtn;
}

CWLJSJavaApplet* CWLJavaJniManager::fsCreateJavaApplet( JSContext*                cx,
													   CLoadGeneralThreadInstanceScript* TIS,
                                                        CWLJSJavaApplet*&         applet,
													                              const char*               appletClassName,
													                              const char*               appletCodeBase,
                                                        const char*               appletDocumentBase,
                                                        RVHashDictionary*         appletParams)
{
  WlAssert(m_Initialized);

  CWLJSJavaApplet* rtn = NULL;

  //
  // Get the JNI environment.
  iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet( cx );
  JNIEnv* env = aEmbedContext->fGetMachineThread().fGetJEnv();

	try 
	{	
  //
  // prepare the parameter.
  jstring jClassName = env->NewStringUTF(appletClassName);
  jstring jCodeBase = env->NewStringUTF(appletCodeBase);
  jstring jDocumentBase = env->NewStringUTF(appletDocumentBase);

  RVCString paramsArray;
  RVHashDictionaryIterator iter(*appletParams);

		while(iter()) 
		{
    paramsArray.append( *((RVCollectableString*)iter.key()) );
    paramsArray.append("\001");

    const char* value = *((RVCollectableString*)iter.value());
    if (strlen(value)==0)
				value = "null";

    paramsArray.append(value);
    paramsArray.append("\001");
  }

		CPointerToJava ptj(reinterpret_cast<void*> (TIS));

  jstring jParams = env->NewStringUTF(paramsArray.data());
		jstring jThreadData = env->NewStringUTF(ptj.getString());

  jint    jThreadNum = TIS->m_MyThreadInstance->GetLGThreadNum();

  //
  // Call the setStream function.
  jobject jApplet = env->CallStaticObjectMethod(m_appletHandlerClass,
																									m_midCreateAndRunApplet,
                                                jClassName,
                                                jCodeBase,
                                                jDocumentBase,
                                                jParams,
                                                jThreadNum,
                                                jThreadData);

		if( !TIS->isAbort() && !fsHandleJVMException(env, cx, appletClassName, 400) ) 
  {
    jsval jsApplet;
    CJSGCRequestBeginEnd GCGuard( cx );

			if (JSJ_ConvertJavaObjectToJSValue(cx, jApplet, &jsApplet)) 
			{
				if(!JSVAL_IS_NULL(jsApplet) && JSVAL_IS_OBJECT(jsApplet)) 
				{
					applet->Init(cx, jsApplet, jApplet, jThreadNum);
					rtn = applet;
      }
 	  }
  }

  //
  // delete variable references
  env->DeleteLocalRef(jClassName);
  env->DeleteLocalRef(jCodeBase);
  env->DeleteLocalRef(jDocumentBase);
  env->DeleteLocalRef(jParams);
  env->DeleteLocalRef(jThreadData);

		//
		// Eyal(25.01.04): for now this is just overhead (performance & memory)
		/*
		//
		// initApplet
  if (!rtn)
    return NULL;

  env->CallStaticObjectMethod( m_appletHandlerClass, m_midInitApplet, jThreadNum, jApplet);

		if(!TIS->isAbort() && fsHandleJVMException(env, cx, appletClassName, 401)) 
		{
    return NULL;
  }

  //
  // startApplet
  env->CallStaticObjectMethod( m_appletHandlerClass, m_midStartApplet, jThreadNum, jApplet);

		if(!TIS->isAbort() && fsHandleJVMException(env, cx, appletClassName, 402)) 
		{
			return NULL;
		}
		*/
	}
	catch(...)
	{
		if (!TIS->isAbort()) 
		{
			fsHandleJVMException(env, cx, appletClassName, 400);
		}
  }

  return rtn;
}


bool CWLJavaJniManager::fsStopJavaApplet(JSContext* cx, jobject applet, jint threadNum)
{
  WlAssert(m_Initialized);

	bool       retVal = true;
	
  //
  // Get the JNI environment.
  iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet( cx );
  JNIEnv* env = aEmbedContext->fGetMachineThread().fGetJEnv();

	try
	{
  //
  // Call the setStream function.
  env->CallStaticObjectMethod( m_appletHandlerClass, m_midStopApplet, threadNum, applet);
	}
	catch(...)
	{
		retVal = false;
	}
	
  jthrowable exc = env->ExceptionOccurred();

	if( exc ) 
	{
		CLoadGeneralThreadInstanceScript* pTIS = getCurrentThreadInstanceScript(cx);
		
		if( !pTIS->isAbort() ) 
		{
      RVCString excString;
      fsGetStackTrace(env, exc, excString);
      WLErrorException e(LoadExceptionFrmFile, 403, excString.data());
      LoadReport(cx, WLLogMessage::Error, e.GetErrorMsg());
      env->ExceptionClear();
		}
    retVal = false;
  }

  return retVal;
}

// #END.

/*****************************************************************
* Class CWLJSJavaApplet
*****************************************************************/
// #BEGIN:

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWLJSJavaApplet::CWLJSJavaApplet()
:m_Applet(NULL), m_JavaApplet(NULL), m_ThreadNum(NULL), m_cx(NULL), readyState(NULL)
{
}

CWLJSJavaApplet::CWLJSJavaApplet(JSContext* cx, jsval& applet, jobject jApplet, jint threadNum)
  :m_Applet(applet), m_JavaApplet(jApplet), m_ThreadNum(threadNum), m_cx(cx)
{
	CJSGCRequestBeginEnd guard(cx);
  JS_AddNamedRoot(cx, (void*)&m_Applet, "CWLJSJavaApplet::ctor");
}

CWLJSJavaApplet::~CWLJSJavaApplet()
{
	if( m_JavaApplet && m_cx )
	{
		StopApplet(m_cx);
	}
}

bool CWLJSJavaApplet::Init(JSContext* cx, jsval& applet, jobject jApplet, jint threadNum)
{
	if(m_cx)
	{
		return false;
	}

	{
		m_Applet = applet;
		
		CJSGCRequestBeginEnd guard(cx);
		JS_AddNamedRoot(cx, (void*)&m_Applet, "CWLJSJavaApplet::Init");
	}
	
	m_cx         = cx;
	m_JavaApplet = jApplet;
	m_ThreadNum  = threadNum;
	readyState   = 4;

	return true;
}

bool CWLJSJavaApplet::StopApplet(JSContext* cx)
{
  if (cx == NULL) cx = m_cx;

	bool ok = true;

	if( m_Applet)
	{
		CJSGCRequestBeginEnd guard(cx);
  JS_RemoveRoot(m_cx, (void*)&m_Applet);
	}

	if( m_JavaApplet )
	{
		ok = CWLJavaJniManager::fsGetSingleton().fsStopJavaApplet(cx, m_JavaApplet, m_ThreadNum);

  iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet( cx );
  JNIEnv* env = aEmbedContext->fGetMachineThread().fGetJEnv();

	env->DeleteLocalRef(m_JavaApplet);
	m_JavaApplet = NULL;
	}

  return ok;
}

// #END.


/*****************************************************************
* Class CWAppletFactory
*****************************************************************/
// #BEGIN:

///////////////////////////////////////////////////////////////////////
// Construction/Destruction
///////////////////////////////////////////////////////////////////////
/*** None - all methods are static and there are no static members ***/

//////////////////////////////////////////////////////////////////////
// Methods implementation
//////////////////////////////////////////////////////////////////////

//
//
// @@Yahali - convert to JNI and use
//  jsj_ConvertJavaObjectToJSValue or jsj_WrapJavaObject
//
CWLJSJavaApplet* CWLAppletFactory::CreateJavaApplet(JSContext*                cx,
                                                    CLoadGeneralThreadInstanceScript*
                                                                              TIS,
                                                    CWLJSJavaApplet*&         applet,
													                          const char*               appletClassName,
													                          const char*               appletCodeBase,
                                                    const char*               appletDocumentBase,
                                                    RVHashDictionary*         appletParams)
{
  return CWLJavaJniManager::fsGetSingleton().fsCreateJavaApplet(cx,
                                                                TIS,
                                                                applet,
                                                                appletClassName,
                                                                appletCodeBase,
                                                                appletDocumentBase,
                                                                appletParams);
}

bool CWLAppletFactory::ClassAppletExists(JSContext*   cx,
                                         const char* appletClassName,
                                         bool& classAppletExists)
{
  return CWLJavaJniManager::fsGetSingleton().fsClassAppletExists(cx,
                                                                 appletClassName,
                                                                 classAppletExists);
}

// #END.


/*****************************************************************
* Class CWLJavaStreamTable
*****************************************************************/
// #BEGIN:

///////////////////////////////////////////////////////////////////////
// Construction/Destruction
///////////////////////////////////////////////////////////////////////
/*** None - all methods are static and there are no static members ***/

//////////////////////////////////////////////////////////////////////
// Methods implementation
//////////////////////////////////////////////////////////////////////

bool CWLJavaStreamTable::AddJavaStream( /*[IN]*/JSContext*  cx,
                                        /*[IN]*/const char* streamName,
                                        /*[IN]*/const char* codeBase,
									                      /*[IN]*/const char* javaStream,
                                        /*[IN]*/size_t      streamLen)

{
  //
  // If the stream already exists, return
  //{
		//RVReadersWriterLock::ReadLockGuard lock(m_InsertionLock);
    if ( JavaStreamExists(streamName) )
      return true;
  //}

  //
  // Only one at a time can pass the following line.
	RVReadersWriterLock::WriteLockGuard lock(m_InsertionLock);
  //
  // We still have to make sure that no one else already entered the stream.
  if ( JavaStreamExists(streamName) )
    return true;

  bool ret = false;

  RVCString streamNameTmp(streamName);
  if(streamNameTmp.contains(".cab",RVCString::ignoreCase))
	{
		//
		// Eyal(28.01.04) - currently not supported, causes exceptions
    //ret = (HandleCabinet((char*)streamName, codeBase, cx, javaStream, streamLen));

		WLErrorException e(LoadExceptionFrmFile, 406, streamName);
		LoadReport(cx, WLLogMessage::Error, e.GetErrorMsg());
  }
  else
	{
    ret = (CWLJavaJniManager::fsGetSingleton().fsAddStream(cx, streamName, codeBase, javaStream, streamLen));
  }

  if (ret)
	{
    insert(new RVCollectableString(streamName));
	}

	return ret;
}

void CWLJavaStreamTable::RemoveJavaStream(/*[IN]*/const char* streamName)
{
	RVCollectableString key(streamName);

	removeAndDestroy(&key);
}

inline RVCollectableString* CWLJavaStreamTable::FindJavaStream(const char* streamName)
{
	RVCollectableString key(streamName);

	return (RVCollectableString*)find(&key);
}

bool CWLJavaStreamTable::JavaStreamExists(const char* streamName)
{
	RVCollectableString key(streamName);
	
	return ( (RVCollectableString*)find(&key) != NULL );
}

bool CWLJavaStreamTable::HandleCabinet(char* streamName,const char* codeBase,JSContext* cx,const char* javaStream,size_t streamLen)
{
#ifdef __TV_WINDOWS__

  FILE*   pFile;

  const char* filename;
	char fullPath[WL_MAX_PATH];

  CWLCabCalbackParam params(cx, codeBase);

  strcpy(fullPath, CArrangeMachineTempFiles::GetWlTempFullPath(cx, WLECopiedFile, streamName).data());
	filename = fullPath;

  if((pFile = fopen(filename, "wb"))!=NULL){
    fwrite(javaStream, sizeof(char), streamLen, pFile );
		fflush(pFile);
    fclose(pFile);
    if (SetupIterateCabinet(filename,0, (PSP_FILE_CALLBACK)CabinetCallback,(void*)&params))
      return true;
  }
#endif // __TV_WINDOWS__
  return false;
}


#ifdef __TV_WINDOWS__
LRESULT WINAPI CabinetCallback (IN PVOID params,
                                IN UINT Notification,
                                IN UINT Param1,
                                IN UINT Param2 )
{
  LRESULT lRetVal = NO_ERROR;
  FILE_IN_CABINET_INFO *pInfo = NULL;
  FILEPATHS *pFilePaths = NULL;

  JSContext* aCx = ((CWLCabCalbackParam*)params)->m_cx;

  switch(Notification)
  {
     case SPFILENOTIFY_FILEINCABINET:

        const char* filename;
	      char fullPath[WL_MAX_PATH];

        pInfo = (FILE_IN_CABINET_INFO *)Param1;

	      strcpy(fullPath, CArrangeMachineTempFiles::GetWlTempFullPath(aCx ,
                                      WLECopiedFile, pInfo->NameInCabinet).data());
	      filename = fullPath;
        lstrcpy(pInfo->FullTargetName, filename);
        lRetVal = FILEOP_DOIT;  // Extract the file.

        break;

     case SPFILENOTIFY_FILEEXTRACTED:
       {

          pFilePaths = (FILEPATHS *)Param1;

          FILE*   pFile;
          RVCString streamName(pFilePaths->Target);

          size_t pos = streamName.last( '\\' );
	        if ( pos != RV_NPOS )
	          streamName = streamName( pos+1, streamName.length());

          if((pFile = fopen(pFilePaths->Target, "rb"))!=NULL){
			      char ch;
            RVCString javaStream;
			      bool more(true);
            bool ok(true);
            size_t streamLen=0;

            do {
				      if( fread( &ch, 1, 1, pFile ) ) {
					      javaStream += ch;
                streamLen++;
              }else{
                if(feof( pFile ))//It returns 0 if the current position is not end of file.
						      ok = true;
					      else
						      ok = false;
					      more = false;
              }
			      } while( more );
            if(ok){
              fclose(pFile);
              CWLJavaJniManager::fsGetSingleton().fsAddStream(((CWLCabCalbackParam*)params)->m_cx,streamName, ((CWLCabCalbackParam*)params)->m_codeBase, javaStream,streamLen);
            }
        }

        lRetVal = NO_ERROR;
       }
        break;

     case SPFILENOTIFY_NEEDNEWCABINET: // Unexpected.
        lRetVal = NO_ERROR;
        break;
  }

  return lRetVal;
}


CWLCabCalbackParam::CWLCabCalbackParam(JSContext* cx, const char* m_codeBase)
  :m_cx(cx), m_codeBase(m_codeBase)
{}

#endif // __TV_WINDOWS__
// #END.

