/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "wljsobjects.h"

#include "tracedll.h"
#include "rvjsapi.h"

#include "tvpsync.h"
#include "loadenginevariablesdata.h"
#include "loadenginedecs.h"
#include "loadengineglobals.h"
#include "rvjsembed.h"
#include "wljsruntime.h"
#include "loadengineinifileproperties.h"

/** local definitions **/
// define a class that will contain all other objects in the appropriate scope
static JSClass stub_class = {
	"stub_class", 
  0,
	JS_PropertyStub,	// add
	JS_PropertyStub,	// del
	JS_PropertyStub,	// get
	JS_PropertyStub,	// set
	JS_EnumerateStub,
	JS_ResolveStub,
	JS_ConvertStub,
	JS_FinalizeStub
};

JSBool
proto_resolve_stub(JSContext *cx, JSObject *obj, jsval id) {
  jsval v;
  JSObject *func;
  const char* name;
  JSBool ok;

  if (!JSVAL_IS_STRING(id)) return JS_FALSE;
  name = JS_GetStringBytes(JSVAL_TO_STRING(id));

  if (!name) return JS_FALSE;
  
  JSObject* proto = JS_GetPrototype(cx,obj);
  if (!proto) return JS_FALSE;

  ok = JS_LookupProperty(cx,proto,name,&v);
  if (!ok) return JS_FALSE; // @@Chaim (inherited from Itaj)

  if (!JSVAL_IS_VOID(v)) {
    if (JS_TypeOfValue(cx,v) == JSTYPE_FUNCTION) {

      func = JS_CloneFunctionObject(cx, JSVAL_TO_OBJECT(v), obj);
      if (!func) return JS_FALSE;

      ok = JS_DefineProperty(cx,obj, name ,OBJECT_TO_JSVAL(func), NULL, NULL, JSPROP_ENUMERATE | JSPROP_PERMANENT);
      if (!ok) return JS_FALSE;
    }
  }

  return JS_TRUE;
}

static JSClass proto_stub_class = {
	"proto_stub_class", 
  JSCLASS_NEW_RESOLVE | JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,	// add
	JS_PropertyStub,	// del
	JS_PropertyStub,	// get
	JS_PropertyStub,	// set
	JS_EnumerateStub,
  proto_resolve_stub,
	JS_ConvertStub,
	JS_FinalizeStub
};


cLEJSGCThread* fsGetJSGCThread( JSContext* cx )
{
  iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet( cx );
	if (!aEmbedContext){
		return NULL;
	}

	iSession* aSession = &(aEmbedContext->fGetSession());
	WlAssert(dynamic_cast<CLoadGeneratorSlave*>(aSession));
	CLoadGeneratorSlave* aSlave = static_cast<CLoadGeneratorSlave*>(aSession);
	return &(aSlave->GetJSGCThread());
}

iLEJSMachineThread* fsGetMachineThread( JSContext* cx )
{
  iLEJSEmbedContext* aEmbedContext = iLEJSEmbedContext::fsGet( cx );
	if (!aEmbedContext){
		return NULL;
	}

	return &(aEmbedContext->fGetMachineThread());
}


//class CJSLoadLoadScope
CJSLoadScope::CJSLoadScope(CJSLoadRuntime& runtime, JSContext* guardcx, UINT contextSize, JSObject* proto)
{

	JSContext* aContext;
  if( m_GuardContext = guardcx ) {
		aContext = RVJS_NewContext(runtime.fGet(), contextSize);
	} else {
		aContext = RVJS_NewContext(runtime.fGet(), contextSize);
	}
	m_Context = aContext;

	if( aContext ) {
    CJSGCRequestBeginEnd GCGuard(aContext);
		iJSEmbedContext::fsUnsignContext( aContext );

		JSClass* aJSClass = proto ? &proto_stub_class : &stub_class;
		JSObject* aJSObject = JS_NewObject(GetContext(), aJSClass, NULL, NULL);
		m_Object = aJSObject;
    JS_SetPrototype(GetContext(),m_Object, proto );
    JS_SetParent(GetContext(),m_Object,NULL);
					
		JS_AddNamedRoot( aContext, &m_Object, "CJSLoadScope::ctor m_Object" );
	}	
	
/*
  m_GuardContext = guardcx;
  if (m_GuardContext == NULL) {
	  m_Context = RVJS_NewContext(runtime.fGet(), contextSize);
		iJSEmbedContext::fsUnsignContext( m_Context ); 
    if (m_Context) {
      if (proto == NULL) {
        m_Object = JS_NewObject(GetContext(), &stub_class, NULL, NULL);
        JS_SetPrototype(GetContext(),m_Object,NULL);
        JS_SetParent(GetContext(),m_Object,NULL);
      } else {
        m_Object = JS_NewObject(GetContext(), &proto_stub_class, NULL, NULL);
        JS_SetPrototype(GetContext(),m_Object,proto);
        JS_SetParent(GetContext(),m_Object,NULL);
      }
    }
  } else {
		{
    CJSGCRequestBeginEnd GCGuard(m_GuardContext);
	  m_Context = RVJS_NewContext(runtime.fGet(), contextSize);
		iJSEmbedContext::fsUnsignContext( m_Context );
  }
    if (m_Context) {
	    CJSGCRequestBeginEnd GCGuard(m_Context);
      if (proto == NULL) {
        m_Object = JS_NewObject(GetContext(), &stub_class, NULL, NULL);
        JS_SetPrototype(GetContext(),m_Object,NULL);
        JS_SetParent(GetContext(),m_Object,NULL);
      } else {
        m_Object = JS_NewObject(GetContext(), &proto_stub_class, NULL, NULL);
        JS_SetPrototype(GetContext(),m_Object,proto);
        JS_SetParent(GetContext(),m_Object,NULL);
      }
    }
  }
	JS_AddNamedRoot( m_Context, &m_Object, "CJSLoadScope::ctor m_Object" );

*/	
	

}

CJSLoadScope::~CJSLoadScope()
{
/*
  if (m_Context) {
    if (m_GuardContext == NULL)
      JS_SetContextThread(m_Context);
    else {
      CJSGCRequestBeginEnd GCGuard(m_GuardContext);
      JS_SetContextThread(m_Context);
    }
  }
*/
  try {
		{
		  CJSGCRequestBeginEnd GCGuard( m_Context );
    JS_SetContextThread(m_Context);
    JS_RemoveRoot( m_Context, &m_Object );
		}

    // Chaim - All contexts are destoryed w/o GC except for the final one (LoadRuntime's context)
    JS_DestroyContext(m_Context);
  } catch (...) {
		WlAssert(0); // if we are here then we exception cut JS_*() in the middle.
	}
}

bool CJSLoadScope::IsMain(){
  return (JS_GetPrototype(m_Context, m_Object) == NULL);
}


//class CJSGCRequestBeginEnd
CJSGCRequestBeginEnd::CJSGCRequestBeginEnd(JSContext* cx, JSGCGuardType guardType) :
  m_Context(cx),
  m_EndDone(false),
	m_GCLockDone(false)
{

	if (guardType == UseDefault){
		m_NewStyle = CLEIniFileProperties::UseNewJSGCGuardStyle;
	}
	else {
		m_NewStyle = (guardType == UseNewStyle);
	}

	if (m_NewStyle)
        {
		iLEJSMachineThread* aMachineThread = fsGetMachineThread(m_Context);

		if (aMachineThread)
                {
			if (aMachineThread->mGCLockDepth == 0)
{
			cLEJSGCThread* aJSGCThread = fsGetJSGCThread(m_Context);
        WlAssert(aJSGCThread);

				aJSGCThread->mJSGCLock.acquireRead();
			}

			++aMachineThread->mGCLockDepth;

			m_GCLockDone = true;
		}
	}
  JS_BeginRequest(m_Context);
}

CJSGCRequestBeginEnd::~CJSGCRequestBeginEnd()
{
  if (!m_EndDone) {
		RequestEnd();
}
}

void CJSGCRequestBeginEnd::RequestEnd()
{
  JS_EndRequest(m_Context);

	if (m_NewStyle)
  {
		iLEJSMachineThread* aMachineThread = fsGetMachineThread(m_Context);
		if (aMachineThread)
    {
			if (!m_GCLockDone) return;

			WlAssert(aMachineThread->mGCLockDepth);
			
			--aMachineThread->mGCLockDepth;

			if (aMachineThread->mGCLockDepth == 0)
      {
  			cLEJSGCThread* aJSGCThread = fsGetJSGCThread(m_Context);
        WlAssert(aJSGCThread);

				aJSGCThread->mJSGCLock.release();
			}
		}
	}
  m_EndDone = true;
}

//class CJSGCRequestSuspendResume
CJSGCRequestSuspendResume::CJSGCRequestSuspendResume(JSContext* cx, JSGCGuardType guardType) :
  m_Context(cx),
  m_ResumeDone(false),
	m_GCUnlockDone(false),
	m_GCLockDepthSave(0)
{
  //JS_EndRequest(m_Context);

	if (guardType == UseDefault){
		m_NewStyle = CLEIniFileProperties::UseNewJSGCGuardStyle;
	}
	else {
		m_NewStyle = (guardType == UseNewStyle);
	}

	m_DepthSave = JS_SuspendRequest( m_Context );
	if (m_NewStyle){
		iLEJSMachineThread* aMachineThread = fsGetMachineThread(m_Context);
		if (aMachineThread){
			WlAssert(fsGetJSGCThread(m_Context));
			cLEJSGCThread* aJSGCThread = fsGetJSGCThread(m_Context);
			long aGCLockDepth = aMachineThread->mGCLockDepth;
			if (aGCLockDepth > 0){
				aJSGCThread->mJSGCLock.release();
			}
			m_GCLockDepthSave = aGCLockDepth;
			aMachineThread->mGCLockDepth = 0;
			m_GCUnlockDone = true;
		}
	}
}
CJSGCRequestSuspendResume::~CJSGCRequestSuspendResume()
{
  if (!m_ResumeDone) {
		RequestResume();
	}
}
void CJSGCRequestSuspendResume::RequestResume()
{
  //JS_BeginRequest(m_Context);
	if (m_NewStyle){
		iLEJSMachineThread* aMachineThread = fsGetMachineThread(m_Context);
		if (aMachineThread){
			WlAssert(fsGetJSGCThread(m_Context));
			cLEJSGCThread* aJSGCThread = fsGetJSGCThread(m_Context);
			if (!m_GCUnlockDone) return;
			if (m_GCLockDepthSave > 0){
				aJSGCThread->mJSGCLock.acquireRead();
			}
			aMachineThread->mGCLockDepth = m_GCLockDepthSave;
		}
	}
	JS_ResumeRequest( m_Context, m_DepthSave );
  m_ResumeDone = true;
}

void YieldRequest(JSContext* cx) {

//	JS_YieldRequest(cx);
// There is a bug in JS_YieldRequest() -
/*  when inside such:
  {
    JS_BeginRequest()
    {
      JS_SuspentRequest()
      {
        JS_BeginRequest()
        
        JS_YieldRequest() 
        // after the yield the GC is not locked, as if the yield just suspended.
        ...
 
  */
  //so instead we do:
  CJSGCRequestSuspendResume tempGuard( cx );
}
