/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//@@Merged
#include "pcui.h"
#include "njsdxml.h"
#include "loadreportinfo.h"

namespace nPCUi {

// using namespace nJsd;

cScriptFile::cScriptFile(
	const char* Name,
	iPCUi& PCUi )
:
	mName(Name),
	mUi(PCUi)
{}

cScriptFile::~cScriptFile() {}

const char* cScriptFile::fGetName() const {
	return mName.data();
}

void cScriptFile::fSourceExport( 
	iEmbedderCallContext& EC,
	iSourceCommit& Sourcer ) 
{

	std::string Source;
	std::ifstream FileInput( mName.data() );
	
	char ch;
	while( FileInput.get(ch) ) {
		Source += ch;
}


	std::string Message("");
	if( Source.length() > 0 ) {

#if doBreakpoints
		printf( "File %s breakpoints:\n", (const char*)mName.data() );

		iPCUi::cInputElement Str;
		int IntX0;
		do {		
			mUi.fInputPull( Str );
			sscanf( Str.data(), "%d", &IntX0 );

			if( IntX0 ) {
				mBreakpointList.push_back( IntX0 );
			}
		} while( IntX0 );

#endif

		Sourcer.fTake( Source.length(), Source.data() );
	}	else {
		Message += "bad ";
		Message += "iAgendaFile: " + mName + "\n";
		EC.fReport( cLoadReportInfoAgenda( WLLogMessage::Fatal, Message.data() ) );
	}

}	



std::string iGeneralUi::fsReportFormat(
	WLLogMessage::Severity Severity, 
	const char* Message ) {

	std::string Rtn;

	switch( Severity ) {
	case WLLogMessage::Note:
		Rtn += "N";
		break;
	case WLLogMessage::Warning:
		Rtn += "W";
		break;
	case WLLogMessage::Error:
		Rtn += "E";
		break;
	case WLLogMessage::Fatal:
		Rtn += "F";
		break;
	default:
		WlAssert(0); // bad switch
		break;
		}
	Rtn += " ";
	Rtn += Message;

	return Rtn;
	}








iPCUi::iPCUi(
	cSingleClientProcess& rProcess)
:
	mStepping(sSteppingLine),
	iLoadProbingClient( rProcess, dIsDebugger ),
	mLastRound(0)
{}

iPCUi::~iPCUi() {
	//fFinal(); // this happens in the wrong thread.
}


void iPCUi::fSetLastRound(
	long int Val )
{
	mLastRound = Val;
}


iAgendaFile* iPCUi::fGetFile(
	bool& IsNew, 
	iEmbedderCallContext& EC,
	const char* Name ) 
{
	cScriptFile* FileP = new cScriptFile( Name, *this );
	WlAssert( FileP );
	cScriptFile& File = *FileP;

	mFiles.push_back( std::auto_ptr<cScriptFile>(&File) );
	IsNew = true;
	return &File;
}


void iPCUi::fCopyFile( 
	iEmbedderCallContext& EC, 
	const char* Source, 
	const char* Target) {
	
	if( Source ) {
		printf( "CopyFile( %s, %s )\n", Source, Target );
		}
	else {
		EC.fReport( cLoadReportInfoAgenda( WLLogMessage::Warning, "no source file" ) );
		}
	}


void iPCUi::fScriptHook( 
	iScriptHookable& Script,
	bool Creating ) 
{

	cScriptFile* FileP = dynamic_cast<cScriptFile*>( Script.fGetFile() );

	tLineNumber BaseLine( Script.fGetBaseLine() );
	tLineNumber LineExtent( Script.fGetLineExtent() ); 

#if doScriptHook
	printf( "%s script %s[ %s ] %d-%d> ", 
		(Creating ? "begin" : "end" ),
		( FileP ? FileP->fGetName() : Script.fGetName() ),
		Script.fGetFunctionName(), (int)BaseLine, (int)(BaseLine+LineExtent) );

	if( FileP ) {
		
		cScriptFile& File = *FileP;

		std::list<tLineNumber>::iterator iter;
		printf( " breakpoints" );
		for( iter = File.mBreakpointList.begin(); iter != File.mBreakpointList.end(); ++iter ) {
			tLineNumber Line(*iter);
			tLineNumber ActualLine = fEnableBreakPoint( Script, Line );
			printf( ", " );
			if( Line==ActualLine ) {
				printf( "[%d]", (int)Line );
			} else {
				if( ActualLine ) {
					printf( "%d:[%d]", (int)Line, (int)ActualLine );
				} else {
					printf( "/%d/", (int)Line );
				}
			}
		}
	}
	printf( "\n" );
#endif

}




cPCUiMT::cPCUiMT(
	cSingleClientProcess& rProcess,
	nMT::cSStm& SStm )
:
	iPCUi(rProcess),
	mSStm(SStm)
{}


void cPCUiMT::vInputPull(
	cInputElement& Str )
{
	mSStm.fPull( Str );
}


cPCUi::cPCUi( 
	cSingleClientProcess& rProcess )
:
	iPCUi( rProcess )
{}


void cPCUi::vInputPull(
	cInputElement& Str )
{
	char Buff[200];
	scanf( "%s", Buff );
	Str = Buff;
}


/*
void iPCUi::fInit()
{
	


	iLoadProbingClient::fInit();
	
}
*/
	
void iPCUi::fJSReporter(
	iReportThreadState& rThreadState,
	iReportInfo& ReportInfo,
	const char* rReporterName ) 
{

	std::string Message;
	iReportThreadState* aThreadState = NULL;

	{
		double aTime = fGetTimeFromBegin();
		char aBufX0[100];
		sprintf( aBufX0, "%f", (float)aTime );
		Message += aBufX0;
		Message += " ";
	}

	if( iLoadReportInfo* NativeInfo = dynamic_cast<iLoadReportInfo*>(&ReportInfo) ) {
		Message += " " + fsReportFormat( NativeInfo->fGetSeverity(), NativeInfo->fGetMessage() );
		if( NativeInfo->fHasTime() ) {
			WlAssert(0);
			Message += " ";
			Message += NativeInfo->fGetTime();
		}
		aThreadState = NativeInfo->fGetifReportThreadState();
	}	else if( cInternalReportInfo* InternalInfo = dynamic_cast<cInternalReportInfo*>(&ReportInfo) ) {
		Message += InternalInfo->fIsWarning() ? 'W' : 'E';
		Message += InternalInfo->fIsException() ? 'X' : '-';
		Message += InternalInfo->fIsStrict() ? 'S' : '-';
		Message += " ";
		Message += InternalInfo->fGetMessage();
		} else {
		WlAssert(0); // unknown report info type;
		}
		
		
	if( !aThreadState ) {
		aThreadState = &rThreadState;
	}
			
	{
		iFile* File = aThreadState->fGetFile();
	printf( "%s rep: %s, %s %d:%d %s\n",
		rReporterName,
		Message.data(),
			(File ? static_cast<cScriptFile*>(File)->fGetName() : aThreadState->fGetName() ),
			aThreadState->fGetLineNumber(),
			aThreadState->fGetColumn(),
			aThreadState->fGetLine()  );
	}

}


void iPCUi::fExecReporter(
	iReportThreadState* rThreadState, 
	iReportInfo& rInfo )
{
	if( rThreadState ) {
		fJSReporter( *rThreadState, rInfo, "JSExec" );
	} else {
		if( iLoadReportInfo* aInfo = dynamic_cast<iLoadReportInfo*>(&rInfo) ) {
			std::string X0 = fsReportFormat( aInfo->fGetSeverity(), aInfo->fGetMessage() );
			printf( "Non JS report: %s\n", X0.data() );
		} else {
			WlAssert(0);
		}
	}
}





tDebugReporterReturn iPCUi::fDebugReporter(
	iReportThreadState& rThreadState,
	iReportInfo& rReportInfo ) 
{
	
	fJSReporter( rThreadState, rReportInfo, "JSD" );


	tDebugReporterReturn Rtn = sDebugReporterReturnApply;

#if doInput
	bool Continue = (false);
	while(Continue) {
		Continue = false;
		cInputElement Str;
		fInputPull( Str );
		if( Str == "deb" ) {
			Rtn = sDebugReporterReturnDM;
			}
		else if( Str == "clr" ) {
			Rtn = sDebugReporterReturnClear;
			}
		else if( Str == "apl" ) {
			Rtn = sDebugReporterReturnApply;
			}
		else if( Str == "con" ) {
			Rtn = sDebugReporterReturnContinue;
			}
		else{
			Continue = true;
			printf( "rep>\n" );
		}
	} 
#endif

	return(Rtn);
}








void iPCUi::fReportDMHook(
	tInterruptionControl& InterruptionControl,
	bool& oIsHookPosition, 
	tStepping rStepping, 
	iDMThreadState& ThreadState ) 
{
	if( !fDMHookX1( InterruptionControl, oIsHookPosition, rStepping, true, "Error", &ThreadState ) ) {
		InterruptionControl = sInterruptionControlAbort;
		}
	}

tNormalDMHookReturn iPCUi::vInterruptDMHook_iLoadProbingClient(
	tInterruptionControl& InterruptionControl,
	bool& oIsHookPosition, 
	tStepping rStepping, 
	iDMThreadState& ThreadState ) 
{
	tNormalDMHookReturn rtn;
	
	if( mStepping <= rStepping ) {
		rtn = fDMHookX2( InterruptionControl, oIsHookPosition, rStepping, rStepping >= mStepping, "Interrupt", &ThreadState );
		oIsHookPosition = true;
	} else {
		rtn = sNormalDMHookReturnContinue;
		oIsHookPosition = false;
	}

	return rtn;
}

tNormalDMHookReturn iPCUi::fBreakPointDMHook(
	tInterruptionControl& InterruptionControl,
	bool& oIsHookPosition, 
	tStepping rStepping, 
	iDMThreadState& ThreadState ) 
{
	return fDMHookX2( InterruptionControl, oIsHookPosition, rStepping, true, "BreakPoint", &ThreadState );
	}

tNormalDMHookReturn iPCUi::fKeywordDMHook(
	tInterruptionControl& InterruptionControl,
	bool& oIsHookPosition, 
	tStepping rStepping, 
	iDMThreadState& ThreadState ) 
{
	return fDMHookX2( InterruptionControl, oIsHookPosition, rStepping, true, "Keyword", &ThreadState );
}

tThrowDMHookReturn iPCUi::fThrowDMHook(
	tInterruptionControl& InterruptionControl,
	bool& oIsHookPosition, 
	tStepping rStepping, 
	iDMThreadState& ThreadState ) 
{
	if( fDMHookX1( InterruptionControl, oIsHookPosition, rStepping, true, "Throw", &ThreadState ) ) {
		return( sThrowDMHookReturnApply );
		}
	else{
		return( sThrowDMHookReturnAbort );
		}
	}


bool iPCUi::fDMHookX3(
	const char* rHookMsg,
	tInvocationDMHookType rType, 
	iDMThreadState* rThreadState )
{
		
	std::string aTypeName( rHookMsg );
	switch( rType ) {
	case sInvocationDMHookTypeEnter:
		aTypeName += "EnterInvocation";
		break;
	case sInvocationDMHookTypeExit:
		aTypeName += "ExitInvocation";
		break;
	default:
		WlAssert(0);
	}

//	bool aX0;
//	tInterruptionControl aX1;
	bool rtn(true);
/*
	if( fDMHookX1( aX1, aX0, sSteppingLine, dInvokeHook, aTypeName.data(), rThreadState ) ) {
		rtn = true;
	} else {
		rtn = false;
	}
*/
	return rtn;
}


bool iPCUi::fInvokeDMHook(
	bool rIsFunction,
	tInvocationDMHookType rType, 
	iDMThreadState* rThreadState )
{
	return fDMHookX3( rIsFunction ? "Function" : "TopLevel", rType, rThreadState );
}

void iPCUi::fRecordEvent(long eventID, long topInstanceID, long instanceID,
												 WLLogMessage::Severity severity, const char* eventName,
												 double time, long lineNumber, const char* fileName,
												 const char* description, const char* extraDataType,
												 const char* extraData)
{

#if doEventLog
	printf("===================================================\n");
	printf("RecordEvent: %s\n", eventName);
	printf("eventID=%d, topInstanceID=%d, instanceID=%d\n", eventID, topInstanceID, instanceID);
	printf("severity=%d, time=%f, lineNumber=%d\n", severity, time, lineNumber);
	printf("fileName=%s, description=%s\n", fileName, description);
	if(extraData == NULL) {
		printf("extraDataType=%s, extraData=NULL\n", extraDataType);
	}
	else {
		printf("extraDataType=%s, extraData=%s\n", extraDataType, extraData);
	}
	printf("===================================================\n");
#endif
}


void iPCUi::fBeginTransaction(long instanceID, 
															const char* transactionName, 
															const char* transactionType) 
{
#if doEventLog
	printf("===================================================\n");
	printf("Begin Transaction: %s\n", transactionName);
	printf("instanceID=%d, transactionType=%s\n", instanceID, transactionType);
	printf("===================================================\n");
#endif
}


void iPCUi::fEndTransaction(long topInstanceID, long instanceID, 
														long parentID, unsigned long roundNum,
														const char* reason, const char* transactionName,
														double time, WLLogMessage::Severity severity,
														const char* transactionType, const char* extraData)
{
#if doEventLog
	printf("===================================================\n");
	printf("End Transaction: %s\n", transactionName);
	printf("topInstanceID=%d, instanceID=%d, parentID=%d\n", topInstanceID, instanceID, parentID);
	printf("reason=%s, time=%f, severity=%d, roundNum=%d\n", reason, time, severity, roundNum);
	if(extraData == NULL) {
		printf("transactionType=%s, extraData=NULL\n", transactionType);
	}
	else {
		printf("transactionType=%s, extraData=%s\n", transactionType, extraData);
	}
	printf("===================================================\n");
#endif
}



tNormalDMHookReturn iPCUi::fDMHookX2( 
	tInterruptionControl& InterruptionControl,
	bool& oIsHookPosition, 
	tStepping rStepping, 
	bool rIsStep,
	const char* DMHookMsg,
	iDMThreadState* ThreadState ) 
{
	if( fDMHookX1(InterruptionControl, oIsHookPosition, rStepping, rIsStep, DMHookMsg, ThreadState )  ) {
		return( sNormalDMHookReturnContinue );
		}
	else{
		return( sNormalDMHookReturnAbort );
		}
	}


bool iPCUi::fDMHookX1( 
	tInterruptionControl& InterruptionControl,
	bool& oIsHookPosition, 
	tStepping rStepping, 
	bool rIsStep,
	const char* DMHookMsg,
	iDMThreadState* ThreadState ) 
{
	bool Rtn = fDMHookX0( DMHookMsg, ThreadState, InterruptionControl, oIsHookPosition, rStepping, rIsStep );
	return(Rtn);
	}

bool iPCUi::fDMHookX0(
	const char* DMHookMsg,
	iDMThreadState* rThreadState,
	tInterruptionControl& oInterruptionControl,
	bool& oIsHookPosition, 
	tStepping rStepping,
	bool rIsStep ) 
{
	bool Rtn = true;

	cScriptFile* File = NULL;
	if( rThreadState ) {

		std::auto_ptr< iStackFrame > StackFrame( rThreadState->fGetTopStackFrame() );
		iFile* aFile_ = StackFrame->fGetFile();
		if( aFile_ ) {
			WlAssert( dynamic_cast<cScriptFile*>(aFile_) );
			File = static_cast<cScriptFile*>(aFile_);
		}
		const char* FileName = File ? File->fGetName() : StackFrame->fGetName();
		const char* aFunctionName = StackFrame->fGetFunctionName();
	
		printf( ">>>DMHook %s in %s() %s[%d]\n", DMHookMsg, aFunctionName, FileName, (int)StackFrame->fGetCurrentLineNumber() );

	} else {

		printf( ">>>DMHook %s in unknown\n", DMHookMsg );

	}

	if( rIsStep ) {

#if doInput
	bool Continue;
	do {
		Continue = false;
		cInputElement Str;
		fInputPull( Str );
		if( Str == "go" ) {
			oInterruptionControl = sInterruptionControlStep;
			mStepping = sSteppingNone;
		}	else if( Str == "now" ) {
			oInterruptionControl = sInterruptionControlStep;
			mStepping = sSteppingLine;
		}	else if( Str == "over" ) {
			oInterruptionControl = sInterruptionControlStep;
			mStepping = sSteppingOver;
		}	else if( Str == "in" ) {
			oInterruptionControl = sInterruptionControlStep;
			mStepping = sSteppingInto;
		}	else if( Str == "out" ) {
			oInterruptionControl = sInterruptionControlStep;
			mStepping = sSteppingOut;
		}	else if( Str == "abt" ) {
			oInterruptionControl = sInterruptionControlAbort;
			mStepping = sSteppingNone;
			Rtn = false;
		} else if( Str == "eval" ) {
			Continue = true;
			cInputElement Str;
			std::string aScript("");
			{
				fInputPull( Str );
				aScript = Str;
			}
			std::auto_ptr< iStackFrame > aStackFrame( rThreadState->fGetTopStackFrame() );
			std::auto_ptr< iValue > aValue( aStackFrame->fEvaluateScript( aScript.data(), aScript.length() ) );
			if (aValue.get() && aValue->fIsValid()){
				const char* aStr = aValue->fToString();

				printf( "%s= %s\n", aScript.c_str(), aStr );
			}
		} else if( Str == "info" ) {
			Continue = true;
			cInputElement Str;
			std::string FileName("");
			{
				fInputPull( Str );
				FileName += dDebuggerInfoPath;
				FileName += Str;
				FileName += ".xml";
			}
			std::string aScript("");
			{
				fInputPull( Str );
				if( Str != "." ) {
					aScript = Str;
				}
			}
			cThreadStateXMLElement::cLevel aLevel;
			{
				fInputPull( Str );
				int aLevelX0;
				sscanf( (const char*)Str.data(), "%d", &aLevelX0 );
				aLevel = aLevelX0;
			}
			cXMLStream Stream( FileName.data() );
			if( aScript == "" ) {
			cThreadStateXMLElement( Stream, *rThreadState, aLevel );
			} else {
				std::auto_ptr< iStackFrame > aStackFrame( rThreadState->fGetTopStackFrame() );
				std::auto_ptr< iValue > aValue( aStackFrame->fEvaluateScript( aScript.data(), aScript.length() ) );
				cValueXMLElement( Stream, aValue.get(), aScript.data(), aLevel );
			}
			printf( "info done\n" );
		} else if( Str == "bp+" ) {
			Continue = true;
			tLineNumber ActualLine = fToggleBreakPoint( *File, 35, true );
			printf( "breakpoint set at %d\n", (int)ActualLine );
		} else if( Str == "bp-" ) {
			Continue = true;
			tLineNumber ActualLine = fToggleBreakPoint( *File, 85, false );
			printf( "breakpoint remove at %d\n", (int)ActualLine );
		}	else {
			Continue = true;
			printf( "DMHook>\n" );
			}
		} while( Continue );
#endif

	} else {
		oInterruptionControl = sInterruptionControlStep;
		Rtn = true;
	}

	
	return(Rtn);
}


void iPCUi::fBeforeInitAgenda( iInterruptionControlCommit& ) {}
void iPCUi::fBeforeTerminateAgenda( iInterruptionControlCommit& ) {}
void iPCUi::fBeforeOnErrorTerminateAgenda( iInterruptionControlCommit& ) {}
void iPCUi::fBeginClient( iInterruptionControlCommit& ) {}
void iPCUi::fBeforeInitClient( iInterruptionControlCommit& ) {}
void iPCUi::fBeforeTerminateClient( iInterruptionControlCommit& ) {}
void iPCUi::fBeforeOnErrorTerminateClient( iInterruptionControlCommit& ) {}
void iPCUi::fEndClient( iInterruptionControlCommit& ) {}


bool iPCUi::fBeforeRound(
	unsigned long RoundNum,
	iInterruptionControlCommit& ) 
{
	bool Rtn( RoundNum <= mLastRound );
	
	printf( "before round %d\n", (int)RoundNum );
#if doFlowControl
	bool While( true );
	do {
		cInputElement Str;
		fInputPull( Str );
		if( "go" == Str ) {
			Rtn = true;
			While = false;
		} else if( "stop" == Str ) {
			Rtn = false;
			While = false;
		} else {
		}
	} while( While );
#endif
	
	return( Rtn );
}

void iPCUi::fBeforeOnScriptAbort( iInterruptionControlCommit& ) {}



/*
TestThreadInstanceScript::TestThreadInstanceScript(
	CLoadGeneralThreadInstance *threadInstance):
	CLoadGeneralThreadInstanceScript (threadInstance){
	mIn=msCount++;
	}

TestThreadInstanceScript::~TestThreadInstanceScript() {}

int TestThreadInstanceScript::msCount = 1;

CLoadGeneralThreadInstanceScript* TestThreadInstanceScript::fsNew(
	CLoadGeneralThreadInstance* Instance ) {
	return new TestThreadInstanceScript(Instance);
}
*/

bool WLInitTestGeneratorThread(
	JSContext* cx, 
	JSObject* obj, 
	CLEVariablesData* pVariablesData) {
	return true;
	}

bool WLInitTestGeneratorScript(
	JSContext* cx, 
	JSObject* global,
	JSObject* GeneralGlobal ){
	return true;
	}



cThreadPC::cThreadPC(
	iPCUi& PC,
	const char* FileName )
:
	mPC(PC),
	mFileName(FileName) 
{}


DWORD cThreadPC::fMain() 
{
	CTraceMsg::Init("ProbingClientTest");
  //	RegisterWLENewThreadInstanceScriptFunc(TestThreadInstanceScript::fsNew);
//	RegisterWLInitGeneratorScript(WLInitTestGeneratorScript);
	//RegisterWLInitGeneratorThread(WLInitTestGeneratorThread);

	cSingleClientMachineThread aMThread;
	aMThread.fInit(THREAD_TYPE_SINGLE_CLIENT);

	mPC.fInit( aMThread );

	RVCString InitializationScript("");
	WlAssert( dynamic_cast<iPCUi*>(&mPC) );
	cScriptFile MainFile( mFileName, static_cast<iPCUi&>(mPC) );
	mPC.fSetLastRound( 3 );
	mPC.fExecute(InitializationScript, MainFile, "PCUI", 3, 1 );

	mPC.fFinal();

//TVSTerminate();
	//WLERRTerminate();

	printf( "PC thread: end\n" );

	aMThread.fFinal();

	return(0);
}







cPCUiThread::cPCUiThread(
	nMT::cSStmManage& SStmManage,
	const char* FileName,
	cSingleClientProcess& rProcess )
:
	mSStmManage(&SStmManage),
	mFileName(FileName),
	mProcess(rProcess)
{}


		
DWORD cPCUiThread::fMain() {


	
	std::string UiId( "u" );
	nMT::cSStm UiSStm;
	mSStmManage->fAdd( UiSStm, UiId );

	cPCUiMT Ui(mProcess,UiSStm);
	iSpace& JsdSpace = Ui;
	
	cThreadPC ThrJS( Ui, mFileName );
	std::string ThrJSId( "j" );
	mSStmManage->fAdd( ThrJS.fGetSStm(), ThrJSId );

	ThrJS.fInit( ThrJSId );
	ThrJS.fBegin();

	printf( "ctl: X0\n" );


	while( !ThrJS.fTryTake() ) {
		nMT::cSStm::cElement Str;
		printf( "ctl>\n" );
		mInput.fPull( Str );
		if( Str == "rpt" ) {
			JsdSpace.fInterrupt();
			}
		else if( Str == "abt" ) {
			JsdSpace.fAbort();
			}
		else {
			printf( "ctl: NOP\n" );
			}
		}

	printf( "ctl: end\n" );
	return(0);
	
}




} // nPCUi

