/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "rvhtmldiettransactiondata.h"
#include "rvhtmldietattributeobject.h"
#include <collstr.h>
#include "float.h"
#include <math.h>


CRVHTMLDietTransactionsObject::CRVHTMLDietTransactionsObject()
{ 
}

CRVHTMLDietTransactionsObject::~CRVHTMLDietTransactionsObject()
{
  m_oAttributeObjects.clearAndDestroy();
  m_oAttributeOrdered.clear();
}

const char* CRVHTMLDietTransactionsObject::GetTagName()
{
  return m_sTag;
}
void CRVHTMLDietTransactionsObject::SetTagName(const char* p_sTagName)
{
  m_sTag = p_sTagName;
}

CHECKSUM_VAR CRVHTMLDietTransactionsObject::CalcCheckSum(const void* p_pvBuff, int p_nLength)
{
  CHECKSUM_VAR Checksum = 0 ;
  CHECKSUM_VAR Modulus = 0 ;
  int nModulus = 0 ;
  int nLength = p_nLength / sizeof(CHECKSUM_VAR);
  CHECKSUM_VAR* pBuff = (CHECKSUM_VAR*)p_pvBuff ;
  
  for ( int nIndex = 0 ; nIndex < nLength ; nIndex++)
  {
    Checksum ^= pBuff[nIndex] ;
  }
  
  //there is a remainder
  if ( nModulus = p_nLength % sizeof(CHECKSUM_VAR) )
  {
    memcpy( (void*)&Modulus, (char*)p_pvBuff + p_nLength-nModulus, nModulus );
    Checksum ^= Modulus ;
  }
  
  return Checksum ;
}

void CRVHTMLDietTransactionsObject::InsertAttributeObject(CRVHTMLDietAttributeObject* p_pAttributeObject)
{
  if (!p_pAttributeObject)
  {
    return;
  }

  RVCollectableString* pAttrName = new RVCollectableString(p_pAttributeObject->GetAttributeName());
  if (m_oAttributeObjects.insertKeyAndValue(pAttrName, p_pAttributeObject))
  {
    m_oAttributeOrdered.insert(p_pAttributeObject);
  } else
  {
    delete pAttrName;
  }
}

CRVHTMLDietAttributeObject* CRVHTMLDietTransactionsObject::GetAttributeObject(RVCString& p_sAttributeName)
{
  RVCollectableString sSearchKey = p_sAttributeName;
  RVCollectable* pResult = NULL;
  m_oAttributeObjects.findKeyAndValue(&sSearchKey, pResult);
  if (pResult)
  {
    return dynamic_cast<CRVHTMLDietAttributeObject*>(pResult);
  } else
  {
    return NULL;
  }
}

CRVHTMLDietAttributeObject* CRVHTMLDietTransactionsObject::GetFirstAttributeObject()
{
  if (m_oAttributeOrdered.isEmpty())
  {
    return NULL;
  }
  return dynamic_cast<CRVHTMLDietAttributeObject*>(m_oAttributeOrdered.at(0));
}

bool CRVHTMLDietTransactionsObject::GetAttributeNames(RVOrdered& p_oAttributeNamesArray)
{
  p_oAttributeNamesArray.clear();
  RVHashDictionaryIterator iter(m_oAttributeObjects);
  while (iter())
  {
    RVCollectableString* pKey = dynamic_cast<RVCollectableString*>(iter.key());
    if (pKey)
    {
      p_oAttributeNamesArray.insert(pKey);
    }
  }
  if (p_oAttributeNamesArray.entries() > 0)
  {
    return true;
  } else
  {
    return false;
  }
}

void CRVHTMLDietTransactionsObject::PrintAll()
{
  for (unsigned int i = 0 ; i < m_oAttributeOrdered.entries() ; i++)
  {
    CRVHTMLDietAttributeObject* pAttrObj = dynamic_cast<CRVHTMLDietAttributeObject*>(m_oAttributeOrdered.at(i));
    cout<<"Name:\t"<<pAttrObj->GetAttributeName()<<endl;
    cout<<"Value:\t"<<pAttrObj->GetAttributeValue()<<endl;
    cout<<"OValue:\t"<<pAttrObj->GetOriginalAttributeValue()<<endl;
    cout<<"Offset:\t"<<pAttrObj->GetAttributeOffset()<<endl;
  }
  cout<<endl;
}
