/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  



// CFTPBXMLWrapper.cpp: implementation of the CFTPBXMLWrapper class.
//
//////////////////////////////////////////////////////////////////////


#include <afxdisp.h>
#include <assert.h>
#include <tvsldict.h>
#include "crvxmldocument.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

using namespace MSXML;

////////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////////
/*!
* IXMLDOMDocument2Ptr CRVXMLDocument::get_IXMLDOMDocument2(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns smart pointer to MSXML DOM Document Interface
* \param p_ErrorInfo (in)
* \return IXMLDOMDocument2Ptr
* \author	Sergei Lvov
* \date		3/11/01
*/
IXMLDOMDocumentPtr CRVXMLDocument::get_IXMLDOMDocument(CWLErrorInfo& p_ErrorInfo)
{
	return m_pDocument;
}
/*!
* CRVXMLDocument::CRVXMLDocument(IXMLDOMDocument2Ptr p_IXMLDOMDocument, RVCString p_sRootElementName)
* \brief 
* Contructor
* \param p_sRootElementName (in)
* \param IXMLDOMDocument2Ptr (in)
* \return void
* \author	Sergei Lvov
* \date		3/11/01
*/

CRVXMLDocument::CRVXMLDocument(IXMLDOMDocumentPtr p_IXMLDOMDocument, RVCString p_sRootElementName) :
	m_pDocument(p_IXMLDOMDocument),
	m_RootElement(p_IXMLDOMDocument,NULL)

{
	IXMLDOMElementPtr pDOMElement = NULL;
	HRESULT hr = m_pDocument->get_documentElement(&pDOMElement);
	
	if (hr == S_FALSE)
	{
		//There is no documentElement
		m_RootElement.m_pXMLDOMElement = m_pDocument->createElement(_bstr_t(p_sRootElementName));
		HRESULT hr = m_pDocument->putref_documentElement(m_RootElement.m_pXMLDOMElement);		
	} else
	{
		m_RootElement.m_pXMLDOMElement = pDOMElement;
	}
}

/*!
* CRVXMLDocument::CRVXMLDocument(XMLParserType p_eXMLParser, RVCString p_sRootElementName) :
* \brief 
* Contructor
* \param p_eXMLParser (in)
* \param p_sRootElementName (in)
* \return void
* \author	Sergei Lvov
* \date		3/11/01
*/

CRVXMLDocument::CRVXMLDocument(XMLParserType p_eXMLParser, RVCString p_sRootElementName) :
	m_pDocument(NULL),
	m_RootElement(NULL,NULL)
{
		HRESULT hr = CoInitialize(NULL); 

		IXMLDOMDocumentPtr pXMLDOMDocument(__uuidof(MSXML::DOMDocument));
		m_pDocument =  pXMLDOMDocument;
		m_pDocument->put_async(VARIANT_FALSE);
		
		m_RootElement.m_pXMLDocument = m_pDocument;
		m_RootElement.m_pXMLDOMElement = m_pDocument->createElement(_bstr_t(p_sRootElementName.data()));
		hr = m_pDocument->putref_documentElement(m_RootElement.m_pXMLDOMElement);		
}

CRVXMLDocument::~CRVXMLDocument()
{}

/*!
* bool CRVXMLDocument::load(RVCString p_sURL, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Loads XML from URL
* \param p_sURL (in)
* \param p_ErrorInfo (out)   - Error Info
* \return bool
* \author	Sergei Lvov
* \date		3/11/01
*/
bool CRVXMLDocument::load(RVCString p_sURL, CWLErrorInfo& p_ErrorInfo)
{
	try
	{
		VARIANT_BOOL varResult = m_pDocument->load(_variant_t(p_sURL)); 
		if (varResult == VARIANT_TRUE)
		{
			IXMLDOMElementPtr newRoot;
			HRESULT hr = m_pDocument->get_documentElement(&newRoot);
			SUCCEEDED(hr) ? 0 : throw hr;
			m_RootElement = CRVXMLElement(m_pDocument, newRoot);
		}
		// do something with p_ErrorInfo
		return varResult == VARIANT_TRUE;
	}
	catch (...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return false;
	}

}



/*!
* bool CRVXMLDocument::loadXML(RVCString p_sXML, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Loads XML from string
* \param p_sXML (in)
* \param p_ErrorInfo (out)   - Error Info
* \return bool
* \author	Sergei Lvov
* \date		12/11/02
*/
bool CRVXMLDocument::loadXML(RVCString p_sXML, CWLErrorInfo& p_ErrorInfo)
{
	try
	{
		VARIANT_BOOL varResult = m_pDocument->loadXML(_bstr_t(p_sXML));
		if (varResult == VARIANT_TRUE)
		{
			IXMLDOMElementPtr newRoot;
			HRESULT hr = m_pDocument->get_documentElement(&newRoot);
			SUCCEEDED(hr) ? 0 : throw hr;
			m_RootElement = CRVXMLElement(m_pDocument, newRoot);
		}
		// do something with p_ErrorInfo
		return varResult == VARIANT_TRUE;
	}
	catch (...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return false;
	}
}

//Galtech
//bool CRVXMLDocument::loadXML(wchar_t* p_sXML, CWLErrorInfo& p_ErrorInfo)
bool CRVXMLDocument::loadXML(gt_wchar_t* p_sXML, CWLErrorInfo& p_ErrorInfo)
{
	try
	{
		VARIANT_BOOL varResult = m_pDocument->loadXML(_bstr_t(p_sXML));
		if (varResult == VARIANT_TRUE)
		{
			IXMLDOMElementPtr newRoot;
			HRESULT hr = m_pDocument->get_documentElement(&newRoot);
			SUCCEEDED(hr) ? 0 : throw hr;
			m_RootElement = CRVXMLElement(m_pDocument, newRoot);
		}
		// do something with p_ErrorInfo
		return varResult == VARIANT_TRUE;
	}
	catch (...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return false;
	}
}


/*!
* bool CRVXMLDocument::save(RVCString p_sURL, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Saves XML to URL
* \param p_sURL (in)
* \param p_ErrorInfo (out)   - Error Info
* \return bool
* \author	Sergei Lvov
* \date		3/11/01
*/
bool CRVXMLDocument::save(RVCString p_sURL, CWLErrorInfo& p_ErrorInfo)
{
	try 
	{
		HRESULT hr = m_pDocument->save(_variant_t(p_sURL)); 
		// do something with p_ErrorInfo
		return hr == S_OK;
	}
	catch (...)
	{
		// do something with p_ErrorInfo
		assert(0);
		return false;
	}
}
/*!
* RVCString CRVXMLDocument::get_xml(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Gets XML String
* \param p_ErrorInfo (out)   - Error Info
* \return RVCString
* \author	Sergei Lvov
* \date		3/11/01
*/
RVCString CRVXMLDocument::get_xml(CWLErrorInfo& p_ErrorInfo)
{
	if (m_pDocument == NULL || m_RootElement.m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return "";
	}
	try 
	{
		return RVCString(m_pDocument->xml);
	} 
	catch (...)
	{
		// do something with p_ErrorInfo
		assert(0);
		return "";
	}
}
/*!
* CRVXMLElement CRVXMLDocument::get_documentElement(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Gets XML root element
* \param p_ErrorInfo (out)   - Error Info
* \return CRVXMLElement
* \author	Sergei Lvov
* \date		3/11/01
*/
CRVXMLElement CRVXMLDocument::get_documentElement(CWLErrorInfo& p_ErrorInfo)
{
	if (m_pDocument == NULL || m_RootElement.m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return CRVXMLElement(NULL, NULL);
	}
	try
	{
		return m_RootElement;
	}
	catch (...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return CRVXMLElement(NULL,NULL);
	}
}

/*!
* RVTValSlistDictionary<RVCString, RVCString> CRVXMLDocument::get_parseError(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns a dictionary with XML parsing errors
* \param p_ErrorInfo (out)   - Error Info
* \return RVTValSlistDictionary<RVCString, RVCString>
* \author	Sergei Lvov
* \date		3/11/01
*/
RVTValSlistDictionary<RVCString, RVCString> CRVXMLDocument::get_parseError(CWLErrorInfo& p_ErrorInfo)
{
	RVTValSlistDictionary<RVCString, RVCString> errorDict;
	IXMLDOMParseErrorPtr pErrObj;
	try
	{
		HRESULT hr = m_pDocument->get_parseError(&pErrObj);
		SUCCEEDED(hr) ? 0 : throw hr;
		errorDict.insertKeyAndValue("errorCode", RVCString("" + pErrObj->errorCode));
		errorDict.insertKeyAndValue("filepos", RVCString("" + pErrObj->filepos));
		errorDict.insertKeyAndValue("line", RVCString(pErrObj->line +""));
		errorDict.insertKeyAndValue("linepos",RVCString(pErrObj->linepos + ""));
		errorDict.insertKeyAndValue("reason",RVCString(pErrObj->reason));
		errorDict.insertKeyAndValue("srcText",RVCString(pErrObj->srcText));
		errorDict.insertKeyAndValue("url",RVCString(pErrObj->url));
	}
	catch (...)
	{
			assert(0);
			// do something with p_ErrorInfo
			errorDict.clear();
	}
	return errorDict;
}
