/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// RVXMLElement.cpp: implementation of the CRVXMLElement class.
//
//////////////////////////////////////////////////////////////////////

#include <afxdisp.h>
#include <assert.h>
#include <tvsldict.h>
#include "crvxmlelement.h"
//Galtech
//#include "gt_strwrapper.h"

using namespace MSXML;

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CRVXMLElement::~CRVXMLElement()
{
}
/*!
* AttrDict CRVXMLElement::get_attributes(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns a dictionary of XML Element attributes.
* \param p_ErrorInfo (out)   - Error Info
* \return RVTValSlistDictionary<RVCString,RVCString>
* \author	Sergei Lvov
* \date		3/11/01
*
*/
AttrDict CRVXMLElement::get_attributes(CWLErrorInfo& p_ErrorInfo)
{
	AttrDict dict;
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return dict;
	}
	try
	{	
		IXMLDOMNamedNodeMapPtr attributeMap;
		m_pXMLDOMElement->get_attributes(&attributeMap);
		for (int i = 0 ; i < attributeMap->Getlength() ; i++)
		{
			IXMLDOMNodePtr listItem;
			HRESULT hr = attributeMap->get_item(i, &listItem);
			// Check hr
			dict.insertKeyAndValue(RVCString(listItem->nodeName), RVCString(listItem->text));
		}
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		dict.clear();
	}
	return dict;
}
/*!
* RVCString CRVXMLElement::get_attribute(RVCString p_sAttributeName, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns attribute value of the submited XML Element.
* \param p_ErrorInfo (out)   - Error Info
* \param p_sAttributeName (in)   - Attribute Name
* \return RVCString
* \author	Sergei Lvov
* \date		3/11/01
*/
RVCString CRVXMLElement::get_attribute(RVCString p_sAttributeName, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return "";
	}

	try
	{
		_variant_t attrValue = m_pXMLDOMElement->getAttribute(_bstr_t(p_sAttributeName));
		if (attrValue.bstrVal == VT_EMPTY)
			return "";
		return RVCString(_bstr_t(attrValue.bstrVal));
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo - attrib ute not found
		return "";
	}
}
/*!
* bool CRVXMLElement::set_attribute(RVCString p_sAttributeName, RVCString p_sAttributeValue, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Sets attribute value of the submited XML Element.
* \param p_ErrorInfo (out)   - Error Info
* \param p_sAttributeName (in)   - Attribute Name
* \param p_sAttributeValue (in)   - Attribute Name
* \return bool
* \author	Sergei Lvov
* \date		3/11/01
*/
bool CRVXMLElement::set_attribute(RVCString p_sAttributeName, RVCString p_sAttributeValue, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return false;
	}

	try
	{
		HRESULT hr = m_pXMLDOMElement->setAttribute(_bstr_t(p_sAttributeName), _variant_t(p_sAttributeValue));
		return (hr == S_OK);
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return false;
	}
}

/*!
* RVCString CRVXMLElement::get_name(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns XML Element name
* \param p_ErrorInfo (out)   - Error Info
* \return RVCString
* \author	Sergei Lvov
* \date		3/11/01
*/
RVCString CRVXMLElement::get_name(CWLErrorInfo& p_ErrorInfo)
{	
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return "";
	}

	try
	{
		BSTR bstrNodeName;
		HRESULT hr = m_pXMLDOMElement->get_nodeName(&bstrNodeName);
		return RVCString(_bstr_t(bstrNodeName));
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return "";
	}
}

/*!
* RVCString CRVXMLElement::get_value(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns XML Element value
* \param p_ErrorInfo (out)   - Error Info
* \return RVCString
* \author	Sergei Lvov
* \date		3/11/01
*/
RVCString CRVXMLElement::get_value(CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return "";
	}

	IXMLDOMNodeListPtr childList;
	try
	{
		HRESULT hr = m_pXMLDOMElement->get_childNodes(&childList);
		if (hr != S_OK) return "";
		for (int i = 0 ; i < childList->length ; i++)
		{
			IXMLDOMNodePtr node;
			hr = childList->get_item(i,&node);
			if (hr == S_OK)
			{
/*				tagDOMNodeType nodeType;
				hr = node->get_nodeType(&nodeType);
				if ((hr == S_OK) && (nodeType == NODE_TEXT))
				{
					_variant_t value;
					hr = node->get_nodeValue(&value);
					return RVCString(_bstr_t(value.bstrVal));
*/
				_bstr_t bstrNodeType = node->nodeTypeString;
				if (bstrNodeType == _bstr_t("text"))
				{
					_variant_t value;
					hr = node->get_nodeValue(&value);
					return RVCString(_bstr_t(value.bstrVal));

				}
			}
		}
		return "";
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return "";
	}
}
/*!
* RVCString CRVXMLElement::set_value(RVCString p_sValue, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Sets XML Element value
* \param p_ErrorInfo (out)   - Error Info
* \param p_sValue (in)   - Value
* \return bool
* \author	Sergei Lvov
* \date		3/11/01
*/
bool CRVXMLElement::set_value(RVCString p_sValue, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return false;
	}

	try
	{
		//IXMLDOMNodePtr textNode = m_pXMLDocument->createTextNode(_bstr_t(p_sValue));
		//HRESULT hr = m_pXMLDOMElement->appendChild(textNode);
		HRESULT hr = m_pXMLDOMElement->put_text(_bstr_t(p_sValue));
		return (hr == S_OK);
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return false;
	}
}
/*!
* CRVXMLElement CRVXMLElement::get_parentElement(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns parent CRVXMLElement object, returns empty object in case of error or if there is no parent
* \param p_ErrorInfo (out)   - Error Info
* \return CRVXMLElement
* \author	Sergei Lvov
* \date		3/11/01
*/
CRVXMLElement CRVXMLElement::get_parentElement(CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return CRVXMLElement(NULL,NULL);
	}

	IXMLDOMNodePtr parent = NULL;
	try
	{
		HRESULT hr = m_pXMLDOMElement->get_parentNode(&parent);
		SUCCEEDED(hr) ? 0 : throw hr;
		return CRVXMLElement(m_pXMLDocument,(IXMLDOMElementPtr)parent);
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return CRVXMLElement(NULL,NULL);
	}
}
/*!
* CRVXMLElementList CRVXMLElement::get_childElements(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns RVTValSlist<CRVXMLElement> of child elements
* \param p_ErrorInfo (out)   - Error Info
* \return CRVXMLElementList (RVTValSlist<CRVXMLElement>)
* \author	Sergei Lvov
* \date		3/11/01
*/
CRVXMLElementList CRVXMLElement::get_childElements(CWLErrorInfo& p_ErrorInfo)
{
	CRVXMLElementList list;
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return list;
	}

	IXMLDOMNodeListPtr childNodes;
	try
	{
		HRESULT hr = m_pXMLDOMElement->get_childNodes(&childNodes);
		//Check hr
		for (int i = 0 ; i<childNodes->length; i++)
		{
			_bstr_t bstrNodeType;
			bstrNodeType = childNodes->item[i]->nodeTypeString;
			if (bstrNodeType == _bstr_t("element"))
			{
				list.insert(CRVXMLElement(m_pXMLDocument,(IXMLDOMElementPtr)childNodes->item[i]));
			}
		}
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		list.clear();
	}
	return list;
}

/*!
* CRVXMLElement CRVXMLElement::add_childElement(RVCString p_sName, RVCString p_sValue, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Adds child element, returns new child
* \param p_ErrorInfo (out)   - Error Info
* \param p_sName (in)   - Name of new element
* \param p_sValue (in)   - Value of new elements
* \return CRVXMLElement
* \author	Sergei Lvov
* \date		3/11/01
*
*/
CRVXMLElement CRVXMLElement::add_childElement(RVCString p_sName, RVCString p_sValue, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		CRVXMLElement(NULL,NULL);
	}

	try
	{
		//Galtech
		// Ofir - No Idea why Galtech put this line here.
		// Removing this line is a fix for bug 16992 - unicode not supported in execution tree.
		// The only place that is relevant for Webload is in CWLXMLReport::EndPageDownloadReport in 
		// httpload, "sUserTitle.data()". So far no scenario could reach this code, so I assume 
		// this fix is ok for httpload as well.

//		p_sValue = ToANSI(p_sValue);

		CRVXMLElement newElement(m_pXMLDocument, p_sName, p_sValue);
		IXMLDOMElementPtr newChild = m_pXMLDOMElement->appendChild(newElement.m_pXMLDOMElement);
		newElement.m_pXMLDOMElement = newChild;
		return newElement;
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return CRVXMLElement(NULL,NULL);
	}

}
/*!
* CRVXMLElement CRVXMLElement::add_childElement(CRVXMLElement& p_CRVXMLElement, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Adds child element, returns new child
* \param p_ErrorInfo (out)   - Error Info
* \param p_CRVXMLElement (in)   - New element
* \return CRVXMLElement
* \author	Sergei Lvov
* \date		3/11/01
*
*/
CRVXMLElement CRVXMLElement::add_childElement(CRVXMLElement& p_CRVXMLElement, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		CRVXMLElement(NULL,NULL);
	}

	try
	{
		IXMLDOMElementPtr clonedNode = p_CRVXMLElement.m_pXMLDOMElement->cloneNode(VARIANT_TRUE);	
		IXMLDOMElementPtr newChild = m_pXMLDOMElement->appendChild(clonedNode);
		return CRVXMLElement(m_pXMLDocument, newChild);
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return CRVXMLElement(NULL,NULL);
	}
}
/* CRVXMLElement CRVXMLElement::add_childElement(RVCString p_sName, RVCString p_sValue, AttrDict p_Attributes, CWLErrorInfo& p_ErrorInfo)
* \brief 
* Adds child element, returns new child
* \param p_ErrorInfo (out)   - Error Info
* \param p_sName (in)   - New element's name
* \param p_sValue (in)   - New element's value
* \param p_Attributes (in)   - New element's attribute dictionary
* \return CRVXMLElement
* \author	Sergei Lvov
* \date		3/11/01
*
*/
CRVXMLElement CRVXMLElement::add_childElement(RVCString p_sName, RVCString p_sValue, AttrDict p_Attributes, CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		CRVXMLElement(NULL,NULL);
	}

	try
	{
		return *this;
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		return *this;
	}
}

/*!
* CRVXMLElement::CRVXMLElement(IXMLDOMDocument* p_pXMLDomDocument, IXMLDOMElement* p_pXMLDomElement) :
* \brief 
* Contructor
* \param p_pXMLDomDocument (in)
* \param p_pXMLDomElement (in)
* \return void
* \author	Sergei Lvov
* \date		3/11/01
*/
CRVXMLElement::CRVXMLElement(IXMLDOMDocumentPtr p_pXMLDomDocument, IXMLDOMElementPtr p_pXMLDomElement) :
	m_pXMLDocument(p_pXMLDomDocument),
	m_pXMLDOMElement(p_pXMLDomElement)
{
}

/*!
* CRVXMLElement::CRVXMLElement(IXMLDOMDocument* p_pXMLDomDocument, RVCString p_sName, RVCString p_sValue)
* \brief 
* Contructor
* \param p_pXMLDomDocument (in)
* \param p_sName (in)
* \param p_sValue (in)
* \return void
* \author	Sergei Lvov
* \date		3/11/01
*/

CRVXMLElement::CRVXMLElement(IXMLDOMDocumentPtr p_pXMLDomDocument, RVCString p_sName, RVCString p_sValue)
	:	m_pXMLDocument(p_pXMLDomDocument),
		m_pXMLDOMElement(NULL)
{
	HRESULT hr;
	m_pXMLDOMElement = m_pXMLDocument->createElement(_bstr_t(p_sName));
	hr = m_pXMLDOMElement->put_text(_bstr_t(p_sValue));
}

/*!
* bool CRVXMLElement::is_empty(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns true is the element is empty
* \param p_ErrorInfo (out)   - Error Info
* \return bool
* \author	Sergei Lvov
* \date		3/11/01
*/
bool CRVXMLElement::is_empty(CWLErrorInfo& p_ErrorInfo)
{
	return (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL);
}
/*!
* RVCString CRVXMLElement::get_xml(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Gets XML String
* \param p_ErrorInfo (out)   - Error Info
* \return RVCString
* \author	Sergei Lvov
* \date		3/11/01
*/
RVCString CRVXMLElement::get_xml(CWLErrorInfo& p_ErrorInfo)
{
	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return "";
	}
	try 
	{
		return RVCString(m_pXMLDOMElement->xml);
	} 
	catch (...)
	{
		// do something with p_ErrorInfo
		return "";
	}
}

/*!
* CRVXMLElementList CRVXMLElement::getElementsByTagName(CWLErrorInfo& p_ErrorInfo)
* \brief 
* Returns a list of all descendant elements that match the supplied name.
* \param p_ErrorInfo (out)   - Error Info
* \return CRVXMLElementList
* \author	Sergei Lvov
* \date		3/11/01
*/

CRVXMLElementList CRVXMLElement::getElementsByTagName(RVCString p_sTagName, CWLErrorInfo& p_ErrorInfo)
{
	IXMLDOMNodeListPtr p_resultList;
	CRVXMLElementList list;

	if (m_pXMLDocument == NULL || m_pXMLDOMElement == NULL)
	{
		// do something with p_ErrorInfo
		return list;
	}

	try
	{
		IXMLDOMNodeListPtr p_resultList = m_pXMLDOMElement->getElementsByTagName(_bstr_t(p_sTagName));
		//Check hr
		for (int i = 0 ; i<p_resultList->length; i++)
		{
			list.insert(CRVXMLElement(m_pXMLDocument,(IXMLDOMElementPtr)p_resultList->item[i]));
		}
	}
	catch(...)
	{
		assert(0);
		// do something with p_ErrorInfo
		list.clear();
	}
	return list;
}
