/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// "WLStatisticsGenerator.h"

#ifndef __TV_WLSTATISTICSGENERATOR_H
#define __TV_WLSTATISTICSGENERATOR_H

#include <sys/types.h>
#include <sys/timeb.h>
//#include <sys/timeb.h>

#include <collint.h>
#include <hashdict.h>
#include <collstr.h>
#include <thr.h>

#include "platform.h"
#include "tvpsync.h"
#include "tvpeventqueue.h"
#include "wlstatpointtypeenum.h"
//#include "wlstatmanagerpublic.h"
///////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////

class CWLStatGenSlice;
class CWLStatGenMeasurement;
class CWLStatGenMain;
class CWLStatGenMeasurementDictionary;
class CWLStatGenTimerDictionary;
class CWLStatGenClient;


//////////////////////////////////////////////////////////

typedef enum {
  tminimum,
  tmaximum,
  tcount,
  tsum,
  tsumSqr,
	tstddev,
  taverage,
  tlastmin,
  tlastmax,
  tlastcount,
  tlastsum,
  tlastsumSqr,
  tlastaverage,
  tStatMeasurementermentTypeSize
} StatMeasurementermentType ;


typedef enum{
	dumpSlice,
      addSlice
} AddType;

////////////////////////////////////////////////////////////////////

class DllExport CWLStatGenMeasurement : public RVCollectable
{
private:
  double array[tStatMeasurementermentTypeSize];

public:
  CWLStatPointTypeEnum::PointType typeName;

public:
  CWLStatGenMeasurement(CWLStatPointTypeEnum::PointType type);
  virtual ~CWLStatGenMeasurement()
  {
  }

  void      Print();
  CWLStatPointTypeEnum::PointType    GetType()					{ return typeName ;}
  double		GetValue(StatMeasurementermentType t) {return array[t];}
  double		GetMinimum()			{return array[tminimum];}
  double		GetMaximum()			{return array[tmaximum];}
  int				GetCount()				{return (int)(array[tcount]);}
  double		GetSum()					{return array[tsum];}
  double		GetSumSqr()				{return array[tsumSqr];}
  double		GetAverage()			{return (array[tcount]>0) ? array[tsum] / array[tcount] : -1;}
  double		GetStddev()				{return array[tstddev];}
  double		GetLastsMinimum() {return array[tlastmin];}
  double		GetLastsMaximum() {return array[tlastmax];}
  int				GetLastsCount()		{return (int)(array[tlastcount]);}
  double		GetLastsSum()			{return array[tlastsum];}
  double		GetLastsSumSqr()	{return array[tlastsumSqr];}
  double		GetLastsAverage()	{return (array[tlastcount]>0) ?
                             array[tlastsum] / array[tlastcount] : -1;}

	bool	  IsResetLasts();

	void    Reset();
	void    ResetLasts();

  void	  AddLast(double l);
  void		AddValues(double min, double max, int cnt, double s,double sSqr,
                 double l_min, double l_max, int l_cnt, double l_sum, double l_sumSqr, AddType type);

  void		SetValues(double min, double max, int cnt, double s,double sSqr,
                    double l_min, double l_max, int l_cnt, double l_sum, double l_sumSqr,
                    CWLStatPointTypeEnum::PointType type);

  void		AppendDump(RVCString &str, const char* name = "");
  void		DumpLasts(RVCString &str, const char* name = "");
};

/////////////////////////////////////////////////////////////////

class DllExport CWLStatGenTimer  : public RVCollectable
{
private:
	TVPTimerObject timer;
public:
	virtual CWLStatGenTimer::~CWLStatGenTimer(){}
	void		ResetTimer(){timer.reset();}
	double	SendTimer();
};


/////////////////////////////////////////////////////////////////


class DllExport CWLStatGenSlice : public RVHashDictionary
{
private:
	int			          sliceNum;
	bool		          flagUse;
	double            endTime;
	int			          count;

  // This flag is used to signify that the slice is not the
  // actual owner of the dictionaries it holds.
  // This is to facilitate sharing of dictionaries.
  // For example, the implementation of 'raw' client
  // client statistics is based on creating a temporary
  // slice for each threa's current dictionary - since this
  // is actually the same dictionary held by the entire
  // generator's current slice, it shouldn't be destroyed
  // when the temporary slice is dtored. 
	bool							ownDict;
  CWLStatGenMain    *pGenerator;

	RVTRecursiveLock<RVMutexLock>       *countMutex;
public:

	CWLStatGenSlice( int slice, double endInterval, CWLStatGenMain *pGen, bool OwnDict=true );
	virtual ~CWLStatGenSlice();

	CWLStatGenMeasurementDictionary *DumpSlice(RVCString &str);
	void AddDictionarysSlice(CWLStatGenSlice *pSliceToAdd);
	void AddDictionary( CWLStatGenMeasurementDictionary  *pDic, int clientId);
	void ResetLasts();

	int			 GetCount();
	void		 DecCount();
	int		   GetSliceNum()       {return sliceNum; } // Should be deleted ...
	double   GetEndTimeSlice()	 {return endTime;   }
	void		 NoUse()						 {flagUse=false; }
	bool		 GetFlag()					 {return flagUse; }

  void     ReleaseDicts();
  CWLStatGenMain* GetGenerator() { return pGenerator; }
};
///////////////////////////////////////////////////////////////////

class DllExport CWLStatGenMeasurementDictionary : public RVHashDictionary
{
  friend class CWLStatGenClient;
private:
	RVTRecursiveLock<RVMutexLock>       *mutex;
  CWLStatGenSlice				 *pSlice;
  CWLStatGenClient       *pMyClient;

  // Chaim Apr. 16 2001 - Stats cache
  #define STAT_STORE_SIZE 20
  #define MAX_STAT_NAME_LEN 256
  int     m_stat_counter;
  struct {
    char    name[MAX_STAT_NAME_LEN + 1];
    double  l;
    CWLStatPointTypeEnum::PointType
            type;
  } m_stat[STAT_STORE_SIZE];

public:
  CWLStatGenMeasurementDictionary(CWLStatGenSlice *pCS);
  virtual ~CWLStatGenMeasurementDictionary();

  void SetClient(CWLStatGenClient *pClient) { pMyClient=pClient; }
  CWLStatGenClient* GetClient() { return pMyClient; }
  void ReleaseClient() { pMyClient=NULL; }

	CWLStatGenMeasurement  *GetMeasurement(const char *name);
	double GetMinimum(const char *name);
  double GetMaximum(const char *name);
  int    GetCount(const char *name);
  double GetSum(const char *name);
  double GetSumSqr(const char *name);
  double GetAverage(const char *name);

  double GetLastsMinimum(const char *name);
  double GetLastsMaximum(const char *name);
  int    GetLastsCount(const char *name);
  double GetLastsSum(const char *name);
  double GetLastsSumSqr(const char *name);
  double GetLastsAverage(const char *name);

  double GetStddev(const char *name);

	void Print();

	CWLStatGenSlice *GetSlice()	 {return pSlice;}
  bool	 GetFlag()						 {return (pSlice?pSlice->GetFlag():true); }
  double GetSliceNum()				 {return pSlice->GetSliceNum();}

	void ResetLasts();

  bool FlushStatsCache ();
	bool AddLast(const char *name, double l, CWLStatPointTypeEnum::PointType type);
  void Reset(const char *name);

  void SetValues(const char *name,
                 double min, double max, int cnt, double s,double sd,
                 double l_min, double l_max, int l_cnt, double l_sum,
                 CWLStatPointTypeEnum::PointType type);

  void AddValuesOfDict(CWLStatGenMeasurementDictionary *from, AddType type);
  void DumpDict(RVCString &str);
  void DumpLasts(RVCString &str);
};


//////////////////////////////////////////////////////////////////

class DllExport CWLStatGenTimerDictionary : public RVHashDictionary
{
public:

	CWLStatGenTimerDictionary() : RVHashDictionary(33){}
  virtual ~CWLStatGenTimerDictionary() {
    clearAndDestroy();
  }

	void		ResetTimer(const char *name);
  double	SendTimer(const char *name);
};



/////////////////////////////////////////////////////////////////

class DllExport CWLStatGenMain
{
friend class CAgendaStatistics;

private:
	RVCString							  agendaName;
	double								  systemBeginRunTime;
	double								  sliceSize; // The time that a current slice take
	int											sliceNum;  // The current slice number
	double									deltaEnd;
	bool										terminated;
	bool										terminationInProgress;
	bool										freeze;

  //
  // statistics slice container
  RVHashDictionary        sliceContainer;
	RVTRecursiveLock<RVMutexLock>       *sliceMutex;
	CWLStatGenSlice	        *pNewestSlice;
  //
  // pack queue stuff.
	TVPEventQueue										packQueue;
	CWLStatGenSlice									*historySlice;
	CWLStatGenMeasurementDictionary *total;
	TVPManualEvent				          finishPack;
	TVPManualEvent				          finishGarbage;
  //
  // garbage queue stuff
	TVPEventQueue					          garbageQueue;
	RVTRecursiveLock<RVMutexLock>       *garbageMutex;

public:

	CWLStatGenMain(const char* agenda);
	virtual ~CWLStatGenMain();

  // Request for termination: do termination work without waiting
  // at the end.
  void			  Terminate();

  // Waits until all termination work is done.
  // In this way Terminate can be called by a client of this class
  // then the client can proceed executing other code and at the end
  // wait for terminate to finish, so it may save time.
  void        WaitForTerminateToEnd();

	void        Start( double BeginTime, double sliceSizeResolution );
	void        Freeze();
  bool        IsFrozen() { return freeze; }
	void        DeFrost(double freeze_time);
  void        TerminationInProgress()      { terminationInProgress = true; }
	double      GetBeginSystem()						 { return systemBeginRunTime;}
	double		  GetEndTime()								 { return deltaEnd;}
	double      GetSliceSize()							 { return sliceSize;}
	void        SetSliceSize(double size)    { sliceSize=size;}
	const char* GetAgendaName()						   { return agendaName.data();}

  //
  // statistics slice container
  void                AddSlice(CWLStatGenSlice *pSlice, int sliceNum);
  void                RemoveSlice(CWLStatGenSlice *pSlice);
  CWLStatGenSlice		  *GetNewestSlice() { return pNewestSlice; }
  CWLStatGenMeasurementDictionary* GetNewDictionary(int clientId, CWLStatGenMeasurementDictionary* pOldDictionary);


 	void              doPackWork();
  CWLStatGenSlice*  WaitForSlice();
  void              SendSlice( CWLStatGenSlice *slice );

	void			        doGarbageWork();

	CWLStatGenMeasurementDictionary *GetTotal() { return total; }

  virtual void CalculateExtraStatisticsForDictionary( CWLStatGenMeasurementDictionary *pDic, double lastSliceSize, double lastSliceTime) = 0;
  virtual void CalculateExtraStatisticsForCurrentClientDictionary( CWLStatGenMeasurementDictionary *pDic, int ClientId, double SliceEndTime) = 0;
  virtual void SendDictionaryString( RVCString &str, double sliceTime, const char* agendaName ) {}

  // @@Chaim - expose nicely later
  RVCriticalSection                       *newDictMutex;
  int     m_stat_cache_size;
  double  m_flush_interval;

};

//////////////////////////////////////////////////////////////////

class DllExport CWLStatGenClient
{
private:
	CWLStatGenMeasurementDictionary		*pCurrentDictionary;
	CWLStatGenTimerDictionary				  timingDictionary;
	CWLStatGenMain										*pGenerator;
	long                              clientId;

public:

	CWLStatGenClient(CWLStatGenMain *pGS, int name);
	virtual ~CWLStatGenClient();

	RVCollectableInt GetClientId() {return clientId;}

  double  GetCurrentEndTime();

  void    FlushStatsCache ();
	void		AddLast (const char *measurement, double v ,CWLStatPointTypeEnum::PointType type);

	void		ResetTimer(const char *name);
  double 	SendTimer(const char *name);

  void    GetNewDictionary();
  void    ReleaseDictionary();
};

#endif //__TV_WLSTATISTICSGENERATOR_H

