/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
 // wlstatisticsgenerator.cpp

#include <math.h>
#include <thr.h>

#include "tracedll.h"
#include "wllightstring.h"
#include "tvpnumconversions.h"
#include "wlstatisticsgenerator.h"
#include "wlstatmanagerglobals.h"
#include "tvenv.h"
 

//////////////////////////////////////////////////////////////////
//                      CWLStatGenTimer
//////////////////////////////////////////////////////////////////

double CWLStatGenTimer::SendTimer()
{
	double t = timer.get();
	 // AddLast(t); Ask Yahali diffirent types ???
	return t;
}


//////////////////////////////////////////////////////////////////
//                      CWLStatGenTimerDictionary
//////////////////////////////////////////////////////////////////

double CWLStatGenTimerDictionary::SendTimer(const char *name)
{
  CWLCollectableString k(name, false);
  CWLStatGenTimer *sb;

  if ((sb = (CWLStatGenTimer *) findValue((RVCollectable *) &k)) == NULL)
	{
		return -1;
  }
  double t = sb->SendTimer();
  return t;

}


void CWLStatGenTimerDictionary::ResetTimer(const char *name)
{
  CWLCollectableString k(name, false);
  CWLStatGenTimer *sb;

  if ((sb = (CWLStatGenTimer *) findValue((RVCollectable *) &k)) == NULL)
	{
    sb = new CWLStatGenTimer;
    insertKeyAndValue((RVCollectable *) new CWLCollectableString(name), 
			                (RVCollectable *) sb);
  }
  sb->ResetTimer();
}

  
/////////////////////////////////////////////////////////////////
///                        CWLStatGenSlice
/////////////////////////////////////////////////////////////////

CWLStatGenSlice::CWLStatGenSlice( int slice, double endInterval, CWLStatGenMain *pGen, bool OwnDict ) : 
  RVHashDictionary(33),
  sliceNum(slice),
  endTime(endInterval),
  count(0),
  flagUse(true),
	ownDict(OwnDict),
  pGenerator(pGen)
{
  countMutex = new RVTRecursiveLock<RVMutexLock>;
}

CWLStatGenSlice::~CWLStatGenSlice() { 
  if (ownDict) clearAndDestroy(); 
  delete countMutex;
}


CWLStatGenMeasurementDictionary * CWLStatGenSlice::DumpSlice( RVCString &str)
{
	RVHashDictionaryIterator next((RVHashDictionary &) *this);
	CWLStatGenMeasurementDictionary *tmp = new CWLStatGenMeasurementDictionary(NULL);
	while (next())
	{
		RVCollectableInt *k= (RVCollectableInt *) next.key();
		CWLStatGenMeasurementDictionary *pD = (CWLStatGenMeasurementDictionary *) next.value();
		tmp->AddValuesOfDict(pD,dumpSlice);
	}
	tmp->DumpDict(str);
	char c=1;
	//char buf[2048];
  RVCString buf;
  RVCString num;
  TVPDoubleToString(endTime-pGenerator->GetBeginSystem(), num, 3);
  buf.append(num);
  buf.append(c);
  TVPIntToString(sliceNum, num);
  buf.append(num);
  buf.append(c);
	//sprintf(buf, "%.18f%c" 
	//						 "%d%c",
	//						 endTime-pGenerator->GetBeginSystem(),c,
	//						 sliceNum,c);							 
	str.prepend(buf);
	return (tmp);
}


void CWLStatGenSlice::ResetLasts()
{
	RVHashDictionaryIterator next((RVHashDictionary &) *this);
	while (next())
	{
		CWLStatGenMeasurementDictionary *pD = (CWLStatGenMeasurementDictionary *) next.value();
		if (pD!=NULL)	pD->ResetLasts();
	}
}
 

void CWLStatGenSlice::AddDictionarysSlice( CWLStatGenSlice *pSliceToAdd )
{
	// The function used to handle the updating the history of bool
	
	// Adding one Container Slice to Another Container Slice, 
	// Go over all the dictionary in the pSliceToadd Dictionry. 
	// If exist at the slice then add to the exist dictionary the measurementurments at the pSliceToAdd dictionary 
	// If not exist add this dictionary to the Container Slice
	
	CWLStatGenMeasurementDictionary *dicFind;
	RVHashDictionaryIterator next((RVHashDictionary &) *pSliceToAdd);
	while (next()!=NULL)
	{
		RVCollectableInt *k = (RVCollectableInt *) next.key();
		CWLStatGenMeasurementDictionary *pD = (CWLStatGenMeasurementDictionary *) next.value();
		dicFind = (CWLStatGenMeasurementDictionary *) findValue((RVCollectable *) k);
		if ( dicFind == NULL) 
		{
					dicFind=new CWLStatGenMeasurementDictionary(NULL);	
					insertKeyAndValue((RVCollectable  *) new RVCollectableInt(k->value()),(RVCollectable *) dicFind);
		}	
      pD->FlushStatsCache();
			dicFind->AddValuesOfDict(pD,addSlice);
	}
		
	// Updating the details
	sliceNum=pSliceToAdd->GetSliceNum();
	endTime=pSliceToAdd->GetEndTimeSlice();
}
		 
void CWLStatGenSlice::AddDictionary(CWLStatGenMeasurementDictionary *pDic, int clientId)
{
  RVTRecursiveLock<RVMutexLock>::LockGuard lock(*countMutex);
	count++; 
  insertKeyAndValue((RVCollectable *) new RVCollectableInt(clientId),(RVCollectable *) pDic);
}


int CWLStatGenSlice::GetCount()
{
//	int tmp;
//	countMutex.Lock();
//	tmp=count;
//	countMutex.Unlock();
  RVTRecursiveLock<RVMutexLock>::LockGuard lock(*countMutex);
	return count;
}

void CWLStatGenSlice::DecCount()
{
  RVTRecursiveLock<RVMutexLock>::LockGuard lock(*countMutex);
	if (count > 0) count--; 
}

void CWLStatGenSlice::ReleaseDicts(){
  RVCriticalSection::LockGuard lock(*pGenerator->newDictMutex);
	RVHashDictionaryIterator next((RVHashDictionary &) *this);
	while (next())
	{
		CWLStatGenMeasurementDictionary *pD = (CWLStatGenMeasurementDictionary *) next.value();
    if (pD && pD->GetClient())	{
       pD->GetClient()->GetNewDictionary();
    }
	}

}

/////////////////////////////////////////////////////////////////////
//									CWLStatGenMain
////////////////////////////////////////////////////////////////////


CWLStatGenMain::CWLStatGenMain(const char *agenda) :
  agendaName(agenda)
{ 
  sliceMutex = new RVTRecursiveLock<RVMutexLock>;
  pNewestSlice = NULL;

	freeze=false;
	terminated=false;
  terminationInProgress=false;
  total=NULL;
  historySlice = NULL;
  finishPack.Set();
  finishGarbage.Set();

  newDictMutex = new RVCriticalSection;

  TVEnvContainerClass tvpc;
  const char* sStatsCacheSize = tvpc.TVEGetEnv("LGStatsCacheSize");
  if (sStatsCacheSize!=NULL)
    m_stat_cache_size = atoi(sStatsCacheSize);
  else
    m_stat_cache_size = STAT_STORE_SIZE;

  if (m_stat_cache_size > STAT_STORE_SIZE) m_stat_cache_size = STAT_STORE_SIZE;
  WlTraceMsg ("Stats cache size = %d", m_stat_cache_size);

  tvpc.reset();
  const char* sStatsCacheFlushInterval = tvpc.TVEGetEnv("LGStatsCacheFlushInterval");
  if (sStatsCacheFlushInterval!=NULL)
    m_flush_interval = atof(sStatsCacheFlushInterval);
  else
    m_flush_interval = 5.0;

  if (m_flush_interval > 5.0) m_flush_interval = 5.0;
  WlTraceMsg ("Stats cache flush interval = %f", m_flush_interval);
  
}

CWLStatGenMain::~CWLStatGenMain()
{
  if ( historySlice != NULL) {
    delete  historySlice; 
    historySlice = NULL;
  }
  if ( total != NULL) {
    delete total;
    total = NULL;
  }
  if ( pNewestSlice != NULL) {
    delete pNewestSlice;
    pNewestSlice = NULL;
  }
  delete sliceMutex;
  delete newDictMutex;
}

void CWLStatGenMain::Start( double BeginTime, double sliceSizeResolution)
{
	//terminated=false;

	systemBeginRunTime=BeginTime;
	sliceSize=sliceSizeResolution;
  //
  // Insert an empty start slice
  sliceNum=1;
  deltaEnd = sliceSize;
	pNewestSlice = new CWLStatGenSlice(sliceNum, systemBeginRunTime+sliceSize, this);
  historySlice = new CWLStatGenSlice(0, 0, this); 
  
  RVTRecursiveLock<RVMutexLock>::LockGuard lock(*sliceMutex);
  AddSlice(pNewestSlice, sliceNum);
}
 
void CWLStatGenMain::Freeze()
{
	freeze=true;
}

void CWLStatGenMain::DeFrost(double freeze_time)
{
	systemBeginRunTime+=freeze_time;
	freeze=false;
}


void CWLStatGenMain::Terminate()
{
  if( terminated )
    return;

  terminationInProgress = true;

  packQueue.putEvent(pNewestSlice);
  packQueue.release();
  terminated=true;
  garbageQueue.release();
}

void CWLStatGenMain::WaitForTerminateToEnd()
{
  if( !terminated )
    return;

	finishPack.Wait();
  finishGarbage.Wait();
}

CWLStatGenMeasurementDictionary* CWLStatGenMain::GetNewDictionary(int clientId, CWLStatGenMeasurementDictionary* pOldDictionary)
{
  if (pOldDictionary && pOldDictionary->GetSlice()) {
    pOldDictionary->GetSlice()->DecCount();
  }

  //
  // Changed in order to send a slice (even empty) in "No Data" case.
  // Yahali 9-Apr-2000
  double curtime = TVPSysTime();
  double nextEndTime = pNewestSlice->GetEndTimeSlice();
  RVTRecursiveLock<RVMutexLock>::LockGuard lock(*sliceMutex);
  if (nextEndTime <= curtime) {
    nextEndTime = pNewestSlice->GetEndTimeSlice();
    curtime = TVPSysTime();
    while (nextEndTime <= curtime) {
      nextEndTime += sliceSize;
      deltaEnd += sliceSize;
      sliceNum++;
      packQueue.putEvent(pNewestSlice);
      pNewestSlice = new CWLStatGenSlice(sliceNum, nextEndTime, this);
      WlTraceMsg("CWLStatGenMain::GetNewDictionary, AddSlice (Time: %f, SliceTime: %f)\n",TVPSysTime()-systemBeginRunTime, nextEndTime-systemBeginRunTime);
      AddSlice(pNewestSlice ,sliceNum);
    }
  }
  CWLStatGenMeasurementDictionary *newDict = new CWLStatGenMeasurementDictionary(pNewestSlice);

  pNewestSlice->AddDictionary(newDict, clientId);

  return newDict;
}


void CWLStatGenMain::AddSlice(CWLStatGenSlice *pSlice, int sliceNum)
{
	sliceContainer.insertKeyAndValue(
    (RVCollectable *) new RVCollectableInt(sliceNum), 
    (RVCollectable *) pSlice
  ); 
}


void CWLStatGenMain::RemoveSlice(CWLStatGenSlice *pSlice)
{
  RVCollectableInt k(pSlice->GetSliceNum());
  RVTRecursiveLock<RVMutexLock>::LockGuard lock(*sliceMutex);
	sliceContainer.removeAndDestroy((RVCollectable *) &k);
}

void CWLStatGenMain::doPackWork()	
{
  TVPManualEvent::SetGuard PackGuard(finishPack);
  
  try {
    CWLStatGenSlice *slice;
    while(slice = WaitForSlice())
      SendSlice(slice);
  } catch (...) {
    throw;
  }

  return;
}	

CWLStatGenSlice *CWLStatGenMain::WaitForSlice()
{
	RVCollectable *slice = NULL;
  
	while ( !terminated )
  {
    TVPEventQueue::WaitReturnType ret;
	  ret = packQueue.waitEvent(slice, 5000);
    switch (ret) {
      case TVPEventQueue::Success :
        return (CWLStatGenSlice*)slice;

      case TVPEventQueue::Released :
      case TVPEventQueue::Failed :
        return NULL;

      case TVPEventQueue::TimedOut :
      default:
        break; 
    }
	}	
	
  return NULL;
}

void CWLStatGenMain::SendSlice( CWLStatGenSlice *slice )  
{ 
	RVCString string;
	CWLStatGenMeasurementDictionary *tmp;

	slice->NoUse();							// Don't use the Slice
	historySlice->AddDictionarysSlice(slice);

	tmp=total;
 	total=historySlice->DumpSlice(string);

	// Go over slice and call CalculateExtraStatisticsForCurrentClientDictionary
	RVHashDictionaryIterator next((RVHashDictionary &) *historySlice);
	while (next())
	{
		RVCollectableInt *k= (RVCollectableInt *) next.key();
		CWLStatGenMeasurementDictionary *pD = (CWLStatGenMeasurementDictionary *) next.value();
		CalculateExtraStatisticsForCurrentClientDictionary( pD, *k, slice->GetEndTimeSlice() );
	}
	
	CalculateExtraStatisticsForDictionary( total, sliceSize , slice->GetEndTimeSlice() - GetBeginSystem());

	string="";
	char c=1;
	total->DumpDict(string);
	//char buffer[2048];
  RVCString buf;
  RVCString num;
  double sliceTime = slice->GetEndTimeSlice() - GetBeginSystem(); 
  TVPDoubleToString(sliceTime, num, 3);
  buf.append(num);
  buf.append(c);
  TVPIntToString(slice->GetSliceNum(), num);
  buf.append(num);
  buf.append(c);
	//sprintf(buffer, "%.18f%c" 
	//						 "%d%c",
	//						 slice->GetEndTimeSlice() - GetBeginSystem(),
	//						 c,
	//						 slice->GetSliceNum(),
	//						 c);							 
	string.prepend(buf);

	if (tmp!=NULL) delete tmp;

	//char buf[2048];
	//sprintf(buf, "%.18f%c",sliceSize, c);	
  TVPDoubleToString(sliceSize, num, 3);
  buf = num;
  buf.append(c);
	string.prepend(buf);
	string.prepend(c);						// Adding the nameof the Agenda
	string.prepend(GetAgendaName());
	historySlice->ResetLasts();

	SendDictionaryString(string, sliceTime, GetAgendaName());
	 
	garbageQueue.putEvent(slice);
}

void CWLStatGenMain::doGarbageWork() 
{
  TVPManualEvent::SetGuard GarbageGuard(finishGarbage);
  CWLStatGenClient dummyClient(this, 1000001);

  try {
    CWLStatGenSlice *slice; 
    RVCollectable *cslice;
    while( !(terminated && garbageQueue.isEmpty()) ) {
      TVPEventQueue::WaitReturnType ret;
	    ret = garbageQueue.waitEvent(cslice, 5000);
      switch (ret) {
        case TVPEventQueue::Success :
          slice = (CWLStatGenSlice*)cslice;
          if(slice->GetCount()>0) {
            slice->ReleaseDicts();
          }
          RemoveSlice(slice);
          continue;
          break;

        case TVPEventQueue::Released :
        case TVPEventQueue::Failed :
          return;

        case TVPEventQueue::TimedOut :
        default:
          break;
      }
      //printf("CWLStatGenMain::WaitForSlice, before MutexLock (Time: %f)\n",TVPSysTime()-systemBeginRunTime);
      if (pNewestSlice->GetEndTimeSlice()+5 < TVPSysTime() && !terminationInProgress)
      {
        //printf("CWLStatGenMain::doGarbageWork, call dummyClient.GetNewDictionary (Time: %f, SliceTime:%f)\n",TVPSysTime()-systemBeginRunTime,pNewestSlice->GetEndTimeSlice()-systemBeginRunTime);
        WlTraceMsg("CWLStatGenMain::doGarbageWork, call dummyClient.GetNewDictionary (Time: %f, SliceTime:%f)\n",TVPSysTime()-systemBeginRunTime,pNewestSlice->GetEndTimeSlice()-systemBeginRunTime);
        //
        // this will create a new dictionary.
        dummyClient.GetNewDictionary();
        continue;
      }
 	    TVPWaitTime(1000);
    }
  } catch (...) {
    throw;
  }
}


/////////////////////////////////////////////////////////////////////
//									CWLStatGenClient
////////////////////////////////////////////////////////////////////



CWLStatGenClient::CWLStatGenClient(CWLStatGenMain *pGS, int id) : 
  pGenerator(pGS),clientId(id), pCurrentDictionary(NULL)
{
}

CWLStatGenClient::~CWLStatGenClient()
{
  if (pCurrentDictionary) {
    pCurrentDictionary->ReleaseClient();
    pCurrentDictionary=NULL;
  }
}

void CWLStatGenClient::GetNewDictionary()
{
  if (pCurrentDictionary)
    pCurrentDictionary->ReleaseClient();
  pCurrentDictionary = pGenerator->GetNewDictionary(clientId, pCurrentDictionary);
  pCurrentDictionary->SetClient(this);
}

void CWLStatGenClient::ReleaseDictionary()
{
  pCurrentDictionary = NULL;
}

double CWLStatGenClient::GetCurrentEndTime()
{
  return pCurrentDictionary->GetSlice()->GetEndTimeSlice();
}

// Chaim April 2001 - Stats cache
void CWLStatGenClient::FlushStatsCache ()
{
  if (pCurrentDictionary) pCurrentDictionary->FlushStatsCache();
}

void CWLStatGenClient::AddLast (const char *measurement, double v , CWLStatPointTypeEnum::PointType type)
{
  RVCriticalSection::LockGuard lock(*pGenerator->newDictMutex);
  if (!pCurrentDictionary || !pCurrentDictionary->pMyClient || (TVPSysTime() > GetCurrentEndTime())) 
    GetNewDictionary();

	while ( pCurrentDictionary->AddLast(measurement,v,type) == false)
    GetNewDictionary();

}


void CWLStatGenClient::ResetTimer(const char *name)
{
	timingDictionary.ResetTimer(name);
}


double CWLStatGenClient::SendTimer(const char *name)
{
	double time=timingDictionary.SendTimer(name);
  AddLast(name,time,CWLStatPointTypeEnum::Timer);
	return time;
}

