/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// Local Variables:
// mode: c++
// End:
//
// Definition of Mainloop : a server which performs loop and allow
// Registration for input events.
//

#ifndef __TestTalkClientObjImp_H
#define __TestTalkClientObjImp_H

//#include <collstr.h>
#include <collint.h>
#include <slistcolque.h>
#include <hashtab.h>

#ifdef __TV_UNIX__
#define INFINITE 0xFFFFFFFF
#endif

#include "msghdr.h"
#include "netmsg.h"

#include "platform.h"
#include "tvpsync.h"
#include "tvpeventqueue.h"

class CTestTalkClientWindow;
class CTestTalkClientThread;
class TestTalkClientObject;
class CTestTalkClientCommunicator;

class CTestTalkClientQueue : public TVPEventQueue
{
public:
	CTestTalkClientQueue();
	~CTestTalkClientQueue();

	void putMessage(TypedMessageObject* tmo);

	TypedMessageObject* peekMessage();
	TypedMessageObject* waitMessage(DWORD dwTimeout = INFINITE);

	TestTalkMessage* peekTTMessage();
	TestTalkMessage* waitTTMessage(DWORD dwTimeout = INFINITE);
};

class CTestTalkClientObject : public RVCollectableInt {
  friend class CTestTalkClientCommunicator;
private:
  TestTalkClientObject        *m_Client;
  CTestTalkClientCommunicator *m_CTestTalkClientCommunicator;
	RVThread                     m_ReaderThread;
	bool                         m_IsNETTmlUp;
	RVCString                    m_ErrorMsg;
	NetClientData                m_NetClientData;
	int                          m_NetLastStatus;
	CTestTalkClientQueue*        m_MsgQs[LastQType];
	TestTalkMessage             *m_pRetTtm;
  RVCString                    m_WhoAmI;
  unsigned short               m_TestTalkClientsPort;
	int                          m_WaitForAckOnSendTimout;

	TestTalkMessage* peekTTMessage(QType qtype)
	{
		return m_MsgQs[qtype]->peekTTMessage();
	}
	TypedMessageObject* waitTypedMessage(QType qtype, DWORD dwTimeout = INFINITE)
	{
		return m_MsgQs[qtype]->waitMessage(dwTimeout);
	}
	TestTalkMessage* waitTTMessage(QType qtype, DWORD dwTimeout = INFINITE)
	{
		return m_MsgQs[qtype]->waitTTMessage(dwTimeout);
	}

	int sendMessageToTestTalk(TestTalkMessage& ce);

	bool copyFile(long msgId, const char *data, size_t dataSize, const char *toName, const char *toHost, QType toQ);
	bool sendMessage(const char *data, size_t dataSize, const char *toName, const char *toHost, QType toQ, int timeout);
	bool sendMessageNoresponse(const char *data, size_t dataSize, const char *toName, const char *toHost, QType toQ);
	bool sendMessageNowait(long msgId, const char *data, size_t dataSize, const char *toName, const char *toHost, QType toQ);
	const char* waitMessage(QType qtype);

public:
	CTestTalkClientObject(const char* name, TestTalkClientObject *client);
	virtual ~CTestTalkClientObject();

	bool NETInit(const char* tmlInstance, unsigned short port);
	bool NETTurnOn();
	bool NETTurnOff();
	void NETTerminate();

	bool NETCopyFile(long msgId, const char *data, size_t dataSize, const char *addr);
	bool NETSendMultiMessage(const char *data, size_t dataSize, const char* addr);
	bool NETSendMessage(const char *data, size_t dataSize, const char *addr);
	bool NETSendMessageNoresponse(const char *data, size_t dataSize, const char *addr);
	bool NETSendMessageNowait(long msgId, const char *data, size_t dataSize, const char *addr);

	const char* NETFetchMessage();
	const char* NETWaitMessage();
	const char* NETWhoAmI();
	const char* NETLocalHostName();
	int NETGetLastStatus();
	const char* NETGetLastErrorMessage();

  void NewMessageEvent(TypedMessageObject* tmo, short qtype);
};

// **** Macros: **** //
/*
#define tmlnetError(em)                   \
{                                         \
	strcpy(m_ErrorMsg,GetTmlNetError(em));          \
	errorPrint((m_ErrorMsg));          \
}

#define tmlnetFatalError(em)              \
{                                         \
	strcpy(m_ErrorMsg,GetTmlNetError(em));          \
	errorPrint((m_ErrorMsg));          \
	m_IsNETTmlUp = false;                     \
	NETTerminate();                         \
}

#define checkNetActiveCond  (!m_IsNETTmlUp)
*/
#define checkNetActive(ret)                        \
{                                                  \
	if (!m_IsNETTmlUp)                                 \
	{                                                \
    m_NetLastStatus = NETERROR_NETISDOWN;            \
		return (ret);                                  \
  }                                                \
}

#endif //__TestTalkClientObjImp_H

