/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// UDPSender.h: interface for the UDPSender class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_UDPCLIENT_H__7079ABCA_70F2_11D4_8831_0004AC15A422__INCLUDED_)
#define AFX_UDPCLIENT_H__7079ABCA_70F2_11D4_8831_0004AC15A422__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "socketlib.h"
#include "tvpsync.h"
#include <RVCString.h>
#include <thr.h>

#define NO_FLAGS_SET 0
#define MAXBUFLEN 256


class UDPSender  
{
public:
	UDPSender(void);
	virtual ~UDPSender(void);

	// hostName is a default host that method SendToHost will
	// send it's messages to. This host is 'binded' to the Sender.
	// If NULL the default will be localhost.
	virtual int Init(u_short port, const char* hostName = NULL);

	// Starts a sender thread that sends message in loop.
	// The loop waits the duration time between messages.
	// Start() uses SendToHost() to send the messages, so
	// there is a need to specify the toHostName before 
	// using this method (in Init or UpdateToHost).
	virtual int Start(const char* message, long duration);

	// Stops the sender thread.
	// Used only if Start() was called, else use
	// Terminate() to close net connections. 
	virtual void Stop(void);

	// Terminate net connections.
	void Terminate(void);

	// Sends the buffer to the default host binded to this Sender.
	int  SendToHost(const char* buffer); 

	// Sends the buffer to a given host.
	int  SendTo(const char* hostName, const char* buffer);
	int  UpdateToHost(const char* hostName = NULL);
	bool SenderStatusRunning(void);
	
protected:
	// Activated by calling Start().
	void ThreadLoop(void);

	// Called in a loop from ThreadLoop.
	virtual int Run(void);

protected:
	struct sockaddr_in  m_DestSockAddr;
  SOCKET              m_DestSocket;
	RVCString           m_HostName;
	RVCString           m_MessageToSend;
	RVThreadFunction    m_SenderThread;
	TVPManualEvent			m_SenderThreadEvent;
	u_short							m_SenderPort;
	bool								m_Running;
	bool                m_Initialized;
	long								m_timeVal;
};

#endif // !defined(AFX_UDPCLIENT_H__7079ABCA_70F2_11D4_8831_0004AC15A422__INCLUDED_)

