/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// CEASTestTalkClient.cpp: implementation of the CEASTestTalkClient class.
//
//////////////////////////////////////////////////////////////////////



#include "ceastesttalkclient.h"
#include "associativearray.h"
#include "tracedll.h"
#include "tvenv.h"
#include "tvtables.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEASTestTalkClient::CEASTestTalkClient(void)
{
	m_SenderPort = 0;
	TVEnvContainerClass tvpc;
	m_ConfigFilePath = tvpc.TVEGetEnv("TV_BIN"); 
	m_ConfigFilePath += "\\";
	m_ConfigFilePath += PING_CONFIGURATION_FILE;
}

CEASTestTalkClient::~CEASTestTalkClient(void)
{
}

int CEASTestTalkClient::LoadConfigFile(void)
{
	TableHandle configTable = TVSCreateTable();

	if(!TVSLoadTable(m_ConfigFilePath.data(), configTable) ||
									 !TVSGetValue(configTable, "REPLY_HOST")) {
		return 0;
	}
	
	m_EASTTServerName = TVSGetValue(configTable, "REPLY_HOST");
	m_SenderPort      = atoi(TVSGetValue(configTable, "REPLY_PORT"));  
	m_timeVal         = atoi(TVSGetValue(configTable, "PONG_DELAY_DURATION"));
	m_EASTTClientName = TVSGetValue(configTable, "MY_NAME");

	TVSDestroyTable(configTable);
	return 1;
}

int CEASTestTalkClient::UpdateConfigFile(void)
{
	char buffer[MAXBUFLEN*4];
	CAssociativeArray initArray;

	if(!m_Receiver.ReceiveFrom(buffer, MAXBUFLEN*4))
		return 0;

	try {
		initArray.FromString(buffer);

		m_EASTTServerName = initArray.GetValueOfParameter("$RHOST");
		m_SenderPort      = initArray.GetLongValueOfParameter("$RPORT");
		m_timeVal         = initArray.GetLongValueOfParameter("$TIME");
		m_EASTTClientName = initArray.GetValueOfParameter("$NAME");

		TableHandle configTable = TVSCreateTable();
		TVSAssignValue(configTable, "REPLY_HOST", m_EASTTServerName.data());
		TVSAssignValue(configTable, "REPLY_PORT", initArray.GetValueOfParameter("$RPORT"));
		TVSAssignValue(configTable, "PONG_DELAY_DURATION", initArray.GetValueOfParameter("$TIME"));
		TVSAssignValue(configTable, "MY_NAME", m_EASTTClientName.data());

		fclose( fopen(m_ConfigFilePath.data(), "w") );

		TVSSaveTable(m_ConfigFilePath.data(), configTable);
		TVSDestroyTable(configTable);
	}
	catch(...) {
		return 0;
	}
	return 1;
}

int CEASTestTalkClient::Init(u_short listenPort)
{
	if(m_Receiver.Init(listenPort)) {
		if( !LoadConfigFile() && !UpdateConfigFile()) {
			WlTraceMsg("ERROR: CNetKernel::Run - could not load udp ping client");
			return 0;
		}
	}
	else {
		WlTraceMsg("ERROR: CNetKernel::Run - UDPReceiver::Init failed");
		return 0;
	}
	
	if( !UDPSender::Init(m_SenderPort, m_EASTTServerName.data()) ) {
		WlTraceMsg("ERROR: CNetKernel::Run - UDPSender::Init failed");
		return 0;
	}
	return 1;
}

void CEASTestTalkClient::Start(void)
{
	UDPSender::Start(m_EASTTClientName, m_timeVal);
}

void CEASTestTalkClient::Stop(void)
{
	UDPSender::Stop();
	m_Receiver.Terminate();
}

int CEASTestTalkClient::Run(void)
{
	if(m_Receiver.SocketReadyForRead()) {
		UpdateConfigFile();
		UDPSender::Terminate();
		m_Receiver.Terminate();
		TVEnvContainerClass tvpc;
		if(!Init( atoi(tvpc.TVEGetEnv("UDP_CLIENT_PORT")) )) {
			WlTraceMsg("CEASTestTalkClient::Run - failed.");
			m_Receiver.Terminate();
			return 0;
		}
	}
	if( !SendToHost(m_MessageToSend.data()) ) {		
		WlTraceMsg("ERROR: CEASTestTalkClient::Run - failed sending message(%s)", m_EASTTClientName);
		return 0;
	}
	return 1;
}

const char* CEASTestTalkClient::GetClientName(void)
{
	return m_EASTTClientName.data();
}

