/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <fstream.h>
#include <RVCString.h>
#include <RVCTokenizer.h>
#include <collstr.h>
#include <collint.h>



#include "tracedll.h"

#include "platform.h"
#include "tvpsync.h"
#include "tvenv.h"

//Galtech
#include "gt_unicode.h"

//#ifndef __TV_UNIX__
#include "supportinformationgenerator.h"
#include "tvver.h"
//#endif
#include "eastesttalkclient.h"
#include "associativearray.h"
#include "testtalktoolkit.h"
#include "netkernel.h"
#ifdef __TV_UNIX__
#include <errno.h>
#define GetLastError() errno
#endif

#ifdef __TV_WINDOWS__
#include <windows.h>
#endif


/////////////////////////////////////////////////////



CNetKernel::CNetKernel(bool isTestTalkService) :
  CNetMessageHandler("theTml"), seqNo(1)
{
  m_SenderName = "theTml";
  m_EASPingClient = NULL;
	m_RFSService    = NULL;
	m_IsTestTalkService = isTestTalkService;

  GetDefaultValues();
}

CNetKernel::~CNetKernel()
{
	if(m_EASPingClient) {
		m_EASPingClient->Stop();
		delete m_EASPingClient;
	}

	if(m_RFSService) {
		delete m_RFSService;
	}
}

void CNetKernel::Start(unsigned short port)
{
  m_TestTalkPort = port;
  #if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)
    m_Thread = rvMakeThreadFunction(
                  *this,                        // instance
                  &CNetKernel::Run              // member function
                );
  #else
    m_Thread = rvMakeThreadFunctionM(
                CNetKernel                      // class
                *this,                          // instance
                void,                           // return value
                &CNetKernel::Run                // member function
              );
  #endif
  m_Thread.start();
}

void CNetKernel::Run()
{
	Init(m_TestTalkPort);

	if( !m_RFSService )
	{
	  m_RFSService = new RFSService(this);
	}

	TVEnvContainerClass tvpc;
	const char* udpPort = tvpc.TVEGetEnv("UDP_CLIENT_PORT");
	
	if(udpPort) {
		u_short port    = atoi(udpPort);
		m_EASPingClient = new EASTestTalkClient();

		if( m_EASPingClient->Init(port) )
			m_EASPingClient->Start();
		else
			WlTraceMsg("CNetKernel::Run - could not start EAS UDP client");
	}
}

void CNetKernel::Stop()
{
  Terminate();
	if(m_EASPingClient)
		m_EASPingClient->Stop();
}

void CNetKernel::GetDefaultValues()
{
  TVEnvContainerClass tvpc;

  
  m_TimeBeforeForceKill = 60; // real value should get from the console


  if(tvpc.TVEGetEnv("GENERATOR_FORCE_KILL_TIMEOUT_FACTOR")) {
    tvpc.reset();
    m_TimeBeforeForceKillProductFactor = atoi(tvpc.TVEGetEnv("GENERATOR_FORCE_KILL_TIMEOUT_FACTOR"));
  }
  else {
    m_TimeBeforeForceKillProductFactor = 1;
  }

  tvpc.reset();
  if(tvpc.TVEGetEnv("KILLER_THREAD_WAKE_UP_TIMEOUT")) {
    tvpc.reset();
    m_KillerThreadWakeUpTimeOut = atol(tvpc.TVEGetEnv("KILLER_THREAD_WAKE_UP_TIMEOUT"));
  }
  else {
    m_KillerThreadWakeUpTimeOut  = 60000;
  }
}

bool CNetKernel::CreateLoaderProcess(CAssociativeArray& AssoArrayCommand)
{
	bool retVal = true;
	
    //
	// Create the LG
	if ( AssoArrayCommand.KeyExists("$CREATELOADGENERATOR") )
  {
		//
		// Restart (Tanya):
		// If create called in result to restart LG command we need to make sure
		// the old LG is down and not trust the automatic algo'
		if ( AssoArrayCommand.KeyExists("$RESTARTGENERATOR") )
		{
			RVCString oldGeneratorUID = AssoArrayCommand.GetValueOfParameter("$RESTARTGENERATOR");
			KillProcesses(oldGeneratorUID);
			WlTraceMsg("CNetKernel::MessageHandler receive CREATELOADGENERATOR with RESTARTGENERATOR = %s", oldGeneratorUID );
		}
		
    CreateLoadGenerator(AssoArrayCommand);
		WlTraceMsg("CNetKernel::MessageHandler receive message : %s.",AssoArrayCommand.ToString());
  }
	else if ( AssoArrayCommand.KeyExists("$CREATELOADGENERATORSLAVE") )
	{
		CreateLoadGeneratorSlave(AssoArrayCommand);
		WlTraceMsg("CNetKernel::MessageHandler receive message : %s.",AssoArrayCommand.ToString());
	}
	else if ( AssoArrayCommand.KeyExists("$CREATELOADGENERATOR_50") )
	{
		CreateWebLOAD50LoadGenerator(AssoArrayCommand);
		WlTraceMsg("CNetKernel::MessageHandler receive message : %s.",AssoArrayCommand.ToString());
	}
	else
	{
		retVal = false;
	}
	
	
	return retVal;
}

void CNetKernel::HandleProcessCreationFailure(CAssociativeArray& origCreationMessage, const char* filePath, int errorCode)
{
	char sErrorCode[10];
  WlTraceMsg("TVPCreateProcess failed to create process: Error %d ", errorCode);

	CAssociativeArray errorReply;
	
	errorReply.AddCouple("$CREATEPROCESSFAILED", filePath);
	errorReply.AddCouple("$ERRORCODE", itoa(errorCode, sErrorCode, 10));
	errorReply.AddCouple("$ERRORSTRING", TestTalkToolKit::GetErrorString(errorCode));
		
  SendMessage(errorReply, origCreationMessage["$FROM"], "");
}

void CNetKernel::MessageHandler( CAssociativeArray& AssoArrayCommand )
{
	try
	{
		if ( CreateLoaderProcess(AssoArrayCommand) )
    {
			WlTraceMsg("CNetKernel::MessageHandler receive message : %s.",AssoArrayCommand.ToString());
    }
		else if( AssoArrayCommand.KeyExists("$TTPING") )
		{
			CAssociativeArray pingReply;

			pingReply.AddCouple("$TTPINGREPLY", AssoArrayCommand.GetValueOfParameter("$TTPING"));
#ifdef __TV_WINDOWS__
			if(AssoArrayCommand.KeyExists("$TTVERSION"))
				pingReply.AddCouple("$TTVERSION", TVVProductVersionName());
#else
			if(AssoArrayCommand.KeyExists("$TTVERSION"))
			{
				TVEnvContainerClass tvpc;
				pingReply.AddCouple("$TTVERSION", tvpc.TVEGetEnv("PRODUCT_VERSION_NAME"));
			}
#endif // __TV_WINDOWS__
			if(AssoArrayCommand.KeyExists("$TTSYSTEMINFO")) {
				CAssociativeArray infoDB;
				CSupportInformationGenerator infoGenerator;

				infoGenerator.CreateInformationDB(infoDB);

				pingReply.AddCouple("$TTSYSTEMINFO", infoDB.ToString());
			}

			SendMessage(pingReply, AssoArrayCommand.GetValueOfParameter("$FROM"), "");
		}
		else if ( AssoArrayCommand.KeyExists("$CREATEWEBLOAD") )
		{
#ifdef __TV_WINDOWS__
			CreateWebLOAD(AssoArrayCommand);
#endif // __TV_WINDOWS__
			WlTraceMsg("CNetKernel::MessageHandler receive message : %s.",AssoArrayCommand.ToString());
		}
		else if ( AssoArrayCommand.KeyExists("$CREATEWEBFT") )
		{
#ifdef __TV_WINDOWS__
			CreateWebFT(AssoArrayCommand);
#endif // __TV_WINDOWS__
			WlTraceMsg("CNetKernel::MessageHandler receive message : %s.",AssoArrayCommand.ToString());
		}
    else if ( AssoArrayCommand.KeyExists("$KILLLOADGENERATOR") )
		{
      long killTimeout;
			CAssociativeArray* Msg = new CAssociativeArray();
			
			Msg->AddCouple("$GENERTORUID", AssoArrayCommand["$MONITORUID"]);
			Msg->AddCouple("$KILLREQUESTTIME",TVPSysTime());
			
      if( AssoArrayCommand.KeyExists("$TERMINATESESSIONTIMEOUT") ) {
        killTimeout = AssoArrayCommand.GetLongValueOfParameter("$TERMINATESESSIONTIMEOUT");
				
        if( killTimeout < 60 ) {
          killTimeout = 60;
        }
      }
      else {
        GetDefaultValues();
        killTimeout = m_TimeBeforeForceKill;
      }
			
      killTimeout *= m_TimeBeforeForceKillProductFactor;
      Msg->AddCouple("$KILLTIMEOUT", killTimeout);
			
			m_queueKillRequest.insert(Msg);
			WlTraceMsg("CNetKernel::MessageHandler receive message : %s.",AssoArrayCommand.ToString());
		}		
    else if ( AssoArrayCommand.KeyExists( "$KILLPROCESS" ) )
		{
			bool bRet = false;
			long waitBeforeKill = 0;
			const char* status = "Failed";
			CAssociativeArray retMessage;

			try
			{
				waitBeforeKill = AssoArrayCommand.GetLongValueOfParameter("$WAITBEFOREKILL");

				if( waitBeforeKill == 0 )
				{
				//bRet = m_processesHashDictinary.TerminateGroup( RVCString(AssoArrayCommand["$PROCESSUID"]) );
				RVCString tmpStr(AssoArrayCommand["$PROCESSUID"] );
				bRet = m_processesHashDictinary.TerminateGroup( tmpStr );

				if( bRet )
					status = "Ok";
			}
				else
				{
					CAssociativeArray* killMessage = new CAssociativeArray();
					
					killMessage->AddCouple("$FROM", AssoArrayCommand["$FROM"]);
					killMessage->AddCouple("$GENERTORUID", AssoArrayCommand["$PROCESSUID"]);
					killMessage->AddCouple("$KILLREQUESTTIME",TVPSysTime());			
					killMessage->AddCouple("$KILLTIMEOUT", waitBeforeKill);
					
					m_queueKillRequest.insert(killMessage);
					WlTraceMsg("CNetKernel::MessageHandler receive message : %s.",AssoArrayCommand.ToString());
				}
			}
			catch(...)
			{
				bRet = false;
			}
			
			retMessage.AddCouple("$KILLPROCESSSTATUS", status);

			SendMessage(retMessage, AssoArrayCommand["$FROM"], "");
		}
    else if ( AssoArrayCommand.KeyExists( "$SERVERREQUEST" ) )
    {
      RVCString reqestType = AssoArrayCommand["$SERVERREQUEST"];
      if (reqestType == "BECLIENT" || reqestType == "BESERVER")
      {
        iAmClient(AssoArrayCommand);
      }
    }
    else if ( AssoArrayCommand.KeyExists( "$ISTESTTALKSERVICE" ) )
    {
			CAssociativeArray retMessage;
		
			if( m_IsTestTalkService )
			{
				retMessage.AddCouple("$ISTESTTALKSERVICE", "True");
			}
			else
			{
				retMessage.AddCouple("$ISTESTTALKSERVICE", "False");
			}

			SendMessage(retMessage, AssoArrayCommand["$FROM"], "");
		}
    else
    {
#ifndef __TV_solaris__
      WlTraceMsg("CNetKernel::MainLoop : Illegal command (%s)",AssoArrayCommand.ToString());
#endif
#ifdef _DEBUG
      CAssociativeArray Message;
      Message.AddCouple("$SERVERREQUEST","BESERVER");\
      Message.AddCouple("$MYNAME","1@localhost");\
      SendMessageNowait(Message, "1@localhost", "Bla Bla");
      SendMessageNowait(Message, "2@localhost", "Bla Bla");
#endif
      //SendMessage( WhoAmI(), "Wrong command: " + AssoArrayString
    }

  } catch (WLErrorException e) {
#ifndef __TV_solaris__
	  WlTraceMsg("CNetKernel::MainLoop : Exception: %s",e.GetErrorMsg());
#endif
  } catch (...) {
#ifndef __TV_solaris__
      WlTraceMsg("CNetKernel::MainLoop : Unknown exception");
#endif
    throw;
  }
}

void CNetKernel::SendMessageErrorHandler( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, bool status)
{
}

void CNetKernel::iAmClient(CAssociativeArray& Message)
{
  RVCString serverName = Message["$FROM"];
  CAssociativeArray OutMessage;

  OutMessage.AddCouple("$HOSTSTATUS", "CONNECTED");
  OutMessage.AddCouple("$MONITOR", serverName);
  OutMessage.AddCouple("$MYNAME", Message["$MYNAME"]);
#ifdef __TV_WINDOWS__
  const char* productVersionStr = TVVProductVersion();
#else
  // ATTENTION ATTENTION ATTENTION
  // on unix platforms we will take ProductVersion from webload.ini and NOT from tvcti.dll
  // since module tvver LIBRARY is not beeing using by unix platform
  // I think that we should use the same mechasim for BOTH windows and unix
  // but since WebLoad 5.0 is out we will have to do it for in WebLoad 5.5
  // ATTENTION ATTENTION ATTENTION
  TVEnvContainerClass tvpc;
  const char* productVersionStr = tvpc.TVEGetEnv("PRODUCT_VERSION");
#endif // __TV_WINDOWS__
  OutMessage.AddCouple("$VERSION", productVersionStr); 

  SendMessage(OutMessage, serverName, "Error sending host status message.");
}


void CNetKernel::CreateWebLOAD50LoadGenerator(CAssociativeArray& Message)
{
  CreateLoadGenerator(Message, true);
  
  /*
  const char* lgProgram = NULL;
  
  RVCString GeneratorType = Message["$LOADTYPE"];
  PROCESS_HANDLE pHandle;
  
  RVCString typStr = "\"";
  typStr += Message["$TYPE"];
  typStr += "\"";
  
  RVCString realNameStr = "\"";
  realNameStr += Message["$REALNAME"];
  //Galtech - Eyal
  char ansirealName[_MAX_PATH];
  UTF8ToANSI(GetACP(), realNameStr,  -1, ansirealName, _MAX_PATH);
  realNameStr = ansirealName;
  realNameStr += "\"";
  
  RVCString monitorStr = "\"";
  monitorStr += Message["$MONITOR"];
  monitorStr += "\"";
  

  char seqNoStr[10];
  (void)sprintf(seqNoStr,"%d",seqNo++);
  
  if (GeneratorType == "Java") 
{  
    pHandle = TVPCreateProcess("java", 
      "JavaLoad.LoadGenerator",
      typStr.data(),
      realNameStr.data(),
      monitorStr.data(),
      seqNoStr,
      NULL);
    return;
  }

  if (GeneratorType == "Web" ||
    GeneratorType == "Emulation" ||
    GeneratorType == "JavaScript") 
  {
    TVEnvContainerClass tvpc;
    lgProgram = tvpc.TVEGetEnv("WL50_LOAD_ENGINE_PROGRAM");
    
    if (!lgProgram) 
    {
      return;//throw Error;
    }
  }
  
  pHandle = TVPCreateProcess(lgProgram, 
                             monitorStr.data(),
                             typStr.data(),
                             realNameStr.data(),
                             seqNoStr,
                             NULL);
  if(!pHandle) 
  {
    WlTraceMsg("Error %d while running (%s %s %s %s %s)", GetLastError(), lgProgram,typStr.data(),realNameStr.data(),monitorStr.data(),seqNoStr);
    
    //
    // report error message
    CAssociativeArray ErrorMessage;
    RVCString errorMsg = "Error running ";
    errorMsg += lgProgram;
    ErrorMessage.AddCouple("$REALNAMERUNNINGERROR", errorMsg.data());
    ErrorMessage.AddCouple("$REALNAME", Message["$REALNAME"]);
    SendMessage(ErrorMessage, Message["$MONITOR"], "Error Sendine $REALNAMERUNNINGERROR Message.");
  } 
  else 
  {//the process created successfully
    RVCString keyProcessGroupManager ( Message["$MONITORUID"] );
    ProcessGroup* retVal = m_processesHashDictinary.AddProcessToGroup(keyProcessGroupManager, pHandle, PG_VER_LG_50);

    if( retVal == NULL ) 
    { 
      WlTraceMsg("Error : in function 'CNetKernel::CreateLoadGenerator'. failed to insert handle to table");
    }
    else 
    {
      retVal->processGroupVersion = PG_VER_LG_50;
    }
  }
  */
}


void CNetKernel::CreateLoadGenerator(CAssociativeArray& Message, bool IsProtocol)
{
	int lgProgramVersion  = 0;
  RVCString sLGProgram;
  RVCString sAdditionalPath;

  static bool pathIsSet = false;

  RVCString GeneratorType = Message["$LOADTYPE"];
  PROCESS_HANDLE pHandle;

  RVCString typStr = "\"";
  typStr += Message["$TYPE"];
  typStr += "\"";
  
  RVCString realNameStr = "\"";
  realNameStr += Message["$REALNAME"];
  //Galtech
  char ansirealName[_MAX_PATH];
  UTF8ToANSI(GetACP(), realNameStr,  -1, ansirealName, _MAX_PATH);
  realNameStr = ansirealName;
  realNameStr += "\"";
  
  RVCString monitorStr = "\"";
  monitorStr += Message["$MONITOR"];
  monitorStr += "\"";
  
  char seqNoStr[10];
  sprintf(seqNoStr,"%d",seqNo++);

  if (GeneratorType == "Web" ||
              GeneratorType == "Emulation" ||
              GeneratorType == "JavaScript") 
  {
	  TVEnvContainerClass tvpc;

	  sLGProgram = tvpc.TVEGetEnv("WL_LOAD_ENGINE_PROGRAM");
		lgProgramVersion = PG_VER_LG_GENERAL;
  }
  else if(GeneratorType == "GUI")
  {
		TVEnvContainerClass tvpc;
		
    sLGProgram = tvpc.TVEGetEnv("WL_GUI_LOAD_ENGINE_PROGRAM");
		lgProgramVersion = PG_VER_GUI_LG;
		
  }
  else if(GeneratorType == "DDoS")
	{
	  TVEnvContainerClass tvpc;

    sLGProgram = tvpc.TVEGetEnv("WL_DDOS_LOAD_ENGINE_PROGRAM");
		lgProgramVersion = PG_VER_DDOS_LG;

  }
  else if(GeneratorType == "WLRPROXY")
	{
	  TVEnvContainerClass tvpc;

    sLGProgram = tvpc.TVEGetEnv("WL_RPROXY_LOAD_ENGINE_PROGRAM");
		lgProgramVersion = PG_VER_RPROXY_LG;

  }

  if (pathIsSet == false)
  {
    pathIsSet = true;
    const char *path = getenv("PATH");
    char newpath[2048];
    sprintf(newpath,"WLBASEPATH=%s",path);
    putenv(newpath);
  }

  if (GeneratorType == "WLRPROXY") 
  {
    TVEnvContainerClass tvpc;
    sAdditionalPath = tvpc.TVEGetEnv("WL_PROXY_ADDITIONAL_PATH");
    const char *path = getenv("WLBASEPATH");
    char newpath[2048];
    sprintf(newpath,"PATH=%s;%s",sAdditionalPath.data(),path);
    putenv(newpath);
  } 
  else
  {
    TVEnvContainerClass tvpc;
    sAdditionalPath = tvpc.TVEGetEnv("WL_ADDITIONAL_PATH");
    const char *path = getenv("WLBASEPATH");
    char newpath[2048];
    sprintf(newpath,"PATH=%s;%s",sAdditionalPath.data(),path);
    putenv(newpath);
  }

  // @@Migration - special code for 5.0
  if (IsProtocol){
	  TVEnvContainerClass tvpc;
    sLGProgram = tvpc.TVEGetEnv("WL_LOAD_ENGINE_PROGRAM");
    lgProgramVersion = PG_VER_LG_50;
    RVCString dummy = "protocol";
    pHandle = TVPCreateProcess(sLGProgram.data(), 
			         monitorStr.data(),
			         typStr.data(),
			         realNameStr.data(),
			         seqNoStr,
               dummy,
			         NULL);
  }
  else {
  pHandle = TVPCreateProcess(sLGProgram.data(), 
			       monitorStr.data(),
			       typStr.data(),
			       realNameStr.data(),
			       seqNoStr,
			       NULL);
  }

  if (!pHandle) 
  {
    WlTraceMsg("Error %d while running (%s %s %s %s %s)", GetLastError(),
	 sLGProgram.data(),typStr.data(),realNameStr.data(),monitorStr.data(),seqNoStr);

    //
    // report error message
//#ifndef _DEBUG
    CAssociativeArray ErrorMessage;
    RVCString errorMsg = "Error running ";
    errorMsg += sLGProgram.data();
    ErrorMessage.AddCouple("$REALNAMERUNNINGERROR", errorMsg.data());
    ErrorMessage.AddCouple("$REALNAME", Message["$REALNAME"]);
    SendMessage(ErrorMessage, Message["$MONITOR"], "Error Sendine $REALNAMERUNNINGERROR Message.");
//#endif
  } 
  else 
	{//the process created successfully
    RVCString keyProcessGroupManager ( Message["$MONITORUID"] );
    ProcessGroup* retVal = m_processesHashDictinary.AddProcessToGroup(keyProcessGroupManager, pHandle, lgProgramVersion);

    if( retVal == NULL ) 
		{
      WlTraceMsg("Error : in function 'CNetKernel::CreateLoadGenerator'. failed to insert handle to table");
    }
  }
}

	
bool CNetKernel::CreateLoadGeneratorSlave(CAssociativeArray& Message)
{
  bool Rtn = true;
  ProcessGroup* retVal = NULL;

  RVCString masterTTGroupID;
  PROCESS_HANDLE pHandle;
  TVEnvContainerClass tvpc;


  masterTTGroupID = Message["$MASTERTTGROUPUID"];
  retVal = m_processesHashDictinary.GetProcessGroup(masterTTGroupID);

  const char* lgProgram = NULL;

  // @@Migration - special code for 5.0
  if( retVal && retVal->processGroupVersion == PG_VER_LG_50 ) {
		lgProgram = tvpc.TVEGetEnv("WL_LOAD_ENGINE_PROGRAM");
//    return CreateWebLOAD50LoadGeneratorSlave(Message);
  }

  bool IsBrowserLoad = false;

	if(retVal->processGroupVersion == PG_VER_LG_GENERAL)
	{
		lgProgram = tvpc.TVEGetEnv("WL_LOAD_ENGINE_PROGRAM");
	}
	else if(retVal->processGroupVersion == PG_VER_GUI_LG)
	{
		IsBrowserLoad = true;
		lgProgram = tvpc.TVEGetEnv("WL_GUI_LOAD_ENGINE_PROGRAM");
	}
	else if(retVal->processGroupVersion == PG_VER_DDOS_LG)
	{
		lgProgram = tvpc.TVEGetEnv("WL_DDOS_LOAD_ENGINE_PROGRAM");
	}

  if (!lgProgram) {
    return false;//throw Error;
  }

  RVCString masterAddressStr = "\"";
  masterAddressStr += Message["$MASTERADDRESS"];
  masterAddressStr += "\"";

  RVCString slaveNameStr = "\"";
  slaveNameStr += Message["$SLAVENAME"];
  slaveNameStr += "\"";
  WlTraceMsg("CNetKernel::CreateLoadGeneratorSlave : Executing (%s %s %s)", lgProgram,
	     masterAddressStr.data(),slaveNameStr.data());

  if (IsBrowserLoad) {
	  long SlaveId = Message.GetLongValueOfParameter("$SLAVEID");
	  pHandle = TVPCreateProcessAsUser(SlaveId, lgProgram, masterAddressStr.data(),slaveNameStr.data(),NULL);
  }

  else if( retVal && retVal->processGroupVersion == PG_VER_LG_50 ) {
    RVCString dummy = "protocol";
    pHandle = TVPCreateProcess(lgProgram, masterAddressStr.data(),slaveNameStr.data(),dummy,NULL);
  }
  else{
    pHandle = TVPCreateProcess(lgProgram, masterAddressStr.data(),slaveNameStr.data(),NULL);
  }

  if (!  pHandle ) 
	{
    Rtn = false;
    WlTraceMsg("TVPCreateProcess failed: Error %d ", GetLastError());
				//the theTml failed to create slave.
				//On failure a message will be send to indicate the failure.
				//On success the message will be sent in the main loop of the slave.
				//(logicaly it should be in that function but I want to make sure that this message that indicates the 
				//success creation will get to the loadgeneratorproxy before the message $SLAVEREADY ).
    CAssociativeArray reply;
    reply.AddCouple("$CREATESLAVEREPLY", "FAILEDTOCREATSLAVE");
    SendMessage(reply, Message["$FROM"], "");
  } 
  else 
	{ //the process created successfully.
    masterTTGroupID = Message["$MASTERTTGROUPUID"];
    retVal = m_processesHashDictinary.AddProcessToGroup(masterTTGroupID, pHandle);

    if( retVal == NULL ) { //@@avidan42
      WlTraceMsg("Error : in function 'CNetKernel::CreateLoadGeneratorSlave'. failed to insert handle to table");
    }
  }
  return Rtn;
}

// @@Migration - can be deleted
bool CNetKernel::CreateWebLOAD50LoadGeneratorSlave(CAssociativeArray& Message)
{
  bool Rtn = true;
  RVCString masterTTGroupID;
  PROCESS_HANDLE pHandle;
  TVEnvContainerClass tvpc;

  const char* lgProgram = tvpc.TVEGetEnv("WL50_LOAD_ENGINE_PROGRAM");

  if (!lgProgram) {
    return false;//throw Error;
  }

  RVCString masterAddressStr = "\"";
  masterAddressStr += Message["$MASTERADDRESS"];
  masterAddressStr += "\"";
  
  RVCString slaveNameStr = "\"";
  slaveNameStr += Message["$SLAVENAME"];
  slaveNameStr += "\"";

  WlTraceMsg("CNetKernel::Create_WL50_LoadGeneratorSlave : Executing (%s %s %s)", lgProgram, masterAddressStr.data(),slaveNameStr.data());
  
  pHandle = TVPCreateProcess(lgProgram, masterAddressStr.data(),slaveNameStr.data(),NULL);

  if ( !pHandle ) 
	{
    Rtn = false;
    WlTraceMsg("TVPCreateProcess failed: Error %d ", GetLastError());
				//the theTml failed to create slave.
				//On failure a message will be send to indicate the failure.
				//On success the message will be sent in the main loop of the slave.
				//(logicaly it should be in that function but I want to make sure that this message that indicates the 
				//success creation will get to the loadgeneratorproxy before the message $SLAVEREADY ).
    CAssociativeArray reply;
    reply.AddCouple("$CREATESLAVEREPLY", "FAILEDTOCREATSLAVE");
    SendMessage(reply, Message["$FROM"], "");
    
  } 
  else
	{ //the process created successfully.
    masterTTGroupID = Message["$MASTERTTGROUPUID"];
    ProcessGroup* retVal = m_processesHashDictinary.AddProcessToGroup(masterTTGroupID, pHandle);

    if( retVal == NULL ) { //@@avidan42
      WlTraceMsg("Error : in function 'CNetKernel::CreateLoadGeneratorSlave'. failed to insert handle to table");
    }
  }
  return Rtn;
}

#ifdef __TV_WINDOWS__
void CNetKernel::CreateWebLOAD(CAssociativeArray& creationMessage)
{
	PROCESS_HANDLE pHandle;
	TVEnvContainerClass tvpc;
	const char* webLoadExePath;

  //       
  //Eyal: uc???
	try 
	{
		RVCString runFile = "\"";
		runFile += creationMessage["$RUNFILE"]; // AutoRun mode param: /a
		runFile += "\"";

		RVCString runFileName = "\"";
		runFileName += creationMessage["$RUNFILENAME"];
		runFileName += "\"";
  
		RVCString saveFile = "\"";
		saveFile += creationMessage["$SAVEFILE"];
		saveFile += "\"";
  
		RVCString saveFileName = "\"";
		saveFileName += creationMessage["$SAVEFILENAME"];
		saveFileName += "\"";

		RVCString duration = "\"";
		duration += creationMessage["$DURATION"];
		duration += "\"";

		RVCString durationValue = "\"";
		durationValue += creationMessage["$DURATIONVALUE"];
		durationValue += "\"";

		RVCString inputParams = "\"";
		inputParams += creationMessage["$INPUTPARAMS"];
		inputParams += "\"";

		RVCString inputParamsStr = "\"";
		inputParamsStr += creationMessage["$INPUTPARAMSSTRING"];
		inputParamsStr += "\"";

		RVCString testViewFlag  = "\"";
		testViewFlag += creationMessage["$TESTVIEWFLAG"];
		testViewFlag += "\"";
		
		RVCString testViewName  = "\"";
		testViewName += creationMessage["$TESTVIEWNAME"];
		testViewName += "\"";


		webLoadExePath = tvpc.TVEGetEnv("WEBLOAD_PROGRAM_PATH");

		pHandle = TVPCreateProcess(webLoadExePath, 
			                         runFile, runFileName,
			                         saveFile, saveFileName,
															 duration, durationValue,
															 inputParams, inputParamsStr,
															 testViewFlag, testViewName, 
															 NULL);
	}
	catch(...)
	{
		pHandle = NULL;
	}

	if( !pHandle )
	{
		HandleProcessCreationFailure(creationMessage, webLoadExePath, GetLastError());
	}
	else if( creationMessage.KeyExists("$PROCESSUID") )
	{
    RVCString keyProcessGroupManager( creationMessage["$PROCESSUID"] );
    ProcessGroup* retVal = m_processesHashDictinary.AddProcessToGroup(keyProcessGroupManager, pHandle, PG_VER_WEBLOAD);
		
    if( retVal == NULL ) 
		{
      WlTraceMsg("Error : in function 'CNetKernel::CreateLoadGenerator'. failed to insert handle to table");
    }
	}
}

void CNetKernel::CreateWebFT(CAssociativeArray& creationMessage)
{
	PROCESS_HANDLE pHandle;
	TVEnvContainerClass tvpc;
  const char* webFTExePath;

  //       
  //Eyal: uc???
	try
	{
  RVCString aParam = "\"";
		aParam += creationMessage["$ARPARAM"]; // AutoRun mode param: /a
  aParam += "\"";

  RVCString fileName = "\"";
		fileName += creationMessage["$FILENAME"];
  fileName += "\"";

  RVCString fileNameToSave = "\"";
		fileNameToSave += creationMessage["$FILENAMETOSAVE"];
  fileNameToSave += "\"";
  
  RVCString roundNum = creationMessage["$ROUNDNUM"];

  //RVCString roundNum = "\"";
//		roundNum += creationMessage["$ROUNDNUM"];
  //roundNum += "\"";
  
  RVCString mode = "\"";
		mode += creationMessage["$MODE"];
  mode += "\"";

  RVCString testViewNetName  = "\"";
		testViewNetName += creationMessage["$TESTVIEWNETNAME"];
  testViewNetName += "\"";

  RVCString ipParam = "\"";
		ipParam += creationMessage["$IPPARAM"]; // Input parameters flag: /ip
  ipParam += "\"";

  RVCString inputParams = "\"";
		inputParams += creationMessage["$INPUTPARAMS"];
  inputParams += "\"";

		RVCString browserTypeFlag = "\"";
		browserTypeFlag += creationMessage["$BROWSER_TYPE_FLAG"]; 
		browserTypeFlag += "\"";

		//RVCString browserType = "\"";
		//browserType += creationMessage["$BROWSER_TYPE"];
		//browserType += "\"";
		RVCString browserType = creationMessage["$BROWSER_TYPE"];
		//

		webFTExePath = tvpc.TVEGetEnv("WEBFT_PROGRAM_PATH");

	pHandle = TVPCreateProcess(webFTExePath, aParam, fileName, fileNameToSave, roundNum, mode, testViewNetName, ipParam, inputParams, browserTypeFlag, browserType, NULL);
	}
	catch(...)
	{
		pHandle = NULL;
	}

	if( !pHandle )
	{
		HandleProcessCreationFailure(creationMessage, webFTExePath, TestTalkToolKit::GetLastError());
	}
	else if( creationMessage.KeyExists("$PROCESSUID") )
	{
    RVCString keyProcessGroupManager( creationMessage["$PROCESSUID"] );
    ProcessGroup* retVal = m_processesHashDictinary.AddProcessToGroup(keyProcessGroupManager, pHandle, PG_VER_WEBFT);
		
    if( retVal == NULL ) 
		{
      WlTraceMsg("Error : in function 'CNetKernel::CreateLoadGenerator'. failed to insert handle to table");
    }
	}
}

void CNetKernel::CreateReomteProcess(CAssociativeArray& creationMessage)
{
	PROCESS_HANDLE pHandle;
	TVEnvContainerClass tvpc;
  const char* exePath;

  //       
  //Eyal: uc???
	try
	{
		CAssociativeArray& cm = creationMessage;
		exePath = tvpc.TVEGetEnv(cm["$EXEFILEPATH"]);
		int argc = atoi(cm["$ARGC"]);

		switch(argc)
		{
		case 0:
			pHandle = TVPCreateProcess(exePath, NULL);
			break;
		case 1:
			pHandle = TVPCreateProcess(exePath, cm["$ARGV0"], NULL);
			break;
		case 2:
			pHandle = TVPCreateProcess(exePath, cm["$ARGV0"], cm["$ARGV1"], NULL);
			break;
		case 3:
			pHandle = TVPCreateProcess(exePath, cm["$ARGV0"], cm["$ARGV1"], cm["$ARGV2"], NULL);
			break;
		case 4:
			pHandle = TVPCreateProcess(exePath, cm["$ARGV0"], cm["$ARGV1"], cm["$ARGV2"], cm["$ARGV3"], NULL);
			break;
		case 5:
			pHandle = TVPCreateProcess(exePath, cm["$ARGV0"], cm["$ARGV1"], cm["$ARGV2"], cm["$ARGV3"], cm["$ARGV4"], NULL);
			break;
		case 6:
			pHandle = TVPCreateProcess(exePath, cm["$ARGV0"], cm["$ARGV1"], cm["$ARGV2"], cm["$ARGV3"], cm["$ARGV4"], cm["$ARGV5"], NULL);
			break;
		case 7:
			pHandle = TVPCreateProcess(exePath, cm["$ARGV0"], cm["$ARGV1"], cm["$ARGV2"], cm["$ARGV3"], cm["$ARGV4"], cm["$ARGV5"], cm["$ARGV6"], NULL);
			break;

		default:
			pHandle = TVPCreateProcess(exePath, NULL);
		}
	}
	catch(...)
	{
		pHandle = NULL;
	}

	if( !pHandle )
	{
		HandleProcessCreationFailure(creationMessage, exePath, TestTalkToolKit::GetLastError());
	}
}
#endif // __TV_WINDOWS__
bool CNetKernel::IsTestTalkService()
{
	return m_IsTestTalkService;
}

// ATTENTION
// We did not want to make changes for the Windows version therefore we put it under ifdef !!!!
// maybe when we will merge it we can omit this function permanently
CNetHandlerMessage* CNetKernel::SenderWaitMessage()
{
  RVCollectable* pMessage;
  TVPPriorityQueue::WaitReturnType ret;

  while (1)
    {
      ret = waitEvent(pMessage, m_KillerThreadWakeUpTimeOut);//@@avidan42
      switch (ret)
	{
	case TVPPriorityQueue::Success :
	  //Althougth this isn't the functionality of this thread, the thread is being used to 
	  //kill processes.
	  if(TryToKillProcesses()) {
	    WlTraceMsg("killed processes of the load generator successfully.");
	  }
	  return (CNetHandlerMessage*)pMessage;

	case TVPPriorityQueue::TimedOut :
	  //Althougth this isn't the functionality of this thread, the thread is being used to 
	  //kill processes.
	  if(TryToKillProcesses()) {
	    WlTraceMsg("killed processes of the load generator successfully.");
	  }
	  break;

	default:
	  return NULL;
	}
    }

}

bool CNetKernel::TryToKillProcesses() 
{
//	get the first from the queue and if time exceded kill him
	bool ret(true);
	bool keepTryToKill(true);
	CAssociativeArray* Msg = NULL;

	while(keepTryToKill) 
	{
		if(! m_queueKillRequest.isEmpty() ) 
		{
			Msg = (CAssociativeArray*)m_queueKillRequest.first();

			if ( Msg->KeyExists("$KILLREQUESTTIME") )
			{		
        long killTimeout = Msg->GetLongValueOfParameter("$KILLTIMEOUT");

        if( keepTryToKill = ( ( TVPSysTime() - Msg->GetLongValueOfParameter("$KILLREQUESTTIME") )  >= killTimeout ) ) 
				{
					Msg = (CAssociativeArray*)m_queueKillRequest.get();

					if(Msg) 
					{
    				ret = KillProcesses( (RVCString)(Msg->GetValueOfParameter("$GENERTORUID")));

						if( Msg->KeyExists("$FROM") )
						{
							CAssociativeArray message;

							message.AddCouple("KILLPROCESSDONE", Msg->GetValueOfParameter("$GENERTORUID"));
							SendMessage(message, Msg->GetValueOfParameter("$FROM"), "");
						}

						delete Msg;
					}
					
				}
			}
		} 
                else {//no more messeges the queue is empty.
				keepTryToKill = false;
		}
	}
	return ret;
}

bool CNetKernel::KillProcesses(RVCString& generatorUID)
{
	return m_processesHashDictinary.TerminateGroup( generatorUID );
}
