/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  

#include <RVCString.h>
#include <collstr.h>
#include <collint.h>

#include "platform.h"
#ifndef __TV_UNIX__
#include "tracedll.h"
#endif

#include "tvpsync.h"
#include "netmsg.h"

#include "associativearray.h"
#include "netkernelmessagehandler2.h"


///////////////////  CNetHandlerMessage decleration ////////////////////

class CNetHandlerMessage : public RVCollectableInt {
private:
  RVCString m_Message;
  RVCString m_Recipient;
  RVCString m_ErrorMsg;

public:
  CNetHandlerMessage(const char* Message, const char* Recipient, const char* ErrorMsg) 
    : m_Recipient(Recipient), m_Message(Message), m_ErrorMsg(ErrorMsg) {}
  ~CNetHandlerMessage () {}

  const char* GetMessage() {return m_Message;}
  const char* GetRecipient() {return m_Recipient;}
  const char* GetErrorMsg() {return m_ErrorMsg;}
};

static long MessageHandlerId = 0;

///////////////////  CNetMessageTTObject implementation ////////////////////
/*
void CNetMessageTTObject::SetMessageHandler(CNetMessageHandler2* nmh)
{
  m_MyNetMessageHandler = nmh;
}

bool CNetMessageTTObject::NewMessageEvent(TypedMessageObject* tmo)
{
  TestTalkMessage* ttm = NVSTmo2Ttm(tmo);
  m_MyNetMessageHandler->HandleEvent((const char*)ttm->getData());
  delete ttm;
  return true;
}
*/

///////////////////  CNetMessageHandler2 implementation ////////////////////

CNetMessageHandler2::CNetMessageHandler2(const char* basename) :
  m_SenderName(basename), 
  m_ErrorMsg(""),
  m_Initialized(false)
{
  id = ++MessageHandlerId;
  char cid[20];
  sprintf(cid,"%d",id);
  m_SenderName += "Sender";
  m_SenderName += cid;

  m_LastRecievedMessageTime = 0;

  m_SenderThread = NULL;
  m_RecieverThread = NULL;
}

CNetMessageHandler2::~CNetMessageHandler2()
{
  Terminate();
  if (m_SenderThread)
    delete m_SenderThread;
  if (m_RecieverThread)
    delete m_RecieverThread;
}

bool CNetMessageHandler2::Init(unsigned short testtalkport)
{
  //WlTraceMsg("CNetMessageHandler2::Init(%s) : Starting.",m_TestTalkClient.NETWhoAmI());
  //
  m_TestTalkPort = testtalkport;
  // if alread Initialized, terminate first.
  if (m_Initialized)
    Terminate();

  //
  // create the sender thread functor:
  if (m_SenderThread)
    delete m_SenderThread;
  m_SenderThread = new RVThread;
  if (m_RecieverThread)
    delete m_RecieverThread;
  m_RecieverThread = new RVThread;
  #if !defined(RV_THR_CANT_EXTRACT_TEMPLATE_ARGS_FROM_SIGNATURE)            
    *m_SenderThread = rvMakeThreadFunction(
                        *this,                                // instance
                        &CNetMessageHandler2::StartSender      // member function
                      );
    *m_RecieverThread = rvMakeThreadFunction(
                        *this,                                // instance
                        &CNetMessageHandler2::StartReciever      // member function
                      );
  #else
    *m_SenderThread = rvMakeThreadFunctionM( 
                        CNetMessageHandler2                    // class
                        *this,                                // instance
                        void,                                 // return value
                        &CNetMessageHandler2::StartSender      // member function
                      );
    *m_RecieverThread = rvMakeThreadFunctionM( 
                        CNetMessageHandler2                    // class
                        *this,                                // instance
                        void,                                 // return value
                        &CNetMessageHandler2::StartReciever      // member function
                      );
  #endif

  m_Initialized = true;
  m_MessagesQueue.Init();
  //
  // Start the sender thread.
  m_SenderStartEvent.Reset();
  m_SenderDoneEvent.Reset();
  m_SenderThread->start();
  m_SenderStartEvent.Wait();

  //
  // Start the receiver thread.
  m_RecieverStartEvent.Reset();
  m_RecieverDoneEvent.Reset();
  m_RecieverThread->start();
  m_RecieverStartEvent.Wait();

  m_LastRecievedMessageTime = TVPSysTime();

  if (!m_SenderDoneEvent.Get() && !m_RecieverDoneEvent.Get() ) {
    //WlTraceMsg("CNetMessageHandler2::Init(%s) : Done successfully.",m_TestTalkClient.NETWhoAmI());
    return true;
  } else {
    Terminate();
    //WlTraceMsg("CNetMessageHandler2::Init(%s) : Done failure.",m_TestTalkClient.NETWhoAmI());
    return false;
  }
}

void CNetMessageHandler2::Terminate()
{
  //WlTraceMsg("CNetMessageHandler2::Terminate(%s) : Starting.",m_TestTalkClient.NETWhoAmI());
  if (!m_Initialized)
    return;

  m_Initialized = false;

  //
  // Make sure there are no more messages, 
  // than terminate the messages queue, and wait for the sender to terminate
  TVPTimerObject timeout;
  while (!IsSendQueueEmpty() && timeout.get() < 10)
    ::rvSleep(10);
  
  m_MessagesQueue.Terminate();
	//
	// Tedrminate the sender 
  if (m_SenderDoneEvent.Wait(5000) == RV_THR_TIMEOUT) {
    try {
      m_SenderThread->terminate();
    } catch (...) {}
  }

  //
  // Terminate the Networking
  m_TestTalkClient.NETTerminate();

	//
	// Tedrminate the receiver
  if (m_RecieverDoneEvent.Wait(2000) == RV_THR_TIMEOUT) {
    try {
      m_RecieverThread->terminate();
    } catch (...) {}
  }

  //WlTraceMsg("CNetMessageHandler2::Terminate(%s) : Done.",m_TestTalkClient.NETWhoAmI());
}

void CNetMessageHandler2::SendMessage( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, int priority)
{
  if (!Message.KeyExists("$FROM"))
    Message.AddCouple("$FROM",m_TestTalkClient.NETWhoAmI());

  //WlTraceMsg("CNetMessageHandler2::SendMessage(%s): (to:%s, msg:%s)",m_TestTalkClient.NETWhoAmI() , SendTo, Message.ToString());
	m_MessagesQueue.putEvent(new CNetHandlerMessage(Message.ToString(), SendTo, ErrorMsg), priority);
}

CNetHandlerMessage* CNetMessageHandler2::SenderWaitMessage()
{
  //WlTraceMsg("CNetMessageHandler2::SenderWaitMessage(%s) - Started",m_TestTalkClient.NETWhoAmI());
  return (CNetHandlerMessage*)m_MessagesQueue.waitEvent();
}

void CNetMessageHandler2::SendMessageSucceded( const char* Message , const char* SendTo)
{
}

void CNetMessageHandler2::SendMessageErrorHandler( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, bool status)
{
}

void CNetMessageHandler2::MessageHandler( CAssociativeArray& AssoArrayCommand )
{
	if ( AssoArrayCommand.KeyExists( "$EXIT" ) )
  {
    //throw Exiting;
  }
}

void CNetMessageHandler2::StartSender()
{
  //WlTraceMsg("CNetMessageHandler2::StartSender(%s) - Started",m_TestTalkClient.NETWhoAmI());
  RVThreadSelf self = ::rvThread();
  self.setPriority(self.getMaxPriority());
  TVPManualEvent::SetGuard event(m_SenderDoneEvent);
  try {
    SenderMainLoop();
  } catch (...) {
    //WlTraceMsg("Fatal Error in CNetMessageHandler2::StartSender(%s)",m_TestTalkClient.NETWhoAmI());
    #ifdef _DEBUG
    throw;
    #endif
  }
  //WlTraceMsg("CNetMessageHandler2::StartSender(%s) - Done", m_TestTalkClient.NETWhoAmI());
  return;
}

void CNetMessageHandler2::StartReciever()
{
  //WlTraceMsg("CNetMessageHandler2::StartReciever(%s) - Started",m_TestTalkClient.NETWhoAmI());
  RVThreadSelf self = ::rvThread();
  self.setPriority(self.getMaxPriority());
  TVPManualEvent::SetGuard event(m_RecieverDoneEvent);
  m_RecieverStartEvent.Set();
  try {
    const char* message;
    while ((message = m_TestTalkClient.NETWaitMessage()) != NULL) {
      m_LastRecievedMessageTime = TVPSysTime();
      HandleEvent(message);
    }
  } catch (...) {
    //WlTraceMsg("Fatal Error in CNetMessageHandler2::StartReciever(%s)",m_TestTalkClient.NETWhoAmI());
    #ifdef _DEBUG
    throw;
    #endif
  }
  //WlTraceMsg("CNetMessageHandler2::StartReciever(%s) - Done",m_TestTalkClient.NETWhoAmI());
  return;
}

void CNetMessageHandler2::SenderMainLoop()
{
  //WlTraceMsg("CNetMessageHandler2::SenderMainLoop(%s) : started", m_TestTalkClient.NETWhoAmI());
  //WlTraceMsg("CNetMessageHandler2::SenderMainLoop : Initializing %s", m_SenderName.data());
  if (!m_TestTalkClient.NETInit(m_SenderName, m_TestTalkPort)) {
    WlTraceMsg("CNetMessageHandler2::SenderMainLoop Error initializing sender %s.", m_SenderName.data());
    m_SenderStartEvent.Set();
    return;
  }
  m_SenderStartEvent.Set();

  CNetHandlerMessage* pMessage;
  while ((pMessage = SenderWaitMessage()) != NULL)
  {
    //WlTraceMsg("CNetMessageHandler2::SenderMainLoop(%s) - Sending message (to:%s, %s)", m_TestTalkClient.NETWhoAmI(), pMessage->GetRecipient(), pMessage->GetMessage());
    bool status = m_TestTalkClient.NETSendMultiMessage(pMessage->GetMessage(), pMessage->GetRecipient());
    //WlTraceMsg("CNetMessageHandler2::SenderMainLoop(%s) - Sending message (ret:%d)", m_TestTalkClient.NETWhoAmI(), (int)status);
    if (!status) {
 	    CAssociativeArray	AssoArrayCommand;
      AssoArrayCommand.FromString(pMessage->GetMessage());
      SendMessageErrorHandler(AssoArrayCommand, pMessage->GetRecipient(), pMessage->GetErrorMsg(), status);
    } else {
      SendMessageSucceded(pMessage->GetMessage(), pMessage->GetRecipient());
    }
    delete pMessage;
  }
}

void CNetMessageHandler2::HandleEvent(const char* AssoArrayString)
{
	try
	{
	  CAssociativeArray	AssoArrayCommand;
		if( AssoArrayString == NULL )
			return;

    //WlTraceMsg("CNetMessageHandler2::HandleEvent: Received command (%s)",(const char*)AssoArrayString);

		AssoArrayCommand.FromString( AssoArrayString );

    MessageHandler(AssoArrayCommand);
		
  } catch (WLErrorException e) {
    WlTraceMsg("CNetMessageHandler2::HandleEvent: Exception: %s",e.GetErrorMsg());
    //
    //return;
  } catch (...) {
    WlTraceMsg("CNetMessageHandler2::HandleEvent: Unhandled exception.");
    #ifdef _DEBUG
    throw;
    #endif
  }
}

