/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
//
// clntinst.cpp -  NetView Server session module deals with treating
//                the clientInstance class.
//
//    Who         When          What
//  Yahali     10-Feb-94     Creation
//

#include "tracedll.h"
#include "testtalkdefs.h"
#include "testtalkclass.h"

#include "testtalkeventserver.h"
#include "testtalknetconnection.h"
#include "testtalkclientinstance.h"
#include "associativearray.h"
#include "testtalktoolkit.h"

// constructor to ClientInstance
TestTalkClientInstance::TestTalkClientInstance(const char* name, 
                                               CTestTalkClientConnection *conn, 
                                               CTestTalkClass *ttMain) :
	m_ClientName(name), clientConn(conn), m_TestTalkMain(ttMain)
{
	//EnterProcedure("TestTalkClientInstance::TestTalkClientInstance");
    WlTraceMsg("TestTalkClientInstance::TestTalkClientInstance(%s)", name);
	//
	// registering the Client-event.
	int op = signOn((FUNCP)&TestTalkClientInstance::sendMsgToClient);
	m_TestTalkMain->m_EventServer->registerClientEvent(name, *this, op);
}

TestTalkClientInstance::~TestTalkClientInstance()
{
	//EnterProcedure("TestTalkClientInstance::~TestTalkClientInstance");
	//
	// unregister myself.
  try {
    //debugPrint(3,("Deleted client '%s'\n",name()));
	  delete clientConn;
  } catch (...) {
  }
}

int TestTalkClientInstance::sendMsgToClient(BaseEvent& e)
{
	//EnterProcedure("TestTalkClientInstance::sendMsgToClient");
	TestTalkClientEvent& ce = (TestTalkClientEvent&)e;
	//
  // use the client-connection object to send the data
	// to the client.
	clientConn->newMessage(ce);

	return (TRUE);
}

void TestTalkClientInstance::Terminate()
{
  //
  // request the Global-connection to terminate me.
  if (m_TestTalkMain)
    m_TestTalkMain->m_EventServer->removeClient(*this);
  if (clientConn != NULL)
	  clientConn->Terminate();
}

//
// print function for TmlInstance.
void TestTalkClientInstance::printOn( ostream& os) const
{
  os << nameOf() << " Named: " << m_ClientName << endl;
}

int TestTalkClientInstance::doneReadingMessage(void* msg,
                                               int   len,
                                               short type, 
                                               short subtype)
{
  //
  // check the message-type and act accordingly.
  switch (type)
  {
    case NV_SENDMESSAGE:
		case NV_SENDMESSAGE_NORESPONSE:
    case NV_SENDMESSAGE_NOWAIT:
		case NV_FILECOPY:
    {
        //
        // dispatch a host-event.
        TestTalkHostEvent& e = * new TestTalkHostEvent(msg, len, type, subtype);
        //debugPrint(5,("Sending %s\n",e.printOnString()));
        //
        // Send error only if response is required.
        if (!m_TestTalkMain->m_EventServer->dispatchHostEvent(e) && type!=NV_SENDMESSAGE_NORESPONSE)
        {
          //
          // The queue is determined according to the type, regular
          // message to the control queue and the nowait to the dataQ.
          QType queue;
          if (type==NV_SENDMESSAGE_NOWAIT)
            queue = AckQ;
          else //if (type==NV_SENDMESSAGE)
            queue = ControlQ;
          //
          // if sending fails - return error.
          const char* error = "Error dispatching host event";
          TestTalkClientEvent& cevent =
            * new TestTalkClientEvent( e.getMessageId(),
                                       "","","","",
                                       queue,
                                       NV_SENDMESSAGE_STATUS_FAIL,
                                       0,
                                       (void*)error,
                                       strlen(error)+1);
          clientConn->newMessage(cevent);
        }
      }
      break;

    case NV_LOCALMESSAGE:
    case NV_LOCALMESSAGE_NOWAIT:
    case NV_LOCALMESSAGE_NORESPONSE:
      {
        //
        // dispatch a client-event.
        TestTalkClientEvent& e = * new TestTalkClientEvent(msg, len, type, subtype);
        //debugPrint(5,("Sending local %s\n",e.printOnString()));
        short st = m_TestTalkMain->m_EventServer->dispatchClientEvent(e);
        //
        // if sending fails - return error.
        if (type!=NV_LOCALMESSAGE_NORESPONSE) 
        {
          //
          // The queue is determined according to the type, regular
          // message to the control queue and the nowait to the dataQ.
          QType queue;
          if (type==NV_LOCALMESSAGE_NOWAIT)
            queue = AckQ;
          else //if (type==NV_SENDMESSAGE)
            queue = ControlQ;

          const char* error = "Error dispatching client event";
          TestTalkClientEvent& cevent =
            * new TestTalkClientEvent( e.getMessageId(),
                                       "","","","",
                                       queue,
            										       (st?NV_SENDMESSAGE_STATUS_SUCCESS:NV_SENDMESSAGE_STATUS_FAIL),
                                       st,
                                       (void*)error,
                                       strlen(error)+1);
          clientConn->newMessage(cevent);
        }
      }
      break;

		case NV_LOCAL_FILECOPY:
		{
			TestTalkToolKit::CopyFileCompletionState copyStatus;
			char* statusFileName;
			CAssociativeArray message;
			TestTalkClientEvent event(msg, len, type, subtype);

			message.FromString((const char*)event.getData());

			const char* srcFilePath = message.GetValueOfParameter("$LOCALFILE");
	    const char* dstFilePath = message.GetValueOfParameter("$REMOTEFILE");

			TestTalkToolKit::CopyFile((char*)srcFilePath, (char*)dstFilePath,
																copyStatus, statusFileName);

			char*  buffer;
			size_t bufferLen;
			CAssociativeArray completionState;

			completionState.AddCouple("$FILECOPYCOMPLETIONSTATE", long(copyStatus));
			completionState.AddCouple("$FILENAME", statusFileName);
			completionState.ToString(buffer, bufferLen);
			TestTalkClientEvent *cevent =
      TestTalkToolKit::CreateReturnClientEvent(&event, 
																							 buffer, bufferLen, 
																							 NV_SENDMESSAGE_STATUS_SUCCESS, 
																							 AckQ);

			clientConn->newMessage(*cevent);
		}
		break;

    case NV_MULTIMESSAGE:
      {
        //TestTalkHostEvent& e = * new TestTalkHostEvent(msg, len, type, subtype); 
        ////debugPrint(5,("Sending multy %s\n",e.printOnString()));

        //MultiMessageClient& mmc = * new MultiMessageClient(&e, getTestTalkMainLoop());
        //if (mmc.sessionTerminated())
        //  delete &mmc;
      }
      break;
		
		
    case NV_CLIENT_TERMINATED:
      if (clientConn != NULL) {
	      clientConn->Terminate(false);
        clientConn = NULL;
      }
      Terminate();
      break;

    }

  return TRUE;
}

