/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/***********************************************************************
/* Name : TestTalkEventServer.cpp
/* Purpose : the CTestTalkEventServer implementation.
/*
/* Description :
/*
/* Creation: Yahali, 16-Feb-95
/* Updates :  Who          When         What
/*
/***********************************************************************/

#include "tracedll.h"

#include "testtalkclass.h"
#include "testtalknetconnection.h"
#include "testtalkeventserver.h"

CTestTalkEventServer::CTestTalkEventServer(CTestTalkClass *ttMain) :
  m_TestTalkMain(ttMain)
{
  m_InsideCleanup = false;
}

CTestTalkEventServer::~CTestTalkEventServer()
{
}

void CTestTalkEventServer::cleanUp()
{
  RVTRecursiveLock<RVMutexLock>::LockGuard lock(m_EventLock);

  m_InsideCleanup = true;

  RVHashTableIterator htiHosts(m_HostsTable);
  try {
	while (htiHosts())
	{
		CTestTalkHostConnection* hc = (CTestTalkHostConnection*)htiHosts.key();
		hc->Terminate();
    WlTraceMsg("CTestTalkEventServer::cleanUp - Removing host %s", hc->m_Name.data());
    htiHosts.remove();
    delete hc;
	}

//#ifdef _DEBUG
    RVHashTableIterator htiClients(m_ClientsTable);
	  while (htiClients())
	  {
		  TestTalkClientInstance* ci = (TestTalkClientInstance*)htiClients.key();
		  ci->Terminate();
      WlTraceMsg("CTestTalkEventServer::cleanUp - Removing client %s", ci->name());
		  htiClients.remove();
		  delete ci;
	  }
  } catch (...) {
    #ifdef _DEBUG
      //throw;
    #endif
  }
  m_InsideCleanup = false;
//#endif
}

void CTestTalkEventServer::registerClientEvent(const char* cn,
                                           TestTalkClientInstance& clnt,
                                           int op)
{
  WlTraceMsg("CTestTalkEventServer::registerClientEvent");

  RVTRecursiveLock<RVMutexLock>::LockGuard lock(m_EventLock);

  m_ClientsTable.insert((RVCollectable*)&clnt);
  registerEvent(* new TestTalkClientEvent(cn, clnt, op));
}

void CTestTalkEventServer::removeClient(TestTalkClientInstance& client)
{
  RVTRecursiveLock<RVMutexLock>::LockGuard lock(m_EventLock);

  if (m_InsideCleanup)
    return;

  unRegisterClient(client);
	RVHashTableIterator hti(m_ClientsTable);
	while (hti())
	{
		TestTalkClientInstance* ci = (TestTalkClientInstance*)hti.key();
		if (ci == (TestTalkClientInstance*)&client) {
      WlTraceMsg("CTestTalkEventServer::removeClient - Removing client %s", ci->name());
			hti.remove();
			break;
		}
	}
}

int CTestTalkEventServer::dispatchClientEvent(TestTalkClientEvent& event)
{
  //WlTraceMsg("CTestTalkEventServer::dispatchClientEvent");

  RVTRecursiveLock<RVMutexLock>::LockGuard lock(m_EventLock);

  TestTalkClientEvent* ce = (TestTalkClientEvent*)&findEvent(event);
  if (ce != NULL)
	 {
		//WlTraceMsg("CTestTalkEventServer::dispatchClientEvent, Dispatching: %s", event.printOnString());

		ce->dispatchEvent(event);
		return TRUE;
	 }
  else
	 return FALSE;
}

void CTestTalkEventServer::registerHostEvent(const char* thn,
                                         CTestTalkHostConnection& hc,
                                         int op)

{
  WlTraceMsg("CTestTalkEventServer::registerHostEvent");

  RVTRecursiveLock<RVMutexLock>::LockGuard lock(m_EventLock);

  m_HostsTable.insert((RVCollectable*)&hc);
  registerEvent(* new TestTalkHostEvent(thn, (ClientObj&)hc, op));
}

void CTestTalkEventServer::removeHost(CTestTalkHostConnection& host)
{
  RVTRecursiveLock<RVMutexLock>::LockGuard lock(m_EventLock);

  if (m_InsideCleanup)
    return;

  unRegisterClient((ClientObj&)host);
	RVHashTableIterator hti(m_HostsTable);
	while (hti())
	{
		CTestTalkHostConnection* hc = (CTestTalkHostConnection*)hti.key();
		if (hc == (CTestTalkHostConnection*)&host) {
      WlTraceMsg("CTestTalkEventServer::removeHost - Removing host %s", hc->m_Name.data());
			hti.remove();
			break;
		}
	}
}

int CTestTalkEventServer::dispatchHostEvent(TestTalkHostEvent& event)
{
  EnterProcedure("CTestTalkEventServer::dispatchHostEvent");
  RVTRecursiveLock<RVMutexLock>::LockGuard lock(m_EventLock);

  try {
    TestTalkHostEvent* he = (TestTalkHostEvent*)&findEvent(event);
    if (he != NULL)
	  {
		  //WlTraceMsg("CTestTalkEventServer::dispatchHostEvent, Dispatching: %s", event.printOnString()); //@@eyal: trace
      he->dispatchEvent(event);
	  }
    else
	  {
      WlTraceMsg("CTestTalkEventServer::dispatchHostEvent, Creating new CTestTalkHostConnection(%s)", event.getToHost()); //@@eyal: trace
		  CTestTalkHostConnection& hc =
		    * new CTestTalkHostConnection(event.getToHost(), m_TestTalkMain);
		  if (!hc.valid())
		    {
			    delete &hc;
			    return FALSE;
		    }
		  hc.newMessage(event);
	  }
    return TRUE;
  }  catch (...) {
    WlTraceMsg("CTestTalkEventServer::dispatchHostEvent - Fata Error Occured");
    return FALSE;
  }
}

