/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef _HANDLERS_H
#define _HANDLERS_H


#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <iostream.h>
#ifndef __TV_UNIX__
#include <process.h>    /* _beginthread, _endthread */
#include <io.h>
#endif
#include <RVCString.h>

#include "netmsg.h"
#include "nvspackmsg.h"
#include "testtalkclientobj.h"
#include "associativearray.h"
#include "netkernelmessagehandler.h"
#include "testtalktoolkit.h"

//********************************************************************//

class NetMsgHandler: public CNetMessageHandler
{
public:
	NetMsgHandler(const char* basename): CNetMessageHandler(basename){}

	void CopyFile(const char* LocalFileName,
								const char* RemoteFileName,
								const char* RemoteHostAddress,
								const char* ErrorMsg = "CopyFile Failed", int priority = 1);

	virtual void SendMessageErrorHandler( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, bool status);

	virtual void CopyFileDone(CNetMessageHandler::CopyFileCompletionState copyStatus,
														const char* copyStatusFileName,
														const char* LocalFileName,
														const char* RemoteFileName,
														const char* RemoteHostAddress);

	double duration;
	clock_t start, finish;
	TVPManualEvent waitForCopyDone;
};

void NetMsgHandler::CopyFile(const char* LocalFileName,
														 const char* RemoteFileName,
														 const char* RemoteHostAddress,
														 const char* ErrorMsg, int priority)
{
	waitForCopyDone.Reset();

	start = clock();
	CNetMessageHandler::CopyFile(LocalFileName, RemoteFileName,
															 RemoteHostAddress, ErrorMsg, priority);

	//waitForCopyDone.Wait(6000);
}

void NetMsgHandler::CopyFileDone(CNetMessageHandler::CopyFileCompletionState copyStatus,
																	const char* copyStatusFileName,
																	const char* LocalFileName,
																	const char* RemoteFileName,
																	const char* RemoteHostAddress)
{
	finish = clock();
	duration = (double)(finish - start) / CLOCKS_PER_SEC;
	cout<<"\nTotal Time = "<<duration<<endl;

	switch(copyStatus) {
		case CNetMessageHandler::CopyFileSucceed:
			cout<<"CopyFileSucceed"<<endl<<copyStatusFileName<<endl<<RemoteHostAddress<<endl;
			break;
		case CNetMessageHandler::OpenFileFailed:
			cout<<"OpenFileFailed"<<endl<<copyStatusFileName<<endl<<RemoteHostAddress<<endl;
			break;
		case CNetMessageHandler::CloseFileFailed:
			cout<<"CloseFileFailed"<<endl<<copyStatusFileName<<endl<<RemoteHostAddress<<endl;
			break;
		case CNetMessageHandler::ReadFileFailed:
			cout<<"ReadFileFaild"<<endl<<copyStatusFileName<<endl<<RemoteHostAddress<<endl;
			break;
		case CNetMessageHandler::WriteFileFailed:
			cout<<"WriteFileFailed"<<endl<<copyStatusFileName<<endl<<RemoteHostAddress<<endl;
			break;
		case CNetMessageHandler::CopyFileFailed:
			cout<<"CopyFileFailed"<<endl<<copyStatusFileName<<endl<<RemoteHostAddress<<endl;
			break;
	}
	cout<<endl;

	waitForCopyDone.Set();
}

void NetMsgHandler::SendMessageErrorHandler( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, bool status)
{
	printf("\nNetMsgHandler::SendMessageErrorHandler.\n");
	printf("Got Error Msg on msg: %s,\nTo Address: %s\n", Message.ToString(), SendTo);
	printf("Error Messge : %s\n", ErrorMsg);
}


//********************************************************************//

class DllExport CNetMessageHandlerTest: public CNetMessageHandler
{
private:
	bool sendDone;

public:
	CNetMessageHandlerTest(): CNetMessageHandler("eyal"), sendDone(false){}
	struct SimpleStruct {
		int i;
		double d;
		char c;
	};
	
	void WaitForTerminate()
	{
		while(!sendDone) {
			rvSleep(200);
		}
	}

	void SendFile(void)
	{
		/*size_t length = 0;
		FILE* infile;
		CAssociativeArray array;
		char* _origFileName = "f:\\temp\\trace.txt";
		char*  buffer;
		SimpleStruct simple;
		simple.i = 10; simple.d = 12.34464; simple.c = 'M';

		infile  = fopen(_origFileName, "r+b");

		TestTalkToolKit::ReadFileToBuffer(infile, buffer, length);

		array.AddCouple("$BINARYFILE", buffer, length);
		delete buffer;
		fclose(infile);

		array.AddCouple("$SIMPLESTRUCT", &simple, sizeof(SimpleStruct));
		array.AddCouple("$STRING", "String Test");
		array.AddCouple("$DOUBLE", 123.456);

		SendMessage(array, WhoAmI(), "");*/
	}

  virtual void MessageHandler( CAssociativeArray& AssoArrayCommand )
	{
		/*void* buffer;
		size_t length = 0;
		FILE* outfile;
		SimpleStruct simple;
		char* _newFileName  = "f:\\temp\\trace.txt";

		outfile = fopen(_newFileName, "w+b");

		AssoArrayCommand.GetValueOfParameter("$BINARYFILE", buffer, length);

		TestTalkToolKit::SaveBufferToFile(outfile, (char*)buffer, length);
		fclose(outfile);

		AssoArrayCommand.GetValueOfParameter("$SIMPLESTRUCT", buffer, length);
		simple = *((SimpleStruct*)buffer);

		cout<<AssoArrayCommand.GetValueOfParameter("$STRING")<<endl
			  <<AssoArrayCommand.GetValueOfParameter("$DOUBLE")<<endl
				<<simple.i<<", "<<simple.d<<", "<<simple.c<<endl;*/
	}

	virtual void SendMessageSucceded( const char* Message , const char* SendTo)
	{
		cout<<Message<<endl<<SendTo<<endl;
		cout<<"SendMessageSucceded"<<endl;
		sendDone = true;
	}

	virtual void SendMessageErrorHandler( CAssociativeArray& Message , const char* SendTo, const char* ErrorMsg, bool status)
	{
		cout<<"Send to "<<SendTo<<" failed:"<<ErrorMsg<<endl;
		sendDone = true;
	}
};
//********************************************************************//

#endif

