/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __TV_ASSOCIATIVEARRAY_H
#define __TV_ASSOCIATIVEARRAY_H

#include <hashdict.h>
#include <RVCString.h>
 
#include "platform.h"
//#include "wlerrorlib.h"
#include <stdexcpt.h>


#define MAX_NUM_SIZE 20
#define BUFFER_SIZE 1024

class CAssociativeArray : public RVHashDictionary
{
private:
	RVCString toString;

	inline size_t ParseInt(const char* &input);

	const char* LoadString(const char* input, char*& tokenBuffer, 
					  		  			 size_t& bufferSize, RVCString& s); 
	
public:
	CAssociativeArray( int Capacity = 11);
	
	~CAssociativeArray();
	
	const char* ToString();

	// This method should be used if using AddCouple(key, value, valueLength)
	// to hold binary data and there is a need to do some manipulations on 
	// the out string like copy it with strcpy.
	// In order to clone an associative array there is no need in this 
	// method because FromString knows to 'analyze' a string without it's
	// length so  a1.FromString(a2.ToString())  is valid with any kind of data.
	void  ToString(/*[OUT]*/char*& outString, 
								 /*[OUT]*/size_t& outStringLength);

	void	FromString( const char* AssoString );
	
	void	FromString( const RVCString& AssoString ) {FromString((const char*)AssoString); }
	
	// Puts the value and it's length in OutValue and OutValueLength.
	// Use this method if you use AddCouple(key, value, valueLength)
	void  GetValueOfParameter( /*[IN]*/ const char* Key, 
							  						 /*[OUT]*/void*&  OutValue,
														 /*[OUT]*/size_t& OutValueLength);

	const char*	GetValueOfParameter( const char* Key );
	
	const char*	operator [](const char* Key) { return GetValueOfParameter(Key); }
	
	long GetLongValueOfParameter( const char* Key );
	
	double GetDoubleValueOfParameter( const char* Key );
	
	const char*	GetValueOfParameter( const char* Key , const char* DefaultValue );
	
	long GetLongValueOfParameter( const char* Key , long DefaultValue );
	
	double GetDoubleValueOfParameter( const char* Key, double DefaultValue );
	
	const char*	GetValueOfParameterArray( const char* Key, int index );
	
	long GetLongValueOfParameterArray( const char* Key, int index );
	
	double GetDoubleValueOfParameterArray( const char* Key, int index );
	
	const char*	GetValueOfParameterArray( const char* Key, int index, const char* DefaultValue );
	
	long GetLongValueOfParameterArray( const char* Key, int index, long DefaultValue );
	
	double GetDoubleValueOfParameterArray( const char* Key, int index, double DefaultValue );
	
	// Add value from any type and it's length
	// Use for binary data, simple structures etc'.
	// When using this method use GetValueOfParameter(key, outVal, outValLen)
	// and ToString(outStr, outStrLen)
	void	AddCouple( /*[IN]*/const char* Key, 
									 /*[IN]*/const void* Value, /*[IN]*/size_t valueLength ); 

	void	AddCouple( const char* Key, const char* Value );
	
	void	AddCouple( const char* Key, double Value );
	
	void	AddCouple( const char* Key, long Value );
	
	void	AddArrayCouple( const char* Key, const char* Value, int index );
	
	void	AddArrayCouple( const char* Key, double Value, int index );
	
	void	AddArrayCouple( const char* Key, long Value, int index );
	
	bool	KeyExists( const char* Key );
	
	void	Reset() ;
};



#endif

