/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef _SOCKETS_HPP
#define _SOCKETS_HPP

#include "platform.h"
#include "socketlib.h"

// **************************************************************************************
class DllExport Socket {
public:
   enum ShutMode{nShutModeNoRecv = 0, nShutModeNoSend = 1, nShutModeNoRecvSend = 2};

   SOCKET         m_Socket;                   //The Socket handle
   short          bBound;              //A flag that we can use to tell if socket is bound or not.  It gets set on automatically, but doesn't get set off automatically.
   sockaddr_in    mySocketAddress;     //My address.
   sockaddr_in    myPeerSocketAddress; //Address of Peer to communicate with.  Used for datagrams and connections.

   Socket();
   Socket(SOCKET& newS);
   virtual ~Socket();
   operator SOCKET() const {return m_Socket;}
   virtual int CloseSocket();
   virtual int ShutDownSocket(int nShutMode = nShutModeNoRecvSend);
   Socket& operator=(Socket& newSocket);
   friend int operator==(const Socket& socket1, const Socket& socket2);

   //Commands
   virtual int  CreateSocket();
   virtual int  BindSocket(const char* hostname, u_short port); //binds to a given address.
   virtual int  BindSocket(u_short port);                       //binds to our address.
   //virtual void SetMyAddress(const char* hostname, short port);
   virtual void SetSocketStyle(int nNewFamily=AF_INET, int nNewType=SOCK_STREAM, int nNewProtocol=0);
   short SetSaveSocketOnDelete(short bNewSaveSocket=TRUE){return bSaveSocket = bNewSaveSocket;} //Tells class Sockete not to close the socket on deletion.

   //Info
   int GetLastError(){return nLastError;}
   virtual unsigned long GetDriverWaitingSize();
   virtual unsigned long GetTotalWaitingSize();
   virtual int GetMyAddress(sockaddr_in& socketAddress, SOCKET_LEN& nAddressLength, SOCKET& socket);
   virtual int GetMyAddress(sockaddr_in& socketAddress, SOCKET_LEN& nAddressLength);
   virtual int GetPeerAddress(sockaddr_in& socketAddress, SOCKET_LEN& nAddressLength, SOCKET& socket);
   virtual int GetPeerAddress(sockaddr_in& socketAddress, SOCKET_LEN& nAddressLength);

   //Options:
   int SetBroadcastOption(BOOL bBroadcast);
   int SetDebugOption(BOOL bDebug);
   int SetLingerOption(BOOL bLinger, u_short nLingerTime=0);
   int SetRouteOption(BOOL bRoute);
   int SetKeepAliveOption(BOOL bKeepAlive);
   int SetOOBOption(BOOL bSendOOBDataInline);
   int SetReceiveBufferOption(int nReceiveBufferSize);
   int SetSendBufferOption(int nSendBufferSize);
   int SetReuseAddressOption(BOOL bAllowReuseAddress);

   int GetBroadcastOption(BOOL& bBroadcast);
   int GetDebugOption(BOOL& bDebug);
   int GetLingerOption(BOOL& bLinger, u_short& nLingerTime);
   int GetRouteOption(BOOL& bRoute);
   int GetKeepAliveOption(BOOL& bKeepAlive);
   int GetOOBOption(BOOL& bSendOOBDataInline);
   int GetReceiveBufferOption(int& nReceiveBufferSize);
   int GetSendBufferOption(int& nSendBufferSize);
   int GetReuseAddressOption(BOOL& bAllowReuseAddress);

   // i/o
   int SetBlockingMode(int blockMode);


protected:
   int            nFamily;             //PF_INET, etc. (this is the protocol family)
   int            nType;               //SOCK_STREAM, etc.
   int            nProtocol;           //IPPROTO_TCP, etc.
   int            nLastError;          //myLast Error.
   int            bSaveSocket;
};

#endif //_SOCKETS_HPP



