/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// Local Variables:
// mode: c++
// End:
//
#ifndef __TestTalk_NETMESSAGE_H
#define __TestTalk_NETMESSAGE_H

#include "nvsmsg.h"
#include "packmsg.h"
#include "netview.h"
#include "platform.h"
#include "typedmsg.h"

#define TESTTALK_MESSAGE_ILLEGAL (-1)

typedef enum {
  DataQ     = 0,
  ControlQ  = 1,
  AckQ      = 2,
  LastQType = 3
} QType;

typedef struct {
  //unsigned short clientQ;
  unsigned short TestTalkPort;
  char clientName[MAXTMLNAMELEN];
  char localHostName[MAXTMLNAMELEN];
} NetClientData;

#define TestTalkMessageDllExport  DllExport

class TestTalkMessageDllExport TestTalkMessage
{
private:
  void packMe();
  void reMapFields();
  void mapFields(Pack& msgPack);
	void unPackMe(void* buf, long l);

protected:
  long        m_MessageID;
  long        m_MessageVersion;

  const char* toHost;
  const char* toClient;
	const char* fromHost;
  const char* fromClient;
  QType       qtype;
  short       type;
  short       subtype;
  void*       data;
  int         datalen;
  void*       packedBuf;
  int         packedBufLen;

public:

  TestTalkMessage(
       long msgId,
       const char* th,
       const char* tc,
       const char* fh,
       const char* fc,
       QType qt,
       short mt,
       short st,
       void* d,
       long  l);

  TestTalkMessage(void* buf, long l, short t, short s_t);

  TestTalkMessage(TypedMessageObject& tmo);

  TestTalkMessage();

  virtual ~TestTalkMessage();

  void rePackMe(const char* th, const char* tc);

  virtual char *nameOf() const { return ("TestTalkMessage"); }

  //
  // functions to return the event fields.
  long        getMessageId()    const { return m_MessageID; }
  const char* getToHost()       const { return toHost; }
  const char* getToList()       const { return toHost; }
  const char* getToClient()     const { return toClient; }
  const char* getFromHost()     const { return fromHost; }
  const char* getFromClient()   const { return fromClient; }
  QType       getQtype()        const { return qtype; }
  short       getType()         const { return type; }
  short       getSubtype()      const { return subtype; }
  void*       getData()         const { return data; }
  long        getDatalen()      const { return datalen; }
  void*       getPackedBuf()    const { return packedBuf; }
  long        getPackedBufLen() const { return packedBufLen; }

  void       setSubtype(short s_t) { subtype = s_t; }

  const char* printOnString();
};

#endif

