/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// FilesAssociationArchive.cpp: implementation of the CFilesAssociationArchive class.
//
//////////////////////////////////////////////////////////////////////

#include "filesassociationarchive.h"
#include "tvenv.h"

//#include "wlStuffer.h"


RVCString FilesAssociationArchive::mGlobalDefualtDestinationDirectory = "";

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

FilesAssociationArchive::FilesAssociationArchive() 
:mIterator(NULL)
{

}

FilesAssociationArchive::~FilesAssociationArchive()
{

}

void FilesAssociationArchive::AddFileAssociation(const char *srcFile, const char *dstFile)
{
	if( dstFile == NULL || strlen(dstFile) == 0 )
	{
		AddCouple(srcFile, BuildDefDestPath(srcFile, false));
	}
	else
	{
		AddCouple(srcFile, BuildDefDestPath(dstFile, true));
	}
}

const char* FilesAssociationArchive::GetFileAssociate(const char *srcFile)
{
	return GetValueOfParameter(srcFile);
}

RVHashDictionaryIterator& FilesAssociationArchive::GetIterator()
{
	if( !mIterator )
	{
		mIterator = new RVHashDictionaryIterator(*this);
	}
	else
	{
		mIterator->reset();
	}

	return *mIterator;
}

void FilesAssociationArchive::SetDefualtDestinationDirectory(const char* dirPath)
{
	mDefualtDestinationDirectory = dirPath;
}

const char* FilesAssociationArchive::GetDefualtDestinationDirectory()
{
	return mDefualtDestinationDirectory;
}

void FilesAssociationArchive::SetDefualtGlobalDestinationDirectory(const char* dirPath)
{
	mGlobalDefualtDestinationDirectory = dirPath;
}

RVCString FilesAssociationArchive::BuildDefDestPath(const char* srcFile, bool fileOnly)
{
  char *dir, *file, *ext;
	TVEnvContainerClass tvpc;

#ifndef __TV_UNIX__
	tvpc.TVPSplitPath(srcFile, &dir, &file, &ext);

	if( fileOnly && strlen(dir) > 0 )
	{
		return srcFile;
	}

	if( mDefualtDestinationDirectory.length() > 0 )
	{
		RVCString& defPath = mDefualtDestinationDirectory + "\\";
		defPath += file;
		defPath += ".";
		defPath += ext;

		return defPath;
	}
	else if( mGlobalDefualtDestinationDirectory.length() > 0 )
	{
		RVCString& defPath = mGlobalDefualtDestinationDirectory + "\\";
		defPath += file;
		defPath += ".";
		defPath += ext;

		return defPath;
	}
#endif

	return "";
}
