/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/***********************************************************************
/* Name : netevent.cpp
/* Purpose : the net-event implementation.
/*
/* Description :
/*
/* Creation: Yahali, 16-Feb-95
/* Updates :  Who          When         What
/*
/***********************************************************************/
#ifdef __TV_WINDOWS__
//#include "stdafx.h"
#endif
#include <malloc.h>
#include <string.h>
#include <stdlib.h>

#include "tvdebug.h"
#include "netmsg.h"

#define TESTTALK_MESSAGE_VERSION 352

/**********************  TestTalkEvent Functions **********************/

TestTalkMessage::TestTalkMessage(void* buf, long l, short t, short s_t) :
  type(t), subtype(s_t)
{
  unPackMe(buf, l);
}

TestTalkMessage::TestTalkMessage( long msgId,
                                  const char* th,
																  const char* tc,
                                  const char* fh,
                                  const char* fc,
                                  QType qt,
                                  short mt,
                                  short st,
                                  void* d,
                                  long  l ) :
  m_MessageID(msgId), m_MessageVersion(TESTTALK_MESSAGE_VERSION),
	toHost(th), toClient(tc), fromHost(fh), fromClient(fc),
	qtype(qt), type(mt), subtype(st), data(d), datalen(l),
	packedBuf(NULL)
{
	CHECK(data != NULL);
	CHECK(datalen != 0);
	packMe();
}

TestTalkMessage::TestTalkMessage(TypedMessageObject& tmo)
{
  type = tmo.type;
	subtype = tmo.subtype;
  packedBuf = tmo.MsgBuf;
  packedBufLen = tmo.len;
  tmo.MsgBuf = NULL;
  delete &tmo;

  reMapFields();
}

TestTalkMessage::TestTalkMessage() : packedBuf(NULL)
{
}

TestTalkMessage::~TestTalkMessage()
{
  if (packedBuf != NULL)
  {
    free((char*)packedBuf);
	packedBuf=NULL;
  }
 }


void TestTalkMessage::reMapFields()
{
	Pack msgPack(packedBuf);
	mapFields(msgPack);
}

void TestTalkMessage::mapFields(Pack& msgPack)
{
	toHost   = (const char*)msgPack.field(0);
	toClient = (const char*)msgPack.field(1);
	fromHost = (const char*)msgPack.field(2);
	fromClient = (const char*)msgPack.field(3);
	qtype = (QType)atoi((const char*)msgPack.field(4));
	data = msgPack.field(5);
	datalen = msgPack.fieldLen(5);
  if (msgPack.getNumOfPacks() == 8) {
    m_MessageID = atoi((const char*)msgPack.field(6));
    m_MessageVersion = atoi((const char*)msgPack.field(7));
  } else {
    m_MessageID = TESTTALK_MESSAGE_ILLEGAL;
    m_MessageVersion = TESTTALK_MESSAGE_ILLEGAL;
  }
}

void TestTalkMessage::packMe()
{
	if (packedBuf == NULL)
		{
			char cqtype[12];
			sprintf(cqtype,"%d",qtype);
			char cmsgId[12];
			sprintf(cmsgId,"%d",m_MessageID);
			char cMsgVersion[12];
			sprintf(cMsgVersion,"%d",m_MessageVersion);
			Pack msgPack((void*)((const char*)toHost), strlen(toHost)+1,
									 (void*)((const char*)toClient), strlen(toClient)+1,
									 (void*)((const char*)fromHost), strlen(fromHost)+1,
									 (void*)((const char*)fromClient), strlen(fromClient)+1,
									 (void*)&cqtype, strlen(cqtype)+1,
										data, datalen,
									 (void*)&cmsgId, strlen(cmsgId)+1,
									 (void*)&cMsgVersion, strlen(cMsgVersion)+1,
                    NULL);
			packedBuf = msgPack.thePackedMsg();
			packedBufLen = msgPack.thePackedMsgLen();
			//
			// re-directing the data pointer.
			mapFields(msgPack);
		}
}

void TestTalkMessage::rePackMe(const char* th, const char* tc)
{
	toHost = th;
	toClient = tc;
	void* temp = packedBuf;
	packedBuf = NULL;
	packMe();
	free((char*)temp);
}

void TestTalkMessage::unPackMe(void* buf, long l)
{
  // Set flag to zero
	assert(buf != NULL);
	//
	// copy the buffer.
	packedBufLen = l;
	packedBuf = malloc(packedBufLen);
	memcpy(packedBuf, buf, packedBufLen);
	//
	// re-map the fields.
	reMapFields();
}

const char* TestTalkMessage::printOnString()
{
  // Set flag to zero
	static char* buf = new char[4096];
	sprintf(buf,"%s: , Message len:%d, type (%d,%d), QType=%d from %s@%s to %s@%s\n",
					nameOf(), datalen, type, subtype,qtype,
					(const char*)fromClient, (const char*)fromHost,
					(const char*)toClient, (const char*)toHost);
return buf;
}

