/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "socketlib.h"
#include "socketsmanager.h"

//-********************************************************************
SocketInfo::SocketInfo(){
   wVersion=0;
   wHighVersion=0;
   szDescription[0]=0;
   szSystemStatus[0]=0;
   iMaxSockets=0;
   iMaxUdpDg=0;
   lpVendorInfo=0;
}



//-********************************************************************
//The SocketManager constructor takes a major and minor version as parameters.
//  These version parameters are the Winsock versions that you request, as
//  in WSAStartup() calls.
SocketManager::SocketManager(short nNewVersionMajor, short nNewVersionMinor, short bAutoStartup){
   bAvailable    = 1;
   nStartupCount = 0;
   ISocketManager(nNewVersionMajor, nNewVersionMinor);
   if(bAutoStartup)
      Startup();
}

//The SocketManager destructor cleans up after itself.  For every time you
//  called Startup() (same as WSAStartup()), this destructor will call
//  ShutDown (same as WSACleanup()).
SocketManager::~SocketManager(){
   while(nStartupCount){
      ShutDown();  //Call ShutDown() enough times to equal times Startup() was called.
   }
}

//The ISocketManager function does some ititialization for the SocketManager.
//  This function is separate from the constructor so that you can call it at
//  any time in order to re-specify the desired major and minor versions before
//  a call to Startup().
//nNewVersionMajor and nNewVersionMinor will be the values specified in the
//  Winsock WSAStartup() call that SocketManager::Startup() makes.
//The return value is TRUE or FALSE.
SocketManager::ISocketManager(short nNewVersionMajor, short nNewVersionMinor){
   nLastError    = 0;
   nVersionMajor = nNewVersionMajor;
   nVersionMinor = nNewVersionMinor;
   return 1;
}

//The Startup() function is equivalent to the Winsock WSAStartup() function.
//This function fills out the SocketManager's SocketInfo structure (same as a
//  WSAData structure) with the return information.
//You can then call the GetMaxSocketsAvailable(), GetVendorInfo(), Available(),
//  GetMajorVersion(), GetMinorVersion(), or Information() functions to get the
//  results of the call.
SocketManager::Startup(){
   WORD wPackedValues = (((WORD)nVersionMinor)<< 8)|(WORD)nVersionMajor;
   nLastError = WSAStartup(wPackedValues, &mySocketInfo); //sockets call
   if(nLastError)
      return WINSOCK_ERROR;
   nStartupCount++;
   nVersionMajor = HIBYTE(mySocketInfo.wVersion);
   nVersionMinor = LOBYTE(mySocketInfo.wVersion);
   return WINSOCK_NOERROR;
}

//The ShutDown() function is the equivalent to the Winsock WSACleanup() function.
//You can call this function if you want, but in its destructor, the SocketManager
//  will automatically call it once for every time Startup() was called.  Of course,
//  the SocketManager cannot know about any independent WSAStartup() direct calls
//  that you may make.
SocketManager::ShutDown(){
   if(WSACleanup()){   //sockets call
      nLastError = CSocketLib::GetLastError();  //sockets call
      return WINSOCK_ERROR;
   }
   nStartupCount--;
   return WINSOCK_NOERROR;
}

