/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include <assert.h>
#include "socketlib.h"
#include "streamsocket.h"
#ifdef __TV_UNIX__
#include <strings.h>
#include <string.h>
#include <errno.h>
extern int errno;
#define h_errno errno
#define WSAEWOULDBLOCK EWOULDBLOCK
#endif


//-******************************************************************************************
StreamSocket::StreamSocket()
   : Socket(), nConnectStatus(nNotConnected)
{
}


StreamSocket::StreamSocket(SOCKET& newS)
   : Socket(newS)
{
   //Important:  Unless there is a way I haven't found yet, you can't tell
   //  what the nConnectStatus is, given just a SOCKET descriptor.
   //Needs to be set somehow:
   //nConnectStatus = ???;
}



StreamSocket& StreamSocket::operator=(StreamSocket& newStreamSocket)
{
   Socket::operator=(newStreamSocket);
   return *this;
}


int StreamSocket::Listen(int nMaxQueuedConnections){
//This function puts this socket into a passive listening mode.
   int nReturnValue;

   nReturnValue = listen(m_Socket, nMaxQueuedConnections);
   if(nReturnValue){
      nLastError = CSocketLib::GetLastError(); 
      return WINSOCK_ERROR;
   }
   nConnectStatus = nListening;
   return WINSOCK_NOERROR;
}


int StreamSocket::Connect(u_short port)
{
  char localhost[N_MAX_HOST_NAME];

  if (!CSocketLib::GetLocalHostName(localhost, sizeof(localhost)))
  {
    nLastError=CSocketLib::GetLastError();  
    return WINSOCK_ERROR;
  }
  return Connect(localhost, port);
}


int StreamSocket::Connect(const char* hostname, u_short port) {
  int nReturnValue;
  if(!CSocketLib::GetAddress(hostname, port, myPeerSocketAddress))
  {
    nLastError=CSocketLib::GetLastError();  
    return WINSOCK_ERROR;
  }

  nReturnValue = connect(m_Socket, (sockaddr*)&myPeerSocketAddress, sizeof(sockaddr)); 
  if(nReturnValue){
    nLastError = CSocketLib::GetLastError(); 
    return WINSOCK_ERROR;
  }
  nConnectStatus = nConnected;
  return WINSOCK_NOERROR;
}


int StreamSocket::Accept(StreamSocket& socket){
//This function will try to accept a connection with the first connecting peer that
//  is waiting in the queue.  If successful, the StreamSocket& 'socket' will have a
//  valid and connected socket, a proper status of nConnected, and the correct
//  peer socket address.
//The caller will usually call this function in response to an Accept notification.
//  The caller merely needs to create a new StreamSocket and pass it to this
//  function. The default constructor for StreamSocket can be used, as this function
//  fixes up the missing parts.    
   int nReturnValue;

   nReturnValue = Accept(socket.m_Socket, socket.myPeerSocketAddress);
   if(nReturnValue == WINSOCK_NOERROR){
      socket.nConnectStatus = nConnected;
      socket.SetSocketStyle(nFamily, nType, nProtocol); //Copy our nFamily, etc. just to make sure.
   }
   return nReturnValue;
}


int StreamSocket::Accept(SOCKET& socket, sockaddr_in& sAddress){
//This function will try to accept a connection with the first connecting peer that is
//  waiting in the queue.  If successful, the 'socket' reference argument will be set to
//  a new connected socket.  The sAddress reference argument will get set to the address
//  of the connecting peer.  The caller of this function may immediately use the
//  new socket with data sends, etc.  Note that the caller may want to flag the socket
//  as connected.  If the 'socket' belongs to a 'StreanSocket', then you can set its
//  nConnectionStatus as nConnected.
//The return value is either WINSOCK_ERROR or WINSOCK_NOERROR.  If there is an error,
//  then nLastError will be set with the appropriate error.
//Note that it is possible that this call could be made when there are no pending
//  socket connecions in the queue.  If this is the case, the call will block
//  if the socket is marked as blocking, and will return WINSOCK_ERROR with
//  WSAEWOULDBLOCK if the socket is marked as non-blocking.
//This function is usually called in response to an accept notification.  A socket is
//  set up as a stream socket and listen() is called.  When a connection is ready, the
//  driver notifies the listening socket with a DoAcceptNotification() call.  See the
//  DoAcceptNotification() call for what to do with it.  You will want to call this Accept()
//  function as a result.
  SOCKET_LEN nAddressLength = sizeof(sockaddr);
#ifdef __TV_UNIX__
  if( ! SelectForRead() )
    return WINSOCK_ERROR;
#endif
  
   socket = accept(m_Socket, (sockaddr*)&sAddress, &nAddressLength);  
   if(socket == INVALID_SOCKET){
      nLastError = CSocketLib::GetLastError();  
      return WINSOCK_ERROR;
   }
   return WINSOCK_NOERROR;
}

//**********************************************************************
// Name : ReadData.
// Purpose : 
//
// Input :  
//
// Output : none.
//
// Return values: 
//
// Creation: Yahali, 21-Nov-94
// Updates :  Who          When         What
//
//**********************************************************************/
int StreamSocket::ReadData(void* data, int len)
{
  int rdFlags = 0;
  int charsToRecv = len;
  char* pb = (char*)data;
  int nchars;

  //
  // check the socket's validity.
  assert(data != NULL);
  assert(len > 0);

  //
  while (charsToRecv>0)
  {
#ifdef __TV_UNIX__
    if( ! SelectForRead() )
      return SOCKET_CLOSED;
#endif
      if ((nchars = recv(m_Socket, pb, charsToRecv, rdFlags)) <= 0)
			{
				if (nchars==0) // means the m_Socket closed.
					{
						//debugPrint(2,("Socket %d closed",m_Socket));
						return SOCKET_CLOSED;
					}
				else
					{
						if (h_errno == WSAEWOULDBLOCK)
				    {
					    return len-charsToRecv;
				    }
	          else
				    {
              nLastError = CSocketLib::GetLastError(); 
					    return WINSOCK_ERROR;
				    }
    	    }
		  }
      charsToRecv -= nchars;
      pb += nchars;
      //debugPrint(3,("Read %d chars on socket %d\n",nchars,m_Socket));
    }
      
  return len;
}

//**********************************************************************
// Name : WriteData
// Purpose : 
//
// Input :  
//
// Output : the number of bytes actually written.
//
// Return values: 
//
// Creation: Yahali, 21-Nov-94
// Updates :  Who          When         What
//                          
//**********************************************************************/
int StreamSocket::WriteData(void* data, int len)
{
  int charsAlreadySent = 0;
  int nchars;
  int wrFlags = 0;
  char* pb = (char*)data;

  assert(len > 0);
  assert(data != NULL);
  
  while (charsAlreadySent < len)
    {
      if ((nchars=send(m_Socket, (const char*)pb, len-charsAlreadySent, wrFlags)) < 0)
	    {
	      if (h_errno == WSAEWOULDBLOCK)
	        {
	          return charsAlreadySent;
	        }
	      else
	        {
	          //debugPrint(0,("Error after sending: (%d,%d) err: %d.\n", charsAlreadySent,len,h_errno));
            nLastError = CSocketLib::GetLastError(); 
	          return WINSOCK_ERROR;
	        }
	    }
      charsAlreadySent += nchars;
      pb += nchars;
    }
  
  return len;
}

//**********************************************************************
// Name : SendMessage
// Purpose : 
//
// Input :  
//
// Output :Success = 0, error-code: negative.
//
// Return values: 
//
// Creation: Yahali, 21-Nov-94
// Updates :  Who          When         What
//                          
//**********************************************************************/
int StreamSocket::SendMessage(void* msg, int len)
{
  return SendTypedMessage(0, 0, msg, len);
}

int StreamSocket::SendTypedMessage(short type, short subtype, void* msg, int len)
{
  MessageHeader mh;
  int wrst;
  void* msgbuf;
  char* pbuf;
  int msgsize;

  assert(len >= 0);

  SSMMakeMessageHeader(len, type, subtype, &mh);
  
  //
  // create the buffer for the message.
  msgsize = len + sizeof(MessageHeader);
  msgbuf = malloc(msgsize);
  pbuf = (char*)msgbuf;
  memcpy(pbuf, (char*)&mh, sizeof(MessageHeader));
  pbuf += sizeof(MessageHeader);
  if (len>0 && msg != NULL)
      memcpy(pbuf, (char*)msg, len);

  /* write the message. */
  wrst = WriteData(msgbuf, msgsize);
  if (wrst < 0)  
    return wrst;

	free(msgbuf);

  return 0;
}

//
// get the length of the message from socket.
int StreamSocket::GetMessageHeader(MessageHeader *mh)
{
  int rdst;

  // get the message length
  rdst = ReadData((void*)mh, sizeof(MessageHeader));
  if (rdst != sizeof(MessageHeader))
    return WINSOCK_ERROR;

  return (SSMGetMessageHeaderLen(*mh));
}

//**********************************************************************
// Name : WaitMessage
// Purpose : 
//
// Input :  
//
// Output :Success = 0, error-code: negative.
//
// Return values: 
//
// Creation: Yahali, 21-Nov-94
// Updates :  Who          When         What
//                          
//**********************************************************************/
int StreamSocket::WaitTypedMessage(short& type, short& subtype, void*& msg, int& len)
{
  MessageHeader mh;
  int inMsgLen;
  int rdst;

  inMsgLen = (int)GetMessageHeader(&mh);

  if (inMsgLen<0)
    return inMsgLen;

  if (inMsgLen > 0)
    {
      void* pbuf = malloc(inMsgLen);

      rdst = ReadData(pbuf, inMsgLen);
      if (rdst!=inMsgLen)
				return WINSOCK_ERROR;
      msg = pbuf;
    }

  if (&len != NULL)
    len     = inMsgLen;
  if (&type != NULL)
    type    = SSMGetMessageHeaderType(mh);
  if (&subtype != NULL)
    subtype = SSMGetMessageHeaderSubtype(mh);

  return(WINSOCK_NOERROR);
}

void StreamSocket::FreeMessage(void* msg) {
  free(msg);
}


#ifdef __TV_UNIX__
int  StreamSocket::SelectForRead()
{
  fd_set  read_fs,write_fs,except_fs;

  if(m_Socket < 0)
    return FALSE;

  FD_ZERO(&read_fs);
  FD_ZERO(&write_fs);
  FD_ZERO(&except_fs);

  FD_SET(m_Socket, &read_fs);
  FD_SET(m_Socket, &except_fs);
  if( (select(m_Socket+1, &read_fs, &write_fs, &except_fs, NULL)) < 0 ||
      FD_ISSET(m_Socket, &except_fs) )
    return FALSE;
  else
    return TRUE;
}

#endif
  
/*

int StreamSocket::Read(char* chData, int& nCharsToRead){
   //Try to receive the characters.
   int nCharsReceived;
   nCharsReceived = recv(m_Socket, chData, nCharsToRead, 0);  //I Need to allow the use of flags here.
   if(nCharsReceived == SOCKET_ERROR){
      //It is entirely possible to get a blocking error here.  For example, a call to
      //  recv() could have been made right after the message was posted by the driver,
      //  but before this notification function got called.
      nCharsToRead = 0; //This will be available for the caller to examine.
      nLastError = CSocketLib::GetLastError(); 
      if(nLastError == WSAEWOULDBLOCK)
         return WINSOCK_NOERROR;
      //Some other error occurred.
      return WINSOCK_ERROR;
   }
   nCharsToRead = nCharsReceived;
   return WINSOCK_NOERROR;
}


int StreamSocket::Write(char* chData, int& nCharsToWrite, int nFlags,
                        short bBecomeOwnerOfData, short bCopyData)
{
   int nCharsSent;
   nCharsSent = send(m_Socket, chData, nCharsToWrite, nFlags);  
   if(nCharsSent == SOCKET_ERROR){
      //It is entirely possible to get a blocking error here.  For example, a call to
      //  send() could have been made right after the message was posted by the driver,
      //  but before this notification function got called.
      nCharsToWrite = 0;
      nLastError = CSocketLib::GetLastError(); 
      if(nLastError == WSAEWOULDBLOCK){
         return WINSOCK_NOERROR;
      }
      return WINSOCK_ERROR;
   }
   nCharsToWrite = nCharsSent;
   return WINSOCK_NOERROR;
}


int StreamSocket::ReadOOB(char* chData, int& nCharsToRead){
//Works just like the Read() function but it works on the OOB queue.
   #ifdef B_USE_DATA_QUEUES
   return ReadQueue(myIncomingOOBQueue, chData, nCharsToRead);

   #else //#ifdef B_USE_DATA_QUEUES
   //Try to receive the characters.
   int nCharsReceived;                    //I Need to allow the use of flags here.
   nCharsReceived = recv(m_Socket, chData, nCharsToRead, MSG_OOB); 
   if(nCharsReceived == SOCKET_ERROR){
      //It is entirely possible to get a blocking error here.  For example, a call to
      //  recv() could have been made right after the message was posted by the driver,
      //  but before this notification function got called.
      nCharsToRead = 0;
      nLastError = CSocketLib::GetLastError(); 
      if(nLastError == WSAEWOULDBLOCK)
         return WINSOCK_NOERROR;
      //Some other error occurred.
      return WINSOCK_ERROR;
   }
   nCharsToRead = nCharsReceived;
   return WINSOCK_NOERROR;
   #endif //B_USE_DATA_QUEUES
}


int StreamSocket::WriteOOB(char* chData, int& nCharsToWrite, int nFlags,
#ifdef B_USE_DATA_QUEUES
                        short bBecomeOwnerOfData, short bCopyData){
#else
                        short bBecomeOwnerOfData, short bCopyData){
#endif
//ORs nFlags w MSG_OOB. Otherwise works just like StreamSocket::Write().
   #ifdef B_USE_DATA_QUEUES
   nFlags |= MSG_OOB;
   return WriteQueue(myOutgoingOOBQueue, chData, nCharsToWrite, nFlags, bBecomeOwnerOfData, bCopyData);

   #else //#ifdef B_USE_DATA_QUEUES
   int nCharsSent;
   nFlags |= MSG_OOB;
   nCharsSent = send(m_Socket, chData, nCharsToWrite, nFlags);  
   if(nCharsSent == SOCKET_ERROR){
      //It is entirely possible to get a blocking error here.  For example, a call to
      //  send() could have been made right after the message was posted by the driver,
      //  but before this notification function got called.
      nCharsToWrite = 0;
      nLastError = CSocketLib::GetLastError(); 
      if(nLastError == WSAEWOULDBLOCK){
         return WINSOCK_NOERROR;
      }
      return WINSOCK_ERROR;
   }
   nCharsToWrite = nCharsSent;
   return WINSOCK_NOERROR;
   #endif //B_USE_DATA_QUEUES
}

*/

