/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
/* Copyright (c) 1993-1995, Radview Software LTD. All Rights Reserved. */
/****************************************************************
 *
 * File: tvenv.h
 * Purpose: header file for the tvenv module library.
 *
 * Who      When        What
 * ????     ????        Create
 * Yahali   5-Dec-94    porting to UNIX.
 *
 ****************************************************************/
#ifndef __TVENV_H
#define __TVENV_H

#include "platform.h"

#define TV_TOP "TV_TOP"
#define TV_BIN "TV_BIN"
#define TV_FRM "TV_FRM"
#define TV_ "TV_"

#ifdef WIN32
#ifdef TVENV_EXPORTS
#define TVENVAPI __declspec(dllexport)
#else //not TVENV_EXPORTS
#define TVENVAPI __declspec(dllimport)
#endif //TVENV_EXPORTS
#else //not WIN32
#define TVENVAPI 
#endif //WIN32

TVENVAPI BOOL        WINAPI TVEInit( const char *tvpath );
TVENVAPI const char* WINAPI TVEError();


//TVENVAPI void        WINAPI TVPFatalErrorLog( const char* format, ... );

TVENVAPI const char* WINAPI TVPGetFullName( const char* Old, char* New, int len );
TVENVAPI BOOL        WINAPI TVPAbsPath( const char *path, char *newpath, int len );

TVENVAPI int         WINAPI TVPIsClosedFileADirectory( const char* fileName );

/*
The function 'TVPCreateDirectory' creates a directory according to the path parameter.
It will succeed only if all the directories are exist except the last directory of the path parameter.
If the parameter path is an existing directory the function return true.
*/
TVENVAPI bool        WINAPI TVPCreateDirectory( /*[in]*/const char* path );


/*
The function 'TVPGetTempDirectory' returns the temp directory of the computer. it get it from the system environment
variables. If it doesn't find the "TEMP" environment variable, it returns "".
You don't need to make another copy of the string. you can use the returned pointer.
*/
TVENVAPI const char* WINAPI TVPGetTempDirectory();
TVENVAPI void        WINAPI TVPGetCurrentDir( char *path, int len );
TVENVAPI const char* WINAPI TVPGetUserDir();
TVENVAPI const char* WINAPI TVPGetTempName( const char *pathvar );
TVENVAPI const char* WINAPI TVPGetOSInfo();
TVENVAPI size_t      WINAPI TVPGetWorkingSetSize();

TVENVAPI BOOL        WINAPI TVPCopyFile( const char *FromFileName, const char *ToFileName );

TVENVAPI int         WINAPI TVPChdir( const char* newDir );
TVENVAPI const char* WINAPI TVPGetCwd();

TVENVAPI void        WINAPI TVPRemoveAllDirectories( /*[in]*/const char* path );
// ((((((((((((((((((    BE CAREFUL    ))))))))))))))))))))

TVENVAPI const char* WINAPI TVPGetOSInfo();


class TVENVAPI TVEnvContainerClass
{
private:
  char* m_TVPResolvePathVarNoTVEnv_result;
  char m_searchInPath_result[MAX_PATH];
  char m_TVPGetUserDir_result[MAX_PATH];
  char m_TVPGetCwd_buf[MAX_PATH];
  char m_TVPSplitPath_dirname[MAX_PATH];
  char m_TVPSplitPath_filename[MAX_PATH];
  char m_TVPSplitPath_extname[MAX_PATH];
  char m_TVPMakePath_pathname[MAX_PATH];
  char m_TVERuntimeResolve_retVal[2048];
  char m_TVPGetTempName_name[MAX_PATH];
  char m_TVPGetMyDocuments_result[MAX_PATH];

public: 
  TVEnvContainerClass();
  ~TVEnvContainerClass();
  void reset();

private:
  const char* searchInPath(const char* fileName, const char* Path);

public:
  const char* WINAPI TVPAdjustPathLen(const char* oldpath, char* newpath, int len);
  int         WINAPI TVPFileExist(const char* fileName);
  const char* TVPPathFileExist(const char* dir, const char* fname);
  const char* TVPResolvePathVarNoTVEnv(const char* pathvar);
  const char* WINAPI TVPResolveFilePathNoTVEnv(const char* fileName, const char* pathvar);
  const char* WINAPI TVPSearchPath(const char* fileName);
  const char* WINAPI TVPGetUserDir();
  const char* WINAPI TVPGetMyDocuments();
  const char* WINAPI TVPFindfileNoTVEnv(const char* fileName,const char* where,...);
  const char* WINAPI TVPGetCwd();
  const char* WINAPI TVPGetPathDirectory(const char* path);
  const char* WINAPI TVPGetPathName(const char* path);
  const char* WINAPI TVPGetPathExt(const char* path);
  void        WINAPI TVPSplitPath(const char* path, char** dir, char** fname, char** ext);
  const char* WINAPI TVPMakePath(const char* dir, const char* fname, const char* ext);
  const char* WINAPI TVPConcatDirAndName(const char* dir, const char* fname);
  const char* WINAPI TVPReplacePathExt(const char* path, const char* newext);

public:
  const char* WINAPI TVERuntimeResolve(const char *str);
  const char* WINAPI TVEGetEnv( const char *entry );
  const char* WINAPI TVEGetEnvValue( const char *entry );
  void        WINAPI TVESetEnvValue( const char *entry, const char* value );
  bool        WINAPI TVEEnvDefined(const char *entry,const char* value,bool ignore_case =  true);

public:
  const char* WINAPI TVPGetTempName(const char *pathvar);
  const char* WINAPI TVPFindfile(const char* fileName,const char* where,...);
  const char* WINAPI TVPGetFullName(const char* Old,char* New,int len);
  const char* WINAPI TVPResolveFilePath(const char* fileName, const char* pathvar);

private:
  const char* TVPResolvePathVar(const char* pathvar);
};


class TVENVAPI CFindFileFunctor
{
public:
	virtual bool handleFile( const char* path ) = 0;
};

TVENVAPI bool WINAPI FindFile( const char* findPath, const char* spec, bool lookInSubdirectories, CFindFileFunctor* pObj );


#endif /* __TVENV_H */

