/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
// severr.cpp -- Server Error mechanism
//
//  who         when          what
//  Yahali      26-May-1996   Create
//
//
#include <collstr.h>
#include <RVCTokenizer.h>
#include <assert.h>
#include <stdio.h>
#include <stdarg.h>
#include <fstream.h>
#include <typeinfo>

//#include "tracedll.h"
#include "tvpsync.h"
#include "wlerrorlib.h"
#include "wlerrorlibimp.h"
#include "tvenv.h"

//********************** WLLogMessage Class Implementation ********************************

static const char* LogMessageNames[] = {
  "Success",
  "Note",
  "Warning",
  "Error",
  "Fatal"
};

static const char* LogMessageLongNames[] = {
  "Success",
  "Info Message",
  "Warning",
  "Error Message",
  "Severe Error"
};

const char* WLLogMessage::Name(Severity severity)
{
  return LogMessageNames[severity];
}

const char* WLLogMessage::LongName(Severity severity)
{
  return LogMessageLongNames[severity];
}

//********************** Global Function Implementation ********************************

static WLErrorsDictionary localErrDict;
DllExport void WLERRTerminate()
{
  VOID2DICT(localErrDict.dict)->clearAndDestroy();
  delete VOID2DICT(localErrDict.dict);
}

DllExport int WLERRLoadErrorString(const char* errFileName, const char* data)
{
  int ret;
  try {
    ret = localErrDict.LoadErrorString(errFileName, data);
  } 
  catch (...) {
    ret = false;
  }
  return ret;
}

DllExport const char* WLERRGetErrorFormat(const char* errFileName, int fmtId)
{
  const char* ret;
  try {
    ret = localErrDict.GetErrorFormat(errFileName, fmtId);
  }
  catch (WLErrorException *e) {
		
		//if error id not found add to dictionary and try again 
		
		RVCString err;
		char cFmtId[20];
		itoa(fmtId,cFmtId,10);
		err=cFmtId;
		
		if(!stricmp(errFileName,"protocol_httpresponse.frm"))
		{
			//for custom http errors "Custom Http error. Requested URL: %s ";
			err+=" e ";
			err+="Custom Http error ";
			err+=cFmtId;
			err+=" Requested URL: %s";
			
			
			
		}
		else
		{
			//for other errors
			err+=" e ";
			err+= e->GetErrorMsg();
		}
		delete e;
		try
		{
			localErrDict.LoadErrorString(errFileName,err.data());
			ret = localErrDict.GetErrorFormat(errFileName, fmtId);
		}
		catch (...) {
			ret = "Fatal error occured in WLERRGetErrorFormat";
		}
		
		
  }
  catch (...) {
    ret = "Fatal error occured in WLERRGetErrorFormat";
  }

  return ret;
}

DllExport const char* WLERRGetifErrorFormat(const char* errFileName, int fmtId)
{
  const char* ret;
  try {
    ret = localErrDict.GetErrorFormat(errFileName, fmtId);
  }
  catch (WLErrorException *e) {
    ret = NULL;
		delete e;
  }
  catch (...) {
    ret = "Fatal error occured in WLERRGetErrorFormat";
  }

  return ret;
}

DllExport WLLogMessage::Severity WLERRGetErrorSeverity(const char* errFileName, int fmtId)
{
  WLLogMessage::Severity ret;
  try {
    ret = localErrDict.GetErrorSeverity(errFileName, fmtId);
  } 
  catch (WLErrorException *e) {
    ret = WLLogMessage::Fatal;

//for custom http errors
    if(!stricmp(errFileName,"protocol_httpresponse.frm"))
	 ret = WLLogMessage::Error;
    	
		delete e;
  }
  catch (...) {
    ret = WLLogMessage::Fatal;
  }
  return ret;
}

DllExport const char* WLERRGetErrorString(const char* errFileName, int fmtId, char* errBuf, unsigned int errBufLen, ...)
{
  const char* errFmt = WLERRGetErrorFormat(errFileName, fmtId);
  if (errFmt) {
    va_list ap;
    va_start(ap, errBufLen);
    vsprintf(errBuf, errFmt, ap);
    va_end(ap);
	  assert(strlen(errBuf) <= errBufLen);
  } else {
    throw new WLErrorException("Error id %d  not found in dictionary %s", WLLogMessage::Error, fmtId, errFileName);
  }

  return errBuf;
}

void WLERRGetErrorMsg(std::exception* pException, RVCString& pString){
	pString = "";
//	pString += typeid(*pException).name();
//	pString += "(";
	pString += pException->what(); 
//	pString += ")";
}

bool WLERRGetStackTrace(std::exception* pException, RVCString& pString){
	pString = "";
	if (dynamic_cast<iWLException*>(pException))
		return static_cast<iWLException*>(pException)->StackTrace(pString);
	else
		return true; // No error
}
////////////////  WLErrorsDictionary Implementation ///////////////////////

static TVPMutex fileMutex;

WLErrorsDictionary::WLErrorsDictionary()
{
  dict=(void*) new RVHashDictionary;
}

WLErrorsDictionary::~WLErrorsDictionary()
{
  //delete VOID2DICT(dict);
}

int WLErrorsDictionary::LoadErrorFile(const char* errFileName)
{
  RVCollectableString name(errFileName);
  WLFileDictionary *fdict;
  int ret = false;
  
  //TraceSendText("WLErrorsDictionary::LoadErrorFile - Loading error file (%s)", errFileName);

  TVEnvContainerClass tvpc;
  if ((fdict = (WLFileDictionary*) VOID2DICT(dict)->findValue((RVCollectable *) &name)) == NULL)
  {
    fdict = new WLFileDictionary(errFileName);
  	VOID2DICT(dict)->insertKeyAndValue((RVCollectable *) new RVCollectableString(errFileName), 
	  		                               (RVCollectable *) fdict);
    RVCString realFileName = tvpc.TVPFindfile(errFileName, "#TV_FRM", NULL);
    ret = fdict->LoadErrorFile(realFileName);
  } 
    
  return ret;
}

int WLErrorsDictionary::LoadErrorString(const char* errFileName, const char* data)
{
	int res=0; 
  RVCollectableString name(errFileName);
  WLFileDictionary *fdict;
	fileMutex.Lock();
	try{
  
  if ((fdict = (WLFileDictionary*) VOID2DICT(dict)->findValue((RVCollectable *) &name)) == NULL)
  {
    fdict = new WLFileDictionary(errFileName);
  	VOID2DICT(dict)->insertKeyAndValue((RVCollectable *) new RVCollectableString(errFileName), 
	  		                               (RVCollectable *) fdict);
		} 
		//else
		// return false;
		if( fdict)
			res=fdict->LoadErrorString(data);
	}
	catch(...)
	{
		fileMutex.Unlock();
		throw;
	}
	fileMutex.Unlock();
	return res; 
}

const char* WLErrorsDictionary::GetErrorString(const char* errFileName, int fmtId, char* errBuf, unsigned int errBufLen, ...)
{
  const char* errFmt = GetErrorFormat(errFileName, fmtId);
  if (errFmt) {
    va_list ap;
    va_start(ap, errBufLen);
    vsprintf(errBuf, errFmt, ap);
    va_end(ap);
	  assert(strlen(errBuf) <= errBufLen);
  } else {
    throw new WLErrorException("Error id %d  not found in dictionary %s", WLLogMessage::Error, fmtId, errFileName);
  }

	return errBuf;
}

const char* WLErrorsDictionary::GetErrorFormat(const char* errFileName, int fmtId)
{
  RVCollectableString name(errFileName);
  WLFileDictionary *fdict;
  
  const char* errFmt;
  fileMutex.Lock();
  try {
  if ((fdict = (WLFileDictionary*) VOID2DICT(dict)->findValue((RVCollectable *) &name)) == NULL) 
  {
    (void)LoadErrorFile(errFileName);
    fdict = (WLFileDictionary*) VOID2DICT(dict)->findValue((RVCollectable *) &name);
    //throw new WLErrorException("Error dictionary %s not found", Error, errFileName);
  }

    errFmt = fdict->GetErrorFormat(fmtId);
  }
  catch (...){
    fileMutex.Unlock();
    throw;
  }
  
  fileMutex.Unlock();
	return errFmt;
}

WLLogMessage::Severity WLErrorsDictionary::GetErrorSeverity(const char* errFileName, int fmtId)
{
  RVCollectableString name(errFileName);
  WLFileDictionary *fdict;
  
  fileMutex.Lock();
  try {
  if ((fdict = (WLFileDictionary*) VOID2DICT(dict)->findValue((RVCollectable *) &name)) == NULL)
  {
    (void)LoadErrorFile(errFileName);
    fdict = (WLFileDictionary*) VOID2DICT(dict)->findValue((RVCollectable *) &name);
    //throw new WLErrorException("Error dictionary %s not found", Error, errFileName);
  }
  }
  catch (...){
    fileMutex.Unlock();
    throw;
  }

  fileMutex.Unlock();
  return fdict->GetErrorSeverity(fmtId);
}

////////////////  WLFileDictionary Implementation ////////////////////////

WLFileDictionary::WLFileDictionary(const char* errFileName) : name(errFileName)
{
}

WLFileDictionary::~WLFileDictionary() 
{
  clearAndDestroy();
}

void WLFileDictionary::InsertErrorLine(const char* errLine)
{
  //TraceSendText("Loading Error line: (%s)", errLine);

  RVCString    line(errLine);
  RVCTokenizer next(line);
  RVCString    fmtId, fmtType, fmt;
  fmtId = next();
  fmtType = next();
  fmt = next("");

  int id = atoi(fmtId);
  insertKeyAndValue((RVCollectable *) new RVCollectableInt(id), 
                    (RVCollectable *) new WLErrorFileEntry(fmt, fmtType));

}

int WLFileDictionary::LoadErrorFile(const char* fileName)
{
  //TraceSendText("WLFileDictionary::LoadErrorFile - Loading error file (%s)", fileName);

  ifstream in(fileName);
  if(!in) {
    throw new WLErrorException("Cannot open error file : %s",WLLogMessage::Fatal, fileName);
  }

  char errLine[1024];
  RVCString errBuf;
  while (in.getline(errLine,sizeof(errLine))) {
    int len = strlen(errLine);
    if (len>0) {
      if (errLine[len-1]=='\\') {
        //TraceSendText("Continues line: (%s)", errLine);
        errBuf.append(errLine,len-1);
        errBuf.append("\n");
      } else {
        //TraceSendText("Last line: (%s)", errLine);
        errBuf.append(errLine,len);
        //TraceSendText("Loading buf: (%s)", (const char*)errBuf);
        InsertErrorLine(errBuf);
        errBuf.remove(0);
      }
    }
  }
  return 1;
}

int WLFileDictionary::LoadErrorString(const char* data)
{
  RVCString    input(data);
  RVCTokenizer next(input);
  RVCString    line;
  while (!(line = next("\n")).isNull()) {
    InsertErrorLine(line);
  }
  return 1;
}

const char* WLFileDictionary::GetErrorFormat(int fmtId)
{
  RVCollectableInt id(fmtId);
  WLErrorFileEntry *err;

  if ((err = (WLErrorFileEntry*) findValue((RVCollectable *) &id)) == NULL) {
    throw new WLErrorException("Format id %d not found in dictionary %s", WLLogMessage::Error, fmtId, (const char*)name);
  }

  return err->errorFmt;
}

WLLogMessage::Severity WLFileDictionary::GetErrorSeverity(int fmtId)
{
  RVCollectableInt id(fmtId);
  WLErrorFileEntry *err;

  if ((err = (WLErrorFileEntry*) findValue((RVCollectable *) &id)) == NULL) {
    throw new WLErrorException("Format id %d not found in dictionary %s", WLLogMessage::Error, fmtId, (const char*)name);
  }

  return err->severity;
}


////////////////  WLErrorFileEntry Implementation ////////////////////////

WLErrorFileEntry::WLErrorFileEntry(const char* fmt, const char *sev) : errorFmt(fmt)
{
  switch (*sev) {
  case 'n':
  case 'N':
    severity = WLLogMessage::Note;
    break;
  case 'w':
  case 'W':
    severity = WLLogMessage::Warning;
    break;
  case 'e':
  case 'E':
    severity = WLLogMessage::Error;
    break;
  case 'f':
  case 'F':
    severity = WLLogMessage::Fatal;
    break;
  default:
    severity = WLLogMessage::Fatal;
    break;
  }
}

  
////////////////  iWLException Implementation ////////////////////////

iWLException::iWLException()
{
}

iWLException::~iWLException() RV_EXCEPTION_THROW
{
}

const char* iWLException::what() const RV_EXCEPTION_THROW
{

	return GetErrorMsg();
}


////////////////  WLErrorException Implementation ////////////////////////

WLErrorException::WLErrorException(const char* msg, WLLogMessage::Severity sev, ...)
: severity(sev),
  alreadyReported(false)
{
  char errBuf[100024];
  va_list ap;
  va_start(ap, sev);
  vsprintf(errBuf, msg, ap);
  va_end(ap);
  errorMsg = errBuf;
}

WLErrorException::WLErrorException(bool reported, const char* msg, WLLogMessage::Severity sev, ...)
: severity(sev),
  alreadyReported(reported)
{
  char errBuf[100024];
  va_list ap;
  va_start(ap, sev);
  vsprintf(errBuf, msg, ap);
  va_end(ap);
  errorMsg = errBuf;
}

WLErrorException::WLErrorException(const char* errFileName, int fmtId, ...):
  alreadyReported(false)
{
  const char* errFmt = WLERRGetErrorFormat(errFileName, fmtId);
  char errBuf[100024];
  va_list ap;
  va_start(ap, fmtId);
  vsprintf(errBuf, errFmt, ap);
  va_end(ap);
  errorMsg = errBuf;
  severity = WLERRGetErrorSeverity(errFileName, fmtId);
}

WLErrorException::WLErrorException(bool reported, const char* errFileName, int fmtId, ...):
  alreadyReported(reported)
{
  const char* errFmt = WLERRGetErrorFormat(errFileName, fmtId);
  char errBuf[100024];
  va_list ap;
  va_start(ap, fmtId);
  vsprintf(errBuf, errFmt, ap);
  va_end(ap);
  errorMsg = errBuf;
  severity = WLERRGetErrorSeverity(errFileName, fmtId);
}

  // @@ofer: the format of copy ctor should be X(const X&) but since WebRM 4.6 uses the same dlls we cannot modify the interface
  // in 6.0 we should use X(const X&) for ALL platforms
#ifdef __TV_UNIX__
WLErrorException::WLErrorException(const WLErrorException& errorException) :
#else
WLErrorException::WLErrorException(WLErrorException& errorException) :
#endif // __TV_UNIX__
  errorMsg(errorException.GetErrorMsg()), 
  severity(errorException.GetErrorSeverity())
{
}


WLErrorException::~WLErrorException() RV_EXCEPTION_THROW
{
}

const char* WLErrorException::GetErrorMsg() const
{
  return errorMsg.data();
}

WLLogMessage::Severity WLErrorException::GetErrorSeverity() const
{
  return severity;
}

bool WLErrorException::StackTrace(RVCString& outBuf, unsigned skip, const char* fmt, unsigned int maxDepth){
	outBuf = "";
	return true;
}


////////////////  WLErrorExceptionWithStackTrace Implementation ////////////////////////

WLErrorExceptionWithStackTrace::WLErrorExceptionWithStackTrace(const char* msg, WLLogMessage::Severity sev, unsigned int maxDepth)
: WLErrorException(msg, sev)
{
	SetStackTrace(maxDepth);
}

// @@ofer: the format of copy ctor should be X(const X&) but since WebRM 4.6 uses the same dlls we cannot modify the interface
// in 6.0 we should use X(const X&) for ALL platforms
#ifdef __TV_UNIX__
WLErrorExceptionWithStackTrace::WLErrorExceptionWithStackTrace(const WLErrorExceptionWithStackTrace& errorException) :
#else
WLErrorExceptionWithStackTrace::WLErrorExceptionWithStackTrace(WLErrorExceptionWithStackTrace& errorException) :
#endif // __TV_UNIX__
	m_stackTrace(errorException.m_stackTrace),
	WLErrorException(errorException)
{
}

bool WLErrorExceptionWithStackTrace::StackTrace(RVCString& outBuf, unsigned skip, const char* fmt, unsigned int maxDepth){
	outBuf = m_stackTrace;
	return true;
}

WLErrorExceptionWithStackTrace::~WLErrorExceptionWithStackTrace() RV_EXCEPTION_THROW
{
}

void WLErrorExceptionWithStackTrace::SetStackTrace(unsigned int maxDepth){
		m_stackTrace = "Call stack unavailable";
	}

////////////////  CWLErrorInfo Implementation ////////////////////////

void CWLErrorInfo::SetError(int p_nErrorId, ...)
{
  const char* sErrorFormat = WLERRGetErrorFormat(m_sErrorFile, p_nErrorId);
  char sErrorBuffer[100024];
  va_list ap;
  va_start(ap, p_nErrorId);
  vsprintf(sErrorBuffer, sErrorFormat, ap);
  va_end(ap);

  m_nErrorId = p_nErrorId;
  m_sErrorMsg = sErrorBuffer;
  m_nErrorSeverity = WLERRGetErrorSeverity(m_sErrorFile, p_nErrorId);
}
 
void CWLErrorInfo::Clear()
{
  m_nErrorId = 0;
  m_sErrorMsg = "";
  m_nErrorSeverity = WLLogMessage::Success;

  return;
}

  
const char*
CWLErrorInfo::GetErrorMsg()
{
  if (m_nErrorId != 0 && m_sErrorMsg.isNull())
    SetError(m_nErrorId);
  
  return m_sErrorMsg.data();
}

WLLogMessage::Severity
CWLErrorInfo::GetErrorSeverity()
{
  if (m_nErrorId != 0 && m_sErrorMsg.isNull())
    SetError(m_nErrorId);
    
  return m_nErrorSeverity;
}

