/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __TV_TVPSYNC_H
#define __TV_TVPSYNC_H

#include <stdlib.h>
#include "platform.h"

#ifdef __TV_WINDOWS__
#include <sys/types.h>
#include <sys/timeb.h>
#include <windows.h>
#include <process.h>
#else
#ifdef __TV_linux__
#include <sys/time.h>
#else
#include <thread.h>
#include <synch.h>
#endif
#include <unistd.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#endif

#include <thr.h>
#include <thr.h>
#include <thr.h>
#include <thr.h>
#include <thr.h>

typedef unsigned ThrFunc(void *);

class DllExport TVPSysThread {
public:
  typedef enum {
    Minimal,
    Low,
    BelowNormal,
    Normal,
    AboveNormal,
    High,
    Critical
  } PriorityLevel;
private:
  RVThread *m_Thread;

public:
  TVPSysThread();
  ~TVPSysThread();
  int create(ThrFunc *start_addr, void *arg, int suspend_flag=0);
  //int suspend();
  //int resume();
	BOOL setPriority(int hPriority);
	static bool setPriority(PriorityLevel priorityLevel);
  //void end(unsigned retval);
  //unsigned long getId();
};


class DllExport TVPMutex {
protected:
  RVCriticalSection *mutex;
public:
  TVPMutex();
  ~TVPMutex();

  int Lock();
  int Unlock();
};
 

class DllExport TVPSafeFlag {
private:
  RVCriticalSection *m_Mutex;
	bool               m_FlagValue;

public:
	TVPSafeFlag(bool value = false);
	~TVPSafeFlag();

	bool Set(bool newVal = true);
	bool Get();
};

class DllExport TVPSafeCounter {
private:
  RVCriticalSection *m_Mutex;
	long               m_Counter;

public:
  class DllExport IncrementGuard {
    private:
      TVPSafeCounter& m_MySC;
    public:
      IncrementGuard(TVPSafeCounter& sc);
      ~IncrementGuard();
  };

public:
	TVPSafeCounter(long initValue = 0);
	~TVPSafeCounter();

	long Increment();
	long Decrement();
	long Get();
  long Reset();
};

class DllExport TVPManualEvent {
private:
  RVMutexLock       *m_Mutex;
  RVCondition       *m_Condition;
  bool              m_Flag;

public:
  class DllExport SetGuard {
    private:
      TVPManualEvent& m_MyME;
    public:
      SetGuard(TVPManualEvent& me);
      ~SetGuard();
  };

public:
  TVPManualEvent(); 
  ~TVPManualEvent();

  RVWaitStatus Wait();
  RVWaitStatus Wait(long waitTime);
  bool Set();
  bool Get() { return m_Flag; }
  bool Reset();
};

class DllExport TVPTimerObject {
private:
#ifdef __TV_WINDOWS__
  struct timeb mytime;
#else
  struct timeval mytimeval;
#endif
 public:
  TVPTimerObject();
  void reset();
  double get();
};

#ifdef __TV_UNIX__
#define MAX_SIG  30

class TVPSignalHandler {
 public:
  TVPSignalHandler();
  TVPSignalHandler( const char *disp );
  ~TVPSignalHandler();
  void TVPSigSet(int sig, void (*disp)(int));
  void TVPSigReset(int sig);


private:
  int  mask[MAX_SIG];  // the status of the signal
};

void reportSignal(int sig);

#define SIG_DEFAULT  0
#define SIG_IGNORE   1
#define SIG_REPORT   2
#define SIG_HANDLER  3

// Environment Variable to hold signal MASK

#define ENV_SIGNAL_MASK  "TV_SIGNAL_MASK"

#endif
#endif // __TV_TVPSYNC_H

