/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef __TV_UNIX__
#include <crtdbg.h>
#endif

#include "tvpeventqueue.h"
//#include "tracedll.h"


TVPEventQueue::TVPEventQueue() 
{
  m_Q = new RVSlistCollectablesQueue;
  m_Touch = new RVMutexLock;
  m_Condition  = new RVCondition(*m_Touch);
  Init();
}

TVPEventQueue::~TVPEventQueue() 
{
  Terminate();
  delete m_Q;
  delete m_Touch;
  delete m_Condition;
}

void TVPEventQueue::Init()
{
  RVMutexLock::LockGuard lock(*m_Touch);
  m_Q->clearAndDestroy();
  m_Released = false;
  m_Active = true;
}

void TVPEventQueue::Terminate()
{
  RVMutexLock::LockGuard lock(*m_Touch);
  m_Active = false;
  m_Released = true;
  m_Q->clearAndDestroy();
	m_Condition->signalAll();
}

void TVPEventQueue::release()
{
  RVMutexLock::LockGuard lock(*m_Touch);
  m_Released = true;
	m_Condition->signalAll();
}

void TVPEventQueue::putEvent(RVCollectable* event)
{
  RVMutexLock::LockGuard lock(*m_Touch);
  if (!m_Active || m_Released)
		return;
	m_Q->append((RVCollectable *)event);
	m_Condition->signal();
}

RVCollectable* TVPEventQueue::waitEvent()
{
  RVMutexLock::LockGuard lock(*m_Touch);

	//
	// wait for the message
  while (m_Active && !m_Released && m_Q->isEmpty()) {
  	m_Condition->wait();
  }

  if (!m_Active || (m_Released && m_Q->isEmpty()))
		return NULL;

	RVCollectable* e = (RVCollectable *)m_Q->get();
	return e;
}
	
TVPEventQueue::WaitReturnType TVPEventQueue::waitEvent(RVCollectable*& event, long waitTime)
{
  RVMutexLock::LockGuard lock(*m_Touch);
  event = NULL;

	//
	// wait for the message
  while (m_Active && !m_Released && m_Q->isEmpty()) {
  	RVWaitStatus status = m_Condition->wait(waitTime);
    if (status == RV_THR_TIMEOUT)
	  if (!m_Q->isEmpty()){
		event = (RVCollectable *)m_Q->get();
		return Success;
	  }
	  else {
      return TimedOut;
  }
  }

  if (!m_Active || (m_Released && m_Q->isEmpty())) {
    return Released;
  }

	event = (RVCollectable *)m_Q->get();
	return Success;
}

RVCollectable* TVPEventQueue::get()
{ 
  RVMutexLock::LockGuard lock(*m_Touch);
  if (!m_Active)
		return NULL;

  return m_Q->get(); 
}
															  
bool TVPEventQueue::isEmpty() const
{ 
  RVMutexLock::LockGuard lock(*m_Touch);
  return m_Q->isEmpty(); 
}

