/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
///////////////////////////////////////////////////////////////////////////////
//
// (c) 2000 GalTech Soft. All rights reserved.
//
// UniServAPIs.h
// Provides 32 bit declarations of general 
// Unicode Service Library API entities. 
///////////////////////////////////////////////////////////////////////////////

#ifndef _UNISERVAPI_H
#define _UNISERVAPI_H
#include "portable.h"

#ifdef __cplusplus
extern "C" {
#endif

/*The UL_IsValidCodePage determines whether a specified code
 page is valid.*/

BOOL ULAPI UL_IsValidCodePage(UINT CodePage);

/*The UL_MultiByteToWideChar function maps a character string to
a wide-character (Unicode) string. The character string mapped by
this function is not necessarily from a multibyte character set.
This  function is similar to MultiByteToWideChar, but doesn't 
depends on locale of OS

Return Values:
If the function succeeds, and cchWideChar is nonzero, the return
value is the number of wide characters written to the buffer pointed
to by lpWideCharStr. 
If the function succeeds, and cchWideChar is zero, the return value
is the required size, in wide characters, for a buffer that can 
receive the translated string. For perfomance issues UL_StrLength,
could be used
*/


int ULAPI UL_MultiByteToWideChar(
	UINT CodePage,         // code page
	DWORD dwFlags,         // character-type options
	LPCSTR lpMultiByteStr, // string to map
	int cbMultiByte,       // number of bytes in string
	LPWSTR lpWideCharStr,  // wide-character buffer
	int cchWideChar,        // size of buffer
	int *pnBytesUsed = NULL,
	int *piCharMode = NULL		// char mode if escape sequence encodings
);

/*Return Values:
If the function succeeds, the return value is the required size,
in wide characters, for a buffer that can receive the translated string. 
*/

int ULAPI UL_WideCharToMultiByte(
	UINT CodePage,         // code page
	DWORD dwFlags,         // character-type options
	LPCWSTR lpWideCharStr,  // wide-character buffer
	int cchWideChar,        // size of buffer
	LPSTR lpMultiByteStr, // string to map
	int cbMultiByte,       // number of bytes in string
    LPSTR   lpDefaultChar,
    LPBOOL   lpUsedDefaultChar
);


int ULAPI UL_StrLength(
	UINT CodePage,			//  code page 
    LPCSTR lpSrcCharStr,    // source (string) buffer pointer
    int ccSrc,				// bytes in Src
	int *pnBytesUsed = NULL
);

int UL_SJis2Jis(LPSTR lpSJis, int ccSJis,
	LPSTR lpJis, int ccJis, int *iCharMode
);


int UL_SJis2Euc(LPSTR lpSJis, int ccSJis, 
	LPSTR lpEuc, int ccEuc
);


int UL_MultiToMulti(UINT SrcCodePage, 
	UINT TrgetCodePage, 
	LPCSTR lpSource, 
	int cbSrc, 
	LPSTR lpTarget, 
	int cbTarget
);

int ULAPI UL_WideCharToUTF8(
	LPCWSTR lpWideCharStr,  // wide-character string to map buffer
	int cchWideChar,        // size of string
	LPSTR lpUTF8ByteStr, // string buffer
	int cbUTF8Byte       // number of bytes in string

);

int ULAPI UL_UTF8ToWideChar(
        LPCSTR lpUTF8ByteStr, // string buffer
        int cbUTF8Byte,       // number of bytes in string
        LPWSTR lpWideCharStr,  // wide-character string to map buffer
        int cchWideChar        // size of string

	);
int ULAPI UL_WideCharToUTF7(
	LPCWSTR lpWideCharStr,  // wide-character string to map buffer
	int cchWideChar,        // size of string
	LPSTR lpUTF7ByteStr, // string buffer
	int cbUTF7Byte       // number of bytes in string
);

/*The UL_CodePageNameToID function maps the charset values of HTML/XML, to
Code Page values
*Return Values: 
If the function succeeds, the return value is the code page of HTML/XML
*/

UINT ULAPI UL_CodePageNameToID(LPCSTR lpStr);

/*The UL_CodePageIDToName function maps the code page to the default
charset values of HTML/XML
*Return Values: 
If the function succeeds, the return value is the char set value of HTML/XML
*/

BOOL ULAPI UL_CodePageIDToName(UINT CodePage, LPSTR lpStr);

UINT ULAPI UL_JapaneseAutoDetect(LPCSTR str, int ccStr, BOOL *pbDetected);

int ULAPI UL_GetCharSet(long lang);

#ifdef WIN32
LOGFONT  ULAPI *UL_RealFont();
  int CALLBACK   RetFont(
  const LOGFONT *lpelfe,    // logical-font data
  const TEXTMETRIC *lpntme,  // physical-font data
  DWORD FontType,           // type of font
  LPARAM lParam             // application-defined data
);
#endif //WIN32

#ifdef __cplusplus
}
#endif

#endif //_UNISERVAPI_H
