/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
///////////////////////////////////////////////////////////////////////////////
//
// (c) 2000 GalTech Soft. All rights reserved.
//
// UniServ.h
// Internal miscelenious functions 
// Unicode Library. 
///////////////////////////////////////////////////////////////////////////////

#ifndef _UNISERV_H
#define _UNISERV_H


#define BEG_URANGE_1    ((WORD) 0x0000)  /* beg of interesting UniCode range #1 */
#define END_URANGE_1    ((WORD) 0x07FF)  /* end of interesting UniCode range #1 */
#define BEG_URANGE_2    ((WORD) 0x2000)  /* beg of interesting UniCode range #2 */
#define END_URANGE_2    ((WORD) 0x27FF)  /* end of interesting UniCode range #2 */
#define BEG_URANGE_3    ((WORD) 0xFE00)  /* beg of interesting UniCode range #3 */
#define END_URANGE_3    ((WORD) 0xFFFF)  /* end of interesting UniCode range #3 */
#define MAX_16_BITS     END_URANGE_2 //((WORD) 0x2800)  /* 0x2700 intresting part (for now) of UniCode table */


typedef enum { eUnknown = 0, eWindows, eUnix, eMac, eLinux }
	e_MajorOS;

typedef enum { eUnknownMinor = 0, eWin95, eWin98, eWin98based, eWinNT,
				eWin2000, eWin2000based, eUnixMinor, eMacMinor }
	e_MinorOS;

typedef struct _tagOSLevel{
	e_MajorOS m_MajorOS;
	e_MinorOS m_MinorOS;
} OSLevel;

typedef enum  {eUnknownConvert = 0, eOSConvert, eUniConvert}
	e_Convert;

typedef enum  {eConvFailed = 0, eConvFinished, eConvForward, eConvRegular, eConvertAfter}
	e_ConveresionResult;

typedef struct _RANGE {
    WORD start;  // first number of range
    WORD end;    // last number of range
    WORD offset; // offset of this range from the start of the CodeRecs array
} RANGE;

typedef struct _CODEREC {
    WORD code;
//    WORD unicode;
} CODEREC, *PCODEREC;


typedef BOOL  (WINAPI *FPTRENUM)(int iIterator, UINT *CodePage,
		char* szResourceName, BOOL* pbMultiTable);

typedef struct _tagCONVERT{
	UINT CodePage;
	e_Convert ConvertType;
	BOOL bMultiTable;

	// variables for single-table transformations (like Latin1)
	//From Unicode:
	BYTE  *Uni_To_THIS_1;// from Unicode part 1
	size_t SizeOfUni_To_THIS_1;
	BYTE  *Uni_To_THIS_2;// from Unicode part 2
	size_t SizeOfUni_To_THIS_2;

	//To Unicode:
	WORD    *THIS_To_Uni;   // current to Unicode
	size_t SizeOfTHIS_To_Uni;

	// variables for multi-table transformations (like Shift-JIS)
	//From Unicode:
	int     nDstRanges;
	CODEREC    *DstCodeRecs;   /* actual code page records */
	DWORD SizeOfDstCodeRecs;
	RANGE     *DstRanges;
	DWORD SizeOfDstRanges;

	//To Unicode:
	int     nRanges;
	WORD *CodeRecs;   /* actual code page records */
	DWORD SizeOfCodeRecs;
	RANGE   *Ranges;
	DWORD SizeOfRanges;
} CONVERT;


BOOL EnumCodePages(int iIterator, UINT *CodePage,
		char* szResourceName, BOOL* pbMultiTable);

BOOL _Initialize();

BOOL _IsValidCodePage(UINT CodePage);

CONVERT* GetConversionRecord(UINT CodePage);

int ConvertWithErrors(
	UINT SrcCodePage,		// source code page 
	DWORD dwFlags,			// character-type options 	
    LPCSTR lpSrcCharStr,    // source (string) buffer pointer
    int ccSrc,				// bytes in Src
    LPWSTR lpDstCharStr,     // destination buffer pointer
    int cchWideChar				// output buffer limit
	);

BOOL GetRealCodepage(UINT& SrcCodePage, LPCSTR lpSrcCharStr,
			int ccSrc);

int CheckForAlgorithmConversion(UINT& SrcCodePage, LPCSTR lpSrcCharStr,
		int ccSrc, LPSTR* pString, e_ConveresionResult *peResult, 
		int *piCharMode);

int MultiToDest(LPSTR lpszDst, gt_wchar_t u, int nMaxWChars, BOOL bNormalize, CONVERT *pConvert);

int ToDest(LPSTR lpszDst, gt_wchar_t u,int nMaxWChars, BOOL bNormalize);


int UL_Jis2SJis(LPCSTR lpJis, int ccJis,
	LPSTR lpSJis, int ccSJis, int *iCharMode);

int UL_Euc2SJis(LPCSTR lpEuc, int ccEuc,
	LPSTR lpSJis, int ccSJis);

int UL_strlen(LPSTR str, UINT CodePage, int iLength, int *pnBytesUsed = 0);

int UL_DBCSstrlen(LPSTR str, UINT CodePage, int iLength, int *pnBytesUsed = 0);

BOOL UL_IsLeadByte(BYTE uByte, UINT CodePage);

int UL_EscapeSeqstrlen(LPSTR str, UINT CodePage, int iLength, int *pnBytesUsed = 0);


#ifdef WIN32
UINT GetCPFromLanguage(WORD langid);

UINT GetCurrentThreadCP();
#endif //WIN32


#define USV_INFORMATION_LIBRARY_EXT		"usv"
#define USV_LIBRARY_NAME				"uniserv.dll"

#endif // _UNISERV_H
