/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
///////////////////////////////////////////////////////////////////////////////
//
// (c) 2000 GalTech Soft. All rights reserved.
//
// UniServAPIs.cpp
// Provides  API for code page conversions.
// 
///////////////////////////////////////////////////////////////////////////////

#include <stdio.h>

#ifdef __TV_WINDOWS__
#include "windows.h"
#else //UNIX
#include "strings.h"
#endif

#define ID_FILE ID_FILE_UNICODEAPI

#include "gt_portable.h"
#include "gt_unidef.h"
#include "gt_uniservapis.h"
#include "uniserv.h"
#include "utfcode.h"
#include "gt_portable.h"
#include "gt_unicodectype.h"

//////////////////////////////
#define KOREAN 949
#define JAPANESE 932
#define CHINESE_T 950
#define CHINESE_S 936


#define KOREAN_CHARSET 129
#define JAPANESE_CHARSET 128
#define CHINESE_T_CHARSET 136
#define CHINESE_S_CHARSET 134
////////////////////////////////


#ifdef __TV_WINDOWS__
BOOL WINAPI DllEntryPoint(HINSTANCE hinstDll, DWORD fdwReason, LPVOID lpReserved)
{
    switch (fdwReason) {

      case DLL_PROCESS_ATTACH:
		break;
      case DLL_THREAD_ATTACH:
        break;
      case DLL_PROCESS_DETACH:
        break;
      case DLL_THREAD_DETACH:
        break;
    }
    return(TRUE);
}
#endif

BOOL ULAPI UL_IsValidCodePage(UINT CodePage)
{
	return _IsValidCodePage(CodePage);
}

int UL_MultiToMulti(UINT SrcCodePage, UINT TrgetCodePage, LPCSTR lpSource, 
					int cbSrc, LPSTR lpTarget, int cbTarget)
{
	if (SrcCodePage == TrgetCodePage)
	{
		strncpy(lpTarget, lpSource, cbSrc);
		lpTarget[cbSrc] = '\0';
		return cbSrc;
	}

	int nRet = 0;

	switch (SrcCodePage)
	{
	case CP_EUCJP:
		if (TrgetCodePage == CP_MS_JAPANESE)
			UL_Euc2SJis(lpSource, cbSrc, lpTarget, cbTarget);
		else if (TrgetCodePage == CP_JIS)
		{
			char *pTmp = new char[cbTarget+1];
			int nSize = UL_Euc2SJis(lpSource, cbSrc, pTmp, cbTarget);
			int nCharMode = 1;
			nRet = UL_SJis2Jis(pTmp, nSize, lpTarget, cbTarget, &nCharMode);
			if ( pTmp)
			delete [] pTmp;
		}
		break;
	case CP_JIS:
		break;
	case CP_MS_JAPANESE:
		int nCharMode = 1;
		if (TrgetCodePage == CP_JIS)
			nRet = UL_SJis2Jis((LPSTR)lpSource, cbSrc, lpTarget, cbTarget, &nCharMode);
		else if (TrgetCodePage == CP_EUCJP)
			nRet = UL_SJis2Euc((LPSTR)lpSource, cbSrc, lpTarget, cbTarget);
		break;
	}
	return nRet;
}

int ULAPI UL_MultiByteToWideChar(
	UINT CodePage,         // code page
	DWORD dwFlags,         // character-type options
	LPCSTR lpMultiByteStr, // string to map
	int cbMultiByte,       // number of bytes in string
	LPWSTR lpWideCharStr,  // wide-character buffer
	int cchWideChar,        // size of buffer
	int *pnBytesUsed,
	int *piCharMode			// char mode if escape sequence encoding
)
{
	//If there is the code page for detection detect it
	GetRealCodepage(CodePage, lpMultiByteStr, cbMultiByte);
		
#ifdef _NORMALIZE	
	BOOL bNormalize = dwFlags & MB_PRECOMPOSED;
#else
	BOOL bNormalize = 0;
#endif
	int nNormalizedLen = 0;
	gt_wchar_t* pStringNormalized = NULL;

	//if the destination length is 0, the caller wan to get 
	//just a length of the string in WCHARs
	if(cchWideChar == 0 /* && (!bNormalize) */)
		return UL_strlen((LPSTR)lpMultiByteStr, CodePage, cbMultiByte, pnBytesUsed);
	else if (pnBytesUsed)  // set pnBytesUsed
		UL_strlen((LPSTR)lpMultiByteStr, CodePage, cbMultiByte, pnBytesUsed);

	e_ConveresionResult eResult;
	char* pString = NULL;
	BOOL bDeleteString = FALSE;

	int nLen = CheckForAlgorithmConversion(CodePage, lpMultiByteStr,
			cbMultiByte, &pString, &eResult, piCharMode);
	switch(eResult)
	{
		case eConvFailed:
			return 0;
		case eConvFinished:
			bDeleteString = TRUE;
			if(bNormalize)
			{
				nNormalizedLen = w_normalize((LPWSTR)pString, 
					nLen /sizeof(gt_wchar_t), NULL, 0);
				pStringNormalized = new gt_wchar_t[nNormalizedLen + 1];
				w_normalize((LPWSTR)pString, 
					nLen /sizeof(gt_wchar_t), pStringNormalized, nNormalizedLen + 1);
				nLen = (nNormalizedLen + 1) * sizeof(gt_wchar_t);
				if(bDeleteString)
				{
					if (pString)
						delete [] pString;
					bDeleteString = FALSE;
				}
				pString = (char*)pStringNormalized;
			}
			if(cchWideChar != 0)
			{
				nLen = min((signed)nLen, (signed)sizeof(gt_wchar_t) * (cchWideChar));
				memcpy(lpWideCharStr, pString, nLen);
			}
			if(bDeleteString)
			{
				if (pString)
					delete [] pString;
				bDeleteString = FALSE;
			}
			if(pStringNormalized)
			{
				
				delete [] pStringNormalized;
				pStringNormalized = NULL;
			}
			return nLen / sizeof(gt_wchar_t) ;
		case eConvForward:
			bDeleteString = TRUE;
			break;
		case eConvRegular:
			pString = (LPSTR)lpMultiByteStr;
			break;
		default : return FALSE;
	}
	if(nLen != 0)
		cbMultiByte = nLen;
	else if (cbMultiByte != -1 && pnBytesUsed && *pnBytesUsed != -1)
		cbMultiByte = *pnBytesUsed;

	int iResult = 0;
	CONVERT* pConvert = GetConversionRecord(CodePage);
	if(!pConvert || pConvert->ConvertType == eOSConvert)
	{
#ifdef __TV_UNIX__
		iResult = UL_MultiByteToWideChar(CodePage, 0, pString,
			cbMultiByte, lpWideCharStr, cchWideChar);
#else //__TV_WINDOWS__
		iResult = MultiByteToWideChar(CodePage, 0, pString,
			cbMultiByte, lpWideCharStr, cchWideChar);
#endif //UNIX
		if(bDeleteString)
		{
			if (pString) 
				delete []pString;
			bDeleteString = FALSE;
		}
		return iResult;
	}

	BOOL    bMultiTable = pConvert->bMultiTable;

/*	iResult = ConvertWithErrors(CodePage, 0, pString,
		cbMultiByte, lpWideCharStr, cchWideChar);

	if(bDeleteString)
	{
		if (pString)
			delete [] pString;
		bDeleteString = FALSE;
	}
	return iResult / sizeof(gt_wchar_t);
*/
	BYTE a1;
	WORD u;
	LPSTR  lpszSrc = (LPSTR)pString;
	LPSTR  lpszDst =  (LPSTR)lpWideCharStr;
	int nin = 0, nCount = 0;
	unsigned char hi, lo;
	BOOL bNullTermination = cbMultiByte == -1 ? TRUE : FALSE;
    while ((nin < cbMultiByte ||  (bNullTermination && *lpszSrc != '\0')) && 
		(cchWideChar - iResult > 0 || cchWideChar == 0)) 
	{
        if (bMultiTable) 
		{
            hi = *lpszSrc++;
            ++nin;
			if (UL_IsLeadByte(hi, CodePage))
			{
				if ((bNullTermination && *lpszSrc=='\0') || 
							(!bNullTermination && (nin >= cbMultiByte)))
					break; // a truncated char - don't convert
				else
				{  // 2 byte char
					lo = *lpszSrc++;
					nin++;
					u = UL_MAKEWORD(lo,hi); //(WORD)(((WORD)(hi & 0x00FF) << 8) & (WORD)0xFF00) |
							//	(WORD)(lo & 0x00FF);
					nCount = MultiToDest(lpszDst, u, 
						cchWideChar ? cchWideChar - iResult : 0, bNormalize, pConvert);
				}
			}
            else 
			{
                u = (WORD)(hi & 0x00FF);
				nCount = MultiToDest(lpszDst, u, 
						cchWideChar ? cchWideChar - iResult : 0, bNormalize, pConvert);
//                nCount = ToDest(lpszDst, u, 
//					cchWideChar ? cchWideChar - iResult : 0, bNormalize);
            }

        }
        else
		{
			int j;

            a1 = *lpszSrc++;
            ++nin;
			nCount = 0;
			for (j = 0; j < MAX_CODES; j++) 
			{
                u  = pConvert->THIS_To_Uni[(a1 * MAX_CODES) + j];
                if (u == 0 && j > 0)
                    continue;   // ligature support
//		nCount += ToDest(lpszDst, u,
//				 cchWideChar ? cchWideChar - iResult : 0, bNormalize);

		// Chaim fix
		if (bNormalize){
				nCount += ToDest(lpszDst, u,
					cchWideChar ? cchWideChar - iResult : 0, bNormalize);
		}

		else if (lpszDst){
			*((LPWSTR) lpszDst) = (u?u:'?');
//			lpszDst += sizeof(gt_wchar_t);
			nCount += 1;
		}

		
            }   // for
		}     
		lpszDst += nCount * sizeof(gt_wchar_t);
		iResult += nCount;

    }           // while
	if(bNullTermination && cchWideChar - iResult >= 1)
	{
		//*lpszDst++ = '\0';
		//*lpszDst = '\0';
		*((LPWSTR)lpszDst) = '\0';
		iResult ++;
	}
	if(bDeleteString)
	{
		if (pString)
			delete []pString;
		bDeleteString = FALSE;
	}
    return iResult ;
}

int ULAPI UL_WideCharToUTF8(
	LPCWSTR lpWideCharStr,  // wide-character string to map buffer
	int cchWideChar,        // size of string
	LPSTR lpUTF8ByteStr, // string buffer
	int cbUTF8Byte       // number of bytes in string

)
{
	return Ucs2ToUtf8(lpWideCharStr, cchWideChar, 
		lpUTF8ByteStr, cbUTF8Byte);

}

int ULAPI UL_UTF8ToWideChar(
	LPCSTR lpUTF8ByteStr, // string buffer
	int cbUTF8Byte,       // number of bytes in string
	LPWSTR lpWideCharStr,  // wide-character string to map buffer
	int cchWideChar        // size of string

)
{
	return Utf8ToUcs2	( lpUTF8ByteStr, cbUTF8Byte,lpWideCharStr, cchWideChar);

}


int ULAPI UL_StrLength(
	UINT SrcCodePage,		// source code page 
    LPCSTR lpSrcCharStr,    // source (string) buffer pointer
    int ccSrc,				// bytes in Src
	int *pnBytesUsed
)
{
	return UL_strlen((LPSTR)lpSrcCharStr, SrcCodePage, ccSrc, pnBytesUsed);
}

UINT ULAPI UL_CodePageNameToID(LPCSTR lpStr)
{
	csname2id_t	*csn2idp;

	csn2idp = &csname2id_tbl[1];	/* First one is reserved, skip it. */
	UINT CodePage = 0;
	/* Linear search for charset string */
	while (TRUE) 
	{
		if(csn2idp->cs_id == CP_LAST)
			break;
#ifdef __TV_UNIX__
		if(strcasecmp(csn2idp->cs_name, lpStr)==0)
#else //__TV_WINDOWS__
		if(stricmp(csn2idp->cs_name, lpStr)==0)
#endif //UNIX
		{
			CodePage = csn2idp->cs_id;
			break;
		}
 		csn2idp++;
	}
	return CodePage;
}
BOOL ULAPI UL_CodePageIDToName(UINT CodePage, LPSTR lpStr)
{
	csname2id_t	*csn2idp;

	csn2idp = &csname2id_tbl[1];	/* First one is reserved, skip it. */
	BOOL bResult = FALSE;
	/* Linear search for charset string */
	while (TRUE) 
	{
		if(csn2idp->cs_id == CP_LAST)
			break;
		if(CodePage == csn2idp->cs_id)
		{
			strcpy(lpStr, csn2idp->cs_name);
			bResult = TRUE;
			break;
		}
 		csn2idp++;
	}
	return bResult;

}

#if __TV_WINDOWS__

static LOGFONT lfLanguage;
static BOOL bFontSet = FALSE;


int ULAPI UL_GetCharSet(long lang)
{
	switch (lang)
	{
	case KOREAN:
		return KOREAN_CHARSET;
	case CHINESE_T:
		return CHINESE_T_CHARSET;
	case CHINESE_S:
		return CHINESE_S_CHARSET;
	case JAPANESE:
		return JAPANESE_CHARSET;
	default:
		return CP_DEFAULT;
	}
}

LOGFONT  ULAPI *UL_RealFont()
{
	if(bFontSet)	return &lfLanguage;//We search for real font only once

	LOGFONT lf;
	//	filling of logfont request
	memset(&lf,0,sizeof(LOGFONT));
	lf.lfCharSet = DEFAULT_CHARSET;
	//to enumerate all styles and character sets of the MS Sans Serif font:
	strcpy (lf.lfFaceName, TEXT("Microsoft Sans Serif"));
	EnumFontFamiliesEx(
		GetDC(NULL),                    // handle to DC
		&lf,					// font information
		RetFont,				// callback function
		UL_GetCharSet(GetACP()),                    // additional data
		0                     // not used; must be 0
	);


	if (bFontSet) 
		return &lfLanguage;
	else 
	{
		HFONT hSystem = (HFONT)::GetStockObject(DEFAULT_GUI_FONT);
   		int succ = GetObject(hSystem, sizeof lfLanguage, &lfLanguage); 
		if (succ)
		{
			bFontSet = TRUE;
			return &lfLanguage;
		}
		else // if "MS San Serif" does not exist fetch the default font
		{
			memset(&lf,0,sizeof(LOGFONT));
			lf.lfCharSet = DEFAULT_CHARSET;
			EnumFontFamiliesEx(
				GetDC(NULL),                    // handle to DC
				&lf,					// font information
				RetFont,				// callback function
				UL_GetCharSet(GetACP()),                    // additional data
				0                     // not used; must be 0
			);
   		GetObject(hSystem, sizeof lfLanguage, &lfLanguage); 
		bFontSet = TRUE;
				return &lfLanguage;
	}
}
}

////////////////////////////////////////////////////////////////
int  CALLBACK   RetFont(
	const LOGFONT *lpelfe,    // logical-font data
	const TEXTMETRIC *lpntme,  // physical-font data
	DWORD FontType,           // type of font
	LPARAM lParam             // application-defined data
)
{
	if (lpelfe->lfFaceName[0] != '@' && lpelfe->lfCharSet == lParam &&
		(lpelfe->lfPitchAndFamily & FIXED_PITCH) && lpelfe->lfOutPrecision != OUT_STRING_PRECIS)
	{
		memset(&lfLanguage, 0, sizeof LOGFONT);
		lfLanguage.lfHeight = -10;
		lfLanguage.lfWeight = 400;
		lfLanguage.lfCharSet = lpelfe->lfCharSet;
		strcpy(lfLanguage.lfFaceName, lpelfe->lfFaceName);
		bFontSet = TRUE;

		return 0;
	}
	return 1;
}
////////////////////////////////////////////////////////////

#endif // __TV_WINDOWS__
