/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  


#include "njsd.h"
#include "tracedll.h"

iVirtualLoadEngine* piVirtualLoadEngine = NULL;

void SetVirtualLoadEngine(iVirtualLoadEngine* _X0){
  piVirtualLoadEngine = _X0;
}

namespace nJs {


extern "C" {
	static void fsScriptHook(JSDContext*,JSDScript*,JSBool,void*);
	static uintN fsCallbackErrorReporter(JSDContext*,JSContext*,const char*,JSErrorReport*,void*);
	static uintN fsCallbackExecutionHook(JSDContext*,JSDThreadState*,uintN,void*,jsval*);
	static JSBool fsCallbackTopLevelHook(JSDContext*,JSDThreadState*,uintN type,void* callerdata);
	static JSBool fsCallbackFunctionHook(JSDContext*,JSDThreadState*,uintN type,void* callerdata);
}

iScript::iScript() {}

iScript::~iScript() {}

const char* iScript::fGetName() const
{
	return JSD_GetScriptFilename( fGetJSDC(), fGetJSDScript() );
}

iFile* iScript::fGetFile()
	const
{
	const char* aFilename = fGetName();
	iFile* rtn = ZTSEncriptionToFile( aFilename );
	return rtn;
}

tLineNumber iScript::fGetBaseLine()
	const
{
	return JSD_GetScriptBaseLineNumber( fGetJSDC(), fGetJSDScript() )-1;
}

tLineNumber iScript::fGetLineExtent()
	const
{
	return JSD_GetScriptLineExtent( fGetJSDC(), fGetJSDScript() );
}

const char* iScript::fGetFunctionName()
	const
{
	return JSD_GetScriptFunctionName( fGetJSDC(), fGetJSDScript() );
}



iScriptHookable::iScriptHookable() {}

iScriptHookable::~iScriptHookable() {}


cScriptHookable::cScriptHookable(
	iContext& Context,
	rcType Value )
:
	iScriptHookable(),
	jcHandlePointer(Value),
	mContext(Context)
{}

cScriptHookable::~cScriptHookable() {}

JSDScript* cScriptHookable::vGetJSDScript_iScript() 
	const
{
	return fGet();
}


JSDContext* cScriptHookable::vGetJSDC_iScript() const
{
	return mContext.fGetJSDC();
}


iDMThreadState::iDMThreadState(
	iContext& Context, 
	rcType Handle )
:
	jcHandlePointer( Handle ),
	mContext(Context)
{}

iDMThreadState::~iDMThreadState() {}

tStackFrameIndex iDMThreadState::fGetStackFramesCount() const
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return JSD_GetCountOfStackFrames( mContext.fGetJSDC(), fGet() );
}

iStackFrameLevel* iDMThreadState::fGetTopStackFrame() 
{
	cContextThreadAcquire aContextThreadAcquire( RVJSD_GetJSContext( fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	JSDStackFrameInfo* X = JSD_GetStackFrame( mContext.fGetJSDC(), fGet() );
	iStackFrameLevel* Rtn = ( X ? new iStackFrameLevel( *this, X ) : 0 );
	return Rtn;
}

iValue* iDMThreadState::fGetException() const
{
	cContextThreadAcquire aContextThreadAcquire( RVJSD_GetJSContext( fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	JSDValue* X = JSD_GetException( mContext.fGetJSDC(), fGet() );
	iValue* Rtn = ( X ? new cValueHold( mContext, *this, X ) : 0 );
	return Rtn;
}


cDMThreadState::cDMThreadState(
	iContext& Context, 
	rcType Handle )
:
	iDMThreadState( Context, Handle ) 
{}

cDMThreadState::~cDMThreadState() {}



iStackFrame::iStackFrame(
	iDMThreadState& ThreadState )
:
	mThreadState(ThreadState)
{}

iStackFrame::~iStackFrame() {}

iDMThreadState& iStackFrame::fGetThreadState() 
	const
{
	return mThreadState;
}


JSDScript* iStackFrame::vGetJSDScript_iScript() 
	const
{
	cContextThreadAcquire aContextThreadAcquire( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mThreadState.mContext.fGetDumbJSContext() );
	return JSD_GetScriptForStackFrame( mThreadState.mContext.fGetJSDC(), mThreadState.fGet(), fGet() );
}

JSDContext* iStackFrame::vGetJSDC_iScript() 
	const
{
	return mThreadState.mContext.fGetJSDC();
}



tLineNumber iStackFrame::fGetCurrentLineNumber() 
	const
{
/*
extern JSD_PUBLIC_API(jsuword)
JSD_GetPCForStackFrame(JSDContext* jsdc,
                       JSDThreadState* jsdthreadstate,
                       JSDStackFrameInfo* jsdframe);
*/

	cContextThreadAcquire aContextThreadAcquire( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mThreadState.mContext.fGetDumbJSContext() );
	tLineNumber rtn = JSD_GetClosestLine( mThreadState.mContext.fGetJSDC(), fGetJSDScript(),
		JSD_GetPCForStackFrame( mThreadState.mContext.fGetJSDC(), mThreadState.fGet(), fGet() ) )
		-1;

	
	return rtn;
}


iValue* iStackFrame::fGetCallObject() const
{
	cContextThreadAcquire aContextThreadAcquire( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mThreadState.mContext.fGetDumbJSContext() );
	JSDValue* X = JSD_GetCallObjectForStackFrame( mThreadState.mContext.fGetJSDC(), mThreadState.fGet(), fGet() );
	iValue* Rtn = ( X ? new cValueRooted( mThreadState.mContext, mThreadState, X ) : 0 );
	return Rtn;
}

iValue* iStackFrame::fGetScopeChain() const
{
	cContextThreadAcquire aContextThreadAcquire( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mThreadState.mContext.fGetDumbJSContext() );
	JSDValue* X = JSD_GetScopeChainForStackFrame( mThreadState.mContext.fGetJSDC(), mThreadState.fGet(), fGet() );
	iValue* Rtn = ( X ? new cValueRooted( mThreadState.mContext, mThreadState, X ) : 0 );
	return Rtn;
}

iValue* iStackFrame::fGetThisValue() const
{
	cContextThreadAcquire aContextThreadAcquire( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mThreadState.mContext.fGetDumbJSContext() );
	JSDValue* X = JSD_GetThisForStackFrame( mThreadState.mContext.fGetJSDC(), mThreadState.fGet(), fGet() );
	iValue* Rtn = ( X ? new cValueRooted( mThreadState.mContext, mThreadState, X ) : 0 );
	return Rtn;
}

iValue* iStackFrame::fEvaluateScript(
	const char* rBytes,
	int rLength ) 
	const
{
	iValue* Rtn = 0;

	jsval Result;
	cContextThreadAcquire aContextThreadAcquire( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mThreadState.mContext.fGetDumbJSContext() );

	RVCString aX0( fsJSCodeString( rBytes, rLength ) ); //SpiderMonkey bug can't deal with local variables.
	RVCString aX1;
	aX1 += "eval(\"";
	aX1 += aX0;
	aX1 += "\")";
	const char* aBytesX1 = aX1.data();
	int aLengthX1( aX1.length() );

	if( JSD_GetScopeChainForStackFrame( mThreadState.mContext.fGetJSDC(), mThreadState.fGet(), fGet() ) ) { //@@Chaim (inherited from Itaj): see bugzilla 92087
		if( JSD_EvaluateScriptInStackFrame( mThreadState.mContext.fGetJSDC(), mThreadState.fGet(), fGet(), aBytesX1, aLengthX1, "DebbugerEvaluate", 1, &Result ) ) {
			JSDValue* X = JSD_NewValue( mThreadState.mContext.fGetJSDC(), Result );
			if( X ) {
				Rtn = new cValueHold( mThreadState.mContext, mThreadState, X );
			}
		}
	}
	return Rtn;
}

iStackFrameLevel::iStackFrameLevel( 
	iDMThreadState& ThreadState, 
	cHandle Handle )
:
	iStackFrame( ThreadState ),
	mHandle( Handle )
{}

iStackFrameLevel* iStackFrameLevel::fGetNext() const
{
	cContextThreadAcquire aContextThreadAcquire( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mThreadState.mContext.fGetDumbJSContext() );
	JSDStackFrameInfo* X = JSD_GetCallingStackFrame( fGetThreadState().mContext.fGetJSDC(), fGetThreadState().fGet(), fGet() );
	iStackFrameLevel* Rtn = ( X ? new iStackFrameLevel( mThreadState, X ) : 0 );
	return Rtn;
}

JSDStackFrameInfo* iStackFrameLevel::vGet_iStackFrame()
	const
{
	return mHandle;
}


cIteratableStackFrame::cIteratableStackFrame(
	iDMThreadState& ThreadState )
:
	iStackFrame(ThreadState),
	mHandle( ( 
		/*temporary guard*/cContextThreadAcquire( RVJSD_GetJSContext( ThreadState.fGet() ) ), 
		/*temporary guard*/cContextThreadAcquire( mThreadState.mContext.fGetDumbJSContext() ),
		JSD_GetStackFrame( ThreadState.mContext.fGetJSDC(), ThreadState.fGet() ) ) )
{}

cIteratableStackFrame::~cIteratableStackFrame() {}

void cIteratableStackFrame::fNext()
{
	cContextThreadAcquire aContextThreadAcquire( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mThreadState.mContext.fGetDumbJSContext() );
	cHandle OldHandle = mHandle;
	mHandle = JSD_GetCallingStackFrame( mThreadState.mContext.fGetJSDC(), mThreadState.fGet(), OldHandle );
}

bool cIteratableStackFrame::fIsValid()
{
	return( mHandle ? true : false );
}

JSDStackFrameInfo* cIteratableStackFrame::vGet_iStackFrame()
	const
{
	return mHandle;
}



iValue::iValue( 
	const iContext& Context,
	const iDMThreadState& ThreadState,
	rcType Handle )
:
	mContext(const_cast<iContext&>(Context)),
	mThreadState(const_cast<iDMThreadState&>(ThreadState)),
	jcHandlePointer( Handle )
{}

iValue::~iValue() {}

const char* iValue::fToString()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	JSString* jsstring = RVJSD_GetValueString( mContext.fGetJSDC(), mThreadState.fGet(), fGet() );
	const char* Rtn = ( jsstring ? JS_GetStringBytes( jsstring ) : 0 );
	return Rtn;
}

tPropertyIndex iValue::fGetPropertiesCount() 
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return (tPropertyIndex)RVJSD_GetCountOfProperties( mContext.fGetJSDC(), mThreadState.fGet(), fGet() );
}

iValue* iValue::fGetParent()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	JSDValue* X = JSD_GetValueParent( mContext.fGetJSDC(), fGet() );
	iValue* Rtn = ( X ? new cValueHold( mContext, mThreadState, X ) : 0 );
	return Rtn;
}

iValue* iValue::fGetProto()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	JSDValue* X = JSD_GetValuePrototype( mContext.fGetJSDC(), fGet() );
	iValue* Rtn = ( X ? new cValueHold( mContext, mThreadState, X ) : 0 );
	return Rtn;
}

iValue* iValue::fGetCtor()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	JSDValue* X = RVJSD_GetValueConstructor( mContext.fGetJSDC(), mThreadState.fGet(), fGet() );
	iValue* Rtn = ( X ? new cValueHold( mContext, mThreadState, X ) : 0 );
	return Rtn;
}

const char* iValue::fGetClassName()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return JSD_GetValueClassName( mContext.fGetJSDC(), fGet() );
}


bool iValue::fIsObject()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return (bool)JSD_IsValueObject( mContext.fGetJSDC(), fGet() );
}

bool iValue::fIsPrimitive()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return JSD_IsValuePrimitive( mContext.fGetJSDC(), fGet() );
}

bool iValue::fIsNull()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return (bool)JSD_IsValueNull( mContext.fGetJSDC(), fGet() );
}

bool iValue::fIsVoid()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return (bool)JSD_IsValueVoid( mContext.fGetJSDC(), fGet() );
}

bool iValue::fIsBoolean()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return (bool)JSD_IsValueBoolean( mContext.fGetJSDC(), fGet() );
}

bool iValue::fIsInt()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return (bool)JSD_IsValueInt( mContext.fGetJSDC(), fGet() );
}

bool iValue::fIsDouble()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return (bool)JSD_IsValueDouble( mContext.fGetJSDC(), fGet() );
}

bool iValue::fIsNumber()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return (bool)JSD_IsValueNumber( mContext.fGetJSDC(), fGet() );
}

bool iValue::fIsString()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return (bool)JSD_IsValueString( mContext.fGetJSDC(), fGet() );
}

bool iValue::fIsFunction()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return (bool)RVJSD_IsValueFunction( mContext.fGetJSDC(), mThreadState.fGet(), fGet() );
}

bool iValue::fIsNative()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return (bool)RVJSD_IsValueNative( mContext.fGetJSDC(), mThreadState.fGet(), fGet() );
}

const char* iValue::fGetFunctionName()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return RVJSD_GetValueFunctionName( mContext.fGetJSDC(), mThreadState.fGet(), fGet() );
}
	

cValueRooted::cValueRooted( 
	const iContext& Context,
	const iDMThreadState& ThreadState,
	rcType Handle )
:
	iValue( Context, ThreadState, Handle )
{}


cValueHold::cValueHold( 
	const iContext& Context,
	const iDMThreadState& ThreadState,
	rcType Handle )
:
	iValue( Context, ThreadState, Handle )
{}

cValueHold::~cValueHold() 
{
	if( fGet() ) {
		cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
		cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
		JSD_DropValue( mContext.fGetJSDC(), fGet() );
	}
}


cPropertiesIter::cPropertiesIter(
	iValue& Value )
:
	mValue(Value),
	mIter(0)
{}

cPropertiesIter::~cPropertiesIter() {}	

iProperty* cPropertiesIter::fGetNext()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mValue.mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mValue.mContext.fGetDumbJSContext() );
	JSDProperty* X = RVJSD_IterateProperties( mValue.mContext.fGetJSDC(), mValue.mThreadState.fGet(), mValue.fGet(), &mIter );
	iProperty* Rtn = ( X ? new cProperty( mValue.mContext, mValue.mThreadState, X ) : 0 );
	return Rtn;
}

iProperty::iProperty(
	iContext& Context,
	iDMThreadState& ThreadState,
	rcType Handle )
:
	mContext(Context),
	mThreadState(ThreadState),
	jcHandlePointer( Handle ),
	mName( (
		/*temporary guard*/cContextThreadAcquire( RVJSD_GetJSContext( ThreadState.fGet() ) ),
		/*temporary guard*/cContextThreadAcquire( Context.fGetDumbJSContext() ),
		JSD_GetPropertyName( mContext.fGetJSDC(), fGet() ) ) 
		)
{}

iProperty::~iProperty()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	if( mName ) {
		JSD_DropValue( mContext.fGetJSDC(), mName );
	}
	if( fGet() ) {
		JSD_DropProperty( mContext.fGetJSDC(), fGet() );
	}
}

bool iProperty::fIsNameString() const
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return( JSD_IsValueString( mContext.fGetJSDC(), mName ) ? true : false );
}	

const char* iProperty::fGetNameString() const
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return( JS_GetStringBytes( RVJSD_GetValueString( mContext.fGetJSDC(), mThreadState.fGet(), mName ) ) );
}

bool iProperty::fIsNameInt() const
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return( JSD_IsValueInt( mContext.fGetJSDC(), mName ) ? true : false );
}	

tPropertyIndex iProperty::fGetNameInt() const
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return( (tPropertyIndex)JSD_GetValueInt( mContext.fGetJSDC(), mName ) );
}

iValue* iProperty::fGetValue() const
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	JSDValue* X = JSD_GetPropertyValue( mContext.fGetJSDC(), fGet() );
	iValue* Rtn = ( X ? new cValueHold( mContext, mThreadState, X ) : 0 );
	return Rtn;
}

bool iProperty::fIsEnumerate()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return( 0 != (JSDPD_ENUMERATE & JSD_GetPropertyFlags( mContext.fGetJSDC(), fGet() ) ));
}

bool iProperty::fIsReadOnly()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return( 0 != (JSDPD_READONLY & JSD_GetPropertyFlags( mContext.fGetJSDC(), fGet() ) ));
}

bool iProperty::fIsPermanent()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return( 0 != (JSDPD_PERMANENT & JSD_GetPropertyFlags( mContext.fGetJSDC(), fGet() ) ));
}

bool iProperty::fIsAlias()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return( 0 != (JSDPD_ALIAS & JSD_GetPropertyFlags( mContext.fGetJSDC(), fGet() ) ));
}

bool iProperty::fIsArgument()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return( 0 != (JSDPD_ARGUMENT & JSD_GetPropertyFlags( mContext.fGetJSDC(), fGet() ) ));
}

bool iProperty::fIsVariable()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return( 0 != (JSDPD_VARIABLE & JSD_GetPropertyFlags( mContext.fGetJSDC(), fGet() ) ));
}

bool iProperty::fIsHinted()
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return( 0 != (JSDPD_HINTED & JSD_GetPropertyFlags( mContext.fGetJSDC(), fGet() ) ));
}

tPropertyIndex iProperty::fGetSlotIndex() const
{
	cContextThreadAcquire aContextThreadAcquireX0( RVJSD_GetJSContext( mThreadState.fGet() ) );
	cContextThreadAcquire aContextThreadAcquireX1( mContext.fGetDumbJSContext() );
	return JSD_GetPropertyVarArgSlot( mContext.fGetJSDC(), fGet() );
}	

cProperty::cProperty( 
	iContext& Context,
	iDMThreadState& ThreadState, 
	rcType Handle )
:
	iProperty( Context, ThreadState, Handle )
{}


cCallChain::cCallChain()
:
	mJSDC(0),
	mChain(0),
	mDepth(0)
{}

cCallChain::~cCallChain()
{
	fFinal();
}

void cCallChain::fInit(
	JSDContext* JSDC,
	JSDThreadState* ThreadState,
	JSDStackFrameInfo* StackFrame )	
{
	mJSDC = JSDC;

	WlAssert(mJSDC);
	WlAssert(ThreadState);

	mDepth = (int) JSD_GetCountOfStackFrames(JSDC, ThreadState);

	WlAssert(mDepth);

	mChain = (JSDScript**) new JSDScript*[mDepth];
	WlAssert(mChain);

	for(int I = mDepth; I-->0; ) {
		WlAssert(StackFrame);
		mChain[I] = JSD_GetScriptForStackFrame(mJSDC, ThreadState, StackFrame);
		WlAssert(mChain[I]);
		StackFrame = JSD_GetCallingStackFrame(mJSDC, ThreadState, StackFrame);
	}
}


void cCallChain::fFinal() 
{
	if( mChain ) {
		delete[] (char*) mChain; mChain=0;
	}
}

void cCallChain::fMove(
	cCallChain& Other ) 
{
	fFinal();
	mJSDC = Other.mJSDC;
	mDepth = Other.mDepth;
	mChain = Other.mChain;

	Other.mChain = 0;
}


JSDScript* cCallChain::fGetTopScript() 
{
	JSDScript* rtn = NULL;
	if( mChain ) {
		rtn = mChain[mDepth-1];
	}
	return rtn;
}



cCallChain::tCompareResult cCallChain::fCompare(
	const cCallChain& Other) const
{

	if( this == &Other ) {
		return sEqual;
	}

	if( !( mChain && Other.mChain && mDepth && Other.mDepth ) ) {
		return sDisjoin;
	}


	int I = (mDepth < Other.mDepth) ? mDepth : Other.mDepth; // min( mDepth, Other.mDepth ); 

	while( I-- ) { // Depth-1 downto 0 
		if( mChain[I] != Other.mChain[I] ) {
			return sDisjoin;
		}
	}

	if( mDepth > Other.mDepth ) {
		return sCallee;
	}
	if( mDepth < Other.mDepth ) {
		return sCaller;
	}
	return sEqual;
}






iSpace::iSpace(
	bool IsDebugger,
	JSRuntime* Runtime )
:
	mIsDebugger(IsDebugger),
	mRuntime( Runtime ),
	mJSDC(0),
	mDumbJSContext(0),
	mExecutionLock(0),
	mInterruptionControl(sInterruptionControlNone),
	mInterruptionControlLocker(0),
	mHookPositionPC(),
	mHookPositionLineNumber(),
	mHookPositionStackDepth(-1),
	mInvokeEnterFlag(0),
	mDMDepth(0),
	mReportDepth(0)
{}

iSpace::~iSpace() 
{
	//fFinal();
}

void iSpace::fSetRuntime(
	JSRuntime* Rt ) 
{
	mRuntime = Rt;
}

void* iSpace::fThisVoid() 
{
	return reinterpret_cast<void*>(static_cast<iSpace*>(this));
}

void iSpace::vInit_iExecutor() 
{

	mExecutionLock = new RVMutexLock();

	mInterruptionControlLocker = new RVMutexLock();

	if(mIsDebugger) {

		mJSDC = JSD_DebuggerOnForUser( mRuntime, 0, fThisVoid() );
		if(!mJSDC){
			throw "JSD_DebuggerOnForUser failed";
		}

		if(!JSD_SetScriptHook( mJSDC, fsScriptHook, fThisVoid() )) {
			throw "JS_SetSourceHandler failed";
		}
		if(!JSD_SetErrorReporter( mJSDC, fsCallbackErrorReporter, fThisVoid() )) {
			throw "JSD_SetErrorReporter failed";
		}

		mDumbJSContext = JSD_GetDefaultJSContext( mJSDC );
		fSetHooks();
	}
}


void iSpace::fSetHooks() 
{
	int Status = 0;
	JSD_SetInterruptHook( mJSDC, fsCallbackExecutionHook, fThisVoid() ) ? 0 : ++Status;
	JSD_SetDebuggerHook( mJSDC, fsCallbackExecutionHook, fThisVoid() ) ? 0 : ++Status;
	JSD_SetThrowHook( mJSDC, fsCallbackExecutionHook, fThisVoid() ) ? 0 : ++Status;
	JSD_SetDebugBreakHook( mJSDC, fsCallbackExecutionHook, fThisVoid() ) ? 0 : ++Status;
	JSD_SetTopLevelHook( mJSDC, fsCallbackTopLevelHook, fThisVoid() ) ? 0 : ++Status;
	JSD_SetFunctionHook( mJSDC, fsCallbackFunctionHook, fThisVoid() ) ? 0 : ++Status;
	if( Status ) {
		throw( "can't set hooks" );
	}
}


void iSpace::vInterrupt_iDebugger()
{
	if(mIsDebugger) {
		fSetInterruptionControl( sInterruptionControlStep );
	}
}
	
void iSpace::fAbort_iDebugger() 
{
	RVMutexLock::LockGuard Locking( *mExecutionLock );
	if(mIsDebugger) {
		fSetInterruptionControl( sInterruptionControlAbort );
	}
}

	
tLineNumber iSpace::fEnableBreakPoint( 
	iScriptHookable& Script, 
	tLineNumber rLine )
{
	JSD_LockScriptSubsystem(mJSDC);
	uintN aLine(rLine+1);

	JSDScript* JsdScript = static_cast<cScriptHookable&>(Script).fGet();

	bool IsLineBelong( true );

	{
		bool Active = ( JSD_IsActiveScript( mJSDC, JsdScript ) 
			? true : false );
		WlAssert(Active);
	}

	{
		// is the line inside this script
		uintN BaseLine = JSD_GetScriptBaseLineNumber( mJSDC, JsdScript );
		WlAssert( BaseLine>0 );
		uintN LineExtent = JSD_GetScriptLineExtent( mJSDC, JsdScript );
		if( ( aLine < BaseLine ) || ( aLine > BaseLine+LineExtent ) ) {
			IsLineBelong = false;
			goto lblAfterCheck;
		}

		// looking for another JSDScript that owns that line (enclosed inside this JSDScript).
		JSDScript* CurrentJsdScript;
		JSDScript* Iter = 0;
		while((CurrentJsdScript = JSD_IterateScripts( mJSDC, &Iter)) != NULL) {
			if( (CurrentJsdScript != JsdScript ) && JSD_IsActiveScript( mJSDC, CurrentJsdScript ) ) {
				if( !strcmp( JSD_GetScriptFilename( mJSDC, JsdScript ), JSD_GetScriptFilename( mJSDC, CurrentJsdScript ) ) ) {
					uintN CurrentBaseLine = JSD_GetScriptBaseLineNumber( mJSDC, CurrentJsdScript );
					WlAssert( CurrentBaseLine>0 );
					uintN CurrentLineExtent = JSD_GetScriptLineExtent( mJSDC, CurrentJsdScript );
					if( ( aLine >= CurrentBaseLine ) && ( CurrentBaseLine >= BaseLine ) && ( aLine <= CurrentBaseLine+CurrentLineExtent ) && ( CurrentBaseLine+CurrentLineExtent < BaseLine+LineExtent ) ) {
						IsLineBelong = false;
						goto lblAfterCheck;
					}
				}
			}
		}
	}
lblAfterCheck:


	JSD_UnlockScriptSubsystem(mJSDC);
// if got here, then JSDScript owns the line and will try set	breakpoint

	tLineNumber Rtn;

	if( IsLineBelong ) {

		jsuword PC = JSD_GetClosestPC( mJSDC, JsdScript, aLine );
		WlAssert(PC);
	
		if( !fSetExecutionHook( JsdScript, PC ) ) {
			WlAssert( 0 );
		}
	
		Rtn = JSD_GetClosestLine( mJSDC, JsdScript, PC )
			-1;
	} else {
		Rtn = 0;
	}

	return Rtn;
}

tLineNumber iSpace::fToggleBreakPoint( 
	iFile& rFile, 
	tLineNumber rLineNumber, 
	bool rState )
{
	tLineNumber aActualLine = 0;

	JSD_LockScriptSubsystem(mJSDC);

	// the breakpoint should be set on script with smallest line range within it fits. and there may be a few such scripts due to cloning (for example by ExecScriptPart).

	// looking for smallest linerange
	uintN aLine = rLineNumber+1;
	uintN aBaseLine = 0;
	uintN aLineExtent = 0;

	{
		JSDScript* aCurrentJsdScript;
		JSDScript* aIter = 0;

		while((aCurrentJsdScript = JSD_IterateScripts( mJSDC, &aIter)) != NULL) {
			if( JSD_IsActiveScript( mJSDC, aCurrentJsdScript ) ) {
				iFile* aCurrentFile = NULL;
				if( iFile* aFile = ZTSEncriptionToFile( JSD_GetScriptFilename( mJSDC, aCurrentJsdScript ) ) ) {
					aCurrentFile = aFile;
				}
				if( &rFile == aCurrentFile ) {
					uintN aCurrentBaseLine = JSD_GetScriptBaseLineNumber( mJSDC, aCurrentJsdScript );
					WlAssert( aCurrentBaseLine>0 );
					uintN aCurrentLineExtent = JSD_GetScriptLineExtent( mJSDC, aCurrentJsdScript );
					if( ( aLine >= aCurrentBaseLine ) && ( aLine <= aCurrentBaseLine+aCurrentLineExtent ) ) {
						// if range initialized, then check if current range is better
						if( (!aBaseLine) || ( ( aBaseLine < aCurrentBaseLine ) || ( aBaseLine+aLineExtent > aCurrentBaseLine+aCurrentLineExtent ) ) ) {
							aBaseLine = aCurrentBaseLine;
							aLineExtent = aCurrentLineExtent;
						}
					}
				}
			}
		}
	}
	
	
	// set breakpoints on scripts that fit to the range
	{
		JSDScript* aCurrentJsdScript;
		JSDScript* aIter = 0;

		while((aCurrentJsdScript = JSD_IterateScripts( mJSDC, &aIter)) != NULL) {
			if( JSD_IsActiveScript( mJSDC, aCurrentJsdScript ) ) {
				iFile* aCurrentFile = NULL;
				if( iFile* aFile = ZTSEncriptionToFile( JSD_GetScriptFilename( mJSDC, aCurrentJsdScript ) ) ) {
					aCurrentFile = aFile;
				}
				if( &rFile == aCurrentFile ) {
					uintN aCurrentBaseLine = JSD_GetScriptBaseLineNumber( mJSDC, aCurrentJsdScript );
					WlAssert( aCurrentBaseLine>0 );
					uintN aCurrentLineExtent = JSD_GetScriptLineExtent( mJSDC, aCurrentJsdScript );
					if( ( aLine >= aCurrentBaseLine ) && ( aLine <= aCurrentBaseLine+aCurrentLineExtent ) && ( aCurrentBaseLine >= aBaseLine ) && ( aCurrentBaseLine+aCurrentLineExtent <= aBaseLine+aLineExtent ) ) {
						
						jsuword aPC = JSD_GetClosestPC( mJSDC, aCurrentJsdScript, aLine );
						WlAssert(aPC);

						if( rState ) {
							if( !fSetExecutionHook( aCurrentJsdScript, aPC ) ) {
								WlAssert(0);
							}
						} else {
							if( !JSD_ClearExecutionHook( mJSDC, aCurrentJsdScript, aPC) ) {
								WlAssert(0);
							}
						}

						tLineNumber aCurrentActualLine = JSD_GetClosestLine( mJSDC, aCurrentJsdScript, aPC );
						WlAssert( aCurrentActualLine );
						WlAssert( (!aActualLine) || (aActualLine==aCurrentActualLine) ); // if already been set, it must be at same place now.
						aActualLine = aCurrentActualLine;
					}
				}
			}
		}
	}

	JSD_UnlockScriptSubsystem(mJSDC);

	return aActualLine-1;
}



bool iSpace::fSetExecutionHook(
	JSDScript* JsdScript,
	jsuword PC )
{
	return ( JSD_SetExecutionHook( mJSDC, JsdScript, PC, fsCallbackExecutionHook, fThisVoid() ) 
		? true : false );
}

void iSpace::vFinal_iExecutor() 
{
	{
		RVMutexLock::LockGuard Locking( *mExecutionLock );
			

		WlAssert( mInterruptionControlLocker );
		delete mInterruptionControlLocker; mInterruptionControlLocker = 0;

		if(mIsDebugger) {
			if( mJSDC ) {
				JSD_DebuggerOff( mJSDC );
				mJSDC = 0;
			}
		}
	}

	delete mExecutionLock; mExecutionLock = 0;
}

void iSpace::fSignJSContext(
	JSContext* Cx ) 
{
	if(mIsDebugger) {
		JSD_JSContextInUse( mJSDC, Cx );
	}
}




bool iSpace::fIsDebugger() const 
{
	return mIsDebugger;
}


JSDContext* iSpace::fGetJSDC() const 
{
	return mJSDC;
}

JSContext* iSpace::fGetDumbJSContext() const 
{
	return mDumbJSContext;
}


tInterruptionControl iSpace::fGetInterruptionControl() const 
{
	tInterruptionControl Rtn;
	{
		RVMutexLock::LockGuard Locking( *mInterruptionControlLocker );
		Rtn = mInterruptionControl;
	}
	return(Rtn);
}

void iSpace::fSetInterruptionControl(
	tInterruptionControl Value) 
{
	{
		RVMutexLock::LockGuard Locking( *mInterruptionControlLocker );
		mInterruptionControl = Value;
	}
}


void iSpace::fScriptHook(
	JSDContext* JSDC,
	JSDScript* JsdScript,
	JSBool Creating) 
{

	JSD_LockScriptSubsystem(mJSDC);
	bool Active = ( JSD_IsActiveScript( mJSDC, JsdScript )
		? true : false );
	JSD_UnlockScriptSubsystem(mJSDC);
	WlAssert(Active); 

	cScriptHookable aScript( *this, JsdScript);
	iFile* aFile = aScript.fGetFile();
	if( Creating ) {
		if( aFile ) {
			aFile->fIncRef();
		}
	} else {
	}
	fGetUi().fScriptHook( aScript, Creating );
	if( Creating ) {
	} else {
		if( aFile ) {
			if( aFile->fDecRef() ) {
				fClearFile( *aFile );
			}
		}
	}

}


uintN iSpace::fCallbackErrorReporter(
	JSDContext* DCx,
	JSContext* Cx, 
	const char* Message,
	JSErrorReport* Report	) 
{

	uintN Rtn;

	if( fCanReport() ) {
		fEnterReport();
		{
			cJSDReportCommit ReportCommit( fGetUi(), Message, Report );
      WlAssert(piVirtualLoadEngine);
			ReportCommit.fCommit(piVirtualLoadEngine->fShouldReportInternalCSJSErrors(Cx));
		
			Rtn = ReportCommit.fGetReturnValue();
		
		}
		fExitReport();
	} else {
		Rtn = JSD_ERROR_REPORTER_CLEAR_RETURN;
	}

	return(Rtn);
}


uintN iSpace::fCallbackExecutionHook(
	JSDContext* JSDC,
	JSDThreadState* ThreadState,
	uintN Type,
	jsval* Rval	) 
{
	uintN rtn( JSD_HOOK_RETURN_CONTINUE );

	if( fCanDM() ) {
		fEnterDM();

		if( 0 < mInvokeEnterFlag ) {
      cDMThreadState tmp_cDMThreadState(*this, ThreadState );
			//fGetUi().fInvokeDMHook( mInvokeIsFunction, sInvocationDMHookTypeEnter, ThreadState ? &cDMThreadState( *this, ThreadState ) : 0 );
			fGetUi().fInvokeDMHook( mInvokeIsFunction, sInvocationDMHookTypeEnter, ThreadState ? &tmp_cDMThreadState : 0 );
			// @@Chaim (inherited from Itaj): check return value
			mInvokeEnterFlag = 0;
		}


		if( iJSEmbedContext* aEmbedContext = iJSEmbedContext::fsGet( RVJSD_GetJSContext( ThreadState ) ) ) {
		if( aEmbedContext->fIsUserSection() ) {

			tInterruptionControl aInterruptionControl = fGetInterruptionControl();

			JSDStackFrameInfo* aStackFrame = JSD_GetStackFrame( JSDC, ThreadState );
			jsuword aPC( JSD_GetPCForStackFrame( JSDC, ThreadState, aStackFrame ) );
			JSDScript* aJSDScript = JSD_GetScriptForStackFrame( JSDC, ThreadState, aStackFrame);
			tLineNumber aLineNumber( JSD_GetClosestLine( JSDC, aJSDScript, aPC )-1 );

			tStepping aStepping;
			if( mHookPositionStackDepth == -1 ) {
				aStepping = sSteppingOut;
			} else if( mHookPositionStackDepth == 0 ) {
				if( ( (aPC < mHookPositionPC) && (aLineNumber != mHookPositionLineNumber) ) /* definitely jumping back */ ||
					( aLineNumber > mHookPositionLineNumber ) /* definately jumping forward */ ) {
					aStepping = sSteppingOver;
				} else {
					aStepping = sSteppingLine;
				}
			} else if( mHookPositionStackDepth > 0 ) {
					aStepping = sSteppingInto;
			} else {
				WlAssert(0);
			}
				
			bool aIsHookPosition( false );
			bool aIsHook( true );
			switch(Type) {
			case JSD_HOOK_INTERRUPTED: {
				if( aInterruptionControl != sInterruptionControlAbort ) {
					cContextThreadRelease aContextThreadRelease( RVJSD_GetJSContext( ThreadState ) );
					cContextThreadRelease aContextThreadReleaseX1( fGetDumbJSContext() );
					cDMThreadState aThreadState( *this, ThreadState );
					rtn = fsJsdNormalDMHookReturn( fGetUi().fInterruptDMHook( aInterruptionControl, aIsHookPosition, aStepping, aThreadState ) );
				} else {
					aIsHook = false;
					rtn = JSD_HOOK_RETURN_ABORT;
				}
			} break;

			case JSD_HOOK_BREAKPOINT: {
				cContextThreadRelease aContextThreadRelease( RVJSD_GetJSContext( ThreadState ) );
				cContextThreadRelease aContextThreadReleaseX1( fGetDumbJSContext() );
				cDMThreadState aThreadState( *this, ThreadState );
				rtn = fsJsdNormalDMHookReturn( fGetUi().fBreakPointDMHook( aInterruptionControl, aIsHookPosition, aStepping, aThreadState ) );
				break;
			}

			case JSD_HOOK_DEBUGGER_KEYWORD: {
				cContextThreadRelease aContextThreadRelease( RVJSD_GetJSContext( ThreadState ) );
				cContextThreadRelease aContextThreadReleaseX1( fGetDumbJSContext() );
				cDMThreadState aThreadState( *this, ThreadState );
				rtn = fsJsdNormalDMHookReturn( fGetUi().fKeywordDMHook( aInterruptionControl, aIsHookPosition, aStepping, aThreadState ) );
				break;
			}

			case JSD_HOOK_DEBUG_REQUESTED: {
				cContextThreadRelease aContextThreadRelease( RVJSD_GetJSContext( ThreadState ) );
				cContextThreadRelease aContextThreadReleaseX1( fGetDumbJSContext() );
				cDMThreadState aThreadState( *this, ThreadState );
				fGetUi().fReportDMHook( aInterruptionControl, aIsHookPosition, aStepping, aThreadState );
				rtn = JSD_HOOK_RETURN_ABORT;
				break;
			}

			case JSD_HOOK_THROW: {
				cContextThreadRelease aContextThreadRelease( RVJSD_GetJSContext( ThreadState ) );
				cContextThreadRelease aContextThreadReleaseX1( fGetDumbJSContext() );
				cDMThreadState aThreadState( *this, ThreadState );
				rtn = fsJsdThrowDMHookReturn( fGetUi().fThrowDMHook( aInterruptionControl, aIsHookPosition, aStepping, aThreadState ) );
				break;
			}

			default:
				WlAssert(0); // switch error - should never get here
				break;
			}

			if(aIsHook) {
				fSetInterruptionControl( aInterruptionControl );
			}

			if(aIsHookPosition) {
				mHookPositionPC = aPC;
				mHookPositionLineNumber = aLineNumber;
				mHookPositionStackDepth = 0;
			}
		}}
		fExitDM();
	}

	return rtn;
}


JSBool iSpace::fCallbackTopLevelHook(
	JSDContext* rJSDC,
	JSDThreadState* rThreadState,
	uintN rType) 
{
	JSBool rtn = JS_TRUE;

	if( fCanDM() ) {
		fEnterDM();


		tInvocationDMHookType aType;
                cDMThreadState tmp_cDMThreadState(*this, rThreadState);
		switch(rType) {
		case JSD_HOOK_TOPLEVEL_START:
			if( -1 != mHookPositionStackDepth ) {
				++mHookPositionStackDepth;
			}
			++mInvokeEnterFlag;
			mInvokeIsFunction = false;
	//		aType = sInvocationDMHookTypeEnter;
			break;
		case JSD_HOOK_TOPLEVEL_END:
			if( 0 < mHookPositionStackDepth ) {
				--mHookPositionStackDepth;
			} else {
				mHookPositionStackDepth = -1;
			}
			aType = sInvocationDMHookTypeExit;
			//rtn = fGetUi().fInvokeDMHook( false, aType, rThreadState ? &cDMThreadState( *this, rThreadState ) : 0 );	
			rtn = fGetUi().fInvokeDMHook( false, aType, rThreadState ? &tmp_cDMThreadState : 0 );	
			break;
		default:
			WlAssert(0);
			break;
		};

		fExitDM();
	}
	
	return rtn;
}


JSBool iSpace::fCallbackFunctionHook(
	JSDContext* rJSDC,
	JSDThreadState* rThreadState,
	uintN rType) 
{
	JSBool rtn = JS_TRUE;

	if( fCanDM() ) {
		fEnterDM();


		tInvocationDMHookType aType;
                cDMThreadState tmp_cDMThreadState(*this, rThreadState);
		switch(rType) {
		case JSD_HOOK_FUNCTION_CALL:
			if( -1 != mHookPositionStackDepth ) {
				++mHookPositionStackDepth;
			}
			++mInvokeEnterFlag;
			mInvokeIsFunction = true;
	//		aType = sInvocationDMHookTypeEnter;
			break;
		case JSD_HOOK_FUNCTION_RETURN:
			if( 0 < mHookPositionStackDepth ) {
				--mHookPositionStackDepth;
			} else {
				mHookPositionStackDepth = -1;
			}
			aType = sInvocationDMHookTypeExit;
			//rtn = fGetUi().fInvokeDMHook( true, aType, rThreadState ? &cDMThreadState( *this, rThreadState ) : 0 );	
			rtn = fGetUi().fInvokeDMHook( true, aType, rThreadState ? &tmp_cDMThreadState : 0 );	
			break;
		default:
			WlAssert(0);
			break;
		};

		fExitDM();
	}

	return rtn;
}


/*
void iSpace::fHookParameters(
	JSDThreadState* ThreadState,
	const char*& FileName,
	tLineNumber& LineNumber ) 
{
	JSDStackFrameInfo* StackFrame = JSD_GetStackFrame( mJSDC, ThreadState );
	JSDScript* Script = JSD_GetScriptForStackFrame( mJSDC, ThreadState, StackFrame );

	JSD_LockScriptSubsystem(mJSDC);
	bool Active = ( JSD_IsActiveScript( mJSDC, Script ) ?
		true : false );
	JSD_UnlockScriptSubsystem(mJSDC);
	WlAssert(Active); 

	FileName = JSD_GetScriptFilename( mJSDC, Script );
	LineNumber = JSD_GetClosestLine( mJSDC, Script,
		JSD_GetPCForStackFrame( mJSDC, ThreadState, StackFrame ) ) -1;
}
*/

uintN iSpace::fsJsdNormalDMHookReturn( 
	tNormalDMHookReturn Source ) 
{
	uintN Rtn;

	switch( Source ) {
	case sNormalDMHookReturnContinue:
		Rtn = JSD_HOOK_RETURN_CONTINUE;
		break;
	case sNormalDMHookReturnAbort:
		Rtn = JSD_HOOK_RETURN_ABORT;
		break;
	default:
		WlAssert(0); // switch error - should never get here
		break;
	}
	return(Rtn);
}


uintN iSpace::fsJsdThrowDMHookReturn( 
	tThrowDMHookReturn Source ) 
{

	uintN Rtn;

	switch( Source ) {
	case sThrowDMHookReturnApply:
		Rtn = JSD_HOOK_RETURN_CONTINUE_THROW;
		break;
	case sThrowDMHookReturnAbort:
		Rtn = JSD_HOOK_RETURN_ABORT;
		break;
	case sThrowDMHookReturnClear:
		Rtn = JSD_HOOK_RETURN_CONTINUE;
		break;
	default:
		WlAssert(0); // switch error - should never get here
		break;
	}
	return(Rtn);
}


extern "C" {

uintN fsCallbackErrorReporter(
	JSDContext* jsdc,
	JSContext* cx, 
	const char* message,
	JSErrorReport* report,
	void* callerdata ) 
{
	return( reinterpret_cast<iSpace*>(callerdata)->fCallbackErrorReporter( jsdc, cx, message, report ) );
}

uintN fsCallbackExecutionHook(
	JSDContext* jsdc,
	JSDThreadState* jsdthreadstate,
	uintN type,
	void* callerdata,
	jsval* rval
	)
{
	return( reinterpret_cast<iSpace*>(callerdata)->fCallbackExecutionHook( jsdc, jsdthreadstate, type, rval ) );
}


JSBool fsCallbackTopLevelHook(
	JSDContext* jsdc,
	JSDThreadState* jsdthreadstate,
	uintN type,
	void* callerdata)
{
	return( reinterpret_cast<iSpace*>(callerdata)->fCallbackTopLevelHook( jsdc, jsdthreadstate, type ) );
}

JSBool fsCallbackFunctionHook(
	JSDContext* jsdc,
	JSDThreadState* jsdthreadstate,
	uintN type,
	void* callerdata)
{
	return( reinterpret_cast<iSpace*>(callerdata)->fCallbackFunctionHook( jsdc, jsdthreadstate, type ) );
}


void fsScriptHook(
	JSDContext* jsdc,
	JSDScript* jsdscript,
	JSBool creating,
	void* callerdata ) {
	reinterpret_cast<iSpace*>(callerdata)->fScriptHook( jsdc, jsdscript, creating );
	}

} // extern "C"

void iSpace::fEnterDM()
{
	WlAssert( 0<=mDMDepth );
	++mDMDepth;
}

void iSpace::fExitDM()
{
	WlAssert( 0<mDMDepth );
	--mDMDepth;
}

bool iSpace::fCanDM() const 
{
	return( 0 == mDMDepth );
}

void iSpace::fEnterReport()
{
	WlAssert( 0<=mReportDepth );
	++mReportDepth;
}

void iSpace::fExitReport()
{
	WlAssert( 0<mReportDepth );
	--mReportDepth;
}

bool iSpace::fCanReport() const 
{
	return( 0 == mReportDepth );
}


cInterruptionControlCommit::cInterruptionControlCommit(
	iSpace& Space )
:
	mSpace(Space) 
{}

void cInterruptionControlCommit::fSet( tInterruptionControl InterruptionControl ) 
{
	mSpace.fSetInterruptionControl( InterruptionControl );
}



iSpaceUi::iSpaceUi(
	bool Act,
	JSRuntime* Runtime)
:
	iSpace(Act, Runtime) 
{}

iUi& iSpaceUi::vGetUi_iSpace() {
	iUi* X0 = (static_cast<iSpaceUi*>(this));
	return *X0;
}



cSpaceCompose::cSpaceCompose(
	iUi& Ui,
	bool Act,
	JSRuntime* Runtime )
:
	mUi(Ui),
	iSpace(Act, Runtime) 
{}

iUi& cSpaceCompose::vGetUi_iSpace() 
{
	return mUi;
}


} // nJsd

