/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  


#ifndef __COMProtocol_H
#define __COMProtocol_H

#include "WLLEProtocol.h"//the loadEngine h file that contains all the Base Class.

//The Protocol global static pointer.  
//extern static void* piLEProtocolProcess;
extern iLEProtocolProcess* piLEProtocolProcess;

//The function 'NewProtocolProcces' is the only function that will be exported 
//from the dll. The function will create an instance of a class that 
//inherits the base class iLEProtocolProcess, and return a pointer to
// the interface 'iLEProtocolProcess'.
ExportLEProtocol iLEProtocolProcess* NewProtocolProcess();


class comProtocolSession;
class comProtocolAgendaInMix;
class comProtocolAgendaMachineThread;
class comProtocolVCInMix;


////////////////////////////////////////////////////////////////
//////////////// class comProtocolProcess//////////////////////
////////////////////////////////////////////////////////////////


//the protocol will have one class like this for each Process.
class comProtocolProcess : public iLEProtocolProcess {
public:
	explicit comProtocolProcess();
public:
	~comProtocolProcess();

public:
	// the functions of the class iLEProtocolProcess
	virtual iLEProtocolSession& NewProtocolSession();
	virtual iLEProtocolAgendaMachineThread& NewProtocolAgendaMachineThread(WLProtocolThreadType);
	virtual iLEProtocolJSRuntime& NewProtocolJSRuntime();
	virtual bool ProcessInitialize();
	virtual bool ProcessInitializeJava(cRVJVM& rJvm, JNIEnv* rEnv);
	virtual bool ProcessFinalizeJava(cRVJVM& rJvm, JNIEnv* rEnv);
	virtual bool ProcessTermination();

	bool Get_bForceVcInMTA();
	void ForceVcInMTA();

private:
	bool m_bForceVcInMTA;
};





////////////////////////////////////////////////////////////////
//////////////// class comProtocolSession/////////////////////
////////////////////////////////////////////////////////////////


//the protocol will have one class like this for each Session.
class comProtocolSession : public iLEProtocolSession {
public:
	explicit comProtocolSession(iLEProtocolProcess& father);
	~comProtocolSession();

public:
	// the functions of the class iLEProtocolSession
	virtual iLEProtocolAgendaInMix& NewProtocolAgendaInMix();
	virtual bool SessionInitialize();
	virtual bool SessionTermination();//to do

};




////////////////////////////////////////////////////////////////
//////////////// class comProtocolJSRuntime///////////////////
////////////////////////////////////////////////////////////////

class comProtocolJSRuntime : public iLEProtocolJSRuntime {
public:
	explicit comProtocolJSRuntime(iLEProtocolProcess& father);
	~comProtocolJSRuntime();

public:
	virtual bool JSRuntimeInitialize(JSRuntime* rt);
	virtual bool JSRuntimeAfterNewMainContext(JSContext* cx);
	virtual bool JSRuntimeBeforeDestroyMainContext(JSContext* cx);

private:
	JSRuntime* m_JSRuntime;
};




////////////////////////////////////////////////////////////////
//////////////// class comProtocolAgendaInMix///////////////////
////////////////////////////////////////////////////////////////

class comProtocolAgendaInMix : public iLEProtocolAgendaInMix {
public:
	explicit comProtocolAgendaInMix(iLEProtocolSession & father);
	~comProtocolAgendaInMix();

public:
	// the functions of the class iLEProtocolAgendaInMix
	virtual iLEProtocolVCInMix& NewProtocolVCInMix();//??
	virtual bool BeforeInitAgenda();
	virtual bool AfterTerminateAgenda();
	virtual bool JSInitializeForScript(JSContext* cx, JSObject* AgendaGlobal, JSObject* GeneralGlobal );
	virtual bool JSTerminateForScript(JSContext*, JSObject*, JSObject*);

public:
	void SetComTest(char* s);//temp avidan
	RVCString& GetTestString();//temp avidan

private:
	RVCString test;//temp avidan
};




////////////////////////////////////////////////////////////////
//////////////// class comProtocolAgendaMachineThread/////////////////
////////////////////////////////////////////////////////////////

class comProtocolAgendaMachineThread : public iLEProtocolAgendaMachineThread {
public:
	explicit comProtocolAgendaMachineThread(iLEProtocolProcess & father, WLProtocolThreadType);
	~comProtocolAgendaMachineThread();
public:
	// the functions of the class iLEProtocolAgendaInMix
	virtual bool MachineThreadInitialize();
	virtual bool MachineThreadTermination();//to do

//	bool Getm_RuningBrowserPlayBack() { return m_RuningBrowserPlayBack; }
//private:
//	bool m_RuningBrowserPlayBack;
};


////////////////////////////////////////////////////////////////
//////////////// class comProtocolVCInMix///////////////////////
////////////////////////////////////////////////////////////////

class comProtocolVCInMix : public iLEProtocolVCInMix {
public:
	explicit comProtocolVCInMix(iLEProtocolAgendaInMix & father);
	~comProtocolVCInMix();
public:
	// the functions of the class iLEProtocolVCInMix
	virtual bool JSInitializeForClient(JSContext* cx, JSObject* obj, CLEVariablesData* variables);
	virtual bool OnThreadActivation();
	virtual bool BeforeInitClient();
	virtual bool AfterInitClient();
	virtual bool BeforeRound();
	virtual bool AfterRound();
	virtual bool AfterTerminateClient();
	virtual bool BeforeTerminateClient();
	virtual bool OnThreadTermination();//to do
	virtual bool ExceptionHandler(std::exception&) { return true; }
	virtual bool BeforeSleep(long int) { return true; }
	virtual bool AfterSleep(long int) { return true; }
};


#endif // __ComProtocol_H


