/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#ifndef _wlxmlfuncs_h_
#define _wlxmlfuncs_h_

#include "rvjsapi.h"
#include "platform.h"
#include <RVCString.h>
#include <RVWString.h>
#include <xercesc/util/XercesDefs.hpp>
#include <xercesc/dom/DOMNode.hpp>
#include <xercesc/dom/DOM.hpp>
#include <xercesc/util/XMLString.hpp>
#include <xercesc/util/PlatformUtils.hpp>


typedef RVWString DOMString;

#define XMLPARSER_ERROR_FILE   "xmlparser.frm"

XERCES_CPP_NAMESPACE_USE

//#define PROPERTY_FLAGS          JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_GET_DEFINES
//#define READONLY_PROPERTY_FLAGS JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_GET_DEFINES | JSPROP_READONLY
#define PROPERTY_FLAGS          JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_EXPORTED
#define READONLY_PROPERTY_FLAGS JSPROP_ENUMERATE | JSPROP_PERMANENT | JSPROP_EXPORTED | JSPROP_READONLY




char* GetNodeTypeString(const DOMNode* p_pNode);
bool CreateNodeByType(JSContext* cx, JSObject* obj, DOMNode* p_pNode, jsval* rval, bool toDelete);

JSBool WLXmlSetXMLChValue(JSContext* cx, JSObject* obj, const XMLCh* value, jsval* rval);
JSBool WLXmlSetIntValue(JSContext* cx, JSObject* obj, unsigned value, jsval* rval);
JSBool WLXmlSetBoolValue(JSContext* cx, JSObject* obj, bool value, jsval* rval);
void WLXmlGenericFinalize(JSContext* cx, JSObject* obj);


// this will activate toString() on the specified argument
#define XML_GET_DOMSTRING_ARGUMENT(retName, jsArgs, argNum, funcName)                   \
  char* macro_tempStr##argNum = "";                                                      \
  if ( argc > argNum )                                                                   \
  {                                                                                      \
	  macro_tempStr##argNum = JS_GetStringBytes( JS_ValueToString( cx, jsArgs[argNum] ) ); \
		if ( !macro_tempStr##argNum )                                                        \
  	{                                                                                    \
    	WLErrorException e( XMLPARSER_ERROR_FILE, 22, funcName, argNum+1 );               \
			return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );                    \
  	}                                                                                    \
  }                                                                                      \
	DOMString retName(macro_tempStr##argNum,RVWString::ascii);


// this will return XMLCh* from JSString
#define XML_GET_XMLCHAR_ARGUMENT(retName, jsArgs, argNum, funcName)                   \
  const XMLCh* retName = 0;                                                      \
  if ( argc > argNum )                                                                   \
  {                                                                                      \
	  retName = JS_GetStringChars( JS_ValueToString( cx, jsArgs[argNum] ) ); \
  }

// this will return XMLCh* from jsval and return with error code if there is argument
#define XML_GET_XMLCHAR_ARGUMENT_CHECK(retName, jsArgs, argNum, funcName)        \
  const XMLCh* retName = 0;                                                      \
  if ( argc > argNum )                                                           \
  {                                                                              \
	  retName = JS_GetStringChars( JS_ValueToString( cx, jsArgs[argNum] ) );       \
  }                                                                              \
  else                                                                           \
  {                                                                              \
 	  WLErrorException e( XMLPARSER_ERROR_FILE, 22, funcName, argNum+1 );         \
	  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );              \
  }

// this will return XMLSize_t* from jsval and return with error code if there is argument
#define XML_GET_XMLSIZE_ARGUMENT_CHECK(retName, jsArgs, argNum, funcName)        \
  XMLSize_t retName = 0;                                                         \
  if ( argc > argNum && JSVAL_IS_INT(argv[argNum]))                                                           \
  {                                                                              \
	  retName = (XMLSize_t)JSVAL_TO_INT(argv[argNum]);                             \
  }                                                                              \
  else                                                                           \
  {                                                                              \
 	  WLErrorException e( XMLPARSER_ERROR_FILE, 23, funcName, argNum+1 );         \
	  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );              \
  }

// this will return bool from jsval and return with error code if there is argument
#define XML_GET_BOOL_ARGUMENT_CHECK(retName, jsArgs, argNum, funcName)        \
  bool retName = false;                                                         \
  if ( argc > argNum && JSVAL_IS_BOOLEAN(jsArgs[argNum]))                             \
  {                                                                              \
	  retName = (JSVAL_TO_BOOLEAN(jsArgs[argNum])==JS_TRUE);                       \
  }                                                                              \
  else                                                                           \
  {                                                                              \
 	  WLErrorException e( XMLPARSER_ERROR_FILE, 24, funcName, argNum+1 );         \
	  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );              \
  }


//get object from jsval
#define XML_GET_OBJECT_ARGUMENT_CHECK(retName, jsArgs, argNum, className, funcName) \
  className* retName = 0;                                                         \
  if ( argc > argNum && JSVAL_IS_OBJECT(jsArgs[argNum]))                          \
  {                                                                              \
    retName = (className*)CWLXmlParserPrivate::GetThisObject(cx, JSVAL_TO_OBJECT(jsArgs[argNum]));\
  }                                                                              \
  else                                                                           \
  {                                                                              \
 	  WLErrorException e( XMLPARSER_ERROR_FILE, 26, funcName, argNum+1,#className );         \
	  return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );              \
  }


#define PROPERTY_SETTER_FOR_STRING(prefix,propName ,methodName)                                   \
static JSBool prefix##_prop_##methodName(JSContext *cx, JSObject *obj, jsval id, jsval *rval) \
{                                                                                        \
	CLASSNAME* cppObj = ##prefix##_GetThisObject( cx, obj );       \
	if (cppObj) {                                                                          \
    const XMLCh* value = JS_GetStringChars( JS_ValueToString( cx, *rval) );              \
    TRY_XML_PARSER_STANDARD                                                               \
		  cppObj->methodName(value);													                 \
    CATCH_XML_PARSER_STANDARD(prefix,propName##_setter)                                          \
	}																																								       \
	return JS_TRUE;																																	       \
}

#define PROPERTY_GETTER_RET_XMLCHAR(prefix,propName ,methodName)                                   \
static JSBool prefix##_prop_##methodName(JSContext *cx, JSObject *obj, jsval id, jsval *rval) \
{                                                                                        \
	CLASSNAME* cppObj = ##prefix##_GetThisObject( cx, obj );       \
	if (cppObj) {                                                                          \
    const XMLCh* value = 0;                                                               \
    TRY_XML_PARSER_STANDARD                                                               \
		  value = cppObj->methodName();													                 \
    CATCH_XML_PARSER_STANDARD(prefix,propName##_getter)                                          \
		WLXmlSetXMLChValue( cx, obj, value, rval );											                     \
	}																																								       \
	return JS_TRUE;																																	       \
}

#define PROPERTY_GETTER_RET_INT(prefix,propName ,methodName)                                   \
static JSBool prefix##_prop_##methodName(JSContext *cx, JSObject *obj, jsval id, jsval *rval) \
{                                                                                        \
	CLASSNAME* cppObj = ##prefix##_GetThisObject( cx, obj );       \
	if (cppObj) {                                                                          \
    int value = 0;                                                               \
    TRY_XML_PARSER_STANDARD                                                               \
		  value = cppObj->methodName();													                 \
    CATCH_XML_PARSER_STANDARD(prefix,propName##_getter)                                          \
		WLXmlSetIntValue( cx, obj, value, rval );											                     \
	}																																								       \
	return JS_TRUE;																																	       \
}

#define PROPERTY_GETTER_RET_OBJECT(prefix,propName ,methodName, retObject, toDelete)                                   \
static JSBool prefix##_prop_##methodName(JSContext *cx, JSObject *obj, jsval id, jsval *rval) \
{                                                                                        \
	CLASSNAME* cppObj = ##prefix##_GetThisObject( cx, obj );       \
	if (cppObj) {                                                                          \
    DOM##retObject* value = 0;                                                               \
    TRY_XML_PARSER_STANDARD                                                               \
		  value = cppObj->methodName();													                 \
    CATCH_XML_PARSER_STANDARD(prefix,propName##_getter)                                          \
		*rval = WLCreate##retObject##Object( cx, obj, value, toDelete );                    \
	}																																								       \
	return JS_TRUE;																																	       \
}

#define METHOD_NO_RET_NO_ARGS(prefix, methodName)                                   \
static JSBool prefix##_##methodName(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) \
{                                                                                        \
	CLASSNAME* cppObj = ##prefix##_GetThisObject( cx, obj );       \
	if (cppObj) {                                                                          \
    TRY_XML_PARSER_STANDARD                                                               \
		  cppObj->methodName();													                 \
    CATCH_XML_PARSER_STANDARD(prefix,methodName)                                          \
	}																																								       \
	return JS_TRUE;																																	       \
}


#define METHOD_RET_XMLCHAR_NO_ARGS(prefix, methodName)                                   \
static JSBool prefix##_##methodName(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) \
{                                                                                        \
	CLASSNAME* cppObj = ##prefix##_GetThisObject( cx, obj );                               \
	if (cppObj) {                                                                          \
    const XMLCh* value = 0;                                                               \
    TRY_XML_PARSER_STANDARD                                                               \
		  value = cppObj->methodName();													                 \
    CATCH_XML_PARSER_STANDARD(prefix,methodName)                                          \
		WLXmlSetXMLChValue( cx, obj, value, rval );											                     \
	}																																								       \
	return JS_TRUE;																																	       \
}

#define METHOD_RET_INT_NO_ARGS(prefix, methodName)                                   \
static JSBool prefix##_##methodName(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) \
{                                                                                        \
	CLASSNAME* cppObj = ##prefix##_GetThisObject( cx, obj );                               \
	if (cppObj) {                                                                          \
    int value = 0;                                                                        \
    TRY_XML_PARSER_STANDARD                                                               \
		  value = cppObj->methodName();													                 \
    CATCH_XML_PARSER_STANDARD(prefix,methodName)                                          \
		WLXmlSetIntValue( cx, obj, value, rval );											                     \
	}																																								       \
	return JS_TRUE;																																	       \
}

#define METHOD_RET_OBJECT_NO_ARGS(prefix, methodName, retObject, toDelete)              \
static JSBool prefix##_##methodName(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) \
{                                                                                        \
	CLASSNAME* cppObj = ##prefix##_GetThisObject( cx, obj );                               \
	if (cppObj) {                                                                          \
    DOM##retObject* value = 0;                                                                        \
    TRY_XML_PARSER_STANDARD                                                               \
		  value = cppObj->methodName();													               \
    CATCH_XML_PARSER_STANDARD(prefix,methodName)                                          \
		*rval = WLCreate##retObject##Object( cx, obj, value, toDelete );							       \
	}																																								       \
	return JS_TRUE;																																	       \
}

#define METHOD_RET_OBJECT_ONE_ARG_XMLCH(prefix, methodName, retObject, toDelete)              \
static JSBool prefix##_##methodName(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) \
{                                                                                        \
	CLASSNAME* cppObj = ##prefix##_GetThisObject( cx, obj );                               \
	if (cppObj) {                                                                          \
    DOM##retObject * value = 0;                                                                        \
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcValue, argv, 0, #methodName);                       \
    TRY_XML_PARSER_STANDARD                                                               \
		  value = cppObj->methodName(pcValue);													       \
    CATCH_XML_PARSER_STANDARD(prefix,methodName)                                          \
		*rval = WLCreate##retObject##Object( cx, obj, value, toDelete );							       \
	}																																								       \
	return JS_TRUE;																																	       \
}

#define METHOD_RET_OBJECT_TWO_ARG_XMLCH(prefix, methodName, retObject, toDelete)              \
static JSBool prefix##_##methodName(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval) \
{                                                                                        \
	CLASSNAME* cppObj = ##prefix##_GetThisObject( cx, obj );                               \
	if (cppObj) {                                                                          \
    DOM##retObject * value = 0;                                                                        \
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcValue1, argv, 0, #methodName);                      \
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcValue2, argv, 1, #methodName);                      \
    TRY_XML_PARSER_STANDARD                                                               \
		  value = cppObj->methodName(pcValue1, pcValue2);											 \
    CATCH_XML_PARSER_STANDARD(prefix,methodName)                                          \
		*rval = WLCreate##retObject##Object( cx, obj, value, toDelete );							       \
	}																																								       \
	return JS_TRUE;																																	       \
}

#define DEFAULT_GET_THIS_OBJECT(prefix)                                             \
CLASSNAME* ##prefix##_GetThisObject(JSContext *p_pCX, JSObject *p_pObj)             \
{                                                                                   \
  CJSGCRequestBeginEnd GCGuard(p_pCX);                                              \
	CWLXmlParserPrivate* pPrv = (CWLXmlParserPrivate*)JS_GetPrivate( p_pCX, p_pObj ); \
	if ( !pPrv )                                                                      \
		return NULL;                                                                    \
  return (CLASSNAME*)pPrv->GetCPPObj();                                           \
}


#define TRY_XML_PARSER_STANDARD try{

#define CATCH_XML_PARSER_STANDARD(objName,methodName)                       \
  }catch (const XMLException& toCatch)                                                     \
  {                                                                                       \
    char* message = XMLString::transcode(toCatch.getMessage());                           \
    WlTraceMsg("\"%s.%s\" on \"%s\" failed with XMLException. xerces message: %s",#objName,#methodName,CWLXmlParserPrivate::GetObjectEClassStr(cx, obj),message);\
    XMLString::release(&message);                                                         \
   	WLErrorException e( XMLPARSER_ERROR_FILE, 11, "XMLException" , #objName,#methodName,CWLXmlParserPrivate::GetObjectEClassStr(cx, obj)); \
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );                       \
  }                                                                                       \
  catch (const DOMException& toCatch) {                                                   \
    char* message = XMLString::transcode(toCatch.msg);                                    \
    WlTraceMsg("\"%s.%s\" on \"%s\" failed with DOMException. xerces message: %s",#objName,#methodName,CWLXmlParserPrivate::GetObjectEClassStr(cx, obj),message);          \
    XMLString::release(&message);                                                         \
    WLErrorException e( XMLPARSER_ERROR_FILE, 11, "DOMException" ,#objName,#methodName, CWLXmlParserPrivate::GetObjectEClassStr(cx, obj) );    \
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );                       \
  }                                                                                       \
  catch (...) {                                                                           \
    WlTraceMsg("\"%s.%s\" on \"%s\" failed with ... Exception. xerces message: \"Unexpected Exception \"",#objName,#methodName,CWLXmlParserPrivate::GetObjectEClassStr(cx, obj));   \
    WLErrorException e( XMLPARSER_ERROR_FILE, 11, "Unexpected Exception" ,#objName,#methodName, CWLXmlParserPrivate::GetObjectEClassStr(cx, obj));    \
		return LoadReport( cx, e.GetErrorSeverity(), e.GetErrorMsg() );                       \
  }


jsval WLCreateNodeObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval DllExport WLCreateDocumentObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateDocumentFragmentObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateNodeListObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateNamedNodeMapObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateCharacterDataObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateAttrObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateElementObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateTextObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateCommentObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateCDATASectionObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateDocumentTypeObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateNotationObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateEntityObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateEntityReferenceObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);
jsval WLCreateProcessingInstructionObject(JSContext* cx, JSObject* obj, void* value, bool toDelete);



#endif //_wlxmlfuncs_h_
