/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadreportinfo.h"
#include "wlxmlfuncs.h"
#include "wlxmlprivate.h"
#include <xercesc/dom/DOMAttr.hpp>

XERCES_CPP_NAMESPACE_USE

#define CLASSNAME DOMAttr
DEFAULT_GET_THIS_OBJECT(attribute)

JSBool WLXMLAttrEnumerate(JSContext *cx, JSObject *obj );


static JSClass wl_XMLAttrClass = {
	"XMLAttrObject",
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	WLXMLAttrEnumerate,//JS_EnumerateStub
	JS_ResolveStub,
	JS_ConvertStub,
	WLXmlGenericFinalize
};




JSBool WLXMLAttrEnumerate(JSContext *cx, JSObject *obj )
{
  //enumerate class is implemented for: (for a in xmlparse), it can be implemented later.
  DOMAttr* pObj = attribute_GetThisObject(cx, obj);
  
  if (pObj==NULL)
    return JS_TRUE;;

	return JS_TRUE;
}


METHOD_RET_XMLCHAR_NO_ARGS(attribute, getName)
METHOD_RET_XMLCHAR_NO_ARGS(attribute, getValue)
METHOD_RET_OBJECT_NO_ARGS(attribute, getOwnerElement, Element, false);

static JSBool
attribute_setValue(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMAttr* pAttr = attribute_GetThisObject(cx, obj);
  if (pAttr)
  {
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcValue, argv, 0, "attribute_setValue")
    if (pcValue)
      pAttr->setValue(pcValue);
  }  
	return JS_TRUE;
}

static JSBool
attribute_getSpecified(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMAttr* pAttr = attribute_GetThisObject(cx, obj);
  if (pAttr)
  {
    bool bSpecified = pAttr->getSpecified();
    WLXmlSetBoolValue(cx, obj, bSpecified, rval);
  }  
	return JS_TRUE;
}


static JSFunctionSpec xmlAttr_methods[] = {
  {"getName",   attribute_getName, 0},
  {"getValue",   attribute_getValue, 0},
  {"setValue",   attribute_setValue, 1},
  {"getOwnerElement",   attribute_getOwnerElement, 0},
  {"getSpecified",   attribute_getSpecified, 0},
  
	{0}
};


PROPERTY_GETTER_RET_XMLCHAR(attribute,name,getName)
PROPERTY_GETTER_RET_XMLCHAR(attribute,value,getValue)

PROPERTY_SETTER_FOR_STRING(attribute,value,setValue)

static JSPropertySpec xmlAttr_properties[] = {
  { "name"       , 0, READONLY_PROPERTY_FLAGS         , attribute_prop_getName             , JS_PropertyStub },
  { "value"      , 0, PROPERTY_FLAGS                  , attribute_prop_getValue            , attribute_prop_setValue    },
	{0}
};



//create new Attr object
jsval WLCreateAttrObject(JSContext* cx, JSObject* obj, void* value, bool toDelete)
{
  if (!value)
    return JSVAL_NULL;

  CJSGCRequestBeginEnd GCGuard(cx); //lock GC for this action
  CWLXmlParserPrivate* prv = new CWLXmlParserPrivate((void*)value, CWLXmlParserPrivate::ATTRIBUTE, toDelete);
  JSObject* newObj = JS_NewObject(cx, &wl_XMLAttrClass, NULL, NULL);
	JS_SetPrivate(cx, newObj, (void*)prv);
  return OBJECT_TO_JSVAL(newObj);
}

//when we'll define all classes
JSObject* rvAddXMLAttrClass(JSContext *cx, JSObject *obj, JSObject* parent_proto)
{
  return JS_InitClass(cx, obj, parent_proto, &wl_XMLAttrClass, NULL, 0, xmlAttr_properties, xmlAttr_methods, NULL, NULL);
}


