/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadreportinfo.h"
#include "wlxmlfuncs.h"
#include "wlxmlprivate.h"
#include <xercesc/dom/DOMElement.hpp>

XERCES_CPP_NAMESPACE_USE

#define CLASSNAME DOMElement
DEFAULT_GET_THIS_OBJECT(element);

JSBool WLXMLElementEnumerate(JSContext *cx, JSObject *obj );


static JSClass wl_XMLElementClass = {
	"XMLElementObject",
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	WLXMLElementEnumerate,//JS_EnumerateStub
	JS_ResolveStub,
	JS_ConvertStub,
	WLXmlGenericFinalize
};




JSBool WLXMLElementEnumerate(JSContext *cx, JSObject *obj )
{
  //enumerate class is implemented for: (for a in xmlparse), it can be implemented later.
  DOMElement* pObj = element_GetThisObject(cx, obj);
  
  if (pObj==NULL)
    return JS_TRUE;;

	return JS_TRUE;
}



static JSBool
element_getAttribute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMElement* pElement = element_GetThisObject(cx, obj);
  if (pElement)
  {
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcName, argv, 0, "element_getAttribute")
    const XMLCh* pAttrName =0;
    TRY_XML_PARSER_STANDARD
      pAttrName = pElement->getAttribute(pcName);
    CATCH_XML_PARSER_STANDARD(element,getAttribute)
    WLXmlSetXMLChValue(cx, obj, pAttrName, rval);
  }  
	return JS_TRUE;
}


static JSBool
element_getAttributeNS(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMElement* pElement = element_GetThisObject(cx, obj);
  if (pElement)
  {
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcNS, argv, 0, "element_getAttributeNS")
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcName, argv, 1, "element_getAttributeNS")
    const XMLCh* pAttrName =0;
    TRY_XML_PARSER_STANDARD
      pAttrName = pElement->getAttributeNS(pcNS,pcName);
    CATCH_XML_PARSER_STANDARD(element,getAttribute)
    WLXmlSetXMLChValue(cx, obj, pAttrName, rval);
  }  
	return JS_TRUE;
}


static JSBool
element_hasAttribute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMElement* pElement = element_GetThisObject(cx, obj);
  if (pElement)
  {
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcName, argv, 0, "element_hasAttribute")
    bool pHasAttr = false;
    TRY_XML_PARSER_STANDARD
      pHasAttr = pElement->hasAttribute(pcName);
    CATCH_XML_PARSER_STANDARD(element,getAttribute)
    WLXmlSetBoolValue(cx, obj, pHasAttr, rval);
  }  
	return JS_TRUE;
}

static JSBool
element_hasAttributeNS(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMElement* pElement = element_GetThisObject(cx, obj);
  if (pElement)
  {
    XML_GET_XMLCHAR_ARGUMENT_CHECK(nsName, argv, 0, "element_hasAttributeNS")
    XML_GET_XMLCHAR_ARGUMENT_CHECK(locName, argv, 1, "element_hasAttributeNS")
    bool pHasAttr = false;
    TRY_XML_PARSER_STANDARD
      pHasAttr = pElement->hasAttributeNS(nsName,locName);
    CATCH_XML_PARSER_STANDARD(element,getAttribute)
    WLXmlSetBoolValue(cx, obj, pHasAttr, rval);
  }  
	return JS_TRUE;
}

static JSBool
element_removeAttribute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMElement* pElement = element_GetThisObject(cx, obj); 
  if (pElement)
  {
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcName, argv, 0, "element_removeAttribute")
    TRY_XML_PARSER_STANDARD
      pElement->removeAttribute(pcName);
    CATCH_XML_PARSER_STANDARD(element,getAttribute)
  }  
	return JS_TRUE;
}

static JSBool
element_removeAttributeNS(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMElement* pElement = element_GetThisObject(cx, obj);
  if (pElement)
  {
    XML_GET_XMLCHAR_ARGUMENT_CHECK(nsName, argv, 0, "element_removeAttributeNS")
    XML_GET_XMLCHAR_ARGUMENT_CHECK(locName, argv, 1, "element_removeAttributeNS")
    TRY_XML_PARSER_STANDARD
      pElement->removeAttributeNS(nsName,locName);
    CATCH_XML_PARSER_STANDARD(element,getAttribute)
  }  
	return JS_TRUE;
}

static JSBool
element_setAttribute(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMElement* pElement = element_GetThisObject(cx, obj);
  if (pElement)
  {
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcName, argv, 0, "element_setAttribute")
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcVal, argv, 1, "element_setAttribute")
    TRY_XML_PARSER_STANDARD
      pElement->setAttribute(pcName,pcVal);
    CATCH_XML_PARSER_STANDARD(element,getAttribute)
  }  
	return JS_TRUE;
}


static JSBool
element_setAttributeNS(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMElement* pElement = element_GetThisObject(cx, obj);
  if (pElement)
  {
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcNS, argv, 0, "element_setAttributeNS")
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcName, argv, 1, "element_setAttributeNS")
    XML_GET_XMLCHAR_ARGUMENT_CHECK(pcVal, argv, 2, "element_setAttributeNS")
    TRY_XML_PARSER_STANDARD
      pElement->setAttributeNS(pcNS,pcName,pcVal);
    CATCH_XML_PARSER_STANDARD(element,getAttribute)
  }  
	return JS_TRUE;
}

static JSBool
element_setAttributeNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMElement* pElement = element_GetThisObject(cx, obj);
  if (pElement)
  {
    XML_GET_OBJECT_ARGUMENT_CHECK(pcAttrObj, argv,0,DOMAttr,"element_setAttributeNode")
    DOMAttr* pAttrNode =0;
    TRY_XML_PARSER_STANDARD
      pAttrNode = pElement->setAttributeNode(pcAttrObj);
    CATCH_XML_PARSER_STANDARD(element,getAttribute)
    *rval = WLCreateAttrObject(cx,obj,pAttrNode,false);
  }  
	return JS_TRUE;
}


static JSBool
element_setAttributeNodeNS(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMElement* pElement = element_GetThisObject(cx, obj);  
  if (pElement)
  {
    XML_GET_OBJECT_ARGUMENT_CHECK(pcAttrObj, argv,0,DOMAttr,"element_setAttributeNodeNS")
    DOMAttr* pAttrNode =0;
    TRY_XML_PARSER_STANDARD
      pAttrNode = pElement->setAttributeNodeNS(pcAttrObj);
    CATCH_XML_PARSER_STANDARD(element,getAttribute)
    *rval = WLCreateAttrObject(cx,obj,pAttrNode,false);
  }  
	return JS_TRUE;
}

static JSBool
element_removeAttributeNode(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMElement* pElement = element_GetThisObject(cx, obj);
  
  if (pElement)
  {
    XML_GET_OBJECT_ARGUMENT_CHECK(pcAttrObj, argv,0,DOMAttr,"element_removeAttributeNode")
    DOMAttr* pAttrNode =0;
    TRY_XML_PARSER_STANDARD
      pAttrNode = pElement->removeAttributeNode(pcAttrObj);
    CATCH_XML_PARSER_STANDARD(element,getAttribute)
    *rval = WLCreateAttrObject(cx,obj,pAttrNode,true);
  }  
	return JS_TRUE;
}


METHOD_RET_XMLCHAR_NO_ARGS(element, getTagName);
METHOD_RET_OBJECT_ONE_ARG_XMLCH(element, getAttributeNode, Attr, false);
METHOD_RET_OBJECT_ONE_ARG_XMLCH(element, getElementsByTagName, NodeList, false);
METHOD_RET_OBJECT_TWO_ARG_XMLCH(element, getElementsByTagNameNS, NodeList, false);
METHOD_RET_OBJECT_TWO_ARG_XMLCH(element, getAttributeNodeNS, Attr, false);


static JSFunctionSpec xmlElement_methods[] = {
  {"getElementsByTagName",   element_getElementsByTagName, 0},
  {"getElementsByTagNameNS",   element_getElementsByTagNameNS, 2},
  {"getAttribute",   element_getAttribute, 0},
  {"getAttributeNS",   element_getAttributeNS, 2},
  {"getAttributeNode",   element_getAttributeNode, 1},
  {"getAttributeNodeNS",   element_getAttributeNodeNS, 2},
  {"setAttributeNode",   element_setAttributeNode, 1},
  {"setAttributeNodeNS",   element_setAttributeNodeNS, 1},
  {"getTagName",   element_getTagName, 0},
  {"hasAttribute",   element_hasAttribute, 1},
  {"hasAttributeNS",   element_hasAttributeNS, 2},
  {"removeAttribute",   element_removeAttribute, 1},
  {"removeAttributeNS",   element_removeAttributeNS, 2},
  {"setAttribute",   element_setAttribute, 2},
  {"setAttributeNS",   element_setAttributeNS, 3},
  {"removeAttributeNode",   element_removeAttributeNode, 1},
  {0}
};


PROPERTY_GETTER_RET_XMLCHAR(element,tagName,getTagName) 

static JSPropertySpec xmlElement_properties[] = {
  { "tagName"    , 0, READONLY_PROPERTY_FLAGS  , element_prop_getTagName   , JS_PropertyStub },
	{0}
};



//create new Element object
jsval WLCreateElementObject(JSContext* cx, JSObject* obj, void* value, bool toDelete)
{
  if (!value)
    return JSVAL_NULL;

  CJSGCRequestBeginEnd GCGuard(cx); //lock GC for this action
	CWLXmlParserPrivate* prv = new CWLXmlParserPrivate((void*)value, CWLXmlParserPrivate::ELEMENT, toDelete);
  JSObject* newObj = JS_NewObject(cx, &wl_XMLElementClass, NULL, NULL);
	JS_SetPrivate(cx, newObj, (void*)prv);
  return OBJECT_TO_JSVAL(newObj);
}

//when we'll define all classes
JSObject* rvAddXMLElementClass(JSContext *cx, JSObject *obj, JSObject* parent_proto)
{
  return JS_InitClass(cx, obj, parent_proto, &wl_XMLElementClass, NULL, 0, xmlElement_properties, xmlElement_methods, NULL, NULL);
}


