/* WebLOAD Open Source is an automated quality assurance testing tool for internet applications.
 * Copyright (C) 1996 Radview Software, Ltd.
 *
 * This program is free software; you can redistribute it and/or modify it 
 * only under the terms of Version 2 of the GNU General Public License as 
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; Without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details.
 * 
 * You should have received a copy of the GNU General Public License along 
 * with this program; If not, you can download it from 
 * http://www.gnu.org/licenses/
 *
 */
  
#include "tracedll.h"
#include "wlerrorlib.h"
#include "loadexceptions.h"
#include "loadreportinfo.h"
#include "wlxmlfuncs.h"
#include "wlxmlprivate.h"
#include <xercesc/dom/DOMNamedNodeMap.hpp>

XERCES_CPP_NAMESPACE_USE

#define CLASSNAME DOMNamedNodeMap
DEFAULT_GET_THIS_OBJECT(namednodemap);

JSBool WLXMLNamedNodeMapEnumerate(JSContext *cx, JSObject *obj );


static JSClass wl_XMLNamedNodeMapClass = {
	"XMLNamedNodeMapObject",
	JSCLASS_HAS_PRIVATE,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	JS_PropertyStub,
	WLXMLNamedNodeMapEnumerate,//JS_EnumerateStub
	JS_ResolveStub,
	JS_ConvertStub,
	WLXmlGenericFinalize
};




JSBool WLXMLNamedNodeMapEnumerate(JSContext *cx, JSObject *obj )
{
  //enumerate class is implemented for: (for a in xmlparse), it can be implemented later.
  DOMNamedNodeMap* pNamedNodeMap = namednodemap_GetThisObject(cx, obj);
  
  if (pNamedNodeMap==NULL)
    return JS_TRUE;;

	return JS_TRUE;
}


METHOD_RET_INT_NO_ARGS(namednodemap, getLength);
METHOD_RET_OBJECT_ONE_ARG_XMLCH(namednodemap, getNamedItem, Node, false);
METHOD_RET_OBJECT_ONE_ARG_XMLCH(namednodemap, removeNamedItem, Node, true);//@@yossi - check this
METHOD_RET_OBJECT_TWO_ARG_XMLCH(namednodemap, getNamedItemNS, Node, false);
METHOD_RET_OBJECT_TWO_ARG_XMLCH(namednodemap, removeNamedItemNS, Node, true);//@@yossi - check this

static JSBool
namednodemap_setNamedItem(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNamedNodeMap* pNodeMap = namednodemap_GetThisObject(cx, obj);
  if (pNodeMap)
  {
    XML_GET_OBJECT_ARGUMENT_CHECK(pNode1, argv, 0, DOMNode, "namednodemap_setNamedItem")
    DOMNode* pNode = 0;
    TRY_XML_PARSER_STANDARD
      pNode = pNodeMap->setNamedItem(pNode1);
    CATCH_XML_PARSER_STANDARD(namednodemap,setNamedItem)
    *rval = WLCreateNodeObject(cx,obj,pNode,false);
  }  
	return JS_TRUE;
}

static JSBool
namednodemap_setNamedItemNS(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNamedNodeMap* pNodeMap = namednodemap_GetThisObject(cx, obj);
  if (pNodeMap)
  {
    XML_GET_OBJECT_ARGUMENT_CHECK(pNode1, argv, 0, DOMNode, "namednodemap_setNamedItem")
    DOMNode* pNode = 0;
    TRY_XML_PARSER_STANDARD
      pNode = pNodeMap->setNamedItemNS(pNode1);
    CATCH_XML_PARSER_STANDARD(namednodemap,setNamedItem)
    *rval = WLCreateNodeObject(cx,obj,pNode,false);
  }  
	return JS_TRUE;
}

static JSBool
namednodemap_item(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  DOMNamedNodeMap* pNodeMap = namednodemap_GetThisObject(cx, obj);
  if (pNodeMap)
  {
    XML_GET_XMLSIZE_ARGUMENT_CHECK(nIndex, argv, 0, "namednodemap_item");
    DOMNode* pNode = 0;
    TRY_XML_PARSER_STANDARD
      pNode = pNodeMap->item(nIndex);
    CATCH_XML_PARSER_STANDARD(namednodemap,setNamedItem)
    CreateNodeByType(cx, obj, pNode, rval, false);  
  }  
	return JS_TRUE;
}

static JSFunctionSpec xmlnamednodemap_methods[] = {
  {"getLength",   namednodemap_getLength, 0},
  {"getNamedItem",   namednodemap_getNamedItem, 0},
  {"removeNamedItem",   namednodemap_removeNamedItem, 0},
  {"getNamedItemNS",   namednodemap_getNamedItemNS, 0},
  {"removeNamedItemNS",   namednodemap_removeNamedItemNS, 0},
  {"setNamedItem",   namednodemap_setNamedItem, 1},
  {"setNamedItemNS",   namednodemap_setNamedItemNS, 1},
  {"item",   namednodemap_item, 1},
	{0}
};


static JSPropertySpec xmlnamednodemap_properties[] = {
	{0}
};



//create new NamedNodeMap object
jsval WLCreateNamedNodeMapObject(JSContext* cx, JSObject* obj, void* value, bool toDelete)
{
  if (!value)
    return JSVAL_NULL;

  CJSGCRequestBeginEnd GCGuard(cx); //lock GC for this action
	CWLXmlParserPrivate* prv = new CWLXmlParserPrivate((void*)value, CWLXmlParserPrivate::NAMEDNODEMAP, toDelete);
  JSObject* newObj = JS_NewObject(cx, &wl_XMLNamedNodeMapClass, NULL, NULL);
	JS_SetPrivate(cx, newObj, (void*)prv);
  return OBJECT_TO_JSVAL(newObj);
}

//when we'll define all classes
JSObject* rvAddXMLNamedNodeMapClass(JSContext *cx, JSObject *obj, JSObject* parent_proto)
{
  return JS_InitClass(cx, obj, parent_proto, &wl_XMLNamedNodeMapClass, NULL, 0, xmlnamednodemap_properties, xmlnamednodemap_methods, NULL, NULL);
}


